{$P}
{[a-,body+,h-,o=100,r+,rec+,t=4,u+,#+,j=20/57/1$,n-]}
{ UFloatWindow.inc1.p }
{ Copyright  1987-1990 by Apple Computer Inc.	All rights reserved. }

CONST
	kWindoidWDEF		= 16 * 3;						{ !!! our default windoid WDEF }

TYPE
	WindowPtrPtr		= ^WindowPtr;					{ used to access low memory globals }

VAR
	{ window mgr patch records }
	pBringToFrontPatch: TrapPatch;
	pDragWindowPatch:	TrapPatch;
	pSelectWindowPatch: TrapPatch;
	pMoveWindowPatch:	TrapPatch;
	pZoomWindowPatch:	TrapPatch;
	pShowWindowPatch:	TrapPatch;
	pShowHidePatch: 	TrapPatch;
	pHiliteWindowPatch: TrapPatch;

	{ GNE patch record to manage window mgr low memory globals }
	pGetNextEventPatch: TrapPatch;

	{ private globals needed by window mgr patches }
	pHasLayerMgr:		BOOLEAN;						{ flag tells us if the Layer Mgr is in }
	pDuringSelectWindow: BOOLEAN;						{ flag to correctly handle re-entrancy}
	{														 problems in SelectWindow}

{--------------------------------------------------------------------------------------------------}

{ The Layer Mgr included as of System 7.0 and AUX 2.0 presents problems with the Window Mgr patches}
	{we've installed. Specifically, the problems are that when switching in an application that has}
	{been switched out under MultiFinder, the application's patches are re-installed prior to the}
	{application's layer being pulled to front. So, the patches are re-installed and then MF calls}
   {SelectWindow on the application's layer. However, our patches get control and they need to know}
   {if the windowPtr that SelectWindow has been handed is a real window or a layer. The only way to}
   {know is to make a call to the Layer Mgr. However, the Layer Mgr interfaces aren't public, so we}
{have to make our own interface to the one Layer Mgr call that we need: IsLayer. This problem _may_}
	{be fixed if MultiFinder is modified so that it pulls the application's layer to the front}
	{*before* re-installing the application's patches.}

FUNCTION MAIsLayer(window: WindowPtr): BOOLEAN;
	INLINE $7002, $A829;

{--------------------------------------------------------------------------------------------------}
{$S AInit}

{ FORWARD declarations for patches that are applied in InitUFloatWindow }

PROCEDURE MABringToFront(window: WindowPtr);
	FORWARD;

PROCEDURE MADragWindow(window: WindowPtr;
					   startPt: Point;
					   bounds: Rect);
	FORWARD;

PROCEDURE MASelectWindow(window: WindowPtr);
	FORWARD;

PROCEDURE MAMoveWindow(window: WindowPtr;
					   hGlobal: INTEGER;
					   vGlobal: INTEGER;
					   front: BOOLEAN);
	FORWARD;

PROCEDURE MAZoomWindow(theWindow: WindowPtr;
					   partCode: INTEGER;
					   front: BOOLEAN);
	FORWARD;

PROCEDURE MAShowWindow(theWindow: WindowPtr);
	FORWARD;

PROCEDURE MAShowHide(theWindow: WindowPtr;
					 showFlag: BOOLEAN);
	FORWARD;

PROCEDURE MAHiliteWindow(theWindow: WindowPtr;
						 fHilite: BOOLEAN);
	FORWARD;

FUNCTION MAGetNextEvent(eventMask: INTEGER;
						VAR theEvent: EventRecord): BOOLEAN;
	FORWARD;

PROCEDURE InitUFloatWindow;

	BEGIN
	IF qTemplateViews & gDeadStripSuppression THEN
		IF Member(TObject(NIL), TFloatWindow) THEN; 	{ So the linker doesn't dead strip these }

	{ install window mgr patches }
	FailOSErr(PatchTrap(pBringToFrontPatch, _BringToFront, @MABringToFront));
	FailOSErr(PatchTrap(pDragWindowPatch, _DragWindow, @MADragWindow));
	FailOSErr(PatchTrap(pSelectWindowPatch, _SelectWindow, @MASelectWindow));
	FailOSErr(PatchTrap(pMoveWindowPatch, _MoveWindow, @MAMoveWindow));
	FailOSErr(PatchTrap(pZoomWindowPatch, _ZoomWindow, @MAZoomWindow));
	FailOSErr(PatchTrap(pHiliteWindowPatch, _HiliteWindow, @MAHiliteWindow));
	FailOSErr(PatchTrap(pShowWindowPatch, _ShowWindow, @MAShowWindow));
	FailOSErr(PatchTrap(pShowHidePatch, _ShowHide, @MAShowHide));

	{ tail patch to GNE which redirects activate/deactivate events }
	FailOSErr(PatchTrap(pGetNextEventPatch, _GetNextEvent, @MAGetNextEvent));

	pHasLayerMgr := TrapExists($A829);
	pDuringSelectWindow := FALSE;
	gUFloatWindowInitialized := TRUE;
	END;

{--------------------------------------------------------------------------------------------------}
{$Push}
{$IFC qTrace}
{$D+}
{$ENDC}
{$R-}
{$V-}
{$S MAWindowRes}

PROCEDURE FixedSendBehind(window, behind: WindowPtr;
						  doActivatePalette: BOOLEAN);

	VAR
		windPeek:			WindowPeek;
		savePort:			GrafPtr;
		clobbered:			RgnHandle;
		rgnTopLeft: 		Point;
		activeWindow:		WindowPtr;

	BEGIN
	{ set up }
	windPeek := WindowPeek(window);
	GetPort(savePort);
	SetPort(window);

	{ create a rgn so that we can figure out what rgn needs to be recalculated }
	clobbered := NewRgn;								{ don't call MakeNewRgn from patch }
	CopyRgn(window^.visRgn, clobbered);

	{ convert the rgn into globals }
	rgnTopLeft := clobbered^^.rgnBBox.topLeft;
	LocalToGlobal(rgnTopLeft);
	WITH clobbered^^.rgnBBox, rgnTopLeft DO
		OffsetRgn(clobbered, h - left, v - top);

	{ put that window where it belongs and activate its palette if requested }
	SendBehind(window, behind);
	IF gConfiguration.hasColorQD & doActivatePalette THEN
		ActivatePalette(window);

	{ figure out what needs to be painted, paint it, and recalculate vis rgns }
	DiffRgn(windPeek^.strucRgn, clobbered, clobbered);
	PaintOne(windPeek, clobbered);
	CalcVisBehind(windPeek, clobbered); 				{windPeek^.strucRgn}

	{ clean up }
	DisposeRgn(clobbered);
	SetPort(savePort);
	END;
{$Pop}

{--------------------------------------------------------------------------------------------------}
{$Push}
{$IFC qTrace}
{$D+}
{$ENDC}
{$R-}
{$V-}
{$S MAWindowRes}

PROCEDURE CallBringToFront(window: WindowPtr;
						   oldTrap: LongInt);
	INLINE $205F, $4E90;

PROCEDURE MABringToFront(window: WindowPtr);

{ Patch for BringToFront that preserves window layering:}
{	 for floaters, leave the routine & call the real BringToFront;}
{	 for dialogs, leave the routine & call the real BringToFront;}
{	 if no floaters, leave the routine & call the real BringToFront;}
{	 otherwise, call SendBehind }

	VAR
		oldA5:				LongInt;
		lastFloatingWindowPtr: WindowPtr;
		savePort:			GrafPtr;
		windPeek:			WindowPeek;
		clobbered:			RgnHandle;
		rgnTopLeft: 		Point;
		activeWindow:		WindowPtr;

	BEGIN
	oldA5 := SetCurrentA5;
	windPeek := WindowPeek(window);

	{  Floaters: call the real BringToFront;}
	IF IsFloatWindow(window) THEN
		BEGIN
		IF NOT gInBackground & (window <> FrontWindow) THEN
			CallBringToFront(window, pBringToFrontPatch.oldTrapAddr);
		END

		{  Layers or System Windows: call the real BringToFront;}
	ELSE IF (pHasLayerMgr & MAIsLayer(window)) | (windPeek^.windowKind < 0) THEN
		CallBringToFront(window, pBringToFrontPatch.oldTrapAddr)

		{  Dialogs: call the real BringToFront and then de-hilite and deactivate the active window}
	ELSE IF IsDialog(window) THEN
		BEGIN
		{ bring the dialog to the front }
		CallBringToFront(window, pBringToFrontPatch.oldTrapAddr);

		{ de-hilite the active window and deactivate it }
		activeWindow := MAGetActiveWindow;
		IF (activeWindow <> NIL) & (activeWindow <> window) THEN
			BEGIN
			HiliteWindow(activeWindow, FALSE);
			WindowPtrPtr(CurDeactive)^ := activeWindow; { set curDeactive }
			IF activeWindow <> NIL THEN
				IntegerPtr(CurDeKind)^ := WindowPeek(activeWindow)^.windowKind;
			END;
		END

		{  Document windows }
	ELSE
		BEGIN
		{  Document Windows: if no floaters, call the real BringToFront}
		lastFloatingWindowPtr := GetLastFloatingWindowPtr;
		IF (lastFloatingWindowPtr = NIL) THEN
			CallBringToFront(window, pBringToFrontPatch.oldTrapAddr)

			{  Document Windows: if there are floaters, call SendBehind}
		ELSE IF (window <> MAFrontWindow) THEN
			BEGIN
			SendSystemWindowsToBack;					{ send system windows to back }
			FixedSendBehind(window, lastFloatingWindowPtr, TRUE);
			END;
		END;
	IF SetA5(oldA5) <> 0 THEN;
	END;
{$Pop}

{--------------------------------------------------------------------------------------------------}
{$Push}
{$IFC qTrace}
{$D+}
{$ENDC}
{$R-}
{$V-}
{$S MAWindowRes}

PROCEDURE MADragWindow(window: WindowPtr;
					   startPt: Point;
					   bounds: Rect);

{ Complete replacement of DragWindow that preserves window layering. }

	CONST
		kCommandKey 		= 55;

	VAR
		oldA5:				LongInt;
		wMgrPort:			GrafPtr;
		savePort:			GrafPtr;
		dragRgn:			RgnHandle;
		result: 			LongInt;
		newLoc: 			Point;
		portBounds: 		Rect;
		tempRgn:			RgnHandle;
		clipWindow: 		WindowPtr;
		commandFlag:		BOOLEAN;
		theKeys:			KeyMap;

	BEGIN
	oldA5 := SetCurrentA5;

	{ set up some parameters }
	GetKeys(theKeys);
	commandFlag := theKeys[kCommandKey];
	IF StillDown THEN
		BEGIN
		GetPort(savePort);
		GetWMgrPort(wMgrPort);
		SetPort(wMgrPort);

		tempRgn := NewRgn;								{ what if NewRgn returns NIL??? }
		GetClip(tempRgn);

		{ drag the gray outline }
		IF commandFlag THEN
			clipWindow := window						{ we won't be activating this window }
		ELSE IF IsFloatWindow(window) | (WindowPeek(window)^.windowKind < 0) THEN
			clipWindow := FrontWindow					{ floaters & DAs get dragged at front }
		ELSE
			BEGIN
			SendSystemWindowsToBack;					{ send system windows behind selected window
														 }
			clipWindow := FrontDocumentWindow;
			END;
		MAClipAbove(clipWindow);
		dragRgn := NewRgn;								{ what if NewRgn returns NIL??? }
		CopyRgn(WindowPeek(window)^.strucRgn, dragRgn);
		result := DragGrayRgn(dragRgn, startPt, bounds, bounds, noConstraint, NIL);

		SetClip(tempRgn);
		DisposeRgn(tempRgn);
		DisposeRgn(dragRgn);

		{ determine what to do with the window }
		WITH newLoc DO
			BEGIN
			v := HiWrd(result);
			h := LoWrd(result);
			IF v = $8000 THEN
				BEGIN
				{ in certain situations, we sent the system windows to back.}
				{Since we sent the system windows to back, we need to activate the front}
				{window if it is now the front document window. }
				IF (NOT commandFlag) & (NOT IsFloatWindow(window)) &
				   (WindowPeek(window)^.windowKind >= 0) & (MAFrontWindow =
				   FrontDocumentWindow) THEN
					SelectWindow(window);
				END
			ELSE
				BEGIN
				IF BAND(CGrafPtr(window)^.portVersion, $C000) <> 0 THEN
					portBounds := CGrafPtr(window)^.portPixMap^^.bounds
				ELSE
					portBounds := window^.portBits.bounds;
				v := v + window^.portRect.top - portBounds.top;
				h := h + window^.portRect.left - portBounds.left;
				MoveWindow(window, h, v, FALSE);
				IF NOT commandFlag THEN
					SelectWindow(window);
				END;
			END;

		{ restore various values }
		SetPort(savePort);
		END;

	IF SetA5(oldA5) <> 0 THEN;
	END;
{$Pop}

{--------------------------------------------------------------------------------------------------}
{$Push}
{$IFC qTrace}
{$D+}
{$ENDC}
{$R-}
{$V-}
{$S MAWindowRes}

	var
		pFrontWindow: WindowPtr;			{ stores the reference to the window that the MAFrontWindow patch should return }
		pFrontWindowPatch: TrapPatch;

	function FrontWindowPatch: WindowPtr;		{ patch to FrontWindow that is active while calling ActivatePalette }

	begin
		FrontWindowPatch := pFrontWindow;
	end;

	procedure MAActivatePalette (window: WindowPtr);
{ ensures that the call to ActivatePalette on "window" actually takes effect }

		var
			needsFrontWindowPatch: BOOLEAN;

	begin
		needsFrontWindowPatch := FrontWindow <> window;
		if needsFrontWindowPatch then		{ patch FrontWindow so that it returns the "window" }
			FailOSErr(PatchTrap(pFrontWindowPatch, _FrontWindow, @FrontWindowPatch));
		ActivatePalette(window);
		if needsFrontWindowPatch then
			UnpatchTrap(pFrontWindowPatch);
	end;

{$Pop}

{--------------------------------------------------------------------------------------------------}
{$Push}
{$IFC qTrace}
{$D+}
{$ENDC}
{$R-}
{$V-}
{$S MAWindowRes}

PROCEDURE CallSelectWindow(window: WindowPtr;
						   oldTrap: LongInt);
	INLINE $205F, $4E90;

PROCEDURE MASelectWindow(window: WindowPtr);
{ Patch for SelectWindow that preserves window layering. }

	VAR
		oldA5:				LongInt;
		savePort:			GrafPtr;
		activeWindow:		WindowPtr;

	BEGIN
	oldA5 := SetCurrentA5;
	IF NOT pDuringSelectWindow THEN
		BEGIN
		IF pHasLayerMgr & MAIsLayer(window) THEN
			CallSelectWindow(window, pSelectWindowPatch.oldTrapAddr)
		ELSE
			BEGIN

			GetPort(savePort);

			IF IsFloatWindow(window) THEN
				BEGIN
				BringToFront(window);
				
				{ for color environments, set the floater's palette to be the active
				palette iff the active window has no palette }
				IF gConfiguration.hasColorQD THEN
					BEGIN
					activeWindow := MAGetActiveWindow;
					IF (GetPalette(activeWindow) = NIL) THEN
						ActivatePalette(window);
					END
				END
			ELSE
				BEGIN
				{ unhilite and deactivate the active document window }
				activeWindow := MAGetActiveWindow;
				IF (activeWindow <> NIL) & (activeWindow <> window) THEN
					BEGIN
					HiliteWindow(activeWindow, FALSE);
					WindowPtrPtr(CurDeactive)^ := activeWindow; { set curDeactive }
					IF activeWindow <> NIL THEN
						IntegerPtr(CurDeKind)^ := WindowPeek(activeWindow)^.windowKind;
					END;

				{ move the window to the correct position, hilite, and activate it }
				pDuringSelectWindow := TRUE;
				BringToFront(window);
				pDuringSelectWindow := FALSE;
				HiliteWindow(window, TRUE);
				IF (WindowPeek(window)^.visible) THEN
					WindowPtrPtr(CurActivate)^ := window; { set curActivate }
				
				IF gConfiguration.hasColorQD THEN
					MAActivatePalette(window);
				END;
			SetPort(savePort);
			END;
		END;
	IF SetA5(oldA5) <> 0 THEN;
	END;
{$Pop}

{--------------------------------------------------------------------------------------------------}
{$Push}
{$IFC qTrace}
{$D+}
{$ENDC}
{$R-}
{$V-}
{$S MAWindowRes}

PROCEDURE CallMoveWindow(window: WindowPtr;
						 hGlobal, vGlobal: INTEGER;
						 front: BOOLEAN;
						 oldTrap: LongInt);
	INLINE $205F, $4E90;

PROCEDURE MAMoveWindow(window: WindowPtr;
					   hGlobal: INTEGER;
					   vGlobal: INTEGER;
					   front: BOOLEAN);
{ Tail Patch for MoveWindow that preserves window layering.}
{	We do this b/c MoveWindow doesn't call SelectWindow trap, instead jumps directly there}

	VAR
		oldA5:				LongInt;

	BEGIN
	oldA5 := SetCurrentA5;

	{ force FALSE for "front" parameter }
	CallMoveWindow(window, hGlobal, vGlobal, FALSE, pMoveWindowPatch.oldTrapAddr);

	IF front THEN
		SelectWindow(window);
	IF SetA5(oldA5) <> 0 THEN;
	END;
{$Pop}

{--------------------------------------------------------------------------------------------------}
{$Push}
{$IFC qTrace}
{$D+}
{$ENDC}
{$R-}
{$V-}
{$S MAWindowRes}

PROCEDURE CallZoomWindow(theWindow: WindowPtr;
						 partCode: INTEGER;
						 front: BOOLEAN;
						 oldTrap: LongInt);
	INLINE $205F, $4E90;

PROCEDURE MAZoomWindow(theWindow: WindowPtr;
					   partCode: INTEGER;
					   front: BOOLEAN);

	VAR
		oldA5:				LongInt;

	BEGIN
	oldA5 := SetCurrentA5;

	{ force FALSE for "front" parameter }
	CallZoomWindow(theWindow, partCode, FALSE, pZoomWindowPatch.oldTrapAddr);

	IF front THEN
		SelectWindow(theWindow);
	IF SetA5(oldA5) <> 0 THEN;
	END;
{$Pop}

{--------------------------------------------------------------------------------------------------}
{$Push}
{$IFC qTrace}
{$N+}
{$ENDC}
{$D-}
{$R-}
{$V-}
{$S MAWindowRes}

PROCEDURE CallHiliteWindow(theWindow: WindowPtr; fHilite: BOOLEAN; oldTrap: LongInt);
	INLINE $205F, $4E90;

PROCEDURE MAHiliteWindow(theWindow: WindowPtr; fHilite: BOOLEAN);

{ patch to HiliteWindow which ensures that the correct window gets hilited/dehilited }
{ called from NewDialog/CloseDialog }

	VAR
		oldA5: LongInt;
		toHilite: WindowPtr;

	BEGIN
	oldA5 := SetCurrentA5;
	IF (WindowPeek(theWindow)^.visible) THEN
		BEGIN
		IF NOT IsFloatWindow(theWindow) THEN
			CallHiliteWindow(theWindow, fHilite, pHiliteWindowPatch.oldTrapAddr)
		ELSE
			BEGIN
			IF fHilite THEN
				BEGIN
				toHilite := MAFrontWindow;
				WindowPtrPtr(CurActivate)^ := toHilite; { set curActivate }
				END
			ELSE
				BEGIN
				toHilite := MAGetActiveWindow;
				WindowPtrPtr(CurDeactive)^ := toHilite; { set CurDeactive }
				IF toHilite <> NIL THEN
					IntegerPtr(CurDeKind)^ := WindowPeek(theWindow)^.windowKind;
				END;
			IF toHilite <> NIL THEN
				CallHiliteWindow(toHilite, fHilite, pHiliteWindowPatch.oldTrapAddr);
			END;
		END;

	IF SetA5(oldA5) <> 0 THEN;
	END;
{$Pop}
{--------------------------------------------------------------------------------------------------}
{$Push}
{$IFC qTrace}
{$D+}
{$ENDC}
{$R-}
{$V-}
{$S MAWindowRes}

PROCEDURE CallShowWindow(theWindow: WindowPtr;
						 oldTrap: LongInt);
	INLINE $205F, $4E90;

PROCEDURE MAShowWindow(theWindow: WindowPtr);

{ tail patch to ShowWindow which ensures that SelectWindow gets called for a shown window }

	VAR
		oldA5:				LongInt;

	BEGIN
	oldA5 := SetCurrentA5;
	IF NOT WindowPeek(theWindow)^.visible THEN
		BEGIN
		CallShowWindow(theWindow, pShowWindowPatch.oldTrapAddr);
		IF (MAFrontWindow = theWindow) THEN
			SelectWindow(theWindow);
		END;
	IF SetA5(oldA5) <> 0 THEN;
	END;
{$Pop}

{--------------------------------------------------------------------------------------------------}
{$Push}
{$IFC qTrace}
{$D+}
{$ENDC}
{$R-}
{$V-}
{$S MAWindowRes}

PROCEDURE CallShowHide(theWindow: WindowPtr;
					   showFlag: BOOLEAN;
					   oldTrap: LongInt);
	INLINE $205F, $4E90;

PROCEDURE MAShowHide(theWindow: WindowPtr;
					 showFlag: BOOLEAN);

{ This patch manages the global pActiveWindow, and activates the next document window in the case}
{where we're hiding the currently active document window and there are floaters present.}
{}
{  Note that this patch to ShowHide actually gets called from 5 other traps, since:}
{}
{		HideWindow calls ShowHide}
{		CloseWindow calls ShowHide}
{		DisposWindow calls CloseWindow}
{		CloseDialog calls CloseWindow}
{		DisposDialog calls CloseDialog}
{}
{  So, ShowHide is the routine through which all other Window Mgr routines eventually bottleneck to}
{actually hide the window.}

	VAR
		oldA5:				LongInt;
		needActivate:		BOOLEAN;
		toActivate: 		WindowPtr;

	BEGIN
	oldA5 := SetCurrentA5;

	{ if we're hiding a front document window that's not the actual front window (i.e. it's in back}
	{	of a floater) then we need to select the next document window}
	needActivate := (NOT showFlag) & (theWindow = MAFrontWindow) & (FrontWindow <> theWindow);

	CallShowHide(theWindow, showFlag, pShowHidePatch.oldTrapAddr);

	{ activate the next window if necessary }
	IF needActivate THEN
		BEGIN
		toActivate := MAFrontWindow;
		IF toActivate <> NIL THEN
			BEGIN										{ instead, call SelectWindow ??? }
			WindowPtrPtr(CurActivate)^ := toActivate;	{ set curActivate }
			HiliteWindow(toActivate, TRUE);
			END;
		END;

	IF SetA5(oldA5) <> 0 THEN;
	END;
{$Pop}

{--------------------------------------------------------------------------------------------------}
{$Push}
{$IFC qTrace}
{$N+}
{$ENDC}
{$D-}
{$R-}
{$V-}
{$S MAWindowRes}

FUNCTION CallGetNextEvent(eventMask: INTEGER; VAR theEvent: EventRecord; oldTrap: LongInt): BOOLEAN;
	INLINE $205F, $4E90;

FUNCTION MAGetNextEvent(eventMask: INTEGER; VAR theEvent: EventRecord): BOOLEAN;

{ tail patch to GetNextEvent that:}
{	 redirects activate events from floating windows to front doc window}
{	 redirects deactivate events from floating windows to front doc window}

	CONST
		osEvt = app4Evt;
		osEvtMessageMask = $FF000000;
		suspendResumeMessage = $01;

	VAR
		oldA5: LongInt;
		myToActivate: WindowPtr;
		myToDeactivate: WindowPtr;

	BEGIN
	oldA5 := SetCurrentA5;

	MAGetNextEvent := CallGetNextEvent(eventMask, theEvent, pGetNextEventPatch.oldTrapAddr);

	WITH theEvent DO
		BEGIN
		CASE what OF
			activateEvt:
				BEGIN
				IF Odd(modifiers) THEN {  activate event }
					BEGIN
				 { redirect all activate calls from float windows to the front, non-floater window }
					IF IsFloatWindow(WindowPtr(message)) THEN
						BEGIN
						myToActivate := MAFrontWindow; { the front, non-floater window }
						IF myToActivate = NIL THEN
							what := nullEvent { then zap the activate event! }
						ELSE
							BEGIN
							HiliteWindow(myToActivate, TRUE); { restore hiliting }
							message := ORD(myToActivate); { redirect activate event }
							END
						END
					END { IF Odd(modifiers) THEN }
				ELSE {  deactivate event }
				{ redirect all deactivate calls from float windows to the active window }
				IF IsFloatWindow(WindowPtr(message)) THEN { if deactivate evt is for floater }
					what := nullEvent; { then zap the deactivate event! }
				END; { IF what = activateEvt THEN }
			osEvt:
				IF (BSR(BAND(message, osEvtMessageMask), 24) = suspendResumeMessage) THEN
					BEGIN
					myToActivate := MAFrontWindow; { the front, non-floater window }
					IF myToActivate <> NIL THEN
						HiliteWindow(myToActivate, Odd(message)); { switching in/out }
					END;
		END; { CASE what OF }
		END; { WITH theEvent DO }

	IF SetA5(oldA5) <> 0 THEN;
	END;
{$Pop}
{--------------------------------------------------------------------------------------------------}
{$S MAOpen}

PROCEDURE TFloatWindow.IFloatWindow(itsDocument: TDocument;
									itsWMgrWindow: WindowPtr;
									canResize, canClose, disposeOnFree: BOOLEAN);

	VAR
		saveStdWTopleft:	Point;

	BEGIN
	saveStdWTopleft := gStdWSizeRect.topLeft;
	gStdWSizeRect.topLeft := gZeroPt;
	IWindow(itsDocument, itsWMgrWindow, canResize, canClose, disposeOnFree);
	gStdWSizeRect.topLeft := saveStdWTopleft;

	fHideOnSuspend := TRUE;
	fDoFirstClick := TRUE;
	fFloats := TRUE;
	WindowPeek(itsWMgrWindow)^.windowKind := kFloatWindowKind; { mark it as a floating window }
	END;

{--------------------------------------------------------------------------------------------------}
{$S MAOpen}

PROCEDURE TFloatWindow.IRes(itsDocument: TDocument;
							itsSuperView: TView;
							VAR itsParams: Ptr);

	VAR
		saveStdWTopleft:	Point;

	BEGIN
	saveStdWTopleft := gStdWSizeRect.topLeft;
	gStdWSizeRect.topLeft := gZeroPt;
	inherited IRes(itsDocument, itsSuperView, itsParams);
	gStdWSizeRect.topLeft := saveStdWTopleft;

	fHideOnSuspend := TRUE;
	fDoFirstClick := TRUE;
	fFloats := TRUE;
	WindowPeek(fWMgrWindow)^.windowKind := kFloatWindowKind; { mark it as a floating window }
	END;

{--------------------------------------------------------------------------------------------------}
{$S MAOpen}

FUNCTION TFloatWindow.GetBehindWindowPtr: WindowPtr;

	BEGIN
	GetBehindWindowPtr := WindowPtr( - 1);				{ bring the float window up in front of all}
	{														 other windows}
	END;

{--------------------------------------------------------------------------------------------------}
{$S MAActivate}

PROCEDURE TFloatWindow.Activate(entering: BOOLEAN); OVERRIDE;

	VAR
		different:			BOOLEAN;

	BEGIN
	InvalidateFocus;									{ Must refocus to make sure thePort is set }
	different := entering <> fIsActive;

	IF different THEN
		BEGIN
		Update; 										{ Logically the order is wrong, but it
														 gives}
   {														the correct visual effect. ??? Revisit?}
		inherited Activate(entering);

		IF entering THEN
			fIsActive := entering;
		END;

	IF fIsResizable & Focus & IsVisible THEN
		DrawResizeIcon;
	END;

{--------------------------------------------------------------------------------------------------}
{$Push}
{$IFC qTrace}
{$D+}
{$ENDC}
{$R-}
{$V-}
{ called from patches }
{$S MAWindowRes}

PROCEDURE MAClipAbove(window: WindowPtr);
{ fixes ROM anomaly in which ClipAbove clips to the current clip rgn.}
{	we assume the current port is the window mgr port}

	BEGIN
	SetClip(GetGrayRgn);
	ClipAbove(WindowPeek(window));
	END;
{$Pop}

{--------------------------------------------------------------------------------------------------}
{$Push}
{$IFC qTrace}
{$D+}
{$ENDC}
{$R-}
{$V-}
{ called from patches }
{$S MAWindowRes}

FUNCTION FrontDocumentWindow: WindowPtr;

{ returns the first visible document window in the window list }

	VAR
		aWindowPtr: 		WindowPtr;
		wPeek:				WindowPeek;

	BEGIN
	aWindowPtr := GetWindowList;
	WHILE (aWindowPtr <> NIL) DO
		BEGIN
		wPeek := WindowPeek(aWindowPtr);
		IF (wPeek^.visible) & IsDocumentWindow(aWindowPtr) THEN
			LEAVE;
		aWindowPtr := WindowPtr(wPeek^.nextWindow);
		END;
	FrontDocumentWindow := aWindowPtr;
	END;
{$Pop}

{--------------------------------------------------------------------------------------------------}
{$Push}
{$IFC qTrace}
{$D+}
{$ENDC}
{$R-}
{$V-}
{$S MAWindowRes}

PROCEDURE DeactivateSystemWindow(window: WindowPtr);
{ dehilite & deactivate hilited system windows }

	VAR
		cacheActivate:		WindowPtr;
		theEvent:			EventRecord;

	BEGIN
	IF (window <> NIL) & (WindowPeek(window)^.windowKind < 0) & (WindowPeek(window)^.hilited) THEN
		BEGIN
		{ cache the CurActivate so that EventAvail gives us the Deactivate event }
		cacheActivate := WindowPtrPtr(CurActivate)^;
		WindowPtrPtr(CurActivate)^ := NIL;

		{ de-hilite the system window }
		HiliteWindow(window, FALSE);

		{ process the deactivate event }
		WindowPtrPtr(CurDeactive)^ := window;			{ set curDeactive }
		IntegerPtr(CurDeKind)^ := WindowPeek(window)^.windowKind;
		IF EventAvail(activMask, theEvent) THEN 		{ gimme that deactivate! }
			WITH theEvent DO
				IF (what = activateEvt) & NOT Odd(modifiers) & (WindowPtr(message) = window) &
				   SystemEvent(theEvent) THEN
					WindowPtrPtr(CurDeactive)^ := NIL;	{ zap it, it got handled }

		{ restore CurActivate }
		WindowPtrPtr(CurActivate)^ := cacheActivate;
		END;
	END;
{$Pop}

{--------------------------------------------------------------------------------------------------}
{$Push}
{$IFC qTrace}
{$D+}
{$ENDC}
{$R-}
{$V-}
{$S MAWindowRes}

PROCEDURE SendSystemWindowsToBack;
{ send system windows behind all windows }

	VAR
		nextWindow: 		WindowPeek;
		thisWindow: 		WindowPeek;
		lastWindow: 		WindowPeek;

	BEGIN
	{ find out which non-system window is the last window }
	thisWindow := WindowPeek(FrontWindow);
	lastWindow := thisWindow;
	WHILE (thisWindow <> NIL) DO
		BEGIN
		IF WindowPeek(thisWindow)^.windowKind >= 0 THEN
			lastWindow := thisWindow;
		thisWindow := thisWindow^.nextWindow;
		END;

	{ iterate until no more windows or we've reached the last window }
	thisWindow := WindowPeek(FrontWindow);
	WHILE (thisWindow <> NIL) & (thisWindow <> lastWindow) DO
		BEGIN
		nextWindow := thisWindow^.nextWindow;			{ what is it *NOW* before we munge the
														 window list }
		IF thisWindow^.windowKind < 0 THEN				{ if it's a System window }
			BEGIN
			FixedSendBehind(WindowPtr(thisWindow), NIL, FALSE); { send thisWindow to back }
			DeactivateSystemWindow(WindowPtr(thisWindow));
			END;
		thisWindow := nextWindow;
		END;
	END;
{$Pop}

{--------------------------------------------------------------------------------------------------}
