{[a-,body+,h-,o=100,r+,rec+,t=4,u+,#+,j=20/57/1$,n-]}
{ UTearOffMenu.p }
{ Copyright  1989-1990 by Apple Computer Inc. All rights reserved. }

{[f-]}
(*
*)
{[f+]}

UNIT UTearOffMenu;

	INTERFACE

		USES
			{  MacApp}
			UMacApp,

			{  Building blocks }
			UMenu, 

			{  Implementation uses }
			Errors;

		CONST
			kTearOffMargin = 10;	{ the slop around the displayed menu within which we don't tear-off;
									see Apple Human Interface Guidelines p. 91;
									note: some programs (eg. HyperCard) use a 15 pixel slop value }
			kTearOffOffset = 3;		{ pixel offset from pointer to gray rgn being dragged around }

		TYPE

			TTearOffMenu = OBJECT; FORWARD;

			TShowTearOffWindowCmd = OBJECT (TNoChangesCommand)
			{ moves, shows and selects a TTearOffMenu }

				fTearOffWindow: TWindow;

				fWhere: Point;

				PROCEDURE TShowTearOffWindowCmd.IShowTearOffWindowCmd(tearOffWindow:TWindow; where: Point);

				PROCEDURE TShowTearOffWindowCmd.DoIt; OVERRIDE;

				PROCEDURE TShowTearOffWindowCmd.Fields(PROCEDURE DoToField(fieldName: Str255;
														   fieldAddr: Ptr;
														   fieldType: INTEGER)); OVERRIDE;
				END;

			TTearOffTracker = OBJECT (TCommand)
			{ abstract superclass providing basic tear-off menu tracking: tracks the grey outline of
			the menu around the screen }

				fTearOffMenu: TTearOffMenu;

				fTearOffWindowOutline: RgnHandle;

				fExitTracking: BOOLEAN;
				
				fOrigin: Point;

				PROCEDURE TTearOffTracker.ITearOffTracker(hitPt: Point; tearOffMenu: TTearOffMenu);

				PROCEDURE TTearOffTracker.Free; OVERRIDE;

				PROCEDURE TTearOffTracker.DoIt; OVERRIDE;

				PROCEDURE TTearOffTracker.TrackFeedback(anchorPoint, nextPoint: VPoint; turnItOn,
													   mouseDidMove: BOOLEAN); OVERRIDE;

				FUNCTION TTearOffTracker.TrackMouse(aTrackPhase: TrackPhase; VAR anchorPoint, previousPoint,
												   nextPoint: VPoint; mouseDidMove: BOOLEAN): TCommand; OVERRIDE;

				FUNCTION TTearOffTracker.IsDoneTracking: BOOLEAN; OVERRIDE;

				PROCEDURE TTearOffTracker.Fields(PROCEDURE DoToField(fieldName: Str255;
														   fieldAddr: Ptr;
														   fieldType: INTEGER)); OVERRIDE;

				END;

			TTearOffMenu = OBJECT (TMenu)
			{ abstract superclass providing basic tear-off menu behavior }

				fTearOffTrackingRgn: RgnHandle;
				
				fTearOffTracker: TTearOffTracker;

				fTearOffWindow:	TWindow;

				PROCEDURE TTearOffMenu.ITearOffMenu(rsrcID: INTEGER; menuWidth, menuHeight: INTEGER; tearOffWindow: TWindow);

				PROCEDURE TTearOffMenu.InstallTearOffTracker;
				{ For the sake of efficiency when tracking a tear-off, we'll create the tear-off tracker and
				 keep it with us, so we don't need to allocate a new tracker every time we tear off a menu. If you
				 want a different tracker in your subclass of TTearOffMenu, then override this method. }

				PROCEDURE TTearOffMenu.Free; OVERRIDE;
				
				PROCEDURE TTearOffMenu.HandleChooseMessage(message: INTEGER;
													theMenu: MenuHandle;
													VAR menuRect: Rect;
													hitPt: Point;
													VAR whichItem: INTEGER); OVERRIDE;
				{ initiates tracking if mouse is in fTearOffTrackingRgn }

				FUNCTION TTearOffMenu.GetTearOffTrackingRgn(menuRect: Rect): RgnHandle;
				{ accessor for fTearOffTrackingRgn: a rgn that represents the desktop less the menu rect (with some buffer area
				around the menu rect). This is useful for testing whether tracking should occur because the mouse
				just might be in the menu bar (which isn't in the desktop & isn't in the menu rect) and so tracking
				should not occur while the mouse is in the menu bar (MenuSelect does that tracking just fine). }
				
				PROCEDURE TTearOffMenu.Fields(PROCEDURE DoToField(fieldName: Str255;
														   fieldAddr: Ptr;
														   fieldType: INTEGER)); OVERRIDE;
				END;

	IMPLEMENTATION

		{$I UTearOffMenu.inc1.p}

END.
