{	This file has been processed by The THINK Pascal Source Converter, v1.03.	}

{[a-,body+,h-,o=100,r+,rec+,t=4,u+,#+,j=20/57/1$,n+]}
{ UBetterFeedbackCmd.p }
{ Copyright  1988-1990 by Apple Computer, Inc. All rights reserved.}

{[f-]}
{	This unit attempts to provide smooth flicker-free feedback during command tracking.}
{}
{		  T H E O R Y   O F   O P E R A T I O N}
{}
{	By synchronizing drawing to the vertical retrace interrupt, this unit attempts}
{	to improve the feedback that mouse trackers provide.}
{	}
{	Ideally, this behavior would be part of MacApp's TApplication.TrackMouse method.}
{	What TApplication.TrackMouse should really do is:}
{		a) wait for the vertical retrace interrupt of the device that the view most intersects}
{		b) erase the old tracker feedback by calling tracker.TrackFeedback(, turn It Off, )}
{		c) immediately draw the new feedback by calling tracker.TrackFeedback(, turn It on, )}
{}
{	However, with the current architecture, this is impossible. TApplication.TrackMouse currently}
{	does the following:}
{		a) call tracker.TrackFeedback(, turn It Off, ) to erase the previous tracker feedback}
{		b) call tracker.TrackMouse(trackMove, ) to tell the tracker the track phase}
{		c) call tracker.TrackFeedback(, turn It On, ) to draw the new tracker feedback}
{}
{	UBetterFeedbackCmd provides a way to work within the current architecture to provide a close}
{	attempt at smooth flicker-free feedback. To take advantage of this unit, create a}
{	subclass of TBetterFeedbackCmd. Then, OVERRIDE PROCEDURE TBetterFeedbackCmd.TrackFeedback()}
{	and as the first statement of this method call:}
{			INHERITED TrackFeedback(anchorPoint, nextPoint, turnItOn, mouseDidMove);}
{	so that TBetterFeedbackCmd.TrackFeedback can synch to the vertical retrace interrupt.}
{	Then the tracker should go ahead and draw/erase the feedback.}
{	}
{	For an example, see DrawShapes: TShapeSketcher, TShapeDragger, and TShapeSelector.}
{	}
{}
{[f+]}

unit UBetterFeedbackCmd;

interface

	uses
		SysEqu, Traps, ULoMem, UMacAppUtilities, UPatch, UObject, UViewCoords, UMemory, UFailure, UMenuSetup, UList,
{  MacApp }
		UMacApp,
{  Building Blocks }
{  Required for this unit's interface }
{  Implementation Use }
		Start, Retrace;

	const

		kInstall = TRUE;							{ pass to BetterFeedback:}
{																 install our VBL?}
		kBetterFeedbackDesired = TRUE;						{ pass to IBetterFeedbackCmd:}
{																 desire better feedback?}

	type
		TBetterFeedbackCmd = object(TCommand)

				fBetterFeedbackInstalled: BOOLEAN;			{ are the synchronization routines in? }

				fBetterFeedbackDesired: BOOLEAN;			{ want the synchronization routines? }

				procedure TBetterFeedbackCmd.IBetterFeedbackCmd (itsCmdNumber: CmdNumber; itsDocument: TDocument; itsView: TView; itsScroller: TScroller; betterFeedbackDesired: BOOLEAN);

				procedure TBetterFeedbackCmd.Free;
				OVERRIDE;

						{ for tracking }
				procedure TBetterFeedbackCmd.BetterFeedback (install: BOOLEAN);

				procedure TBetterFeedbackCmd.WaitBetterFeedback;

				function TBetterFeedbackCmd.TrackMouse (aTrackPhase: TrackPhase; var anchorPoint, previousPoint, nextPoint: VPoint; mouseDidMove: Boolean): TCommand;
				OVERRIDE;

				procedure TBetterFeedbackCmd.TrackFeedback (anchorPoint, nextPoint: VPoint; turnItOn, mouseDidMove: Boolean);
				OVERRIDE;

						{ Inspecting }

				procedure TBetterFeedbackCmd.Fields (procedure DoToField (fieldName: Str255; fieldAddr: Ptr; fieldType: INTEGER));
				OVERRIDE;

			end;

implementation

{[a-,body+,h-,o=100,r+,rec+,t=4,u+,#+,j=20/57/1$,n+]}
{ UBetterFeedbackCmd.inc1.p }
{ Copyright  1988-1990 by Apple Computer, Inc. All rights reserved.}

{--------------------------------------------------------------------------------------------------}

	type
		QElemWithA5AndCounterPtr = ^QElemWithA5AndCounter;
		QElemWithA5AndCounter = record
				OldA5: Longint;					{ A place to store the old value of A5 since}
{														 when debugging the compiler trashes the}
{														 value of A0 for any locals in the VBL task}
{														 thus making the pointer to the}
{														 paramblockrec unavailable}
				A5: Longint;					{ The value of A5 will be stored here to be}
{														 available at VBL time}
				q: QElem;						{ our VBL queue element }
			end;

{--------------------------------------------------------------------------------------------------}

	var
		gCounter: Longint;						{ our counter incremented in the VBL task }
		gOurVBLTask: QElemWithA5AndCounter;			{ our VBL task record }
		gSlot: INTEGER;						{ the slot our VBL is installed in }

{--------------------------------------------------------------------------------------------------}
{$Push}
 {$IFC qTrace}
 {$N+}
 {$ENDC}
{$D-}
{$R-}

{$V-}
{$S ARes}
	{ NOTE: must be resident! We *don't* want a segment load at interrupt time. }

	procedure IncOurVBLCounter;
{ our VBL task that increments the counter which is tested in WaitBetterFeedback }

		const
			theOffset = sizeof(Longint) * 2;

	begin
		gOurVBLTask.OldA5 := SetA5(QElemWithA5AndCounterPtr(GetParmBlockPtr - theOffset)^.A5);

		with gOurVBLTask do
			begin
				q.vblQElem.vblCount := 1;						{ reprime the pump }

				gCounter := gCounter + 1;						{ increment our counter }

				if SetA5(OldA5) = 0 then
					;						{ must discard the function result because}
{														 when A5 gets reset we have no place to put}
{														 the result}
			end;
	end;
{$Pop}

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

	procedure InstallOurVBL (aView: TView);

{ install a VBL task to increment a counter - used by TShapeCommand's TrackFeedback method }

		function GetAVideoSlot: INTEGER;

	{ return the video slot associated with the view's port }

			var
				aWindow: TWindow;
				aDefVideoRec: DefVideoRec;
				aGDHandle: GDHandle;
				aRect: Rect;
				aAuxDCEHandle: AuxDCEHandle;

		begin
			aWindow := aView.GetWindow;
			if (aWindow = nil) then
				begin
					GetVideoDefault(@aDefVideoRec);
					GetAVideoSlot := aDefVideoRec.sdSlot;
				end
			else
				begin
					aGDHandle := aWindow.GetMaxIntersectedDevice(aRect);
					aAuxDCEHandle := AuxDCEHandle(GetDCtlEntry(aGDHandle^^.gdRefNum));
					GetAVideoSlot := aAuxDCEHandle^^.dCtlSlot;
				end
		end;

	begin
		gCounter := 0;
		with gOurVBLTask.q.vblQElem do
			begin
				qType := ORD(vType);
				vblAddr := @IncOurVBLCounter;
				vblCount := 1;
				vblPhase := 0;
			end;
		gOurVBLTask.A5 := Longint(GetA5);
		if TrapExists(_SlotVInstall) then
			begin
				gSlot := GetAVideoSlot;
				FailOSErr(SlotVInstall(@gOurVBLTask.q, gSlot));
			end
		else
			FailOSErr(VInstall(@gOurVBLTask.q));
	end;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

	procedure RemoveOurVBL;

{ remove the VBL task we installed }

	begin
		if TrapExists(_SlotVRemove) then
			FailOSErr(SlotVRemove(@gOurVBLTask.q, gSlot))
		else
			FailOSErr(VRemove(@gOurVBLTask.q));
	end;

{--------------------------------------------------------------------------------------------------}
{$S ASelCommand}

	procedure TBetterFeedbackCmd.IBetterFeedbackCmd (itsCmdNumber: CmdNumber; itsDocument: TDocument; itsView: TView; itsScroller: TScroller; betterFeedbackDesired: BOOLEAN);

	begin
		ICommand(itsCmdNumber, itsDocument, itsView, itsScroller);
		fBetterFeedbackInstalled := not kInstall;
		fBetterFeedbackDesired := betterFeedbackDesired;
		fTracksMouse := TRUE;
	end;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

	procedure TBetterFeedbackCmd.Free;
		OVERRIDE;

	begin
		BetterFeedback(not kInstall);
		inherited Free;
	end;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

	function TBetterFeedbackCmd.TrackMouse (aTrackPhase: TrackPhase; var anchorPoint, previousPoint, nextPoint: VPoint; mouseDidMove: Boolean): TCommand;
		OVERRIDE;

	begin
		if aTrackPhase = trackRelease then
			BetterFeedback(not kInstall);
		TrackMouse := SELF;
	end;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

	procedure TBetterFeedbackCmd.TrackFeedback (anchorPoint, nextPoint: VPoint; turnItOn, mouseDidMove: Boolean);

	begin
		WaitBetterFeedback;
	end;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

	procedure TBetterFeedbackCmd.BetterFeedback (install: BOOLEAN);

{ use a VBL to improve TrackFeedback; eventually, this should go in TApplication.TrackMouse }

	begin
		if (fBetterFeedbackDesired = kBetterFeedbackDesired) then
			begin
				if (install = kInstall) & (fBetterFeedbackInstalled = not kInstall) then
					InstallOurVBL(fView)
				else if (install = not kInstall) & (fBetterFeedbackInstalled = kInstall) then
					RemoveOurVBL;
				fBetterFeedbackInstalled := install;
			end;
	end;

{--------------------------------------------------------------------------------------------------}
{$S MADoCommand}

	procedure TBetterFeedbackCmd.WaitBetterFeedback;
{ WaitBetterFeedback waits for our counter to change value }

		var
			t: Longint;

	begin
		if (fBetterFeedbackDesired = kBetterFeedbackDesired) then
			begin
				if (fBetterFeedbackInstalled = not kInstall) then
					BetterFeedback(kInstall);
				t := gCounter;									{ we'll wait til screen refresh }
				repeat
				until t <> gCounter;						{ let's hope this changes soon }
			end;
	end; {WaitBetterFeedback}

{--------------------------------------------------------------------------------------------------}
{$S AFields}

	procedure TBetterFeedbackCmd.Fields (procedure DoToField (fieldName: Str255; fieldAddr: Ptr; fieldType: INTEGER));
		OVERRIDE;

	begin
		DoToField('TBetterFeedbackCmd', nil, bClass);
		DoToField('fBetterFeedbackInstalled', @fBetterFeedbackInstalled, bBoolean);
		DoToField('fBetterFeedbackDesired', @fBetterFeedbackDesired, bBoolean);
		inherited Fields(DoToField);
	end;



end.