{	This file has been processed by The THINK Pascal Source Converter, v1.03.	}

{[a-,body+,h-,o=100,r+,rec+,t=4,u+,#+,j=20/57/1$,n-]}
{ UMenu.p }
{ Copyright  1989-1990 by Apple Computer Inc. All rights reserved. }

{[f-]}
(*
		T H E O R Y   O F	O P E R A T I O N

This is Larry Rosenstein's excellent unit: UMenu, that supports an
object-oriented way of defining custom menus.

Instead of writing a menu defproc, you simply create a subtype of TMenu
and override certain methods.  Not only is this easier to program,
but this unit takes care of some of the housekeeping that a menu
defproc would normally have to do.

Changes since MacApp 2.0: You can now add subviews to your subclass of TMenu, and
these subviews can have their own trackers.
This is accomplished in HandleChooseMessage by creating a dummy mouse down
event and passing it to the TMenu via the HandleMouseDown method which then
asks its subviews if any of them are interested in the mousedown (eg. to create
a tracker). The tracker should be posted via PostCommand, and the TMenu override
of PostCommand will check to see if the command is a tracker, in which case it
will track it immediately.
*)
{[f+]}

UNIT UMenu;

	INTERFACE

		USES    SysEqu,Traps,ULoMem,UMacAppUtilities,UPatch,UObject,UViewCoords,UMemory,UFailure,UMenuSetup,UList,
{  MacApp}
UMacApp 
{}
;												

		CONST
			kNoMenuItem 		= 0;					{ used to indicate that no item is selected
}

		TYPE

			MenuColors			= RECORD				{ a composite of the MCEntries }
				itemColor:			RGBColor;			{ the foreground color to draw the item with
														 (the color of the title if title or
														 menubar)}
				backgroundColor:	RGBColor;			{ the background color to draw the item with
														 (the color of the menubar if title or
														 menubar)}
				markColor:			RGBColor;			{ the foreground color to draw the mark with
														 (checkmarks, etc.)}
				commandColor:		RGBColor;			{ the foreground color to draw the command
														 key equivalent with (command-v for paste,
														 etc).}
				END;
			MenuColorsPtr		= ^MenuColors;
			MenuColorsHandle	= ^MenuColorsPtr;

			TMenu				= OBJECT (TView)
				fMenuHandle:		MenuHandle; 		{ handle to menu itself }
				fFlashInterval: 	LONGINT;			{ Time (in ticks) between flashes of item
														 highlighting; default is -1 which means
														 only change highlighting w when item
														 changes}
				fNextFlash: 		LONGINT;			{ Used internally }
				fMenuRect:			Rect;				{ menuRect that was last passed to the MDef
}
				fHitPt: 			Point;				{ hitPt that was last passed to the MDef.
														 gets offset in Focus.}
				fHighlighted:		BOOLEAN;			{ True if an item is highlighted }
				fBorder:			Rect;				{ Added to menuRect to get actual hit rect;
														 FindItem will not be called if the mouse
														 is in the border.}

								{ Initialization }

				PROCEDURE TMenu.IMenu(rsrcID: INTEGER;
									  menuWidth, menuHeight: INTEGER);
				{ IMenu reads the menu and sets up the MenuHandle and defproc
				  so that the Menu Manager can communicate with the TMenu object.
}

				{ Menu Definition; all these methods must be overridden }

				FUNCTION TMenu.FindItem(hitPt: Point): INTEGER;
				{ This is called to convert a point to an item number. }

				PROCEDURE TMenu.PostCommand(command: TCommand); OVERRIDE;
				{ ensure that trackers get tracked immediately }

				PROCEDURE TMenu.Highlight(whichItem: INTEGER;
										  turnItOn: BOOLEAN);
				{ Given an item number as returned by FindItem, this is called
				to highlight the item.	This will not be called with
				whichItem = kNoMenuItem.}

				{ Other methods }

				PROCEDURE TMenu.GetMenuColors(theMenu, theItem: INTEGER;
											  VAR theMenuColors: MenuColors);
				{ Called to get the appropriate menu colors for the menu and item.
				If the menu and item are 0 then info is for the menubar.
				If item is 0 then info is for the menu title.
				Attempts to return best guess at colors based on all available info.}

				FUNCTION TMenu.IsItemEnabled(item: INTEGER): BOOLEAN;
				{ Returns the enabled status of an item }

				PROCEDURE TMenu.UpdateHighlight(oldItem, newItem: INTEGER);
				{ Called to update menu item highlighting. }

				{ Private }

				PROCEDURE TMenu.HandleDefproc(message: INTEGER;
											  theMenu: MenuHandle;
											  VAR menuRect: Rect;
											  hitPt: Point;
											  VAR whichItem: INTEGER);
				{ This is called by the custom defproc; it does a CASE
				on the message and calls 1 or more of the methods above.}

				PROCEDURE TMenu.HandleChooseMessage(message: INTEGER;
													theMenu: MenuHandle;
													VAR menuRect: Rect;
													hitPt: Point;
													VAR whichItem: INTEGER);
				{ Called by the message dispatcher when a choose message has been sent }

				PROCEDURE TMenu.HandleDrawMessage(message: INTEGER;
												  theMenu: MenuHandle;
												  VAR menuRect: Rect;
												  hitPt: Point;
												  VAR whichItem: INTEGER);
				{ Called by the message dispatcher when a draw message has been sent }

				PROCEDURE TMenu.HandleSizeMessage(message: INTEGER;
												  theMenu: MenuHandle;
												  VAR menuRect: Rect;
												  hitPt: Point;
												  VAR whichItem: INTEGER);
				PROCEDURE TMenu.HandlePopUpMessage(message: INTEGER;
												   theMenu: MenuHandle;
												   VAR menuRect: Rect;
												   hitPt: Point;
												   VAR whichItem: INTEGER);
				{ Focusing Methods }

				FUNCTION TMenu.Focus: BOOLEAN; OVERRIDE;

				FUNCTION TMenu.FocusOnSuperView: BOOLEAN; OVERRIDE;

				{ Misc. Methods }

				FUNCTION TMenu.GetGrafPort: GrafPtr; OVERRIDE;

				PROCEDURE TMenu.CreateMouseDownEventInfo(hitPt: Point;
														 VAR anEventRecord: EventRecord;
														 VAR info: EventInfo);
				{ Called by HandleChooseMessage when creating a phony mousedown event which is then
				passed off to the subviews of TMenu via HandleMouseDown}

				{ Debugging Methods }

				PROCEDURE TMenu.Fields(PROCEDURE DoToField(fieldName: Str255;
														   fieldAddr: Ptr;
														   fieldType: INTEGER)); OVERRIDE;

				END;

		PROCEDURE InitUMenu;
		{ Call this at the start of your program, before creating any TMenu objects.
		This can signal Failure.}

		VAR
			gTrackingInMenu: BOOLEAN;

	IMPLEMENTATION

{[a-,body+,h-,o=100,r+,rec+,t=4,u+,#+,j=20/57/1$,n-]}

TYPE
	MenuRec 			= RECORD
		mID:				INTEGER;
		mObject:			TMenu;
		END;
	MenuArray			= ARRAY [1..4000] OF MenuRec;
	MenuArrayPtr		= ^MenuArray;
	MenuArrayHandle 	= ^MenuArrayPtr;

VAR
	pMenuCPort: 		CGrafPort;						{ Color port for compatibility. }
														{ Private grafPort used to focus the menu w/o
														 messing up the Window Manager port.}
	pMenuArray: 		MenuArrayHandle;				{ Used to find the TMenu given a MenuHandle.
}
	pNumMenus:			INTEGER;

	pCustDefproc:		Handle; 						{ Replaces the menu's menuProc field }

{--------------------------------------------------------------------------------------------------}
	{ Returns the TickCount some time in the future. }

FUNCTION Future(delta: LONGINT): LONGINT;

	BEGIN
	Future := TickCount + delta;
	END;

{--------------------------------------------------------------------------------------------------}

{$S ARes}

PROCEDURE WaitTickChange;

	VAR
		now:				LONGINT;

	BEGIN
	now := TickCount;
	REPEAT
	UNTIL TickCount <> now;
	END;

{--------------------------------------------------------------------------------------------------}

{$S ARes}

FUNCTION FindTMenu(theMenu: MenuHandle): TMenu;

	VAR
		i:					INTEGER;
		p:					MenuArrayPtr;
		id: 				INTEGER;

	BEGIN
	FindTMenu := NIL;
	p := pMenuArray^;
	id := theMenu^^.menuID;

	FOR i := 1 TO pNumMenus DO
		WITH p^[i] DO
			IF mID = id THEN
				BEGIN
				FindTMenu := mObject;
				Exit(FindTMenu);
				END;
	END;

{--------------------------------------------------------------------------------------------------}
{ Called by the MDEF resource. }

{$S ARes}

PROCEDURE MenuDefproc(message: INTEGER;
					  theMenu: MenuHandle;
					  VAR menuRect: Rect;
					  hitPt: Point;
					  VAR whichItem: INTEGER);

	VAR
		menuObj:			TMenu;

	BEGIN
	menuObj := FindTMenu(theMenu);
	{$IFC qDebug}
	IF menuObj = NIL THEN
		ProgramBreak('MenuDefproc called with no TMenu object');
	{$ENDC}

	{ Dispatch to the TMenu object }
	menuObj.HandleDefproc(message, theMenu, menuRect, hitPt, whichItem);
	END;

{--------------------------------------------------------------------------------------------------}
{$S AInit}

PROCEDURE InitUMenu;

	TYPE
		JMP 				= RECORD
			opcode: 			INTEGER;
			address:			Ptr;
			END;
		JmpPtr				= ^JMP;
		JmpHandle			= ^JmpPtr;

	VAR
		h:					JmpHandle;

	BEGIN
	IF qNeedsColorQD | gConfiguration.hasColorQD THEN
		OpenCPort(@pMenuCPort)
	ELSE
		OpenPort(GrafPtr(@pMenuCPort));
	pNumMenus := 0;
	pMenuArray := MenuArrayHandle(NewHandle(0));
	FailNIL(pMenuArray);

	h := JmpHandle(NewHandle(6));
	FailNIL(h);
	WITH h^^ DO
		BEGIN
		opcode := $4EF9;
		address := @MenuDefproc;
		END;
	pCustDefproc := Handle(h);

	gTrackingInMenu := FALSE;
	END;

{--------------------------------------------------------------------------------------------------}
{$S AInit}

PROCEDURE TMenu.IMenu(rsrcID: INTEGER;
					  menuWidth, menuHeight: INTEGER);

	VAR
		m:					MenuHandle;
		s:					LONGINT;
		p:					LongintPtr;
		i:					INTEGER;

		r:					Rect;
		item:				INTEGER;
		vp: 				VPoint;

	BEGIN
	{ Initialize fields }
	fBorder := gZeroRect;
	fMenuHandle := NIL;
	vp.h := menuWidth;
	vp.v := menuHeight;
	IView(NIL, NIL, gZeroVPt, vp, SizeVariable, SizeVariable);

	fFlashInterval := - 1;
	fNextFlash := 0;

	IF rsrcID = 0 THEN
		fMenuHandle := NIL
	ELSE
		BEGIN
		{ Read in menu and set its defproc }
		m := MenuHandle(GetResMenu(rsrcID));

		IF m = NIL THEN
			BEGIN
			{$IFC qDebug}
			Writeln('rsrcID = ', rsrcID: 1);
			ProgramBreak('No such MENU!');
			{$ENDC}

			Free;
			Failure(resNotFound, 0);
			END;

		pNumMenus := pNumMenus + 1;
		SetHandleSize(Handle(pMenuArray), SIZEOF(MenuRec) * pNumMenus);
		WITH pMenuArray^^[pNumMenus] DO
			BEGIN
			mID := m^^.menuID;
			mObject := SELF;
			END;

		m^^.menuProc := pCustDefproc;
		fMenuHandle := m;

		r := gZeroRect;
		item := 0;
		MenuDefproc(mSizeMsg, m, r, Point(0), item);	{ recompute the menu size }
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S MenuNever}

FUNCTION TMenu.FindItem(hitPt: Point): INTEGER;

	BEGIN
	{$IFC qDebug}
	ProgramBreak('You must override TMenu.FindItem.');
	{$ENDC}
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TMenu.PostCommand(command: TCommand); OVERRIDE;
{ ensure that trackers get tracked immediately }

	BEGIN
	IF (command <> NIL) & command.fTracksMouse THEN
		BEGIN
		gEventLevel := gEventLevel + 1;
		gApplication.PerformCommand(command);
		gEventLevel := gEventLevel - 1;
		END
	ELSE
		INHERITED PostCommand(command);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TMenu.HandleDefproc(message: INTEGER;
							  theMenu: MenuHandle;
							  VAR menuRect: Rect;
							  hitPt: Point;
							  VAR whichItem: INTEGER);

	VAR
		p:					LongintPtr;
		savePort:			GrafPtr;
		r:					Rect;

	BEGIN
	{ Save the wmgr port & set our private port }
	GetPort(savePort);

	fMenuRect := menuRect;
	fHitPt := hitPt;

	IF Focus THEN
		BEGIN
		hitPt := fHitPt;

		ViewEnable(Odd(fMenuHandle^^.enableFlags), false {no redraw} );
		CASE message OF
			mDrawMsg:
				BEGIN
				{$IFC qDebug}
				IF gIntenseDebugging THEN
					Writeln('mDrawMsg');
				{$ENDC}
				HandleDrawMessage(message, theMenu, menuRect, hitPt, whichItem);
				END;

			mChooseMsg:
				BEGIN
				{$IFC qDebug}
				IF gIntenseDebugging THEN
					Writeln('mChooseMsg');
				{$ENDC}
				HandleChooseMessage(message, theMenu, menuRect, hitPt, whichItem);
				END;

			mSizeMsg:
				BEGIN
				{$IFC qDebug}
				IF gIntenseDebugging THEN
					Writeln('mSizeMsg');
				{$ENDC}
				HandleSizeMessage(message, theMenu, menuRect, hitPt, whichItem);
				END;

			mPopUpMsg:
				BEGIN
				{$IFC qDebug}
				IF gIntenseDebugging THEN
					Writeln('mPopUpMsg');
				{$ENDC}
				HandlePopUpMessage(message, theMenu, menuRect, hitPt, whichItem);
				END;
			{$IFC qDebug}
			OTHERWISE
				IF gIntenseDebugging THEN
					Writeln('otherwise message');
			{$ENDC}
		END;
		InvalidateFocus;
		END;

	SetPort(savePort);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TMenu.HandleChooseMessage(message: INTEGER;
									theMenu: MenuHandle;
									VAR menuRect: Rect;
									hitPt: Point;
									VAR whichItem: INTEGER);

	VAR
		newItem:			INTEGER;
		hitRect: Rect;
		hitVPt: VPoint;
		hysteresis: Point;
		info: EventInfo;
		anEventRecord: EventRecord;
		theCommand: TCommand;
		saveTrackingInMenu: BOOLEAN;

	BEGIN
	newItem := kNoMenuItem; 							{ default return }

	{ so that trackers get a chance to know that they're tracking in menus }
	saveTrackingInMenu := gTrackingInMenu;
	gTrackingInMenu := TRUE;

	IF IsViewEnabled THEN								{ menu enabled }
		BEGIN
		{ see if point is within hit area }
		GetQDExtent(hitRect);
		AddPt(fBorder.topLeft, hitRect.topLeft);
		AddPt(fBorder.botRight, hitRect.botRight);

		IF PtInRect(hitPt, hitRect) THEN				{ in menu (not border) }
			BEGIN
			{ NOTE: Either your subclass of TTearOffMenu should override DoMouseCommand or one of 
			TTearOffMenu's view's subview's should override DoMouseCommand so that it
			creates and posts a TCommand. TTearOffMenu's override of PostCommand will
			ensure that the tracker is tracked immediately. The tracker, having been posted and
			tracked, will then get executed next time PerformCommand is called. So, when we're
			done with HandleMouseDown below, we simply tell the menu manager that no menu item
			was selected, ie newItem = kNoMenuItem.}
			
			PtToVPt(hitPt, hitVPt);
			CreateMouseDownEventInfo(hitPt, anEventRecord, info);
			hysteresis := gStdHysteresis;
			IF NOT HandleMouseDown(hitVPt, info, hysteresis, theCommand) THEN
				BEGIN
				{ NOTE: for backwards compatibility with TMenu in MacApp 2.0 - override
				HandleMouseDown method to return FALSE, and all should work as before}

				newItem := FindItem(hitPt);
				UpdateHighlight(whichItem, newItem);			{ Update highlighting }
				END
			ELSE
				IF (theCommand <> NIL) THEN
					PostCommand(theCommand);
			END;
		END
	ELSE
		UpdateHighlight(whichItem, newItem);	{ Update highlighting }

	gTrackingInMenu := saveTrackingInMenu;

	{ Tell MenuManager about new item }
	whichItem := newItem;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TMenu.HandleDrawMessage(message: INTEGER;
								  theMenu: MenuHandle;
								  VAR menuRect: Rect;
								  hitPt: Point;
								  VAR whichItem: INTEGER);
	var
		extent: Rect;

	BEGIN
	DrawContents;
	fHighlighted := false;
	if not fViewEnabled THEN
		begin
		PenPat(Gray);
		PenMode(notSrcBic);
		GetQDExtent(extent);
		PaintRect(extent);
		end;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TMenu.HandleSizeMessage(message: INTEGER;
								  theMenu: MenuHandle;
								  VAR menuRect: Rect;
								  hitPt: Point;
								  VAR whichItem: INTEGER);

	VAR
		vp: 				VPoint;

	BEGIN
	ComputeSize(vp);
	fMenuHandle^^.menuWidth := vp.h;
	fMenuHandle^^.menuHeight := vp.v;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TMenu.HandlePopUpMessage(message: INTEGER;
								   theMenu: MenuHandle;
								   VAR menuRect: Rect;
								   hitPt: Point;
								   VAR whichItem: INTEGER);

	VAR
		vp: 				VPoint;

	BEGIN
	{  SubPt(origin, hitPt);}

	menuRect.top := hitPt.h;
	menuRect.left := hitPt.v;
	ComputeSize(vp);
	menuRect.bottom := menuRect.top + vp.v;
	menuRect.right := menuRect.left + vp.h;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TMenu.Highlight(whichItem: INTEGER;
						  turnItOn: BOOLEAN);

	BEGIN
	{$IFC qDebug}
	ProgramBreak('You must override TMenu.Highlight.');
	{$ENDC}
	END;

{--------------------------------------------------------------------------------------------------}
{$S AFields}

PROCEDURE TMenu.Fields(PROCEDURE DoToField(fieldName: Str255;
										   fieldAddr: Ptr;
										   fieldType: INTEGER)); OVERRIDE;

	BEGIN
	DoToField('fFlashInterval', @fFlashInterval, bLongInt);
	DoToField('fNextFlash', @fNextFlash, bLongInt);
	DoToField('fMenuRect', @fMenuRect, bRect);
	DoToField('fHitPt', @fHitPt, bPoint);
	DoToField('fHighlighted', @fHighlighted, bBoolean);
	DoToField('fMenuHandle', @fMenuHandle, bHandle);
	DoToField('fBorder', @fBorder, bRect);

	INHERITED Fields(DoToField);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

FUNCTION TMenu.IsItemEnabled(item:INTEGER): Boolean;

	BEGIN
	IsItemEnabled := BTst(fMenuHandle^^.enableFlags, item)
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TMenu.GetMenuColors(theMenu, theItem: INTEGER; VAR theMenuColors: MenuColors);

	TYPE
		TypeOfMenuInfo = (aMenuItem, aMenuTitle, aMenuBar, noType);

	VAR
		aMCEntryPtr: MCEntryPtr;
		typeOfRequest: TypeOfMenuInfo;
		typeOfEntryFound: TypeOfMenuInfo;
		theEntryMenu, theEntryItem: INTEGER;

	BEGIN
	WITH theMenuColors DO
		BEGIN
		IF qNeedsColorQD | gConfiguration.hasColorQD THEN
			BEGIN
			IF theItem <> 0 THEN
				typeOfRequest := aMenuItem
			ELSE IF theMenu <> 0 THEN
				typeOfRequest := aMenuTitle
			ELSE
				typeOfRequest := aMenuBar;

			theEntryMenu := theMenu;
			theEntryItem := theItem;

			aMCEntryPtr := GetMCEntry(theEntryMenu, theEntryItem);
			IF aMCEntryPtr = NIL THEN { not found, try as title }
				BEGIN
				theEntryItem := 0;
				aMCEntryPtr := GetMCEntry(theEntryMenu, theEntryItem);
				IF aMCEntryPtr = NIL THEN { not found, try as menubar }
					BEGIN
					theEntryMenu := 0;
					aMCEntryPtr := GetMCEntry(theEntryMenu, theEntryItem);
					END;
				END;

			IF aMCEntryPtr = NIL THEN
				typeOfEntryFound := noType
			ELSE
				BEGIN
				IF theEntryItem <> 0 THEN
					typeOfEntryFound := aMenuItem
				ELSE IF theEntryMenu <> 0 THEN
					typeOfEntryFound := aMenuTitle
				ELSE
					typeOfEntryFound := aMenuBar;
				END;

			CASE typeOfEntryFound OF
				aMenuItem:
					WITH aMCEntryPtr^ DO
						BEGIN
						itemColor := mctRGB1;
						backgroundColor := mctRGB4;
						markColor := mctRGB1;
						commandColor := mctRGB1;
						END;
				aMenuTitle:
					CASE typeOfRequest OF
						aMenuItem:
							WITH aMCEntryPtr^ DO
								BEGIN
								itemColor := mctRGB3;
								backgroundColor := mctRGB4;
								markColor := mctRGB3;
								commandColor := mctRGB3;
								END;
						aMenuTitle:
							WITH aMCEntryPtr^ DO
								BEGIN
								itemColor := mctRGB1;
								backgroundColor := mctRGB2;
								markColor := mctRGB1;
								commandColor := mctRGB1;
								END;
					END;
				aMenuBar:
					CASE typeOfRequest OF
						aMenuItem:
							WITH aMCEntryPtr^ DO
								BEGIN
								itemColor := mctRGB3;
								backgroundColor := mctRGB2;
								markColor := mctRGB3;
								commandColor := mctRGB3;
								END;
						aMenuTitle:
							WITH aMCEntryPtr^ DO
								BEGIN
								itemColor := mctRGB1;
								backgroundColor := mctRGB4;
								markColor := mctRGB1;
								commandColor := mctRGB1;
								END;
						aMenuBar:
							WITH aMCEntryPtr^ DO
								BEGIN
								itemColor := mctRGB1;
								backgroundColor := mctRGB4;
								markColor := mctRGB1;
								commandColor := mctRGB1;
								END;
					END;
				noType:
					BEGIN
					itemColor := gRGBBlack;
					backgroundColor := gRGBWhite;
					markColor := gRGBBlack;
					commandColor := gRGBBlack;
					END;
			END;
			END
		ELSE
			BEGIN
			itemColor := gRGBBlack;
			backgroundColor := gRGBWhite;
			markColor := gRGBBlack;
			commandColor := gRGBBlack;
			END;
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TMenu.UpdateHighlight(oldItem, newItem: INTEGER);

	BEGIN
	{ Update highlighting }
	IF newItem <> oldItem THEN
		BEGIN
		IF fHighlighted THEN
			IF oldItem <> kNoMenuItem THEN
				Highlight(oldItem, false);

		fHighlighted := newItem <> kNoMenuItem;
		IF fHighlighted THEN
			Highlight(newItem, TRUE);

		IF fFlashInterval >= 0 THEN
			fNextFlash := Future(fFlashInterval);
		END

	ELSE IF fFlashInterval >= 0 THEN
		IF TickCount > fNextFlash THEN
			BEGIN
			fHighlighted := NOT fHighlighted;
			Highlight(oldItem, fHighlighted);
			fNextFlash := Future(fFlashInterval);
			END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

FUNCTION TMenu.Focus: BOOLEAN; OVERRIDE;

	VAR
		r:					Rect;
		vorigin:			VPoint;
		origin: 			Point;
		{$IFC qDebug}
		currentPort:		GrafPtr;
		{$ENDC}
		theMenuColors:		MenuColors;

	BEGIN
	IF IsFocused THEN
		BEGIN
		{$IFC FALSE}
		IF LONGINT(pMenuCPort.portRect.topLeft) <> 0 THEN
			ProgramBreak('TMenu.Focus: Origin is not (0,0)');

		GetPort(currentPort);
		IF currentPort <> @pMenuCPort THEN
			ProgramBreak('TMenu.Focus: Port is incorrect');
		{$ENDC}
		END

	ELSE												{IF @pMenuCPort <> NIL THEN}
		BEGIN
		IF qNeedsColorQD | gConfiguration.hasColorQD THEN
			InitCPort(@pMenuCPort)
		ELSE
			InitPort(GrafPtr(@pMenuCPort)); 			{ set the port to default settings }

		SetPort(@pMenuCPort);
		gLongOffset := gZeroVPt;



		{ Try to make the best match for the menu colors without requiring programmer intervention.
		by setting the color environment to be for items.}

		GetMenuColors(fMenuHandle^^.menuID, 1, theMenuColors);
		SetIfColor(theMenuColors.itemColor);
		SetIfBkColor(theMenuColors.backgroundColor);

		{ Change the origin so that drawing is relative to fLocation }
		{$Push}
		origin := VPtToPt(fLocation);
		{$Pop}

		SubPt(fMenuRect.topLeft, origin);
		SetOrigin(origin.h, origin.v);

		{$Push}
		AddPt(origin, fHitPt);
		OffsetRect(fMenuRect, origin.h, origin.v);
		ClipRect(fMenuRect);
		{$Pop}
		gFocusedView := SELF;
		END;
	Focus := TRUE;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TMenu.CreateMouseDownEventInfo(hitPt: Point; VAR anEventRecord: EventRecord; VAR info: EventInfo);

	VAR
		hitVPt: VPoint;

	FUNCTION IsCommandKeyDown: Boolean;
	
		CONST
			kCommandKey 		= 55;
	
		VAR
			theKeys:			KeyMap;
	
		BEGIN
		GetKeys(theKeys);
		IsCommandKeyDown := theKeys[kCommandKey];
		END;
	
	FUNCTION IsOptionKeyDown: Boolean;
	
		CONST
			kOptionKey			= 58;
	
		VAR
			theKeys:			KeyMap;
	
		BEGIN
		GetKeys(theKeys);
		IsOptionKeyDown := theKeys[kOptionKey];
		END;

	BEGIN
	{ !!! it would be nice to ask the application for these values from the last EventInfo }
	PtToVPt(hitPt, hitVPt);
	LocalToWindow(hitVPt);
	hitPt := VPtToPt(hitVPt);
	LocalToGlobal(hitPt);
	WITH anEventRecord DO
		BEGIN
		what := mouseDown;
		message := 0;
		when := TickCount;
		where := hitPt;
		modifiers := 0;
		END;

	WITH info DO
		BEGIN
		thePEvent := @anEventRecord;
		theBtnState := Button;
		theCmdKey := IsCommandKeyDown;
		theShiftKey := FALSE;
		theAlphaLock := FALSE;
		theOptionKey := IsOptionKeyDown;
		theControlKey := FALSE;
		theAutoKey := FALSE;
		theCharacter := Char(0);
		theKeyCode := 0;
		theClickCount := gClickCount;
		affectsMenus := FALSE;
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

FUNCTION TMenu.FocusOnSuperView: BOOLEAN; OVERRIDE;

	BEGIN
	FocusOnSuperView := false;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

FUNCTION TMenu.GetGrafPort: GrafPtr; OVERRIDE;

	BEGIN
	GetGrafPort := GrafPtr(@pMenuCPort);
	END;


END.
