/*********************************************************************************/
/* IconEdit.r																	 */
/*																				 */
/* Copyright  1989 Apple Computer, Inc.  All rights reserved. 			 		 */
/* 																				 */
/*********************************************************************************/

/* ==============================   Includes   ================================= */

#ifndef _TYPES.R_
#include "Types.r"
#endif

#ifndef _MacAppTypes_
#include "MacAppTypes.r"
#endif

#ifndef __ViewTypes__
#include "ViewTypes.r"
#endif

#if qDebug
include "Debug.rsrc";
#endif
include "MacApp.rsrc";

include "IconEdit" 'CODE';


/* ==============================   Constants   ================================ */

#define cZoomIn				1000
#define cZoomOut			1001
#define cInvert				1002
#define cDrawCommand		1003

#define kSeedIconId			1000

#define kIconWindowId		1000

#define kIconWindowId		1000
#define kIconViewId			1001

#define kIconViewHeight		32 * 7 + 10	 /* 7 times actual size, 5 pixel borders */
#define kIconViewWidth		32 * 7 + 10	 /* 7 times actual size, 5 pixel borders */

#define kIconWindowHeight	kIconViewHeight + kSBarSizeMinus1
#define kIconWindowWidth	kIconViewWidth + kSBarSizeMinus1



/* ==============================   Seed Icon   ================================ */

resource 'ICON' (kSeedIconId, purgeable) {
	$"0001 0000 0002 8000 0004 4000 0008 2000 0010 1000 0020 0800 0040 0400 0080 0200"
	$"0100 0100 0200 0080 0400 0040 0800 0020 1000 0010 2000 0008 4000 3F04 8000 4082"
	$"4000 8041 2001 3022 1001 C814 080E 7F8F 0402 3007 0201 0007 0100 8007 0080 6007"
	$"0040 1FE7 0020 021F 0010 0407 0008 0800 0004 1000 0002 2000 0001 4000 0000 8000"
};




/* =============================   Icon Window   =============================== */

resource 'view' (kIconWindowId, purgeable) {
	{
		root, 'WIND', { 50, 20 }, { kIconWindowHeight, kIconWindowWidth },
		sizeVariable, sizeVariable, shown, enabled,
		Window		{ "", zoomDocProc, goAwayBox, resizable, modeless,
					      ignoreFirstClick, freeOnClosing, disposeOnFree, closesDocument,
					      openWithDocument, dontAdaptToScreen, stagger, forceOnScreen,
					      dontCenter, 'ICON', "" };
					
		'WIND',  'SCLR',	{ 0, 0 },
		{ kIconWindowHeight-kSBarSizeMinus1, kIconWindowWidth-kSBarSizeMinus1 },
		sizeRelSuperView, sizeRelSuperView, shown, enabled,
		Scroller	{ "", vertScrollBar, horzScrollBar, 0, 0, 16, 16,
					      vertConstrain, horzConstrain, { 0, 0, 0, 0 } };

		'SCLR', IncludeViews	{ kIconViewId }
	}
};

resource 'view' (kIconViewId, purgeable) {
	{
		root, 'ICON', { 0, 0 }, { kIconViewHeight, kIconViewWidth },
		sizeVariable, sizeVariable, shown, enabled,
		View		{ "TIconView"}
	}
};


/* NOTE: if you used ViewEdit, then replace the above 'view' resources with:     */
/* include "Views.rsrc" 'view';                                                  */




/* ================================   Menus   ================================== */

resource 'cmnu' (mApple) {
	1,
	textMenuProc,
	0x7FFFFFFD,
	enabled,
	apple,
	 {
	/* [1] */	"About IconEdit",	noIcon,	noKey,	noMark,	plain,	cAboutApp;
	/* [2] */	"-",				noIcon,	noKey,	noMark,	plain,	nocommand
	}
};


/* ------------------------------------------------------------------------------ */

resource 'cmnu' (mFile) {
	2,
	textMenuProc,
	allEnabled,
	enabled,
	"File",
	 {
	/* [1] */	"New",				noIcon,	"N",	noMark,	plain,	cNew;
	/* [2] */	"Open",			noIcon,	"O",	noMark,	plain,	cOpen;
	/* [3] */	"-",				noIcon,	noKey,	noMark,	plain,	nocommand;
	/* [4] */	"Close",			noIcon,	noKey,	noMark,	plain,	cClose;
	/* [5] */	"Save",				noIcon,	noKey,	noMark,	plain,	cSave;
	/* [6] */	"Save As",			noIcon,	noKey,	noMark,	plain,	cSaveAs;
	/* [7] */	"Revert to Saved",	noIcon,	noKey,	noMark,	plain,	cRevert;
	/* [8] */	"-",				noIcon,	noKey,	noMark,	plain,	nocommand;
	/* [9] */	"Page Setup",		noIcon,	noKey,	noMark,	plain,	cPageSetup;
	/* [10] */	"Print",			noIcon,	noKey,	noMark,	plain,	cPrint;
	/* [11] */	"-",				noIcon,	noKey,	noMark,	plain,	nocommand;
	/* [12] */	"Quit",				noIcon,	"Q",	noMark,	plain,	cQuit
	}
};


/* ------------------------------------------------------------------------------ */

resource 'cmnu' (mEdit) {
	3,
	textMenuProc,
	allEnabled,
	enabled,
	"Edit",
	 {
	/* [1] */	"Undo",				noIcon,	"Z",	noMark,	plain,	cUndo;
	/* [2] */	"-",				noIcon,	noKey,	noMark,	plain,	nocommand;
	/* [3] */	"Cut",				noIcon,	"X",	noMark,	plain,	cCut;
	/* [4] */	"Copy",				noIcon,	"C",	noMark,	plain,	cCopy;
	/* [5] */	"Paste",			noIcon,	"V",	noMark,	plain,	cPaste;
	/* [6] */	"Clear",			noIcon,	noKey,	noMark,	plain,	cClear;
	/* [7] */	"-",				noIcon,	noKey,	noMark,	plain,	nocommand;
	/* [8] */	"Show Clipboard",	noIcon,	noKey,	noMark,	plain,	cShowClipboard
	}
};


/* ------------------------------------------------------------------------------ */

resource 'cmnu' (4) {
	4,
	textMenuProc,
	allEnabled,
	enabled,
	"Icon",
	 {
	/* [1] */	"Zoom In", 			noIcon, "M",	noMark, plain,	cZoomIn;
	/* [2] */	"Zoom Out", 		noIcon, "L", 	noMark, plain,	cZoomOut;
	/* [3] */	"-",				noIcon,	noKey,	noMark,	plain,	nocommand;
	/* [4] */	"Invert", 			noIcon, "I", 	noMark, plain,	cInvert
	}
};


/* ------------------------------------------------------------------------------ */

resource 'cmnu' (mBuzzWords) {
	mBuzzWords,
	textMenuProc,
	allEnabled,
	enabled,
	"Buzzwords",
	 {
	/* [1] */	"Page Setup Change", noIcon, noKey, noMark, plain, cChangePrinterStyle;
	/* [2] */	"Drawing", noIcon, noKey, noMark, plain, cDrawCommand
	}
};


/* ------------------------------------------------------------------------------ */

resource 'MBAR' (128) { {mApple; mFile; mEdit; 4} };
