Unit UDesignDocument;

INTERFACE

USES
	UMacApp,
	ToolUtils,
	Resources,
	Packages,
	UPrinting,
	UTEView,
	UDialog,
	UGridView,
	UCRCCards,
	UPopText,
	UItem,
	UCard,
	UMyViews,
	UMyCommands,
	UTextHandler;

TYPE
	AccessorPreferences = RECORD
		fMakeGetField,
		fMakeSetField,
		fMakeGetCollaborator,
		fMakeSetCollaborator: BOOLEAN;
	END;
	
	TDesignDocument = OBJECT(TDocument)
		fCurrentCard: TCard;
		fCardList: TList;
		fDesignView: TDesignView;
		fTextHandler: TTextHandler;
		fUnitName : Str255;
		fMPWCodeView: TMyTEView;
		fMPWIncCodeView: TMyTEView;
		fAccessorPreferences: AccessorPreferences;
		fChgCountAtLastWriteCode: LongInt;  {the changes since the last time we wrote the code}

		PROCEDURE TDesignDocument.IDesignDocument;
		PROCEDURE TDesignDocument.SetCurrentCard(theCard: TCard);
		FUNCTION TDesignDocument.GetCurrentCard: TCard;
		PROCEDURE TDesignDocument.SetCardList(theCardList: TList);
		FUNCTION TDesignDocument.GetCardList: TList;
		FUNCTION TDesignDocument.GetNumCardsStr: str255;
		
		FUNCTION TDesignDocument.GetDesignView: TDesignView;
		PROCEDURE TDesignDocument.SetDesignView(aDesignView: TDesignView);
		FUNCTION TDesignDocument.GetTextHandler: TTextHandler;
		PROCEDURE TDesignDocument.SetTextHandler(theTextHandler: TTextHandler);
		FUNCTION TDesignDocument.GetMPWCodeView: TMyTEView;
		PROCEDURE TDesignDocument.SetMPWCodeView(theMPWCodeView: TMyTEView);
		FUNCTION TDesignDocument.GetMPWIncCodeView: TMyTEView;
		PROCEDURE TDesignDocument.SetMPWIncCodeView(theMPWIncCodeView: TMyTEView);
		FUNCTION TDesignDocument.GetUnitName:Str255;
		PROCEDURE TDesignDocument.SetUnitName(theUnitName: Str255);
		PROCEDURE TDesignDocument.SetChgCountAtLastWriteCode(theCount: LongInt);
		FUNCTION TDesignDocument.GetChgCountAtLastWriteCode: LongInt;
		FUNCTION TDesignDocument.DesDocChangedSinceWriteCode: BOOLEAN;
		PROCEDURE TDesignDocument.SetTitle(aTitle: Str255); OVERRIDE;
		FUNCTION TDesignDocument.GetPreferences: AccessorPreferences;
		
		PROCEDURE TDesignDocument.AddCard(aCard: TCard);
		PROCEDURE TDesignDocument.AddCardAtIndex(aCard: TCard; theIndex: ArrayIndex);
		PROCEDURE TDesignDocument.AddCardToObjectLists(theCardToAdd: TCard);
		PROCEDURE TDesignDocument.RemoveCardFromObjectLists(theCardToRemove: TCard);
		PROCEDURE TDesignDocument.ChangeNameInObjectLists(theOldName, theNewName: Str255);
		PROCEDURE TDesignDocument.DeleteCard(aCard: TCard);
		FUNCTION TDesignDocument.GetNumCards: ArrayIndex;
		FUNCTION TDesignDocument.GetIndexOfCard(theCard: TCard): ArrayIndex;
		PROCEDURE TDesignDocument.DoMakeViews(forPrinting: BOOLEAN); OVERRIDE;
		PROCEDURE TDesignDocument.CloseOldCodeViews;
		PROCEDURE TDesignDocument.OpenNewCodeViews;
		PROCEDURE TDesignDocument.SelectCurrentCodeViews;
		PROCEDURE TDesignDocument.WriteInterface(theTEView: TMyTEView);
		PROCEDURE TDesignDocument.WriteImplementation(theTEView: TMyTEView);
		FUNCTION TDesignDocument.GetUnitNameFromUser: BOOLEAN;
		PROCEDURE TDesignDocument.WriteCode;
		PROCEDURE TDesignDocument.GetNewPreferences;
		PROCEDURE TDesignDocument.ReadPreferencesFromDisk;
		PROCEDURE TDesignDocument.WritePreferencesToDisk;
		PROCEDURE TDesignDocument.FreeData; OVERRIDE;
		PROCEDURE TDesignDocument.Free; OVERRIDE;
		PROCEDURE TDesignDocument.AboutToSave(itsCmd: CmdNumber; VAR newName: Str255;
																					VAR newVolRefnum: INTEGER; 
																					VAR makingCopy: BOOLEAN); OVERRIDE;
		PROCEDURE TDesignDocument.DoNeedDiskSpace(VAR dataForkBytes, rsrcForkBytes: LongInt); 
				OVERRIDE;
		PROCEDURE TDesignDocument.DoWrite(aRefNum: INTEGER; makingCopy: BOOLEAN); OVERRIDE;
		PROCEDURE TDesignDocument.DoRead(aRefNum: INTEGER; rsrcExists, forPrinting: BOOLEAN); 
				OVERRIDE;
		FUNCTION  TDesignDocument.DoMenuCommand(aCmdNumber: CmdNumber):TCommand;OVERRIDE;
		PROCEDURE TDesignDocument.DoSetupMenus; OVERRIDE;

		PROCEDURE TDesignDocument.Fields(PROCEDURE DoToField(fieldName: Str255; fieldAddr: Ptr; 
				fieldType: INTEGER)); OVERRIDE;
	END;
	
	TMPWTextDocument = OBJECT(TDocument)
		fDocText: Handle;
		fTEView:	TMyTEView;
		fDesignDocument: TDesignDocument;

		FUNCTION TMPWTextDocument.GetDocText: Handle;
		PROCEDURE TMPWTextDocument.SetDocText(theHandle: Handle);
		FUNCTION TMPWTextDocument.GetTEView: TMyTEView;
		PROCEDURE TMPWTextDocument.SetTEView(theTEView: TMyTEView);
		FUNCTION TMPWTextDocument.GetDesignDoc: TDesignDocument;
		PROCEDURE TMPWTextDocument.SetDesignDoc(theDesignDoc: TDesignDocument);
		PROCEDURE TMPWTextDocument.SetTitleNoDialog(theName: str255);
		PROCEDURE TMPWTextDocument.SetTitle(aTitle: str255); OVERRIDE;
		PROCEDURE TMPWTextDocument.IMPWTextDocument(theDesignDoc: TDesignDocument; theName: str255);
		PROCEDURE TMPWTextDocument.DoMakeViews(forPrinting: BOOLEAN); OVERRIDE;
		PROCEDURE TMPWTextDocument.DoNeedDiskSpace(VAR dataForkBytes, rsrcForkBytes: LongInt); 
				OVERRIDE;
		PROCEDURE TMPWTextDocument.DoWrite(aRefNum: INTEGER; makingCopy: BOOLEAN); OVERRIDE;
		PROCEDURE TMPWTextDocument.FreeData; OVERRIDE;
		PROCEDURE TMPWTextDocument.Free; OVERRIDE;
		PROCEDURE TMPWTextDocument.Fields(PROCEDURE DoToField(fieldName: Str255; fieldAddr: Ptr; 
				fieldType: INTEGER)); OVERRIDE;
	END;

	TMPWInterfaceDocument = OBJECT(TMPWTextDocument)
		
		PROCEDURE TMPWInterfaceDocument.IMPWInterfaceDocument
				(theDesignDoc: TDesignDocument; theTextHandler: TTextHandler; theName: str255);
		PROCEDURE TMPWInterfaceDocument.Close;	OVERRIDE;
	END;
	
	TMPWImplementationDocument = OBJECT(TMPWTextDocument)

		PROCEDURE TMPWImplementationDocument.IMPWImplementationDocument
				(theDesignDoc: TDesignDocument; theTextHandler: TTextHandler;theName: str255);
		PROCEDURE TMPWImplementationDocument.Close;	OVERRIDE;
	END;

IMPLEMENTATION

{$I UDesignDocument.inc1.p}

END.