Unit UMyCommands;

INTERFACE

USES
	UMacApp,
	ToolUtils,
	Resources,
	Packages,
	UPrinting,
	UTEView,
	UDialog,
	UGridView,
	UCRCCards,
	UPopText,
	UItem,
	UCard,
	UDesignDocument,
	UMyViews,
	UTextHandler;

TYPE
	TMyCommand = OBJECT(TCommand)
		fDocument: TDesignDocument;
		fCard: TCard;
		
		PROCEDURE TMyCommand.IMyCommand(theCommandNum: CmdNumber; theCard: TCard);
		FUNCTION TMyCommand.GetDocument: TDesignDocument;
		PROCEDURE TMyCommand.SetDocument(theDesign: TDesignDocument);
		FUNCTION TMyCommand.GetCard: TCard;
		PROCEDURE TMyCommand.SetCard(theCard: TCard);
		PROCEDURE TMyCommand.DetermineInitializationFromCard(	theCard: TCard;
																													VAR theDesignDoc: TDesignDocument;
																													VAR theView: TView;
																													VAR theScroller: TScroller);
		PROCEDURE TMyCommand.RedoIt; OVERRIDE;
		PROCEDURE TMyCommand.Fields(PROCEDURE DoToField(fieldName: Str255; fieldAddr: Ptr; 
				fieldType: INTEGER)); OVERRIDE;
	END;
	
TDesignViewCommand = OBJECT(TMyCommand)
		PROCEDURE TDesignViewCommand.DetermineInitializationFromCard(	theCard: TCard;
																													VAR theDesignDoc: TDesignDocument;
																													VAR theView: TView;
																													VAR theScroller: TScroller);
																													OVERRIDE;
	END;
	
TEditViewCommand = OBJECT(TMyCommand)
		PROCEDURE TEditViewCommand.DetermineInitializationFromCard(	theCard: TCard;
																													VAR theDesignDoc: TDesignDocument;
																													VAR theView: TView;
																													VAR theScroller: TScroller);
																													OVERRIDE;
	END;
	
TDraggerCommand = OBJECT(TDesignViewCommand)
		fSimpleView: TSimpleView;
		fOldLocation: VPoint;
		fNewLocation: VPoint;
		
		FUNCTION TDraggerCommand.GetSimpleView: TSimpleView;
		PROCEDURE TDraggerCommand.SetSimpleView(theSimpleView: TSimpleView);
		FUNCTION TDraggerCommand.GetOldLocation: VPoint;
		PROCEDURE TDraggerCommand.SetOldLocation(theLocation: VPoint);
		FUNCTION TDraggerCommand.GetNewLocation: VPoint;
		PROCEDURE TDraggerCommand.SetNewLocation(theLocation: VPoint);
		PROCEDURE TDraggerCommand.IDraggerCommand(theView: TSimpleView; theDownPoint: Point);
		PROCEDURE TDraggerCommand.TrackFeedback(anchorPoint, nextPoint: VPoint;
									turnItOn, mouseDidMove: BOOLEAN); OVERRIDE;
		FUNCTION TDraggerCommand.TrackMouse(aTrackPhase: TrackPhase;
							 VAR anchorPoint, previousPoint, nextPoint: VPoint;
							 mouseDidMove: BOOLEAN): TCommand; OVERRIDE;
		PROCEDURE TDraggerCommand.DoIt; OVERRIDE;
		PROCEDURE TDraggerCommand.UndoIt; OVERRIDE;
		
		PROCEDURE TDraggerCommand.Fields(PROCEDURE DoToField(fieldName: Str255; fieldAddr: Ptr; 
				fieldType: INTEGER)); OVERRIDE;
	END;
	
	TCardCommand = OBJECT(TDesignViewCommand)

		{Fields}
		fIndex: ArrayIndex;

		{Methods}
		FUNCTION TCardCommand.GetIndex: ArrayIndex;
		PROCEDURE TCardCommand.SetIndex(theIndex: ArrayIndex);
		PROCEDURE TCardCommand.ICardCommand(theCommandNum: CmdNumber; theCard: TCard; 
																			itsIndex: ArrayIndex);
		PROCEDURE TCardCommand.Fields(PROCEDURE DoToField(
			fieldName: Str255; fieldAddr: Ptr; fieldType: INTEGER)); OVERRIDE;

	END;


	TAddCardCommand = OBJECT(TCardCommand)

		{Methods}
		PROCEDURE TAddCardCommand.IAddCardCommand(theLocation: Vpoint; 
								theDocument: TDesignDocument);
		PROCEDURE TAddCardCommand.DoIt; OVERRIDE;
		PROCEDURE TAddCardCommand.UndoIt; OVERRIDE;
		PROCEDURE TAddCardCommand.Free; OVERRIDE;

	END;


	TDeleteCardCommand = OBJECT(TCardCommand)

		{Methods}
		PROCEDURE TDeleteCardCommand.IDeleteCardCommand(theCard: TCard; itsIndex: ArrayIndex);
		PROCEDURE TDeleteCardCommand.DoIt; OVERRIDE;
		PROCEDURE TDeleteCardCommand.UndoIt; OVERRIDE;
		PROCEDURE TDeleteCardCommand.Free; OVERRIDE;

	END;


	TFieldItemCommand = OBJECT(TEditViewCommand)

		{Fields}
		fFieldIndex: ArrayIndex;
		fGetIndex: ArrayIndex;
		fSetIndex: ArrayIndex;
		fMakeGetAccessor: Boolean;
		fMakeSetAccessor: Boolean;

		{Collaborators}
		fFieldItem: TFieldItem;
		fGetAccessor: TGetAccessorItem;
		fSetAccessor: TSetAccessorItem;

		{Methods}
		PROCEDURE TFieldItemCommand.GetTheIndecies(VAR 	theFieldIndex,
																										theGetIndex,
																										theSetIndex: ArrayIndex);
		PROCEDURE TFieldItemCommand.SetTheIndecies(	theFieldIndex,
																								theGetIndex,
																								theSetIndex: ArrayIndex);
		PROCEDURE TFieldItemCommand.DetermineAccessorsAndIndecies(theFieldItem: TFieldItem;
																															theCard: TCard;
																															itsIndex: ArrayIndex);
		PROCEDURE TFieldItemCommand.IFieldItemCommand(theCommandNum: CmdNumber; 
																									theCard: TCard;
																									theFieldItem: TFieldItem;
																									itsIndex: ArrayIndex);
		FUNCTION TFieldItemCommand.GetFieldItem: TFieldItem;
		PROCEDURE TFieldItemCommand.SetFieldItem(theFieldItem: TFieldItem);
		FUNCTION TFieldItemCommand.GetGetAccessor: TGetAccessorItem;
		PROCEDURE TFieldItemCommand.SetGetAccessor(theGetAccessor: TGetAccessorItem);
		FUNCTION TFieldItemCommand.GetSetAccessor: TSetAccessorItem;
		PROCEDURE TFieldItemCommand.SetSetAccessor(theSetAccessor: TSetAccessorItem);
		PROCEDURE TFieldItemCommand.CreateAccessors(theFieldItem: TFieldItem);
		PROCEDURE TFieldItemCommand.DeleteAccessors;
		PROCEDURE TFieldItemCommand.Fields(PROCEDURE DoToField(
			fieldName: Str255; fieldAddr: Ptr; fieldType: INTEGER)); OVERRIDE;

	END;


	TAddFieldItemCommand = OBJECT(TFieldItemCommand)

		{Methods}
		PROCEDURE TAddFieldItemCommand.IAddFieldItemCommand(theCard: TCard;
																												theName, theType: str255);
		PROCEDURE TAddFieldItemCommand.DoIt; OVERRIDE;
		PROCEDURE TAddFieldItemCommand.UndoIt; OVERRIDE;
		PROCEDURE TAddFieldItemCommand.Free; OVERRIDE;

	END;


	TDeleteFieldItemCommand = OBJECT(TFieldItemCommand)

		{Methods}
		PROCEDURE TDeleteFieldItemCommand.IDeleteFieldItemCommand(theCard: TCard;
																														theFieldItem: TFieldItem;
																														itsIndex: ArrayIndex);
		PROCEDURE TDeleteFieldItemCommand.DoIt; OVERRIDE;
		PROCEDURE TDeleteFieldItemCommand.UndoIt; OVERRIDE;
		PROCEDURE TDeleteFieldItemCommand.Free; OVERRIDE;

	END;

	TMethodItemCommand = OBJECT(TEditViewCommand)

		{Fields}
		fIndex: ArrayIndex;

		{Collaborators}
		fMethodItem: TMethodItem;

		{Methods}
		FUNCTION TMethodItemCommand.GetIndex: ArrayIndex;
		PROCEDURE TMethodItemCommand.SetIndex(theIndex: ArrayIndex);
		PROCEDURE TMethodItemCommand.IMethodItemCommand(theCommandNum: CmdNumber;
																											theCard: TCard;
																											theMethodItem: TMethodItem;
																											itsIndex: ArrayIndex);
		FUNCTION TMethodItemCommand.GetMethodItem: TMethodItem;
		PROCEDURE TMethodItemCommand.SetMethodItem(theMethodItem: TMethodItem);
		PROCEDURE TMethodItemCommand.Fields(PROCEDURE DoToField(
			fieldName: Str255; fieldAddr: Ptr; fieldType: INTEGER)); OVERRIDE;

	END;


	TAddMethodItemCommand = OBJECT(TMethodItemCommand)

		{Methods}
		PROCEDURE TAddMethodItemCommand.IAddMethodItemCommand(theCard: TCard;
																													theItemName: str255;
																													makeItAnOverride: boolean);
		PROCEDURE TAddMethodItemCommand.DoIt; OVERRIDE;
		PROCEDURE TAddMethodItemCommand.UndoIt; OVERRIDE;
		PROCEDURE TAddMethodItemCommand.Free; OVERRIDE;

	END;


	TDeleteMethodItemCommand = OBJECT(TMethodItemCommand)

		{Methods}
		PROCEDURE TDeleteMethodItemCommand.IDeleteMethodItemCommand(theCard: TCard;
																																theMethodItem: TMethodItem;
																																itsIndex: ArrayIndex);
		PROCEDURE TDeleteMethodItemCommand.DoIt; OVERRIDE;
		PROCEDURE TDeleteMethodItemCommand.UndoIt; OVERRIDE;
		PROCEDURE TDeleteMethodItemCommand.Free; OVERRIDE;

	END;


	TAddCollaboratorItemCommand = OBJECT(TAddFieldItemCommand)

		{Methods}
		PROCEDURE TAddCollaboratorItemCommand.IAddCollaboratorItemCommand(theCard: TCard;
																																			theItemName: str255);
		PROCEDURE TAddCollaboratorItemCommand.DoIt; OVERRIDE;
		PROCEDURE TAddCollaboratorItemCommand.UndoIt; OVERRIDE;

	END;


	TDeleteCollabItemCommand = OBJECT(TDeleteFieldItemCommand)

		{Methods}
		PROCEDURE TDeleteCollabItemCommand.IDeleteCollabItemCommand(theCard: TCard;
																																theCollabItem: TCollaboratorItem;
																																itsIndex: ArrayIndex);
		PROCEDURE TDeleteCollabItemCommand.DoIt; OVERRIDE;
		PROCEDURE TDeleteCollabItemCommand.UndoIt; OVERRIDE;

	END;
	
	TChangeNameCommand = OBJECT(TEditViewCommand)
		fOldName: str255;
		fNewName: str255;
		
		FUNCTION TChangeNameCommand.GetOldName: str255;
		PROCEDURE TChangeNameCommand.SetOldName(theOldName: str255);
		FUNCTION TChangeNameCommand.GetNewName: str255;
		PROCEDURE TChangeNameCommand.SetNewName(theNewName: str255);
		PROCEDURE TChangeNameCommand.IChangeNameCommand(theCommandNum: CmdNumber;
																										theCard: TCard; 
																										theOldName, theNewName: str255);
		PROCEDURE TChangeNameCommand.Fields(PROCEDURE DoToField(fieldName: Str255; 
																				fieldAddr: Ptr; fieldType: INTEGER)); OVERRIDE;
	END;
	
	TChangeClassCommand = OBJECT(TChangeNameCommand)
		PROCEDURE TChangeClassCommand.DoIt; OVERRIDE;
		PROCEDURE TChangeClassCommand.UndoIt; OVERRIDE;
	END;
	
	TChangeSuperClassCommand = OBJECT(TChangeNameCommand)
		PROCEDURE TChangeSuperClassCommand.DoIt; OVERRIDE;
		PROCEDURE TChangeSuperClassCommand.UndoIt; OVERRIDE;
	END;

	TChangeInstVarCommand = OBJECT(TChangeNameCommand)
		fFieldItem: TFieldItem;
		fGetAccessor: TGetAccessorItem;
		fSetAccessor: TSetAccessorItem;
		PROCEDURE TChangeInstVarCommand.IChangeInstVarCommand(theCommandNum: CmdNumber; 
																													theCard:TCard; 
																													theFieldItem: TFieldItem;
																													theNewName: Str255);
		PROCEDURE TChangeInstVarCommand.DoIt; OVERRIDE;
		PROCEDURE TChangeInstVarCommand.UndoIt; OVERRIDE;
	END;
	
	TChangeFieldCommand = OBJECT(TChangeInstVarCommand)
		fOldType: Str255;
		fNewType: Str255;
		
		FUNCTION TChangeFieldCommand.GetOldType: Str255;
		PROCEDURE TChangeFieldCommand.SetOldType(theType: Str255);
		FUNCTION TChangeFieldCommand.GetNewType: Str255;
		PROCEDURE TChangeFieldCommand.SetNewType(theType: Str255);
		PROCEDURE TChangeFieldCommand.IChangeFieldCommand(theCommandNum: CmdNumber; 
																									theCard: TCard; theFieldItem: TFieldItem;
																									theNewName, theNewType: Str255);
		PROCEDURE TChangeFieldCommand.DoIt; OVERRIDE;
		PROCEDURE TChangeFieldCommand.UndoIt; OVERRIDE;
		PROCEDURE TChangeFieldCommand.RedoIt; OVERRIDE;
	END;
	
	TChangeCollaboratorCommand = OBJECT(TChangeInstVarCommand)
		PROCEDURE TChangeCollaboratorCommand.IChangeCollaboratorCommand
																										(theCommandNum: CmdNumber; 
																										theCard:TCard; 
																										TheFieldItem: TFieldItem;
																										theNewName: Str255);
		PROCEDURE TChangeCollaboratorCommand.DoIt; OVERRIDE;
		PROCEDURE TChangeCollaboratorCommand.UndoIt; OVERRIDE;
		PROCEDURE TChangeCollaboratorCommand.RedoIt; OVERRIDE;
	END;

	TChangeMethodCommand = OBJECT(TChangeNameCommand)
		fMethodItem : TMethodItem;
		fOldIsOverride : BOOLEAN;
		fNewIsOverride : BOOLEAN;
		FUNCTION TChangeMethodCommand.GetTheMethodItem:TMethodItem;
		PROCEDURE TChangeMethodCommand.SetTheMethodItem(theMethodItem : TMethodItem);
		FUNCTION TChangeMethodCommand.GetOldIsOverride: Boolean;
		PROCEDURE TChangeMethodCommand.SetOldIsOverride(isOverride : Boolean);
		FUNCTION TChangeMethodCommand.GetNewIsOverride: Boolean;
		PROCEDURE TChangeMethodCommand.SetNewIsOverride(isOverride : Boolean);
		PROCEDURE TChangeMethodCommand.IChangeMethodCommand(theCommandNum: CmdNumber; 
																										theCard:TCard; 
																										theMethodItem: TMethodItem;
																										theNewName: Str255;
																										newIsOverride: Boolean);
		PROCEDURE TChangeMethodCommand.DoIt; OVERRIDE;
		PROCEDURE TChangeMethodCommand.UndoIt; OVERRIDE;
		PROCEDURE TChangeMethodCommand.RedoIt; OVERRIDE;
	END;

IMPLEMENTATION

{$I UMyCommands.inc1.p}

END.