Unit UMyViews;

INTERFACE

USES
	UMacApp,
	ToolUtils,
	Resources,
	Packages,
	UPrinting,
	UTEView,
	UDialog,
	UGridView,
	UCRCCards,
	UPopText,
	UItem,
	UCard,
	UDesignDocument,
	UMyCommands,
	UTextHandler;

TYPE

	TDesignView = OBJECT(TView)
		fDesignDocument: TDesignDocument;
		fSimpleViewList: TList;
		
		FUNCTION TDesignView.GetDocument: TDesignDocument;
		PROCEDURE TDesignView.SetDocument(theDesign:TDesignDocument);
		FUNCTION TDesignView.GetSimpleViewList: TList;
		PROCEDURE TDesignView.SetSimpleViewList(theSimpleViewList:TList);
		PROCEDURE TDesignView.IRes(itsDocument: TDocument; 
						 itsSuperView: TView; VAR itsParams: Ptr); OVERRIDE;
		PROCEDURE TDesignView.CalcMinSize(VAR minSize: VPoint); OVERRIDE;
		FUNCTION TDesignView.MakeDeleteCardCmd: TDeleteCardCommand;
		FUNCTION TDesignView.MakeAddCardCmd(theVPoint: VPoint): TAddCardCommand;
		FUNCTION TDesignView.DoMouseCommand(VAR theMouse: Point; VAR info: EventInfo;
							  VAR hysteresis: Point): TCommand; OVERRIDE;
		FUNCTION  TDesignView.DoMenuCommand(aCmdNumber: CmdNumber):TCommand;OVERRIDE;
		PROCEDURE TDesignView.DoSetupMenus; OVERRIDE;
		PROCEDURE TDesignView.Free; OVERRIDE;
		PROCEDURE TDesignView.Fields(PROCEDURE DoToField(fieldName: Str255; fieldAddr: Ptr; 
				fieldType: INTEGER)); OVERRIDE;
	END;
	
	TEditView = OBJECT(TDialogView)
		fCard: TCard;
		fClassNameItem: TEditText;
		fSuperClassNameItem: TPopText;
		fFieldEntryItem: TEditText;
		fFieldEntryName: TStaticText;
		fFieldTypeEntryItem: TPopText;
		fFieldTypeEntryName: TStaticText;
		fCollaboratorEntryItem: TPopText;
		fFieldTextListView: TFieldTextListView;
		fMethodTextListView: TMethodTextListView;
		fCollaboratorTextListView: TCollaboratorTextListView;
		fItemTypeMenu: TPopUp;
		fSuperClassTypeMenu: TPopUp;
		fTypeCluster: TCluster;
		fOVERRIDEBox: TCheckBox;
		fEditMode: INTEGER;
		
		FUNCTION TEditView.GetCard: TCard;
		PROCEDURE TEditView.SetCard(theCard:TCard);
		FUNCTION TEditView.GetFieldEntryView: TEditText;
		PROCEDURE TEditView.SetFieldEntryView(theEntryView:TEditText);
		FUNCTION TEditView.GetTypeEntryView: TPopText;
		PROCEDURE TEditView.SetTypeEntryView(theEntryView:TPopText);
		FUNCTION TEditView.GetCollaboratorEntryView: TPopText;
		PROCEDURE TEditView.SetCollaboratorEntryView(theEntryView:TPopText);
		PROCEDURE TEditView.SetOurWindowTitle;
		PROCEDURE TEditView.ChangeOurWindowTitle;
		PROCEDURE TEditView.SetOurClassName;
		FUNCTION TEditView.GetOurClassName: str255;
		PROCEDURE TEditView.SetOurSuperClassName;
		FUNCTION TEditView.GetOurSuperClassName: str255;
		PROCEDURE TEditView.SetUpOurTextListViews;
		PROCEDURE TEditView.SetUpOurEntryViews;
(*		PROCEDURE TEditView.StuffFieldTypeEntryItem;*)
(*		PROCEDURE TEditView.StuffSuperClassEntryItem;*)
		FUNCTION TEditView.GetOVERRIDEBox: TCheckBox;
		PROCEDURE TEditView.SetOVERRIDEBox(theBox:TCheckBox);
		PROCEDURE TEditView.SetOVERRIDE(overrideIt:Boolean);
		PROCEDURE TEditView.SetUpBlankMode;
		PROCEDURE TEditView.StuffAndHiliteField(theFieldItem: TFieldItem);
		PROCEDURE TEditView.StuffAndHiliteMethod(theMethodItem: TMethodItem);
		PROCEDURE TEditView.StuffAndHiliteCollaborator(theCollaboratorItem: TCollaboratorItem);
		PROCEDURE TEditView.SetUpFieldsMode(theFieldItem: TFieldItem);
		PROCEDURE TEditView.SetUpMethodsMode(theMethodItem: TMethodItem);
		PROCEDURE TEditView.SetUpCollaboratorsMode(theCollaboratorItem: TCollaboratorItem);
		PROCEDURE TEditView.AddCardToObjectLists(theCardToAdd: TCard);
		PROCEDURE TEditView.RemoveCardFromObjectLists(theCardToRemove: TCard);
		PROCEDURE TEditView.ChangeNameInObjectLists(theOldName, theNewName: Str255);
		
		PROCEDURE TEditView.IEditView(theCard: TCard);	{note: no IRes, need card}
		FUNCTION TEditView.MakeChangeClassCmd: TCommand;
		FUNCTION TEditView.MakeChangeSuperClassCmd: TCommand;
		FUNCTION TEditView.MakeAddFieldCmd: TAddFieldItemCommand;
		FUNCTION TEditView.MakeChangeFieldCmd: TCommand;
		FUNCTION TEditView.MakeAddMethodCmd: TAddMethodItemCommand;
		FUNCTION TEditView.MakeChangeMethodCmd: TCommand;
		FUNCTION TEditView.MakeAddCollabCmd: TAddCollaboratorItemCommand;
		FUNCTION TEditView.MakeChangeCollaboratorCmd: TCommand;
		FUNCTION TEditView.MakeDeleteFieldCmd: TDeleteFieldItemCommand;
		FUNCTION TEditView.MakeDeleteMethodCmd: TDeleteMethodItemCommand;
		FUNCTION TEditView.MakeDeleteCollabCmd: TDeleteCollabItemCommand;
		PROCEDURE TEditView.ChangeClassName(theName: str255);
		PROCEDURE TEditView.ChangeSuperClassName(theName: str255);
		FUNCTION TEditView.DoKeyCommand(ch: CHAR; aKeyCode: INTEGER; VAR info: EventInfo)
					: TCommand; OVERRIDE
		FUNCTION  TEditView.DoMenuCommand(aCmdNumber: CmdNumber):TCommand;OVERRIDE;
		PROCEDURE TEditView.DoSetupMenus; OVERRIDE;
		PROCEDURE TEditView.AddField(theIndex: ArrayIndex);
		PROCEDURE TEditView.AddFieldLast;
		PROCEDURE TEditView.DeleteField(theIndex: INTEGER);
		PROCEDURE TEditView.AddMethod(theIndex: ArrayIndex);
		PROCEDURE TEditView.AddMethodLast;
		PROCEDURE TEditView.DeleteMethod(theIndex: INTEGER);
		PROCEDURE TEditView.AddCollaborator(theIndex: ArrayIndex);
		PROCEDURE TEditView.AddCollaboratorLast;
		PROCEDURE TEditView.DeleteCollaborator(theIndex: INTEGER);
		PROCEDURE TEditView.AcceptChanges;
		PROCEDURE TEditView.DoChoice(origView: TView; itsChoice: INTEGER); OVERRIDE;
		PROCEDURE TEditView.DeselectFieldTextListView;
		PROCEDURE TEditView.DeselectMethodTextListView;
		PROCEDURE TEditView.DeselectCollaboratorTextListView;
		PROCEDURE TEditView.DeactivateView;
		PROCEDURE TEditView.Fields(PROCEDURE DoToField(fieldName: Str255; fieldAddr: Ptr; 
				fieldType: INTEGER)); OVERRIDE;
	END;
	
	TParenStaticText = OBJECT(TStaticText)
		PROCEDURE TParenStaticText.SetText(theText: Str255; redraw: BOOLEAN); OVERRIDE;
	END;
	
	TSimpleView = OBJECT(TView)
		fCard: TCard;
		fClassTextView: TStaticText;
		fSuperClassTextView: TParenStaticText;
		fSelected: boolean;
		fDragging: boolean;
		
		
		FUNCTION TSimpleView.GetCard: TCard;
		PROCEDURE TSimpleView.SetCard(theCard:TCard);
		FUNCTION TSimpleView.GetClassView: TStaticText;
		PROCEDURE TSimpleView.SetClassView(theClassView:TStaticText);
		FUNCTION TSimpleView.GetSuperClassView: TParenStaticText;
		PROCEDURE TSimpleView.SetSuperClassView(theSuperClassView:TParenStaticText);
		FUNCTION TSimpleView.GetSelected: boolean;
		PROCEDURE TSimpleView.SetSelected(turnItOn:boolean);
		FUNCTION TSimpleView.GetDragging: boolean;
		PROCEDURE TSimpleView.SetDragging(turnItOn:boolean);
		
		PROCEDURE TSimpleView.ISimpleView;	{the subview references}
		PROCEDURE TSimpleView.IRes(itsDocument: TDocument; 
						 itsSuperView: TView; VAR itsParams: Ptr); OVERRIDE;
		PROCEDURE TSimpleView.ChangeClassName(theName: str255);
		PROCEDURE TSimpleView.ChangeSuperClassName(theName: str255);
		PROCEDURE TSimpleView.OpenEditView;
		FUNCTION  TSimpleView.DoMouseCommand(VAR theMouse: Point;
				VAR info: EventInfo; VAR hysteresis: Point): TCommand; OVERRIDE;
		PROCEDURE TSimpleView.DoHighlightSelection(fromHL, toHL: HLState); OVERRIDE;
		PROCEDURE TSimpleView.Fields(PROCEDURE DoToField(fieldName: Str255; fieldAddr: Ptr; 
				fieldType: INTEGER)); OVERRIDE;
	END;
	
	TCardWindow = OBJECT(TWindow)
		fEditView: TEditView;
		
		PROCEDURE TCardWindow.ICardWindow(theEditView: TEditView);
		PROCEDURE TCardWindow.Activate(entering : Boolean); OVERRIDE;
	END;
	
	TNewButton = OBJECT(TButton)
		fEditView: TEditView;
		
		FUNCTION TNewButton.GetEditView: TEditView;
		PROCEDURE TNewButton.SetEditView(theEditView: TEditView);
		PROCEDURE TNewButton.INewButton(theEditView: TEditView);
	END;
	
	TNewFieldButton = OBJECT(TNewButton)
		PROCEDURE TNewFieldButton.DoChoice(origView: TView; itsChoice: INTEGER); OVERRIDE;
	END;
	
	TNewMethodButton = OBJECT(TNewButton)
		PROCEDURE TNewMethodButton.DoChoice(origView: TView; itsChoice: INTEGER); OVERRIDE;
	END;
	
	TNewCollaboratorButton = OBJECT(TNewButton)
		PROCEDURE TNewCollaboratorButton.DoChoice(origView: TView; itsChoice: INTEGER); OVERRIDE;
	END;
	
	TMyTextListView = OBJECT(TTextListView)
		fItemList: TMyList;
		fEditView: TEditView;
		
		FUNCTION TMyTextListView.GetEditView: TEditView;
		PROCEDURE TMyTextListView.SetEditView(theEditView: TEditView);
		FUNCTION TMyTextListView.GetItemList: TMyList;
		PROCEDURE TMyTextListView.SetItemList(theList:TMyList);
		PROCEDURE TMyTextListView.GetItemText(anItem: INTEGER; VAR aString: Str255); OVERRIDE;
		FUNCTION TMyTextListView.GetNumRows: integer;
		PROCEDURE TMyTextListView.DeselectTheView;
		PROCEDURE TMyTextListView.StuffTheItemEditView(theItemNumber:INTEGER);
		PROCEDURE TMyTextListView.SelectItem(anitem: INTEGER; 
											extendSelection, highlight, select: BOOLEAN); OVERRIDE;
		PROCEDURE TMyTextListView.SelectThisItem(theItem: TItem);
		FUNCTION TMyTextListView.GetTheItemToEdit(theItemNumber:INTEGER): TItem;
		PROCEDURE TMyTextListView.SetEditField(kNameField: IDType; aString: Str255; 
					aWindow: TWindow);
		FUNCTION TMyTextListView.GetEditField(kNameField: IDType; aWindow: TWindow): Str255;
		FUNCTION  TMyTextListView.DoMouseCommand(VAR theMouse: Point;
				VAR info: EventInfo; VAR hysteresis: Point): TCommand; OVERRIDE;
		PROCEDURE TMyTextListView.Fields(PROCEDURE DoToField(fieldName: Str255; fieldAddr: Ptr; 
				fieldType: INTEGER)); OVERRIDE;
	END;
	
	TMethodTextListView = OBJECT(TMyTextListView)

		{Methods}
		PROCEDURE TMethodTextListView.StuffTheItemEditView(theItemNumber:INTEGER); OVERRIDE;
		PROCEDURE TMethodTextListView.SelectCell(theCell: GridCell;
							   extendSelection, highlight, select: BOOLEAN); OVERRIDE;

	END;

	TCollaboratorTextListView = OBJECT(TMyTextListView)

		{Methods}
		PROCEDURE TCollaboratorTextListView.StuffTheItemEditView(theItemNumber:INTEGER); OVERRIDE;
		PROCEDURE TCollaboratorTextListView.SelectCell(theCell: GridCell;
							   extendSelection, highlight, select: BOOLEAN); OVERRIDE;

	END;

	TFieldTextListView = OBJECT(TMyTextListView)

		{Methods}
		PROCEDURE TFieldTextListView.StuffTheItemEditView(theItemNumber:INTEGER); OVERRIDE;
		PROCEDURE TFieldTextListView.SelectCell(theCell: GridCell;
							   extendSelection, highlight, select: BOOLEAN); OVERRIDE;

	END;

	TMyTEView = OBJECT(TTEView)
		fDesignDocument: TDesignDocument;
	
		FUNCTION TMyTEView.GetDesignDoc: TDesignDocument;
		PROCEDURE TMyTEView.SetDesignDoc(theDesignDoc: TDesignDocument);
		PROCEDURE TMyTEView.DesignDocumentIsGone;
		PROCEDURE TMyTEView.ClearText;
		PROCEDURE TMyTEView.WriteToMyTEView(theString: str255);
		PROCEDURE TMyTEView.WriteEndOfLine;
		PROCEDURE TMyTEView.FinishWriteOperation;
		PROCEDURE TMyTEView.Fields(PROCEDURE DoToField(fieldName: Str255; fieldAddr: Ptr; 
				fieldType: INTEGER)); OVERRIDE;
	END;

IMPLEMENTATION

{$I UMyViews.inc1.p}

END.