VAR
	gNextMenuId:		INTEGER;		{ for automatic assignment of menu ids }

PROCEDURE InitUPopText(firstMenuId: INTEGER);

	BEGIN
	gNextMenuId := firstMenuId;
	IF qTemplateViews THEN BEGIN
		{ So linker doesn't strip TTEView class }
		IF gDeadStripSuppression THEN BEGIN
			IF Member(TObject(NIL), TPulldown) THEN;
			IF Member(TObject(NIL), TPopText) THEN;
			END;
		RegisterStdType('TPulldown', kStdPulldown);
		RegisterStdType('TPopText', kStdPopText);
		END;
	END; {InitUPopText}

FUNCTION GetNextMenuID: INTEGER;
{ Assign the next menu id number }

	BEGIN
	GetNextMenuID := gNextMenuId;
	gNextMenuId := gNextMenuId + 1;
	END; {GetNextMenuID}

{----------------------------------- TPulldown -----------------------------------}

{$S AOpen}
PROCEDURE TPulldown.IRes(itsDocument: TDocument; itsSuperView: TView; VAR itsParams: Ptr); OVERRIDE;

	VAR
		fi: 				FailInfo;
		itsTemplate:		PulldownTemplatePtr;

	PROCEDURE HandleFailure(error: OSErr; message: LONGINT);

	BEGIN
		Free;
	END;

BEGIN
	fMenuHandle := NIL;
	fSICNHandle := NIL;
	fSICNid := kNoResource;
	fRsrcID := kNoResource;
	fMenuID := kNoMenuID;
	INHERITED IRes(itsDocument, itsSuperView, itsParams);

	itsTemplate := PulldownTemplatePtr(itsParams);
	CatchFailures(fi, HandleFailure);
	InitPulldown(itsTemplate);
	Success(fi);

	OffsetPtr(itsParams, SIZEOF(PulldownTemplate));
END; {TPulldown.IRes}


PROCEDURE TPulldown.IPulldown(itsSuperView: TView; itsLocation: VPoint;
							  itsTemplate: PulldownTemplatePtr);
{ Initialize the pulldown menu from whatever resources are specified in the
  PulldownTemplate (as in IRes). }

	VAR
		fi: 				FailInfo;
		itsSize:			VPoint;

	PROCEDURE HandleFailure(error: OSErr; message: LONGINT);

	BEGIN
		Free;
	END;

BEGIN									{ TPulldown.IPulldown }
	fMenuHandle := NIL;
	fSICNHandle := NIL;
	fSICNid := kNoResource;
	fRsrcID := kNoResource;
	fMenuID := kNoMenuID;
	SetVPt(itsSize, kSICNwidth, kSICNwidth);
	IControl(itsSuperView, itsLocation, itsSize, sizeFixed, sizeFixed);

	CatchFailures(fi, HandleFailure);
	InitPulldown(itsTemplate);
	Success(fi);

	ViewEnable(TRUE, kDontRedraw);		{ Default is to enable hit testing }
END; {TPulldown.IPulldown}


PROCEDURE TPulldown.InitPulldown(pdTemplatePtr: PulldownTemplatePtr);
{ Initialize the pulldown menu from whatever resources are specified in the
  PulldownTemplate. }

	VAR
		aMenu:				MenuHandle;

BEGIN
	{$IFC NOT qNeedsHierarchicalMenus}
	IF NOT gConfiguration.hasHierarchicalMenus THEN 
		BEGIN
			{$IFC qDebug}
			ProgramBreak('Attempt to use popup menus on machine that doesnt support them');
			{$ENDC}
			fShown := FALSE;								{ What's reasonable here ??? }
		END
	ELSE
	{$ENDC}
		BEGIN
			WITH pdTemplatePtr^ DO 
				BEGIN
					SetSICN(SICNid, kDontRedraw);
					(* fSICNid := SICNid;
					IF SICNid <> kNoResource THEN 
						BEGIN
							fSICNHandle := GetResource('SICN', SICNid);
							FailNIL(fSICNHandle);
							DetachResource(fSICNHandle);
							HNoPurge(fSICNHandle);
						END; *)
		
					aMenu := NIL;
					IF menuRsrcID <> kNoResource THEN 
						BEGIN		{ There's a MENU resource }
							aMenu := GetMenu(menuRsrcID);
							{ Don't die because resource not found - just return NIL handle }
							FailResError;
							fRsrcID := menuRsrcID;
						END;
					IF strListID <> kNoResource THEN 
						BEGIN		{ Look for STR# resource }
							IF menuID = kNoMenuID THEN
								menuID := GetNextMenuID;			{ assign next menu id }
							IF aMenu = NIL THEN
								aMenu := NewMenu(menuID, kScratchMenuTitle);
							IF aMenu <> NIL THEN
								AddStrsToMenu(aMenu, strListID);	{ add items to aMenu from the STR# }
						END;
					IF aMenu <> NIL THEN 
						BEGIN
							HNoPurge(Handle(aMenu));
							fMenuHandle := aMenu;
							fMenuID := aMenu^^.menuID;
							fMarkCurrentItem := markCurrentItem;
							fCurrentItem := 0;
							SetCurrentItem(currentItem);
						END;
				END; { WITH }
			fDefChoice := mPulldownHit;
		END;
END; {TPulldown.InitPulldown}


{$S ARes}
FUNCTION CountStrings(strID: INTEGER): INTEGER;
{ Return the number of strings contained in the specified STR# resource }

	TYPE
		strResource = RECORD
			count:		INTEGER;
			firstStr:	Str255;		{ actually, an array of variable-length strings }
			END;
		strPointer = ^strResource;
		strHandle  = ^strPointer;

	VAR
		strRes:		strHandle;

BEGIN
	strRes := strHandle(GetResource('STR#', strID));
	IF strRes = NIL THEN 
		CountStrings := 0
	ELSE 
		CountStrings := strRes^^.count;
	ReleaseResource(Handle(strRes));
END; {CountStrings}
	

PROCEDURE TPulldown.AddItemAfter(aString: Str255; theIndex: INTEGER);
BEGIN
	InsMenuItem(fMenuHandle, aString, theIndex);
END; {TPulldown.AddItemAfter}


PROCEDURE TPulldown.AppendItem(aString: Str255);
BEGIN
	AppendMenu(fMenuHandle, aString);
END; {TPulldown.AppendItem}


PROCEDURE TPulldown.RemoveItem(aString: Str255);
VAR
	theIndex: INTEGER;
	
BEGIN
	theIndex:=GetItemIndex(aString);
	IF theIndex<>0 THEN
		DelMenuItem(fMenuHandle, theIndex);
END; {TPulldown.RemoveItem}


PROCEDURE TPulldown.ChangeItem(theOldValue, theNewValue: Str255);
VAR
	theIndex: INTEGER;
	
BEGIN
	theIndex:=GetItemIndex(theOldValue);
	IF theIndex<>0 THEN
		SetItem(fMenuHandle, theIndex,theNewValue);
END; {TPulldown.ChangeItem}


FUNCTION TPulldown.GetItemIndex(aString: Str255):INTEGER;
	VAR
		index:			INTEGER;
		item:			INTEGER;
		itemCount:		INTEGER;
		theName:		Str255;
		itemName:		Str255;

BEGIN
	{ Try to match theName with the items in the pulldown menu. }
	index := 0;
	itemCount := SELF.GetItemCount;
	FOR item := 1 TO itemCount DO 
		BEGIN
			SELF.GetItemText(item, itemName);
			IF RelString(aString, itemName, FALSE, TRUE) = kAEqualB THEN 
				BEGIN
					index := item;
					LEAVE;
				END;
		END;
	GetItemIndex := index;
END; {TPulldown.GetItemIndex}


PROCEDURE TPulldown.AddStrsToMenu(theMenu: MenuHandle; theStrRsrcID: INTEGER);

	VAR
		count, i:		INTEGER;
		itemString:		Str255;

BEGIN
	{ Create a menu item for each string in the STR# resource }
	count := CountStrings(theStrRsrcID);
	FOR i := 1 TO count DO 
		BEGIN
			GetIndString(itemString, theStrRsrcID, i);
			IF LENGTH(itemString) > 0 THEN
				SELF.AppendItem(itemString);
		END;
END; {TPulldown.AddStrsToMenu}


{$S AClose}
PROCEDURE TPulldown.Free; OVERRIDE;

BEGIN
	ReleaseMenu;
	ReleaseSICN;

	INHERITED Free;
END; {TPulldown.Free}


{$S AClose}
PROCEDURE TPulldown.ReleaseMenu;

BEGIN
	IF fMenuHandle <> NIL THEN 
		BEGIN
			HPurge(Handle(fMenuHandle));
			IF fRsrcID = kNoResource THEN 
				DisposeMenu(fMenuHandle)
			ELSE 
				ReleaseResource(Handle(fMenuHandle));
			fMenuHandle := NIL;
		END;
	fMenuID := kNoMenuID;
END; {TPulldown.ReleaseMenu}


PROCEDURE TPulldown.ReleaseSICN;

BEGIN
	fSICNid := kNoResource;
	fSICNHandle := DisposeIfHandle(fSICNHandle);
END; {TPulldown.ReleaseSICN}


{$S ARes}
FUNCTION TPulldown.DoMouseCommand(VAR theMouse: Point;
								  VAR info: EventInfo;
								  VAR hysteresis: Point): TCommand; OVERRIDE;

	VAR
		newChoice:			INTEGER;
		currentItem:		INTEGER;
		result: 			LONGINT;
		menuPt: 			Point;
		menuRect:			Rect;
		fi: 				FailInfo;

BEGIN
	DoMouseCommand := NIL;
	ControlArea(menuRect);

	IF fMenuHandle <> NIL THEN 
		BEGIN
			MAInsertMenu(fMenuHandle, hierMenu);
			WITH menuRect DO
				SetPt(menuPt, left, top); 				{ Don't overwrite stuff next to the label }
			LocalToGlobal(menuPt);
			CalcMenuSize(fMenuHandle);					{ Fix for Menu Manager bug }
		(*	currentItem := Max(1, fCurrentItem); *)
			currentItem := fCurrentItem;
			result := PopUpMenuSelect(fMenuHandle, menuPt.v, menuPt.h, currentItem);
			newChoice := LoWord(result);
			IF HiWord(result) <> 0 THEN 
				BEGIN
					SetCurrentItem(newChoice);
					DoChoice(SELF, fDefChoice);
				END;
			DeleteMenu(fMenuID);
		END;
END; {TPulldown.DoMouseCommand}


{$S MouserEdit}
FUNCTION TPulldown.GetCurrentItem: INTEGER;

BEGIN
	GetCurrentItem := fCurrentItem;
END; {TPulldown.GetCurrentItem}


FUNCTION TPulldown.GetItemCount: INTEGER;
{ Return the number of items in this pulldown menu. }

BEGIN
	IF fMenuHandle = NIL THEN 
		GetItemCount := 0
	ELSE 
		GetItemCount := CountMItems(fMenuHandle);
END; {TPulldown.GetItemCount}


PROCEDURE TPulldown.GetItemText(item: INTEGER; VAR theText: Str255);

BEGIN
	IF fMenuHandle <> NIL THEN
		GetItem(fMenuHandle, item, theText)
	ELSE
		theText := '';
END; {TPulldown.GetItemText}


{$S MouserEdit}
PROCEDURE TPulldown.Draw(area: Rect); OVERRIDE;

	VAR
		oldState:			SignedByte;
		theRect:			Rect;
		srcBits:			BitMap;
		port:				GrafPtr;

BEGIN
	IF (fSICNHandle <> NIL) & (fSICNHandle^ <> NIL) THEN 
		BEGIN
			ControlArea(theRect);
			oldState := GetHandleBits(fSICNHandle);
			HNoPurge(fSICNHandle);
			WITH srcBits DO 
				BEGIN					{ construct a source BitMap }
					baseAddr := fSICNHandle^;
					rowBytes := 2;
					SetRect(bounds, 0, 0, kSICNwidth, kSICNwidth);
				END;
			GetPort(port);
			CopyBits(srcBits, port^.portBits, srcBits.bounds, theRect, srcCopy, NIL);
			SetHandleBits(fSICNHandle, oldState);
		END;

	INHERITED Draw(area);
END; {TPulldown.Draw}
	

{$S MouserEdit}
PROCEDURE TPulldown.SetCurrentItem(item: INTEGER);

BEGIN
	IF (fMenuHandle <> NIL) & (item <> fCurrentItem) THEN 
		BEGIN
			IF fMarkCurrentItem THEN 
				BEGIN
					IF fCurrentItem <> 0 THEN
						SetItemMark(fMenuHandle, fCurrentItem, ' ');
					IF item <> 0 THEN
						SetItemMark(fMenuHandle, item, CHR(checkMark));
				END;
			fCurrentItem := item;		{ can be 0, which is changed to 1 at selection time }
		END;
END; {TPulldown.SetCurrentItem}


PROCEDURE TPulldown.SetSICN(theSICNid: INTEGER; redraw: BOOLEAN);

BEGIN
	ReleaseSICN;

	IF theSICNid <> kNoResource THEN 
		BEGIN
			fSICNHandle := GetResource('SICN', theSICNid);
			FailNILResource(fSICNHandle);
			DetachResource(fSICNHandle);
			HNoPurge(fSICNHandle);
			fSICNid := theSICNid;
		END;

	IF redraw THEN ForceRedraw;
END; {TPulldown.SetSICN}


PROCEDURE TPulldown.SetMenu(aMenu: MenuHandle; menuRsrcID, currentItem: INTEGER);

BEGIN
	ReleaseMenu;
	IF aMenu <> NIL THEN 
		BEGIN
			HNoPurge(Handle(aMenu));
			fMenuHandle := aMenu;
			fRsrcID := menuRsrcID;
			fMenuID := aMenu^^.menuID;
			SetCurrentItem(currentItem);
		END;
END; {TPulldown.SetMenu}


PROCEDURE TPulldown.SetMenuFromRsrc(menuRsrcID, currentItem: INTEGER);

	VAR
		aMenu:			MenuHandle;

BEGIN
	IF menuRsrcID <> kNoResource THEN 
		BEGIN
			aMenu := GetMenu(menuRsrcID);
			{ Don't die because resource not found - just return NIL handle }
			FailResError;
			SetMenu(aMenu, menuRsrcID, currentItem);
		END;
END; {TPulldown.SetMenuFromRsrc}


(*PROCEDURE TPulldown.SetMenuFromList(itemList: TList; menuID, currentItem: INTEGER);

	VAR
		fi: 				FailInfo;
		theMenu:			MenuHandle;

	PROCEDURE AddToMenu(anItem: TNameObject);
		VAR	itemString:	Str255;
	BEGIN
		itemString := anItem.GetName;
		SELF.AppendItem(itemString);
	END;

BEGIN
	IF menuID = kNoMenuID THEN
		menuID := GetNextMenuID;			{ assign next menu id }
	{$IFC qDebug} WRITELN('SetMenuFromList: NewMenu id', menuID); {$ENDC}
	theMenu := NewMenu(menuID, kScratchMenuTitle);
	FailNIL(theMenu);
	itemList.Each(AddToMenu);				{ Create a menu item for each item in the list }
	SetMenu(theMenu, kNoResource, currentItem);
END; {TPulldown.SetMenuFromList}
*)


PROCEDURE TPulldown.SetMenuFromStrs(strListID, menuID, currentItem: INTEGER);

	VAR
		aMenu:			MenuHandle;

BEGIN
	IF menuID = kNoMenuID THEN
		menuID := GetNextMenuID;			{ assign next menu id }
	{$IFC qDebug} WRITELN('SetMenuFromStrs: NewMenu id', menuID); {$ENDC}
	aMenu := NewMenu(menuID, kScratchMenuTitle);
	FailNIL(aMenu);
	IF strListID <> kNoResource THEN		{ Look for STR# resource }
		AddStrsToMenu(aMenu, strListID);	{ add items to aMenu from the STR# }
	SetMenu(aMenu, kNoResource, currentItem);
END; {TPulldown.SetMenuFromStrs}

{$S AFields}

PROCEDURE TPulldown.Fields(PROCEDURE DoToField(fieldName: Str255; fieldAddr: Ptr;
											   fieldType: INTEGER)); OVERRIDE;

BEGIN
	DoToField('TPulldown', NIL, bClass);
	DoToField('fRsrcID', @fRsrcID, bInteger);
	DoToField('fMenuID', @fMenuID, bInteger);
	DoToField('fSICNid', @fSICNid, bInteger);
	DoToField('fMenuHandle', @fMenuHandle, bHandle);
	DoToField('fSICNHandle', @fSICNHandle, bHandle);
	DoToField('fCurrentItem', @fCurrentItem, bInteger);
	INHERITED Fields(DoToField);
END; {TPulldown.Fields}

	
{----------------------------------- TPopTextPulldown ---------------------------}
PROCEDURE TPopTextPulldown.IPopTextPulldown(itsPopText: TPopText;
																						itsSuperView: TView; itsLocation: VPoint;
																						itsTemplate: PulldownTemplatePtr);
BEGIN
	fPopText:= itsPopText;
	fTempValueStored:=FALSE;
	SELF.IPullDown(itsSuperView, itsLocation, itsTemplate);
END; {TPopTextPulldown.IPopTextPulldown}
					

PROCEDURE TPopTextPulldown.RemoveTempEntry;

BEGIN
	IF (fMenuHandle <> NIL) AND (fTempValueStored) THEN 
		BEGIN
			DelMenuItem(fMenuHandle,1);
			DelMenuItem(fMenuHandle,1);
			SELF.SetCurrentItem(0);
			fTempValueStored:=FALSE;
		END;
END; {TPopTextPulldown.RemoveTempEntry}


PROCEDURE TPopTextPulldown.AddTempEntry;
VAR 
	theCurrentItemText: Str255;
	
BEGIN
	fPopText.GetText(theCurrentItemText);
	IF fMenuHandle <> NIL THEN 
		BEGIN
			IF theCurrentItemText<> '' THEN
				SELF.AddItemAfter(CONCAT('(-;',theCurrentItemText),0)
			ELSE
				SELF.AddItemAfter('(-;(no name',0);
			SELF.SetCurrentItem(1);
			fTempValueStored:= TRUE;
		END;
END; {TPopTextPulldown.AddTempEntry}


FUNCTION TPopTextPulldown.DoMouseCommand(VAR theMouse: Point; VAR info: EventInfo;
									 VAR hysteresis: Point): TCommand; OVERRIDE;
VAR
	theItemNumber: Integer;
	
BEGIN
	theItemNumber:=fPopText.GetTextItem;
  IF (theItemNumber=0) THEN
		SELF.AddTempEntry
	ELSE IF (theItemNumber <> SELF.GetCurrentItem) THEN
		SELF.SetCurrentItem(theItemNumber);
	DoMouseCommand:= INHERITED DoMouseCommand(theMouse, info, hysteresis);
	SELF.RemoveTempEntry;
	SELF.SetCurrentItem(0);   {make sure that no item is checked}
END; {TPopTextPulldown.DoMouseCommand}


{----------------------------------- TPopText -----------------------------------}

{$S AOpen}
PROCEDURE TPopText.IRes(itsDocument: TDocument; itsSuperView: TView;
						VAR itsParams: Ptr); OVERRIDE;

VAR
	theSize: VPoint; 
	
	PROCEDURE HandleFailure(error: OSErr; message: LONGINT);
		BEGIN
		Free;
		END;

BEGIN
	fEditText := NIL;
	fPulldown := NIL;
	INHERITED IRes(itsDocument, itsSuperView, itsParams); { get Control data }
	fDefChoice := mControlHit;
	
	theSize:= fSize;
	SELF.InitTheEditText(theSize, 255);

	WITH fEditText DO 
		BEGIN
			WITH StaticTextTemplatePtr(itsParams)^ DO 
				BEGIN	{ get StaticText data }
					fJust := just;
					SetText(data, kDontRedraw);
				END;
			OffsetPtrWStr(itsParams, SIZEOF(StaticTextTemplate));
			WITH EditTextTemplatePtr(itsParams)^ DO 
				BEGIN	{ get EditText data }
					fMaxChars := maxChars;
					fControlChars := controlChars;
				END;
			fDefChoice := mEditTextHit;
			OffsetPtr(itsParams, SIZEOF(EditTextTemplate));	{ point to Pulldown data }
		END;

	SELF.InitThePulldown(theSize, PulldownTemplatePtr(itsParams));

	OffsetPtr(itsParams, SIZEOF(PulldownTemplate));
END; {TPopText.IRes}


PROCEDURE TPopText.IPopText(itsSuperView: TView; itsLocation, itsSize: VPoint;
							itsHSizeDet, itsVSizeDet: SizeDeterminer;
							itsPdInfo: PulldownTemplatePtr;
							itsMaxChars: INTEGER);

BEGIN
	fEditText := NIL;
	fPulldown := NIL;
	IControl(itsSuperView, itsLocation, itsSize, itsHSizeDet, itsVSizeDet);
	fDefChoice := mControlHit;
	
	SELF.InitTheEditText(itsSize, itsMaxChars);
	SELF.InitThePulldown(itsSize, itsPdInfo);
END; {TPopText.IPopText}


PROCEDURE TPopText.InitTheEditText(itsSize: VPoint; itsMaxChars: INTEGER);

VAR
		anEditText:		TEditText;

BEGIN
	NEW(anEditText);
	FailNIL(anEditText);
	fEditText := anEditText;
	WITH itsSize DO
		h := h - kSICNwidth - kDefaultInset;
	anEditText.IEditText(SELF, gZeroVPt, itsSize, itsMaxChars);
END; {TPopText.InitTheEditText}


PROCEDURE TPopText.InitThePulldown(itsSize: VPoint; itsPdInfo: PulldownTemplatePtr);

	VAR
		aPulldown:		TPopTextPulldown;
		aLocation:		VPoint;
		aSize:				VPoint;
		
BEGIN
	NEW(aPulldown);
	FailNIL(aPulldown);
	fPulldown := aPulldown;
	SetVPt(aLocation, itsSize.h - kSICNwidth, (itsSize.v - kSICNwidth) DIV 2);
	SetVPt(aSize, kSICNwidth, kSICNwidth);
	aPulldown.IPopTextPulldown(SELF, SELF, aLocation, itsPdInfo);
END; {TPopText.InitThePulldown}


{$AOpen}
PROCEDURE TPopText.IPopTextFromEditText(VAR theEditText:TEditText);

VAR 
	thePulldownTemplate: PulldownTemplatePtr;
	itsLocation, itsSize: VPoint;
	theHSizeDeterminer, theVSizeDeterminer: SizeDeterminer;
	theSuperView: TView;

BEGIN
	{get all the pertinent information from TEditText}
	itsLocation:= theEditText.fLocation;
	itsSize:= theEditText.fSize;
	theHSizeDeterminer:= theEditText.fSizeDeterminer[h];
	theVSizeDeterminer:= theEditText.fSizeDeterminer[v];
	theSuperView:= theEditText.fSuperView;
	
	{now initialize the global part of the PopText}
	fPulldown := NIL;
	IControl(theSuperView, itsLocation, itsSize, theHSizeDeterminer, theVSizeDeterminer);
	fDefChoice := mControlHit;

	{re-initialize the EditText, where necessary}
	theEditText.Resize(itsSize.h - kSICNwidth - kDefaultInset, itsSize.v, kDontRedraw);
	theEditText.fLocation:= gZeroVPt;
	theSuperView.RemoveSubView(theEditText);
	SELF.AddSubView(theEditText);
	fEditText := theEditText;
	
	{and initialize the pulldown}
	NEW(thePulldownTemplate);
	FailNil(thePulldownTemplate);
	WITH thePulldownTemplate^ DO
		BEGIN
			SICNid:= 128;
			menuRsrcId:= 64;
			strListID:= kNoResource;
			currentItem:=1;
			markCurrentItem:= True;
			filler:= True;
		END;
	
	{now create the Pulldown}
	SELF.InitThePulldown(itsSize, thePulldownTemplate);
	
END; {TPopText.IPopTextFromEditText}


{$S MouserEdit}
PROCEDURE TPopText.DoChoice(origView: TView; itsChoice: INTEGER); OVERRIDE;

	VAR
		item:			INTEGER;
		theText:		Str255;

BEGIN
	{$IFC qDebug}
	IF fEditText = NIL THEN ProgramBreak('TPopText.DoChoice: fEditText is NIL!');
	IF fPulldown = NIL THEN ProgramBreak('TPopText.DoChoice: fPulldown is NIL!');
	{$ENDC}

(*	IF itsChoice = mEditTextHit THEN
			fSuperView.DoChoice(fEditText, itsChoice)
	ELSE *) 
		IF itsChoice = mPulldownHit THEN 
			BEGIN
				item := fPulldown.GetCurrentItem;
				fPulldown.GetItemText(item, theText);
				fEditText.SetText(theText, kRedraw);
			END;
(*	ELSE *)
		INHERITED DoChoice(origView, itsChoice);
END; {TPopText.DoChoice}



(*FUNCTION TPopText.HandleMouseDown(theMouse: VPoint; VAR info: EventInfo; VAR hysteresis: Point;
								  VAR theCommand: TCommand): BOOLEAN; OVERRIDE;

	VAR
		theQDMouse, subViewPt:			VPoint;

BEGIN
	HandleMouseDown := FALSE;

	IF fPulldown <> NIL THEN
		HandleMouseDown := fPulldown.HandleMouseDown(theMouse, info, hysteresis, theCommand);
	subViewPt := theMouse;
	fPulldown.SuperToLocal(subViewPt);
	IF fPulldown.ContainsMouse(subViewPt) & IsViewEnabled & Focus THEN					{see if we can handle it}
		BEGIN
			theQDMouse := ViewToQDPt(theMouse);
			theCommand := DoMouseCommand(theQDMouse, info, hysteresis);
				 THEN BEGIN
				theCommand := fPulldown.DoMouseCommand(theMouse, info, hysteresis);
				HandleMouseDown := TRUE;
				END;
		END;

	{ fEditText can fend for itself }
	END; {TPopText.HandleMouseDown}

*)

{$S MouserEdit}
PROCEDURE TPopText.GetText(VAR theText: Str255);

BEGIN
	{$IFC qDebug}
	IF fEditText = NIL THEN ProgramBreak('TPopText.GetText: fEditText is NIL!');
	{$ENDC}

	IF fEditText <> NIL THEN
		fEditText.GetText(theText);
END; {TPopText.GetText}


FUNCTION TPopText.GetEditText: TEditText;

BEGIN
	GetEditText := fEditText;
END; {TPopText.GetEditText}


FUNCTION TPopText.GetPulldown: TPulldown;

BEGIN
	GetPulldown := fPulldown;
END; {TPopText.GetPulldown}


FUNCTION TPopText.GetTextItem: INTEGER;
{ Try to match the EditText string with an item in the Pulldown menu.
  Return the item number, or 0 if no match. }

	VAR
		theName:		Str255;

BEGIN
	fEditText.GetText(theName);
	GetTextItem:=fPulldown.GetItemIndex(theName);
END; {TPopText.GetTextItem}


PROCEDURE TPopText.SetText(theText: StringPtr; redraw: BOOLEAN);

BEGIN
	{$IFC qDebug}
	IF fEditText = NIL THEN ProgramBreak('TPopText.SetText: fEditText is NIL!');
	{$ENDC}
	(* Enforce string length <= fEditText.fMaxChars? *)

	IF fEditText <> NIL THEN
		fEditText.SetText(theText^, redraw);
END; {TPopText.SetText}


PROCEDURE TPopText.SetTextToCurrentItem(redraw: BOOLEAN);

	VAR
		item:			INTEGER;
		theText:		Str255;

BEGIN
	item := fPulldown.GetCurrentItem;
	IF item > 0 THEN 
		BEGIN
			fPulldown.GetItemText(item, theText);
			IF redraw THEN 
				fEditText.RestartEdit(theText)
			ELSE 
				fEditText.SetText(theText, redraw);
		END;
END; {TPopText.SetTextToCurrentItem}


PROCEDURE TPopText.TrackMouse(aTrackPhase: TrackPhase;
							  VAR anchorPoint, previousPoint, nextPoint: VPoint;
							  mouseDidMove: BOOLEAN);
BEGIN
	IF (aTrackPhase = trackRelease) AND ContainsMouse(nextPoint) THEN 
		DoChoice(SELF, fDefChoice);
END; {TPopText.TrackMouse}


PROCEDURE TPopText.Show(state, redraw: BOOLEAN); OVERRIDE;

BEGIN
	fEditText.Show(state, redraw);
	fPulldown.Show(state, redraw);
	INHERITED Show(state, redraw);
END; {TPopText.Show}


PROCEDURE TPopText.ViewEnable(state, redraw: BOOLEAN); OVERRIDE;

BEGIN
	fEditText.ViewEnable(state, redraw);
	fPulldown.ViewEnable(state, redraw);
	INHERITED ViewEnable(state, redraw);
END; {TPopText.ViewEnable}


{$S AFields}
PROCEDURE TPopText.Fields(PROCEDURE DoToField(fieldName: Str255; fieldAddr: Ptr;
											  fieldType: INTEGER)); OVERRIDE;

BEGIN
	DoToField('TPopText', NIL, bClass);
	DoToField('fEditText', @fEditText, bObject);
	DoToField('fPulldown', @fPulldown, bObject);
	INHERITED Fields(DoToField);
END; {TPopText.Fields}

