UNIT UPopText;

	INTERFACE

		USES
			UMacApp, UTEView, UDialog,
		{	UMouserObjects, UPath, UArray,}
			ToolIntf, OSIntf;

		CONST

			
			kNotFound = 0;
			mPulldownHit		= 24;

			kItemOffset			= 50;					{ fItemOffset value for TPulldown }

			kSICNwidth			= 16;
			kDefaultInset		=  3;					{ gap between TEditText box and TPulldown SICN }

			kNoMenuID		 	=  0;					{ pass this value in the menuID field
														  of the PulldownTemplate if you want
														  the menu id to be assigned automagically. }
			kScratchMenuTitle	= 'Menu X';

			kStdPulldown		= 'puld';				{ resource signature for a TPulldown }
			kStdPopText			= 'popt';				{ resource signature for a TPopText }

		TYPE

			PulldownTemplate	= RECORD
				SICNid:				INTEGER;			{ id of SICN resource }
				menuRsrcID: 		INTEGER;			{ id of MENU resource, or kNoResource if none }
				menuID:				INTEGER;			{ id of menu, when there's no resource }
				strListID:			INTEGER;			{ id of STR# resource, or kNoResource if none }
				currentItem:		INTEGER;			{ default menu item }
				markCurrentItem:	BOOLEAN;			{ T: put check mark by current item }
				filler:				BOOLEAN;			{ word alignment }
				END;
			PulldownTemplatePtr	= ^PulldownTemplate;

			TPulldown			= OBJECT (TControl)
			{ This class is similar to a popup menu, but it has no label, and instead of
			  showing a rectangle with text it shows a small icon. }

				fSICNid:			INTEGER;			{ id of small icon resource }
				fSICNHandle:		Handle;				{ Handle to SICN resource data }
				fRsrcID:			INTEGER;			{ id of MENU resource, or kNoResource }
				fMenuID:			INTEGER;			{ id of menu }
				fMenuHandle:		MenuHandle;			{ Handle to menu data }
				fCurrentItem:		INTEGER;
				fMarkCurrentItem:	BOOLEAN;			{ T: put check mark by current item }

				{ Initialization & Disposal }

				PROCEDURE TPulldown.IRes(itsDocument: TDocument; itsSuperView: TView;
										 VAR itsParams: Ptr); OVERRIDE;

				PROCEDURE TPulldown.IPulldown(itsSuperView: TView; itsLocation: VPoint;
											  itsTemplate: PulldownTemplatePtr);
				{ Initialize the pulldown menu from whatever resources are specified in the
				  PulldownTemplate.
				  SICNid is the resource id of the small icon placeholder.
				  menuRsrcID is the resource id of a MENU resource. If menuRsrcID is 0,
				  IPulldown just creates a new, empty menu with the id menuID.
				  strListID is the resource id of a STR# resource. If strListID is not 0,
				  menu items are built from the strings and appended to the menu. }

				PROCEDURE TPulldown.InitPulldown(pdTemplatePtr: PulldownTemplatePtr);
				{ Initialize the pulldown menu from whatever resources are specified in
				  the PulldownTemplate. Used by both IRes and IPulldown. }

				PROCEDURE TPulldown.AddItemAfter(aString: Str255; theIndex: INTEGER);
				PROCEDURE TPulldown.AppendItem(aString: Str255);
				PROCEDURE TPulldown.RemoveItem(aString: Str255);
				PROCEDURE TPulldown.ChangeItem(theOldValue, theNewValue: Str255);
				FUNCTION TPulldown.GetItemIndex(aString: Str255):INTEGER;
				PROCEDURE TPulldown.Free; OVERRIDE;

				PROCEDURE TPulldown.ReleaseMenu;
				{ Release the menu stuff }

				PROCEDURE TPulldown.ReleaseSICN;
				{ Release the small icon stuff }

				{ Our OVERRIDing Concern }

				FUNCTION  TPulldown.DoMouseCommand(VAR theMouse: Point;
												   VAR info: EventInfo;
												   VAR hysteresis: Point): TCommand; OVERRIDE;

				PROCEDURE TPulldown.Draw(area: Rect); OVERRIDE;
				{ Draw the small icon, which shows the user where to click to
				  pull down the menu. }

				{ Various getters and setters }

				PROCEDURE TPulldown.AddStrsToMenu(theMenu: MenuHandle; theStrRsrcID: INTEGER);
				{ Create a menu item from each string in the specified STR# resource,
				  and append it to theMenu. }
				
				FUNCTION  TPulldown.GetCurrentItem: INTEGER;
				{ Whats the current item? }
				
				FUNCTION TPulldown.GetItemCount: INTEGER;
				{ Return the number of items in this pulldown menu. }

				PROCEDURE TPulldown.GetItemText(item: INTEGER; VAR theText: Str255);
				{ Retrieve the text of the given item. }

				PROCEDURE TPulldown.SetCurrentItem(item: INTEGER);
				{ Set a new current item }
				
				PROCEDURE TPulldown.SetSICN(theSICNid: INTEGER; redraw: BOOLEAN);
				{ Assign the specified SICN resource to the pulldown. }
				
				PROCEDURE TPulldown.SetMenu(aMenu: MenuHandle; menuRsrcID, currentItem: INTEGER);
				{ This pulldown gets a new menu! This method is called by the next three
				  to change the menu. }
				
				PROCEDURE TPulldown.SetMenuFromRsrc(menuRsrcID, currentItem: INTEGER);
				{ Make a new menu from the specified MENU resource. }
				
				(*PROCEDURE TPulldown.SetMenuFromList(itemList: TList; menuID, currentItem: INTEGER);
				{ Make a new menu from the items in the given list. The list elements
				  are assumed to be TNameObjects. }*)
				
				PROCEDURE TPulldown.SetMenuFromStrs(strListID, menuID, currentItem: INTEGER);
				{ Make a new menu from the strings in the specified STR# resource. }
				
				
				{ Debugging }

				PROCEDURE TPulldown.Fields(PROCEDURE DoToField(fieldName: Str255;
															   fieldAddr: Ptr;
															   fieldType: INTEGER)); OVERRIDE;
				END;

			TPopTextPulldown			= OBJECT (TPulldown)
				fPopText: TPopText;
				fTempValueStored: BOOLEAN;
				PROCEDURE TPopTextPulldown.IPopTextPulldown(itsPopText: TPopText;
													itsSuperView: TView; 
													itsLocation: VPoint;
											 		itsTemplate: PulldownTemplatePtr);
				FUNCTION TPopTextPulldown.DoMouseCommand(VAR theMouse: Point;
												   VAR info: EventInfo;
												   VAR hysteresis: Point): TCommand; OVERRIDE;
				PROCEDURE TPopTextPulldown.RemoveTempEntry;
				PROCEDURE TPopTextPulldown.AddTempEntry;
			END;

(*	From UTEView.p--
			ControlCharSet		= SET OF Chr00..Chr1F;
*)

(*	From UDialog.p--
			EditTextTemplate	= RECORD
				maxChars:			INTEGER;
				controlChars:		ControlCharSet;
				END;
*)
			PopTextTemplate	= RECORD
				edittextInfo:		EditTextTemplate;	{ info about EditText part of control }
				pulldownInfo: 		PulldownTemplate;	{ info about Pulldown part of control }
				END;
			PopTextTemplatePtr	= ^PopTextTemplate;

			TPopText			= OBJECT (TControl)
			{ TPopText is a control object composed of an EditText part and a Popup part.
			  The user can either type in her choice or select it from the attached menu. }

				fEditText:			TEditText;			{ the editText part of this control }
				fPulldown:			TPopTextPulldown; 	{ the menu part of this control }

				{ Initialization & Disposal }

				PROCEDURE TPopText.IRes(itsDocument: TDocument; itsSuperView: TView;
										VAR itsParams: Ptr); OVERRIDE;

				PROCEDURE TPopText.IPopText(itsSuperView: TView; itsLocation, itsSize: VPoint;
											itsHSizeDet, itsVSizeDet: SizeDeterminer;
											itsPdInfo: PulldownTemplatePtr;
											itsMaxChars: INTEGER);
				{ Create and initialize the EditText and menu components. }

				PROCEDURE TPopText.InitTheEditText(itsSize: VPoint; itsMaxChars: INTEGER);
				PROCEDURE TPopText.InitThePulldown(itsSize: VPoint; 
																						itsPdInfo: PulldownTemplatePtr);

				PROCEDURE TPopText.IPopTextFromEditText(VAR theEditText: TEditText);

(*				PROCEDURE TPopText.Free; OVERRIDE;
		* dont need, because fEditText and fPulldown are real subviews,
		  so TView.Free will take care of them. *)

				{ The Right Stuff }

				PROCEDURE TPopText.DoChoice(origView: TView; itsChoice: INTEGER); OVERRIDE;

(*				FUNCTION TPopText.HandleMouseDown(theMouse: VPoint; VAR info: EventInfo; 
																					VAR hysteresis: Point;
																					VAR theCommand: TCommand): BOOLEAN; OVERRIDE;
*)

				PROCEDURE TPopText.GetText(VAR theText: Str255);
				{ Return the control's current text. }

				PROCEDURE TPopText.SetText(theText: StringPtr; redraw: BOOLEAN);
				{ Change the control's string to that pointed to by theText }

				{ Misc getters & setters }

				FUNCTION  TPopText.GetEditText: TEditText;
				FUNCTION  TPopText.GetPulldown: TPulldown;

				FUNCTION  TPopText.GetTextItem: INTEGER;
				{ Try to match the EditText string with an item in the Pulldown menu.
				  Return the item number, or 0 if no match. }

				PROCEDURE TPopText.SetTextToCurrentItem(redraw: BOOLEAN);
				{ Set the EditText to that of the Pulldown's current item }

				{ Debugging }

				PROCEDURE TPopText.TrackMouse(aTrackPhase: TrackPhase;
												VAR anchorPoint, previousPoint, nextPoint: VPoint;
												mouseDidMove: BOOLEAN); OVERRIDE;

				PROCEDURE TPopText.Show(state, redraw: BOOLEAN); OVERRIDE;

				PROCEDURE TPopText.ViewEnable(state, redraw: BOOLEAN); OVERRIDE;

				PROCEDURE TPopText.Fields(PROCEDURE DoToField(fieldName: Str255;
															  fieldAddr: Ptr;
															  fieldType: INTEGER)); OVERRIDE;
				END;

		PROCEDURE InitUPopText(firstMenuId: INTEGER);
		{ Call this at the start of your program, before creating any TPulldown or
		  TPopText objects.  }

	IMPLEMENTATION

		{$I UPopText.inc1.p}

END.