/*******************************************************************************
*
* Application:   MacApp Drafter, a sample C++ program by Danie Underwood
*
* File:   UMacApp Drafter.cp
*
* Description: Main Program
*
* Apple Integrated Systems
*
* 1990 Apple Computer, Inc. All rights reserved.
*
* This software is a sample program.  Use it at your own risk!
*
********************************************************************************/



/*******************************************************************************

	NOTES

	The standard file interface requires specific segmentation for unkown reasons
	and does not correctly display the menubar as disabled.

	The failure handling mechanism has not been tested with a chain of handlers
	installed. (Multiple, nested calls to CatchFailures)  The mechanism has been
	tested as coded.

	The two filter procs should be coded as application methods and short global
	procs would call them.  This allows clients to override them!!!

	File names, tags and other string constants should be moved to resources.

*/

#include <SetJmp.h>
#include <CursorCtl.h>
#include "UMacApp Drafter.h"

//
// GLOBAL constants & whatnot:

const long kSignature			='HELP';		//Application Signature
const ResType rTextResource		='Text';		//Template resource type
const short kPrimeColorWindow	=2000;			//'view' id for main window [COLOR]
const short kPrimeBWWindow		=2001;			//'view' id for main window [B&W]
const short kSFPDialogView		=1888;			//'view' id for Standard File Dialog
const long kCurDirStore			=0x00000398;	//low memory standard file: current directory id
const long kSFSaveDisk			=0x00000214;	//low memory standard file: current volume refnum

struct PDimStateData {			//subroutine data for an object iterator (EachSubViewDo)
	Boolean			doDim;
};

struct PSubViewClipData {		//subroutine data for an object iterator (EachSubViewDo)
	RgnHandle			scratchRgn;
	RgnHandle			clipRgn;
};


//
// GLOBAL variables & notwhat:

TDrafterApplication 			*pApplication;		//The application, in its own class.
TClearWindow					*pSFDialogWindow;	//The MacApp TWindow in standard file
DialogPtr						pSFDialog;			//The toolbox window in standard file
SFReply							pReply;				//The standard file result record
Boolean							pMenusDone;			//Flag to setup menus under standard file

// Failure Handler variables & notnot:
short	gFailError;				//Most recent Failure Handler error code
long	gFailMessage;			//Most recent Failure Handler message code




//
// Global routines & nonsense

// ==Failure Handler==
//	   This procedure is called by the MacApp failure handling system each time
//		a failure occurs.  It jumps to the most recent failure handling expression
//		for actual failure processing.

#pragma segment Main
pascal void CommonHandler(short errorCode,long messageCode,void *Failure_StaticLink)
{
	gFailError=errorCode;
	gFailMessage=messageCode;

	longjmp((jmp_buf)Failure_StaticLink,1);
};

// ==Membership tester==
//	   This function accepts a pascal object and a pascal string containing
//		a class name.  If the object is a member of the named class, the value
//		(true) is returned, otherwise (false).

Boolean Member(TObject *anObject,StringPtr className)
{
	return (anObject->IsMemberClass(GetClassIDFromName(className)));
};


// ==String List Element Counter==
//	   This function returns the count of strings in a string list (STR#) resource.
//		The resource id number for the string list is required.  If the list does not
//		exist, zero is returned as the count.

#pragma segment ADrafter
short StrListCount(short listId)
{
	short		**stringList;

	stringList=(short**)GetResource('STR#',listId);
	if (stringList)
		return (**stringList);
	else
		return 0;
};

// ==RGB value retriever==
//	   This procedure accepts a resource id for an RGB value list (RGB#) resource
//		and an index value.  It returns the RGB value stored at that index.
//		NO EXPLICIT RANGE CHECKING IS IMPLEMENTED!

typedef RGBColor RGBColorList[256];

#pragma segment ADrafter
void GetRGBIndexed(short listId,short colorIndex,RGBColor *theRGB)
{
	RGBColorList	**rgbList;

	rgbList=(RGBColorList**)GetResource('RGB#',listId);
	*theRGB=(**rgbList)[colorIndex-1];
};

// ==Numeric String To Long Maker==
//	   This function returns long integer value corresponding to the numeric text
//		string supplied.

#pragma segment ADrafter
long NumOf(const Str255 theString)
{
	long		num;

	StringToNum(theString,&num);

	return num;
};

// ==Hexadecimal String To Long Maker==
//	   This function returns long integer value corresponding to the hexadecimal text
//		string supplied.  Non-hexadecimal characters are ignored.

#pragma segment ADrafter
long NumOfHex(const Str255 theString)
{
	long			num;
	long			digit;
	short			i;
	unsigned char	hexCh;

	num=0;
	for (i=1;i <= Length(theString);i++)
	{
		digit=-1;
		hexCh=theString[i];
		if ((hexCh >= '0') && (hexCh <= '9'))
			digit=hexCh-'0';
		else if ((hexCh >= 'A') && (hexCh <= 'F'))
			digit=hexCh-'A'+10;
		else if ((hexCh >= 'a') && (hexCh <= 'f'))
			digit=hexCh-'a'+10;

		if (digit >= 0)
			num=((num<<4) & 0xFFFFFFF0) | (digit & 0x0000000F);
	};
	return num;
};

// ==Pascal string joiner==
//	   This procedure concatenates two pascal strings into the given target string.
//		The source and destinations may overlap and the destination length is
//		truncated to a maximum value of 255.

#pragma segment ADrafter
pascal void PConcat(StringPtr result,const Str255 s1,const Str255 s2)
{
	short		lth1;
	short		lth2;
	short		totalLength;
	Str255		tempStr;	//Allows overlapping source & targets

	lth1=Length(s1);
	lth2=Length(s2);

	if (lth1)
		BlockMove((Ptr)&s1[1],(Ptr)&tempStr[1],lth1);

	if ((lth1+lth2) > 255)
		lth2=255-lth1;

	if (lth2)
		BlockMove((Ptr)&s2[1],(Ptr)&tempStr[lth1+1],lth2);

	totalLength=lth1+lth2;

	BlockMove((Ptr)&tempStr[1],(Ptr)&result[1],totalLength);
	result[0]=totalLength;
};

//
// Object methods & sensenon

// ==Application initializer==
//	   This method prepares the application to operate.

#pragma segment ARes
pascal void TDrafterApplication::IDrafterApplication(void)
{
	TObject			*anyThing;

	pApplication=NULL;			//Set early to be prepared to handle initialization exceptions.

	IApplication(kSignature);

	pApplication=this;

	pSFDialogWindow=NULL;
	pSFDialog=NULL;

	InitCursorCtl(NULL);		//For animated busy cursor

	if (gDeadStripSuppression)	//Some of the many wonderful classes entertaining us today.
	{
		anyThing=(TObject *) new TClearDialog;
		anyThing=(TObject *) new TClearWindow;
		anyThing=(TObject *) new TDrafterDoc;
		anyThing=(TObject *) new TDrafterButton;
		anyThing=(TObject *) new TDrafterWindow;
		anyThing=(TObject *) new TDrafterView;
		anyThing=(TObject *) new TDrafterCluster;
		anyThing=(TObject *) new TDrafterNoClipCluster;
		anyThing=(TObject *) new TDrafterClearCluster;
		anyThing=(TObject *) new TDrafterFrameCluster;
		anyThing=(TObject *) new TDrafterHollowCluster;
		anyThing=(TObject *) new TDrafterBrassCluster;
		anyThing=(TObject *) new TDlgGeneral;
		anyThing=(TObject *) new TDlgFinderInfo;
		anyThing=(TObject *) new TDlgPrefs;
		anyThing=(TObject *) new TDlgDocs;
		anyThing=(TObject *) new TDlgViews;
		anyThing=(TObject *) new TDlgBuild;
		anyThing=(TObject *) new TDlgMenus;
	};
};

// ==Application Debugging==
//	   This method traps to the debugger when the F15 key is pressed.
//		THIS SHOULD BE MADE CONDITIONAL SO THAT IT DOESNT LIVE IN THE FINAL VERSION!

#pragma segment ADrafter
pascal TCommand *TDrafterApplication::DoKeyCommand(short ch,short aKeyCode,EventInfo *info)
{
#if qDebug
	if (aKeyCode==kF15VirtualCode)
		Debugger();
#endif qDebug

	return (inherited::DoKeyCommand(ch,aKeyCode,info));
};

// ==Creating The Document==
//	   This method creates a new document object given ANY input.

#pragma segment ADrafter
pascal TDocument *TDrafterApplication::DoMakeDocument(CmdNumber itsCmdNumber)
{
	TDrafterDoc		*doc;

	switch (itsCmdNumber)
	{
		case cFinderNew:
		case cNew:
		default:
			FailNIL(doc=new TDrafterDoc);
			doc->IDrafterDoc();
			break;
	};
	return doc;
};

// ==Controlling Menus==
//	   This method disables the creation of documents.  Since no documents
//		are filed Open makes no sense and New is not needed since the
//		default, untitled document is not closeable.

#pragma segment ACommand
pascal void TDrafterApplication::DoSetupMenus(void)
{
	inherited::DoSetupMenus();

	Enable(cNew,false);
	Enable(cOpen,false);
};

// ==MacApp Standard File Patch==
//	   This method is necessary for views in the standard file dialogs.
//		MacApp stores its TWindows in the window refCon field, which the
//		standard file package uses for its own purposes.  The patch is to
//		use two global variables: one for the standard file toolbox window
//		and one for the MacApp TWindow.  When the given window is the
//		standard file window, the value of the global variable pSFDialogWindow
//		is returned.

#pragma segment ACommand
pascal TWindow *TDrafterApplication::WMgrToWindow(WindowPtr aWMgrWindow)
{
	if (aWMgrWindow==(WindowPtr)pSFDialog)
		return pSFDialogWindow;
	else
		return inherited::WMgrToWindow(aWMgrWindow);
};

// ==MacApp Standard File Patch==
//	   This method is necessary for views in the standard file dialogs.
//		It is a copy of the MacApp TWindow.IWindow source modified to not
//		use the refCon field of the window manager window and to default
//		settings for this specific use.  Changes from the MacApp source are
//		marked: CHANGE.

#pragma segment ADrafter
pascal void TClearWindow::IWindow(TDocument *itsDocument,GrafPtr itsWMgrWindow,Boolean canResize,
	   Boolean canClose,Boolean disposeOnFree)
{
	Point			p;
	VPoint			location;
	VPoint			siz;

	fWMgrWindow=itsWMgrWindow;
	fFreeOnClosing=true;				//CHANGE
	fDisposeOnFree=disposeOnFree;

	SetPort(itsWMgrWindow);
	p.v=itsWMgrWindow->portRect.top;
	p.h=itsWMgrWindow->portRect.left;
	LocalToGlobal(&p);
	PtToVPt(p,&location);
	p.v=itsWMgrWindow->portRect.bottom-itsWMgrWindow->portRect.top;
	p.h=itsWMgrWindow->portRect.right-itsWMgrWindow->portRect.left;
	PtToVPt(p,&siz);

	IView(NULL,NULL,&location,&siz,sizeVariable,sizeVariable);		//CHANGE
	fShown=false;						//CHANGE

	fProcID=dBoxProc;					//CHANGE

	fAdapted=true;						//CHANGE
	fHorzCentered=true;					//CHANGE
	fVertCentered=true;					//CHANGE
	fStaggered=true;					//CHANGE
	fForcedOnScreen=true;				//CHANGE
	fMustAdapt=false;
	fMustHorzCenter=false;
	fMustVertCenter=false;
	fMustStagger=false;
	fMustForceOnScreen=false;

	fIsActive=false;
	fIsResizable=canResize;
	fIsClosable=canClose;
	fTarget=this;
	fIdentifier='WIND';					//CHANGE
	fTargetID=fIdentifier;
	fClosesDocument=false;				//CHANGE
	fOpenInitially=false;				//CHANGE
	fIsModal=true;						//CHANGE
	fDoFirstClick=true;					//CHANGE
	fFloats=false;

//This section has been rewritten to avoid calls that are unnecessary for this
//	type of window.  The failure handling, document and name code is different.

	fContRgnInset.v=0;
	fContRgnInset.h=0;
	fContDifference.v=0;
	fContDifference.h=0;
	fMoveBounds=gStdWMoveBounds;
	BlockMove((Ptr)&gStdWSizeRect,(Ptr)&fResizeLimits,sizeof(Rect));
	fPreDocname=0;
	fConstTitle=0;

	fNextHandler=gApplication;
	fDismissView='    ';
	fSFHitItem=0;						//SPECIFIC to this purpose
};

// ==MacApp Standard File Patch==
//	   This method permits the disposal of the TWindow object without
//		disposing the window manager window that the standard file package
//		created (and will dispose of).

#pragma segment LockDown
pascal void TClearWindow::Free(void)
{
	fWMgrWindow=NULL;		//Not created by MacApp- so cant hide or dispose of it here.

	if (fDocument)
		fDocument->DeleteWindow(this);
	else
		gApplication->DeleteFreeWindow(this);

	TView::Free();			//Trick to skip right over TWindow.Free and still release subviews!!!
};

// ==MacApp Standard File Patch==
//	   This method modifies the window activation mechanism for use under standard file.

#pragma segment ADrafter
pascal void TClearWindow::Activate(Boolean entering)
{
	TView			*subView;

	InvalidateFocus();
	fIsActive=entering;

	if (fSubViews)
		if (subView=(TView*)fSubViews->First())	//Only 1 direct subview is allowed in this window
			subView->Activate(entering);

	if (entering)
		gApplication->SetTarget(fTarget);
};

// ==MacApp Standard File Patch==
//	   This method should always return false for standard file dialogs.

#pragma segment ADrafter
pascal Boolean TClearWindow::AllowsMenuAccess(void)
{
	return false;
};

// ==MacApp Standard File Patch==
//	   This method draws the windows subviews, but not the window since it would
//		erase the standard file items.

#pragma segment ADrafter
pascal void TClearWindow::DrawContents(void)
{
	TView::DrawContents();	//Trick to skip right over TWindow.DrawContents and draw subviews!!!
							//This doesnt translate to Object Pascal.  This feature was removed.
};

// ==MacApp Standard File Addition==
//	   This method returns the identifier of the view that dismissed the standard
//		file dialog.

#pragma segment ADrafter
pascal ResType TClearWindow::GetDisMissView(void)
{
	return fDismissView;
};

// ==MacApp Standard File Patch==
//	   This method discards the TWindow HandleMouseDown in favor of its inherited behavior.

#pragma segment ADrafter
pascal Boolean TClearWindow::HandleMouseDown(VPoint *theMouse, EventInfo *info, Point *hysteresis, 
	   struct TCommand **theCommand)
{
			//This implementation is a trick to get to TView.HandleMouseDown
			//without calling TWindow.HandleMouseDown.  TWindow does things
			//that dont work under the standard file package.

	return TView::HandleMouseDown(theMouse,info,hysteresis,theCommand);
};

// ==MacApp Standard File Addition==
//	   This method records the last item hit among the standard file items.

#pragma segment ADrafter
pascal void TClearWindow::HitItem(short itemNumber)
{
	fSFHitItem=itemNumber;
};

// ==MacApp Standard File Addition==
//	   This method returns the number of the last item hit among the standard file items.

#pragma segment ADrafter
pascal short TClearWindow::LastItemHit(void)
{
	return fSFHitItem;
};

// ==MacApp Standard File Addition==
//	   This method records the identifier of the view dismissing the standard file dialog.

#pragma segment ADrafter
pascal void TClearWindow::SetDisMissView(ResType viewId)
{
	fDismissView=viewId;
};

// ==MacApp Standard File Patch==
//	   This method draws the windows subviews without erasing the window first or
//		calling BeginUpdate/EndUpdate.  It also saves and restores the port configuration
//		as expected by standard file.

#pragma segment ADrafter
pascal void TClearWindow::Update(void)
{
	PenState		savePen;
//	TextStyle		theTextStyle;

	GetPenState(&savePen);
//	GetPortTextStyle(&theTextStyle);	****This doesnt work under C++ x????????

	InvalidateFocus();
	DrawContents();
	InvalidateFocus();

//	SetPortTextStyle(&theTextStyle);
	TextFont(0);
	TextSize(12);
	TextFace(0);
	SetPenState(&savePen);
};

// ==MacApp Standard File Patch==
//	   This function is a modal dialog filter proc.  It provides a hook into
//		the MacApp world from the toolbox world.

#pragma segment LockDown
pascal Boolean ModalObjectFilter(DialogPtr theDialog,EventRecord *theEvent,short *itemHit)
{
	TClearDialog	*view;
	TCommand		*aCommand;
	Point			theMouse;
	VPoint			theVMouse;
	Point			hysteresis;
	Handle			item;
	Rect			itsBox;
	short			itemType;
	EventInfo		theEventInfo;
	Boolean			okEnabled;
	Boolean			result;
	Boolean			cancelKey;
	Boolean			okKey;

	result=false;

	if (pSFDialogWindow)
	{


//	The MacApp views in the standard file dialog must be rooted to a TDialogView
//	with the identifier: 'DLOG'.


		if (view=(TClearDialog*)pSFDialogWindow->FindSubView('DLOG'))
		{
			theEventInfo.thePEvent=theEvent;
			theEventInfo.theBtnState=((theEvent->modifiers & btnState)!=0);
			theEventInfo.theCmdKey=((theEvent->modifiers & cmdKey)!=0);
			theEventInfo.theShiftKey=((theEvent->modifiers & shiftKey)!=0);
			theEventInfo.theAlphaLock=((theEvent->modifiers & alphaLock)!=0);
			theEventInfo.theOptionKey=((theEvent->modifiers & optionKey)!=0);
			theEventInfo.theControlKey=((theEvent->modifiers & controlKey)!=0);
			theEventInfo.theAutoKey=(theEvent->what==autoKey);
			theEventInfo.theClickCount=gClickCount;
			theEventInfo.theCharacter=0;
			theEventInfo.theKeyCode=0;
			theEventInfo.affectsMenus=false;

//	Test the standard OPEN/SAVE/OK button to see if it is enabled.

			GetDItem(theDialog,getOpen,&itemType,&item,&itsBox);
			okEnabled=(itemType<=ctrlItem) &&
					((*(ControlHandle)item)->contrlVis > 0) &&
					((*(ControlHandle)item)->contrlHilite!=255);

			view->SetOkEnable(okEnabled);

//	Process selected events

			switch (theEvent->what)
			{
				case keyDown:
				case autoKey:
					gTarget->KeyEventToComponents(&theEventInfo);
					okKey=(theEventInfo.theCharacter==3) || (theEventInfo.theCharacter==13);
					cancelKey=(theEventInfo.theCharacter=='.') && theEventInfo.theCmdKey;

//	If no textedit in the standard dialog, let the application process all of
//	the keys except return and enter keys.

					if ((((DialogPeek)theDialog)->editField < 0) &&	//No TE items in dialog
								!okKey && !cancelKey)
					{
						if (aCommand=gApplication->HandleKeyDownEvent(&theEventInfo))
						{
							gApplication->PostCommand(aCommand);

							*itemHit=100;		//NULL event (idle)
							result=true;
						};
					}
					else

//otherwise pass all keys to standard file except enter and return

					{					
						if (okEnabled)
						{
							if (okKey)
							{
								result=true;
								*itemHit=1;
							}
							else if (cancelKey)
							{
								result=true;
								*itemHit=getCancel;
							};
						};
					};
					break;

//	Handle mouse down events in the MacApp way.

				case mouseDown:
					if (view->Focus())
					{
						theMouse=theEvent->where;
						GlobalToLocal(&theMouse);
						view->QDToViewPt(theMouse,&theVMouse);
						hysteresis=gStdHysteresis;
						if (view->HandleMouseDown(&theVMouse,&theEventInfo,&hysteresis,&aCommand))
						{
							if (aCommand)
							{
								aCommand->fTracksMouse=true;
								aCommand->fInitialPt=theEvent->where;
								gApplication->PostCommand(aCommand);
							};
							if (view->fDismissed)
							{
								pSFDialogWindow->SetDisMissView(view->fDismisser);	//Record real dismisser
								*itemHit=getCancel;		//Tell SF to exit
								result=true;
							};
						};
					};
					break;

				case updateEvt:
					pSFDialogWindow->Update();	//Update MacApp world and pass update to SF too.
					break;
			};
		};
	};

	return result;
};

// ==MacApp Standard File Patch==
//	   This filter proc interfaces the standard file package world to the MacApp
//		world.  It also adjusts the standard dialog window to include the additional
//		views defined in the 'view' kSFPDialogView resource.

#pragma segment LockDown
pascal short SFObjectFilter(short item,DialogPtr theDialog)
{
	TClearDialog	*view;
	TCommand		*commandToPerform;
	VPoint			sizeDiff;
	short			message;

	pSFDialog=theDialog;

	if (item== -1)		//Install the MacApp world just before the dialog is made visible.
	{
		if (!pSFDialogWindow)
		{
			FailNIL(pSFDialogWindow=new TClearWindow);

			SetPort((GrafPtr)theDialog);
			pSFDialogWindow->IWindow(NULL,(GrafPtr)theDialog,false,false,true);

			pSFDialogWindow->DoCreateViews(NULL,pSFDialogWindow,kSFPDialogView,&gZeroVPt);

			if (view=(TClearDialog*)pSFDialogWindow->FindSubView('DLOG'))	//NOTE identifier
			{
				sizeDiff.h=(view->fSize.h - pSFDialogWindow->fSize.h);
				sizeDiff.v=(view->fSize.v - pSFDialogWindow->fSize.v);

				if (sizeDiff.h || sizeDiff.v)
				{
					pSFDialogWindow->Locate(pSFDialogWindow->fLocation.h-(sizeDiff.h /2),
										pSFDialogWindow->fLocation.v-(sizeDiff.v /2),false);
					pSFDialogWindow->Resize(view->fSize.h,view->fSize.v,false);
				};
			};
			InvalRect(&((GrafPtr)theDialog)->portRect);
		};
	}
	else if ((item==getCancel) || (item==getOpen))
	{
		if (pSFDialogWindow && (item==getCancel))	//about to go away
		{
			pSFDialogWindow->fShown=false;
			pSFDialogWindow->fIsActive=false;
		};
	}
	else if (pSFDialogWindow)
	{
		if (!pSFDialogWindow->fShown)		//Make MacApp window logicly visible
		{
			pApplication->InvalidateFocus();
			pApplication->SetTarget(pApplication);

			if (pSFDialogWindow->Focus())
			{
				pSFDialogWindow->Open();
				pApplication->SetTarget(pSFDialogWindow);
				pApplication->SetupTheMenus();
			};
		};

		if (!pMenusDone)
		{
			pApplication->SetupTheMenus();
			pMenusDone=true;
		};

		view=(TClearDialog*)pSFDialogWindow->FindSubView('DLOG');

		if (item == 100)		//Idle time
		{
			if (pSFDialogWindow->fShown)
				if (commandToPerform=pApplication->GetNextCommand())
					pApplication->PerformCommand(commandToPerform);
		}
		else	//Give the MacApp world a chance to look at the message
		{
			message=item+10000;
			pApplication->InvalidateFocus();
			if (view)
				view->DoChoice(NULL,message);
			else
				pSFDialogWindow->DoChoice(NULL,message);
		};

		if (view && view->fDismissed && (item!=getCancel))
		{
			pSFDialogWindow->SetDisMissView(view->fDismisser);
			item=getCancel;
		};

		if (pSFDialogWindow)
			pSFDialogWindow->HitItem(item);
	};
	return item;
};

// ==Folder Selector==
//	   This method selects a folder using the standard file package.  If the user cancels,
//		it returns false, otherwise it returns true and the results in the supplied variables
//		are updated.  Set setDefaults to true to request default values before posing the
//		dialog.

//This method must be in the same segment as TView.Free.  Why?, well nobody knows!
//You get heap damage if these two methods arent in the same segment!!!

#pragma segment MAClose
pascal Boolean TDrafterApplication::PickFolder(short *volRefNum,long *dirId,Str255 *name,Boolean setDefaults)
{
	GrafPtr					savePort;
	TEvtHandler				*oldTarget;
	TypeListHandle			typeList;
	FileFilterProcPtr		fileFilter;
	DlgHookProcPtr			dlgHook;
	ModalFilterProcPtr		filterProc;
	Point					where;
  	ResType					dismissViewId;
	short					dlgID;
	short					numTypes;
	Boolean					didNotFail;

	FailInfo				fi;
	jmp_buf					fb;

	pSFDialogWindow=NULL;
	pSFDialog=NULL;
	pMenusDone=false;

	UpdateAllWindows();				//work around problem in SF

	FailNIL(typeList=(TypeListHandle)NewHandle(0));
	SFGetParms(getDlgID,&dlgID,&where,(Ptr*)&fileFilter,(Ptr*)&dlgHook,(Ptr*)&filterProc,typeList);

//If subclass didnt provide filters, use the default ones.

	if (!filterProc)
		filterProc=&ModalObjectFilter;

	if (!dlgHook)
		dlgHook=&SFObjectFilter;

	numTypes=GetHandleSize((Handle)typeList) / sizeof(OSType);
	if (numTypes)
		LockHandleHigh((Handle)typeList);
	else
		numTypes= -1;

	pApplication->CommitLastCommand();

	if (setDefaults)
	{
		//stuff initial values into SFReply record (if any)
	};

	oldTarget=gTarget;
	pApplication->InvalidateFocus();
	pApplication->SetTarget(pApplication);

	GetPort(&savePort);

	if (!setjmp(fb))	//attempt the main logic
	{
		CatchFailures(&fi,CommonHandler,fb);

		SFPGetFile(where,"\p ",fileFilter,numTypes,(SFTypeList)*typeList,dlgHook,&pReply,
									dlgID,filterProc);
		Success(&fi);
		didNotFail=true;
	}
	else	//handle the caught failure
	{
		didNotFail=false;
//		goto successPlace;

//	IT IS NORMAL to pass the problem up the chain!!
//		Failure(gFailError,gFailMessage);
	};
//successPlace:

	SetPort(savePort);
	typeList=(TypeListHandle)DisposeIfHandle((Handle)typeList);

	pReply.good=false;

	if (didNotFail)
	{
		dismissViewId=pSFDialogWindow->GetDisMissView();

		if (dismissViewId=='CFLD')		// dependent on specific identifier BAD BAD BAD!!!
		{
			*volRefNum= -*((short*)kSFSaveDisk);
			*dirId=*((long*)kCurDirStore);
			CopyStr255(pReply.fName,name);
			pReply.good=true;
		};
	};

	pApplication->CommitLastCommand();
	pApplication->InvalidateFocus();
	pApplication->InvalidateCursorRgn();
	pApplication->SetTarget(oldTarget);

	if (pSFDialogWindow)
		pSFDialogWindow->Free();
	pSFDialogWindow=NULL;

	pApplication->SetupTheMenus();

	if (didNotFail)
		UpdateAllWindows();
	else
		Failure(gFailError,gFailMessage);

	return pReply.good;
};

// ==MacApp Standard File Patch==
//	   This method redirects standard file window choices to the window object.

#pragma segment ADrafter
pascal void TClearDialog::DoChoice(TView *origView, short itsChoice)
{
	if (itsChoice >= 10000)		//Standard File codes +10000
	{
		itsChoice -= 10000;
		if (itsChoice != 100)	//Null events are ignored for now
			pSFDialogWindow->HitItem(itsChoice);
	}
	else if (origView)			//NEVER pass NULL views to inherited!
	{
		inherited::DoChoice(origView,itsChoice);

		if (fDismissed)
			pSFDialogWindow->SetDisMissView(fDismisser);
	};
};

// ==MacApp Standard File Patch==
//	   This method replaces the normal TDialogView.Draw with a null action.

#pragma segment ADrafter
pascal void TClearDialog::Draw(Rect *area)
{
	//If called, do nothing.
};

// ==MacApp Standard File Addition==
//	   This method modifies the views as necessary when the standard file package
//		enables or disables the OPEN/SAVE/OK button.

#pragma segment ADrafter
pascal void TClearDialog::SetOkEnable(Boolean okIsOn)
{
};

// ==Cluster View Tree Dimmer==
//	   This iterator procedure expands a dim state change over its entore view tree.

pascal void TDrafterClusterDimOne(TView *theSubView, void *DoToSubView_StaticLink)
{
	PDimStateData *frameData=(PDimStateData*)DoToSubView_StaticLink;

	if (!Member(theSubView,"\pTDrafterCluster"))
		theSubView->EachSubView(&TDrafterClusterDimOne,DoToSubView_StaticLink);

	theSubView->ViewEnable(!frameData->doDim,kDontRedraw);
	if (Member(theSubView,"\pTControl"))
		((TControl*)theSubView)->DimState(frameData->doDim,kDontRedraw);
};

// ==Cluster View Tree Dimmer==
//	   This method changes the dim state and enable state of a cluster and its
//		view tree.

#pragma segment AInit
pascal void TDrafterCluster::DimState(Boolean state,Boolean redraw)
{
	PDimStateData	frameData;

	frameData.doDim=state;

	EachSubView(&TDrafterClusterDimOne,&frameData);

	ViewEnable(false,kDontRedraw);

	inherited::DimState(state,redraw);
};

// ==Alternate Cluster==
//	   This method draws a cluster view in a different way.  It also clips out subview
//		areas so that they are not drawn twice.

#pragma segment ADrafter
pascal void TDrafterCluster::Draw(Rect *area)
{
	RgnHandle	savedClip;
	Rect		theFrame;
	FontInfo	theFontInfo;
	RGBColor	oldColor;
	TextStyle	aTextStyle;

	savedClip=MakeNewRgn();
	GetClip(savedClip);

	if (SetClipRgn())		//If unable to set the clipping, Focus failed and therefore cant draw.
	{
		PenNormal();
		if (gConfiguration.hasColorQD)
			GetIfColor(&oldColor);

//	Draw the main area.
		ControlArea(&theFrame);
		DrawInterior(area,&theFrame);

		PenSize(fPenSize.h,fPenSize.v);
		FrameRect(&theFrame);

//	Draw the title area.

		if (fDataHandle && Length(*((StringHandle)fDataHandle)))
		{
			aTextStyle=fTextStyle;
			SetPortTextStyle(&aTextStyle);
			GetFontInfo(&theFontInfo);

			theFrame.top+=(theFontInfo.ascent+theFontInfo.descent+3)>>1;
			PenPat(qd.gray);
			MoveTo(theFrame.left,theFrame.top);
			LineTo(theFrame.right,theFrame.top);
		};

//	Draw the title text.

		DrawTitle(area);

		if (gConfiguration.hasColorQD)
			SetIfColor(&oldColor);
	};
	SetClip(savedClip);
	DisposeRgn(savedClip);

	TControl::Draw(area);	//Skips over TCluster.Draw to TControl.Draw (for adornments)
};

// ==Alternate Cluster Drawer==
//	   This method erases the interior of the alternate cluster to the background color.

#pragma segment ADrafter
pascal void TDrafterCluster::DrawInterior(Rect *area,Rect *insideRect)
{
	EraseRect(insideRect);
};

// ==Cluster Title Text Drawer==
//	   This method draws the title of the cluster.

#pragma segment ADrafter
pascal void TDrafterCluster::DrawTitle(Rect *area)
{
	Rect		theFrame;
	FontInfo	theFontInfo;
	TextStyle	aTextStyle;
	TDialogView	*aDialogView;
	short		widthAdjustment;
	Str255		theText;

	PenNormal();
	if (fDataHandle)
	{
		aTextStyle=fTextStyle;
		SetPortTextStyle(&aTextStyle);
		GetFontInfo(&theFontInfo);

		ControlArea(&theFrame);
		theFrame.left+=fPenSize.h;
		theFrame.right-=fPenSize.v;
		theFrame.top++;
		theFrame.bottom=theFrame.top+
				theFontInfo.ascent+theFontInfo.descent+1;

		CopyStr255(*((StringHandle)fDataHandle),theText);
		if (aDialogView=(TDialogView*)GetDialogView())
			aDialogView->ReplaceText(theText);

		if (Length(theText))
		{
			widthAdjustment=(theFrame.right-theFrame.left-StringWidth(theText)-4) / 2;
			theFrame.left+=widthAdjustment;
			theFrame.right-=widthAdjustment;

			MATextBox((Ptr)&theText[1],Length(theText),&theFrame,teJustCenter,kNoAutoWrap,NULL,
					kEraseFirst,kNoSpaceForCaret);
		};
	};
};

// ==Subview Clip Builder==
//	   This iterator procedure removes the areas of subviews from the parent area.

#pragma segment ADrafter
pascal void TDrafterClusterExcSubCls(TView *theSubView, void *DoToSubView_StaticLink)
{
	Rect		itsQDFrame;
	VRect		itsFrame;

	PSubViewClipData *frameData=(PSubViewClipData*)DoToSubView_StaticLink;

	theSubView->GetFrame(&itsFrame);
	VRectToRect(&itsFrame,&itsQDFrame);
	RectRgn(frameData->scratchRgn,&itsQDFrame);
	DiffRgn(frameData->clipRgn,frameData->scratchRgn,frameData->clipRgn);

};

// ==Subview Clip Builder==
//	   This method further clips the drawing region of the alternate cluster to exclude
//		any subviews.  This keeps subview areas from drawing more than once.

#pragma segment ADrafter
pascal Boolean TDrafterCluster::SetClipRgn(void)
{
	PSubViewClipData	frameData;

	if (Focus())
	{
		frameData.scratchRgn=MakeNewRgn();
		frameData.clipRgn=MakeNewRgn();

		GetClip(frameData.clipRgn);

		EachSubView(&TDrafterClusterExcSubCls,&frameData);

		SetClip(frameData.clipRgn);

		DisposeRgn(frameData.clipRgn);
		DisposeRgn(frameData.scratchRgn);

		return true;
	}
	else
		return false;
};

// ==Nonclipping Alternate Cluster==
//	   This method leaves the area for drawing in an alternate cluster unchanged.

#pragma segment ADrafter
pascal Boolean TDrafterNoClipCluster::SetClipRgn(void)
{
	return true;
};

// ==Frame Cluster Drawer==
//	   This method modifies the edges of the cluster area to darken or lighten them
//		resulting in a picture frame effect.

#pragma segment ADrafter
pascal void TDrafterFrameCluster::Draw(Rect *area)
{
	RgnHandle	savedClip;
	Rect		theFrame;
	Rect		workRect;
	FontInfo	theFontInfo;
	RGBColor	oldColor;
	RGBColor	adjustColor;
	TextStyle	aTextStyle;
	RgnHandle	aRgn;

	savedClip=MakeNewRgn();
	GetClip(savedClip);

	if (SetClipRgn())
	{
		PenNormal();
		if (gConfiguration.hasColorQD)
			GetIfColor(&oldColor);
		ControlArea(&theFrame);

		workRect=theFrame;
		workRect.top+=fInset.top;			//The frame width is the control inset
		workRect.left+=fInset.left;
		workRect.bottom-=fInset.bottom;
		workRect.right-=fInset.right;

		PenSize(fPenSize.h,fPenSize.v);

		if (fDataHandle && Length(*((StringHandle)fDataHandle)))	//Allow for title height
		{
		
			aTextStyle=fTextStyle;
			SetPortTextStyle(&aTextStyle);
			GetFontInfo(&theFontInfo);

			workRect.top=theFrame.top+
				theFontInfo.ascent+theFontInfo.descent+2;
		};

		aRgn=MakeNewRgn();
		if (gConfiguration.hasColorQD)
			SetRGBColor(&adjustColor,0x2000,0x2000,0x2000);			//Small change in gray

//top bar
		OpenRgn();
		MoveTo(theFrame.left,theFrame.top);
		LineTo(workRect.left,workRect.top);
		LineTo(workRect.right,workRect.top);
		LineTo(theFrame.right,theFrame.top);
		LineTo(theFrame.left,theFrame.top);
		CloseRgn(aRgn);
		if (gConfiguration.hasColorQD)
		{
			PenMode(subPin);					//Darken
			OpColor(&gRGBBlack);
			SetIfColor(&adjustColor);
		}
		else
			PenPat(qd.gray);
		PaintRgn(aRgn);
//left bar
		OpenRgn();
		MoveTo(theFrame.left,theFrame.top);
		LineTo(workRect.left,workRect.top);
		LineTo(workRect.left,workRect.bottom);
		LineTo(theFrame.left,theFrame.bottom);
		LineTo(theFrame.left,theFrame.top);
		CloseRgn(aRgn);
		if (gConfiguration.hasColorQD)
		{
			PenMode(addPin);					//Lighten
			OpColor(&gRGBWhite);
			SetIfColor(&adjustColor);
		}
		else
			PenPat(qd.ltGray);
		PaintRgn(aRgn);
//bottom bar
		OpenRgn();
		MoveTo(theFrame.left,theFrame.bottom);
		LineTo(workRect.left,workRect.bottom);
		LineTo(workRect.right,workRect.bottom);
		LineTo(theFrame.right,theFrame.bottom);
		LineTo(theFrame.left,theFrame.bottom);
		CloseRgn(aRgn);
		if (gConfiguration.hasColorQD)
		{
			PenMode(subPin);
			OpColor(&gRGBBlack);
			SetIfColor(&adjustColor);
		}
		else
			PenPat(qd.gray);
		PaintRgn(aRgn);
//right bar
		OpenRgn();
		MoveTo(theFrame.right,theFrame.top);
		LineTo(workRect.right,workRect.top);
		LineTo(workRect.right,workRect.bottom);
		LineTo(theFrame.right,theFrame.bottom);
		LineTo(theFrame.right,theFrame.top);
		CloseRgn(aRgn);
		if (gConfiguration.hasColorQD)
		{
			PenMode(addPin);
			OpColor(&gRGBWhite);
			SetIfColor(&adjustColor);
		}
		else
			PenPat(qd.ltGray);
		PaintRgn(aRgn);

		DisposeRgn(aRgn);

		InsetRect(&workRect,1,1);
		DrawInterior(area,&workRect);

		DrawTitle(area);

		if (gConfiguration.hasColorQD)
			SetIfColor(&oldColor);
	};
	SetClip(savedClip);
	DisposeRgn(savedClip);

	TControl::Draw(area);	//Skips over TCluster.Draw to TControl.Draw
};

// ==Frame Cluster Interior==
//	   This method leaves the interior of the cluster alone.

#pragma segment ADrafter
pascal void TDrafterFrameCluster::DrawInterior(Rect *area,Rect *insideRect)
{
};

// ==Frmae Cluster Titler==
//	   This method draws the title in a frame cluster.

#pragma segment ADrafter
pascal void TDrafterFrameCluster::DrawTitle(Rect *area)
{
	Rect		theFrame;
	FontInfo	theFontInfo;
	TextStyle	aTextStyle;
	TDialogView	*aDialogView;
	short		widthAdjustment;
	Str255		theText;

	PenNormal();
	if (fDataHandle)
	{
		aTextStyle=fTextStyle;
		SetPortTextStyle(&aTextStyle);
		GetFontInfo(&theFontInfo);

		ControlArea(&theFrame);
		theFrame.left+=fPenSize.h;
		theFrame.right-=fPenSize.v;
		theFrame.bottom=theFrame.top+
				theFontInfo.ascent+theFontInfo.descent;

		CopyStr255(*((StringHandle)fDataHandle),theText);
		if (aDialogView=(TDialogView*)GetDialogView())
			aDialogView->ReplaceText(theText);

		if (Length(theText))
		{
			widthAdjustment=(theFrame.right-theFrame.left-StringWidth(theText)-4) / 2;
			theFrame.left+=widthAdjustment;
			theFrame.right-=widthAdjustment;

			MATextBox((Ptr)&theText[1],Length(theText),&theFrame,teJustCenter,kNoAutoWrap,NULL,
					kEraseFirst,kNoSpaceForCaret);

			theFrame.bottom++;
			FrameRect(&theFrame);
		};
	};
};

// ==Brass Plate Cluster Drawer==
//	   This method fills the cluster area with a brass color and then allows the
//		the frame to be drawn by its inherited draw.

#pragma segment ADrafter
pascal void TDrafterBrassCluster::Draw(Rect *area)
{
	Rect		theFrame;
	RGBColor	oldColor;
	RGBColor	brassColor;

	PenNormal();
	ControlArea(&theFrame);

	if (gConfiguration.hasColorQD)
	{
		GetIfColor(&oldColor);
		GetRGBIndexed(1001,1,&brassColor);		//Defined in this file (not toolbox!)
		SetIfColor(&brassColor);
		PenPat(qd.black);
		PaintRect(&theFrame);
		SetIfColor(&oldColor);
	}
	else
	{
		PenPat(qd.ltGray);
		PaintRect(&theFrame);
	};

	inherited::Draw(area);
};

// ==Nonclipping Hollow Cluster==
//	   This method leaves the area for drawing in a hollow cluster unchanged.

#pragma segment ADrafter
pascal Boolean TDrafterHollowCluster::SetClipRgn(void)
{
	return true;
};

// ==Hollow Cluster Drawer==
//	   This method does no drawing.  Hollow cluster have no screen image, they just
//		coordinate subview behavior as a group.

#pragma segment ADrafter
pascal void TDrafterClearCluster::Draw(Rect *area)
{
};

// ==Memory Button Openner==
//	   This method modifies the appearance of buttons that have been clicked at
//		any previous time.

#pragma segment ADrafter
pascal void TDrafterButton::Open(void)
{
	TDrafterDoc		*doc;
	TDialogView		*dialog;
	RGBColor		hasSeenColor;
	Str255			str;

	if (dialog=(TDialogView*)GetDialogView())
	{
		if ((fIdentifier!=vMainScreen) && Member(dialog,"\pTDrafterView"))
		{
			if (doc=((TDrafterView*)dialog)->GetDrafterDoc())
			{
				BlockMove((Ptr)&fIdentifier,(Ptr)&str[1],4);
				str[0]=4;

				if ((fIdentifier!=vGenerateOutput) && doc->HaveSeen(str))
				{
					fTextStyle.tsFace=fTextStyle.tsFace | (italic+condense);
					GetText(str);
/**/			PConcat(str,str,"\p ");		//Add a checkmark to name [NON INTERNATIONAL]
					SetText(str,kDontRedraw);

					if (gConfiguration.hasColorQD)	//Change to use GETRGBIndexed
					{
						hasSeenColor.red=0;
						hasSeenColor.green=0xD000;
						hasSeenColor.blue=0;
		
						InstallColor(&hasSeenColor,kDontRedraw);
					};
				};
			};
		};
	};

	inherited::Open();
};

// ==Key Event Monitor==
//	   This method detects data key events within dialogs for synchronizing multiple views.

#pragma segment ADrafter
pascal TCommand *TDrafterDialogTEView::DoKeyCommand(short ch, short aKeyCode, EventInfo *info)
{
	TCommand			*result;
	TDialogView			*dialog;
	TEditText			*te;

	result=inherited::DoKeyCommand(ch,aKeyCode,info);

	if ((ch!=chReturn) && (ch!=chEnter) && (ch!=chEscape))	//These FREE me!!!
		if (dialog=(TDialogView*)GetDialogView())
			if (te=dialog->fCurrentEditText)
				dialog->DoChoice(te,te->fDefChoice);

	return result;
};

// ==Key Event Monitor==
//	   This method detects data key events within dialogs for synchronizing multiple views.

#pragma segment ADrafter
pascal TTETypingCommand *TDrafterDialogTEView::DoMakeTypingCommand(short ch)
{
	TTETypingCommand	*result;
	TDialogView			*dialog;
	TEditText			*te;

	result=inherited::DoMakeTypingCommand(ch);

	if (dialog=(TDialogView*)GetDialogView())
		if (te=dialog->fCurrentEditText)
			dialog->DoChoice(te,te->fDefChoice);

	return result;
};

// ==Key Event Monitor==
//	   This method detects data key events within dialogs for synchronizing multiple views.

#pragma segment ADrafter
pascal TCommand *TDrafterDialogTEView::DoMenuCommand(CmdNumber aCmdNumber)
{
	TCommand			*result;
	TDialogView			*dialog;
	TEditText			*te;

	result=inherited::DoMenuCommand(aCmdNumber);

	if (dialog=(TDialogView*)GetDialogView())
		if (te=dialog->fCurrentEditText)
			dialog->DoChoice(te,te->fDefChoice);

	return result;
};

// ==Prime Choice Coordinator For HyperCard Like Window Cards==
//	   This method detects button choices and either switches screens or
//		generates output.  All other activities just support standard card
//		behaviors.

#pragma segment ADrafter
pascal void TDrafterView::DoChoice(TView *origView,short itsChoice)
{
	TDrafterWindow	*window;
	TDrafterDoc		*doc;
	long			value;
	short			nextDialog;
	Str255			str;

	nextDialog=0;

	BlockMove((Ptr)&origView->fIdentifier,(Ptr)&str[1],4);
	str[0]=4;

	switch (itsChoice)
	{
		case mListItemHit:
		case mListScrollBarHit:
		case mStaticTextHit:
		case mHScrollBarHit:
		case mVScrollBarHit:
			break;
		case mEditTextHit:
			if (origView->IsShown())	//Since TDialogView doesnt check soon enough
				inherited::DoChoice(origView,itsChoice);
			break;
		case mEditTabKey:
		case mEditEnterKey:
			inherited::DoChoice(origView,itsChoice);	//So text views coordinate quicker
			break;
		case mEditReturnKey:
		case mOKHit:
		case mDefaultKey:
		case mCancelHit:
		case mCancelKey:
			nextDialog=vMainScreenValue;	//Default is HOME screen, id=1001
			break;
		case mButtonHit:			//Otherwise view identifier is number of next screen.
			if (value=NumOf(str))
				nextDialog=value;
			if (doc=GetDrafterDoc())
				doc->DoChoice(origView,itsChoice);
			break;
		case mCheckBoxHit:
		case mControlHit:
		case mIconHit:
		case mPictureHit:
		case mPopupHit:
		case mRadioHit:
		case mPatternHit:
			break;
		case mClusterHit:
		default:
			inherited::DoChoice(origView,itsChoice);
			break;
	};
	if (nextDialog)
	{
		if (nextDialog==vGenOutputValue)		//Virtual screen=>output generation, not a view
			GetDrafterDoc()->WriteMacAppWorld();
		else if (window=(TDrafterWindow*)GetWindow())
			window->SwitchDialog(nextDialog);
	};
};

// ==Document Accessor==
//	   This method returns the views document with the correct class type.

#pragma segment ADrafter
pascal TDrafterDoc *TDrafterView::GetDrafterDoc(void)
{
	return (TDrafterDoc*)fDocument;
};

// ==Dialog TEView Maker==
//	   This method creates and returns a suitable DialogTEView for card windows.

#pragma segment ADrafter
pascal TDialogTEView *TDrafterView::MakeTEView(void)
{
	TDrafterDialogTEView	*aDialogTEView;

	FailNIL(aDialogTEView=new TDrafterDialogTEView);
	aDialogTEView->IDialogTEView(NULL,NULL,&gZeroVPt,&gZeroVPt,
						sizeRelSuperView,sizeVariable,
								&gZeroRect,&gSystemStyle,teJustSystem,
								kWithoutStyle,false);

	aDialogTEView->fMinAhead=1;

	return aDialogTEView;
};

// ==Main Card Openner==
//	   This method disables any view with the vDocs identifier when documents are
//		not used in the application being drafted.

#pragma segment ADrafter
pascal void TDrafterView::Open(void)
{
	TButton			*button;
	TDrafterDoc		*doc;

	doc=GetDrafterDoc();

	if (doc && (button=(TButton*)FindSubView(vDocs)))
	{
		button->ViewEnable(doc->fHasDocs,kDontRedraw);
		button->DimState(!doc->fHasDocs,kDontRedraw);
	};

	inherited::Open();
};

// ==General Application Information Collector==
//	   This method saves general application information.

#pragma segment ADrafter
pascal void TDlgGeneral::Close(void)
{
	TDrafterDoc		*doc;
	TEditText		*te;
	TCheckBox		*cb;

	doc=GetDrafterDoc();

	if (te=(TEditText*)FindSubView(vCopyRight))
		te->GetText(doc->fCopyRight);

	if (te=(TEditText*)FindSubView(vAuthors))
		te->GetText(doc->fAuthors);

	if (te=(TEditText*)FindSubView(vAppName))
		te->GetText(doc->fAppName);

	if (cb=(TCheckBox*)FindSubView(vHasDocs))
		doc->fHasDocs=cb->IsOn();

	if (cb=(TCheckBox*)FindSubView(vHasPrefs))
		doc->fHasPrefs=cb->IsOn();

	if (cb=(TCheckBox*)FindSubView(vFreeWinds))
		doc->fFreeWindows=cb->IsOn();

	if (cb=(TCheckBox*)FindSubView(vPrints))
		doc->fPrints=cb->IsOn();

	if (cb=(TCheckBox*)FindSubView(vSplashes))
		doc->fSplash=cb->IsOn();

	inherited::Close();
};

// ==Preference Access Controller==
//	   This method enables or disables access to the preferences screen when
//		the user changes the status of the feature.

#pragma segment ADrafter
pascal void TDlgGeneral::DoChoice(TView *origView,short itsChoice)
{
	TButton		*button;
	Boolean		boxChecked;

	if (origView->fIdentifier==vHasPrefs)
	{
		boxChecked=((TCheckBox*)origView)->IsOn();
		if (button=(TButton*)FindSubView(vPrefs))
		{
			button->ViewEnable(boxChecked,kDontRedraw);
			button->DimState(!boxChecked,kRedraw);
		};
	};
	
	inherited::DoChoice(origView,itsChoice);
};

// ==General Application Information Stuffer==
//	   This method stuffs the current values for various application
//		information values into the screen views.

#pragma segment ADrafter
pascal void TDlgGeneral::Open(void)
{
	TDrafterDoc		*doc;
	TEditText		*te;
	TCheckBox		*cb;
	TButton			*button;

	inherited::Open();

	doc=GetDrafterDoc();

	if (te=(TEditText*)FindSubView(vAppName))	//vAppName=vTEView, so 1st selected
	{
		te->SetText(doc->fAppName,kDontRedraw);
		te->SetSelection(0,255,kDontRedraw);	//Needed on 1st selected
	};

	if (te=(TEditText*)FindSubView(vAuthors))
		te->SetText(doc->fAuthors,kDontRedraw);

	if (te=(TEditText*)FindSubView(vCopyRight))
		te->SetText(doc->fCopyRight,kDontRedraw);

	if (cb=(TCheckBox*)FindSubView(vHasDocs))
		cb->SetState(doc->fHasDocs,kDontRedraw);

	if (cb=(TCheckBox*)FindSubView(vHasPrefs))
	{
		cb->SetState(doc->fHasPrefs,kDontRedraw);
		if (button=(TButton*)FindSubView(vPrefs))
		{
			button->ViewEnable(doc->fHasPrefs,kDontRedraw);
			button->DimState(!doc->fHasPrefs,kDontRedraw);
		};
	};

	if (cb=(TCheckBox*)FindSubView(vFreeWinds))
		cb->SetState(doc->fFreeWindows,kDontRedraw);

	if (cb=(TCheckBox*)FindSubView(vPrints))
		cb->SetState(doc->fPrints,kDontRedraw);

	if (cb=(TCheckBox*)FindSubView(vSplashes))
		cb->SetState(doc->fSplash,kDontRedraw);

};

// ==Finder Information Collector==
//	   This method saves the values from the Finder information screen views.

#pragma segment ADrafter
pascal void TDlgFinderInfo::Close(void)
{
	TDrafterDoc		*doc;
	TEditText		*te;
	TRadio			*rb;
	Str255			str;

	doc=GetDrafterDoc();

	if (te=(TEditText*)FindSubView(vAppSig))
	{
		te->GetText(str);
		BlockMove((Ptr)&str[1],(Ptr)&doc->fSignature,sizeof(OSType));
	};

	if (doc->fHasDocs)
		if (te=(TEditText*)FindSubView(vDocSig))
		{
			te->GetText(str);
			BlockMove((Ptr)&str[1],(Ptr)&doc->fDocType,sizeof(OSType));
		};

	if (te=(TEditText*)FindSubView(vMinMem))
	{
		te->GetText(str);
		doc->fMinMem=NumOf(str);
	};

	if (te=(TEditText*)FindSubView(vDefaultMem))
	{
		te->GetText(str);
		doc->fStandardMem=NumOf(str);
	};

	if (rb=(TRadio*)FindSubView(vVersDVL))
		if (rb->IsOn())
			doc->fVersType=0;
	if (rb=(TRadio*)FindSubView(vVersAlpha))
		if (rb->IsOn())
			doc->fVersType=1;
	if (rb=(TRadio*)FindSubView(vVersBeta))
		if (rb->IsOn())
			doc->fVersType=2;
	if (rb=(TRadio*)FindSubView(vVersRls))
		if (rb->IsOn())
			doc->fVersType=3;

	if (te=(TEditText*)FindSubView(vReleaseNo))
	{
		te->GetText(str);
		doc->fVersRelease=NumOf(str);
	};

	if (te=(TEditText*)FindSubView(vVersNo))
	{
		te->GetText(str);
		doc->fVersVersion=NumOf(str);
	};

	if (te=(TEditText*)FindSubView(vBuildNo))
	{
		te->GetText(str);
		doc->fVersBuild=NumOf(str);
	};

	inherited::Close();
};

// ==Finder Information Screen Controller==
//	   This method coordinates and synchronizes the screen views for
//		related elements of information.

pascal void TDlgFinderInfo::DoChoice(TView *origView, short itsChoice)
{
	TEditText		*te;
	long			l;
	short			i;
	Str255			str;
	TDrafterDoc		*doc;

	doc=GetDrafterDoc();

	switch (itsChoice)
	{
		case mRadioHit:
			if (origView->fIdentifier==vVersRls)
			{
				if (te=(TEditText*)FindSubView(vBuildNo))
				{
					if (te==fCurrentEditText)
						Tab(true);
					te->ViewEnable(false,kDontRedraw);
					te->DimState(true,kRedraw);
				};
			}
			else if ((origView->fIdentifier==vVersDVL) ||
					 (origView->fIdentifier==vVersAlpha) ||
					 (origView->fIdentifier==vVersBeta))
			{
				if (te=(TEditText*)FindSubView(vBuildNo))
				{
					te->ViewEnable(true,kDontRedraw);
					te->DimState(false,kRedraw);
				};
			};
			inherited::DoChoice(origView,itsChoice);
			break;
		case mEditTextHit:
			if (origView->fIdentifier==vAppSig)
			{
				((TEditText*)origView)->GetText(str);
				if (te=(TEditText*)FindSubView(vAppSigHex))
				{
					for (i=Length(str)+1;i <= sizeof(OSType);str[i++]=0);

					BlockMove((Ptr)&str[1],(Ptr)&l,sizeof(OSType));
					LIntToHex(l,str,8);
					te->SetText(str,kRedraw);
				};
			}
			else if (origView->fIdentifier==vAppSigHex)
			{
				((TEditText*)origView)->GetText(str);
				if (te=(TEditText*)FindSubView(vAppSig))
				{
					l=NumOfHex(str);
					BlockMove((Ptr)&l,(Ptr)&str[1],sizeof(OSType));
					str[0]=sizeof(OSType);
					te->SetText(str,kRedraw);
				};
			}
			else if (doc->fHasDocs)
			{
				if (origView->fIdentifier==vDocSig)
				{
					((TEditText*)origView)->GetText(str);
					if (te=(TEditText*)FindSubView(vDocSigHex))
					{
						for (i=Length(str)+1;i <= sizeof(OSType);str[i++]=0);
	
						BlockMove((Ptr)&str[1],(Ptr)&l,sizeof(OSType));
						LIntToHex(l,str,8);
						te->SetText(str,kRedraw);
					};
				}
				else if (origView->fIdentifier==vDocSigHex)
				{
					((TEditText*)origView)->GetText(str);
					if (te=(TEditText*)FindSubView(vDocSig))
					{
						l=NumOfHex(str);
						BlockMove((Ptr)&l,(Ptr)&str[1],sizeof(OSType));
						str[0]=sizeof(OSType);
						te->SetText(str,kRedraw);
					};
				};
			};
			inherited::DoChoice(origView,itsChoice);
			break;
		default:
			inherited::DoChoice(origView,itsChoice);
			break;
	};
};

// ==Finder Information Stuffer==
//	   This method installs the current Finder information values into
//		the screen views.

#pragma segment ADrafter
pascal void TDlgFinderInfo::Open(void)
{
	TDrafterDoc		*doc;
	TCluster		*cluster;
	TEditText		*te;
	TRadio			*rb;
	TIcon			*icon;
	Str255			str;

	inherited::Open();

	doc=GetDrafterDoc();

	if (te=(TEditText*)FindSubView(vAppSig))	//vAppSig=vTEView, so 1st selected
	{
		BlockMove((Ptr)&doc->fSignature,(Ptr)&str[1],sizeof(OSType));
		str[0]=sizeof(OSType);
		te->SetText(str,kDontRedraw);
		te->SetSelection(0,4,kDontRedraw);		//Needed on 1st selected
	};
	if (te=(TEditText*)FindSubView(vAppSigHex))
	{
		LIntToHex((long)doc->fSignature,str,8);
		te->SetText(str,kDontRedraw);
	};

	if (doc->fHasDocs)
	{
		if (te=(TEditText*)FindSubView(vDocSig))
		{
			BlockMove((Ptr)&doc->fDocType,(Ptr)&str[1],sizeof(OSType));
			str[0]=sizeof(OSType);
			te->SetText(str,kDontRedraw);
		};

		if (te=(TEditText*)FindSubView(vDocSigHex))
		{
			LIntToHex((long)doc->fDocType,str,8);
			te->SetText(str,kDontRedraw);
		};
	}
	else
	{
		if (cluster=(TCluster*)FindSubView(vDocBndl))
		{
			cluster->Show(false,kDontRedraw);
			cluster->DimState(true,kDontRedraw);
		};
		if (icon=(TIcon*)FindSubView(vDocIcon))
		{
			icon->ViewEnable(false,kDontRedraw);
			icon->DimState(true,kDontRedraw);
		};
	};

	if (te=(TEditText*)FindSubView(vDefaultMem))
	{
		NumToString(doc->fStandardMem,str);
		te->SetText(str,kDontRedraw);
	};

	if (te=(TEditText*)FindSubView(vMinMem))
	{
		NumToString(doc->fMinMem,str);
		te->SetText(str,kDontRedraw);
	};

	if (rb=(TRadio*)FindSubView(vVersDVL))
		rb->SetState(doc->fVersType==0,kDontRedraw);
	if (rb=(TRadio*)FindSubView(vVersAlpha))
		rb->SetState(doc->fVersType==1,kDontRedraw);
	if (rb=(TRadio*)FindSubView(vVersBeta))
		rb->SetState(doc->fVersType==2,kDontRedraw);
	if (rb=(TRadio*)FindSubView(vVersRls))
		rb->SetState(doc->fVersType==3,kDontRedraw);

	if (te=(TEditText*)FindSubView(vReleaseNo))
	{
		NumToString(doc->fVersRelease,str);
		te->SetText(str,kDontRedraw);
	};

	if (te=(TEditText*)FindSubView(vVersNo))
	{
		NumToString(doc->fVersVersion,str);
		te->SetText(str,kDontRedraw);
	};

	if (te=(TEditText*)FindSubView(vBuildNo))
	{
		NumToString(doc->fVersBuild,str);
		te->SetText(str,kDontRedraw);
	};

};

// ==Preference Information Collector==
//	   This method saves the current preference values from the
//		preference screen views.

#pragma segment ADrafter
pascal void TDlgPrefs::Close(void)
{
	TDrafterDoc		*doc;
	TEditText		*te;
	TRadio			*rb;

	doc=GetDrafterDoc();
	doc->fPrefWithApp=false;
	doc->fPrefInSystem6=false;
	doc->fPrefInSystem7=false;

	if (te=(TEditText*)FindSubView(vPrefFileName))
		te->GetText(doc->fPrefFileName);

	if (rb=(TRadio*)FindSubView(vSys7PrefLoc))
		doc->fPrefInSystem7=rb->IsOn();
	if (rb=(TRadio*)FindSubView(vSys6PrefLoc))
		doc->fPrefInSystem6=rb->IsOn();
	if (rb=(TRadio*)FindSubView(vAppPrefLoc))
		doc->fPrefWithApp=rb->IsOn();

	inherited::Close();
};

// ==Preference Information Stuffer==
//	   This method sets the values in the screen views for
//		the preferences screen.

#pragma segment ADrafter
pascal void TDlgPrefs::Open(void)
{
	TDrafterDoc		*doc;
	TEditText		*te;
	TRadio			*rb;

	inherited::Open();

	doc=GetDrafterDoc();

	if (te=(TEditText*)FindSubView(vPrefFileName))	//vPrefFileName=vTEView, so 1st selected
	{
		te->SetText(doc->fPrefFileName,kDontRedraw);
		te->SetSelection(0,255,kDontRedraw);		//Needed on 1st selected
	};

	if (rb=(TRadio*)FindSubView(vSys7PrefLoc))
		rb->SetState(doc->fPrefInSystem7,kDontRedraw);
	if (rb=(TRadio*)FindSubView(vSys6PrefLoc))
		rb->SetState(doc->fPrefInSystem6,kDontRedraw);
	if (rb=(TRadio*)FindSubView(vAppPrefLoc))
		rb->SetState(doc->fPrefWithApp,kDontRedraw);
};

// ==Document Information Collector==
//	   This method saves document characteristics from the screen views.

#pragma segment ADrafter
pascal void TDlgDocs::Close(void)
{
	TDrafterDoc		*doc;
	TEditText		*te;
	TCheckBox		*cb;
	Str255			str;

	doc=GetDrafterDoc();

	if (te=(TEditText*)FindSubView(vDocSigAlt))
	{
		te->GetText(str);
		BlockMove((Ptr)&str[1],(Ptr)&doc->fDocType,sizeof(OSType));
	};

	doc->fDocDataStaysOpen=false;
	doc->fDocRsrcStaysOpen=false;

	if (cb=(TCheckBox*)FindSubView(vDocReads))
	{
		if (cb=(TCheckBox*)FindSubView(vReadData))
			doc->fDocReadData=cb->IsOn();
		if (cb=(TCheckBox*)FindSubView(vReadRsrc))
			doc->fDocReadRsrc=cb->IsOn();
		if (cb=(TCheckBox*)FindSubView(vRDataOpen))
			doc->fDocDataStaysOpen=cb->IsOn();
		if (cb=(TCheckBox*)FindSubView(vRRsrcOpen))
			doc->fDocRsrcStaysOpen=cb->IsOn();
	}
	else
	{
		doc->fDocReadData=false;
		doc->fDocReadRsrc=false;
	};

	if (cb=(TCheckBox*)FindSubView(vDocWrites))
	{
		if (cb=(TCheckBox*)FindSubView(vWriteData))
			doc->fDocWriteData=cb->IsOn();
		if (cb=(TCheckBox*)FindSubView(vWriteRsrc))
			doc->fDocWriteRsrc=cb->IsOn();
		if (cb=(TCheckBox*)FindSubView(vWDataOpen))
			doc->fDocDataStaysOpen=doc->fDocDataStaysOpen||cb->IsOn();
		if (cb=(TCheckBox*)FindSubView(vWRsrcOpen))
			doc->fDocRsrcStaysOpen=doc->fDocRsrcStaysOpen||cb->IsOn();

		if (cb=(TCheckBox*)FindSubView(vDocReverts))
			doc->fDocReverts=cb->IsOn();
	}
	else
	{
		doc->fDocWriteData=false;
		doc->fDocWriteRsrc=false;
		doc->fDocReverts=false;
	};

	if (cb=(TCheckBox*)FindSubView(vUsesLists))
		doc->fLists=cb->IsOn();
	if (cb=(TCheckBox*)FindSubView(vUsesAssoc))
		doc->fAssociation=cb->IsOn();

	inherited::Close();
};

// ==Document Information Screen Coordinator==
//	   This method coordinates and synchronizes the document information
//		screens views.

#pragma segment ADrafter
pascal void TDlgDocs::DoChoice(TView *origView, short itsChoice)
{
	TEditText		*te;
	TCheckBox		*cb;
	TCluster		*cluster;
	long			l;
	short			i;
	Str255			str;

	switch (itsChoice)
	{
		case mCheckBoxHit:
			if (origView->fIdentifier==vDocReads)
			{
				if (cluster=(TCluster*)FindSubView(vReadCluster))
					cluster->DimState(!((TCheckBox*)origView)->IsOn(),kRedraw);
			}
			else if (origView->fIdentifier==vDocWrites)
			{
				if (cluster=(TCluster*)FindSubView(vWriteCluster))
					cluster->DimState(!((TCheckBox*)origView)->IsOn(),kRedraw);
			}
			else if (origView->fIdentifier==vWDataOpen)
			{
				if (cb=(TCheckBox*)FindSubView(vRDataOpen))
					cb->SetState(((TCheckBox*)origView)->IsOn(),kRedraw);
			}
			else if (origView->fIdentifier==vRDataOpen)
			{
				if (cb=(TCheckBox*)FindSubView(vWDataOpen))
					cb->SetState(((TCheckBox*)origView)->IsOn(),kRedraw);
			}
			else if (origView->fIdentifier==vWRsrcOpen)
			{
				if (cb=(TCheckBox*)FindSubView(vRRsrcOpen))
					cb->SetState(((TCheckBox*)origView)->IsOn(),kRedraw);
			}
			else if (origView->fIdentifier==vRRsrcOpen)
			{
				if (cb=(TCheckBox*)FindSubView(vWRsrcOpen))
					cb->SetState(((TCheckBox*)origView)->IsOn(),kRedraw);
			};
			break;
		case mEditTextHit:
			if (origView->fIdentifier==vDocSigAlt)
			{
				((TEditText*)origView)->GetText(str);
				if (te=(TEditText*)FindSubView(vDocSigHex))
				{
					for (i=Length(str)+1;i <= sizeof(OSType);str[i++]=0);
					BlockMove((Ptr)&str[1],(Ptr)&l,sizeof(OSType));
					LIntToHex(l,str,8);
					te->SetText(str,kRedraw);
				};
			}
			else if (origView->fIdentifier==vDocSigHex)
			{
				((TEditText*)origView)->GetText(str);
				if (te=(TEditText*)FindSubView(vDocSigAlt))
				{
					l=NumOfHex(str);
					BlockMove((Ptr)&l,(Ptr)&str[1],sizeof(OSType));
					str[0]=sizeof(OSType);
					te->SetText(str,kRedraw);
				};
			};
			inherited::DoChoice(origView,itsChoice);
		default:
			break;
	};
	inherited::DoChoice(origView,itsChoice);
};

// ==Document Information Stuffer==
//	   This method installs the current values for document information
//		into the screen views.

#pragma segment ADrafter
pascal void TDlgDocs::Open(void)
{
	TDrafterDoc		*doc;
	TEditText		*te;
	TCheckBox		*cb;
	TCluster		*cluster;
	Boolean			doesDoIt;
	Str255			str;

	inherited::Open();

	doc=GetDrafterDoc();

	if (te=(TEditText*)FindSubView(vDocSigAlt))	//vDocSigAlt=vTEView, so 1st selected
	{
		BlockMove((Ptr)&doc->fDocType,(Ptr)&str[1],sizeof(OSType));
		str[0]=sizeof(OSType);
		te->SetText(str,kDontRedraw);
		te->SetSelection(0,4,kDontRedraw);		//Needed on 1st selected
	};
	if (te=(TEditText*)FindSubView(vDocSigHex))
	{
		LIntToHex((long)doc->fDocType,str,8);
		te->SetText(str,kDontRedraw);
	};

	if (cb=(TCheckBox*)FindSubView(vReadData))
		cb->SetState(doc->fDocReadData,kDontRedraw);
	if (cb=(TCheckBox*)FindSubView(vReadRsrc))
		cb->SetState(doc->fDocReadRsrc,kDontRedraw);
	if (cb=(TCheckBox*)FindSubView(vRDataOpen))
		cb->SetState(doc->fDocDataStaysOpen,kDontRedraw);
	if (cb=(TCheckBox*)FindSubView(vRRsrcOpen))
		cb->SetState(doc->fDocRsrcStaysOpen,kDontRedraw);

	if (cb=(TCheckBox*)FindSubView(vDocReads))
	{
		doesDoIt=doc->fDocReadData || doc->fDocReadRsrc;
		cb->SetState(doesDoIt,kDontRedraw);
		if (cluster=(TCluster*)FindSubView(vReadCluster))
			cluster->DimState(!doesDoIt,kDontRedraw);
	};

	if (cb=(TCheckBox*)FindSubView(vWriteData))
		cb->SetState(doc->fDocWriteData,kDontRedraw);
	if (cb=(TCheckBox*)FindSubView(vWriteRsrc))
		cb->SetState(doc->fDocWriteRsrc,kDontRedraw);
	if (cb=(TCheckBox*)FindSubView(vWDataOpen))
		cb->SetState(doc->fDocDataStaysOpen,kDontRedraw);
	if (cb=(TCheckBox*)FindSubView(vWRsrcOpen))
		cb->SetState(doc->fDocRsrcStaysOpen,kDontRedraw);
	if (cb=(TCheckBox*)FindSubView(vDocReverts))
		cb->SetState(doc->fDocReverts,kDontRedraw);

	if (cb=(TCheckBox*)FindSubView(vDocWrites))
	{
		doesDoIt=doc->fDocWriteData || doc->fDocWriteRsrc;
		cb->SetState(doesDoIt,kDontRedraw);
		if (cluster=(TCluster*)FindSubView(vWriteCluster))
			cluster->DimState(!doesDoIt,kDontRedraw);
	};

	if (cb=(TCheckBox*)FindSubView(vUsesLists))
		cb->SetState(doc->fLists,kDontRedraw);
	if (cb=(TCheckBox*)FindSubView(vUsesAssoc))
		cb->SetState(doc->fAssociation,kDontRedraw);
};

// ==View Information Collector==
//	   This method saves the view related information from
//		the view options screen.

#pragma segment ADrafter
pascal void TDlgViews::Close(void)
{
	TDrafterDoc		*doc;
	TCheckBox		*cb;

	doc=GetDrafterDoc();

	if (cb=(TCheckBox*)FindSubView(vViewsRefViews))
		doc->fViewsRefViews=cb->IsOn();

	if (cb=(TCheckBox*)FindSubView(vViewsArePrefs))
		doc->fViewsSaveInPref=cb->IsOn();

	if (cb=(TCheckBox*)FindSubView(vSepPrintViews))
		doc->fSeparatePrintView=cb->IsOn();

	if (cb=(TCheckBox*)FindSubView(vUseGridView))
		doc->fGridView=cb->IsOn();

	if (cb=(TCheckBox*)FindSubView(vUseTEView))
		doc->fTEView=cb->IsOn();

	if (cb=(TCheckBox*)FindSubView(vUseDialogView))
		doc->fDialogView=cb->IsOn();

	inherited::Close();
};

// ==View Information Stuffer==
//	   This method installs the current view information into
//		the view screens views.

#pragma segment ADrafter
pascal void TDlgViews::Open(void)
{
	TDrafterDoc		*doc;
	TCheckBox		*cb;

	inherited::Open();

	doc=GetDrafterDoc();

	if (cb=(TCheckBox*)FindSubView(vViewsRefViews))
		cb->SetState(doc->fViewsRefViews,kDontRedraw);

	if (cb=(TCheckBox*)FindSubView(vViewsArePrefs))
		cb->SetState(doc->fViewsSaveInPref,kDontRedraw);

	if (cb=(TCheckBox*)FindSubView(vSepPrintViews))
		cb->SetState(doc->fSeparatePrintView,kDontRedraw);

	if (cb=(TCheckBox*)FindSubView(vUseGridView))
		cb->SetState(doc->fGridView,kDontRedraw);
	if (cb=(TCheckBox*)FindSubView(vUseTEView))
		cb->SetState(doc->fTEView,kDontRedraw);
	if (cb=(TCheckBox*)FindSubView(vUseDialogView))
		cb->SetState(doc->fDialogView,kDontRedraw);
};

// ==Build Options Collector==
//	   This method saves the chosen output file set build choices.

#pragma segment ADrafter
pascal void TDlgBuild::Close(void)
{
	TDrafterDoc		*doc;
	TEditText		*te;
	TRadio			*rb;

	doc=GetDrafterDoc();

	if (te=(TEditText*)FindSubView(vMainName))
		te->GetText(doc->fPGMAppName);

	if (rb=(TRadio*)FindSubView(vOneSourceFile))
		if (rb->IsOn())
			doc->fOneFile=true;
	if (rb=(TRadio*)FindSubView(vStdSourceFiles))
		if (rb->IsOn())
		{
			doc->fOneFile=false;
			doc->fUseIncludes=true;
		};
	if (rb=(TRadio*)FindSubView(vStdNoInclSrcFil))
		if (rb->IsOn())
		{
			doc->fOneFile=false;
			doc->fUseIncludes=false;
		};

	if (rb=(TRadio*)FindSubView(vPascal))
		if (rb->IsOn())
			doc->fPascal=true;
	if (rb=(TRadio*)FindSubView(vCPlusPlus))
		if (rb->IsOn())
			doc->fPascal=false;

	inherited::Close();
};

// ==Build Options Controller==
//	   This method synchronizes and control the related views
//		of the build options screen.

#pragma segment ADrafter
pascal void TDlgBuild::DoChoice(TView *origView, short itsChoice)
{
	TRadio			*rb;
	TCluster		*cluster;

	switch (itsChoice)
	{
		case mRadioHit:
			if (origView->fIdentifier==vCPlusPlus)
			{
				if (rb=(TRadio*)FindSubView(vOneSourceFile))
					rb->SetState(false,kRedraw);
				if (rb=(TRadio*)FindSubView(vStdSourceFiles))
					rb->SetState(true,kRedraw);
				if (rb=(TRadio*)FindSubView(vStdNoInclSrcFil))
					rb->SetState(false,kRedraw);
				if (cluster=(TCluster*)FindSubView(vFileSet))
					cluster->DimState(true,kRedraw);
			}
			else if (origView->fIdentifier==vPascal)
				if (cluster=(TCluster*)FindSubView(vFileSet))
					cluster->DimState(false,kRedraw);
			break;
		default:
			break;
	};
	inherited::DoChoice(origView,itsChoice);
};

// ==Build Information Stuffer==
//	   This method installs the current build choices into the
//		file set build screens views.

#pragma segment ADrafter
pascal void TDlgBuild::Open(void)
{
	TDrafterDoc		*doc;
	TEditText		*te;
	TRadio			*rb;
	TCluster		*cluster;

	inherited::Open();

	doc=GetDrafterDoc();

	if (te=(TEditText*)FindSubView(vMainName))	//vMainName=vTEView, so 1st selected
	{
		te->SetText(doc->fPGMAppName,kDontRedraw);
		te->SetSelection(0,255,kDontRedraw);		//Needed on 1st selected
	};

	if (rb=(TRadio*)FindSubView(vPascal))
		rb->SetState(doc->fPascal,kDontRedraw);
	if (rb=(TRadio*)FindSubView(vCPlusPlus))
		rb->SetState(!doc->fPascal,kDontRedraw);

	if (doc->fPascal)
	{
		if (rb=(TRadio*)FindSubView(vOneSourceFile))
			rb->SetState(doc->fOneFile,kDontRedraw);
		if (rb=(TRadio*)FindSubView(vStdSourceFiles))
			rb->SetState(!doc->fOneFile && doc->fUseIncludes,kDontRedraw);
		if (rb=(TRadio*)FindSubView(vStdNoInclSrcFil))
			rb->SetState(!doc->fOneFile && !doc->fUseIncludes,kDontRedraw);
	}
	else
	{
		if (rb=(TRadio*)FindSubView(vOneSourceFile))
			rb->SetState(false,kDontRedraw);
		if (rb=(TRadio*)FindSubView(vStdSourceFiles))
			rb->SetState(true,kDontRedraw);
		if (rb=(TRadio*)FindSubView(vStdNoInclSrcFil))
			rb->SetState(false,kDontRedraw);
		if (cluster=(TCluster*)FindSubView(vFileSet))
			cluster->DimState(true,kDontRedraw);
	};
};

// ==Menu Information Collector==
//	   This method stores the menu usage information from the
//		menu screen views.

#pragma segment ADrafter
pascal void TDlgMenus::Close(void)
{
	TDrafterDoc		*doc;
	TCheckBox		*cb;

	doc=GetDrafterDoc();

	if (cb=(TCheckBox*)FindSubView(vTearMenus))
		doc->fTearOffMenus=cb->IsOn();
	if (cb=(TCheckBox*)FindSubView(vPopMenus))
		doc->fHierMenus=cb->IsOn();
	if (cb=(TCheckBox*)FindSubView(vViewDoesMenus))
		doc->fViewDoesMenuCommands=cb->IsOn();

	if (doc->fHasDocs)
		if (cb=(TCheckBox*)FindSubView(vDocDoesMenus))
			doc->fDocDoesMenuCommands=cb->IsOn();

	if (cb=(TCheckBox*)FindSubView(vWdwDoesMenus))
		doc->fWindowDoesMenuCommands=cb->IsOn();
	if (cb=(TCheckBox*)FindSubView(vAppDoesMenus))
		doc->fAppDoesMenuCommands=cb->IsOn();

	inherited::Close();
};

// ==Menu Information Stuffer==
//	   This method installs the current values for the menu
//		usage options into the menu screen views.

#pragma segment ADrafter
pascal void TDlgMenus::Open(void)
{
	TDrafterDoc		*doc;
	TCheckBox		*cb;

	inherited::Open();

	doc=GetDrafterDoc();

	if (cb=(TCheckBox*)FindSubView(vTearMenus))
		cb->SetState(doc->fTearOffMenus,kDontRedraw);
	if (cb=(TCheckBox*)FindSubView(vPopMenus))
		cb->SetState(doc->fHierMenus,kDontRedraw);
	if (cb=(TCheckBox*)FindSubView(vViewDoesMenus))
		cb->SetState(doc->fViewDoesMenuCommands,kDontRedraw);
	if (cb=(TCheckBox*)FindSubView(vDocDoesMenus))
		if (doc->fHasDocs)
			cb->SetState(doc->fDocDoesMenuCommands,kDontRedraw);
		else
		{
			cb->SetState(false,kDontRedraw);
			cb->ViewEnable(false,kDontRedraw);
			cb->DimState(true,kDontRedraw);
		};
	if (cb=(TCheckBox*)FindSubView(vWdwDoesMenus))
		cb->SetState(doc->fWindowDoesMenuCommands,kDontRedraw);
	if (cb=(TCheckBox*)FindSubView(vAppDoesMenus))
		cb->SetState(doc->fAppDoesMenuCommands,kDontRedraw);
};

// ==Non-Erase Window Drawer==
//	   This method provides window drawing without first eraseing the
//		update region.  This is useful when the update area is always
//		updated by subviews that draw over the window area completely.

#pragma segment AInit
pascal void TDrafterWindow::DrawContents(void)
{
	if (Focus())
	{
		//This TRICK gets to TVIEW.DrawContents skipping TWINDOW!!!
		TView::DrawContents();

		if (fIsResizable)
			DrawResizeIcon();
	};
};

// ==Screen (Card) Switcher==
//	   This method switches the views shown in the window.  Given a 'view'
//		resouce id, it replaces the current equivalent view with the new view.
//		The old view is hidden and closed before it is freed.  The new view
//		is opened and made visible.

#pragma segment AInit
pascal void TDrafterWindow::SwitchDialog(short newViewId)
{
	TDialogView		*oldMainView;
	TDialogView		*newMainView;
	TView			*rootView;
	TEditText		*editView;
	Boolean			wasShown;

	pApplication->SetTarget(pApplication);		//Deinstall the selection.
	fTarget=this;	//Cant use SetTarget since it will change the app target.

	rootView=this;
	wasShown=IsShown();

	if (oldMainView=(TDialogView*)FindSubView(vMainView))
	{

//	De-install the current switchable view.

		rootView=oldMainView->fSuperView;
		if (!rootView)
			rootView=this;

		newMainView=(TDialogView*)rootView->DoCreateViews(fDocument,this,newViewId,&gZeroVPt);
		if (!newMainView)
			newMainView=(TDialogView*)rootView->DoCreateViews(fDocument,this,vMainScreenValue,&gZeroVPt);

		if (wasShown=oldMainView->IsShown())	//Only showing views should be open
		{
			oldMainView->Show(false,kRedraw);
			oldMainView->Close();
		};
		oldMainView->Free();
	}
	else
		newMainView=(TDialogView*)rootView->DoCreateViews(fDocument,this,newViewId,&gZeroVPt);

	if (newMainView)
	{

//	Install the new switchable view.

		newMainView->fNextHandler=rootView;		//This window usually.
		newMainView->fIdentifier=vMainView;

		newMainView->Show(true,kRedraw);

		editView=(TEditText*)newMainView->FindSubView(vTEView);	//NOTE required TextEdit id

		if (wasShown)
			newMainView->Open();
		else if (editView)
		{
			gApplication->UpdateAllWindows();
			newMainView->DoSelectEditText(editView,true);		//Turn on TextEdit
		};

		if (editView)
			SetTarget(editView);
		else
			SetTarget(this);
	}
	else
		SetTarget(this);
};

// ==Document Value Clearer==
//	   This method zeros all document fields defined by this document class.

#pragma segment AInit
pascal void TDrafterDoc::Initialize(void)
{
	inherited::Initialize();

	fSpinState=0;

	fAppIcon=NULL;
	fDocIcon=NULL;
	fWhatsBeenSeen=NULL;

	fSignature=0;
	fDocType=0;
	fMinMem=0;
	fStandardMem=0;

	fVersRelease=0;
	fVersVersion=0;
	fVersBuild=0;

	fVersType=0;

  	fFreeWindows=false;
	fHasPrefs=false;
  	fPrefInSystem6=false;
  	fPrefInSystem7=false;
  	fPrefWithApp=false;

  	fOneFile=false;
  	fPascal=false;
  	fFloaters=false;
  	fTearOffMenus=false;
  	fHierMenus=false;
  	fSplash=false;
  	fUseIncludes=false;
  	fLaunchWithDoc=false;
	fHasDocs=false;
  	fPrints=false;
  	fAssociation=false;
  	fLists=false;
  	fTEView=false;
  	fDialogView=false;
  	fGridView=false;
  	fAppDoesMenuCommands=false;
  	fWindowDoesMenuCommands=false;
  	fDocDoesMenuCommands=false;
  	fViewDoesMenuCommands=false;
  	fDocReadData=false;
  	fDocWriteData=false;
  	fDocReadRsrc=false;
	fDocWriteRsrc=false;
  	fDocDataStaysOpen=false;
  	fDocRsrcStaysOpen=false;
  	fDocReverts=false;
	fViewsRefViews=false;
	fViewsSaveInPref=false;
	fSeparatePrintView=false;

	fPGMAppName[0]=0;
	fAppName[0]=0;
	fCopyRight[0]=0;
	fAuthors[0]=0;
};

// ==Debugger/Inspector==
//	   This method provides MacApp Debug support.

#pragma segment AFields
pascal void TDrafterDoc::Fields(pascal void (*DoToField)(StringPtr fieldName,Ptr fieldAddr,
							short fieldType,void *DoToField_StaticLink),void *DoToField_StaticLink)
{
	(*DoToField)("\pTDrafterDoc",	NULL,bClass,DoToField_StaticLink);
	(*DoToField)("\pfSpinState",	(Ptr)&fSpinState,bLongInt,DoToField_StaticLink);

	(*DoToField)("\pfWhatsBeenSeen",	(Ptr)&fWhatsBeenSeen,bObject,DoToField_StaticLink);
	(*DoToField)("\pfAppIcon",	(Ptr)&fAppIcon,bObject,DoToField_StaticLink);
	(*DoToField)("\pfDocIcon",	(Ptr)&fDocIcon,bObject,DoToField_StaticLink);
	(*DoToField)("\pfSignature",	(Ptr)&fSignature,bIDType,DoToField_StaticLink);
	(*DoToField)("\pfDocType",	(Ptr)&fDocType,bIDType,DoToField_StaticLink);
	(*DoToField)("\pfMinMem",	(Ptr)&fMinMem,bLongInt,DoToField_StaticLink);
	(*DoToField)("\pfStandardMem",	(Ptr)&fStandardMem,bLongInt,DoToField_StaticLink);
	(*DoToField)("\pfVersRelease",	(Ptr)&fVersRelease,bInteger,DoToField_StaticLink);
	(*DoToField)("\pfVersVersion",	(Ptr)&fVersVersion,bInteger,DoToField_StaticLink);
	(*DoToField)("\pfVersBuild",	(Ptr)&fVersBuild,bInteger,DoToField_StaticLink);
	(*DoToField)("\pfVersType",	(Ptr)&fVersType,bByte,DoToField_StaticLink);

	(*DoToField)("\pfHasPrefs",	(Ptr)&fHasPrefs,bBoolean,DoToField_StaticLink);
	(*DoToField)("\pfPrefInSystem6",	(Ptr)&fPrefInSystem6,bBoolean,DoToField_StaticLink);
	(*DoToField)("\pfPrefInSystem7",	(Ptr)&fPrefInSystem7,bBoolean,DoToField_StaticLink);
	(*DoToField)("\pfPrefWithApp",	(Ptr)&fPrefWithApp,bBoolean,DoToField_StaticLink);

	(*DoToField)("\pfFreeWindows",	(Ptr)&fFreeWindows,bBoolean,DoToField_StaticLink);
	(*DoToField)("\pfOneFile",	(Ptr)&fOneFile,bBoolean,DoToField_StaticLink);
	(*DoToField)("\pfPascal",	(Ptr)&fPascal,bBoolean,DoToField_StaticLink);
	(*DoToField)("\pfFloaters",	(Ptr)&fFloaters,bBoolean,DoToField_StaticLink);
	(*DoToField)("\pfTearOffMenus",	(Ptr)&fTearOffMenus,bBoolean,DoToField_StaticLink);
	(*DoToField)("\pfHierMenus",	(Ptr)&fHierMenus,bBoolean,DoToField_StaticLink);
	(*DoToField)("\pfSplash",	(Ptr)&fSplash,bBoolean,DoToField_StaticLink);
	(*DoToField)("\pfUseIncludes",	(Ptr)&fUseIncludes,bBoolean,DoToField_StaticLink);
	(*DoToField)("\pfLaunchWithDoc",	(Ptr)&fLaunchWithDoc,bBoolean,DoToField_StaticLink);
	(*DoToField)("\pfHasDocs",	(Ptr)&fHasDocs,bBoolean,DoToField_StaticLink);
	(*DoToField)("\pfPrints",	(Ptr)&fPrints,bBoolean,DoToField_StaticLink);
	(*DoToField)("\pfAssociation",	(Ptr)&fAssociation,bBoolean,DoToField_StaticLink);
	(*DoToField)("\pfViewsRefViews",	(Ptr)&fViewsRefViews,bBoolean,DoToField_StaticLink);
	(*DoToField)("\pfViewsSaveInPref",	(Ptr)&fViewsSaveInPref,bBoolean,DoToField_StaticLink);
	(*DoToField)("\pfSeparatePrintView",	(Ptr)&fSeparatePrintView,bBoolean,DoToField_StaticLink);
	(*DoToField)("\pfLists",	(Ptr)&fLists,bBoolean,DoToField_StaticLink);
	(*DoToField)("\pfTEView",	(Ptr)&fTEView,bBoolean,DoToField_StaticLink);
	(*DoToField)("\pfDialogView",	(Ptr)&fDialogView,bBoolean,DoToField_StaticLink);
	(*DoToField)("\pfGridView",	(Ptr)&fGridView,bBoolean,DoToField_StaticLink);
	(*DoToField)("\pfAppDoesMenuCommands",	(Ptr)&fAppDoesMenuCommands,bBoolean,DoToField_StaticLink);
	(*DoToField)("\pfWindowDoesMenuCommands",	(Ptr)&fWindowDoesMenuCommands,bBoolean,DoToField_StaticLink);
	(*DoToField)("\pfDocDoesMenuCommands",	(Ptr)&fDocDoesMenuCommands,bBoolean,DoToField_StaticLink);
	(*DoToField)("\pfViewDoesMenuCommands",	(Ptr)&fViewDoesMenuCommands,bBoolean,DoToField_StaticLink);
	(*DoToField)("\pfDocReadData",	(Ptr)&fDocReadData,bBoolean,DoToField_StaticLink);
	(*DoToField)("\pfDocWriteData",	(Ptr)&fDocWriteData,bBoolean,DoToField_StaticLink);
	(*DoToField)("\pfDocReadRsrc",	(Ptr)&fDocReadRsrc,bBoolean,DoToField_StaticLink);
	(*DoToField)("\pfDocWriteRsrc",	(Ptr)&fDocWriteRsrc,bBoolean,DoToField_StaticLink);
	(*DoToField)("\pfDocDataStaysOpen",	(Ptr)&fDocDataStaysOpen,bBoolean,DoToField_StaticLink);
	(*DoToField)("\pfDocRsrcStaysOpen",	(Ptr)&fDocRsrcStaysOpen,bBoolean,DoToField_StaticLink);
	(*DoToField)("\pfDocReverts",	(Ptr)&fDocReverts,bBoolean,DoToField_StaticLink);

	(*DoToField)("\pfPGMAppName",	(Ptr)&fPGMAppName,bString,DoToField_StaticLink);
	(*DoToField)("\pfAppName",	(Ptr)&fAppName,bString,DoToField_StaticLink);
	(*DoToField)("\pfCopyRight",	(Ptr)&fCopyRight,bString,DoToField_StaticLink);
	(*DoToField)("\pfAuthors",	(Ptr)&fAuthors,bString,DoToField_StaticLink);

	inherited::Fields(DoToField,DoToField_StaticLink);
}

// ==Document Initializer==
//	   This method initializes the document.  There is only one instance of this
//		document.  It is mainly used as a value store for user input.

#pragma segment ADrafter
pascal void TDrafterDoc::IDrafterDoc(void)
{
	IDocument('HYPD',kSignature,kUsesDataFork,kUsesRsrcFork,!kDataOpen,kRsrcOpen);

	fWhatsBeenSeen=NULL;
	fAppIcon=NULL;
	fDocIcon=NULL;
};

// ==Document Discarder==
//	   This method is called when the documents changes are being discarded.
//		***NOT USED IN CURRENT VERSION SINCE DOCUMENTS DONT GET SAVED***.

#pragma segment ADrafter
pascal void TDrafterDoc::Abandon(void)
{
	if (fWhatsBeenSeen)
		fWhatsBeenSeen->Free();
	if (fAppIcon)
		fAppIcon->Free();
	if (fDocIcon)
		fDocIcon->Free();
};

// ==Button Recorder==
//	   This method records any button choices and timestamps them too!

#pragma segment ADrafter
pascal void TDrafterDoc::DoChoice(TView *origView, short itsChoice)
{
	Str255			str;
	Str255			timeStr;

	if (itsChoice==mButtonHit)	//Track whats been seen.
	{
		BlockMove((Ptr)&origView->fIdentifier,(Ptr)&str[1],4);
		str[0]=4;

		NumToString(TickCount(),timeStr);

		if (fWhatsBeenSeen)
			fWhatsBeenSeen->InsertEntry(str,timeStr);
	};

	inherited::DoChoice(origView,itsChoice);
};

// ==Initial State Setter==
//	   This method sets all document values to their new untitled states.
//		It makes no assumptions about the current values of any fields.

pascal void TDrafterDoc::DoInitialState(void)
{
	inherited::DoInitialState();

	fSpinState=0;

	FailNIL(fWhatsBeenSeen=new TAssociation);
	fWhatsBeenSeen->IAssociation();

	fAppIcon=NULL;
	fDocIcon=NULL;
	fSignature='appl';
	fDocType='adoc';

	fMinMem=128*1024;
	fStandardMem=768*1024;

	fVersRelease=1;		//1.0d1
	fVersVersion=0;
	fVersType=0;
	fVersBuild=1;

  	fFreeWindows=false;
	fHasPrefs=false;
  	fPrefInSystem6=true;
  	fPrefInSystem7=false;
  	fPrefWithApp=false;

  	fOneFile=false;
  	fPascal=true;
  	fFloaters=false;
  	fTearOffMenus=false;
  	fHierMenus=false;
  	fSplash=false;
  	fUseIncludes=true;
  	fLaunchWithDoc=true;
	fHasDocs=true;
  	fPrints=true;
  	fAssociation=false;
	fViewsRefViews=false;
	fViewsSaveInPref=false;
	fSeparatePrintView=false;
  	fLists=false;
  	fTEView=false;
  	fGridView=false;
	fDialogView=true;
  	fAppDoesMenuCommands=false;
  	fWindowDoesMenuCommands=false;
  	fDocDoesMenuCommands=true;
  	fViewDoesMenuCommands=true;
  	fDocReadData=true;
  	fDocWriteData=true;
  	fDocReadRsrc=false;
  	fDocWriteRsrc=false;
  	fDocDataStaysOpen=false;
  	fDocRsrcStaysOpen=false;
  	fDocReverts=true;

	CopyStr255("\pMyApps Preferences",fPrefFileName);
	CopyStr255("\pMyApp",fPGMAppName);
	CopyStr255("\pMy Application",fAppName);
	CopyStr255("\p1990 Apple Computer, Inc. All rights reserved.",fCopyRight);	//Yes, I must.
	CopyStr255("\p1990 My Biz, Inc. All rights reserved.",fCopyRight);
	CopyStr255("\pMe, Myself & I and Danie",fAuthors);
};

// ==Window Maker==
//	   This method creates and opens the document window.

pascal void TDrafterDoc::DoMakeViews(Boolean forPrinting)
{
	TDrafterWindow		*aWindow;
	TView				*aView;

	if (!forPrinting)
	{
		if (gConfiguration.hasColorQD)
			FailNIL(aView=NewTemplateWindow(kPrimeColorWindow,this));
		else
			FailNIL(aView=NewTemplateWindow(kPrimeBWWindow,this));
		aWindow=(TDrafterWindow*)aView;
		aWindow->fOpenInitially=true;
		aWindow->SwitchDialog(vMainScreenValue);		//Install the main screen view.
	};
};

// ==Menu Controller==
//	   This method turns off unused standard commands.

#pragma segment ACommand
pascal void TDrafterDoc::DoSetupMenus(void)
{
	inherited::DoSetupMenus();

	Enable(cRevert,false);
	Enable(cSave,false);
	Enable(cSaveAs,false);
	Enable(cSaveCopy,false);
};

// ==Value Broadcaster==
//	   This method edits the given text using the information stored in the
//		document.  The supplied text must include tags that mark inclusion/exclusion
//		sections as well as vlue insertions.  Tag format depends on the transcriber
//		being used.

#pragma segment ATranscriber
pascal void TDrafterDoc::EnumerateInfo(class TTranscriber *transcriber,Handle text,short kind)
{
	Str255			str;

	transcriber->EditSection(kind&kMainFile,text,"\pMain");
	transcriber->EditSection(kind&kInterfaceFile,text,"\pInterface");

	transcriber->EditSection(kind&kImplementationFile,text,"\pImplementation");
	transcriber->EditSection(kind&kResource,text,"\pResource");
	transcriber->EditSection(kind&kMakeFile,text,"\pMake");

	transcriber->EditSection(!fHasDocs || !fLaunchWithDoc,text,"\pLaunchWithDoc");
	transcriber->EditSection(fHasDocs,text,"\pHasDocs");

	transcriber->EditSection(fHasPrefs,text,"\pPreferences");
	transcriber->EditSection(fPrefInSystem6,text,"\pPrefSys6");
	transcriber->EditSection(fPrefInSystem7,text,"\pPrefSys7");

	transcriber->EditSection(fPrefWithApp,text,"\pPrefWithApp");
	transcriber->EditSection(fOneFile,text,"\pSingleSourceFile");

	transcriber->EditSection(fUseIncludes,text,"\pIncludes");

	transcriber->EditSection(fFreeWindows,text,"\pAppHasWindows");
	transcriber->EditSection(fFloaters,text,"\pFloatWindows");
	transcriber->EditSection(fTearOffMenus,text,"\pTearMenus");
	transcriber->EditSection(fHierMenus,text,"\pHierMenus");

	transcriber->EditSection(fSplash,text,"\pSplashes");


	transcriber->EditSection(fFreeWindows||fHasDocs||fViewDoesMenuCommands,text,"\pHasViews");
	transcriber->EditSection(fViewsRefViews,text,"\pViewsRefViews");
	transcriber->EditSection(fSeparatePrintView,text,"\pViewsForPrint");

	transcriber->EditSection(fViewsSaveInPref,text,"\pViewsWriteRsrc");

	transcriber->EditSection(fPrints,text,"\pPrints");
	transcriber->EditSection(fAssociation,text,"\pAssocs");
	transcriber->EditSection(fLists,text,"\pLists");
	transcriber->EditSection(fTEView,text,"\pTEViews");
	transcriber->EditSection(fDialogView,text,"\pDlgViews");
	transcriber->EditSection(fGridView,text,"\pGridViews");

	transcriber->EditSection(fAppDoesMenuCommands,text,"\pAppMenus");
	transcriber->EditSection(fWindowDoesMenuCommands,text,"\pWindowMenus");
	transcriber->EditSection(fDocDoesMenuCommands,text,"\pDocMenus");

	transcriber->EditSection(fViewDoesMenuCommands,text,"\pViewMenus");
	transcriber->EditSection(fDocReadData||fDocWriteData,text,"\pDocUsesData");

	transcriber->EditSection(fDocReadRsrc||fDocWriteRsrc,text,"\pDocUsesRsrc");
	transcriber->EditSection(fDocReadData||fDocReadRsrc,text,"\pDocReads");
	transcriber->EditSection(fDocWriteData||fDocWriteRsrc,text,"\pDocWrites");

	transcriber->EditSection(fDocReadData,text,"\pDocReadData");

	transcriber->EditSection(fDocWriteData,text,"\pDocWriteData");
	transcriber->EditSection(fDocReadRsrc,text,"\pDocReadResources");
	transcriber->EditSection(fDocWriteRsrc,text,"\pDocWriteResources");
	transcriber->EditSection(fDocDataStaysOpen,text,"\pDocKeepsDataOpen");
	transcriber->EditSection(fDocRsrcStaysOpen,text,"\pDocKeepsRsrcOpen");

	transcriber->EditSection(fDocReverts,text,"\pDocReverts");

	transcriber->EditSection(fVersType<=2,text,"\pBuildNumber");

	transcriber->Substitute(text,"\pApple Integrated Systems","\p");
	transcriber->Substitute(text,"\pɩ1990 Apple Computer, Inc.","\p");
	transcriber->Substitute(text,"\pappPgmName",fPGMAppName);
	transcriber->Substitute(text,"\pappName",fAppName);
	transcriber->Substitute(text,"\pcopyright",fCopyRight);
	transcriber->Substitute(text,"\pauthors",fAuthors);
	transcriber->Substitute(text,"\pprefFileName",fPrefFileName);

	BlockMove((Ptr)&fSignature,(Ptr)&str[1],4);
	str[0]=4;
	transcriber->Substitute(text,"\psignature",str);
	BlockMove((Ptr)&fDocType,(Ptr)&str[1],4);
	str[0]=4;
	transcriber->Substitute(text,"\pdocType",str);

	NumToString(fMinMem,str);
	transcriber->Substitute(text,"\pminHeapSize",str);
	NumToString(fStandardMem,str);
	transcriber->Substitute(text,"\pstandardHeapSize",str);
	NumToString(fVersVersion,str);
	transcriber->Substitute(text,"\pappVersion",str);
	NumToString(fVersRelease,str);
	transcriber->Substitute(text,"\pappRelease",str);

	switch (fVersType)
	{
	case 0:
		transcriber->Substitute(text,"\pbuildType","\pdevelopment");	//[NOT INTERNATIONAL]
		transcriber->Substitute(text,"\pbuildLetter","\pd");
		break;
	case 1:
		transcriber->Substitute(text,"\pbuildType","\palpha");
		transcriber->Substitute(text,"\pbuildLetter","\pa");
		break;
	case 2:
		transcriber->Substitute(text,"\pbuildType","\pbeta");
		transcriber->Substitute(text,"\pbuildLetter","\p");
		break;
	default:
		transcriber->Substitute(text,"\pbuildType","\pfinal");
		transcriber->Substitute(text,"\pbuildLetter","\p");
		break;
	};

	NumToString(fVersBuild,str);
	transcriber->Substitute(text,"\pappBuild",str);
};

// ==Button Watcher==
//	   This method returns true when the button with the given id
//		has been previously chosen.

#pragma segment ADrafter
pascal Boolean TDrafterDoc::HaveSeen(StringPtr key)
{
	Str255	str;

	if (fWhatsBeenSeen)
		return fWhatsBeenSeen->ValueAt(key,str);
	else
		return false;
};

// ==Animated Cursor Controller==
//	   This method turns the animated cursor on or off.

#pragma segment ADrafter
pascal void TDrafterDoc::SpinControl(Boolean switchOn)
{
	gApplication->ActivateBusyCursor(!switchOn);

	if (switchOn)
		SpinCursor(0);

	fSpinState=0;
};

// ==Cursor Guardian==
//	   This method advances the time for the animated cursor.

#pragma segment ADrafter
pascal void TDrafterDoc::SpinTheCursor(void)
{
	RotateCursor(fSpinState+=4);	//See MPW Cursor unit for info.
};

// ==Output Generator==
//	   This method prompts the user for a target folder and
//		if successful, writes out a file set containing the
//		draft MacApp program based on the gathered information.

#pragma segment ADrafter
pascal void TDrafterDoc::WriteMacAppWorld(void)
{
	TTranscriber		*transcriber;
	long				dirId;
	short				volRefNum;
	short				segmentId;
	Str255				name;

	segmentId=GetSegNumber((Ptr)&ModalObjectFilter);
	SetResidentSegment(segmentId,true);

	if (pApplication->PickFolder((short*)&volRefNum,(long*)&dirId,(Str255*)&name,(Boolean)false))
	{
		SetResidentSegment(segmentId,false);

		if (fPascal)
			FailNIL(transcriber=new TPascalTranscriber);
		else
			FailNIL(transcriber=new TCPlusPlusTranscriber);

		SpinControl(true);
		SpinTheCursor();

		transcriber->ITranscriber(this,fOneFile,fUseIncludes);
		transcriber->WriteProgram(volRefNum,dirId);
		transcriber->Free();

		SpinControl(false);
	}
	else
		SetResidentSegment(segmentId,false);
};

// ==Transcriber Zeroer==
//	   This method zeros all transcriber fields.

#pragma segment ATranscriber
pascal void TTranscriber::Initialize(void)
{
	inherited::Initialize();

	fSingleFile=false;
	fIncludes=false;
	fDocument=NULL;	//Make sure failure handlers always get a known state!
};

// ==Debugger==
//	   This method privdes MacApp debugging support.

#pragma segment AFields
pascal void TTranscriber::Fields(pascal void (*DoToField)(StringPtr fieldName,Ptr fieldAddr,
							short fieldType,void *DoToField_StaticLink),void *DoToField_StaticLink)
{
	(*DoToField)("\pTTranscriber",	NULL,bClass,DoToField_StaticLink);
	(*DoToField)("\pfDocument",	(Ptr)&fDocument,bObject,DoToField_StaticLink);
	(*DoToField)("\pfSingleFile",	(Ptr)&fSingleFile,bBoolean,DoToField_StaticLink);
	(*DoToField)("\pfIncludes",	(Ptr)&fIncludes,bBoolean,DoToField_StaticLink);

	inherited::Fields(DoToField,DoToField_StaticLink);
};

// ==Transcriber Initializer==
//	   This method initializes a new transcriber.

#pragma segment ATranscriber
pascal void TTranscriber::ITranscriber(TDrafterDoc *doc,Boolean useSingleFile,Boolean useIncludes)
{
	IObject();

	fSingleFile=useSingleFile;
	fIncludes=useIncludes;
	fDocument=doc;
};

// ==File Closer==
//	   This method closes the current output file and returns the Close status.

#pragma segment ATranscriber
pascal OSErr TTranscriber::CloseFile(void)
{
	return FSClose(fFileRefNum);
};

// ==Inclusion/Exclusion Editor==
//	   This method deletes all text between two tags or just deletes the tags
//		themselves.  Each tag is processed first in the exclusion (delete all)
//		context, then in the inclusion (delete just tags) context.

#pragma segment ATranscriber
pascal void TTranscriber::EditSection(Boolean condition,Handle data,const Str31 tagName)
{
	long			offset;
	long			count;
	Size			blockLength;
	long			i;
	long			j;
	short			matches;
	Str255			beginTag;
	Str255			endTag;
	Str255			errorMsg;

	fDocument->SpinTheCursor();

	if (data)
	{
		PConcat(beginTag,"\p[",tagName);		//[Tag?
		PConcat(beginTag,beginTag,"\p?");
		PConcat(endTag,"\p?",tagName);			//?Tag]
		PConcat(endTag,endTag,"\p]");

		matches=1;
		while (matches > 0)
		{
			matches=0;

//Exclude sections (first because most greatly reduces hande size)

			if (condition)
			{
				beginTag[Length(beginTag)-1]='-';	//Zap [tag-  -tag]
				endTag[2]='-';
			}
			else
			{
				beginTag[Length(beginTag)-1]='+';	//Zap [tag+  +tag]
				endTag[2]='+';
			};

			offset=0;
			while (offset >= 0)
			{
				offset=Munger(data,offset,(Ptr)&beginTag[1],Length(beginTag),NULL,0);
				if (offset >= 0)
				{
					matches++;

					i=offset;
					offset+=Length(beginTag);
					offset=Munger(data,offset,(Ptr)&endTag[1],Length(endTag),NULL,0);
					if (offset < 0)
					{
#if qDebug
/**/					PConcat(errorMsg,"\pExclude noticed unbalanced tag ==> ",tagName);
/**/					DebugStr(errorMsg);
#endif qDebug
						offset=GetHandleSize(data);
					}
					else
						offset+=Length(endTag);
					j=offset;

//******Munger causes heap damage when > 32K handle and big deletion  occurs!!!!!!
//					offset=Munger(data,i,NULL,j-i,(Ptr)&i,0);

					blockLength=GetHandleSize(data);
					count=blockLength-j;
					BlockMove((*data)+j,(*data)+i,count);
					SetHandleSize(data,blockLength-(j-i));

				fDocument->SpinTheCursor();
				};
			};
//Include sections

			if (condition)
			{
				beginTag[Length(beginTag)-1]='+';	//Keep [tag+  +tag]
				endTag[2]='+';
			}
			else
			{
				beginTag[Length(beginTag)-1]='-';	//Keep [tag-  -tag]
				endTag[2]='-';
			};

			offset=0;
			while (offset >= 0)
			{
				offset=Munger(data,offset,(Ptr)&beginTag[1],Length(beginTag),(Ptr)&i,0);
				if (offset >= 0)
				{
					matches++;

					offset=Munger(data,offset,(Ptr)&endTag[1],Length(endTag),(Ptr)&i,0);
					if (offset < 0)
					{
#if qDebug
/**/					PConcat(errorMsg,"\pInclude noticed unbalanced tag ==> ",tagName);
/**/					DebugStr(errorMsg);
#endif qDebug
					}
					else
						offset+=Length(endTag);
				};
				fDocument->SpinTheCursor();
			};
		};
	};
};

// ==File Namer==
//	   This method picks file names for output files.  The kind of file
//		must be supplied.

#pragma segment ATranscriber
pascal void TTranscriber::FileNameForOutput(StringPtr fileName,short kind)
{
	Str255			appName;

	CopyStr255(fDocument->fPGMAppName,&appName);

	switch (kind)
	{
		case kMainFile:
			PConcat(fileName,"\pM",appName);
			break;
		case kInterfaceFile:
			PConcat(fileName,appName,"\p.interf");
			break;
		case kImplementationFile:
			PConcat(fileName,appName,"\p.implem");
			break;
		case kResource:
			PConcat(fileName,appName,"\p.r");
			break;
		case kMakeFile:
			PConcat(fileName,appName,"\p.MAMake");
			break;
	};
};

// ==Template Reader==
//	   This method returns a new handle of TEXT to be used as the
//		unedited content of an output file.  The file kind must be supplied.

#pragma segment ATranscriber
pascal Handle TTranscriber::GetRawData(short kind)
{
	Handle		rawData;

	switch (kind)
	{
		case kMainFile:
			rawData=GetNamedResource(rTextResource,"\pMain Program");
			break;
		case kInterfaceFile:
			rawData=GetNamedResource(rTextResource,"\pInterface");
			break;
		case kImplementationFile:
			rawData=GetNamedResource(rTextResource,"\pImplementation");
			break;
		case kResource:
			rawData=GetNamedResource(rTextResource,"\pRez");
			break;
		case kMakeFile:
			rawData=GetNamedResource(rTextResource,"\pMake");
			break;
		default:
			rawData=NULL;
			break;
	};

	FailNILResource(rawData);
	DetachResource(rawData);
	FailSpaceIsLow();

	return rawData;
};

// ==File Opener==
//	   This method opens output files.  (SHOULD BE RENAMED TO OpenOutputFile??)

#pragma segment ATranscriber
pascal OSErr TTranscriber::OpenFile(const Str255 name,short volRefNum,long dirId)
{
	OSErr			result;
	short			fileRefNum;

	result=HOpen(volRefNum,dirId,name,fsWrPerm,&fileRefNum);
	if (result==fnfErr)
	{
		result=HCreate(volRefNum,dirId,name,'MPS ','TEXT'); 
		result=HOpen(volRefNum,dirId,name,fsWrPerm,&fileRefNum);
	};

	if ((result==noErr) || (result==dupFNErr))
	{
		fFileRefNum=fileRefNum;
		if (result==noErr)
			result=SetEOF(fFileRefNum,0); 
	};

	return result;
};

// ==Value Substitutor==
//	   This method replaces the given tag with the given text
//		value in the supplied text.

#pragma segment ATranscriber
pascal void TTranscriber::Substitute(Handle data,const Str255 tag,const Str255 textValue)
{
	long			offset;

	fDocument->SpinTheCursor();

	if (data)
	{
		offset=0;
		while (offset >= 0)
			offset=Munger(data,offset,(Ptr)&tag[1],Length(tag),
						(Ptr)&textValue[1],Length(textValue));
	};
};

// ==File Writer==
//	   This method writes the edited file content from memory to disk.

#pragma segment ATranscriber
pascal OSErr TTranscriber::WriteHandle(Handle fileText)
{
	long			count;
	long			subTotal;
	long			total;
	Ptr				theData;
	OSErr			result;

	if (fileText)
	{
		result=0;		//Zap NULL byte (often one at end of templates).
		total=Munger(fileText,0,(Ptr)&result,1,(Ptr)&result,0);

		if (total=GetHandleSize(fileText))
		{
			HLock(fileText);
			result=noErr;
			subTotal=0;
			theData=*fileText;
			while ((subTotal<total) && (result==noErr))
			{
				count=total-subTotal;
				if (count > 2048)
				{
					count=2048;
					fDocument->SpinTheCursor();
				};
				result=FSWrite(fFileRefNum,&count,theData);		//Write 2K of data at a time
				subTotal+=count;
				theData+=count;
			};
			HUnlock(fileText);
		};

		return result;
	}
	else
		return noErr;
};

// ==File Set Creator==
//	   This method creates all files in the output file set and writes them.
//		The exact names and contents of the file set depend on user suppplied
//		information.

#pragma segment ATranscriber
pascal void TTranscriber::WriteProgram(short vRefNum,long dirId)
{
	short			kind;

	fDocument->SpinTheCursor();

	WriteProgramFile(vRefNum,dirId,kResource,GetRawData(kResource));	//Rez file

	if (fSingleFile)
		kind=kMainFile+kInterfaceFile+kImplementationFile;
	else
		kind=kMainFile;

	WriteProgramFile(vRefNum,dirId,kind,GetRawData(kind));				//Main program

	if (!fSingleFile)
	{
		if (fIncludes)
			kind=kInterfaceFile;
		else
			kind=kInterfaceFile+kImplementationFile;

		WriteProgramFile(vRefNum,dirId,kind,GetRawData(kind));			//Interface

		if (fIncludes)
			WriteProgramFile(vRefNum,dirId,kImplementationFile,GetRawData(kImplementationFile));
	};

	WriteProgramFile(vRefNum,dirId,kMakeFile,GetRawData(kMakeFile));	//Make
};

// ==Fiel Writer==
//	   This method opens (creates) an output file and writes to it.

#pragma segment ATranscriber
pascal void TTranscriber::WriteProgramFile(short vRefNum,long dirId,short kind,Handle fileText)
{
	Str255			fileName;

	FailInfo		fi;
	jmp_buf			fb;

	if (!setjmp(fb))	//attempt the main logic
	{
		fDocument->SpinTheCursor();

		CatchFailures(&fi,CommonHandler,fb);

		fDocument->EnumerateInfo(this,fileText,kind);

		FileNameForOutput(fileName,kind);
		FailOSErr(OpenFile(fileName,vRefNum,dirId));

		fDocument->SpinTheCursor();

		FailOSErr(WriteHandle(fileText));
		FailOSErr(CloseFile());

		fDocument->SpinTheCursor();

		Success(&fi);
		DisposHandle(fileText);
	}
	else
	{
		DisposHandle(fileText);

		Failure(gFailError,gFailMessage);		//Propagate the exception
	};

	DisposHandle(fileText);
};

// ==Pascal File Namer==
//	   This method provides names for Object Pascal files.

#pragma segment APTranscriber
pascal void TPascalTranscriber::FileNameForOutput(StringPtr fileName,short kind)
{
	Str255			scratch;
	Str255			appName;

	CopyStr255(fDocument->fPGMAppName,&appName);

	switch (kind)
	{
		case kMainFile:
			PConcat(scratch,"\pM",appName);
			PConcat(fileName,scratch,"\p.p");
			break;
		case kInterfaceFile:
			PConcat(scratch,"\pU",appName);
			PConcat(fileName,scratch,"\p.p");
			break;
		case kImplementationFile:
			PConcat(scratch,"\pU",appName);
			PConcat(fileName,scratch,"\p.inc1.p");
			break;
		default:
			inherited::FileNameForOutput(fileName,kind);
			break;
	};
};

// ==Pascal File Template Supplier==
//	   This method returns the file templates for Pascal files.

#pragma segment ATranscriber
pascal Handle TPascalTranscriber::GetRawData(short kind)
{
	Handle		rawData;

	switch (kind)
	{
		case kMainFile:
		case kInterfaceFile:
		case kImplementationFile:
			rawData=GetNamedResource(rTextResource,"\pMainPascal");
			break;
		default:
			rawData=inherited::GetRawData(kind);
			break;
	};

	FailNILResource(rawData);
	DetachResource(rawData);
	FailSpaceIsLow();

	return rawData;
};

// ==C++ File Namer==
//	   This method provides the file names for C++ files.

#pragma segment ACTranscriber
pascal void TCPlusPlusTranscriber::FileNameForOutput(StringPtr fileName,short kind)
{
	Str255			scratch;
	Str255			appName;

	CopyStr255(fDocument->fPGMAppName,&appName);

	switch (kind)
	{
		case kMainFile:
			PConcat(scratch,"\pM",appName);
			PConcat(fileName,scratch,"\p.cp");
			break;
		case kInterfaceFile:
			PConcat(scratch,"\pU",appName);
			PConcat(fileName,scratch,"\p.h");
			break;
		case kImplementationFile:
			PConcat(scratch,"\pU",appName);
			PConcat(fileName,scratch,"\p.cp");
			break;
		default:
			inherited::FileNameForOutput(fileName,kind);
			break;
	};
};

// ==C++ File Template Supplier==
//	   This method returns the file templates for C++ files.

#pragma segment ATranscriber
pascal Handle TCPlusPlusTranscriber::GetRawData(short kind)
{
	Handle		rawData;

	switch (kind)
	{
		case kMainFile:
		case kInterfaceFile:
		case kImplementationFile:
			rawData=GetNamedResource(rTextResource,"\pMainCPlusPlus");
			break;
		default:
			rawData=inherited::GetRawData(kind);
			break;
	};

	FailNILResource(rawData);
	DetachResource(rawData);
	FailSpaceIsLow();

	return rawData;
};

