// Nothing.cp
// Copyright  1986-1991 by Apple Computer, Inc. All rights reserved.

/*
  This is a very small sample application which can save files on disk and can
  print.
  The application's windows each contain the word 'MacApp', in large type, and
  are framed by a large gray border.
  Documents can be saved on disk and later reopened, but the only
  document-specific information saved in the disk file is the Print information,
  which means that if you set print specifications using the Page Setup dialog
  for a document, and then save the document (using Save As or Save a Copy),
  those specifications are saved on disk, and when you reopen the document
  you will find those same print specifications holding for the document.
  All applications that create views procedurally need to reimplement at least three
  methods:
  TApplication.DoMakeDocument	--	Launches the appropriate type of Document object
  TDocument.DoMakeViews		--	Launches the appropriate type of View and window objects
  TView.Draw					--	Draws the contents of a view
  This application, however, consists of the reimplementation of only one method -
  TView.Draw.  This is possible since the view is created from templates;  MacApp
  supplies the default 'view' resource.  So in a sense this application is the smallest
  possible MacApp application.
*/

//  {$MC68020-}												// The main program must be universal code
//  {$MC68881-}

//-------------------------------------------------------------------------------------
// INCLUDES


#ifndef __TYPES__
#include <Types.h>
#endif

#ifndef __MACAPPTYPES__
#include <MacAppTypes.h>
#endif

#ifndef __MEMORY__
#include <Memory.h>
#endif

#ifndef __UOBJECT__
#include <UObject.h>
#endif

#ifndef __ULIST__
#include <UList.h>
#endif

#ifndef __APPLEEVENTS__
#include <AppleEvents.h>
#endif

#ifndef __UEVENT__
#include <UEvent.h>
#endif

#ifndef __UCOMMAND__
#include <UCommand.h>
#endif

#ifndef __EDITIONS__
#include <Editions.h>
#endif

#ifndef __SCRAP__
#include <Scrap.h>
#endif


#ifndef __DIALOGS__
#include <Dialogs.h>
#endif

#ifndef __UEVTHANDLER__
#include <UEvtHandler.h>
#endif

#ifndef __UAPPLICATION__
#include <UApplication.h>
#endif

#ifndef __UDOCUMENT__
#include <UDocument.h>
#endif

#ifndef __BALLOONS__
#include <Balloons.h>
#endif

#ifndef __UVIEW__
#include <UView.h>
#endif

#ifndef __UWINDOW__
#include <UWindow.h>
#endif

#ifndef __UPRINTHANDLER__
#include <UPrintHandler.h>
#endif

#ifndef __UFAILURE__
#include <UFailure.h>
#endif

#ifndef __UGEOMETRY__
#include <UGeometry.h>
#endif

#ifndef __UMACAPPUTILITIES__
#include <UMacAppUtilities.h>
#endif

#ifndef __MENUS__
#include <Menus.h>
#endif

#ifndef __UMENUMGR__
#include <UMenuMgr.h>
#endif

#ifndef __UPATCH__
#include <UPatch.h>
#endif

#ifndef __UMACAPPGLOBALS__
#include <UMacAppGlobals.h>
#endif

#ifndef __UFILEBASEDDOCUMENT__
#include <UFileBasedDocument.h>
#endif

#ifndef __PRINTING__
#include <Printing.h>
#endif

#ifndef __UPRINTING__
#include <UPrinting.h>
#endif

#ifndef __FONTS__
#include <Fonts.h>
#endif

#ifndef __UERRORMGR__
#include <UErrorMgr.h>
#endif

//-------------------------------------------------------------------------------------
// CONSTANTS

const OSType kSignature = 'SS01';				// Application signature
const OSType kFileType = 'SF01';				// File-type code used for document files created by this application

//-------------------------------------------------------------------------------------
// TYPES

class TNothingApplication : public TApplication {
public:
	// C++ constructor. 
	virtual pascal void INothingApplication(OSType itsMainFileType);
};


class TDefaultView : public TView {
public:
	// Draws the view seen in the window. Every nonblank view MUST override this method. 
	virtual pascal void Draw(const VRect& area);
    virtual pascal GrafPtr GetGrafPort ();
};

//-------------------------------------------------------------------------------------

pascal void TNothingApplication::INothingApplication(OSType itsMainFileType)
{
	inherited::IApplication(itsMainFileType);
	
	// So my view will be substituted when MacApp creates the "default view"
	RegisterStdType("TDefaultView", 'dflt');

	// So the linker doesn't dead strip class info 
	if (gDeadStripSuppression)
		DontDeadStrip(TDefaultView)
}

// ---------------------------------------------------------------------------------------
pascal GrafPtr TDefaultView::GetGrafPort ()
{
	return inherited::GetGrafPort();
}

// NOTE: we don't define a name for the argument here, so CFront won't complain about
// the argument being unused!
pascal void TDefaultView::Draw(const VRect&)
{
	Rect itsQDExtent;
	Str255 title = "MacApp";

	PenNormal();
	PenSize(10, 10);
	PenPat(qd.dkGray);

	// We know its extent always fits in QuickDraw coordinates.
	GetQDExtent(itsQDExtent);
	// Draw a dark gray frame
	FrameRect(itsQDExtent);

	// Set font and size for subsequent display in the window
	TextFont(times);	// a nice outline font
	TextSize(72);

	MoveTo(45, 90);
	// Draw the word 'MacApp' in large type.
	DrawString(title);
	// Restore the pen state.
	PenNormal();
	
	if (GetGrafPort());
}

// ---------------------------------------------------------------------------------------
// T H E   M A I N	P R O G R A M

// GLOBALS
TNothingApplication *gNothingApplication;		// The application object 

void main()
{
	// Essential toolbox and utilities initialization
	InitToolBox();

	// Make sure we can run 
	if (ValidateConfiguration(gConfiguration))
	{
		// We made it! Continue with remainder of initialization 

		// Initialize MacApp; 8 calls to MoreMasters
		InitUMacApp(8);

		InitUPrinting();

		// Construct a new TNothingApplication object.  Allocation errors are checked for.
		gNothingApplication = new TNothingApplication;
		gNothingApplication->INothingApplication(kFileType);

		// Run the application. When it's done, exit.

		gNothingApplication->Run();
	}
	else
		{
		StdAlert(phUnsupportedConfiguration);
	}

}


