/********************************************************************************/
/*																				*/
/*	File:			Calc.r		 												*/
/*																				*/
/*	Description:	Resource File for the Calc Sample Program.					*/
/*																				*/
/*																				*/
/*	Copyright  1985 - 1991 by Apple Computer, Inc.  All rights reserved.  		*/
/********************************************************************************/

/*  Auto-Include the requirements for this source */

#ifndef __TYPES.R__
#include "Types.r"
#endif

#ifndef __SYSTYPES.R__
#include "SysTypes.r"
#endif

#ifndef __MacAppTypes__
#include "MacAppTypes.r"
#endif

#ifndef __ViewTypes__
#include "ViewTypes.r"
#endif

#ifndef __BalloonTypes__
#include "BalloonTypes.r"
#endif


#if qDebug
include "Debug.rsrc";
#endif

include "MacApp.rsrc";
include "Printing.rsrc";

include "Editions.rsrc";				// since Calc supports the Edition Mgr


include $$Shell("ObjApp")"Calc" 'CODE';

/* Include the separately compiled resources */
include "CalcUtilities.rsrc";

#define mFormat			4
#define mCalculation	5


#define cPrintSelection		180
#define cRecalculate		401
#define cAutoCalc			403
#define cManualCalc			404
#define cGeneral			501
#define cNoDecimal			503
#define cDecimal			504
#define cScientific			505
#define cSystemJustify		506
#define cForceLeftJustify	507
#define cRightJustify		508
#define cCenter				509
#define cSelection			1000
#define cSizeColumn			1001
#define cCutText			1103
#define cCopyText			1104
#define cClearText			1106
#define cCutCells			1203
#define cCopyCells			1204
#define cClearCells			1206
#define cStandardCut		1303
#define cStandardCopy		1304
#define cStandardClear		1306

/* static variables to initialize view templates */
#define	kMaxRows			120				/* Max number of rows */
#define	kMaxColumns			64				/* max number of columns */
#define	kCellWidth			80				/* default width of each cell */
#define	kCellHeight			17				/* default height of each cell */
#define	kRowInset			4				/* pixels separating cell contents from top/bottom edge */
#define	kColumnInset		4				/* pixels separating cell contents from left/right edge */
#define	kRowTitleWidth		32				/* width of row titles */
#define kRowTitleHeight		kCellHeight		/* height of the row titles */
#define	kColumnTitleWidth	kCellWidth		/* width of column titles */
#define	kColumnTitleHeight	20				/* height of column titles */
#define kFontSize			10
#define kMaxChars			255 			/* number of characters in the text edit field */

	
#define kCalcWindowType		1001 			/* ID of the main window or view resource */
#define kCellsViewHelp		128 			/* ID of the help resource for the cells view */
#define kColsViewHelp		129 			/* ID of the help resource for the columns view */
#define kRowsViewHelp		130 			/* ID of the help resource for the row view */


#define kSignature			'SS12'			/* Application signature */
#define kFileType			'SF12'			/* Data file type */

#define phSplash			1001			/* ID of splash screen dialog */

/*--------------------------------------------------------------------------------
 MultiFinder/Switcher info
--------------------------------------------------------------------------------*/
resource 'SIZE' (-1) {
	saveScreen,
	acceptSuspendResumeEvents,
	enableOptionSwitch,
	canBackground,
	doesActivateOnFGSwitch,
	backgroundAndForeground,
	dontGetFrontClicks,
	ignoreAppDiedEvents,
	is32BitCompatible,
	isHighLevelEventAware,
	localAndRemoteHLEvents,
	notStationeryAware,				
	reserved,
	reserved,
	reserved,
	reserved,
#if qDebug
	(720-32) * 1024,
	(620-32) * 1024
#else
	(600-32) * 1024,
	(500-32) * 1024
#endif
};

/*--------------------------------------------------------------------------------
 Memory usage info for MacApp
--------------------------------------------------------------------------------*/
resource 'seg!' (256,
#if qNames
"Calc",
#endif
	purgeable) {
	{	"GTerminate";
		"GNonRes";
		"GOpen";
		"GWriteFile";
		"GFile";
		"GClose";
		"GClipboard";
		"GDoCommand";
		"GSelCommand";
		"ARes"
	}
};

/* memory stuff */
resource 'mem!' (132,
#if qNames
	"Additional Stack Space",
#endif
	purgeable) {
	0,				// Add to temporary reserve
	0,				// Add to permanent reserve
	49152			// Add to stack space
};


/*--------------------------------------------------------------------------------
 About Box
--------------------------------------------------------------------------------*/
resource 'DITL' (phAboutApp,
#if qNames
"phAboutApp",
#endif
	purgeable) {
	 {	/* array DITLarray: 3 elements */
		/* [1] */
		{160, 182, 180, 262},
		Button {
			enabled,
			"OK"
		};
		/* [2] */
		{10, 75, 150, 316},
		StaticText {
			disabled,
			"This sample program implements a spreadsheet with MacApp."
			"\n\nThis program was written "
			"with MacApp  1985-1991 Apple Computer, Inc."
		};
		/* [3] */
		{10, 20, 42, 52},
		Icon {
			disabled,
			1
		}
	}
};

include "ResourceLib.rsrc"  'CURS' (9);	// Grab the grabber hand
include "Defaults.rsrc"  'ALRT' (phAboutApp);	// Grab the default about box

/*--------------------------------------------------------------------------------
 Menus!
--------------------------------------------------------------------------------*/
include "Defaults.rsrc" 'CMNU' (mApple);		// Grab the default Apple menus

resource 'CMNU' (mFile,
#if qNames
"mFile",
#endif
nonpurgeable) {
	mFile,
	textMenuProc,
	EnablingManagedByMacApp,
	enabled,
	"File",
	 {	/* array: 14 elements */
/* [1] */	"New",				noIcon, "N",	noMark, plain, cNew;
/* [2] */	"Open",			noIcon, "O",	noMark, plain, cOpen;
/* [3] */	"-",				noIcon, noKey,	noMark, plain, nocommand;
/* [4] */	"Close",			noIcon, "W",	noMark, plain, cClose;
/* [5] */	"Save",				noIcon, "S",	noMark, plain, cSave;
/* [6] */	"Save As",			noIcon, noKey,	noMark, plain, cSaveAs;
/* [7] */	"Save a Copy In",	noIcon, noKey,	noMark, plain, cSaveCopy;
/* [8] */	"Revert",			noIcon, noKey,	noMark, plain, cRevert;
/* [9] */	"-",				noIcon, noKey,	noMark, plain, nocommand;
/* [10] */	"Page Setup",		noIcon, noKey,	noMark, plain, cPageSetup;
/* [11] */	"Print",			noIcon, "P",	noMark, plain, cPrint;
/* [12] */	"Print Selection",	noIcon, noKey,	noMark, plain, cPrintSelection;
/* [13] */	"-",				noIcon, noKey,	noMark, plain, nocommand;
/* [14] */	"Quit",				noIcon, "Q",	noMark, plain, cQuit
	}
};

include "Defaults.rsrc" 'CMNU' (mEditionMgrEdit);	// Grab the default Edition Manager Edit menu

resource 'CMNU' (mFormat,
#if qNames
"mFormat",
#endif
nonpurgeable) {
	mFormat,
	textMenuProc,
	EnablingManagedByMacApp,
	enabled,
	"Format",
	 {	/* array: 8 elements */
/* [1] */	"General",					noIcon, noKey, noMark, plain, cGeneral;
/* [2] */	"No Decimal",				noIcon, noKey, noMark, plain, cNoDecimal;
/* [3] */	"Decimal",					noIcon, noKey, noMark, plain, cDecimal;
/* [4] */	"Scientific",				noIcon, noKey, noMark, plain, cScientific;
/* [5] */	"-",						noIcon, noKey, noMark, plain, nocommand;
/* [6] */	"System Justification",		noIcon, noKey, noMark, plain, cSystemJustify;
/* [7] */	"Force Left Justification",	noIcon, noKey, noMark, plain, cForceLeftJustify;
/* [8] */	"Center Justification",		noIcon, noKey, noMark, plain, cCenter;
/* [9] */	"Right Justification",		noIcon, noKey, noMark, plain, cRightJustify
	  }
	};

resource 'CMNU' (mCalculation,
#if qNames
"mCalculation",
#endif
nonpurgeable) {
	mCalculation,
	textMenuProc,
	EnablingManagedByMacApp,
	enabled,
	"Calculation",
	 {	/* array: 4 elements */
/* [1] */	"Recalculate",		noIcon, "R",	noMark,	plain, cRecalculate;
/* [2] */	"-",				noIcon, noKey,	noMark,	plain, nocommand;
/* [3] */	"Automatic Calc",	noIcon, noKey,	check,	plain, cAutoCalc;
/* [4] */	"Manual Calc",		noIcon, noKey,	noMark,	plain, cManualCalc
	}
};

resource 'CMNU' (mBuzzwords,
#if qNames
"Buzzwords",
#endif
nonpurgeable
) {
	mBuzzwords,
	textMenuProc,
	EnablingManagedByMacApp,
	enabled,
	"Buzzwords",
	 {	/* array: 13 elements */
		"Page Setup Change",	noIcon,	noKey,	noMark, plain, cChangePrinterStyle;
		"Typing",				noIcon,	noKey,	noMark,	plain, cTyping;
		"Selection",			noIcon,	noKey,	noMark, plain, cSelection;
		"Column Resize",		noIcon,	noKey,	noMark, plain, cSizeColumn;
		"Cut Text",				noIcon,	"X",	noMark, plain, cCutText;
		"Copy Text",			noIcon,	"C",	noMark, plain, cCopyText;
		"Clear Text",			noIcon,	noKey,	noMark, plain, cClearText;
		"Cut Cells",			noIcon,	"X",	noMark, plain, cCutCells;
		"Copy Cells",			noIcon,	"C",	noMark, plain, cCopyCells;
		"Clear Cells",			noIcon,	noKey,	noMark, plain, cClearCells;
		"Cut",					noIcon,	"X",	noMark, plain, cStandardCut;
		"Copy",					noIcon,	"C",	noMark, plain, cStandardCopy;
		"Clear",				noIcon,	 noKey,	noMark, plain, cStandardClear
	}
};

resource 'MBAR' (kMBarDisplayed,
#if qNames
"Calc",
#endif
	purgeable) {
	{mApple; mFile; mEditionMgrEdit; mFormat; mCalculation}
};


/*--------------------------------------------------------------------------------
 For building the main view from templates
--------------------------------------------------------------------------------*/
resource 'view' (kCalcWindowType,
#if qNames
"Spreadsheet",
#endif
	purgeable) {
	{
	/* Main (and only) Window */
		root, 'WIND', { 40, 10 }, { 288, 447 }, sizeVariable, sizeVariable, shown, enabled,
		Window		{ "TCalcWindow", zoomDocProc, goAwayBox, resizable,
			modeless, ignoreFirstClick,freeOnClosing, disposeOnFree, closesDocument,
			openWithDocument, dontAdaptToScreen, stagger, forceOnScreen,
			dontCenter, noID, "Spreadsheet <<<>>>" };
			
	/* Coordinate View */
		'WIND', 'CORD',
		{ 0, 8 }, { 32, 48 }, sizeFixed, sizeFixed, shown, enabled,
		View {
			"TCoordView" },
			
	/* Cells View */
		'WIND', 'SCL1',
		{ 52, 32 }, { 221, 400 }, sizeRelSuperView, sizeRelSuperView, shown, enabled,
		Scroller {
			"TPrimaryScroller",
			VertScrollBar, HorzScrollBar,
			0, 0, 0, 0,
			VertConstrain, HorzConstrain,
			{-16, -31, 0, 0 }},

		'SCL1', 'CELL',
		{ 0, 0 }, { 0, 0 }, sizeVariable, sizeVariable, shown, enabled,
		TextGridView {
			"TCellsView",
			kMaxRows, kMaxColumns, kCellHeight, kCellWidth, kRowInset, kColumnInset,
			AdornRows, AdornCols,
			multipleSelection,
			plain,
			kFontSize, { 0x0, 0x0, 0x0 }, "a" },
	/* Columns View */
		'WIND', 'SCL2',
		{ 35, 32 }, { kCellHeight, 400 }, sizeFixed, sizeRelSuperView, shown, enabled,
		Scroller {
			"TSecondaryScroller",
			noVertScrollBar, noHorzScrollBar,
			0, 0, 0, 0,
			noVertConstrain, noHorzConstrain,
			noInset },
		'SCL2', 'COLS',
		{ 0, 0 }, { kCellHeight, 400 }, sizeVariable, sizeVariable, shown, enabled,
		TextGridView {
			"TColumnsView",
			1, kMaxColumns, kColumnTitleHeight, kCellWidth, 0, kColumnInset,
			dontAdornRows, AdornCols,
			multipleSelection,
			bold,
			kFontSize, { 0x0, 0x0, 0x0 }, "a" },
			
	/* Rows View */
		'WIND', 'SCL3',
		{ 52, 0 }, { 221, 32 }, sizeRelSuperView, sizeFixed, shown, enabled,
		Scroller {
			"TSecondaryScroller",
			noVertScrollBar, noHorzScrollBar,
			0, 0, 0, 0,
			noVertConstrain, noHorzConstrain,
			noInset },
		'SCL3', 'ROWS',
		{ 0, 0 }, { 237, 32 }, sizeVariable, sizeVariable, shown, enabled,
		TextGridView {
			"TRowsView",
			kMaxRows, 1, kCellHeight, kRowTitleWidth, kRowInset, 0,
			AdornRows, dontAdornCols,
			multipleSelection,
			bold,
			kFontSize, { 0x0, 0x0, 0x0 }, "a" },
			
	/* Cell Entry View */
		'WIND', 'ENTV',
		{ 8, 64 }, { 20, 364 }, sizeFixed, sizeFixed, shown, enabled,
		TEView {
			"TEntryView",
			withoutStyle, crOnly, acceptChanges, dontFreeText, cTyping,
			kMaxChars, {2, 2, 2, 2}, justSystem, plain,
			kFontSize, { 0x0, 0x0, 0x0 }, "a"
		}
	}
};

/* Balloon Help for cells view */
resource 'hrct' (kCellsViewHelp) 
{
	HelpMgrVersion,
	hmDefaultOptions,
	0, /* balloon definition function */
	5, /* variation code */
	{
		HMStringItem 
		{
			{ -1,0 }, /* tip location */
			{ 0,0,0,0 }, /* hot rectangle */
			"This is the cells view." 
		}
	}
};

/* Balloon Help for columns view */
resource 'hrct' (kColsViewHelp) 
{
	HelpMgrVersion,
	hmDefaultOptions,
	0, /* balloon definition function */
	5, /* variation code */
	{
		HMStringItem 
		{
			{ -1,0 }, /* tip location */
			{ 0,0,0,0 }, /* hot rectangle */
			"Selecting a labelled box selects the entire column of cells below it." 
		}
	}
};

/* Balloon Help for columns view */
resource 'hrct' (kRowsViewHelp) 
{
	HelpMgrVersion,
	hmDefaultOptions,
	0, /* balloon definition function */
	5, /* variation code */
	{
		HMStringItem 
		{
			{ -1,0 }, /* tip location */
			{ 0,0,0,0 }, /* hot rectangle */
			"Selecting a numbered box selects the entire row of cells alongside it" 
		}
	}
};


/*--------------------------------------------------------------------------------
 Splash screen
--------------------------------------------------------------------------------*/
resource 'DITL' (phSplash,
#if qNames
"phSplash",
#endif
	purgeable, preload) {
	{	/* array DITLarray: 1 elements */
		/* [1] */
		{1, 1, 453-261+1, 490-101+1},
		Picture {
			disabled,
			phSplash
		}
	}
};

resource 'DLOG' (phSplash,
#if qNames
"phSplash",
#endif
	purgeable, preload) {
	{76, 80, 453-261+76+2, 490-101+80+2},
	altDBoxProc,
	visible,
	noGoAway,
	0,
	phSplash,
	""
#if SystemSevenOrLater
	,
	centerMainScreen
#endif
};

#Include "Pict.r"

resource 'PICT' (phSplash, purgeable, preload) {
	{0, 0, 453-261, 490-101},
	VersionOne {
		{	/* array OpCodes: 4 elements */
			/* [1] */
			shortComment {
				130
			},
			/* [2] */
			clipRgn {
				{0, 0, 720, 576},
				$""
			},
			/* [3] */
			packBitsRect {
				50,
				{0, 0, 453-261, 496-96},
				{0, 0, 453-261, 490-101},
				{0, 0, 453-261, 490-101},
				srcCopy,
				$"02CF 0002 CF00 02CF 0002 CF00 02CF 0002"
				$"CF00 02CF 0002 CF00 02CF 0002 CF00 1C18"
				$"0000 0FBE FBEF BEFB EFBE FBEF BEFB EFBE"
				$"FBEF BEFB EFBE FBEF 80E8 001C 1800 000F"
				$"BEFB EFBE FBEF BEFB EFBE FBEF BEFB EFBE"
				$"FBEF BEFB EF80 E800 1C18 0000 0FBE FBEF"
				$"BEFB EFBE FBEF BEFB EFBE FBEF BEFB EFBE"
				$"FBEF 80E8 001C 1800 000F BEFB EFBE FBEF"
				$"BEFB EFBE FBEF BEFB EFBE FBEF BEFB EF80"
				$"E800 1C18 0000 0FBE FBEF BEFB EFBE FBEF"
				$"BEFB EFBE FBEF BEFB EFBE FBEF 80E8 0010"
				$"E400 05FF 80C0 0000 78F8 0001 FF80 FD00"
				$"2103 0000 0F80 EE00 010F 80FE 000B 07FF"
				$"F9C0 0000 7800 0007 FFFC FE00 0207 FFF9"
				$"FD00 2103 0000 0F80 EE00 010F 80FE 0006"
				$"1FC0 FFC0 0000 FCFE 0001 FFE0 FE00 021F"
				$"C0FF FD00 2103 0000 0F80 EE00 010F 80FE"
				$"0006 7F00 3FC0 0000 FCFE 0001 7FC0 FE00"
				$"027F 003F FD00 2103 0000 0F80 EE00 010F"
				$"80FE 0006 FC00 0FC0 0000 FCFE 0001 3F80"
				$"FE00 02FC 000F FD00 2103 0000 0F80 EE00"
				$"0B0F 8000 0001 F800 07C0 0001 FEFE 0007"
				$"3F80 0000 01F8 0007 FD00 18E6 0007 03F0"
				$"0003 E000 01FE FE00 073F 8000 0003 F000"
				$"03FD 0021 0300 000F 80EE 000B 0F80 0000"
				$"07E0 0001 E000 03FF FE00 073F 8000 0007"
				$"E000 01FD 001F 0300 000F 80EE 000B 0F80"
				$"0000 0FE0 0000 E000 03FF FE00 053F 8000"
				$"000F E0FB 001F 0300 000F 80EE 0014 0F80"
				$"0000 0FC0 0000 E000 03FF 8000 003F 8000"
				$"000F C0FB 001F 0300 000F 80EE 0014 0F80"
				$"0000 1FC0 0000 6000 07BF 8000 003F 8000"
				$"001F C0FB 001E 0300 000F 80EE 0005 0F80"
				$"0000 1F80 FD00 0A07 BF80 0000 3F80 0000"
				$"1F80 FB00 15E6 0001 3F80 FD00 0A0F 3FC0"
				$"0000 3F80 0000 3F80 FB00 1E03 0000 0F80"
				$"EE00 050F 8000 003F 80FD 000A 0F1F C000"
				$"003F 8000 003F 80FB 001E 0300 000F 80EE"
				$"0005 0F80 0000 3F80 FD00 0A1E 1FE0 0000"
				$"3F80 0000 3F80 FB00 1C03 0000 0F80 EE00"
				$"040F 8000 007F FC00 091E 0FE0 0000 3F80"
				$"0000 7FFA 001C 0300 000F 80EE 0004 0F80"
				$"0000 7FFC 0009 3C0F F000 003F 8000 007F"
				$"FA00 1C03 0000 0F80 EE00 040F 8000 007F"
				$"FC00 093C 07F0 0000 3F80 0000 7FFA 0013"
				$"E600 007F FC00 0938 07F0 0000 3F80 0000"
				$"7FFA 001C 0300 000F 80EE 0004 0F80 0000"
				$"7FFC 0009 7803 F800 003F 8000 007F FA00"
				$"1C03 0000 0F80 EE00 040F 8000 007F FC00"
				$"0978 03F8 0000 3F80 0000 7FFA 001C 0300"
				$"000F 80EE 0004 0F80 0000 7FFC 0009 F001"
				$"FC00 003F 8000 007F FA00 1C03 0000 0F80"
				$"EE00 040F 8000 007F FC00 09F0 01FC 0000"
				$"3F80 0000 7FFA 001D 0300 000F 80EE 0004"
				$"0F80 0000 7FFD 000A 01FF FFFE 0000 3F80"
				$"0000 7FFA 0016 E600 013F 80FE 000B 01FF"
				$"FFFE 0000 3F80 0000 3F80 FB00 2F1C 0000"
				$"0F80 FBEF BEFB EFBE FBEF BEFB EFBE FBEF"
				$"BEFB EFBE F80F 8000 003F 80FE 000B 03E0"
				$"00FE 0000 3F80 0000 3F80 FB00 2F1C 0000"
				$"0F80 FBEF BEFB EFBE FBEF BEFB EFBE FBEF"
				$"BEFB EFBE F80F 8000 003F 80FE 000B 03C0"
				$"00FF 0000 3F80 0000 3F80 FB00 2F1C 0000"
				$"0F80 FBEF BEFB EFBE FBEF BEFB EFBE FBEF"
				$"BEFB EFBE F80F 8000 001F C0FE 000B 03C0"
				$"007F 0000 3F80 0018 1FC0 FB00 2F2B 0000"
				$"0F80 FBEF BEFB EFBE FBEF BEFB EFBE FBEF"
				$"BEFB EFBE F80F 8000 001F C000 0030 0780"
				$"007F 8000 3F80 0018 1FC0 FB00 2F2B 0000"
				$"0F80 FBEF BEFB EFBE FBEF BEFB EFBE FBEF"
				$"BEFB EFBE F80F 8000 000F E000 0070 0780"
				$"003F 8000 3F80 0038 0FE0 FB00 16E6 0010"
				$"07E0 0000 E00F 0000 3FC0 003F 8000 3807"
				$"E0FB 0031 2D00 000F 8000 0F80 000F 8000"
				$"0F80 000F 8000 0F80 000F 8000 0F80 0000"
				$"07F0 0001 E00F 0000 3FC0 003F 8000 7007"
				$"F000 01FD 0031 2D00 000F 8000 0F80 000F"
				$"8000 0F80 000F 8000 0F80 000F 8000 0F80"
				$"0000 03F8 0003 C01E 0000 1FE0 003F 8000"
				$"F003 F800 03FD 0031 2D00 000F 8000 0F80"
				$"000F 8000 0F80 000F 8000 0F80 000F 8000"
				$"0F80 0000 01FE 0007 801E 0000 1FE0 003F"
				$"8003 F001 FE00 07FD 0031 1800 000F 8000"
				$"0F80 000F 8000 0F80 000F 8000 0F80 000F"
				$"8000 0F80 FE00 11FF 001F 003E 0000 1FF0"
				$"007F FFFF E000 FF00 1FFD 0031 1800 000F"
				$"8000 0F80 000F 8000 0F80 000F 8000 0F80"
				$"000F 8000 0F80 FE00 093F E0FE 007F 0000"
				$"3FF8 00FE FF04 E000 3FE0 FEFD 0017 E500"
				$"090F FFF8 01FF E001 FFFE 07FE FF04 E000"
				$"0FFF F8FD 0028 1800 000F 8000 0F80 000F"
				$"8000 0F80 000F 8000 0F80 000F 8000 0F80"
				$"FE00 0201 FFC0 F500 0201 FFC0 FD00 1C18"
				$"0000 0F80 000F 8000 0F80 000F 8000 0F80"
				$"000F 8000 0F80 000F 80E8 001C 1800 000F"
				$"8000 0F80 000F 8000 0F80 000F 8000 0F80"
				$"000F 8000 0F80 E800 1C18 0000 0F80 000F"
				$"8000 0F80 000F 8000 0F80 000F 8000 0F80"
				$"000F 80E8 001C 1800 000F 8000 0F80 000F"
				$"8000 0F80 000F 8000 0F80 000F 8000 0F80"
				$"E800 02CF 0022 1800 000F 8000 0F80 000F"
				$"8000 0F80 000F 8000 0F80 000F 8000 0F80"
				$"F400 0218 000C F800 2618 0000 0F80 000F"
				$"8000 0F80 000F 8000 0F80 000F 8000 0F80"
				$"000F 80F4 0002 7800 3CFD 0000 04FD 0026"
				$"1800 000F 8000 0F80 000F 8000 0F80 000F"
				$"8000 0F80 000F 8000 0F80 F400 0218 000C"
				$"FD00 000C FD00 2618 0000 0F80 000F 8000"
				$"0F80 000F 8000 0F80 000F 8000 0F80 000F"
				$"80F4 0002 1800 0CFD 0000 0CFD 0026 1800"
				$"000F 8000 0F80 000F 8000 0F80 000F 8000"
				$"0F80 000F 8000 0F80 F400 0218 000C FD00"
				$"000C FD00 18E6 0012 0F80 01F0 C781 9C3E"
				$"01F0 0FD8 1F0C 700F 807C 0CFD 0032 2E00"
				$"000F 80FB EFBE FBEF BEFB EFBE FBEF BEFB"
				$"EFBE FBEF BEF8 0F80 0000 39C0 0633 DFE7"
				$"BCE7 8738 38F8 630D F839 E1CF 3F80 FE00"
				$"312D 0000 0F80 FBEF BEFB EFBE FBEF BEFB"
				$"EFBE FBEF BEFB EFBE F80F 8000 0020 E00C"
				$"10E0 E1C4 C184 1C30 38C1 0E1C 3061 830C"
				$"FD00 3118 0000 0F80 FBEF BEFB EFBE FBEF"
				$"BEFB EFBE FBEF BEFB EFBE F80F 80FE 0011"
				$"600C 00C0 7181 81C0 0C60 18C0 0C0C 6073"
				$"038C FD00 3118 0000 0F80 FBEF BEFB EFBE"
				$"FBEF BEFB EFBE FBEF BEFB EFBE F80F 80FE"
				$"0011 E00F 00C0 3181 80C0 1C60 18F0 0C0C"
				$"6033 018C FD00 312D 0000 0F80 FBEF BEFB"
				$"EFBE FBEF BEFB EFBE FBEF BEFB EFBE F80F"
				$"8000 0007 E007 E0C0 3181 FFC0 FC60 187E"
				$"0C0C 7FF3 FF8C FD00 18E6 0012 3C60 01F0"
				$"C031 8180 078C 6018 1F0C 0C60 0300 0CFD"
				$"0031 2D00 000F 8000 0F80 000F 8000 0F80"
				$"000F 8000 0F80 000F 8000 0F80 0000 7060"
				$"0038 C031 8180 0E0C 6018 038C 0C60 0300"
				$"0CFD 0031 2D00 000F 8000 0F80 000F 8000"
				$"0F80 000F 8000 0F80 000F 8000 0F80 0000"
				$"6060 0018 C031 81C0 0C0C 7018 018C 0C70"
				$"0380 0CFD 0031 2D00 000F 8000 0F80 000F"
				$"8000 0F80 000F 8000 0F80 000F 8000 0F80"
				$"0000 60E0 0818 E061 80E0 CC1C 3838 818C"
				$"0C38 31C1 8CFD 0032 2E00 000F 8000 0F80"
				$"000F 8000 0F80 000F 8000 0F80 000F 8000"
				$"0F80 0000 7360 0C30 F8E1 80FB 8E6C 3FD8"
				$"C30C 0C3E E1F7 0F80 FE00 312D 0000 0F80"
				$"000F 8000 0F80 000F 8000 0F80 000F 8000"
				$"0F80 000F 8000 003E 700F E0DF 83C0 3E07"
				$"CE0F 1CFE 1E0E 0F80 7C07 FD00 06E2 0000"
				$"C0EF 001C 0C00 000F 8000 0F80 000F 8000"
				$"0F80 FA00 040F 8000 0F80 FB00 00C0 EF00"
				$"1C0C 0000 0F80 000F 8000 0F80 000F 80FA"
				$"0004 0F80 000F 80FB 0000 C0EF 001C 0C00"
				$"000F 8000 0F80 000F 8000 0F80 FA00 040F"
				$"8000 0F80 FB00 00C0 EF00 1C0C 0000 0F80"
				$"000F 8000 0F80 000F 80FA 0004 0F80 000F"
				$"80FB 0000 C0EF 001C 0C00 000F 8000 0F80"
				$"000F 8000 0F80 FA00 040F 8000 0F80 FB00"
				$"00C0 EF00 07E3 0001 03F0 EF00 1C13 0000"
				$"0F80 000F 8000 0F80 000F 8000 003E FBEF"
				$"BEF8 FE00 010F 80E8 0021 1300 000F 8000"
				$"0F80 000F 8000 0F80 0000 3EFB EFBE F8FE"
				$"0001 0F80 F800 0101 80F3 0027 1300 000F"
				$"8000 0F80 000F 8000 0F80 0000 3EFB EFBE"
				$"F8FE 0001 0F80 F800 0107 80FC 0002 0C10"
				$"10FB 0027 1300 000F 8000 0F80 000F 8000"
				$"0F80 0000 3EFB EFBE F8FE 0001 0F80 F800"
				$"0101 80FC 0002 0C30 30FB 0026 1300 000F"
				$"8000 0F80 000F 8000 0F80 0000 3EFB EFBE"
				$"F8FE 0001 0F80 F800 0101 80FB 0001 3030"
				$"FB00 0CDF 0001 0180 FB00 0130 30FB 0032"
				$"0F00 000F 80FB EFBE FBEF BEFB EFBE 000F"
				$"80FE 001B 03E0 00F8 0F80 0000 0F81 F031"
				$"C1C0 C781 81F0 03C1 8399 CC30 300F 818E"
				$"FE00 320F 0000 0F80 FBEF BEFB EFBE FBEF"
				$"BE00 0F80 FE00 1B03 E000 F80F 8000 0031"
				$"8738 F7E7 E3DF E187 3C01 8181 7BFC FEFE"
				$"39E7 BFFE 0033 0F00 000F 80FB EFBE FBEF"
				$"BEFB EFBE 000F 80FE 0016 03E0 00F8 0F80"
				$"0000 6084 1C38 7870 E0E1 860C 0183 831C"
				$"4CFE 3002 61C3 80FF 0033 0F00 000F 80FB"
				$"EFBE FBEF BEFB EFBE 000F 80FE 000A 03E0"
				$"00F8 0F80 0000 6000 0CFE 300E C071 8C0E"
				$"00C2 C218 0C30 3060 7181 80FF 0033 0F00"
				$"000F 80FB EFBE FBEF BEFB EFBE 000F 80FE"
				$"000A 03E0 00F8 0F80 0000 7800 1CFE 300E"
				$"C031 8C06 00C2 C218 0C30 3060 3181 80FF"
				$"001A E600 023F 00FC FE30 0EC0 318F FE00"
				$"C6C6 180C 3030 7FF1 8180 FF00 310E 0000"
				$"0F80 000F 8000 0F80 000F 8003 E0FC 0009"
				$"0F80 000F 8000 000F 878C FE30 0EC0 318C"
				$"0000 6464 180C 3030 6001 8180 FF00 310E"
				$"0000 0F80 000F 8000 0F80 000F 8003 E0FC"
				$"0009 0F80 000F 8000 0001 CE0C FE30 0EC0"
				$"318C 0000 6464 180C 3030 6001 8180 FF00"
				$"310E 0000 0F80 000F 8000 0F80 000F 8003"
				$"E0FC 0004 0F80 000F 80FE 0001 CC0C FE30"
				$"0EC0 318E 0000 6C6C 180C 3030 7001 8180"
				$"FF00 310E 0000 0F80 000F 8000 0F80 000F"
				$"8003 E0FC 0009 0F80 000F 8000 0040 CC1C"
				$"FE30 0EE0 6187 0600 3838 180C 3030 3831"
				$"8180 FF00 310E 0000 0F80 000F 8000 0F80"
				$"000F 8003 E0FC 0009 0F80 000F 8000 0061"
				$"8E6C FE30 08F8 E187 DC00 3838 180C FE3E"
				$"02E1 8180 FF00 1AE6 0014 7F07 CE78 7838"
				$"DF83 C1F0 0010 103C 1E1C 1C0F 83C1 C0FF"
				$"001F 1000 000F 8000 0F80 000F 8000 0F80"
				$"F800 3EF8 FD00 033E 000F 80F9 0000 C0F1"
				$"001F 1000 000F 8000 0F80 000F 8000 0F80"
				$"F800 3EF8 FD00 033E 000F 80F9 0000 C0F1"
				$"001F 1000 000F 8000 0F80 000F 8000 0F80"
				$"F800 3EF8 FD00 033E 000F 80F9 0000 C0F1"
				$"001F 1000 000F 8000 0F80 000F 8000 0F80"
				$"F800 3EF8 FD00 033E 000F 80F9 0000 C0F1"
				$"001F 1000 000F 8000 0F80 000F 8000 0F80"
				$"F800 3EF8 FD00 033E 000F 80F9 0000 C0F1"
				$"0006 E000 00C0 F100 1E09 0000 0F80 000F"
				$"8000 0F80 FE00 04FB EF80 03E0 FD00 00F8"
				$"F800 0103 F0F1 0019 0900 000F 8000 0F80"
				$"000F 80FE 0004 FBEF 8003 E0FD 0000 F8E6"
				$"002B 0900 000F 8000 0F80 000F 80FE 0004"
				$"FBEF 8003 E0FD 0000 F8F9 0005 1800 01E0"
				$"0070 FD00 0080 FD00 0101 F0FF 002E 0900"
				$"000F 8000 0F80 000F 80FE 0004 FBEF 8003"
				$"E0FD 0000 F8FB 0007 0608 7800 0070 00E0"
				$"FE00 0101 C0FD 0001 060C FF00 2E09 0000"
				$"0F80 000F 8000 0F80 FE00 04FB EF80 03E0"
				$"FD00 00F8 FB00 0706 1818 0000 7000 E0FE"
				$"0001 01C0 FD00 0108 02FF 0016 E300 0618"
				$"1800 0078 01E0 FE00 0103 E0FD 0001 17E1"
				$"FF00 3119 0000 0F80 FBEF BEFB EFBE 03EF"
				$"BE00 0FBE FBEF BEFB E000 03EF BEF8 FD00"
				$"0618 1800 0058 0160 FE00 0102 60FD 0003"
				$"2330 8000 3331 0000 0F80 FBEF BEFB EFBE"
				$"03EF BE00 0FBE FBEF BEFB E000 03EF BEF8"
				$"00F0 60E6 1818 E000 5C03 607C 03F0 0670"
				$"0C78 18F0 2330 8000 3331 0000 0F80 FBEF"
				$"BEFB EFBE 03EF BE00 0FBE FBEF BEFB E000"
				$"03EF BEF8 0060 605E 7F1B F000 4C02 61CE"
				$"0E38 0430 3DFE 7BFC 4330 4000 3331 0000"
				$"0F80 FBEF BEFB EFBE 03EF BE00 0FBE FBEF"
				$"BEFB E000 03EF BEF8 0060 E0C6 181C 3800"
				$"4E06 6107 0C18 0C38 0E0E 1C1C 43E0 4000"
				$"331D 0000 0F80 FBEF BEFB EFBE 03EF BE00"
				$"0FBE FBEF BEFB E000 03EF BEF8 0030 B086"
				$"FE18 1000 4604 6003 1808 0818 0C07 180E"
				$"4360 4000 1AE6 0002 30B0 86FE 1810 0047"
				$"0C60 0718 0018 1C0C 0318 0643 3040 0030"
				$"1000 000F 8000 0F80 000F 8000 003E 0000"
				$"3EF8 FB00 060F BEF8 0031 B186 FE18 1000"
				$"4308 603F 1800 1FFC 0C03 1806 4330 4000"
				$"3010 0000 0F80 000F 8000 0F80 0000 3E00"
				$"003E F8FB 0006 0FBE F800 1919 06FE 1810"
				$"0043 9861 E318 0030 0E0C 0318 0623 1880"
				$"0030 1000 000F 8000 0F80 000F 8000 003E"
				$"0000 3EF8 FB00 060F BEF8 0019 1906 FE18"
				$"1000 4190 6383 1800 2006 0C03 1806 279C"
				$"8000 3010 0000 0F80 000F 8000 0F80 0000"
				$"3E00 003E F8FB 0006 0FBE F800 1B1B 06FE"
				$"180E 0041 F063 031C 0060 070C 0318 0610"
				$"01FF 0030 1000 000F 8000 0F80 000F 8000"
				$"003E 0000 3EF8 FB00 060F BEF8 000E 0E06"
				$"FE18 0E00 40E0 6307 0E0C 4003 0E06 1C0C"
				$"0802 FF00 1AE6 0014 0E0E 061F 1818 0040"
				$"E063 9B0F B8C0 038F 8E1F 1C06 0CFF 002D"
				$"0900 000F 8000 0F80 000F 80FE 0000 F8F8"
				$"0018 0FBE F800 0404 0F0E 3C1C 01F0 41F1"
				$"F383 E1E0 07CD F81B F001 F0FF 001D 0900"
				$"000F 8000 0F80 000F 80FE 0000 F8F8 0002"
				$"0FBE F8F1 0002 0C00 18FC 001D 0900 000F"
				$"8000 0F80 000F 80FE 0000 F8F8 0002 0FBE"
				$"F8F1 0002 0C00 18FC 001D 0900 000F 8000"
				$"0F80 000F 80FE 0000 F8F8 0002 0FBE F8F1"
				$"0002 0C00 18FC 001D 0900 000F 8000 0F80"
				$"000F 80FE 0000 F8F8 0002 0FBE F8F1 0002"
				$"0C00 18FC 0008 D700 020C 0018 FC00 1E0E"
				$"0000 0F80 000F 8000 0F80 000F 8003 E0F9"
				$"0002 0FBE F8F1 0002 0C00 18FC 001E 0E00"
				$"000F 8000 0F80 000F 8000 0F80 03E0 F900"
				$"020F BEF8 F100 023F 007E FC00 180E 0000"
				$"0F80 000F 8000 0F80 000F 8003 E0F9 0002"
				$"0FBE F8E9 0018 0E00 000F 8000 0F80 000F"
				$"8000 0F80 03E0 F900 020F BEF8 E900 180E"
				$"0000 0F80 000F 8000 0F80 000F 8003 E0F9"
				$"0002 0FBE F8E9 0002 CF00 190F 0000 0F80"
				$"FBEF BEFB EFBE FBEF BE00 0FBE FA00 020F"
				$"BEF8 E900 190F 0000 0F80 FBEF BEFB EFBE"
				$"FBEF BE00 0FBE FA00 020F BEF8 E900 190F"
				$"0000 0F80 FBEF BEFB EFBE FBEF BE00 0FBE"
				$"FA00 020F BEF8 E900 190F 0000 0F80 FBEF"
				$"BEFB EFBE FBEF BE00 0FBE FA00 020F BEF8"
				$"E900 190F 0000 0F80 FBEF BEFB EFBE FBEF"
				$"BE00 0FBE FA00 020F BEF8 E900 02CF 001D"
				$"0C00 000F 8000 0F80 000F 8000 0F80 FE00"
				$"09FB EFBE FBEF BE00 0FBE F8E9 001D 0C00"
				$"000F 8000 0F80 000F 8000 0F80 FE00 09FB"
				$"EFBE FBEF BE00 0FBE F8E9 001D 0C00 000F"
				$"8000 0F80 000F 8000 0F80 FE00 09FB EFBE"
				$"FBEF BE00 0FBE F8E9 001D 0C00 000F 8000"
				$"0F80 000F 8000 0F80 FE00 09FB EFBE FBEF"
				$"BE00 0FBE F8E9 001D 0C00 000F 8000 0F80"
				$"000F 8000 0F80 FE00 09FB EFBE FBEF BE00"
				$"0FBE F8E9 0002 CF00 1A0F 0000 0F80 000F"
				$"8000 0F80 000F 8000 0F80 FB00 03FB EFBE"
				$"F8E9 001A 0F00 000F 8000 0F80 000F 8000"
				$"0F80 000F 80FB 0003 FBEF BEF8 E900 1A0F"
				$"0000 0F80 000F 8000 0F80 000F 8000 0F80"
				$"FB00 03FB EFBE F8E9 001A 0F00 000F 8000"
				$"0F80 000F 8000 0F80 000F 80FB 0003 FBEF"
				$"BEF8 E900 1A0F 0000 0F80 000F 8000 0F80"
				$"000F 8000 0F80 FB00 03FB EFBE F8E9 0002"
				$"CF00 0D03 0000 0F80 EE00 020F BEF8 E900"
				$"0D03 0000 0F80 EE00 020F BEF8 E900 0D03"
				$"0000 0F80 EE00 020F BEF8 E900 0D03 0000"
				$"0F80 EE00 020F BEF8 E900 0D03 0000 0F80"
				$"EE00 020F BEF8 E900 02CF 001A 1600 000F"
				$"BEFB EFBE FBEF BEFB EFBE FBEF BEFB EFBE"
				$"FBEF BEF8 E600 1A16 0000 0FBE FBEF BEFB"
				$"EFBE FBEF BEFB EFBE FBEF BEFB EFBE F8E6"
				$"001A 1600 000F BEFB EFBE FBEF BEFB EFBE"
				$"FBEF BEFB EFBE FBEF BEF8 E600 1A16 0000"
				$"0FBE FBEF BEFB EFBE FBEF BEFB EFBE FBEF"
				$"BEFB EFBE F8E6 001A 1600 000F BEFB EFBE"
				$"FBEF BEFB EFBE FBEF BEFB EFBE FBEF BEF8"
				$"E600 02CF 0002 CF00 02CF 0002 CF00 02CF"
				$"0002 CF00 2C03 0000 01D0 FE00 0A40 0800"
				$"0F00 1000 0080 01D0 FB00 0007 FE00 0C10"
				$"60E3 E018 1000 0408 8004 0080 FA00 0180"
				$"002F 0300 0006 30FD 0009 1810 30C0 1000"
				$"0180 0630 FE00 0302 0000 02FE 000C 3091"
				$"1200 2430 0004 1980 0001 81FB 0002 0180"
				$"002E 0300 0004 10FD 0009 0810 2040 2800"
				$"0080 0410 FE00 0302 0000 02FE 000C 1109"
				$"1200 4210 000A 0880 0000 81FA 0001 8000"
				$"3331 0000 0801 879D F643 AB38 4720 283C"
				$"788E 0801 8D98 F337 39B0 026C 7000 1108"
				$"A380 4210 000A 0883 643A B39C 3671 CE6D"
				$"DDC7 8000 3331 0000 0802 4C48 9AC4 4C90"
				$"4920 4462 C491 0802 4665 8912 44D0 0232"
				$"9800 1108 4040 4210 0011 0881 AC44 C924"
				$"1A8A 5134 8A28 8000 3331 0000 0804 2448"
				$"9044 4890 4820 4422 4491 0804 2444 8912"
				$"4480 0222 8000 1098 A020 2610 0011 0881"
				$"0444 8920 108A 1120 8A28 8000 3331 0000"
				$"0804 2445 1044 4890 4720 7C22 449F 0804"
				$"2444 8912 7C80 0222 8000 1069 102F 1A10"
				$"001F 0881 0444 8918 10F9 9F20 53E8 8000"
				$"3331 0000 0414 2445 1043 8890 2040 8222"
				$"4490 0414 2444 8912 4080 0222 8000 1011"
				$"1020 0410 0020 8881 0438 8904 1080 5020"
				$"5208 8000 3331 0000 0622 4442 1044 0894"
				$"30C0 8222 4491 0622 4444 8932 C480 0222"
				$"8800 1031 1240 0C10 0020 8881 0440 8964"
				$"108A 5120 2228 8000 3331 0000 01C1 8782"
				$"38E7 9DC8 0F01 C73C 79CE 01C1 8EEE F0D9"
				$"39C2 0777 7100 38C0 E180 3038 8071 DDC3"
				$"8E79 DCB8 3873 8E70 21C7 4800 1AFD 0004"
				$"0404 0008 40FD 0001 2040 FC00 0380 0000"
				$"02F2 0000 84F7 001A FD00 0404 0400 0840"
				$"FD00 0120 40FC 0003 8000 0002 F200 0084"
				$"F700 1BFD 0004 0E18 0007 80FD 0001 70E0"
				$"FD00 0401 C000 0004 F200 0078 F700 02CF"
				$"0002 CF00 02CF 0002 CF00 02CF 0002 CF00"
				$"02CF 0002 CF00"
			},
			/* [4] */
			shortComment {
				131
			}
		}
	}
};


#if qDebug
resource 'dbug' (kDebugParamsID,
#if qNames
"Debug",
#endif
	purgeable) {
		normal,							// Bounding rect for debug window
		4, 								// Debug window font rsrc ID (monaco)
		normal,				 			// Debug window font size
		60,								// Number of lines
		90,								// Width of lines in characters
		openInitially,					// show window at startup
		"Calc Debug Transcript"			// Window title
};
#endif

include "Defaults.rsrc"  'STR#' (kDefaultCredits);		// Grab the default credits

// Icons, Bundles and FRefsOh my! (don't forget the Signature)
resource 'ICN#' (128,
#if qNames
"Application",
#endif
	purgeable) {
	{ /* array: 2 elements */
		/* [1] */
		$"00 00 00 00 00 00 00 00 7F FF FF FC 40 00 00 04"
		$"40 00 00 04 40 00 00 04 40 00 00 04 5F FF FF F4"
		$"44 44 44 44 44 44 44 44 44 44 44 44 5F FF FF F4"
		$"44 44 44 44 44 44 40 44 44 44 3F 04 5F FF 40 B4"
		$"44 44 80 44 44 45 30 24 44 41 C8 14 5F EE 7F 8F"
		$"44 42 30 07 44 45 00 07 44 44 80 07 5F FF 60 07"
		$"44 44 1F E7 44 44 40 1F 40 00 00 07 7F FF FF FC",
		/* [2] */
		$"00 00 00 00 00 00 00 00 7F FF FF FC 7F FF FF FC"
		$"7F FF FF FC 7F FF FF FC 7F FF FF FC 7F FF FF FC"
		$"7F FF FF FC 7F FF FF FC 7F FF FF FC 7F FF FF FC"
		$"7F FF FF FC 7F FF FF FC 7F FF FF FC 7F FF FF FC"
		$"7F FF FF FC 7F FF FF FC 7F FF FF FC 7F FF FF FF"
		$"7F FF FF FF 7F FF FF FF 7F FF FF FF 7F FF FF FF"
		$"7F FF FF FF 7F FF FF FF 7F FF FF FF 7F FF FF FC"
	}
};

resource 'ICN#' (129,
#if qNames
"Document",
#endif
	purgeable) {
	{ /* array: 2 elements */
		/* [1] */
		$"0F FF FE 00 08 00 03 00 08 00 02 80 08 80 02 40"
		$"08 80 02 20 08 80 02 10 0F F0 03 F8 08 80 00 08"
		$"08 80 00 08 08 80 00 08 08 00 00 08 08 00 00 08"
		$"08 0F E0 08 08 00 00 08 08 00 00 08 08 00 00 08"
		$"08 00 10 08 08 00 54 08 08 00 38 08 08 00 FE 08"
		$"08 00 38 08 08 00 54 08 08 00 10 08 08 00 00 08"
		$"08 00 00 88 08 00 00 08 08 00 07 F8 08 00 00 08"
		$"08 00 00 88 08 00 00 08 08 00 00 08 0F FF FF F8",
		/* [2] */
		$"0F FF FE 00 0F FF FF 00 0F FF FF 80 0F FF FF C0"
		$"0F FF FF E0 0F FF FF F0 0F FF FF F8 0F FF FF F8"
		$"0F FF FF F8 0F FF FF F8 0F FF FF F8 0F FF FF F8"
		$"0F FF FF F8 0F FF FF F8 0F FF FF F8 0F FF FF F8"
		$"0F FF FF F8 0F FF FF F8 0F FF FF F8 0F FF FF F8"
		$"0F FF FF F8 0F FF FF F8 0F FF FF F8 0F FF FF F8"
		$"0F FF FF F8 0F FF FF F8 0F FF FF F8 0F FF FF F8"
		$"0F FF FF F8 0F FF FF F8 0F FF FF F8 0F FF FF F8"
	}
};


resource 'icl8' (128,
#if qNames
"Application",
#endif
	purgeable) {
	$"0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000"
	$"0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000"
	$"00FF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF 0000"
	$"00FF 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2BFF 0000"
	$"00FF 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2BFF 0000"
	$"00FF 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2BFF 0000"
	$"00FF 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2BFF 0000"
	$"00FF 2BEC ECEC ECEC ECEC ECEC ECEC ECEC ECEC ECEC ECEC ECEC ECEC ECEC 2BFF 0000"
	$"00FF 2B2B 2BEC D8D8 D8EC 2B2B 2BEC 2B2B 2BEC 2B2B 2BEC 2B2B 2BEC 2B2B 2BFF 0000"
	$"00FF 2B2B 2BEC D8D8 D8EC 2B2B 2BEC 2B2B 2BEC 2B2B 2BEC 2B2B 2BEC 2B2B 2BFF 0000"
	$"00FF 2B2B 2BEC 2B2B 2BEC 2B2B 2BEC 2B2B 2BEC 2B2B 2BEC 2B2B 2BEC 2B2B 2BFF 0000"
	$"00FF 2BEC ECEC ECEC ECEC ECEC ECEC ECEC ECEC ECEC ECEC ECEC ECEC ECEC 2BFF 0000"
	$"00FF 2B2B 2BEC 2B2B 2BEC 2B2B 2BEC 2B2B 2BEC 2B2B 2BEC 2B2B 2BEC 2B2B 2BFF 0000"
	$"00FF 2B2B 2BEC 2B2B 2BEC 2B2B 2BEC 2B2B 2BEC 2B2B 2B2B 2B2B 2BEC 2B2B 2BFF 0000"
	$"00FF 2B2B 2BEC 2B2B 2BEC 2B2B 2BEC 2B2B 2B2B FFFF FFFF FFFF 2B2B 2B2B 2BFF 0000"
	$"00FF 2BEC ECEC ECEC ECEC ECEC ECEC ECEC 2BFF 0808 0808 0808 FF2B ECEC 2BFF 0000"
	$"00FF 2B2B 2BEC 2B2B 2BEC 2B2B 2BEC 2B2B FF08 0808 0808 0808 08FF 2B2B 2BFF 0000"
	$"00FF 2B2B 2BEC 2B2B 2BEC 2B2B 2BEC 2BFF 0808 FFFF 0808 0808 0808 FF2B 2BFF 0000"
	$"00FF 2B2B 2BEC 2B2B 2BEC 2B2B 2B2B 2BFF FFFF 2B2B FF08 0808 0808 08FF 2BFF 0000"
	$"00FF 2BEC ECEC ECEC ECEC EC2B FFFF FF08 08FF FFFF FFFF FFFF FF08 0808 FFFF FFFF"
	$"00FF 2B2B 2BEC 2B2B 2BEC 2B2B 2B2B FF08 0808 FFFF 0808 0808 0808 0808 08FF FFFF"
	$"00FF 2B2B 2BEC 2B2B 2BEC 2B2B 2BEC 2BFF 0808 0808 0808 0808 0808 0808 08FF FFFF"
	$"00FF 2B2B 2BEC 2B2B 2BEC 2B2B 2BEC 2B2B FF08 0808 0808 0808 0808 0808 08FF FFFF"
	$"00FF 2BEC ECEC ECEC ECEC ECEC ECEC ECEC 2BFF FF08 0808 0808 0808 0808 08FF FFFF"
	$"00FF 2B2B 2BEC 2B2B 2BEC 2B2B 2BEC 2B2B 2B2B 2BFF FFFF FFFF FFFF FF08 08FF FFFF"
	$"00FF 2B2B 2BEC 2B2B 2BEC 2B2B 2BEC 2B2B 2BEC 2B2B 2B2B 2B2B 2B2B 2BFF FFFF FFFF"
	$"00FF 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2BFF FFFF"
	$"00FF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF"
};

resource 'icl8' (129,
#if qNames
"Document",
#endif
	purgeable) {
	$"0000 0000 FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FF00 0000 0000 0000 0000"
	$"0000 0000 FF2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B FFFF 0000 0000 0000 0000"
	$"0000 0000 FF2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B FFFA FF00 0000 0000 0000"
	$"0000 0000 FF2B 2B2B EC2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B FFFA FAFF 0000 0000 0000"
	$"0000 0000 FF2B 2B2B EC2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B FFFA FAFA FF00 0000 0000"
	$"0000 0000 FF2B 2B2B EC2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B FFFA FAFA FAFF 0000 0000"
	$"0000 0000 FFEC ECEC ECEC ECEC 2B2B 2B2B 2B2B 2B2B 2B2B FFFF FFFF FFFF FF00 0000"
	$"0000 0000 FF2B 2B2B EC2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B FF00 0000"
	$"0000 0000 FF2B 2B2B EC2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B FF00 0000"
	$"0000 0000 FF2B 2B2B EC2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B FF00 0000"
	$"0000 0000 FF2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B FF00 0000"
	$"0000 0000 FF2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B FF00 0000"
	$"0000 0000 FF2B 2B2B 2B2B 2B2B D8D8 D8D8 D8D8 D82B 2B2B 2B2B 2B2B 2B2B FF00 0000"
	$"0000 0000 FF2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B FF00 0000"
	$"0000 0000 FF2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B FF00 0000"
	$"0000 0000 FF2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B FF00 0000"
	$"0000 0000 FF2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2BEC 2B2B 2B2B 2B2B 2B2B FF00 0000"
	$"0000 0000 FF2B 2B2B 2B2B 2B2B 2B2B 2B2B 2BEC 2BEC 2BEC 2B2B 2B2B 2B2B FF00 0000"
	$"0000 0000 FF2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B ECEC EC2B 2B2B 2B2B 2B2B FF00 0000"
	$"0000 0000 FF2B 2B2B 2B2B 2B2B 2B2B 2B2B ECEC ECEC ECEC EC2B 2B2B 2B2B FF00 0000"
	$"0000 0000 FF2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B ECEC EC2B 2B2B 2B2B 2B2B FF00 0000"
	$"0000 0000 FF2B 2B2B 2B2B 2B2B 2B2B 2B2B 2BEC 2BEC 2BEC 2B2B 2B2B 2B2B FF00 0000"
	$"0000 0000 FF2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2BEC 2B2B 2B2B 2B2B 2B2B FF00 0000"
	$"0000 0000 FF2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B FF00 0000"
	$"0000 0000 FF2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B EC2B 2B2B FF00 0000"
	$"0000 0000 FF2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B FF00 0000"
	$"0000 0000 FF2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2BEC ECEC ECEC ECEC FF00 0000"
	$"0000 0000 FF2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B FF00 0000"
	$"0000 0000 FF2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B EC2B 2B2B FF00 0000"
	$"0000 0000 FF2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B FF00 0000"
	$"0000 0000 FF2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B 2B2B FF00 0000"
	$"0000 0000 FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FFFF FF"
};

resource 'FREF' (128,
#if qNames
"Application",
#endif
	purgeable) {
	'APPL',
	0,
	""
};

resource 'FREF' (129,
#if qNames
"Document",
#endif
	purgeable) {
	kFileType,
	1,
	""
};

resource 'BNDL' (128,
#if qNames
"Calc",
#endif
	purgeable) {
	kSignature,
	0,
	{ /* array TypeArray: 2 elements */
		/* [1] */
		'ICN#',
		{ /* array IDArray: 2 elements */
			/* [1] */
			0, 128,
			/* [2] */
			1, 129
		},
		/* [2] */
		'FREF',
		{ /* array IDArray: 2 elements */
			/* [1] */
			0, 128,
			/* [2] */
			1, 129
		}
	}
};

type kSignature as 'STR ';
resource kSignature (0,
#if qNames
"Signature",
#endif
	purgeable) {
	"Calc 3.0a2 Apple Computer, Inc. 1988-1991"
};

// Get the default Version resources
/*--------------------------------------------------------------------------------
 The revision of this particular file
--------------------------------------------------------------------------------*/
RESOURCE 'vers' (1,
#if qNames
"File Version",
#endif
	purgeable) {
	0x01,
	0x00,
	beta,
	0x0A,
	verUs,
	"3.0a2",
	"Calc 3.0a2, Apple Computer, Inc. 1988-1991"
};
include "Defaults.rsrc"  'vers' (2);		// Overall package

/********************************************************************************/
/*	End of File: Calc.r															*/
/********************************************************************************/