{$P}
{[a-,body+,h-,o=100,r+,rec+,t=4,u+,#+,j=20/57/1$,n-]}
{ UCalc.inc1.p}
{ Copyright  1985 - 1991 by Apple Computer, Inc. All rights reserved. }

USES
	Resources, Errors, USectionMgr, UViewServer;
	 
CONST
	kTextScrapType		= 'TEXT';
	kCalcScrapType		= 'CALC';

	kCellWidth			= 80;							{ default width of each cell }
	kCellHeight 		= 17;							{ default height of each cell }
	kCellHBorder		= 2;							{ pixels separating cell contents from
														 left/right edge }

	kCellFont			= applFont;
	kCellFontSize		= 10;
	kRowTitleWidth		= 32;							{ width of row titles }
	kColumnTitleHeight	= 20;							{ height of column titles }
	kTitlesFont 		= applFont; 					{ font for column/row titles }
	kTitlesFontSize 	= 10;							{ font size for column/row titles }

	kEntryFont			= applFont; 					{ default font of cell contents }
	kEntryFontSize		= 10;							{ default font size of cell contents }
	kEntryHeight		= 20;							{ height of the cell entry view }

	kCalcMode			= Automatic;					{ default calculation mode }
	kForceAutomatic 	= TRUE; 						{ force automatic calculation }
	kUpdateDependencies	= TRUE; 						{ set cell's dependents }

	kDefaultJustification = teCenter;				{ default cell justification }
	kNoJustification	= 2;							{ constant representing no justification }

	kValuePrecision 	= 18;							{ digits of precision in our values }
	kTELength			= (80 * 5); 					{ number of characters in the text edit
														 field }

	{ Command Constants }

	cPrintSelection 	= 180;
	cRecalculate		= 401;
	cAutoCalc			= 403;
	cManualCalc 		= 404;
	cGeneral			= 501;
	cNoDecimal			= 503;
	cDecimal			= 504;
	cScientific 		= 505;
	cSystemJustify		= 506;
	cForceLeftJustify	= 507;
	cRightJustify		= 508;
	cCenter 			= 509;
	cSelection			= 1000;
	cSizeColumn 		= 1001;
	cCutText			= 1103;
	cCopyText			= 1104;
	cClearText			= 1106;
	cCutCells			= 1203;
	cCopyCells			= 1204;
	cClearCells 		= 1206;
	cStandardCut		= 1303;
	cStandardCopy		= 1304;
	cStandardClear		= 1306;

	mWidthChanged		= 1002;
	mValueChanged		= 1003;

	{ Cursor Resource ID's }

	kColumnSizingCursor = 256;							{ cursor for resizing a column }
	kRowSizingCursor	= 257;							{ cursor for resizing a row--UNUSED }

TYPE
	TypeOfLine			= (NoLine,						{ for DrawLine routine}
						   SolidLine, BoldLine, VDottedLine, HDottedLine, RowSeparator,
						   ColumnSeparator);

VAR
	gGeneralFormat: 	ValueFormat;					{ SANE conversion formats }
	gDecimalFormat: 	ValueFormat;
	gNoDecimalFormat:	ValueFormat;
	gScientificFormat:	ValueFormat;
	gNoFormat:			ValueFormat;
	gDefaultFormat: 	FormatRecord;					{ default cell format }

	gColumnSeparatorPattern: Pattern;					{ patterns for separator lines }
	gRowSeparatorPattern: Pattern;

{--------------------------------------------------------------------------------------------------}
{$S AFields}

PROCEDURE FormatFields(aTitle: Str255;
					   VAR aFormat: FormatRecord;
					   PROCEDURE DoToField(fieldName: StringPtr;
										   fieldAddr: Ptr;
										   fieldType: INTEGER));

	VAR
		aString:			Str255;
		anInteger:			INTEGER;
		aStyle: 			Style;

	BEGIN
	{ Because our brain-damaged compiler won't allow you to pass the address of byte-aligned
	  fields in packed records, we have to move the fields of a FormatRecord into a temporary
	  variable, then pass the address of the temporary. }

	DoToField(AtStr(aTitle), NIL, bTitle);
	WITH aFormat DO
		BEGIN
		CASE aFormat.fStyle OF
			NoStyle:
				aString := 'NoStyle';
			General:
				aString := 'General';
			DecimalStyle:
				aString := 'DecimalStyle';
			NoDecimal:
				aString := 'NoDecimal';
			Scientific:
				aString := 'Scientific';
			OTHERWISE
				aString := 'UnknownStyle';
		END;
		DoToField(AtStr('  fStyle'), @aString, bString);

		anInteger := fDigits;
		DoToField(AtStr('  fDigits'), @anInteger, bInteger);

		CASE fJustification OF
			teFlushDefault:
				aString := 'System';
			teFlushLeft:
				aString := 'Forced Left';
			teCenter:
				aString := 'Center';
			teFlushRight:
				aString := 'Right';
			kNoJustification:
				aString := 'None';
			OTHERWISE
				aString := 'Unknown';
		END;
		DoToField(AtStr('  fJustification'), @aString, bString);

		anInteger := fFontNumber;
		DoToField(AtStr('  fFontNumber'), @anInteger, bFontName);
		anInteger := fFontSize;
		DoToField(AtStr('  fFontSize'), @anInteger, bInteger);
		aStyle := fFontStyle;
		DoToField(AtStr('  fFontStyle'), @aStyle, bStyle);
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S AReadFile}

PROCEDURE ReadCellCoordinate(aFile: TFile;
							 VAR r: RowNumber;
							 VAR c: ColumnNumber);
{ Reads a cell coordinate from a file }

	VAR
		cellCoordinate: 	Point;
		count:				Longint;

	BEGIN
	count := SIZEOF(Point);
	FailOSErr(aFile.ReadData(@cellCoordinate, count));
	r := cellCoordinate.v;
	c := cellCoordinate.h;
	END;

{***************************************************************************************************
	 T C a l c A p p l i c a t i o n
***************************************************************************************************}
{$S AInit}

PROCEDURE TCalcApplication.ICalcApplication(itsFileType: OSType);

	BEGIN
	IApplication(itsFileType);
	
	InitUSectionMgr;

	gGeneralFormat.Style := FloatDecimal;
	gGeneralFormat.digits := kValuePrecision;

	gDecimalFormat.Style := FixedDecimal;
	gDecimalFormat.digits := 2;

	gNoDecimalFormat.Style := FixedDecimal;
	gNoDecimalFormat.digits := 0;

	gScientificFormat.Style := FloatDecimal;
	gScientificFormat.digits := 2;

	gNoFormat.Style := FixedDecimal;
	gNoFormat.digits := - 1;

	gDefaultFormat.fJustification := kDefaultJustification;
	gDefaultFormat.fStyle := General;
	gDefaultFormat.fDigits := 0;
	gDefaultFormat.fFontNumber := kCellFont;
	gDefaultFormat.fFontSize := kCellFontSize;
	gDefaultFormat.fFontStyle := [];

	gColumnSeparatorPattern := gray;
	gRowSeparatorPattern := gray;

	{ Suppress Linker dead-stripping of these classes }
	IF gDeadStripSuppression THEN
		BEGIN
		IF Member(TObject(NIL), TCalcWindow) THEN;
		IF Member(TObject(NIL), TCellsView) THEN;
		IF Member(TObject(NIL), TRowsView) THEN;
		IF Member(TObject(NIL), TColumnsView) THEN;
		IF Member(TObject(NIL), TEntryView) THEN;
		IF Member(TObject(NIL), TCoordView) THEN;
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TCalcApplication.AboutToLoseControl(convertClipboard: BOOLEAN); OVERRIDE;
{ Remove Edit menu buzzwords for incoming Desk Accessory }

	BEGIN
	SetEditCmdName(cCut, cStandardCut);
	SetEditCmdName(cCopy, cStandardCopy);
	SetEditCmdName(cClear, cStandardClear);

	INHERITED AboutToLoseControl(convertClipboard);
	END;

{--------------------------------------------------------------------------------------------------}
{$S AOpen}

FUNCTION TCalcApplication.DoMakeDocument(itsCmdNumber: CmdNumber; itsFile: TFile): TDocument;

	VAR
		aCalcDocument:		TCalcDocument;
		dimensions: 		Rect;

	BEGIN
	{ Allocate and initialize the document}
	NEW(aCalcDocument);
	SetRect(dimensions, 1, 1, kMaxColumns, kMaxRows);
	aCalcDocument.ICalcDocument(itsFile,dimensions);

	DoMakeDocument := aCalcDocument;
	END;

{--------------------------------------------------------------------------------------------------}
{$S AClipboard}

FUNCTION TCalcApplication.MakeViewForAlienClipboard: TView; OVERRIDE;
 { Launch a view to represent the data found in the Clipboard at
   application start-up time, or when returning from an excursion
   to MultiFinder, or when returning from a Desk Accessory.  This
   creates a clipboard for 'CALC' scrap. }

	VAR
		calcScrap:			Handle;
		scrapOffset:		LONGINT;
		clipDocument:		TCalcDocument;
		clipView:			TCellsView;
		scrapInfo:			ScrapInfoRecord;
		r:					RowNumber;
		c:					ColumnNumber;
		i:					INTEGER;
		cellsRead:			INTEGER;
		cellCoord:			Point;
		offset: 			LONGINT;
		aRow:				TRow;
		aColumn:			TColumn;
		aCell:				TCell;
		perm:				BOOLEAN;
		fi: 				FailInfo;

{--------------------------------------------------------------------------------------------------}

	PROCEDURE HdlScrapFailure(error: OSErr;
							  message: LONGINT);

		BEGIN
		calcScrap := DisposeIfHandle(calcScrap);

		clipDocument := TCalcDocument(FreeIfObject(clipDocument));
		END;

	BEGIN
	{ Before doing anything else, make sure the scrap contains my type }
	IF GetScrap(NIL, kCalcScrapType, offset) > 0 THEN
		BEGIN
		clipDocument := NIL;							{ so failure handler knows to free it }
		calcScrap := NIL;

		CatchFailures(fi, HdlScrapFailure);

		calcScrap := NewPermHandle(0);

		perm := PermAllocation(TRUE);
		scrapOffset := GetScrap(calcScrap, kCalcScrapType, offset);
		perm := PermAllocation(perm);

	{ Only a negative result indicates an error--FailOSErr considers any non-zero result an error. }
		IF scrapOffset < 0 THEN
			FailOSErr(scrapOffset);
		scrapOffset := 0;

		ReadScrap(calcScrap, scrapOffset, @scrapInfo, SIZEOF(scrapInfo));
		FailMemError;
		NEW(clipDocument);
		clipDocument.ICalcDocument(NIL,scrapInfo.selection);
		clipDocument.DoInitialState;

		clipView := NIL;
		NEW(clipView);
		clipView.ICellsView(clipDocument, TRUE, NIL);
		clipDocument.fCellsView := clipView;

		FOR r := 1 TO clipDocument.fNoOfRows DO
			BEGIN
			NEW(aRow);
			aRow.ReadFromScrap(calcScrap, scrapOffset);
			clipDocument.AddRow(aRow);
			END;

		FOR c := 1 TO clipDocument.fNoOfColumns DO
			BEGIN
			NEW(aColumn);
			aColumn.ReadFromScrap(calcScrap, scrapOffset);
			clipDocument.AddColumn(aColumn);
			END;

		cellsRead := 0;
		FOR i := 1 TO scrapInfo.noOfCells DO
			BEGIN
			ReadScrap(calcScrap, scrapOffset, @cellCoord, SIZEOF(cellCoord));
			aCell := clipDocument.GetCell(cellCoord.v, cellCoord.h);
			aCell.ReadFromScrap(calcScrap, scrapOffset);
			cellsRead := cellsRead + 1;
			END;

{$IFC qDebug}
		IF gIntenseDebugging THEN
			BEGIN
			WRITELN('MakeViewForAlienClipboard: cellsRead=', cellsRead: 0, ', scrapInfo.noOfCells=',
					scrapInfo.noOfCells);
			IF cellsRead <> scrapInfo.noOfCells THEN
				ProgramBreak('MakeViewForAlienClipboard: Wrong number of cells');
			END;
{$ENDC}

		calcScrap := DisposeIfHandle(calcScrap);

		Success(fi);
		MakeViewForAlienClipboard := clipView;
		END
	ELSE
		MakeViewForAlienClipboard := INHERITED MakeViewForAlienClipboard;
	END;

{***************************************************************************************************
	T C a l c D o c u m e n t
***************************************************************************************************}
{$S AOpen}

PROCEDURE TCalcDocument.ICalcDocument(itsFile: TFile;dimensions: Rect);

	VAR
		r:					RowNumber;
		c:					ColumnNumber;
		aRect:				Rect;
		aRowList:			TRowList;
		aColumnList:		TColumnList;
		aCellList:			TCellList;
		aRgnDesignator:		TRgnDesignator;
		aRgnHandle:			RgnHandle;

	BEGIN
	IEditionDocument(itsFile,
						kFileType, kSignature, kCalcScrapType,
						kUsesDataFork, kUsesRsrcFork, 
						NOT kDataOpen, NOT kRsrcOpen, kSignature);

	fSavePrintInfo := TRUE; 							{ the 'print info' record of the
														 fDocPrintHandler will be written out to
														 the data fork }

	fDimensions := dimensions;
	WITH dimensions DO
		BEGIN
		fRowOffset := top - 1;
		fColumnOffset := left - 1;
		fNoOfRows := bottom - top + 1;
		fNoOfColumns := right - left + 1;
		END;
	fCalcMode := kCalcMode;
	fSelectionType := NoSelection;

	fCellsView := NIL;
	fRowsView := NIL;
	fColumnsView := NIL;
	fEntryView := NIL;
	fCoordView := NIL;

	fEditRow := 0;
	fEditColumn := 0;
	fEditCell := NIL;

	fRows := NIL;
	NEW(aRowList);
	aRowList.IRowList;
	fRows := aRowList;

	fColumns := NIL;
	NEW(aColumnList);
	aColumnList.IColumnList;
	fColumns := aColumnList;

	fCells := NIL;
	NEW(aCellList);
	aCellList.ICellList;
	fCells := aCellList;

	SetRect(aRect, 1, 1, 1, 1);
	fInUseBounds := aRect;

	{ create the user selection }
	aRgnHandle := MakeNewRgn;
	New(aRgnDesignator);
	aRgnDesignator.IRgnDesignator(SELF, aRgnHandle, TRUE);
	SetUserSelection(aRgnDesignator);
	END;

{--------------------------------------------------------------------------------------------------}
{$S AClose}

PROCEDURE TCalcDocument.Free;

	BEGIN
	FreeData;

	fCells := TCellList(FreeListIfObject(fCells));
	fRows := TRowList(FreeListIfObject(fRows));
	fColumns := TColumnList(FreeListIfObject(fColumns));

	INHERITED Free;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TCalcDocument.AddCell(theCell: TCell;
								r: RowNumber;
								c: ColumnNumber);

	VAR
		aRect:				Rect;

	PROCEDURE DoToPublisher(aPublisher: TPublisher);

		VAR
			aCell:				GridCell;

		BEGIN
		aCell.v := theCell.fRow;
		aCell.h := theCell.fColumn;
		IF PtInRgn(aCell, TRgnDesignator(aPublisher.fDesignator).fDesignation) THEN
			theCell.AddDependent(aPublisher);
		END;

	BEGIN
	WITH theCell DO
		BEGIN
		fCalcDocument := SELF;
		fRow := r;
		fColumn := c;
		END;

	fCells.Insert(theCell);
	fAllocatedCells := fAllocatedCells + 1;

	SetRect(aRect, c, r, c, r);
{$Push} {$H-}
	UnionRect(aRect, fInUseBounds, fInUseBounds);
{$Pop}

	EachPublisherDo(DoToPublisher);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TCalcDocument.AddColumn(theColumn: TColumn);

	BEGIN
	fColumns.Insert(theColumn);
	fAllocatedColumns := fAllocatedColumns + 1;

	fInUseBounds.left := Min(fInUseBounds.left, theColumn.fNumber);
	fInUseBounds.right := Max(fInUseBounds.right, theColumn.fNumber);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TCalcDocument.AddRow(theRow: TRow);

	BEGIN
	fRows.Insert(theRow);
	fAllocatedRows := fAllocatedRows + 1;

	fInUseBounds.top := Min(fInUseBounds.top, theRow.fNumber);
	fInUseBounds.bottom := Max(fInUseBounds.bottom, theRow.fNumber);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

FUNCTION TCalcDocument.CellExists(r: RowNumber;
								  c: ColumnNumber): BOOLEAN;

	VAR
		aCell:				TCell;

	BEGIN
	aCell := fCells.GetCell(r, c);
	CellExists := (aCell <> NIL) & (NOT aCell.fDeleted);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

FUNCTION TCalcDocument.CellInRange(r: INTEGER;
								   c: INTEGER;
								   range: Rect): BOOLEAN;

	BEGIN
	WITH range DO
		CellInRange := (r >= top) & (r <= bottom) & (c >= left) & (c <= right);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

FUNCTION TCalcDocument.ColumnExists(c: ColumnNumber): BOOLEAN;

	BEGIN
	ColumnExists := fColumns.GetColumn(c) <> NIL;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TCalcDocument.ConstrainToUsedCells(VAR cellRange: Rect);
 { Given a range of cells, this returns the range of cells that fall
   within the range of used cells.	This is used to optimize
   performance so that we don't try to operate on cells that
   we know have never been used (i.e. allocated). }

	BEGIN
	WITH cellRange DO
		BEGIN
		top := Max(top, fInUseBounds.top);
		left := Max(left, fInUseBounds.left);
		bottom := Min(bottom, fInUseBounds.bottom);
		right := Min(right, fInUseBounds.right);
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TCalcDocument.DeleteCell(r: RowNumber;
								   c: ColumnNumber);

	VAR
		theCell:			TCell;

	BEGIN
	theCell := GetExistingCell(r, c);
	IF theCell <> NIL THEN
		BEGIN
		theCell.SetDeleteState(TRUE);
		fAllocatedCells := fAllocatedCells - 1;
		IF theCell = fEditCell THEN
			fEntryView.SetToString('');
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S AOpen}

PROCEDURE TCalcDocument.DoInitialState; OVERRIDE;

	BEGIN
	fAllocatedCells := 0;
	fAllocatedRows := 0;
	fAllocatedColumns := 0;
	END;

{--------------------------------------------------------------------------------------------------}
{$S AOpen}

PROCEDURE TCalcDocument.DoMakeViews(forPrinting: BOOLEAN);

	VAR
		aCalcWindow:		TCalcWindow;
		aTEntryView:		TEntryView;
		aCellsView: 		TCellsView;
		aColumnsView:		TColumnsView;
		aRowsView:			TRowsView;
		aCoordView: 		TCoordView;
		aRowScroller:		TSecondaryScroller;
		aColumnScroller:	TSecondaryScroller;
		aCalcScroller:		TPrimaryScroller;
		aPrintHandler:		TCalcPrintHandler;
		aVPt:				VPoint;

	PROCEDURE SetColumnWidths;

		PROCEDURE DoToItem(item: TObject);

			BEGIN
			WITH TColumn(item) DO
				IF fWidth <> kCellWidth THEN
					BEGIN
					fCellsView.SetColWidth(fNumber, 1, fWidth);
					fColumnsView.SetColWidth(fNumber, 1, fWidth);
					END;
			END;

		BEGIN
		fColumns.Each(DoToItem);
		END;

	BEGIN
	aCalcWindow := TCalcWindow(gViewServer.NewTemplateWindow(kCalcWindowType, SELF));
	WITH aCalcWindow DO
		BEGIN
		aCellsView := TCellsView(FindSubView('CELL'));
		aRowsView := TRowsView(FindSubView('ROWS'));
		aColumnsView := TColumnsView(FindSubView('COLS'));
		aTEntryView := TEntryView(FindSubView('ENTV'));
		aCoordView := TCoordView(FindSubView('CORD'));
		END;

	fCellsView := aCellsView;
	fRowsView := aRowsView;
	fColumnsView := aColumnsView;
	fEntryView := aTEntryView;
	fCoordView := aCoordView;

	IF NOT forPrinting THEN
		BEGIN
		aTEntryView.fText := NIL;
		aTEntryView.fText := NewPermHandle(0);
		aTEntryView.StuffText(aTEntryView.fText);		{ Stuff the initial text in }
		END;

	{ Insert the entry view between the cell's view and its scroller in the target chain }
	aCalcWindow.SetTarget(aCellsView);
	{!!!RCR}
	aTEntryView.fNextHandler := aCellsView.fNextHandler;
	aCellsView.fNextHandler := aTEntryView;

	{ set up the cells view scroller to scroll the rows and columns too }
	aCalcScroller := TPrimaryScroller(aCellsView.GetScroller(TRUE));
	SetVPt(aVPt, kCellWidth, kCellHeight);
	aCalcScroller.SetScrollParameters(aVPt, TRUE, TRUE);

	aColumnScroller := TSecondaryScroller(aColumnsView.GetScroller(TRUE));
	SetVPt(aVPt, kCellWidth, 0);
	aColumnScroller.SetScrollParameters(aVPt, TRUE, TRUE);

	aRowScroller := TSecondaryScroller(aRowsView.GetScroller(TRUE));
	SetVPt(aVPt, 0, kCellHeight);
	aRowScroller.SetScrollParameters(aVPt, TRUE, TRUE);

	aCalcScroller.AddSecondaryScroller(aRowScroller, kNotHDependent, kVDependent);
	aCalcScroller.AddSecondaryScroller(aColumnScroller, kHDependent, kNotVDependent);

	NEW(aPrintHandler);
	aPrintHandler.IStdPrintHandler(SELF,				{ its document }
								   aCellsView,			{ its view }
								   NOT kSquareDots, 	{ does not have square dots }
								   NOT kFixedSize,		{ horizontal page size is variable }
								   kFixedSize); 		{ vertical page size is fixed }
	aPrintHandler.fMinimalMargins := FALSE;

	SetColumnWidths;									{ get existing document's column widths }

	IF forPrinting THEN 								{ Finder printing }
		aPrintHandler.RedoPageBreaks
	ELSE
		BEGIN
		ShowReverted;									{ display the views }

		fEditRow := 1;									{ default cell to edit is A1 }
		fEditColumn := 1;
		SetEntry(fEditRow, fEditColumn);
		fSelectionType := CellSelection;
		fCellsView.SelectCell(GridCell($00010001), kDontExtend, kDontHighlight, kSelect);
		END;

	SynchSections(NIL);									{ synch up all sections }
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TCalcDocument.SynchSections(aSection: TSection);

	PROCEDURE SynchASection(aSection: TSection);

		PROCEDURE DoToPublishedCell(aGridCell: GridCell);
	
			VAR
				aCell:		TCell;

			BEGIN
			IF CellExists(aGridCell.v, aGridCell.h) THEN
				BEGIN
				aCell := GetCell(aGridCell.v, aGridCell.h);
				aCell.AddDependent(aSection);
				END;
			END;
	
		BEGIN
		{ adding a section, give it a section adorner }
		IF Member(aSection, TPublisher) THEN
			fCellsView.CreatePublisherAdorner(TPublisher(aSection))
		ELSE IF Member(aSection, TSubscriber) THEN
			fCellsView.CreateSubscriberAdorner(TSubscriber(aSection));
	
		{ make each publisher a dependent of each cell in the designation }
		IF Member(aSection, TPublisher) THEN
			fCellsView.EachInRgn(TRgnDesignator(aSection.fDesignator).fDesignation, DoToPublishedCell);
	
		END;

	BEGIN
	IF (fCellsView <> NIL) THEN
		BEGIN
		IF (aSection = NIL) THEN
			EachSectionDo(SynchASection)				{ synch existing sections with cells view }
		ELSE
			SynchASection(aSection);
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ASelCommand}

PROCEDURE TCalcDocument.DoMenuCommand(aCmdNumber: cmdNumber);

	VAR
		aColumnFormatter:	TColumnFormatter;

	BEGIN												{ TCalcDocument.DoMenuCommand }
	CASE aCmdNumber OF
		cRecalculate:
			DoRecalculate(kForceAutomatic, kUpdateDependencies);
		cAutoCalc:
			BEGIN
			fCalcMode := Automatic;
			DoRecalculate(kForceAutomatic, kUpdateDependencies);
			END;
		cManualCalc:
			fCalcMode := Manual;

		cGeneral, cNoDecimal, cDecimal, cScientific, cSystemJustify, cForceLeftJustify,
		cRightJustify, cCenter:
			BEGIN
			NEW(aColumnFormatter);
			aColumnFormatter.IFormatter(SELF, aCmdNumber);
			PostCommand(aColumnFormatter);
			END;

		OTHERWISE
			INHERITED DoMenuCommand(aCmdNumber);
	END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S AWriteFile}

PROCEDURE TCalcDocument.DoNeedDiskSpace(itsFile: TFile;
									VAR dataForkBytes, rsrcForkBytes: LONGINT);

VAR
	r:					Rect;

{--------------------------------------------------------------------------------------------------}

	PROCEDURE AccountForCell(aCell: TCell);

	BEGIN
		dataForkBytes := dataForkBytes + aCell.GetDiskSize(FALSE);
	END;

BEGIN
	{ get Print record requirements }
	INHERITED DoNeedDiskSpace(itsFile, dataForkBytes, rsrcForkBytes);

	dataForkBytes := dataForkBytes + SIZEOF(CalcDocDiskInfo) + SIZEOF(RowDiskInfo) *
					 fAllocatedRows + SIZEOF(ColumnDiskInfo) * fAllocatedColumns;
	EachExistingCellDo(AccountForCell);

END;

{--------------------------------------------------------------------------------------------------}
{$S AReadFile}

PROCEDURE TCalcDocument.DoRead(aFile: TFile; 
								forPrinting: BOOLEAN); OVERRIDE;

	VAR
		i:					INTEGER;
		noOfCells:			INTEGER;
		noOfRows:			INTEGER;
		noOfColumns:		INTEGER;
		r:					RowNumber;
		c:					ColumnNumber;
		aCell:				TCell;
		aRow:				TRow;
		aColumn:			TColumn;
		fi: 				FailInfo;
		count:				Longint;

{--------------------------------------------------------------------------------------------------}

	PROCEDURE ReadDocInfo;

		VAR
			theDocInfo: 		CalcDocDiskInfo;

		BEGIN
		count := SIZEOF(CalcDocDiskInfo);
		FailOSErr(aFile.ReadData(@theDocInfo, count));

		WITH theDocInfo DO
			BEGIN
			fDimensions := dimensions;
			fCalcMode := calcMode;
			noOfRows := allocatedRows;
			noOfColumns := allocatedColumns;
			fAllocatedCells := allocatedCells;
			fSelectionType := NoSelection;
			fEditRow := editRow;
			fEditColumn := editColumn;
			END;

		WITH fDimensions DO
			BEGIN
			fNoOfRows := bottom - top + 1;
			fNoOfColumns := right - left + 1;
			END;

  { Save the number of cells to be read, then set fAllocatedCells to
	zero.  As each cell is read fAllocatedCells is incremented.
	When we've finished, noOfCells must equal fAllocatedCells. }
		noOfCells := fAllocatedCells;
		DoInitialState; 								{ clear allocation counts }
		END;

{--------------------------------------------------------------------------------------------------}

	PROCEDURE HdlReadFailure(error: OSErr;
							 message: LONGINT);

		BEGIN
		{ Oh Boy are we in trouble }
  { Need to set back rows and columns to reflect the number read in so
	the freedata routine (which will eventually be called by other
	failure handlers on the stack) won't try to free unallocated objects}
		IF c = 0 THEN
			BEGIN
			{ We died while reading the rows in}
			fNoOfRows := r - 1;
			fNoOfColumns := 0;
			END
		ELSE IF i = 0 THEN
			BEGIN
			{ Died in the columns }
			fNoOfColumns := c - 1;
			END;
		END;

	BEGIN												{ DoRead }
	CatchFailures(fi, HdlReadFailure);
	INHERITED DoRead(aFile,forPrinting);

	ReadDocInfo;										{ Get info about the document }
	r := 0; c := 0; i := 0; 							{ Initialized so failure handler knows where
														 we died }

	FOR i := 1 TO noOfRows DO							{ Get info about each row }
		BEGIN
		count := SIZEOF(r);
		FailOSErr(aFile.ReadData(@r, count));
		aRow := GetRow(r);
		aRow.ReadFromDisk(aFile);
		END;

	FOR i := 1 TO noOfColumns DO						{ Get info about each column }
		BEGIN
		count := SIZEOF(c);
		FailOSErr(aFile.ReadData(@c, count));
		aColumn := GetColumn(c);
		aColumn.ReadFromDisk(aFile);
		END;

	FOR i := 1 TO noOfCells DO							{ Read in the cells }
		BEGIN
		ReadCellCoordinate(aFile, r, c);
		aCell := GetCell(r, c);
		aCell.ReadFromDisk(aFile);
		END;

{$IFC qDebug}
	IF gIntenseDebugging THEN
		BEGIN
		WRITELN('TCalcDocument.DoRead: noOfCells=', noOfCells);
		IF noOfCells <> fAllocatedCells THEN
			BEGIN
			WRITELN('TCalcDocument.DoRead: Wrong number of cells.  noOfCells=', noOfCells,
					', fAllocatedCells=', fAllocatedCells);
			ProgramBreak('');
			END;
		END;
{$ENDC}

	Success(fi);
	END;												{ DoRead }

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TCalcDocument.DoRecalculate(forceAutomatic: BOOLEAN;
									  setDependents: BOOLEAN);

	VAR
		r:					Rect;

{--------------------------------------------------------------------------------------------------}

	PROCEDURE RecalcCell(aCell: TCell);

	BEGIN
		aCell.Recalculate(forceAutomatic, setDependents);
	END;

BEGIN
	IF IsAutoCalc | forceAutomatic THEN
		BEGIN
		EachExistingCellDo(RecalcCell);
		END
	ELSE
		{!!!RCR
		SELF.EachSelectedExistingCellDo(RecalcCell);
		}
		SELF.UpdateSelectedCells(mValueChanged,SELF);
	SetChangeCount(Max(fChangeCount + 1, 1));			{ enable Save - document may have changed }
END;

{--------------------------------------------------------------------------------------------------}
{$S ANonRes}

FUNCTION TCalcDocument.GetSubscriberFormatsMask: SignedByte; OVERRIDE;

	BEGIN
	GetSubscriberFormatsMask := kTextformatMask;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

FUNCTION TCalcDocument.CanPublishSelection: BOOLEAN; OVERRIDE;

	BEGIN
	{ we can publish iff:
		1. there's some cell selected in the view }
	CanPublishSelection := fCellsView.IsAnyCellSelected;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

FUNCTION TCalcDocument.CanSubscribe: BOOLEAN; OVERRIDE;

	VAR
		aSection: TSection;

	BEGIN
	{ we can subscribe iff:
		1. there's some cell selected in the view (i.e. it's obvious where the edition goes), AND
		2. there's no selected subscriber (i.e. so that we don't subscribe OVER a subscriber) }
	aSection := GetSelectedSection;
	CanSubscribe := fCellsView.IsAnyCellSelected & ((aSection = NIL) | NOT Member(aSection, TSubscriber));
	END;

{--------------------------------------------------------------------------------------------------}
{$S ANonRes}

PROCEDURE TCalcDocument.DoWriteData(aScrapType: OSType; aDesignator: TDesignator;
										  aStream: TStream); OVERRIDE;

	VAR
		savedState:			SignedByte;
		textHandle:			Handle;
		aPicHandle:			PicHandle;

	BEGIN
	IF aScrapType = 'TEXT' THEN
		BEGIN
		textHandle := fCellsView.DesignatorAsTEXT(aDesignator);
		savedState := LockHandleHigh(textHandle);
		aStream.WriteBytes(textHandle^, GetHandleSize(textHandle));
		HSetState(textHandle, savedState);
		DisposeHandle(textHandle);
		END
	ELSE IF aScrapType = 'PICT' THEN
		BEGIN
		aPicHandle := fCellsView.DesignatorAsPICT(aDesignator);
		savedState := LockHandleHigh(Handle(aPicHandle));
		aStream.WriteBytes(aPicHandle^, GetHandleSize(Handle(aPicHandle)));
		HSetState(Handle(aPicHandle), savedState);
		DisposeHandle(Handle(aPicHandle));
		END
	ELSE IF aScrapType = fScrapType THEN
		DoWritePrivateTypes(aDesignator, aStream);
	END;


{--------------------------------------------------------------------------------------------------}
{$S ANonRes}

PROCEDURE TCalcDocument.DoReadData(aScrapType: OSType; aDesignator: TDesignator;
										  aStream: TStream; length: LONGINT); OVERRIDE;

	BEGIN
	IF aScrapType = 'TEXT' THEN
		DoReadTEXT(aDesignator, aStream, length)
	ELSE IF aScrapType = fScrapType THEN
		DoReadPrivateTypes(aDesignator, aStream);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

FUNCTION TCalcDocument.GetSelectedSection: TSection; OVERRIDE;

	BEGIN
	GetSelectedSection := NIL;
	IF (fCellsView <> NIL) & (fSectionList <> NIL) & (fSectionList.GetSize > 0) THEN
		GetSelectedSection := GetSelectedSectionInRgn(fCellsView.fSelections);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

FUNCTION TCalcDocument.GetSelectedSectionInRgn(selectionRgn: RgnHandle): TSection;

	VAR
		aSelectedSection:	TSection;
		multipleSections:	BOOLEAN;				{ TRUE if > 1 section is selected }

	PROCEDURE DoToSection(aSection: TSection);

		BEGIN
		IF NOT multipleSections THEN				{ optimization }
			BEGIN
			SectRgn(selectionRgn, TRgnDesignator(aSection.fDesignator).fDesignation, gTempRgn);
			IF NOT EmptyRgn(gTempRgn) THEN			{ if this section is in selection }
				IF (aSelectedSection <> NIL) THEN
					multipleSections := TRUE		{ more than 1 section selected }
				ELSE
					aSelectedSection := aSection;
			END;
		END;

	BEGIN
	multipleSections := FALSE;
	aSelectedSection := NIL;
	{$IFC qDebug}
	UseTempRgn('TCalcDocument.GetSelectedSectionInRgn');
	{$ENDC qDebug}
	EachSectionDo(DoToSection);
	{$IFC qDebug}
	DoneWithTempRgn;
	{$ENDC qDebug}
	IF multipleSections THEN
		GetSelectedSectionInRgn := NIL			{ if > 1 section is selected, it's not
													possible to determine which section is selected }
	ELSE
		GetSelectedSectionInRgn := aSelectedSection;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ANonRes}

PROCEDURE TCalcDocument.AddSection(aSection: TSection); OVERRIDE;
	
	BEGIN
	INHERITED AddSection(aSection);
	
	{ synch up the section with the cells in the cells view }
	SynchSections(aSection);
	END;

{--------------------------------------------------------------------------------------------------}
{$S AOpen}

PROCEDURE TCalcDocument.RevealSelection(aSelection: TDesignator); OVERRIDE;

	VAR
		thePixels:			RgnHandle;
		qdExtent:			Rect;
		viewRect:			VRect;

	BEGIN
	INHERITED RevealSelection(aSelection);					{ make window front-most }

	IF Member(aSelection, TRgnDesignator) THEN
		BEGIN
		fCellsView.SetEmptySelection(kRedraw);
		fCellsView.SetSelection(TRgnDesignator(aSelection).fDesignation, FALSE, TRUE, TRUE);
		
		fCellsView.ScrollSelectionIntoView(kRedraw);
		END
	ELSE
		IF qDebug THEN
			ProgramBreak('handed an unknown designator');
	END;

{--------------------------------------------------------------------------------------------------}
{$S ANonRes}

PROCEDURE TCalcDocument.DoWritePrivateTypes(aDesignator: TDesignator; aStream: TStream);

	VAR
		selection:			Rect;
		i:					INTEGER;
		rowInfo:			RowDiskInfo;
		columnInfo: 		ColumnDiskInfo;

	PROCEDURE PublishCell(aGridCell: GridCell);

		VAR
			aCell:				TCell;
			cellInfo:			CellDiskInfo;

		BEGIN
		{ write out cell info }
		aCell := GetCell(aGridCell.v, aGridCell.h);
		WITH aCell, cellInfo DO
			BEGIN
			kind := fKind;
			error := fError;
			value := fValue;
			noOfReferences := 0;							
			formula := fFormula;
			END;
		aStream.WriteBytes(@cellInfo, sizeof(cellInfo));
		END;

	BEGIN
	IF Member(aDesignator, TRgnDesignator) THEN
		BEGIN
		{ Write out extent. }
		selection := GetRgnBounds(TRgnDesignator(aDesignator).fDesignation);
		aStream.WriteBytes(@selection, sizeof(selection));

		{ Write out row info. }
		FOR i := selection.top TO selection.bottom DO
			BEGIN
			rowInfo.number := GetRow(i).fNumber;					{ Need a GetRowDiskInfo(). }
			aStream.WriteBytes(@rowInfo, sizeof(rowInfo));
			END;

		{ Write out column info. }
		FOR i := selection.left TO selection.right DO
			BEGIN
			WITH GetColumn(i), columnInfo DO
				BEGIN
				number := fNumber;
				format := fFormat;
				width := fWidth;
				END;
			aStream.WriteBytes(@columnInfo, sizeof(columnInfo));
			END;

		{ Write out cell info. }
		fCellsView.EachCellDo(selection.topLeft, selection.botRight, PublishCell);
		END
	ELSE
		IF qDebug THEN
			ProgramBreak('I don''t recognize that designatorsigh!');
	END;

{--------------------------------------------------------------------------------------------------}
{$S ANonRes}

PROCEDURE TCalcDocument.DoReadPrivateTypes(aDesignator: TDesignator; aStream: TStream);

	VAR
		subBounds:			Rect;			{ bounds of the subscriber }
		pubBounds:			Rect;			{ bounds of the publisher }
		i:					INTEGER;
		rowInfo:			RowDiskInfo;
		columnInfo: 		ColumnDiskInfo;
		difference:			Point;

	PROCEDURE DoToAPublishedCell(aGridCell: GridCell);

		VAR
			aCell:				TCell;
			cellInfo:			CellDiskInfo;

		BEGIN
		{ map a published cell into subscriber range }
		aGridCell.v := aGridCell.v + difference.v;
		aGridCell.h := aGridCell.h + difference.h;
		aCell := GetCell(aGridCell.v, aGridCell.h);					{ retrieve or create a TCell }
		aStream.ReadBytes(@cellInfo, sizeof(cellInfo));
		WITH aGridCell, subBounds DO
			IF (h >= left) & (h <= right) & (v >= top) & (v <= bottom) THEN
				WITH aCell, cellInfo DO
					IF (fKind <> kind) | (fError <> error) | (fValue <> value) | (fFormula <> formula) THEN
						BEGIN
						fKind := kind;
						fError := error;
						fValue := value;
						fFormula := formula;
						aCell.Recalculate(TRUE, TRUE);
						aCell.Invalidate;
						END;
		END;

	BEGIN
	{ read in the publisher bounds and figure out the difference }
	subBounds := GetRgnBounds(TRgnDesignator(aDesignator).fDesignation);
	aStream.ReadBytes(@pubBounds, sizeof(pubBounds));
	difference.h := subBounds.left - pubBounds.left;
	difference.v := subBounds.top - pubBounds.top;

	{ Read in row info. }
	FOR i := pubBounds.top TO pubBounds.bottom DO
		aStream.ReadBytes(@rowInfo, sizeof(rowInfo));

	{ Read in the col info and set it. }
	FOR i := pubBounds.left TO pubBounds.right DO
		BEGIN
		aStream.ReadBytes(@columnInfo, sizeof(columnInfo));
		IF (i + difference.h >= subBounds.left) & (i + difference.h <= subBounds.right) & (NOT ColumnExists(i)) THEN
			BEGIN
			WITH GetColumn(i), columnInfo DO
				BEGIN
				fNumber := number;
				fFormat := format;
				fWidth := width;
				END;
			END;
		END;
		
	{ read in the cell info }
	WITH pubBounds DO
		fCellsView.EachCellDo(topLeft, botRight, DoToAPublishedCell);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ANonRes}

PROCEDURE TCalcDocument.DoReadTEXT(aDesignator: TDesignator; aStream: TStream; length: LONGINT);

	VAR
		subBounds:			Rect;			{ bounds of the subscriber }
		pubBounds:			Rect;			{ bounds of the publisher }
		i:					INTEGER;
		theText:			Handle;

	FUNCTION ParseSubscribedText(textHdl: Handle; aGridCell: GridCell): Str255;
	
		VAR
			normalizedCell:		GridCell;
			p:					Ptr;
			offset:				LONGINT;
			returnStr:			Str255;
			i:					INTEGER;
			where:				Longint;
			aChar:				Char;
			maxLength:			Longint;

		PROCEDURE ScanTo(ch: Char);
		
			VAR
				p: Ptr;
		
			BEGIN
			{$Push} {$R-}
			p := Ptr(ORD(textHdl^) + offset);
			WHILE (CHR(p^) <> ch) & (offset < length) DO
				BEGIN
				offset := offset + 1;
				p := Ptr(ORD(textHdl^) + offset);
				END;
			{$Pop}
			END;

		FUNCTION NumTabs: INTEGER;
		{ number of tabs between (ORD(textHdl^) + localOffset) and the next chReturn }
		
			VAR
				p: Ptr;
				localOffset: LONGINT;
				num: INTEGER;
		
			BEGIN
			{$Push} {$R-}
			localOffset := offset;
			num := 0;
			p := Ptr(ORD(textHdl^) + localOffset);
			WHILE (CHR(p^) <> chReturn) & (localOffset < length) DO
				BEGIN
				IF (CHR(p^) = chTab) THEN
					num := num + 1;
				localOffset := localOffset + 1;
				p := Ptr(ORD(textHdl^) + localOffset);
				END;
			NumTabs := num;
			{$Pop}
			END;

		BEGIN
		ParseSubscribedText := '';
		IF length > 0 THEN
			BEGIN
			normalizedCell.v := aGridCell.v - subBounds.top;
			normalizedCell.h := aGridCell.h - subBounds.left;
			offset := 0;

			{ skip over returns }
			FOR i := 1 TO normalizedCell.v DO
				BEGIN
				ScanTo(chReturn);
				offset := offset + 1;					{ skip over return }
				IF (offset > length) THEN			{ there's no more input }
					Exit(ParseSubscribedText);
				END;

			{ is there a relevant data item to read in? }
			IF (NumTabs < normalizedCell.h) THEN
				Exit(ParseSubscribedText);

			{ skip over tabs }
			FOR i := 1 TO normalizedCell.h DO
				BEGIN
				ScanTo(chTab);
				offset := offset + 1;					{ skip over tab }
				IF (offset > length) THEN			{ there's no more input }
					Exit(ParseSubscribedText);
				END;

			p := Ptr(ORD(textHdl^) + offset);
			maxLength := Min(length-offset, 255);
			offset := 0;
			aChar := Chr(Ptr(ORD(p) + offset)^);
			WHILE ((offset < maxLength) & (aChar <> chTab) & (aChar <> chReturn)) DO
				BEGIN
				offset := offset + 1;
				aChar := Chr(Ptr(ORD(p) + offset)^);
				END;
			returnStr[0] := Chr(offset);
			BlockMove(p, @returnStr[1], offset);
			ParseSubscribedText := returnStr;
			END;
		END;

	PROCEDURE DoToSubscribedCell(aGridCell: GridCell);

		VAR
			aStr:			Str255;
			aCell:			TCell;

		BEGIN
		aStr := ParseSubscribedText(theText, aGridCell);
		aCell := GetCell(aGridCell.v, aGridCell.h);					{ retrieve or create a TCell }
		aCell.SetToString(aStr);
		END;

	BEGIN
	theText := NewPermHandle(length);
	IF LockHandleHigh(theText) = 0 THEN;
	aStream.ReadBytes(theText^, length);
	subBounds := GetRgnBounds(TRgnDesignator(aDesignator).fDesignation);
	WITH subBounds DO
		fCellsView.EachCellDo(topLeft, botRight, DoToSubscribedCell);
	DisposeHandle(theText);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TCalcDocument.DoSetupMenus; OVERRIDE;

	VAR
		justification:		INTEGER;
		Style:				TypeOfStyle;
		columnFormat:		FormatRecord;

	BEGIN

	INHERITED DoSetupMenus;

	Enable(cRecalculate, TRUE);
	EnableCheck(cAutoCalc, TRUE, IsAutoCalc);
	EnableCheck(cManualCalc, TRUE, fCalcMode = Manual);

	IF fColumnIsSelected THEN
		BEGIN
		IF ColumnExists(fEditColumn) THEN
			columnFormat := GetColumn(fEditColumn).fFormat
		ELSE
			columnFormat := gDefaultFormat;
		justification := columnFormat.fJustification;
		Style := columnFormat.fStyle;

		EnableCheck(cSystemJustify, TRUE, justification = teFlushDefault);
		EnableCheck(cForceLeftJustify, TRUE, justification = teFlushLeft);
		EnableCheck(cRightJustify, TRUE, justification = teFlushRight);
		EnableCheck(cCenter, TRUE, justification = teCenter);

		EnableCheck(cGeneral, TRUE, Style = General);
		EnableCheck(cDecimal, TRUE, Style = DecimalStyle);
		EnableCheck(cNoDecimal, TRUE, Style = NoDecimal);
		EnableCheck(cScientific, TRUE, Style = Scientific);
		END
	ELSE
		BEGIN
		Enable(cSystemJustify, FALSE);
		Enable(cForceLeftJustify, FALSE);
		Enable(cRightJustify, FALSE);
		Enable(cCenter, FALSE);

		Enable(cGeneral, FALSE);
		Enable(cDecimal, FALSE);
		Enable(cNoDecimal, FALSE);
		Enable(cScientific, FALSE);
		END;

	END;

{--------------------------------------------------------------------------------------------------}
{$S AWriteFile}

PROCEDURE TCalcDocument.DoWrite(aFile: TFile;
								makingCopy: BOOLEAN);

	VAR
		cellsWritten:		INTEGER;
		r:					Rect;

{--------------------------------------------------------------------------------------------------}

	PROCEDURE WriteDocInfo;

		VAR
			theDocInfo: 		CalcDocDiskInfo;
			count:				Longint;

		BEGIN
		WITH theDocInfo DO
			BEGIN
			dimensions := fDimensions;
			calcMode := fCalcMode;
			allocatedRows := fAllocatedRows;
			allocatedColumns := fAllocatedColumns;
			allocatedCells := fAllocatedCells;
			selectionType := NoSelection;
			editRow := fEditRow;
			editColumn := fEditColumn;
			END;
		count := SIZEOF(CalcDocDiskInfo);
		FailOSErr(aFile.WriteData(@theDocInfo, count));
		END;

{--------------------------------------------------------------------------------------------------}

	PROCEDURE WriteRow(aRow: TRow);

		BEGIN
		aRow.WriteToDisk(aFile);
		END;

{--------------------------------------------------------------------------------------------------}

	PROCEDURE WriteColumn(aColumn: TColumn);

		BEGIN
		{ aColumn.fWidth := fColumnsView.GetColWidth(aCell.h); }
		aColumn.WriteToDisk(aFile);
		END;

{--------------------------------------------------------------------------------------------------}

	PROCEDURE WriteCell(aCell: TCell);

		BEGIN
		aCell.WriteToDisk(aFile);
		cellsWritten := cellsWritten + 1;
		END;

	BEGIN
	INHERITED DoWrite(aFile, makingCopy);

	IF aFile <> NIL THEN
		BEGIN
		WriteDocInfo;										{ Write info about the document }
	
		EachExistingRowDo(WriteRow);						{ Write info about each row }
		EachExistingColumnDo(WriteColumn);					{ Write info about each column }
	
		cellsWritten := 0;
		EachExistingCellDo(WriteCell);					{ Write out the cells }
	
	{$IFC qDebug}
		IF cellsWritten <> fAllocatedCells THEN
			ProgramBreak('DoWrite: Incorrect number of cells written');
	{$ENDC}
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TCalcDocument.EachExistingRowDo(PROCEDURE
										  DoToRow(aRow: TRow));
{ Perform DoToCell for each ALLOCATED Row }

	PROCEDURE DoToItem(item: TObject);

		BEGIN
		DoToRow(TRow(item));
		END;

	BEGIN
	fRows.Each(DoToItem);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TCalcDocument.EachExistingColumnDo(PROCEDURE
											 DoToColumn(aColumn: TColumn));
{ Perform DoToColumn for each ALLOCATED Column }

	PROCEDURE DoToItem(item: TObject);

		BEGIN
		DoToColumn(TColumn(item));
		END;

	BEGIN
	fColumns.Each(DoToItem);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TCalcDocument.EachExistingCellDo(PROCEDURE
										   DoToCell(aCell: TCell));
{ Perform DoToCell for each ALLOCATED cell }

	PROCEDURE DoToItem(item: TObject);

	BEGIN
		IF NOT TCell(item).fDeleted THEN
			DoToCell(TCell(item));
	END;

BEGIN
	fCells.Each(DoToItem);
END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TCalcDocument.EachSelectedExistingCellDo(PROCEDURE
										   DoToCell(aCell: TCell));

	PROCEDURE CheckCell(aCell: GridCell);

	VAR
		theCell:			TCell;

	BEGIN
		theCell := GetExistingCell(aCell.v, aCell.h);
		IF theCell <> NIL THEN
			DoToCell(theCell);
	END;

BEGIN
	IF fCellsView <> NIL THEN
		fCellsView.EachSelectedCellDo(CheckCell);
END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TCalcDocument.EditCell;
{ Change the formula of the cell being edited to the string in the entry view }

	VAR
		theString:			Str255;

	BEGIN
	fEntryView.GetAsString(theString);
	IF fEditCell = NIL THEN
		fEditCell := GetCell(fEditRow, fEditColumn);
	fEditCell.SetToString(theString);
	END;

{--------------------------------------------------------------------------------------------------}
{$S AFields}

PROCEDURE TCalcDocument.Fields(obj: TObject); OVERRIDE;

	VAR
		aString:			Str255;

	BEGIN
	WITH obj DO
		BEGIN
		DoToField(AtStr('TCalcDocument'), NIL, bClass);
		DoToField(AtStr('fEditCell'), @fEditCell, bObject);
		DoToField(AtStr('fNoOfRows'), @fNoOfRows, bInteger);
		DoToField(AtStr('fNoOfColumns'), @fNoOfColumns, bInteger);
		DoToField(AtStr('fRows'), @fRows, bObject);
		DoToField(AtStr('fColumns'), @fColumns, bObject);
		DoToField(AtStr('fCells'), @fCells, bObject);
		DoToField(AtStr('fRowOffset'), @fRowOffset, bInteger);
		DoToField(AtStr('fColumnOffset'), @fColumnOffset, bInteger);
		DoToField(AtStr('fCellsView'), @fCellsView, bObject);
		DoToField(AtStr('fRowsView'), @fRowsView, bObject);
		DoToField(AtStr('fColumnsView'), @fColumnsView, bObject);
		DoToField(AtStr('fEntryView'), @fEntryView, bObject);
		DoToField(AtStr('fCoordView'), @fCoordView, bObject);
		DoToField(AtStr('fDimensions'), @fDimensions, bRect);
		DoToField(AtStr('fInUseBounds'), @fInUseBounds, bRect);
		IF IsAutoCalc THEN
			aString := 'Automatic'
		ELSE
			aString := 'Manual';
		DoToField(AtStr('fCalcMode'), @aString, bString);
		DoToField(AtStr('fAllocatedRows'), @fAllocatedRows, bInteger);
		DoToField(AtStr('fAllocatedColumns'), @fAllocatedColumns, bInteger);
		DoToField(AtStr('fAllocatedCells'), @fAllocatedCells, bInteger);
		CASE fSelectionType OF
			NoSelection:
				aString := 'NoSelection';
			CellSelection:
				aString := 'CellSelection';
			RowSelection:
				aString := 'RowSelection';
			ColumnSelection:
				aString := 'ColumnSelection';
			AllSelection:
				aString := 'AllSelection';
		END;
		DoToField(AtStr('fSelectionType'), @aString, bString);
		DoToField(AtStr('fEditRow'), @fEditRow, bByte);
		DoToField(AtStr('fEditColumn'), @fEditColumn, bByte);
		END;											{ WITH }

	INHERITED Fields(obj);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TCalcDocument.FreeCell(theCell: TCell);

	BEGIN
{$IFC qDebug}
	IF fCells.GetCell(theCell.fRow, theCell.fColumn) <> theCell THEN
		ProgramBreak('TCalcDocument.FreeCell: Cell table inconsistent');
{$ENDC}

	fCells.Delete(theCell);

	theCell := TCell(FreeIfObject(theCell));
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TCalcDocument.FreeData;

	VAR
		r:					RowNumber;
		c:					ColumnNumber;
		oldState:			BOOLEAN;

	BEGIN
	oldState := Lock(TRUE); 							{ HLock(Handle(SELF)) }

	fCells.FreeAll;
	fRows.FreeAll;
	fColumns.FreeAll;

	IF Lock(oldState) THEN;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TCalcDocument.FreeDeletedCells;
{ Free each deleted cell. }

	PROCEDURE DoToItem(item: TObject);

		BEGIN
		IF TCell(item).fDeleted THEN
			FreeCell(TCell(item));
		END;

	BEGIN
	fCells.Each(DoToItem);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

FUNCTION TCalcDocument.GetCell(r: RowNumber;
							   c: ColumnNumber): TCell;
 { Return the cell object for the given coordinates.  If a cell object
   doesn't already exist, create one. }

	VAR
		theCell:			TCell;

	BEGIN
	theCell := fCells.GetCell(r, c);
	IF theCell = NIL THEN
		BEGIN
		NEW(theCell);
		theCell.ICell(SELF, r, c);
		AddCell(theCell, r, c);
		END;

	GetCell := theCell;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

FUNCTION TCalcDocument.GetColumn(c: ColumnNumber): TColumn;

	VAR
		theColumn:			TColumn;

	BEGIN
	theColumn := fColumns.GetColumn(c);

	IF theColumn = NIL THEN
		BEGIN
		NEW(theColumn);
		theColumn.IColumn(c);
		AddColumn(theColumn);
		END;

	GetColumn := theColumn;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

FUNCTION TCalcDocument.GetExistingCell(r: RowNumber;
									   c: ColumnNumber): TCell;
{ Like GetCell, only return NIL if the cell object doesn't exist }

	BEGIN
	GetExistingCell := fCells.GetCell(r, c);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

FUNCTION TCalcDocument.IsAutoCalc: BOOLEAN;

	BEGIN
	IsAutoCalc := fCalcMode = Automatic;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

FUNCTION TCalcDocument.GetRow(r: RowNumber): TRow;

	VAR
		theRow: 			TRow;

	BEGIN
	theRow := fRows.GetRow(r);
	IF theRow = NIL THEN
		BEGIN
		NEW(theRow);
		theRow.IRow(r);
		AddRow(theRow);
		END;

	GetRow := theRow;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

FUNCTION TCalcDocument.RowExists(r: RowNumber): BOOLEAN;

	BEGIN
	RowExists := fRows.GetRow(r) <> NIL;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TCalcDocument.SetEntry(r: RowNumber;
								 c: ColumnNumber);
{ Set the string in TEntryView to the formula in the cell }

	VAR
		theString:			Str255;
		theCell:			TCell;

	BEGIN
	theCell := GetExistingCell(r, c);

	IF theCell <> NIL THEN
		BEGIN
		theCell.GetAsString(theString);
		fEntryView.SetToString(theString);
		END
	ELSE
		fEntryView.SetToString('');
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TCalcDocument.UndeleteCell(r: RowNumber;
									 c: ColumnNumber);

	VAR
		theCell:			TCell;

	BEGIN
	theCell := GetExistingCell(r, c);
	IF (theCell <> NIL) THEN
		BEGIN
		theCell.SetDeleteState(FALSE);
		fAllocatedCells := fAllocatedCells + 1;
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}
PROCEDURE TCalcDocument.UpdateSelectedCells(theChange: ChangeID; changedBy: TObject);

	PROCEDURE MarkItsDependents(aCell: TCell);
	BEGIN
		aCell.MarkDependents(TRUE);
	END;

	PROCEDURE UpdateItsDependents(aCell: TCell);
	BEGIN
		aCell.UpdateDependents(theChange,aCell,changedBy);
	END;

	PROCEDURE UnMarkItsDependents(aCell: TCell);
	BEGIN
		aCell.MarkDependents(FALSE);
	END;
	
BEGIN
	EachSelectedExistingCellDo(MarkItsDependents);
	EachSelectedExistingCellDo(UpdateItsDependents);
	EachSelectedExistingCellDo(UnMarkItsDependents);
END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TCalcDocument.UserSelectionChanged; OVERRIDE;

	VAR
		userSelection:	TDesignator;

	BEGIN
	{ let the document's selection designator know that the designation has changed }
	userSelection := GetUserSelection;
	IF (userSelection <> NIL) & Member(userSelection, TRgnDesignator) THEN
		CopyRgn(fCellsView.fSelections, TRgnDesignator(userSelection).fDesignation);
	
	INHERITED UserSelectionChanged;
	END;

{***************************************************************************************************
	T C a l c W i n d o w
***************************************************************************************************}
{$S AOpen}

PROCEDURE TCalcWindow.IRes(itsDocument: TDocument;
						   itsSuperView: TView;
						   VAR itsParams: Ptr); OVERRIDE;

	VAR
		minSize:			Point;

	BEGIN
	INHERITED IRes(itsDocument, itsSuperView, itsParams);
	minSize.h := 284;
	minSize.v := 126;
	SetResizeLimits(minSize, gStdWSizeRect.botRight);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TCalcWindow.Draw(area: VRect); OVERRIDE;

	BEGIN
	{ draw the extra borders for the rows and columns views }
	PenNormal;

	{ rows }
	MoveTo(0, 35);
	Line(kRowTitleWidth - 1, 0);

	MoveTo(0, 35 + kCellHeight - 1);
	Line(kRowTitleWidth - 1, 0);

	{ columns }
	MoveTo(kRowTitleWidth - 1, 35);
	Line(0, kCellHeight - 1);

	INHERITED Draw(area);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TCalcWindow.DoKeyCommand(event: TToolboxEvent); OVERRIDE;
VAR
	entryView: TEntryView;
	cellsView: TCellsView; {!!! should have fields for these }
BEGIN
	{!!!RCR Clean up this mode-switching stuff.}
{	
	entryView := TCalcDocument(fDocument).fEntryView;
	cellsView := TCalcDocument(fDocument).fCellsView;
	IF entryView.fTEditing 
		& (event.fCharacter IN [chEnter,chTab,chRight,chLeft,chUp,chReturn, chDown]) THEN
		
		cellsView.DoKeyCommand(event)
	ELSE
		IF (NOT entryView.fTEditing) & ((event.fCharacter >= ' ') | (event.fCharacter = chBackspace)) THEN
			entryView.DoKeyCommand(event)

	ELSE
}
		INHERITED DoKeyCommand(event);
END;


{***************************************************************************************************
	T C e l l s V i e w
***************************************************************************************************}
{$S AOpen}

PROCEDURE TCellsView.ICellsView(itsDocument: TCalcDocument;
								forClipboard: BOOLEAN;
								itsParent: TView);

	BEGIN
	ITextGridView(itsDocument, itsParent, gZeroVPt, gZeroVPt, sizeVariable, sizeVariable,
				  itsDocument.fNoOfRows, itsDocument.fNoOfColumns, 0, kCellWidth, kAdorn, kAdorn, 0,
				  0, FALSE, gSystemStyle);

	fCalcDocument := itsDocument;
	fLastOptionKey := FALSE;							{ Used for DoSetCursor to the grabber hand }

	fIdleFreq := Max(GetCaretTime DIV 2, 1);			{ So we can trackCursor for the GrabberHand.
														 With MF 7.0 we could setup a VBL that
														 monitered the modifiers and called Wakeup
														 with our PID. Even better would be a call
														 that set the event manager so that an
														 event could be returned on modifier key
														 changes oh, well. }
	END;

{--------------------------------------------------------------------------------------------------}
{$S AOpen}

PROCEDURE TCellsView.IRes(itsDocument: TDocument;
						  itsSuperView: TView;
						  VAR itsParams: Ptr); OVERRIDE;

	BEGIN
	fCalcDocument := TCalcDocument(itsDocument);
	INHERITED IRes(itsDocument, itsSuperView, itsParams);
	fIdleFreq := Max(GetCaretTime DIV 2, 1);			{ So we can trackCursor for the GrabberHand.
														 With MF 7.0 we could setup a VBL that
														 monitered the modifiers and called Wakeup
														 with our PID. Even better would be a call
														 that set the event manager so that an
														 event could be returned on modifier key
														 changes oh, well. }
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TCellsView.ComputeBorderRgn(itsBorderRgn, itsDesignation: RgnHandle);

	BEGIN
	IF Focus THEN
		CellsToPixels(itsDesignation, itsBorderRgn);
	WITH itsBorderRgn^^.rgnBBox DO
		BEGIN
		top := top - 1;
		left := left - 1;
		END;
	InsetRgn(itsBorderRgn, -1, -1);						{ leave a 1 pixel margin }
	{$IFC qDebug}
	UseTempRgn('TCellsView.ComputeBorderRgn');
	{$ENDC qDebug}
	CopyRgn(itsBorderRgn, gTempRgn);
	InsetRgn(gTempRgn, -3, -3);							{ make a 3 pixel border }
	DiffRgn(gTempRgn, itsBorderRgn, itsBorderRgn);
	{$IFC qDebug}
	DoneWithTempRgn;
	{$ENDC qDebug}
	END;

{--------------------------------------------------------------------------------------------------}
{$S ANonRes}

PROCEDURE TCellsView.CreatePublisherAdorner(aPublisher: TPublisher);

	VAR
		aPublisherAdorner:	TPublisherAdorner;
		itsBorderRgn:		RgnHandle;

	BEGIN
	itsBorderRgn := MakeNewRgn;
	ComputeBorderRgn(itsBorderRgn, TRgnDesignator(aPublisher.fDesignator).fDesignation);
	
	New(aPublisherAdorner);
	aPublisherAdorner.IPublisherAdorner('publ', aPublisher, itsBorderRgn);
	
	AddAdorner(aPublisherAdorner, kLowAdornPriority, fCalcDocument.IsBorderShown(aPublisher));
	END;

{--------------------------------------------------------------------------------------------------}
{$S ANonRes}

PROCEDURE TCellsView.CreateSubscriberAdorner(aSubscriber: TSubscriber);

	VAR
		aSubscriberAdorner:	TSubscriberAdorner;
		itsBorderRgn:		RgnHandle;

	BEGIN
	itsBorderRgn := MakeNewRgn;
	ComputeBorderRgn(itsBorderRgn, TRgnDesignator(aSubscriber.fDesignator).fDesignation);
	
	New(aSubscriberAdorner);
	aSubscriberAdorner.ISubscriberAdorner('subs', aSubscriber, itsBorderRgn);
	
	AddAdorner(aSubscriberAdorner, kLowAdornPriority, fCalcDocument.IsBorderShown(aSubscriber));
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TCellsView.AdornRow(aRow: INTEGER; area: VRect); OVERRIDE;

	VAR
		qdArea:		Rect;

	BEGIN
	PenPat(gRowSeparatorPattern);
	PenSize(1, 1);

	ViewToQDRect(area, qdArea);
	WITH qdArea DO
		BEGIN
		MoveTo(left, bottom - 1);
		LineTo(right - 1, bottom - 1);
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TCellsView.AdornCol(aCol: INTEGER;
							  area: VRect); OVERRIDE;
	VAR
		qdArea:		Rect;

	BEGIN
	PenPat(gColumnSeparatorPattern);
	PenSize(1, 1);

	ViewToQDRect(area, qdArea);
	WITH qdArea DO
		BEGIN
		MoveTo(right - 1, top);
		LineTo(right - 1, bottom - 1);
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S AClipBoard}

FUNCTION TCellsView.ContainsClipType(aType: ResType): BOOLEAN;

	BEGIN
	ContainsClipType := aType = kCalcScrapType;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ANonRes}

FUNCTION TCellsView.DesignatorAsTEXT(aDesignator: TDesignator): Handle;

	VAR
		bounds:				Rect;
		textHandle:			Handle;
		aHandleStream:		THandleStream;
		theString:			Str255;
		aRgn:				RgnHandle;

	PROCEDURE DoToCell(aCell: GridCell);
	
		VAR
			aChar: SignedByte;
	
		BEGIN
		{ write out the cell's contents if the cell is in the designation & it exists }
		IF PtInRgn(aCell, aRgn) & fCalcDocument.CellExists(aCell.v, aCell.h) THEN
			BEGIN
			fCalcDocument.GetCell(aCell.v, aCell.h).GetValueAsString(theString);
			aHandleStream.WriteBytes(@theString[1], Length(theString));
			END;
		
		{ write out the delimitter - tab or cr }
		IF aCell.h = bounds.right THEN
			aChar := $0D
		ELSE
			aChar := $09;
		aHandleStream.WriteBytes(@aChar, sizeof(SignedByte));
		END;

	BEGIN
	textHandle := NewPermHandle(100);
	New(aHandleStream);
	aHandleStream.IHandleStream(textHandle, 10);
	aRgn := TRgnDesignator(aDesignator).fDesignation;
	bounds := aRgn^^.rgnBBox;
	WITH bounds DO
		BEGIN
		bottom := bottom - 1;
		right := right - 1;
		END;
	EachCellDo(bounds.topLeft, bounds.botRight, DoToCell);
	aHandleStream.Free;
	DesignatorAsTEXT := textHandle;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ANonRes}

FUNCTION TCellsView.DesignatorAsPICT(aDesignator: TDesignator): PicHandle;

	VAR
		thePixels:			RgnHandle;
		pHndl:				PicHandle;
		pictureIsOpen:		BOOLEAN;
		qdExtent:			Rect;
		viewRect:			VRect;
		err:				LONGINT;
		tempPort:			GrafPort;
		tempCPort:			CGrafPort;
		fi: 				FailInfo;

	PROCEDURE HdlAsPict(error: OSErr;
						message: LONGINT);

		BEGIN
		IF pHndl <> NIL THEN
			BEGIN
			IF pictureIsOpen THEN
				ClosePicture;
			KillPicture(pHndl);
			pHndl := NIL;
			END;

		IF qNeedsColorQD | gConfiguration.hasColorQD THEN
			CloseCPort(@tempCPort)
		ELSE
			ClosePort(@tempPort);
		SetPort(gWorkPort);
		BeInPort(NIL);
		gDrawingPictScrap := FALSE;
		gDrawingPictScrapView := NIL;
		END;

	BEGIN
	InvalidateFocus;

	gDrawingPictScrap := TRUE;
	gDrawingPictScrapView := SELF;
	IF qNeedsColorQD | gConfiguration.hasColorQD THEN
		BEGIN
		OpenCPort(@tempCPort);
		BeInPort(@tempCPort);
		END
	ELSE
		BEGIN
		OpenPort(@tempPort);
		BeInPort(@tempPort);
		END;
	{??? how do we catch openport failures? }

	pHndl := NIL;

	CatchFailures(fi, HdlAsPict);

	IF Focus THEN
		BEGIN
		{ determine the size of the view's selection in QD coord's }
		thePixels := MakeNewRgn;
		CellsToPixels(TRgnDesignator(aDesignator).fDesignation, thePixels);
		qdExtent := thePixels^^.rgnBBox;
		DisposeRgn(thePixels);
	
		pHndl := NIL;
		pHndl := OpenPicture(qdExtent);
		pictureIsOpen := TRUE;							{ Failure handler needs to close it }

		ClipRect(qdExtent);

		{ draw - all drawing will be recorded into the picture }
		QDToViewRect(qdExtent, viewRect);				{ convert size to view coord's }
		Draw(viewRect);

		ClosePicture;
		pictureIsOpen := FALSE; 						{ Failure handler doesn't need to close it }

  {On the 128K ROMs the picFrame will be empty if drawing the
   picture failed.	On the 64K ROM's QuickDraw simply bombs.}
		IF EmptyRect(pHndl^^.picFrame) THEN
			BEGIN
			IF qDebug THEN
				WRITELN('Picture frame is empty!');
			FailOSErr(memFullErr);
			END;

		InvalidateFocus;
		END
	ELSE IF qDebug THEN 								{ can't focus }
		ProgramBreak('Can''t focus view while writing picture to desk scrap');

	Success(fi);

	IF qNeedsColorQD | gConfiguration.hasColorQD THEN
		CloseCPort(@tempCPort)
	ELSE
		ClosePort(@tempPort);
	SetPort(gWorkPort);
	gDrawingPictScrap := FALSE;
	gDrawingPictScrapView := NIL;
	BeInPort(NIL);

	DesignatorAsPICT := pHndl;	
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

FUNCTION TCellsView.DoIdle(phase: IdlePhase): BOOLEAN;

	CONST
		kOptionKey			= $3A;

	VAR
		aKeyMap:			KeyMap;

	BEGIN
	DoIdle := FALSE;									{ Didn't free myself }

	GetKeys(aKeyMap);
	IF fLastOptionKey <> aKeyMap[kOptionKey] THEN
		BEGIN
		fLastOptionKey := aKeyMap[kOptionKey];
		gApplication.InvalidateCursorRgn;				{ Since we track the option key, we now need
														 to recalculate cursor rgn and SET the
														 cursor at next opportunity }
		END;

	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TCellsView.DoKeyCommand(event: TToolboxEvent);
 { This view only handles arrow keys, tab, return and enter.  It assumes
   the other keys are handled by the entry view object. }

	VAR
		r:					RowNumber;
		c:					ColumnNumber;
		aCell:				GridCell;
		minToSee:			VPoint;
		aRect:				VRect;

	BEGIN
	c := fCalcDocument.fEditColumn;
	r := fCalcDocument.fEditRow;

	CASE event.fCharacter OF
		chEnter:
			BEGIN
			{ Stay on same cell }
			END;
		chTab, chRight:
			c := Min(c + 1, fCalcDocument.fNoOfColumns);
		chLeft:
			c := Max(c - 1, 1);
		chUp:
			r := Max(r - 1, 1);
		chReturn, chDown:
			r := Min(r + 1, fCalcDocument.fNoOfRows);
		OTHERWISE
			BEGIN
			INHERITED DoKeyCommand(event);
			fCalcDocument.fEditCell := fCalcDocument.GetCell(r, c);
			EXIT(DoKeyCommand);
			END;
	END;

	aCell.h := c;
	aCell.v := r;

	WITH fCalcDocument DO
		BEGIN
		SetChangeCount(Max(fChangeCount + 1, 1));		{ enable Save - document may have changed }

		fEntryView.EditMode(FALSE);
		fColumnsView.SetEmptySelection(kHighlight);
		fRowsView.SetEmptySelection(kHighlight);
		fColumnIsSelected := FALSE;

		SelectCell(aCell, kDontExtend, kHighlight, kSelect);
		IF fEntryView.fTouched THEN
			DoRecalculate(NOT kForceAutomatic, kUpdateDependencies);
		ScrollSelectionIntoView(TRUE);

		{ fix the columns view }
		minToSee.h := fColumnsView.GetColWidth(aCell.h);
		minToSee.v := kCellHeight;
		aCell.v := 1;
		fColumnsView.CellToVRect(aCell, aRect);
		fColumnsView.RevealRect(aRect, minToSee, TRUE);

		{ fix the rows view }
		minToSee.h := kCellWidth;
		minToSee.v := kCellHeight;
		aCell.v := r;
		aCell.h := 1;
		fRowsView.CellToVRect(aCell, aRect);
		fRowsView.RevealRect(aRect, minToSee, TRUE);
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ASelCommand}

PROCEDURE TCellsView.DoMenuCommand(aCmdNumber: cmdNumber); OVERRIDE;

	VAR
		aCellEditCommand:	TCellEditCommand;
		aCellPasteCommand:	TCellPasteCommand;
		cellsToSelect:		RgnHandle;
		aRect:				Rect;

	BEGIN
	CASE aCmdNumber OF
		cSelectAll:
			BEGIN
			aRect := fCalcDocument.fInUseBounds;
			WITH aRect DO
				BEGIN
				right := right + 1;
				bottom := bottom + 1;
				END;
			cellsToSelect := MakeNewRgn;
			RectRgn(cellsToSelect, aRect);
			SetSelection(cellsToSelect, kDontExtend, kHighlight, kSelect);
			DisposeRgn(cellsToSelect);
			WITH fCalcDocument DO
				BEGIN
				fSelectionType := AllSelection;
				fRowsView.SetEmptySelection(kHighlight);
				fColumnsView.SetEmptySelection(kHighlight);
				fColumnIsSelected := FALSE;
				END;
			END;

		cCut, cCopy, cClear:
		{ If user isn't editing this cell, he must want the cell itself. }
			IF NOT fCalcDocument.fEntryView.fTEditing THEN
				BEGIN
				NEW(aCellEditCommand);
				aCellEditCommand.ICellEditCommand(fCalcDocument, aCmdNumber);
				PostCommand(aCellEditCommand);
				END
			ELSE
				BEGIN									{ get ready for TextEdit operation }
				WITH fCalcDocument DO
					fEditCell := GetCell(fEditRow, fEditColumn);
				INHERITED DoMenuCommand(aCmdNumber);
				END;

		cPaste:
			IF gClipboardMgr.fClipView.ContainsClipType(kCalcScrapType) THEN
				BEGIN
				NEW(aCellPasteCommand);
				aCellPasteCommand.ICellPasteCommand(fCalcDocument);
				PostCommand(aCellPasteCommand);
				END
			ELSE
				BEGIN									{ paste text into entry view }
				WITH fCalcDocument DO
					BEGIN
					fEditCell := GetCell(fEditRow, fEditColumn);
					IF NOT fEntryView.fTEditing THEN
						fEntryView.SetEditMode; 		{ prepare view for paste of text }
					END;
				INHERITED DoMenuCommand(aCmdNumber);
				END;

		OTHERWISE
			INHERITED DoMenuCommand(aCmdNumber);
	END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ASelCommand}

PROCEDURE TCellsView.DoMouseCommand(VAR theMouse: VPoint;
									event: TToolboxEvent;
									hysteresis: Point); OVERRIDE;

	VAR
		aCellSelector:		TCalcSelectCommand;
		aColumnSizer:		TColumnSizer;
		aCell:				GridCell;
		whichPart:			GridViewPart;
		aRow:				INTEGER;
		aCol:				INTEGER;
		aGrabber:			TGrabberTracker;

	BEGIN
	whichPart := IdentifyPoint(theMouse, aRow, aCol);
	aCell.h := aCol;
	aCell.v := aRow;

	IF event.fOptionKey THEN
		BEGIN
		NEW(aGrabber);
		aGrabber.IGrabberTracker(cNoCommand, fCalcDocument, SELF, GetScroller(FALSE), theMouse);
		PostCommand(aGrabber);
		END
	ELSE
		BEGIN
		CASE whichPart OF
			inCell:
				BEGIN
				NEW(aCellSelector);
				aCellSelector.ICalcSelectCommand(fCalcDocument, SELF, theMouse, event.fShiftKey,
												 event.fCmdKey);
				PostCommand(aCellSelector);
				fCalcDocument.fSelectionType := CellSelection;
				END;

			inColumn, inVertex:
				BEGIN
				IF aCol > 1 THEN
					BEGIN
					NEW(aColumnSizer);
					aColumnSizer.IColumnSizer(fCalcDocument, aCol - 1);
					PostCommand(aColumnSizer);
					END;
				END;

			OTHERWISE;
		END;
		fCalcDocument.fColumnsView.SetEmptySelection(kHighlight);
		fCalcDocument.fRowsView.SetEmptySelection(kHighlight);
		fCalcDocument.fColumnIsSelected := FALSE;
		fCalcDocument.fEditCell := NIL;
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TCellsView.DoSetCursor(localPoint: VPoint;
								cursorRgn: RgnHandle); OVERRIDE;

	CONST
		kOptionKey			= $3A;

	VAR
		aRow:				INTEGER;
		aCol:				INTEGER;
		aKeyMap:			KeyMap;
		cellsExtent:		VRect;
		cellsQDExtent, columnQDExtent: Rect;

	BEGIN
	GetKeys(aKeyMap);

	{ We may have been called from a mouse-moved.  Be kind to the DoIdle }
	fLastOptionKey := aKeyMap[kOptionKey];

	IF fLastOptionKey THEN
		BEGIN
		SetCursor(GetCursor(kGrabberHand)^^);
		GetQDExtent(cellsQDExtent);
		RectRgn(cursorRgn, cellsQDExtent);
		END
	ELSE
		CASE IdentifyPoint(localPoint, aRow, aCol) OF
			badChoice: ;
			inColumn, inVertex:
				IF aCol > 1 THEN
					BEGIN
					SetCursor(GetCursor(kColumnSizingCursor)^^);
					ColToVRect(Min(aCol, fNumOfCols), 1, cellsExtent);
					ViewToQDRect(cellsExtent, cellsQDExtent);
					columnQDExtent := cellsQDExtent;

					{ Which edge is the mouse closer to? }
					IF abs(columnQDExtent.right - localPoint.h) < abs(columnQDExtent.left -
																	  localPoint.h) THEN
						columnQDExtent.left := columnQDExtent.right;

					columnQDExtent.left := columnQDExtent.left - fColInset DIV 2;
					columnQDExtent.right := columnQDExtent.left + fColInset;
					RectRgn(cursorRgn, columnQDExtent);
					END;
			inRow, inCell:
				BEGIN
				SetCursor(GetCursor(plusCursor)^^);
				ColToVRect(Min(aCol, fNumOfCols), 1, cellsExtent);
				InsetVRect(cellsExtent, fColInset DIV 2, 0); { Account for the column resizer }
				ViewToQDRect(cellsExtent, cellsQDExtent);
				RectRgn(cursorRgn, cellsQDExtent);
				END;
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TCellsView.DoSetupMenus; OVERRIDE;

	VAR
		selection:			TypeOfSelection;

	BEGIN
	INHERITED DoSetupMenus;

	{ If user isn't editing, then assume edit commands refer to cells }
	IF NOT fCalcDocument.fEntryView.fTEditing THEN
		BEGIN
		SetEditCmdName(cCut, cCutCells);
		SetEditCmdName(cCopy, cCopyCells);
		SetEditCmdName(cClear, cClearCells);

		gClipboardMgr.CanPaste(kCalcScrapType);
		END;

	selection := fCalcDocument.fSelectionType;
	Enable(cCut, selection <> NoSelection);
	Enable(cCopy, selection <> NoSelection);
	Enable(cClear, selection <> NoSelection);
	Enable(cSelectAll, TRUE);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TCellsView.DrawCell(aCell: GridCell;
							  aRect: VRect); OVERRIDE;

	VAR
		theCell:			TCell;
		theString:			Str255;
		qdArea: Rect;

	BEGIN
	theCell := fCalcDocument.GetExistingCell(aCell.v, aCell.h);
	IF (theCell <> NIL) THEN
		WITH theCell DO
			BEGIN
			GetValueAsString(theString);
			ViewToQDRect(aRect, qdArea);
			SmartDrawString(theString, qdArea,
							fCalcDocument.GetColumn(aCell.h).fFormat.fJustification);
			END;

	END;

{--------------------------------------------------------------------------------------------------}
{$S AFields}

PROCEDURE TCellsView.Fields(obj: TObject); OVERRIDE;

	BEGIN
	WITH obj DO
		BEGIN
		DoToField(AtStr('TCellsView'), NIL, bClass);
		DoToField(AtStr('fCalcDocument'), @fCalcDocument, bObject);
		END;

	INHERITED Fields(obj);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TCellsView.GetVisibleCells(VAR visibleCells: Rect);

	VAR
		visibleVRect:		VRect;

	BEGIN
	IF Focus THEN;										{ appease QDToViewRect }
	GetVisibleRect(visibleVRect);
	visibleVRect.topLeft := GetScroller(TRUE).fTranslation;
	visibleCells.topLeft := VPointToCell(visibleVRect.topLeft);
	visibleCells.botRight := VPointToCell(visibleVRect.botRight);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

FUNCTION TCellsView.IsCellVisible(aCell: GridCell): BOOLEAN;

	VAR
		visibleCells:		Rect;

	BEGIN
	GetVisibleCells(visibleCells);
	IsCellVisible := PtInRect(aCell, visibleCells); 	{ ??? use fCalcDocument.CellInRange }
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TCellsView.PositionAtCell(aCell: GridCell);

	VAR
		aRect:				VRect;
		minToSee:			VPoint;
		r:					INTEGER;

	BEGIN
	{ position the cells view first }
	CellToVRect(aCell, aRect);
	WITH aRect DO
		SetVPt(minToSee, right - left, bottom - top);
	RevealRect(aRect, minToSee, TRUE);

	WITH fCalcDocument DO
		BEGIN
		{ fix up the columns view }
		r := aCell.v;
		minToSee.h := fColumnsView.GetColWidth(aCell.h);
		minToSee.v := kCellHeight;
		aCell.v := 1;
		fColumnsView.CellToVRect(aCell, aRect);
		fColumnsView.RevealRect(aRect, minToSee, TRUE);

		{ fix up the rows view }
		minToSee.h := kCellWidth;
		minToSee.v := kCellHeight;
		aCell.v := r;
		aCell.h := 1;
		fRowsView.CellToVRect(aCell, aRect);
		fRowsView.RevealRect(aRect, minToSee, TRUE);
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TCellsView.ReSelect(cellRegion: RgnHandle);

	VAR
		aCell:				GridCell;

	BEGIN
	aCell := cellRegion^^.rgnBBox.topLeft;
	IF NOT IsCellVisible(aCell) THEN
		PositionAtCell(aCell);							{ position cellRegion at top left of grid }

	IF NOT EqualRect(cellRegion^^.rgnBBox, fSelections^^.rgnBBox) THEN
		BEGIN
		WITH fCalcDocument DO
			BEGIN
			fColumnsView.SetEmptySelection(kHighlight);
			fRowsView.SetEmptySelection(kHighlight);
			fColumnIsSelected := FALSE;
			END;
		SetSelection(cellRegion, kDontExtend, kHighlight, kSelect);
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TCellsView.ReSelectCell(aCell: GridCell);

	VAR
		cellRect:			Rect;

	BEGIN
	IF NOT IsCellVisible(aCell) THEN
		PositionAtCell(aCell);							{ position aCell at top left of grid }

	WITH aCell DO
		SetRect(cellRect, h, v, h + 1, v + 1);
	IF NOT EqualRect(cellRect, fSelections^^.rgnBBox) THEN
		BEGIN
		WITH fCalcDocument DO
			BEGIN
			fColumnsView.SetEmptySelection(kHighlight);
			fRowsView.SetEmptySelection(kHighlight);
			fColumnIsSelected := FALSE;
			END;
		SelectCell(aCell, kDontExtend, kHighlight, kSelect);
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TCellsView.ScrollSelectionIntoView(redraw: BOOLEAN); OVERRIDE;

	VAR
		topLeftRect:		VRect;
		minToSee:			VPoint;

	BEGIN
	IF NOT (EmptyRgn(fSelections)) THEN
		BEGIN
		CellToVRect(fSelections^^.rgnBBox.topLeft, topLeftRect);
		WITH topLeftRect DO
			SetVPt(minToSee, right - left, bottom - top);
		RevealRect(topLeftRect, minToSee, redraw);
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TCellsView.SetCell(aCell: GridCell);

	BEGIN
	fCalcDocument.EditCell; 							{ cell formula := string in entry view }
	InvalidateCell(aCell);								{ redraw the cell }
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TCellsView.SetColWidth(aCol: INTEGER;
								numOfCols: INTEGER;
								aWidth: INTEGER); OVERRIDE;
	
	VAR
		i: ArrayIndex;

	PROCEDURE DoToCell(aCell: TCell);
		
		BEGIN
		IF (aCell.fColumn = aCol)  THEN
			aCell.Changed(mWidthChanged, SELF);
		END;

	FUNCTION DoToAdorner(elementIndex: ArrayIndex): Boolean;
	
		VAR
			anAdorner: TAdorner;
			aSectionAdorner: TSectionAdorner;

		BEGIN
		anAdorner := AdornerElementPtr(fViewAdornment.fAdorners.ComputeAddress(elementIndex))^.adorner;
		IF (anAdorner <> NIL) & Member(anAdorner, TSectionAdorner) THEN
			BEGIN
			aSectionAdorner := TSectionAdorner(anAdorner);
			ComputeBorderRgn(aSectionAdorner.fBorderRgn,
							TRgnDesignator(aSectionAdorner.fSection.fDesignator).fDesignation);
			END;
		DoToAdorner := FALSE;
		END;

	BEGIN
	INHERITED SetColWidth(aCol, numOfCols, aWidth);
	
	{ notify the cells that the width was changed }
	fCalcDocument.EachExistingCellDo(DoToCell); 

	{ recompute the border rgns }
	i := fViewAdornment.fAdorners.EachElementDoTil(DoToAdorner, kIterateForward);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TCellsView.SetSelection(cellsToSelect: RgnHandle;
								  extendSelection, highlight, select: BOOLEAN); OVERRIDE;

	VAR
		aQDRect:			Rect;
		cellTopLeft:		Point;

	BEGIN
	INHERITED SetSelection(cellsToSelect, extendSelection, highlight, select);

	WITH fCalcDocument DO
		BEGIN
		IF fEntryView.fTouched THEN 					{ "commit" last cell }
			BEGIN
			EditCell;									{ change fEditCell's formula to the string
														 in fEntryView }
			fEntryView.SetToString('');
			END;

		cellTopLeft := cellsToSelect^^.rgnBBox.topLeft;
		IF NOT extendSelection & NOT EqualPt(cellTopLeft, gZeroPt) THEN
			BEGIN
			fEditColumn := cellTopLeft.h;
			fEditRow := cellTopLeft.v;
			fEditCell := fCells.GetCell(fEditRow, fEditColumn);

			fCoordView.ForceRedraw;
			END;

		SetEntry(fEditRow, fEditColumn);				{ set entry view contents to new cell }
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S AClipBoard}

PROCEDURE TCellsView.WriteCalcScrap(calcScrap: Handle);

	VAR
		scrapOffset:		LONGINT;
		scrapInfo:			ScrapInfoRecord;
		cellsWritten:		INTEGER;
		i:					INTEGER;
		r:					Rect;

{--------------------------------------------------------------------------------------------------}

	PROCEDURE WriteCellToScrap(aCell: TCell);

		BEGIN
			aCell.WriteToScrap(calcScrap, scrapOffset);
			cellsWritten := cellsWritten + 1;
		END;

	BEGIN
	SetHandleSize(calcScrap, 0);
	scrapOffset := 0;

	scrapInfo.selection.top := 1;
	scrapInfo.selection.left := 1;
	scrapInfo.selection.bottom := fCalcDocument.fNoOfRows;
	scrapInfo.selection.right := fCalcDocument.fNoOfColumns;
	scrapInfo.noOfCells := fCalcDocument.fAllocatedCells;
	WriteScrap(calcScrap, scrapOffset, @scrapInfo, SIZEOF(scrapInfo));

	cellsWritten := 0;
	WITH fCalcDocument DO
		BEGIN
		FOR i := 1 TO fCalcDocument.fNoOfRows DO
			BEGIN
			WITH fCalcDocument.GetRow(i) DO
				WriteToScrap(calcScrap, scrapOffset);
			END;

		FOR i := 1 TO fCalcDocument.fNoOfColumns DO
			BEGIN
			WITH fCalcDocument.GetColumn(i) DO
				WriteToScrap(calcScrap, scrapOffset);
			END;
		EachExistingCellDo(WriteCellToScrap);
		END;

{$IFC qDebug}
	WRITELN('WriteCalcScrap: Number of cells written: ', cellsWritten: 0);
	IF cellsWritten <> scrapInfo.noOfCells THEN
		BEGIN
		WRITELN('WriteCalcScrap: Incorrect number of cells written.');
		WRITELN('     Should be ', scrapInfo.noOfCells: 0, ', was ', cellsWritten: 0);
		ProgramBreak('');
		END;
{$ENDC}

	END;

{--------------------------------------------------------------------------------------------------}
{$S AClipBoard}

PROCEDURE TCellsView.WriteTextScrap(textScrap: Handle);

	VAR
		r:					RowNumber;
		c:					ColumnNumber;
		theText:			Str255;
		scrapOffset:		LONGINT;
		savedPort:			GrafPtr;

	BEGIN
	GetPort(savedPort); 								{ Use work port because GetValueAsString }
	SetPort(gWorkPort); 								{ sets the current port's font }

	SetHandleSize(textScrap, 0);
	scrapOffset := 0;

	FOR r := 1 TO fCalcDocument.fNoOfRows DO
		BEGIN
		FOR c := 1 TO fCalcDocument.fNoOfColumns DO
			BEGIN
			IF fCalcDocument.CellExists(r, c) THEN
				fCalcDocument.GetCell(r, c).GetValueAsString(theText)
			ELSE
				theText := '';
			IF c > 1 THEN
				theText := CONCAT(chTab, theText);
			WriteScrap(textScrap, scrapOffset, POINTER(ORD4(@theText) + 1), LENGTH(theText));
			END;
		theText := chReturn;
		WriteScrap(textScrap, scrapOffset, POINTER(ORD4(@theText) + 1), LENGTH(theText));
		END;

	SetPort(savedPort);
	END;

{--------------------------------------------------------------------------------------------------}
{$S AClipBoard}

PROCEDURE TCellsView.WriteToDeskScrap; OVERRIDE;

	VAR
		textScrap:			Handle;
		calcScrap:			Handle;
		err:				OSErr;

	BEGIN
	textScrap := NewPermHandle(0);
	WriteTextScrap(textScrap);
	err := gClipboardMgr.PutDeskScrapData(kTextScrapType, textScrap);
	textScrap := DisposeIfHandle(textScrap);

	FailOSErr(err);

	calcScrap := NewPermHandle(0);
	WriteCalcScrap(calcScrap);
	err := gClipboardMgr.PutDeskScrapData(kCalcScrapType, calcScrap);
	calcScrap := DisposeIfHandle(calcScrap);

	FailOSErr(err);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ANonRes}

FUNCTION TCellsView.DoBreakFollowing(vhs: VHSelect;
									 prevBreak: VCoordinate;
									 VAR Automatic: BOOLEAN): VCoordinate; OVERRIDE;
{ Determines where page breaks occur for printing. }

	VAR
		thisBreak:			VCoordinate;
		rowsPerPage:		INTEGER;
		totalWidth: 		INTEGER;
		width:				INTEGER;
		pageWidth:			INTEGER;
		extentRect: 		VRect;
		firstCol:			ColumnNumber;
		c:					ColumnNumber;

	FUNCTION ColumnAtCoord(loc: VCoordinate): INTEGER;

		BEGIN
		ColumnAtCoord := 1;
		IF loc = 0 THEN
			EXIT(ColumnAtCoord);

		c := 0;
		width := 0;
		REPEAT
			c := c + 1;
			width := width + GetColWidth(c);
		UNTIL width >= loc;
		ColumnAtCoord := c + 1;
		END;

	BEGIN
	GetExtent(extentRect);
	CASE vhs OF
		h:
			BEGIN
			rowsPerPage := fPrintHandler.fViewPerPage.v DIV kCellHeight;
			thisBreak := prevBreak + (rowsPerPage * kCellHeight);
			END;
		v:
			BEGIN
			pageWidth := fPrintHandler.fViewPerPage.h;
			totalWidth := 0;
			firstCol := ColumnAtCoord(prevBreak);
			FOR c := firstCol TO fCalcDocument.fNoOfColumns DO
				BEGIN
				width := GetColWidth(c);
				IF totalWidth + width <= pageWidth THEN
					totalWidth := totalWidth + width
				ELSE
					BEGIN
					thisBreak := prevBreak + totalWidth;
					LEAVE;
					END;
				END;
			IF thisBreak = prevBreak THEN				{ Prevent  loop resizing a far-right column
														 (L.T.!) }
				thisBreak := extentRect.right;
			END;
	END;
	thisBreak := Min(thisBreak, extentRect.botRight.vh[gOrthogonal[vhs]]);

{$IFC qDebug}
	IF (thisBreak <= prevBreak) | gDebugPrinting THEN
		BEGIN
		WRITE('TCellsView.DoBreakFollowing: prevBreak=');
		IF vhs = v THEN
			WRITELN('[v]', prevBreak, ', thisBreak=[v]', thisBreak)
		ELSE
			WRITELN('[h]', prevBreak, ', thisBreak=[h]', thisBreak);

		IF thisBreak <= prevBreak THEN
			ProgramBreak('thisBreak <= prevBreak');
		END;
{$ENDC}

	DoBreakFollowing := thisBreak;
	END;

{--------------------------------------------------------------------------------------------------}
{$IFC qDebug}
{$S ANonRes}

PROCEDURE TCellsView.DoDrawPageBreak(vhs: VHSelect;
									 whichBreak: INTEGER;
									 loc: VCoordinate;
									 Automatic: BOOLEAN); OVERRIDE;

	VAR
		vPt:				VPoint;
		qdStartPt:			Point;
		qdEndPt:			Point;

	BEGIN
	IF gDebugPrinting THEN
		BEGIN
		vPt.vh[gOrthogonal[vhs]] := loc;
		vPt.vh[vhs] := 0;
		qdStartPt := ViewToQDPt(vPt);
		vPt.vh[vhs] := fSize.vh[vhs] - gBreaksPenState.pnSize.vh[vhs];
		qdEndPt := ViewToQDPt(vPt);

		MoveTo(qdStartPt.h, qdStartPt.v);
		LineTo(qdEndPt.h, qdEndPt.v);
		END;
	END;
{$ENDC}

{--------------------------------------------------------------------------------------------------}
{$S ANonRes}

PROCEDURE TCellsView.GetPrintExtent(VAR printExtent: VRect); OVERRIDE;
{ Overridden to provide for Print Selection command. }

	VAR
		aRect:				VRect;
		tlCell: 			GridCell;
		brCell: 			GridCell;

	BEGIN
	IF TCalcPrintHandler(fPrintHandler).fCmdNumber = cPrintSelection THEN
		BEGIN
		tlCell := fSelections^^.rgnBBox.topLeft;
		brCell := fSelections^^.rgnBBox.botRight;
		brCell.h := Min(brCell.h - 1, fCalcDocument.fInUseBounds.right);
		brCell.v := Min(brCell.v - 1, fCalcDocument.fInUseBounds.bottom);
		END
	ELSE
		BEGIN
		tlCell := fCalcDocument.fInUseBounds.topLeft;
		brCell := fCalcDocument.fInUseBounds.botRight;
		END;

	CellToVRect(tlCell, aRect);
	printExtent.topLeft := aRect.topLeft;
	CellToVRect(brCell, aRect);
	printExtent.botRight := aRect.botRight;

{$IFC qDebug}
	IF gDebugPrinting THEN
		BEGIN
		WrLblVRect('printExtent ', printExtent);
		WRITELN;
		END;
{$ENDC}
	END;

{***************************************************************************************************
	T R o w s V i e w
***************************************************************************************************}
{$S AOpen}

PROCEDURE TRowsView.IRes(itsDocument: TDocument;
						 itsSuperView: TView;
						 VAR itsParams: Ptr); OVERRIDE;

	BEGIN
	INHERITED IRes(itsDocument, itsSuperView, itsParams);
	fCalcDocument := TCalcDocument(itsDocument);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TRowsView.AdornRow(aRow: INTEGER;
							 area: VRect); OVERRIDE;
	VAR
		qdArea:	Rect;

	BEGIN
	PenSize(1, 1);
	PenPat(black);

	ViewToQDRect(area, qdArea);
	With qdArea DO
		BEGIN
		{ right line }
		MoveTo(right - 1, top);
		LineTo(right - 1, bottom - 1);
	
		{ bottom line }
		MoveTo(left, bottom - 1);
		LineTo(right - 1, bottom - 1);
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ASelCommand}

PROCEDURE TRowsView.DoMouseCommand(VAR theMouse: VPoint;
								   event: TToolboxEvent;
								   hysteresis: Point); OVERRIDE;

	VAR
		aRowSelector:		TRowSelector;

	BEGIN
	NEW(aRowSelector);
	aRowSelector.IRowSelector(fCalcDocument, SELF, theMouse, event.fShiftKey, event.fCmdKey);
	PostCommand(aRowSelector);
	WITH fCalcDocument DO
		BEGIN
		fSelectionType := RowSelection;
		fColumnsView.SetEmptySelection(kHighlight);
		fColumnIsSelected := FALSE;
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TRowsView.DoSetCursor(localPoint: VPoint;
							   cursorRgn: RgnHandle); OVERRIDE;

	BEGIN
	{!!! DoSetCursor := FALSE; }
	INHERITED DoSetCursor(localPoint,cursorRgn);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TRowsView.DrawCell(aCell: GridCell;
							 aRect: VRect); OVERRIDE;

	VAR
		theString:			Str255;
		qdArea:				Rect;

	BEGIN
	NumToString(aCell.v, theString);
	ViewToQDRect(aRect, qdArea);
	WITH qdArea DO 									{ aesthetic adjustment of the rect }
		top := top + 2;
	MADrawString(@theString, qdArea, teCenter);
	END;

{--------------------------------------------------------------------------------------------------}
{$S AFields}

PROCEDURE TRowsView.Fields(obj: TObject); OVERRIDE;

	BEGIN
	WITH obj DO
		BEGIN
		DoToField(AtStr('TRowsView'), NIL, bClass);
		DoToField(AtStr('fCalcDocument'), @fCalcDocument, bObject);
		END;

	INHERITED Fields(obj);
	END;

{***************************************************************************************************
	T C o l u m n s V i e w
***************************************************************************************************}
{$S AOpen}

PROCEDURE TColumnsView.IRes(itsDocument: TDocument;
							itsSuperView: TView;
							VAR itsParams: Ptr); OVERRIDE;

	BEGIN
	INHERITED IRes(itsDocument, itsSuperView, itsParams);
	fCalcDocument := TCalcDocument(itsDocument);
	SetRowHeight(1, fNumOfRows, kCellHeight);
	fCalcDocument.fColumnIsSelected := FALSE;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TColumnsView.AdornCol(aCol: INTEGER;
								area: VRect); OVERRIDE;

VAR
	qdArea:		Rect;

	BEGIN
	PenPat(black);
	PenSize(1, 1);

	ViewToQDRect(area, qdArea);

	{ top line }
	WITH qdArea DO
		BEGIN
		MoveTo(left, top);
		LineTo(right - 1, top);
	
		{ bottom line }
		MoveTo(left, bottom - 1);
		LineTo(right - 1, bottom - 1);
	
		{ right line }
		MoveTo(right - 1, top);
		LineTo(right - 1, bottom - 1);
		END;

	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TColumnsView.CoordToString(coord: INTEGER;
									 VAR theString: Str255);

	BEGIN
	coord := coord - 1;
	IF coord < 26 THEN
		BEGIN
		theString := ' ';
		theString[1] := CHR(ORD('A') + coord);
		END
	ELSE
		BEGIN
		theString := '  ';
		theString[1] := CHR(ORD('A') + (coord DIV 26) - 1);
		theString[2] := CHR(ORD('A') + (coord MOD 26));
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ASelCommand}

PROCEDURE TColumnsView.DoMouseCommand(VAR theMouse: VPoint;
									  event: TToolboxEvent;
									  hysteresis: Point); OVERRIDE;

	VAR
		aColumnSelector:	TColumnSelector;
		aColumnSizer:		TColumnSizer;
		aRow:				INTEGER;
		aCol:				INTEGER;
		whichPart:			GridViewPart;
		aCell:				GridCell;

	BEGIN
	whichPart := IdentifyPoint(theMouse, aRow, aCol);
	aCell.h := aCol;
	aCell.v := aRow;

	CASE whichPart OF
		inCell:
			BEGIN
			fCalcDocument.fColumnIsSelected := TRUE;
			NEW(aColumnSelector);
			aColumnSelector.IColumnSelector(fCalcDocument, SELF, theMouse, event.fShiftKey,
											event.fCmdKey);
			PostCommand(aColumnSelector);
			fCalcDocument.fSelectionType := ColumnSelection;
			END;

		inColumn, inVertex:
			BEGIN
			IF aCol > 1 THEN
				BEGIN
				NEW(aColumnSizer);
				aColumnSizer.IColumnSizer(fCalcDocument, aCol - 1);
				PostCommand(aColumnSizer);
				END;
			END;

		OTHERWISE;
	END;
	fCalcDocument.fRowsView.SetEmptySelection(kHighlight);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TColumnsView.DoSetCursor(localPoint: VPoint;
								  cursorRgn: RgnHandle); OVERRIDE;

	VAR
		aRow:				INTEGER;
		aCol:				INTEGER;
		cellsExtent:		VRect;
		cellsQDExtent, columnQDExtent: Rect;

	BEGIN

	CASE IdentifyPoint(localPoint, aRow, aCol) OF
		badChoice: ;
		inColumn, inVertex:
			IF aCol > 1 THEN
				BEGIN
				SetCursor(GetCursor(kColumnSizingCursor)^^);
				ColToVRect(Min(aCol, fNumOfCols), 1, cellsExtent);
				ViewToQDRect(cellsExtent, cellsQDExtent);
				columnQDExtent := cellsQDExtent;

				{ Which edge is the mouse closer to? }
				IF abs(columnQDExtent.right - localPoint.h) < abs(columnQDExtent.left -
																  localPoint.h) THEN
					columnQDExtent.left := columnQDExtent.right;

				columnQDExtent.left := columnQDExtent.left - fColInset DIV 2;
				columnQDExtent.right := columnQDExtent.left + fColInset;
				RectRgn(cursorRgn, columnQDExtent);
				END;
		inRow, inCell:
			BEGIN
			SetCursor(arrow);
			ColToVRect(Min(aCol, fNumOfCols), 1, cellsExtent);
			InsetVRect(cellsExtent, fColInset DIV 2, 0); { Account for the column resizer }
			ViewToQDRect(cellsExtent, cellsQDExtent);
			RectRgn(cursorRgn, cellsQDExtent);
			END;
	END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TColumnsView.DrawCell(aCell: GridCell;
								aRect: VRect); OVERRIDE;

	VAR
		theString:			Str255;
		qdArea:				Rect;

	BEGIN
	CoordToString(aCell.h, theString);
	WITH aRect DO 									{ aesthetic adjustment of the rect }
		top := top + 2;
	ViewToQDRect(aRect, qdArea);
	MADrawString(@theString, qdArea, teCenter);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TColumnsView.ReSelect(cellRegion: RgnHandle);

	VAR
		aCell:				GridCell;
		cellsInColumn:		RgnHandle;
		cellsToSelect:		RgnHandle;

{--------------------------------------------------------------------------------------------------}

	PROCEDURE GetColumnCells(columnCell: GridCell);

		VAR
			aRect:				Rect;

		BEGIN
		SetRect(aRect, columnCell.h, 1, columnCell.h + 1, fCalcDocument.fCellsView.fNumOfRows + 1);
		RectRgn(cellsInColumn, aRect);
		UnionRgn(cellsInColumn, cellsToSelect, cellsToSelect);
		END;

	BEGIN
	aCell := cellRegion^^.rgnBBox.topLeft;
	IF NOT EqualRect(cellRegion^^.rgnBBox, fSelections^^.rgnBBox) THEN
		BEGIN											{ selection has changed }
		WITH fCalcDocument DO
			BEGIN
			fCellsView.SetEmptySelection(kHighlight);
			fRowsView.SetEmptySelection(kHighlight);
			fColumnIsSelected := TRUE;
			END;
		SetSelection(cellRegion, kDontExtend, kHighlight, kSelect);
		cellsToSelect := MakeNewRgn;
		cellsInColumn := MakeNewRgn;
		EachSelectedCellDo(GetColumnCells); 			{ add cells in the column to cellsToSelect }
		fCalcDocument.fCellsView.SetSelection(cellsToSelect, kDontExtend, kHighlight, kSelect);
		DisposeRgn(cellsToSelect);
		DisposeRgn(cellsInColumn);
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S AFields}

PROCEDURE TColumnsView.Fields(obj: TObject); OVERRIDE;

	BEGIN
	WITH obj DO
		BEGIN
		DoToField(AtStr('TColumnsView'), NIL, bClass);
		DoToField(AtStr('fCalcDocument'), @fCalcDocument, bObject);
		END;

	INHERITED Fields(obj);
	END;

{***************************************************************************************************
	T C a l c P r i n t H a n d l e r
***************************************************************************************************}
{$S ANonRes}

PROCEDURE TCalcPrintHandler.CalcViewPerPage(VAR amtPerPage: VPoint); OVERRIDE;

	VAR
		noOfRows:			INTEGER;

	BEGIN
	INHERITED CalcViewPerPage(amtPerPage);
	noOfRows := amtPerPage.v DIV kCellHeight;
	amtPerPage.v := noOfRows * kCellHeight;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ANonRes}

PROCEDURE TCalcPrintHandler.DoMenuCommand ( aCmdNumber: CmdNumber ); OVERRIDE;
{ Overridden to handle Print Selection. }

	BEGIN
	fCmdNumber := aCmdNumber;							{ Save cmd number for GetPrintExtent }
	IF aCmdNumber = cPrintSelection THEN
		aCmdNumber := cPrint;							{ proceed like regular Print }
	INHERITED DoMenuCommand(aCmdNumber);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ANonRes}

PROCEDURE TCalcPrintHandler.DoSetupMenus; OVERRIDE;
{ Overridden to handle Print Selection. }

	BEGIN
	INHERITED DoSetupMenus;
	Enable(cPrintSelection, gCouldPrint & (fView <> NIL));
	END;

{***************************************************************************************************
	T E n t r y V i e w
***************************************************************************************************}
{$S AOpen}

PROCEDURE TEntryView.IRes(itsDocument: TDocument;
						  itsSuperView: TView;
						  VAR itsParams: Ptr); OVERRIDE;

	VAR
		minSize:			Point;

	BEGIN
	INHERITED IRes(NIL, itsSuperView, itsParams);		{ set itsDocument to NIL since this guy
														 doesn't affect the document }
	fCalcDocument := TCalcDocument(itsDocument);
	fTouched := FALSE;
	fTEditing := FALSE;
	fFirstEdit := FALSE;
	fOldString := '';
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TEntryView.DoKeyCommand(event: TToolboxEvent); OVERRIDE;

	BEGIN
	{ If this is the first character, wipe out old value and activate caret. }
	IF (NOT fTEditing) & ((event.fCharacter >= ' ') | (event.fCharacter = chBackspace)) THEN
		SetEditMode;
	INHERITED DoKeyCommand(event);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ASelCommand}

FUNCTION TEntryView.DoMakeTypingCommand(Ch: CHAR): TTETypingCommand; OVERRIDE;

	VAR
		aTypingCommand: 	TCalcTypingCommand;

	BEGIN
	NEW(aTypingCommand);
	aTypingCommand.ITETypingCommand(SELF, Ch);
	DoMakeTypingCommand := aTypingCommand;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ASelCommand}

PROCEDURE TEntryView.DoMouseCommand(VAR theMouse: VPoint;
									event: TToolboxEvent;
									hysteresis: Point); OVERRIDE;

	BEGIN
	{ If no characters typed, active caret, then handle mouse down. }
	IF NOT fTouched THEN
		BecameApplicationTarget;
	EditMode(TRUE);
	INHERITED DoMouseCommand(theMouse, event, hysteresis);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TEntryView.DoSetupMenus; OVERRIDE;

	BEGIN
	{ TTEView.DoSetupMenus will setup the Edit menu commands for us. }
	INHERITED DoSetupMenus;

	IF fTEditing THEN
		BEGIN
		SetEditCmdName(cCut, cCutText);
		SetEditCmdName(cCopy, cCopyText);
		SetEditCmdName(cClear, cClearText);
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TEntryView.Draw(area: VRect); OVERRIDE;

	VAR
		r:					Rect;

	BEGIN
	INHERITED Draw(area);

	{ We want a rectangle around the whole view }
	PenSize(1, 1);
	PenPat(black);
	GetQDExtent(r);
	FrameRect(r);

	INHERITED Draw(area);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

FUNCTION TEntryView.WantToBecomeTarget : BOOLEAN; OVERRIDE;
BEGIN
	{ Return FALSE for now, because Calc currently relies on the
	cells view being the target }
	WantToBecomeTarget := FALSE; 
END;



{--------------------------------------------------------------------------------------------------}
{$S ASelCommand}

PROCEDURE TEntryView.EditMode(editing: BOOLEAN);

	VAR
		lastCommand:		TCommand;

	BEGIN
	fTEditing := editing;
	fFirstEdit := FALSE;								{ set to TRUE only by DoKeyCommand }

	IF editing THEN
		BEGIN
{$Push} {$H-}
		GetAsString(fOldString);						{ save previous string for Undo/Redo }
{$Pop}
		END
	ELSE
		BEGIN											{ disable undo/redo for TTECommands. ???
														 There must be a better way! this is
														 disgusting! }
		lastCommand := GetLastCommand;
		IF lastCommand <> NIL THEN
			IF GetSuperClassID(GetClassID(lastCommand)) = GetClassIDFromName('TTECommand') THEN
				lastCommand.fCanUndo := FALSE;
		{!!!RCR Handle this mode switching business at a parent view or document level }
		{ TCalcDocument(fDocument).fCellsView.BeTarget;}
		
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TEntryView.GetAsString(VAR theString: Str255);

	VAR
		theText:			CharsHandle;
		numberOfChars:		INTEGER;
		i:					INTEGER;

	BEGIN
	theText := TEGetText(fHTE);
	numberOfChars := Min(255, GetHandleSize(Handle(theText)));
	theString[0] := CHR(numberOfChars);
	FOR i := 1 TO numberOfChars DO
		theString[i] := theText^^[i - 1];
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TEntryView.BecameApplicationTarget; OVERRIDE;

	VAR
		r:					Rect;

	BEGIN
	INHERITED BecameApplicationTarget;
	
	IF fHTE <> NIL THEN 								{ fHTE is set to NIL by TTEView.Free since
														 InstallSelection is then called by
														 INHERITED Free. }
		TESetSelect(0, 0, fHTE);

	fTouched := TRUE;

	IF Focus THEN
		BEGIN
		GetQDExtent(r);
		InsetRect(r, 2, 2);
		InvalidateRect(r);
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TEntryView.ResignedApplicationTarget; OVERRIDE;

	VAR
		r:					Rect;

	BEGIN
	INHERITED ResignedApplicationTarget;
	
	IF fHTE <> NIL THEN 								{ fHTE is set to NIL by TTEView.Free since
														 InstallSelection is then called by
														 INHERITED Free. }
		TESetSelect(0, 0, fHTE);

	fTouched := FALSE;

	IF Focus THEN
		BEGIN
		GetQDExtent(r);
		InsetRect(r, 2, 2);
		InvalidateRect(r);
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ASelCommand}

PROCEDURE TEntryView.SetEditMode;

	BEGIN
	EditMode(TRUE); 									{ sets fFirstEdit to FALSE }
	fFirstEdit := TRUE; 								{ the only place it is set to TRUE }
	SetToString('');
	BecameApplicationTarget;
	{ SELF.BeTarget;}
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TEntryView.SetToString(theString: Str255);

	BEGIN
	IF Focus THEN;
	SetJustification(teFlushDefault, kDontRedraw);		{ initialize text to system-justified }
	ResignedApplicationTarget;
	TESetText(Ptr(ORD4(@theString) + 1), LENGTH(theString), fHTE);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TEntryView.SwapStrings;

	VAR
		newString:			Str255;

	BEGIN
	newString := fOldString;
{$Push} {$H-}
	GetAsString(fOldString);
{$Pop}
	SetToString(newString);
	fTouched := TRUE;
	END;

{--------------------------------------------------------------------------------------------------}
{$S AFields}

PROCEDURE TEntryView.Fields(obj: TObject); OVERRIDE;

	BEGIN
	WITH obj DO
		BEGIN
		DoToField(AtStr('TEntryView'), NIL, bClass);
		DoToField(AtStr('fCalcDocument'), @fCalcDocument, bObject);
		DoToField(AtStr('fTouched'), @fTouched, bBoolean);
		DoToField(AtStr('fTEditing'), @fTEditing, bBoolean);
		DoToField(AtStr('fFirstEdit'), @fFirstEdit, bBoolean);
		DoToField(AtStr('fOldString'), @fOldString, bString);
		END;

	INHERITED Fields(obj);
	END;

{***************************************************************************************************
	T C o o r d V i e w
***************************************************************************************************}
{$S AOpen}

PROCEDURE TCoordView.IRes(itsDocument: TDocument;
						  itsSuperView: TView;
						  VAR itsParams: Ptr); OVERRIDE;

	VAR
		minSize:			Point;

	BEGIN
	INHERITED IRes(itsDocument, itsSuperView, itsParams);
	fCalcDocument := TCalcDocument(itsDocument);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TCoordView.Draw(area: VRect);

	VAR
		aString, anotherString: Str255;
		aRect:				Rect;

	BEGIN
	WITH fCalcDocument DO
		BEGIN
		IF fEditColumn > 0 THEN
			fColumnsView.CoordToString(fEditColumn, aString)
		ELSE
			aString := ' ';

		IF fEditRow > 0 THEN
			NumToString(fEditRow, anotherString)
		ELSE
			anotherString := ' ';
		END;
	aString := CONCAT(aString, anotherString);
	SetTheFont(kEntryFont, kEntryFontSize, [bold]);
	SetRect(aRect, 2, 0, 46, kEntryHeight);
	SmartDrawString(aString, aRect, teFlushDefault);

	INHERITED Draw(area);
	END;

{--------------------------------------------------------------------------------------------------}
{$S AFields}

PROCEDURE TCoordView.Fields(obj: TObject); OVERRIDE;

	BEGIN
	WITH obj DO
		BEGIN
		DoToField(AtStr('TCoordView'), NIL, bClass);
		DoToField(AtStr('fCalcDocument'), @fCalcDocument, bObject);
		END;

	INHERITED Fields(obj);
	END;

{***************************************************************************************************
	T C o l u m n
***************************************************************************************************}
{$S ARes}

PROCEDURE TColumn.Initialize; OVERRIDE;
{ Put the object into a known state from which it may be safely FREEd.
 (and hopefully a usable state) }

	BEGIN
	INHERITED Initialize;

	fNumber := 0;
	fFormat := gDefaultFormat;
	fWidth := kCellWidth;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TColumn.IColumn(number: INTEGER);

	BEGIN
	IObject;

	fNumber := number;
	END;

{--------------------------------------------------------------------------------------------------}
{$S AFields}

PROCEDURE TColumn.Fields(obj: TObject); OVERRIDE;

	VAR
		aString:			Str255;
		anInteger:			INTEGER;
		aStyle: 			Style;

	BEGIN
	WITH obj DO
		BEGIN
		DoToField(AtStr('TColumn'), NIL, bClass);
		DoToField(AtStr('fNumber'), @fNumber, bInteger);
{$Push} {$H-}											{ Because FormatFields is in a debugging
														 (i.e. resident) segment) }
		{	FormatFields('fFormat', fFormat, DoToField);	}

{$Pop}

		DoToField(AtStr('fFormat'), NIL, bTitle);

{$Push} {$H-}

		WITH fFormat DO
			BEGIN
			CASE fStyle OF
				NoStyle:
					aString := 'NoStyle';
				General:
					aString := 'General';
				DecimalStyle:
					aString := 'DecimalStyle';
				NoDecimal:
					aString := 'NoDecimal';
				Scientific:
					aString := 'Scientific';
				OTHERWISE
					aString := 'UnknownStyle';
			END;										{ CASE }

			DoToField(AtStr('  fStyle'), @aString, bString);

			anInteger := fDigits;
			DoToField(AtStr('  fDigits'), @anInteger, bInteger);

			CASE fJustification OF
				teFlushDefault:
					aString := 'System';
				teFlushLeft:
					aString := 'Forced Left';
				teCenter:
					aString := 'Center';
				teFlushRight:
					aString := 'Right';
				kNoJustification:
					aString := 'None';
				OTHERWISE
					aString := 'Unknown';
			END;										{ CASE }

			DoToField(AtStr('  fJustification'), @aString, bString);

			anInteger := fFontNumber;
			DoToField(AtStr('  fFontNumber'), @anInteger, bFontName);
			anInteger := fFontSize;
			DoToField(AtStr('  fFontSize'), @anInteger, bInteger);
			aStyle := fFontStyle;
			DoToField(AtStr('  fFontStyle'), @aStyle, bStyle);
			END;										{ WITH fFormat }
{$Pop}

		DoToField(AtStr('fWidth'), @fWidth, bInteger);
		END;

	INHERITED Fields(obj);
	END;

{--------------------------------------------------------------------------------------------------}
{$S AReadFile}

PROCEDURE TColumn.ReadFromDisk(aFile: TFile);

	VAR
		columnInfo: 		ColumnDiskInfo;
		count:				Longint;

	BEGIN
	count := SIZEOF(ColumnDiskInfo);
	FailOSErr(aFile.ReadData(@columnInfo, count));
	WITH columnInfo DO
		BEGIN
		fNumber := number;
		fFormat := format;
		fWidth := width;
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S AClipBoard}

PROCEDURE TColumn.ReadFromScrap(theScrap: Handle;
								VAR scrapOffset: LONGINT);

	VAR
		columnInfo: 		ColumnDiskInfo;

	BEGIN
	ReadScrap(theScrap, scrapOffset, @columnInfo, SIZEOF(columnInfo));
	WITH columnInfo DO
		BEGIN
		fNumber := number;
		fFormat := format;
		fWidth := width;
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S AWriteFile}

PROCEDURE TColumn.WriteToDisk(aFile: TFile);

	VAR
		columnInfo: 		ColumnDiskInfo;
		c:					ColumnNumber;
		count:				Longint;

	BEGIN
	{ write out the info that TCalcDocument.DoRead expects to see }
	c := fNumber;
	count := SIZEOF(ColumnNumber);
	FailOSErr(aFile.WriteData(@c, count));

	{ now write out the info that TColumn.ReadFromDisk expects to see }
	WITH columnInfo DO
		BEGIN
		number := fNumber;
		format := fFormat;
		width := fWidth;
		END;
	count := SIZEOF(ColumnDiskInfo);
	FailOSErr(aFile.WriteData(@columnInfo, count));
	END;

{--------------------------------------------------------------------------------------------------}
{$S AClipBoard}

PROCEDURE TColumn.WriteToScrap(theScrap: Handle;
							   VAR scrapOffset: LONGINT);

	VAR
		columnInfo: 		ColumnDiskInfo;

	BEGIN
	WITH columnInfo DO
		BEGIN
		number := fNumber;
		format := fFormat;
		width := fWidth;
		END;
	WriteScrap(theScrap, scrapOffset, @columnInfo, SIZEOF(columnInfo));
	END;

{***************************************************************************************************
	T R o w
***************************************************************************************************}
{$S ARes}

PROCEDURE TRow.Initialize; OVERRIDE;
{ Put the object into a known state from which it may be safely FREEd.
 (and hopefully a usable state) }

	BEGIN
	INHERITED Initialize;

	fNumber := 0;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TRow.IRow(number: INTEGER);

	BEGIN
	IObject;

	fNumber := number;
	END;

{--------------------------------------------------------------------------------------------------}
{$S AFields}

PROCEDURE TRow.Fields(obj: TObject); OVERRIDE;

	BEGIN
	WITH obj DO
		BEGIN
		DoToField(AtStr('TRow'), NIL, bClass);
		DoToField(AtStr('fNumber'), @fNumber, bInteger);
		END;

	INHERITED Fields(obj);
	END;

{--------------------------------------------------------------------------------------------------}
{$S AReadFile}

PROCEDURE TRow.ReadFromDisk(aFile: TFile);

	VAR
		rowInfo:			RowDiskInfo;
		count:				Longint;

	BEGIN
	count := SIZEOF(RowDiskInfo);
	FailOSErr(aFile.ReadData(@rowInfo, count));
	WITH rowInfo DO
		fNumber := number;
	END;

{--------------------------------------------------------------------------------------------------}
{$S AClipBoard}

PROCEDURE TRow.ReadFromScrap(theScrap: Handle;
							 VAR scrapOffset: LONGINT);

	VAR
		RowInfo:			RowDiskInfo;

	BEGIN
	ReadScrap(theScrap, scrapOffset, @RowInfo, SIZEOF(RowInfo));
	WITH RowInfo DO
		fNumber := number;
	END;

{--------------------------------------------------------------------------------------------------}
{$S AWriteFile}

PROCEDURE TRow.WriteToDisk(aFile: TFile);

	VAR
		r:					RowNumber;
		rowInfo:			RowDiskInfo;
		count:				Longint;

	BEGIN
	{ need to write our row number which is read in in TCalcDocument.DoRead }
	r := fNumber;
	count := SIZEOF(RowNumber);
	FailOSErr(aFile.WriteData(@r, count));

	{ now, write out the info that TRow.ReadFromDisk expects to see }
	rowInfo.number := fNumber;
	count := SIZEOF(RowDiskInfo);
	FailOSErr(aFile.WriteData(@rowInfo, count));
	END;

{--------------------------------------------------------------------------------------------------}
{$S AClipBoard}

PROCEDURE TRow.WriteToScrap(theScrap: Handle;
							VAR scrapOffset: LONGINT);

	VAR
		RowInfo:			RowDiskInfo;

	BEGIN
	WITH RowInfo DO
		number := fNumber;
	WriteScrap(theScrap, scrapOffset, @RowInfo, SIZEOF(RowInfo));
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TRowList.IRowList;

	BEGIN
	ISortedList;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

FUNCTION TRowList.GetRow(r: RowNumber): TRow;

	VAR
		index:				ArrayIndex;

{$Push} {$IFC qTrace} {$D+} {$ENDC}

	FUNCTION TestItem(anItem: TObject): CompareResult;

		BEGIN
		IF TRow(anItem).fNumber > r THEN
			TestItem := kItemGreaterThanCriteria
		ELSE IF TRow(anItem).fNumber < r THEN
			TestItem := kItemLessThanCriteria
		ELSE
			TestItem := kItemEqualCriteria;
		END;
{$Pop}

	BEGIN
	GetRow := TRow(DoSearch(TestItem, index));
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

FUNCTION TRowList.Compare(item1, item2: TObject): CompareResult; OVERRIDE;

	BEGIN
	IF TRow(item1).fNumber > TRow(item2).fNumber THEN
		Compare := kItem1GreaterThanItem2
	ELSE IF TRow(item1).fNumber < TRow(item2).fNumber THEN
		Compare := kItem1LessThanItem2
	ELSE
		Compare := kItem1EqualItem2;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TColumnList.IColumnList;

	BEGIN
	ISortedList;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

FUNCTION TColumnList.GetColumn(c: ColumnNumber): TColumn;

	VAR
		index:				ArrayIndex;

{$Push} {$IFC qTrace} {$D+} {$ENDC}

	FUNCTION TestItem(anItem: TObject): CompareResult;

		BEGIN
		IF TColumn(anItem).fNumber > c THEN
			TestItem := kItemGreaterThanCriteria
		ELSE IF TColumn(anItem).fNumber < c THEN
			TestItem := kItemLessThanCriteria
		ELSE
			TestItem := kItemEqualCriteria;
		END;
{$Pop}

	BEGIN
	GetColumn := TColumn(DoSearch(TestItem, index));
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

FUNCTION TColumnList.Compare(item1, item2: TObject): CompareResult; OVERRIDE;

	BEGIN
	IF TColumn(item1).fNumber > TColumn(item2).fNumber THEN
		Compare := kItem1GreaterThanItem2
	ELSE IF TColumn(item1).fNumber < TColumn(item2).fNumber THEN
		Compare := kItem1LessThanItem2
	ELSE
		Compare := kItem1EqualItem2;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TCellList.ICellList;

	BEGIN
	ISortedList;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

FUNCTION TCellList.GetCell(r: RowNumber;
						   c: ColumnNumber): TCell;

	VAR
		index:				ArrayIndex;

{$Push} {$IFC qTrace} {$D+} {$ENDC}

	FUNCTION TestItem(anItem: TObject): CompareResult;

		BEGIN
		IF TCell(anItem).fRow > r THEN
			TestItem := kItemGreaterThanCriteria
		ELSE IF TCell(anItem).fRow < r THEN
			TestItem := kItemLessThanCriteria
		ELSE
			BEGIN
			IF TCell(anItem).fColumn > c THEN
				TestItem := kItemGreaterThanCriteria
			ELSE IF TCell(anItem).fColumn < c THEN
				TestItem := kItemLessThanCriteria
			ELSE
				TestItem := kItemEqualCriteria;
			END;
		END;
{$Pop}

	BEGIN
	GetCell := TCell(DoSearch(TestItem, index));
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

FUNCTION TCellList.Compare(item1, item2: TObject): CompareResult; OVERRIDE;

	BEGIN
	IF TCell(item1).fRow > TCell(item2).fRow THEN
		Compare := kItem1GreaterThanItem2
	ELSE IF TCell(item1).fRow < TCell(item2).fRow THEN
		Compare := kItem1LessThanItem2
	ELSE
		BEGIN
		IF TCell(item1).fColumn > TCell(item2).fColumn THEN
			Compare := kItem1GreaterThanItem2
		ELSE IF TCell(item1).fColumn < TCell(item2).fColumn THEN
			Compare := kItem1LessThanItem2
		ELSE
			Compare := kItem1EqualItem2;
		END;
	END;

{***************************************************************************************************
	T C e l l
***************************************************************************************************}
{$S ARes}

PROCEDURE TCell.Initialize; OVERRIDE;

	BEGIN
	INHERITED Initialize;

	fCalcDocument := NIL;
	fDeleted := FALSE;

	fKind := EmptyCell;
	fError := NoError;
	fValue := 0.0;
	fValueString := '';
	fFormula := '';
	FailMemError;

	fRow := 0;
	fColumn := 0;
	fEvaluating := FALSE;
	
	fMarked := FALSE;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TCell.ICell(owningDocument: TCalcDocument;
					  r: RowNumber;
					  c: ColumnNumber);

	BEGIN
	IObject;

	fCalcDocument := owningDocument;

	fRow := r;
	fColumn := c;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TCell.Free;

BEGIN
	INHERITED Free;
END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

FUNCTION TCell.Clone: TObject; OVERRIDE;

	VAR
		clonedCell: 		TCell;

	BEGIN
	clonedCell := TCell(INHERITED Clone);
	Clone := TObject(clonedCell);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TCell.AddDependentCell(aCell: TCell);
BEGIN
	SELF.AddDependent(aCell);
END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TCell.CopyContents(sourceCell: TCell);

	BEGIN
	fKind := sourceCell.fKind;
	fFormula := sourceCell.fFormula;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}
PROCEDURE TCell.EachDependentCell(PROCEDURE DoToCell(aCell: TCell));

	PROCEDURE CheckDependent(anObject: TObject);
	BEGIN
		IF Member(anObject,TCell) THEN
			DoToCell(TCell(anObject));
	END;
	
BEGIN
	SELF.EachDependent(CheckDependent);
END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}
PROCEDURE TCell.EachReferencedCell(PROCEDURE DoToCell(aCell: TCell));	

	PROCEDURE CheckNotifier(anObject: TObject);
	BEGIN
		IF Member(anObject,TCell) THEN
			DoToCell(TCell(anObject));
	END;

BEGIN
	SELF.EachNotifier(CheckNotifier);
END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TCell.EvaluateFormula(updateDependencies: BOOLEAN;VAR cellChanged: BOOLEAN);

	VAR
		formulaLength:		INTEGER;
		formulaIndex:		INTEGER;
		theChar:			CHAR;
		theValue:			ValueType;
		oldValue:			ValueType;
		oldKind:			KindOfCell;

{--------------------------------------------------------------------------------------------------}

	FUNCTION Factor(VAR theValue: ValueType): EvalResult;
		FORWARD;

{--------------------------------------------------------------------------------------------------}

	FUNCTION Expression(VAR theValue: ValueType): EvalResult;
		FORWARD;

{--------------------------------------------------------------------------------------------------}

	PROCEDURE GetNextChar;

		BEGIN
		REPEAT
			IF formulaIndex < formulaLength THEN
				BEGIN
				formulaIndex := formulaIndex + 1;
				theChar := fFormula[formulaIndex];
				END
			ELSE
				theChar := CHR(0);
		UNTIL theChar <> ' ';
		IF (theChar >= 'a') & (theChar <= 'z') THEN
			theChar := CHR(ORD(theChar) - 32);
		END;

{--------------------------------------------------------------------------------------------------}

	FUNCTION DoInteger(VAR theInteger: INTEGER): EvalResult;

		BEGIN
		theInteger := 0;
		WHILE IsDigit(theChar) DO
			BEGIN
			theInteger := theInteger * 10 + ORD(theChar) - ORD('0');
			GetNextChar;
			END;
		DoInteger := NoError;
		END;

{--------------------------------------------------------------------------------------------------}

	FUNCTION DoNumber(VAR theValue: ValueType): EvalResult;
	{ DoNumber is called only when theChar is a numeric digit or a decimal point }

		VAR
			newIndex:			INTEGER;
			decimalNumber:		Decimal;
			IsValidNumber:		BOOLEAN;
			startsWithDecPt:	BOOLEAN;
			theFormula: 		Str255;

		BEGIN
		startsWithDecPt := (theChar = '.');
		newIndex := formulaIndex;
		theFormula := fFormula;
		Str2Dec(theFormula, newIndex, decimalNumber, IsValidNumber);
		{ Str2Dec returns IsValidNumber = FALSE if the digits are followed by non-numerics like * }
		IF startsWithDecPt & (NOT IsValidNumber) THEN
			BEGIN
			GetNextChar;
			IF NOT IsDigit(theChar) THEN
				BEGIN
				DoNumber := BadNumber;
				EXIT(DoNumber);
				END;
			END;
		formulaIndex := newIndex - 1;
		GetNextChar;
		theValue := Dec2Num(decimalNumber);
		DoNumber := NoError;
		END;

{--------------------------------------------------------------------------------------------------}

	FUNCTION DoCellReference(VAR theValue: ValueType): EvalResult;

		VAR
			theResult:			EvalResult;
			r:					INTEGER;				{ Can't use RowNumber because it may be }
			c:					INTEGER;				{ out of range }
			referencedCell: 	TCell;
			aRect:				Rect;

		BEGIN
		theResult := NoError;
		aRect := fCalcDocument.fDimensions; 			{ valid coordinates }
		c := 0;
		WHILE (theChar >= 'A') & (theChar <= 'Z') & (c <= aRect.right) DO
			BEGIN
			c := c * 26 + ORD(theChar) - ORD('A') + 1;
			GetNextChar;
			END;
		IF c > aRect.right THEN 						{ column is out of range, so }
			r := 0										{ don't bother looking for row number }
		ELSE
			theResult := DoInteger(r);
		IF fCalcDocument.CellInRange(r, c, aRect) THEN
			BEGIN
			referencedCell := fCalcDocument.GetExistingCell(r - fCalcDocument.fRowOffset, c -
															fCalcDocument.fColumnOffset);
			IF referencedCell = NIL THEN				{ cell doesn't exist }
				theValue := 0
			ELSE
				BEGIN
				IF referencedCell.fEvaluating THEN
					theResult := SelfReference
				ELSE IF (referencedCell.fKind = ErrorCell) THEN
					theResult := ErrorCellReference
				ELSE IF (referencedCell.fKind = TextCell) THEN
					theValue := 0
				ELSE
					theValue := referencedCell.fValue;

				IF updateDependencies & {!!!RCR} (theResult <> SelfReference) THEN
					BEGIN
					referencedCell.AddDependentCell(SELF);
					END;
				END;
			END
		ELSE
			theResult := BadCellReference;
		DoCellReference := theResult;
		END;

{--------------------------------------------------------------------------------------------------}

	FUNCTION Term(VAR theValue: ValueType): EvalResult;

		BEGIN
		IF theChar = '(' THEN
			BEGIN
			GetNextChar;
			Term := Expression(theValue);
			IF theChar = ')' THEN
				GetNextChar
			ELSE
				Term := MissingRightParen;
			END
		ELSE IF IsDigit(theChar) | (theChar = '.') THEN
			Term := DoNumber(theValue)
		ELSE IF (theChar >= 'A') & (theChar <= 'Z') THEN
			Term := DoCellReference(theValue)
		ELSE IF theChar = '+' THEN
			BEGIN
			GetNextChar;
			Term := Term(theValue);
			END
		ELSE IF theChar = '-' THEN
			BEGIN
			GetNextChar;
			Term := Term(theValue);
			theValue := - theValue;
			END
		ELSE
			Term := IllegalCharacter;
		END;

{--------------------------------------------------------------------------------------------------}

	FUNCTION Factor(VAR theValue: ValueType): EvalResult;

		VAR
			theResult:			EvalResult;
			factorValue:		ValueType;

		BEGIN
		theResult := Term(theValue);
		IF theResult = NoError THEN
			IF theChar = '*' THEN
				BEGIN
				GetNextChar;
				theResult := Factor(factorValue);
				theValue := theValue * factorValue;
				END
			ELSE IF theChar = '/' THEN
				BEGIN
				GetNextChar;
				theResult := Factor(factorValue);
				theValue := theValue / factorValue;
				END;
		Factor := theResult;
		END;

{--------------------------------------------------------------------------------------------------}

	FUNCTION Expression(VAR theValue: ValueType): EvalResult;

		VAR
			factorResult:		EvalResult;
			factorValue:		ValueType;

		BEGIN
		factorResult := Factor(theValue);
		IF factorResult = NoError THEN
			REPEAT
				IF theChar = '+' THEN
					BEGIN
					GetNextChar;
					factorResult := Factor(factorValue);
					theValue := theValue + factorValue;
					END
				ELSE IF theChar = '-' THEN
					BEGIN
					GetNextChar;
					factorResult := Factor(factorValue);
					theValue := theValue - factorValue;
					END;
			UNTIL (theChar <> '+') & (theChar <> '-');
		Expression := factorResult;
		END;

		PROCEDURE DoToCell(aCell: TCell);
		BEGIN
			aCell.RemoveDependentCell(SELF);
		END;

BEGIN												{ EvaluateFormula }
	IF updateDependencies THEN
		SELF.EachReferencedCell(DoToCell);
	fValueString := '';
	oldValue := fValue;
	oldKind := fKind;
	formulaLength := LENGTH(fFormula);
	formulaIndex := 0;
	GetNextChar;
	IF theChar IN ['=', '-', '+', '.', '0'..'9'] THEN
		BEGIN
		IF theChar = '=' THEN
			GetNextChar;

		fEvaluating := TRUE;							{ prevent self reference loop }
		fError := Expression(theValue);
		fEvaluating := FALSE;

		IF fError = NoError THEN
			BEGIN
			fValue := theValue;
			IF (formulaIndex <= formulaLength) & (theChar <> CHR(0)) THEN
				fError := GarbageAtEnd;
			END;
		IF fError = NoError THEN
			fKind := ValueCell
		ELSE
			fKind := ErrorCell;
		END
	ELSE
		fKind := TextCell;

	cellChanged := (oldValue <> fValue) | (oldKind <> fKind);
	IF cellChanged THEN
		SELF.invalidate;
END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TCell.GetAsString(VAR theString: Str255);

	BEGIN
	theString := fFormula;
	END;

{--------------------------------------------------------------------------------------------------}
{$S AWriteFile}

FUNCTION TCell.GetDiskSize(infoRecordOnly: BOOLEAN): INTEGER;

	VAR
		cellSize:			INTEGER;

	BEGIN
	cellSize := SIZEOF(CellDiskInfo) - SIZEOF(Str255) + LENGTH(fFormula) + 1;
	IF NOT infoRecordOnly THEN
		cellSize := cellSize + (SELF.GetNoOfReferencedCells + SELF.GetNoOfDependentCells + 1) * SIZEOF(Point);
	GetDiskSize := cellSize;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TCell.GetValueAsString(VAR theString: Str255);

	BEGIN
	CASE fKind OF
		TextCell:
			theString := fFormula;
		ValueCell:
			BEGIN
			IF fValueString = '' THEN
				ValueToString;
			theString := fValueString;
			END;
		ErrorCell:
			BEGIN
			NumToString(ORD(fError), theString);
			theString := CONCAT('**ERROR ', theString);
			END;
		OTHERWISE
			theString := '';
	END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S AFields}

PROCEDURE TCell.Fields(obj: TObject); OVERRIDE;

	VAR
		aString:			Str255;

	BEGIN
	WITH obj DO
		BEGIN
		DoToField(AtStr('TCell'), NIL, bClass);
		DoToField(AtStr('fDeleted'), @fDeleted, bBoolean);
		DoToField(AtStr('fCalcDocument'), @fCalcDocument, bObject);
		DoToField(AtStr('fRow'), @fRow, bByte);
		DoToField(AtStr('fColumn'), @fColumn, bByte);
		CASE fKind OF
			EmptyCell:
				aString := 'Empty';
			ValueCell:
				aString := 'Value';
			TextCell:
				aString := 'Text';
			ErrorCell:
				aString := 'Error';
		END;
		DoToField(AtStr('fKind'), @aString, bString);
		CASE fError OF
			NoError:
				aString := 'None';
			MissingRightParen:
				aString := 'Missing right parenthesis';
			SelfReference:
				aString := 'Self reference';
			ErrorCellReference:
				aString := 'Reference to error cell';
			BadNumber:
				aString := 'Bad number';
			IllegalCharacter:
				aString := 'Illegal character';
			BadCellReference:
				aString := 'Bad cell reference';
			GarbageAtEnd:
				aString := 'Garbage at end';
		END;
		DoToField(AtStr('fError'), @aString, bString);
		DoToField(AtStr('fValueString'), @fValueString, bString);
		DoToField(AtStr('fFormula'), @fFormula, bString);
		DoToField(AtStr('fEvaluating'), @fEvaluating, bBoolean);
		END;											{ WITH }

	INHERITED Fields(obj);
	END;

{--------------------------------------------------------------------------------------------------}
{$S AFields}

PROCEDURE TCell.GetInspectorName(VAR inspectorName: Str255); OVERRIDE;

	VAR
		aString, titleString: Str255;

	BEGIN
	IF IsObject(fCalcDocument.fColumnsView) THEN
		BEGIN
		fCalcDocument.fColumnsView.CoordToString(fColumn, aString);
		NumToString(fRow, inspectorName);
		fCalcDocument.GetInspectorName(titleString);
		inspectorName := CONCAT(titleString, ' ', aString, inspectorName);
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}
FUNCTION  TCell.GetNoOfDependentCells: ArrayIndex;
VAR 
	count: ArrayIndex;

	PROCEDURE CountCell(aCell: TCell);
	BEGIN
		count := count + 1;
	END;

BEGIN
	count := 0;
	SELF.EachDependentCell(CountCell);
	GetNoOfDependentCells := count;
END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}
FUNCTION  TCell.GetNoOfReferencedCells: ArrayIndex;
VAR 
	count: ArrayIndex;

	PROCEDURE CountCell(aCell: TCell);
	BEGIN
		count := count + 1;
	END;
	
BEGIN
	count := 0;
	SELF.EachReferencedCell(CountCell);
	GetNoOfReferencedCells := count;
END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TCell.invalidate;

	VAR
		aCell:				GridCell;

	BEGIN
	IF (fCalcDocument <> NIL) & (fCalcDocument.fCellsView <> NIL) THEN
		BEGIN
		WITH aCell DO
			BEGIN
			h := fColumn;
			v := fRow;
			END;
		fCalcDocument.fCellsView.InvalidateCell(aCell);
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

FUNCTION TCell.IsEmpty: BOOLEAN;

	BEGIN
	IsEmpty := (fKind = EmptyCell) & (LENGTH(fFormula) = 0);
	END;

{--------------------------------------------------------------------------------------------------}
{$S AReadFile}

PROCEDURE TCell.ReadFromDisk(aFile: TFile);

	VAR
		cellInfo:			CellDiskInfo;
		referencedCell: 	TCell;
		refRow: 			RowNumber;
		refColumn:			ColumnNumber;
		refIndex:			INTEGER;
		count:				Longint;

	BEGIN
	count := SIZEOF(CellDiskInfo);
	FailOSErr(aFile.ReadData(@cellInfo, count));

	WITH cellInfo DO
		BEGIN
		fKind := kind;
		fError := error;
		fValue := value;
		fFormula := formula;

		FOR refIndex := 1 TO noOfReferences DO
			BEGIN
			ReadCellCoordinate(aFile, refRow, refColumn);
			referencedCell := fCalcDocument.GetCell(refRow, refColumn);
			referencedCell.AddDependentCell(SELF);
			END;
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S AClipBoard}

PROCEDURE TCell.ReadFromScrap(theScrap: Handle;
							  VAR scrapOffset: LONGINT);

	VAR
		cellLength: 		INTEGER;
		cellInfo:			CellDiskInfo;
		referencedCell: 	TCell;
		cellCoord:			Point;
		refIndex:			INTEGER;

	BEGIN
	ReadScrap(theScrap, scrapOffset, @cellLength, SIZEOF(cellLength));
	ReadScrap(theScrap, scrapOffset, @cellInfo, cellLength);

	WITH cellInfo DO
		BEGIN
		fKind := kind;
		fError := error;
		fValue := value;
		fFormula := formula;

		FOR refIndex := 1 TO noOfReferences DO
			BEGIN
			ReadScrap(theScrap, scrapOffset, @cellCoord, SIZEOF(cellCoord));
			referencedCell := fCalcDocument.GetCell(cellCoord.v, cellCoord.h);
			referencedCell.AddDependentCell(SELF);
			END;
		END;
	END;


{--------------------------------------------------------------------------------------------------}
{$S ARes}
PROCEDURE TCell.DoUpdate(VAR theChange: ChangeID;
							changedObject: TObject;
							changedBy: TObject); OVERRIDE;
VAR
	cellChanged: BOOLEAN;
BEGIN
	IF theChange = mValueChanged THEN
		BEGIN
			EvaluateFormula(NOT kUpdateDependencies,cellChanged);
			IF NOT cellChanged THEN
				theChange := kNoChange;
		END
	ELSE IF theChange = mWidthChanged THEN
		theChange := kNoChange
	ELSE
		INHERITED DoUpdate(theChange,changedObject,changedBy);
END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TCell.Recalculate(forceAutomatic: BOOLEAN;
							updateDependencies: BOOLEAN);
VAR 
	cellChanged: BOOLEAN;

	PROCEDURE DoToCell(aCell: TCell);

	BEGIN
		aCell.RemoveDependentCell(SELF);
	END;

BEGIN
	SELF.EvaluateFormula(updateDependencies,cellChanged);
	IF cellChanged THEN
		SELF.Changed(mValueChanged, SELF);
END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TCell.RemoveDependentCell(aCell: TCell);
BEGIN
	SELF.RemoveDependent(aCell);
END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TCell.SetDeleteState(deleted: BOOLEAN);

	BEGIN
	fDeleted := deleted;
	END;


{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TCell.SetToString(theString: Str255);

VAR
	theOldString:		Str255;

BEGIN
	GetAsString(theOldString);
	IF theString <> theOldString THEN
		BEGIN
			fFormula := theString;
			fKind := EmptyCell; 							{ Force Recalculate to invalidate the cell }
			Recalculate(NOT kForceAutomatic, kUpdateDependencies); { we want to set new dependents }
	
			{!!!RCR Changed(0, fCalcDocument);}
		END;
END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TCell.ValueToString;
 { Fills fValueString with the string representation of the cell's value.
   The "general" format is complicated because we try to ensure that
   the string will fit within the cell's column width.  Thus we have
   to figure out the appropriate representation for the value. }

	VAR
		aString:			Str255;
		theFormat:			DecForm;
		d:					Decimal;
		i:					INTEGER;
		len:				INTEGER;
		sigDigits:			INTEGER;
		leftDigits: 		INTEGER;
		rightDigits:		INTEGER;
		width:				INTEGER;
		digitWidth: 		INTEGER;
		dotWidth:			INTEGER;
		eWidth: 			INTEGER;
		minusWidth: 		INTEGER;
		plusWidth:			INTEGER;
		aColumn:			TColumn;
		aValueType: 		ValueType;

	BEGIN
	aColumn := fCalcDocument.GetColumn(fColumn);
	WITH aColumn DO
		CASE fFormat.fStyle OF
			General:
				BEGIN
				SetTheFont(fFormat.fFontNumber, fFormat.fFontSize, fFormat.fFontStyle);
				digitWidth := CharWidth('0');
				dotWidth := CharWidth('.');
				eWidth := CharWidth('e');
				minusWidth := CharWidth('-');
				plusWidth := CharWidth('+');
				width := fCalcDocument.fCellsView.GetColWidth(fColumn) - (kCellHBorder * 2);

				theFormat := gGeneralFormat;
				aValueType := fValue;
				Num2Dec(theFormat, aValueType, d);
				aString := d.sig;
				len := LENGTH(aString);

				IF d.sgn = 1 THEN
					width := width - minusWidth;

				IF d.exp > 0 THEN
				{ We've exceeded the precision of our value representation }
					BEGIN
					theFormat.Style := FloatDecimal;
					{ account for '.' and '+e99' }
					width := width - dotWidth - CharWidth('+') - eWidth - digitWidth - digitWidth;
					theFormat.digits := Min(kValuePrecision, width DIV digitWidth);
					END
				ELSE
					BEGIN

					{ determine number of significant digits }
					sigDigits := 1; 					{ in case all digits are zero }
					FOR i := len DOWNTO 2 DO
						IF aString[i] <> '0' THEN
							BEGIN
							sigDigits := i;
							LEAVE;
							END;

					leftDigits := kValuePrecision + d.exp;
					rightDigits := sigDigits - leftDigits;

					IF leftDigits < sigDigits THEN
						width := width - dotWidth;		{ account for decimal point }

					IF leftDigits * digitWidth > width THEN
						BEGIN
						{ We must convert to scientific notation }

						{ account for decimal point and 'e9' }
						width := width - dotWidth - eWidth - digitWidth;

	 { For positive numbers, SANE puts a space (' ') before
	  the first significant digit. }
						IF d.sgn <> 1 THEN
							width := width - CharWidth(' ');
						IF d.exp < - kValuePrecision THEN
							width := width - minusWidth
						ELSE
							width := width - plusWidth;
						IF leftDigits > 10 THEN
							width := width - digitWidth; { account for double-digit exponents }

						theFormat.Style := FloatDecimal;
						theFormat.digits := Min(sigDigits, width DIV digitWidth);
						END
					ELSE
						BEGIN
						theFormat.Style := FixedDecimal;
						theFormat.digits := Min((width - (leftDigits * digitWidth) - dotWidth) DIV
												digitWidth, rightDigits);
						IF theFormat.digits < 0 THEN
							theFormat.digits := 0;
						END;
					END;
				END;
			DecimalStyle:
				theFormat := gDecimalFormat;
			NoDecimal:
				theFormat := gNoDecimalFormat;
			Scientific:
				theFormat := gScientificFormat;
		END;

	aValueType := fValue;
	Num2Str(theFormat, aValueType, DecStr(aString));
	IF aString[1] = ' ' THEN
		Delete(aString, 1, 1);							{ Remove leading space }
	fValueString := Copy(aString, 1, Min(kMaxValueLength, LENGTH(aString)));
	END;

{--------------------------------------------------------------------------------------------------}
{$S AWriteFile}

PROCEDURE TCell.WriteToDisk(aFile: TFile);

	VAR
		cellInfo:			CellDiskInfo;
		count:				Longint;

{--------------------------------------------------------------------------------------------------}

	PROCEDURE WriteCellCoordinate(theCell: TCell);

		VAR
			cellCoordinate: 	Point;

		BEGIN
		cellCoordinate.v := theCell.fRow;
		cellCoordinate.h := theCell.fColumn;
		count := SIZEOF(Point);
		FailOSErr(aFile.WriteData(@cellCoordinate, count));
		END;

	BEGIN
	{ write out the data that TCalcDocument.DoRead expects to see }
	WriteCellCoordinate(SELF);

	{ write out the data that TCell.ReadFromDisk expects to see }
	WITH cellInfo DO
		BEGIN
		kind := fKind;
		error := fError;
		value := fValue;
		noOfReferences := SELF.GetNoOfReferencedCells;
		formula := fFormula;
		END;
	count := SIZEOF(CellDiskInfo);
	FailOSErr(aFile.WriteData(@cellInfo, count));
	SELF.EachReferencedCell(WriteCellCoordinate);
	END;

{--------------------------------------------------------------------------------------------------}
{$S AClipBoard}

PROCEDURE TCell.WriteToScrap(theScrap: Handle;
							 VAR scrapOffset: LONGINT);

	VAR
		cellInfo:			CellDiskInfo;
		cellLength: 		INTEGER;

{--------------------------------------------------------------------------------------------------}

	PROCEDURE WriteCellCoordinate(theObject: TObject);

		VAR
			cellCoordinate: 	Point;

		BEGIN
		cellCoordinate.v := TCell(theObject).fRow;
		cellCoordinate.h := TCell(theObject).fColumn;
		WriteScrap(theScrap, scrapOffset, @cellCoordinate, SIZEOF(cellCoordinate));
		END;

	BEGIN
	WriteCellCoordinate(TObject(SELF));
	cellLength := GetDiskSize(TRUE);					{ don't include ref's & dependents }
	WriteScrap(theScrap, scrapOffset, @cellLength, SIZEOF(cellLength));
	WITH cellInfo DO
		BEGIN
		kind := fKind;
		error := fError;
		value := fValue;
		noOfReferences := 0;							
		formula := fFormula;

		WriteScrap(theScrap, scrapOffset, @cellInfo, cellLength);
		END;
	END;

{***************************************************************************************************
	T C a l c S e l e c t C o m m a n d
***************************************************************************************************}
{$S ASelCommand}

PROCEDURE TCalcSelectCommand.ICalcSelectCommand(itsDocument: TCalcDocument;
												itsView: TGridView;
												VAR theMouse: VPoint;
												theShiftKey, theCmdKey: BOOLEAN);

	BEGIN
	ICellSelectCommand(itsView, theMouse, theShiftKey, theCmdKey);
	fCalcDocument := itsDocument;
	itsDocument.fEntryView.EditMode(FALSE);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TCalcSelectCommand.ComputeNewSelection(VAR clickedCell: GridCell); OVERRIDE;

	VAR
		r:					Rect;
		aSection:			TSection;

	PROCEDURE DoToSection(aSection: TSection);

		BEGIN
		IF Member(aSection, TSubscriber) THEN
			BEGIN
			SectRgn(fThisSelection, TRgnDesignator(aSection.fDesignator).fDesignation, gTempRgn);
			IF NOT EmptyRgn(gTempRgn) THEN			{ if this section is in selection }
				UnionRgn(fThisSelection, TRgnDesignator(aSection.fDesignator).fDesignation, fThisSelection);
			END;
		END;

	BEGIN
	IF fGridView.CanSelectCell(clickedCell) THEN
		BEGIN
		Pt2Rect(fAnchorCell, clickedCell, r);
		r.right := r.right + 1;
		r.bottom := r.bottom + 1;
		RectRgn(fThisSelection, r);

		{ add in the designated region for subscribers }
		{$IFC qDebug}
		UseTempRgn('TCalcSelectCommand.ComputeNewSelection');
		{$ENDC qDebug}
		fCalcDocument.EachSectionDo(DoToSection);
		{$IFC qDebug}
		DoneWithTempRgn;
		{$ENDC qDebug}

		IF fCmdKey & (NOT fGridView.fSingleSelection) THEN
			IF fDeselecting THEN
				DiffRgn(fPrevSelection, fThisSelection, fThisSelection)
			ELSE
				UnionRgn(fPrevSelection, fThisSelection, fThisSelection);
		END;
	END;

{***************************************************************************************************
	T R o w S e l e c t o r
***************************************************************************************************}
{$S ASelCommand}

PROCEDURE TRowSelector.IRowSelector(itsDocument: TCalcDocument;
									itsView: TGridView;
									VAR theMouse: VPoint;
									theShiftKey, theCmdKey: BOOLEAN);

	VAR
		aCellSelector:		TCalcSelectCommand;

	BEGIN
	fCalcDocument := itsDocument;
	IF fCalcDocument.fSelectionType <> RowSelection THEN
		theCmdKey := FALSE;
	ICellSelectCommand(itsView, theMouse, theShiftKey, theCmdKey);

	NEW(aCellSelector);
	aCellSelector.ICalcSelectCommand(itsDocument, itsDocument.fCellsView, theMouse, theShiftKey,
									 theCmdKey);
	fCellSelector := aCellSelector;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TRowSelector.Free; OVERRIDE;

	BEGIN
	fCellSelector := TCalcSelectCommand(FreeIfObject(fCellSelector));

	INHERITED Free;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TRowSelector.ComputeAnchorCell(VAR clickedCell: GridCell); OVERRIDE;

	BEGIN
	INHERITED ComputeAnchorCell(clickedCell);
	fAnchorCell.h := 1;

	clickedCell.h := 1;
	fCellSelector.ComputeAnchorCell(clickedCell);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TRowSelector.ComputeNewSelection(VAR clickedCell: GridCell); OVERRIDE;

	VAR
		r:					Rect;

	BEGIN
	clickedCell.h := fGridView.fNumOfCols;
	INHERITED ComputeNewSelection(clickedCell);

	clickedCell.h := fCalcDocument.fNoOfColumns;
	fCellSelector.ComputeNewSelection(clickedCell);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TRowSelector.DoIt; OVERRIDE;

	BEGIN
	INHERITED DoIt;
	fCellSelector.DoIt;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

FUNCTION TRowSelector.TrackMouse(aTrackPhase: TrackPhase;
								 VAR anchorPoint, previousPoint, nextPoint: VPoint;
								 mouseDidMove: BOOLEAN): TTracker; OVERRIDE;

	VAR
		clickedCell:		GridCell;

	BEGIN
	IF mouseDidMove THEN
		BEGIN
		clickedCell := fGridView.VPointToCell(nextPoint);
		IF aTrackPhase = TrackPress THEN
			BEGIN
			ComputeAnchorCell(clickedCell);
			IF fCmdKey THEN
				BEGIN
				fDeselecting := PtInRgn(fAnchorCell, fGridView.fSelections);
				fCellSelector.fDeselecting := fDeselecting;
				END;
			END;

		IF LONGINT(clickedCell) <> LONGINT(fPrevCell) THEN
			BEGIN
			ComputeNewSelection(clickedCell);
			HighlightNewSelection;
			fCellSelector.HighlightNewSelection;

			CopyRgn(fThisSelection, fPrevSelection);
			fPrevCell := clickedCell;
			WITH fCellSelector DO
				BEGIN
				CopyRgn(fThisSelection, fPrevSelection);
				fPrevCell := clickedCell;
				END;
			END;
		END;
	TrackMouse := SELF;
	END;

{***************************************************************************************************
	T C o l u m n S e l e c t o r
***************************************************************************************************}
{$S ASelCommand}

PROCEDURE TColumnSelector.IColumnSelector(itsDocument: TCalcDocument;
										  itsView: TGridView;
										  VAR theMouse: VPoint;
										  theShiftKey, theCmdKey: BOOLEAN);

	VAR
		aCellSelector:		TCalcSelectCommand;

	BEGIN
	IF itsDocument.fSelectionType <> ColumnSelection THEN
		theCmdKey := FALSE;
	ICellSelectCommand(itsView, theMouse, theShiftKey, theCmdKey);

	fCalcDocument := itsDocument;

	NEW(aCellSelector);
	aCellSelector.ICalcSelectCommand(itsDocument, itsDocument.fCellsView, theMouse, theShiftKey,
									 theCmdKey);
	fCellSelector := aCellSelector;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TColumnSelector.Free; OVERRIDE;

	BEGIN
	fCellSelector := TCalcSelectCommand(FreeIfObject(fCellSelector));

	INHERITED Free;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TColumnSelector.ComputeAnchorCell(VAR clickedCell: GridCell); OVERRIDE;

	BEGIN
	INHERITED ComputeAnchorCell(clickedCell);
	fAnchorCell.v := 1;

	clickedCell.v := 1;
	fCellSelector.ComputeAnchorCell(clickedCell);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TColumnSelector.ComputeNewSelection(VAR clickedCell: GridCell); OVERRIDE;

	VAR
		r:					Rect;

	BEGIN
	clickedCell.v := fGridView.fNumOfRows;
	INHERITED ComputeNewSelection(clickedCell);

	clickedCell.v := fCalcDocument.fCellsView.fNumOfRows;
	fCellSelector.ComputeNewSelection(clickedCell);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TColumnSelector.DoIt; OVERRIDE;

	BEGIN
	INHERITED DoIt;
	fCellSelector.DoIt;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

FUNCTION TColumnSelector.TrackMouse(aTrackPhase: TrackPhase;
									VAR anchorPoint, previousPoint, nextPoint: VPoint;
									mouseDidMove: BOOLEAN): TTracker; OVERRIDE;

	VAR
		clickedCell:		GridCell;

	BEGIN
	IF mouseDidMove THEN
		BEGIN
		clickedCell := fGridView.VPointToCell(nextPoint);
		IF aTrackPhase = TrackPress THEN
			BEGIN
			ComputeAnchorCell(clickedCell);
			IF fCmdKey THEN
				BEGIN
				fDeselecting := PtInRgn(fAnchorCell, fGridView.fSelections);
				fCellSelector.fDeselecting := fDeselecting;
				END;
			END;

		IF LONGINT(clickedCell) <> LONGINT(fPrevCell) THEN
			BEGIN
			ComputeNewSelection(clickedCell);
			HighlightNewSelection;
			fCellSelector.HighlightNewSelection;

			CopyRgn(fThisSelection, fPrevSelection);
			fPrevCell := clickedCell;
			WITH fCellSelector DO
				BEGIN
				CopyRgn(fThisSelection, fPrevSelection);
				fPrevCell := clickedCell;
				END;
			END;
		END;
	TrackMouse := SELF;
	END;

{***************************************************************************************************
	T C o l u m n F o r m a t t e r
***************************************************************************************************}
{$S ASelCommand}

PROCEDURE TColumnFormatter.IFormatter(itsDocument: TCalcDocument;
									  itsCommand: INTEGER);

{--------------------------------------------------------------------------------------------------}

	PROCEDURE SaveFormat(aCell: GridCell);

		VAR
			theColumn:			TColumn;

		BEGIN
		theColumn := fCalcDocument.GetColumn(aCell.h);
		theColumn.fOldFormat := theColumn.fFormat;
		END;

	BEGIN
	ICommand(itsCommand, itsDocument, itsDocument.fCellsView);
	fCalcDocument := itsDocument;
	fSelection := MakeNewRgn;							{ copy the current selection region }
	CopyRgn(itsDocument.fColumnsView.fSelections, fSelection);
	fCalcDocument.fColumnsView.EachSelectedCellDo(SaveFormat);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TColumnFormatter.DoIt; OVERRIDE;

	PROCEDURE SetFormat(aCell: GridCell);

		VAR
			theColumn:			TColumn;

		BEGIN
		theColumn := fCalcDocument.GetColumn(aCell.h);
		WITH theColumn.fFormat DO
			CASE fID OF
				cGeneral:
					fStyle := General;
				cNoDecimal:
					fStyle := NoDecimal;
				cDecimal:
					fStyle := DecimalStyle;
				cScientific:
					fStyle := Scientific;
				cSystemJustify:
					fJustification := teFlushDefault;
				cForceLeftJustify:
					fJustification := teFlushLeft;
				cRightJustify:
					fJustification := teFlushRight;
				cCenter:
					fJustification := teCenter;
			END;
		END;

	BEGIN
	fCalcDocument.fColumnsView.EachSelectedCellDo(SetFormat);
	IF (fID >= cGeneral) & (fID <= cScientific) THEN { Style change }
		fCalcDocument.DoRecalculate(NOT kForceAutomatic, NOT kUpdateDependencies);
	fCalcDocument.fCellsView.InvalidateSelection;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TColumnFormatter.Free; OVERRIDE;

	BEGIN
	fSelection := DisposeIfRgnHandle(fSelection);

	INHERITED Free;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TColumnFormatter.RedoIt; OVERRIDE;

	BEGIN
	fCalcDocument.fColumnsView.ReSelect(fSelection);
	DoIt;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TColumnFormatter.UndoIt; OVERRIDE;

	PROCEDURE RestoreFormat(aCell: GridCell);

		VAR
			theColumn:			TColumn;

		BEGIN
		theColumn := fCalcDocument.GetColumn(aCell.h);
		theColumn.fFormat := theColumn.fOldFormat;
		END;

	BEGIN
	WITH fCalcDocument.fColumnsView DO
		BEGIN
		ReSelect(fSelection);
		EachSelectedCellDo(RestoreFormat);
		END;
	IF (fID >= cGeneral) & (fID <= cScientific) THEN { Style change }
		fCalcDocument.DoRecalculate(NOT kForceAutomatic, NOT kUpdateDependencies);
	fCalcDocument.fCellsView.InvalidateSelection;
	END;

{--------------------------------------------------------------------------------------------------}
{$S AFields}

PROCEDURE TColumnFormatter.Fields(obj: TObject); OVERRIDE;

	BEGIN
	WITH obj DO
		BEGIN
		DoToField(AtStr('TColumnFormatter'), NIL, bClass);
		DoToField(AtStr('fCalcDocument'), @fCalcDocument, bObject);
		END;

	INHERITED Fields(obj);
	END;

{***************************************************************************************************
	T C o l u m n S i z e r
***************************************************************************************************}
{$S ASelCommand}

PROCEDURE TColumnSizer.IColumnSizer(itsDocument: TCalcDocument;
									c: ColumnNumber);

	VAR
		colRect:			VRect;

	BEGIN
	ICommand(cSizeColumn, itsDocument, itsDocument.fCellsView);
	fConstrainsMouse := TRUE;
	fCalcDocument := itsDocument;
	fCellsView := itsDocument.fCellsView;
	fColumn := itsDocument.GetColumn(c);
	fNewWidth := fCellsView.GetColWidth(fColumn.fNumber);
	fOldWidth := fNewWidth;
	fCellsView.ColToVRect(fColumn.fNumber, 1, colRect);
	fLeftEdge := colRect.left;
	fViewConstrain := FALSE;							{ So that rightmost column can grow }
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

FUNCTION TColumnSizer.TrackMouse(aTrackPhase: TrackPhase;
								 VAR anchorPoint, previousPoint, nextPoint: VPoint;
								 mouseDidMove: BOOLEAN): TTracker; OVERRIDE;

	BEGIN
	fNewWidth := Min(nextPoint.h - fLeftEdge, kTELength);
	fNewWidth := Max(fNewWidth, 10);
	TrackMouse := SELF;
	nextPoint.h := Min(nextPoint.h, fLeftEdge + kTELength);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TColumnSizer.TrackFeedback(aTrackPhase: TrackPhase;
									 anchorPoint: VPoint;
									 previousPoint: VPoint;
									 nextPoint: VPoint;
									 mouseDidMove: BOOLEAN;
									 turnItOn: BOOLEAN); OVERRIDE;

	VAR
		viewedRect: 		Rect;
		pState: 			PenState;

	BEGIN
	IF mouseDidMove THEN
		BEGIN
		GetPenState(pState);
		PenPat(black);

		fCellsView.GetQDExtent(viewedRect);

		MoveTo(Min(nextPoint.h, fLeftEdge + kTELength), viewedRect.top);
		Line(0, viewedRect.bottom - viewedRect.top);
		SetPenState(pState);
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TColumnSizer.TrackConstrain(aTrackPhase: TrackPhase;
									  anchorPoint: VPoint;
									  previousPoint: VPoint;
									  VAR nextPoint: VPoint;
									  mouseDidMove: BOOLEAN); OVERRIDE;

	BEGIN
	IF nextPoint.h < fLeftEdge + 10 THEN
		nextPoint.h := fLeftEdge + 10
	ELSE
		nextPoint.h := Min(nextPoint.h, fLeftEdge + kTELength);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TColumnSizer.SetColumnWidth(newWidth: INTEGER);

	BEGIN
	IF newWidth > kTELength THEN
		BEGIN
		newWidth := kTELength;
		fNewWidth := kTELength;
		END
	ELSE IF newWidth < 10 THEN
		newWidth := 10;

	fCellsView.SetColWidth(fColumn.fNumber, 1, newWidth);
	fCalcDocument.fColumnsView.SetColWidth(fColumn.fNumber, 1, newWidth);
	fColumn.fWidth := newWidth;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TColumnSizer.DoIt; OVERRIDE;

	VAR
		minToSee:			VPoint;
		aRect:				VRect;

	BEGIN
	SetColumnWidth(fNewWidth);

	fCellsView.ColToVRect(fColumn.fNumber, 1, aRect);
	WITH aRect DO
		SetVPt(minToSee, right - left, bottom - top);
	fCellsView.RevealRect(aRect, minToSee, TRUE);

	fCalcDocument.fColumnsView.ColToVRect(fColumn.fNumber, 1, aRect);
	WITH aRect DO
		SetVPt(minToSee, right - left, bottom - top);
	fCalcDocument.fColumnsView.RevealRect(aRect, minToSee, TRUE);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TColumnSizer.UndoIt; OVERRIDE;

	BEGIN
	SetColumnWidth(fOldWidth);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TColumnSizer.RedoIt; OVERRIDE;

	BEGIN
	DoIt;
	END;

{--------------------------------------------------------------------------------------------------}
{$S AFields}

PROCEDURE TColumnSizer.Fields(obj: TObject); OVERRIDE;

	BEGIN
	WITH obj DO
		BEGIN
		DoToField(AtStr('TColumnSizer'), NIL, bClass);
		DoToField(AtStr('fCalcDocument'), @fCalcDocument, bObject);
		DoToField(AtStr('fCellsView'), @fCellsView, bObject);
		DoToField(AtStr('fLeftEdge'), @fLeftEdge, bInteger);
		DoToField(AtStr('fColumn'), @fColumn, bObject);
		DoToField(AtStr('fNewWidth'), @fNewWidth, bInteger);
		DoToField(AtStr('fOldWidth'), @fOldWidth, bInteger);
		END;

	INHERITED Fields(obj);
	END;

{***************************************************************************************************
	T C a l c T y p i n g C o m m a n d
***************************************************************************************************}
{$S ADoCommand}

PROCEDURE TCalcTypingCommand.ITETypingCommand(itsTEView: TTEView;
											  itsFirstChar: CHAR); OVERRIDE;

	BEGIN
	INHERITED ITETypingCommand(itsTEView, itsFirstChar);
	fCellsView := TEntryView(itsTEView).fCalcDocument.fCellsView;
	fTargetCell := fCellsView.FirstSelectedCell;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TCalcTypingCommand.AddCharacter(aChar: CHAR); OVERRIDE;
{ Switch the entry view from left to right justification when the typed characters overflow the
  entry box, and switch back to left justification if the text shrinks enough }

	VAR
		selRight:			INTEGER;

{--------------------------------------------------------------------------------------------------}

	FUNCTION AddCharWidths: INTEGER;

		VAR
			aString:			Str255;

		BEGIN
		GetIText(fHTE^^.hText, aString);
		AddCharWidths := StringWidth(aString);
		END;

	BEGIN
	IF (aChar <> chBackspace) & (fTEView.fJustification = teFlushDefault) THEN
		BEGIN
		selRight := fHTE^^.selRect.right;
		IF (selRight < 0) | (selRight > fHTE^^.destRect.right) THEN
			selRight := fHTE^^.selPoint.h;
		IF selRight + CharWidth(aChar) > fHTE^^.destRect.right THEN
			fTEView.SetJustification(teFlushRight, kDontRedraw);
		END;

	INHERITED AddCharacter(aChar);

	IF (aChar = chBackspace) & (fTEView.fJustification = teFlushRight) THEN
		IF AddCharWidths < LengthRect(fHTE^^.destRect, h) THEN
			fTEView.SetJustification(teFlushDefault, kRedraw);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TCalcTypingCommand.UndoIt; OVERRIDE;
{ If the selection has changed since this command was created, restore it to the
  target cell before Undoing. }

	VAR
		entryView:			TEntryView;

	BEGIN
	fCellsView.ReSelectCell(fTargetCell);				{ make sure fTargetCell is selected }
	entryView := TEntryView(fTEView);
	IF (entryView.fTEditing) & (NOT entryView.fFirstEdit) THEN
		INHERITED UndoIt								{ TTE undo }
	ELSE
		BEGIN
		entryView.SwapStrings;							{ exchange current and saved strings }
		fCellsView.SetCell(fTargetCell);				{ change fTargetCell's contents and redraw
														 it }
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TCalcTypingCommand.RedoIt; OVERRIDE;
{ If the selection has changed since this command was created, restore it to the
  target cell before Redoing. }

	VAR
		entryView:			TEntryView;

	BEGIN
	fCellsView.ReSelectCell(fTargetCell);				{ make sure fTargetCell is selected }
	entryView := TEntryView(fTEView);
	IF (entryView.fTEditing) & (NOT entryView.fFirstEdit) THEN
		INHERITED RedoIt								{ TTE redo }
	ELSE
		BEGIN
		entryView.SwapStrings;							{ exchange current and saved strings }
		fCellsView.SetCell(fTargetCell);				{ change fTargetCell's contents and redraw
														 it }
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S AFields}

PROCEDURE TCalcTypingCommand.Fields(obj: TObject); OVERRIDE;

	BEGIN
	WITH obj DO
		BEGIN
		DoToField(AtStr('TCalcTypingCommand'), NIL, bClass);
		DoToField(AtStr('fCellsView'), @fCellsView, bObject);
		DoToField(AtStr('fTargetCell'), @fTargetCell, bPoint);
		END;

	INHERITED Fields(obj);
	END;

{***************************************************************************************************
	T C e l l E d i t C o m m a n d
***************************************************************************************************}
{$S ASelCommand}

PROCEDURE TCellEditCommand.ICellEditCommand(itsDocument: TCalcDocument;
											itsCommand: INTEGER);

	BEGIN
	ICommand(itsCommand, itsDocument, itsDocument.fCellsView);
	fCalcDocument := itsDocument;
	fSelection := MakeNewRgn;							{ copy the current selection region }
	CopyRgn(itsDocument.fCellsView.fSelections, fSelection);
	fChangesClipboard := itsCommand <> cClear;
	fCausesChange := itsCommand <> cCopy;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TCellEditCommand.Free; OVERRIDE;

	BEGIN
	fSelection := DisposeIfRgnHandle(fSelection);

	INHERITED Free;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TCellEditCommand.CopySelection;

VAR
	clipDocument:		TCalcDocument;
	clipView:			TCellsView;
	clipRect:			Rect;

{--------------------------------------------------------------------------------------------------}

	PROCEDURE CopyRowToClipboard(aCell: GridCell);

	VAR
		newRow: 			TRow;

	BEGIN
		newRow := TRow(fCalcDocument.GetRow(aCell.v).Clone);
		newRow.fNumber := newRow.fNumber - clipDocument.fRowOffset;
		clipDocument.AddRow(newRow);
	END;

{--------------------------------------------------------------------------------------------------}

	PROCEDURE CopyColumnToClipboard(aCell: GridCell);

	VAR
		newColumn:			TColumn;

	BEGIN
		newColumn := TColumn(fCalcDocument.GetColumn(aCell.h).Clone);
		newColumn.fNumber := newColumn.fNumber - clipDocument.fColumnOffset;
		clipDocument.AddColumn(newColumn);
	END;

{--------------------------------------------------------------------------------------------------}

	PROCEDURE CopyCellToClipboard(aCell: TCell);

	VAR
		newCell:			TCell;

	BEGIN
		newCell := TCell(aCell.Clone);
		clipDocument.AddCell(newCell, newCell.fRow - clipDocument.fRowOffset, newCell.fColumn -
							 clipDocument.fColumnOffset);
	END;

BEGIN
	NEW(clipDocument);
	clipRect := fSelection^^.rgnBBox;
	clipRect.bottom := clipRect.bottom - 1;
	clipRect.right := clipRect.right - 1;
	clipDocument.ICalcDocument(NIL,clipRect);
	clipDocument.DoInitialState;

	fCalcDocument.fRowsView.EachSelectedCellDo(CopyRowToClipboard);
	fCalcDocument.fColumnsView.EachSelectedCellDo(CopyColumnToClipboard);
	fCalcDocument.EachSelectedExistingCellDo(CopyCellToClipboard);

	NEW(clipView);
	clipView.ICellsView(clipDocument, TRUE, NIL);
	clipDocument.fCellsView := clipView;
	gClipboardMgr.ClaimClipboard(clipView);
	clipView.AdjustSize;
END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TCellEditCommand.DeleteSelection;

{--------------------------------------------------------------------------------------------------}

	PROCEDURE DeleteCell(aCell: TCell);

	BEGIN
		fCalcDocument.DeleteCell(aCell.fRow, aCell.fColumn);
	END;

BEGIN
	fCalcDocument.EachSelectedExistingCellDo(DeleteCell);
	fCalcDocument.DoRecalculate(NOT kForceAutomatic, NOT kUpdateDependencies);
	fCalcDocument.fCellsView.InvalidateSelection;
END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TCellEditCommand.RestoreSelection;

	VAR
		firstCell:			GridCell;

{--------------------------------------------------------------------------------------------------}

	PROCEDURE RestoreCell(aCell: TCell);

		BEGIN
		fCalcDocument.UndeleteCell(aCell.fRow, aCell.fColumn);
		END;

	BEGIN
	fCalcDocument.EachSelectedExistingCellDo(RestoreCell);
	fCalcDocument.DoRecalculate(NOT kForceAutomatic, NOT kUpdateDependencies);
	fCalcDocument.fCellsView.InvalidateSelection;

	firstCell := fCalcDocument.fCellsView.FirstSelectedCell;
	fCalcDocument.SetEntry(firstCell.v, firstCell.h);	{ update entry view }
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TCellEditCommand.ReSelect;

	BEGIN
	fCalcDocument.fCellsView.ReSelect(fSelection);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TCellEditCommand.DoIt;

	BEGIN
	IF fID <> cClear THEN
		CopySelection;
	IF fID <> cCopy THEN
		DeleteSelection;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TCellEditCommand.UndoIt;
{ If the user has changed the selection since this command was created,
  restore it before Undoing so that the correct cells are affected. }

	BEGIN
	IF fID <> cCopy THEN
		BEGIN
		ReSelect;										{ restore command's original selection }
		RestoreSelection;
		fCalcDocument.fCellsView.ScrollSelectionIntoView(TRUE);
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TCellEditCommand.RedoIt;
{ If the user has changed the selection since this command was created,
  restore it before Redoing so that the correct cells are affected. }

	BEGIN
	IF fID <> cCopy THEN
		BEGIN
		ReSelect;										{ restore command's original selection }
		DeleteSelection;
		fCalcDocument.fCellsView.ScrollSelectionIntoView(TRUE);
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TCellEditCommand.Commit;

	BEGIN
	fCalcDocument.FreeDeletedCells;
	END;

{--------------------------------------------------------------------------------------------------}
{$S AFields}

PROCEDURE TCellEditCommand.Fields(obj: TObject); OVERRIDE;

	BEGIN
	WITH obj DO
		BEGIN
		DoToField(AtStr('TCellEditCommand'), NIL, bClass);
		DoToField(AtStr('fCalcDocument'), @fCalcDocument, bObject);
		DoToField(AtStr('fSelection'), @fSelection, bRgnHandle);
		END;

	INHERITED Fields(obj);
	END;

{***************************************************************************************************
	T C e l l P a s t e C o m m a n d
***************************************************************************************************}
{$S ASelCommand}

PROCEDURE TCellPasteCommand.ICellPasteCommand(itsDocument: TCalcDocument);

	BEGIN
{$IFC qDebug}
	IF NOT Member(gClipboardMgr.fClipView, TCellsView) THEN
		ProgramBreak('Attempt to paste a non-TCellsView clipboard');
{$ENDC}
	ICommand(cPaste, itsDocument, itsDocument.fCellsView);
	fCalcDocument := itsDocument;
	fSelection := MakeNewRgn;							{ copy the current selection region }
	CopyRgn(itsDocument.fCellsView.fSelections, fSelection);
	fClipDocument := TCellsView(gClipboardMgr.fClipView).fCalcDocument;
	fReplacedCells := NewList;
{$IFC qDebug}
	fReplacedCells.SetEltType('TCell');
{$ENDC}
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TCellPasteCommand.Free; OVERRIDE;

	BEGIN
	fSelection := DisposeIfRgnHandle(fSelection);

	fReplacedCells := TList(FreeListIfObject(fReplacedCells));

	INHERITED Free;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TCellPasteCommand.DoIt;

	VAR
		r:					INTEGER;					{ Can't use RowNumber or ColumnNumber }
		c:					INTEGER;					{  because they may be out of range }
		fi: 				FailInfo;

{--------------------------------------------------------------------------------------------------}

	PROCEDURE PasteCell(aCell: GridCell);

		VAR
			sourceCell: 		TCell;
			replacedCell:		TCell;
			destCell:			TCell;

		BEGIN
		destCell := fCalcDocument.GetCell(aCell.v, aCell.h);
		replacedCell := TCell(destCell.Clone);
		fReplacedCells.InsertLast(replacedCell);

		sourceCell := fClipDocument.GetCell(r, c);
		c := c + 1;
		IF c > fClipDocument.fNoOfColumns THEN
			BEGIN
			c := 1;
			r := r + 1;
			IF r > fClipDocument.fNoOfRows THEN
				r := 1;
			END;

		destCell.CopyContents(sourceCell);				{ copy the necessary fields }
		END;

{--------------------------------------------------------------------------------------------------}

	PROCEDURE HdlPasteFailure(error: OSErr;
							  message: LONGINT);
	{ We ran out of memory and couldn't complete the paste.
	  So, let's back out the partial paste. All or nothing! }

{--------------------------------------------------------------------------------------------------}

		PROCEDURE RestoreCell(replacedCell: TCell);

			VAR
				pastedCell: 		TCell;

			BEGIN
			pastedCell := fCalcDocument.GetCell(replacedCell.fRow, replacedCell.fColumn);
			IF replacedCell.IsEmpty THEN				{ free up memory used by empty cell }
				BEGIN
				fCalcDocument.DeleteCell(pastedCell.fRow, pastedCell.fColumn);
				fCalcDocument.FreeCell(pastedCell);
				END
			ELSE
				pastedCell.CopyContents(replacedCell);
			replacedCell := TCell(FreeIfObject(replacedCell)); { free up memory used by replacement
																cell }
			END;

		BEGIN
		fReplacedCells.Each(RestoreCell);
		fReplacedCells.DeleteAll;
		UpdateViews;
		END;

	BEGIN												{ TCellPasteCommand.DoIt }
	CatchFailures(fi, HdlPasteFailure);
	r := 1;
	c := 1;
	fCalcDocument.fCellsView.EachSelectedCellDo(PasteCell);
	Success(fi);
	UpdateViews;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TCellPasteCommand.UndoIt;

{--------------------------------------------------------------------------------------------------}

	PROCEDURE RestoreCell(replacedCell: TCell);

		VAR
			pastedCell: 		TCell;

		BEGIN
		pastedCell := fCalcDocument.GetCell(replacedCell.fRow, replacedCell.fColumn);
		pastedCell.CopyContents(replacedCell);
		END;

	BEGIN
	fCalcDocument.fCellsView.ReSelect(fSelection);		{ restore original selection }
	fReplacedCells.Each(RestoreCell);
	UpdateViews;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TCellPasteCommand.RedoIt;

	VAR
		r:					INTEGER;					{ Can't use RowNumber or ColumnNumber }
		c:					INTEGER;					{  because they may be out of range }

{--------------------------------------------------------------------------------------------------}

	PROCEDURE RepasteCell(aCell: GridCell);

		VAR
			sourceCell: 		TCell;
			destCell:			TCell;

		BEGIN
		destCell := fCalcDocument.GetCell(aCell.v, aCell.h);
		sourceCell := fClipDocument.GetCell(r, c);
		c := c + 1;
		IF c > fClipDocument.fNoOfColumns THEN
			BEGIN
			c := 1;
			r := r + 1;
			IF r > fClipDocument.fNoOfRows THEN
				r := 1;
			END;

		destCell.CopyContents(sourceCell);
		END;

	BEGIN
	fCalcDocument.fCellsView.ReSelect(fSelection);		{ restore original selection }
	r := 1;
	c := 1;
	fCalcDocument.fCellsView.EachSelectedCellDo(RepasteCell);
	UpdateViews;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TCellPasteCommand.UpdateViews;

	VAR
		aString:			Str255;
		aCell:				GridCell;

	BEGIN
	WITH fCalcDocument DO
		BEGIN
		DoRecalculate(NOT kForceAutomatic, kUpdateDependencies);
		fCellsView.InvalidateSelection;

		aCell.h := fEditColumn;
		aCell.v := fEditRow;
		IF fCellsView.IsCellSelected(aCell) THEN
			BEGIN
			fEditCell := GetCell(aCell.v, aCell.h);
			fEditCell.GetAsString(aString);
			fEntryView.SetToString(aString);
			END;
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S AFields}

PROCEDURE TCellPasteCommand.Fields(obj: TObject); OVERRIDE;

	BEGIN
	WITH obj DO
		BEGIN
		DoToField(AtStr('TCellPasteCommand'), NIL, bClass);
		DoToField(AtStr('fClipDocument'), @fClipDocument, bObject);
		DoToField(AtStr('fCalcDocument'), @fCalcDocument, bObject);
		DoToField(AtStr('fSelection'), @fSelection, bRgnHandle);
		DoToField(AtStr('fReplacedCells'), @fReplacedCells, bObject);
		END;

	INHERITED Fields(obj);
	END;
