{[a-,body+,h-,o=100,r+,rec+,t=4,u+,#+,j=20/57/1$,n-]}
{ UCalc.p }
{ Copyright  1986-1990 by Apple Computer, Inc. All rights reserved.}

{ This MacApp sample implements a simple spreadsheet }
{ This version, created Christmas Day 1989 by Larry Tesler, uses
	classes TPrimaryScroller and TSecondaryScroller in USynchScroller
	[instead of defining its own TCalcScroller and four AutoScroll methods]
	to achieve synchronous scrolling [instead of near-synchronous scrolling]
	and to enable auto-scrolling during column resizing. }

UNIT UCalc;

	INTERFACE

		USES
		{  requirements for this unit's interface. }
		Types, MacAppTypes,
		{}
		Quickdraw, Textedit, OSUtils, Segload, Files, Traps, Memory, SysEqu, Devices,
		Scrap, ULoMem, Aliases, Editions, UMacAppUtilities, AppleEvents, Balloons,

		{  MacApp}
		UFailure, UPascalObject, UObject, UList, UPatch, UMemory, Menus, UMenuMgr, UGeometry,
		Events, Controls, Windows, Dialogs, UMacAppGlobals, UAppleEvents, UEvent, UCommand, UEvtHandler, UFile,
		UStream, UDesignator, UApplication,
		UBehavior, UViewBehavior, UPrintHandler, UAdorners, UView, UWindow, UControl, UScroller, UDocument, UAssociation,
		UFileHandler, USection, UFileBasedDocument, UEditionDocument, UClipboardMgr,

			{  Building Blocks }
			USynchScroller,
			PrintTraps,									{ Needed because we override printing
														 commands }
			UPrinting, UGridView, UTEView, UTECommands,

			{  Implementation use }
			ToolUtils, Fonts, Packages,

			{  Other Libraries }
			SANE,
			CalcUtilities, UGrabberTracker;

		CONST

			kMaxRows			= 127;					{ Maximum row number (arbitrary)}
			kMaxColumns 		= 63;					{ Maximum column number (arbitrary)}
			kMaxValueLength 	= 24;					{ Max. length of a value as a string }

			kSignature			= 'SS12';				{ Application signature. }
			kFileType			= 'SF12';				{ Data file type. }

			kCalcWindowType 	= 1001; 				{ Resource id of the spreadsheet window }

		TYPE

			CalcMode			= (Manual, Automatic);

			RowNumber			= 0..kMaxRows;			{ Zero indicates no row. }
			ColumnNumber		= 0..kMaxColumns;

			ValueType			= Extended;
			DiskValueType		= Double;				{ Extendeds are for internal use only}
			ValueString 		= STRING[kMaxValueLength];

			ValueFormat 		= DecForm;				{ Have SANE display numeric values in
														 decimal }

			KindOfCell			= (EmptyCell, ValueCell, TextCell, ErrorCell);

			EvalResult			= (NoError, MissingRightParen, SelfReference, ErrorCellReference,
								   BadNumber, IllegalCharacter, BadCellReference, GarbageAtEnd);

			TypeOfSelection 	= (NoSelection, CellSelection, RowSelection, ColumnSelection,
								   AllSelection);

			TypeOfStyle 		= (NoStyle, General, DecimalStyle, NoDecimal, Scientific,
								   UnknownStyle);

			FormatRecord		= RECORD
				fStyle: 			TypeOfStyle;
				fDigits:			0..255;
				fJustification: 	SignedByte;
				fFontNumber:		0..255;
				fFontSize:			0..255;
				fFontStyle: 		Style;
				END;
			FormatPtr			= ^FormatRecord;

			CalcDocDiskInfo 	= RECORD				{ This is the first record in the file }
				dimensions: 		Rect;
				CalcMode:			CalcMode;
				allocatedCells: 	INTEGER;
				allocatedRows:		INTEGER;
				allocatedColumns:	INTEGER;
				selectionType:		TypeOfSelection;
				editRow:			RowNumber;
				editColumn: 		ColumnNumber;
				END;

			RowDiskInfo 		= RECORD				{ Info about the document's rows }
				number: 			INTEGER;
				END;

			ColumnDiskInfo		= RECORD				{ Info about the document's columns }
				number: 			INTEGER;
				format: 			FormatRecord;
				width:				INTEGER;
				END;

			CellDiskInfo		= RECORD				{ The document's cells }
			{ row/column coordinate read/written separately }
				kind:				KindOfCell;
				error:				EvalResult;
				value:				DiskValueType;
				noOfReferences: 	INTEGER;
				formula:			Str255;
				END;

			ScrapInfoRecord 	= RECORD
				selection:			Rect;
				noOfCells:			INTEGER;
				END;

			CellScrapInfo		= RECORD
			{ row/column coordinate read/written separately }
			{ cell length read/written separately }
				format: 			FormatRecord;
				formula:			Str255;
				END;

{--------------------------------------------------------------------------------------------------}
			TCalcApplication	= OBJECT (TApplication)

				PROCEDURE TCalcApplication.ICalcApplication(itsFileType: OSType);
				{ Initializes the Application }

				PROCEDURE TCalcApplication.AboutToLoseControl(convertClipboard: BOOLEAN); OVERRIDE;
				{ Override to reset Edit menu items for incoming DAs }

				FUNCTION TCalcApplication.DoMakeDocument(itsCmdNumber: cmdNumber; itsFile: TFile): TDocument;
					OVERRIDE;
				{ Launches a TCalcDocument }

				FUNCTION TCalcApplication.MakeViewForAlienClipboard: TView; OVERRIDE;
				{ Handle data that was left in the Clipboard by forces outside the application }

				END;									{ TCalcApplication }

{--------------------------------------------------------------------------------------------------}
			TCell				= OBJECT; FORWARD;
			TRow				= OBJECT; FORWARD;
			TColumn 			= OBJECT; FORWARD;
			TCellsView			= OBJECT; FORWARD;
			TRowsView			= OBJECT; FORWARD;
			TColumnsView		= OBJECT; FORWARD;
			TEntryView			= OBJECT; FORWARD;
			TCoordView			= OBJECT; FORWARD;
			TRowList			= OBJECT; FORWARD;
			TColumnList 		= OBJECT; FORWARD;
			TCellList			= OBJECT; FORWARD;

			TCalcDocument		= OBJECT (TEditionDocument)

				fCells: 			TCellList;
				fEditCell:			TCell;				{ the cell being edited (the current
														 selection) }

				fRows:				TRowList;
				fRowOffset: 		INTEGER;

				fColumns:			TColumnList;
				fColumnOffset:		INTEGER;
				fColumnIsSelected:	BOOLEAN;			{ a column has been selected }

				fCellsView: 		TCellsView; 		{ the view of the cells themselves }
				fRowsView:			TRowsView;			{ the view of the row numbers }
				fColumnsView:		TColumnsView;		{ the view of the column letters }
				fEntryView: 		TEntryView; 		{ the view for entering/editing cell values
														 }
				fCoordView: 		TCoordView; 		{ the view that shows the selected cell's
														 coordinates }

				fDimensions:		Rect;				{ the document's dimensions }
				fInUseBounds:		Rect;				{ the area of the document that is in use }
				fCalcMode:			CalcMode;			{ Manual or Automatic recalculation }
				fAllocatedCells:	INTEGER;			{ the number of allocated cells }
				fAllocatedRows: 	INTEGER;			{ the number of allocated rows }
				fAllocatedColumns:	INTEGER;			{ the number of allocated columns }

				fSelectionType: 	TypeOfSelection;

				fEditRow:			RowNumber;			{ the row of the cell being edited }
				fEditColumn:		ColumnNumber;		{ the column of the cell being edited }

				fNoOfRows:			INTEGER;			{ maximum number of rows }
				fNoOfColumns:		INTEGER;			{ maximum number of columns }

								{ Initialization and termination }

				PROCEDURE TCalcDocument.ICalcDocument(itsFile: TFile;dimensions: Rect);
				{ Initialize the document }

				PROCEDURE TCalcDocument.DoInitialState; OVERRIDE;
				{ Initialize allocation counts }

				PROCEDURE TCalcDocument.DoMakeViews(forPrinting: BOOLEAN); OVERRIDE;
				{ Launch the views which are seen in the document's window }

				PROCEDURE TCalcDocument.Free; OVERRIDE;
				{ Free the document }

				PROCEDURE TCalcDocument.FreeData; OVERRIDE;
				{ Delete the cells, rows, and columns }

				{ Filing OVERRIDES }

				PROCEDURE TCalcDocument.DoNeedDiskSpace(itsFile: TFile;
											VAR dataForkBytes, rsrcForkBytes: LONGINT); OVERRIDE;
				{ Tells how many bytes of disk space will be required to store the
				  data for the document in a file on disk }

				PROCEDURE TCalcDocument.DoRead(aFile: TFile; forPrinting: BOOLEAN); OVERRIDE;
				{ Reads in the data from the disk, when a document is Opened or Reverted }

				PROCEDURE TCalcDocument.DoWrite(aFile: TFile; makingCopy: BOOLEAN); OVERRIDE;
				{ Writes the data to the disk, when a document is Saved }

				{ Menu handling }

				PROCEDURE TCalcDocument.DoSetupMenus; OVERRIDE;
				{ Enable menu commands }

				PROCEDURE TCalcDocument.DoMenuCommand(aCmdNumber: cmdNumber); OVERRIDE;
				{ Handle menu commands }

				{ Cell management }

				PROCEDURE TCalcDocument.AddCell(theCell: TCell;
												r: RowNumber;
												c: ColumnNumber);
				{ Add the cell to the document }

				FUNCTION TCalcDocument.CellExists(r: RowNumber;
												  c: ColumnNumber): BOOLEAN;
				{ Is there a cell object at these coordinates? }

				PROCEDURE TCalcDocument.DeleteCell(r: RowNumber;
												   c: ColumnNumber);
				{ Mark the cell at the given coordinates as deleted }

				PROCEDURE TCalcDocument.EachExistingCellDo(PROCEDURE
														   DoToCell(aCell: TCell));
				{ Perform DoToCell on each cell object }

				PROCEDURE TCalcDocument.EachSelectedExistingCellDo(PROCEDURE
														   DoToCell(aCell: TCell));
				{ Perform DoToCell on each non-empty cell in the selection }
				
				PROCEDURE TCalcDocument.EditCell;
				{ Change the formula of the cell being edited to the string in the entry view }

				PROCEDURE TCalcDocument.FreeCell(theCell: TCell);
				{ Free the specified cell object }

				PROCEDURE TCalcDocument.FreeDeletedCells;
				{ Free each cell object that is marked deleted }

				FUNCTION TCalcDocument.GetCell(r: RowNumber;
											   c: ColumnNumber): TCell;
				{ Return the cell object for the given coordinates. }

				FUNCTION TCalcDocument.GetExistingCell(r: RowNumber;
													   c: ColumnNumber): TCell;
				{ Return the cell object or NIL if no cell is allocated for the given coordinates. }

				FUNCTION TCalcDocument.IsAutoCalc: BOOLEAN;
				{ Evaluates fCalcMode = Automatic }

				PROCEDURE TCalcDocument.SetEntry(r: RowNumber;
												 c: ColumnNumber);
				{ Set the text in the entry view to the formula in the cell at the given
				  coordinates }

				PROCEDURE TCalcDocument.UndeleteCell(r: RowNumber;
													 c: ColumnNumber);
				{ If there is a cell at the given coordinates, mark it undeleted }

				{ Row management }

				PROCEDURE TCalcDocument.AddRow(theRow: TRow);
				{ Add the row to the document }

				PROCEDURE TCalcDocument.EachExistingRowDo(PROCEDURE
														  DoToRow(aRow: TRow));

				FUNCTION TCalcDocument.GetRow(r: RowNumber): TRow;

				FUNCTION TCalcDocument.RowExists(r: RowNumber): BOOLEAN;

				{ Column management }

				PROCEDURE TCalcDocument.AddColumn(theColumn: TColumn);
				{ Add the column to the document }

				FUNCTION TCalcDocument.ColumnExists(c: ColumnNumber): BOOLEAN;

				PROCEDURE TCalcDocument.EachExistingColumnDo(PROCEDURE
															 DoToColumn(aColumn: TColumn));

				FUNCTION TCalcDocument.GetColumn(c: ColumnNumber): TColumn;

				{ Ranges of cells }

				FUNCTION TCalcDocument.CellInRange(r: INTEGER;
												   c: INTEGER;
												   range: Rect): BOOLEAN;
				{ Do the given coordinates fall within the range? }

				PROCEDURE TCalcDocument.ConstrainToUsedCells(VAR cellRange: Rect);
				{ Shrink the range of cells to exclude unallocated cells, if possible }

				PROCEDURE TCalcDocument.DoRecalculate(forceAutomatic: BOOLEAN;
													  setDependents: BOOLEAN);
				{ Recalculate all cells if fCalcMode = Automatic, otherwise just selected cells }

				{ Edition Manager Support }
				PROCEDURE TCalcDocument.SynchSections(aSection: TSection);
				{ either pass in NIL to synch all sections, or pass in a section to synch
				it with the cells view }

				PROCEDURE TCalcDocument.UserSelectionChanged; OVERRIDE;
				
				FUNCTION TCalcDocument.GetSubscriberFormatsMask: SignedByte; OVERRIDE;
				FUNCTION TCalcDocument.CanPublishSelection: BOOLEAN; OVERRIDE;
				FUNCTION TCalcDocument.CanSubscribe: BOOLEAN; OVERRIDE;

				PROCEDURE TCalcDocument.DoWriteData(aScrapType: OSType; aDesignator: TDesignator;
														  aStream: TStream); OVERRIDE;
				{ Given a designator and a stream, writes the designated data to the stream. }

				PROCEDURE TCalcDocument.DoWritePrivateTypes(aDesignator: TDesignator; aStream: TStream);
				{ called by DoWriteData if aScrapType = kCalcScrapType }

				PROCEDURE TCalcDocument.DoReadData(aScrapType: OSType; aDesignator: TDesignator;
														  aStream: TStream; length: LONGINT); OVERRIDE;
				{ Given a designator and a stream, read the designated data from the stream. }

				PROCEDURE TCalcDocument.DoReadPrivateTypes(aDesignator: TDesignator; aStream: TStream);
				{ called by DoReadData if aScrapType = kCalcScrapType }

				PROCEDURE TCalcDocument.DoReadTEXT(aDesignator: TDesignator; aStream: TStream; length: LONGINT);
				{ called by DoReadData if aScrapType = 'TEXT' }

				FUNCTION TCalcDocument.GetSelectedSection: TSection; OVERRIDE;
				FUNCTION TCalcDocument.GetSelectedSectionInRgn(selectionRgn: RgnHandle): TSection;
				PROCEDURE TCalcDocument.AddSection(aSection: TSection); OVERRIDE;
				PROCEDURE TCalcDocument.RevealSelection(aSelection: TDesignator); OVERRIDE;

				{ === change notification stuff === }
				
				PROCEDURE TCalcDocument.UpdateSelectedCells(theChange: ChangeID; changedBy: TObject);

				{ Inspecting }

				PROCEDURE TCalcDocument.Fields(obj: TObject); OVERRIDE;

				END;									{ TCalcDocument }

{--------------------------------------------------------------------------------------------------}

			TCalcWindow 		= OBJECT (TWindow)

				PROCEDURE TCalcWindow.IRes(itsDocument: TDocument;
										   itsSuperView: TView;
										   VAR itsParams: Ptr); OVERRIDE;
				{ Create the window from a resource template}

				PROCEDURE TCalcWindow.Draw(area: VRect); OVERRIDE;

				PROCEDURE TCalcWindow.DoKeyCommand(event: TToolboxEvent); OVERRIDE;
				END;									{ TCalcWindow }

{--------------------------------------------------------------------------------------------------}
			TCellsView			= OBJECT (TTextGridView)
			{ TCellsView is the view used to show the cell values. }

				fCalcDocument:		TCalcDocument;
				fLastOptionKey: 	BOOLEAN;			{ TRUE if option key down at last idle.
														 helps in tracking modifier keys for views
														 that care. }

				{ /// added to support adornment /// }
				PROCEDURE TCellsView.CreatePublisherAdorner(aPublisher: TPublisher);
				PROCEDURE TCellsView.CreateSubscriberAdorner(aSubscriber: TSubscriber);
				PROCEDURE TCellsView.ComputeBorderRgn(itsBorderRgn, itsDesignation: RgnHandle);
				{ /// end   consider putting into TView /// }

				{ Initialization and termination }

				PROCEDURE TCellsView.ICellsView(itsDocument: TCalcDocument;
												forClipboard: BOOLEAN;
												itsParent: TView);
				{ Initialize the view }

				PROCEDURE TCellsView.IRes(itsDocument: TDocument;
										  itsSuperView: TView;
										  VAR itsParams: Ptr); OVERRIDE;
				{ Create the view from a resource template}

				{ Commands and menus }

				PROCEDURE TCellsView.DoMouseCommand(VAR themouse: VPoint;
													event: TToolboxEvent;
													hysteresis: Point); OVERRIDE;

				PROCEDURE TCellsView.DoSetupMenus; OVERRIDE;

				PROCEDURE TCellsView.DoMenuCommand(aCmdNumber: cmdNumber); OVERRIDE;
				{ Handle menu commands }

				PROCEDURE TCellsView.DoKeyCommand(event: TToolboxEvent); OVERRIDE;

				{ Display and Selection }

				PROCEDURE TCellsView.AdornCol(aCol: INTEGER;
											  area: VRect); OVERRIDE;
				{ Draw a column delimiter ("adornment") in the given area }

				PROCEDURE TCellsView.AdornRow(aRow: INTEGER;
											  area: VRect); OVERRIDE;
				{ Draw a row delimiter in the given area }

				FUNCTION TCellsView.DesignatorAsPICT(aDesignator: TDesignator): PicHandle;

				FUNCTION TCellsView.DesignatorAsTEXT(aDesignator: TDesignator): Handle;

				FUNCTION TCellsView.DoIdle(phase: IdlePhase): BOOLEAN; OVERRIDE;
				{ Checks the state of the option key and decides whether to make the cursor
				need to be reset.  We used to use gAlwaysTrackCursor, but this is better. }

				PROCEDURE TCellsView.DoSetCursor(localPoint: VPoint;
												cursorRgn: RgnHandle); OVERRIDE;

				PROCEDURE TCellsView.DrawCell(aCell: GridCell;
											  aRect: VRect); OVERRIDE;
				{ Draw the cell's value, using the proper justification }

				PROCEDURE TCellsView.GetVisibleCells(VAR visibleCells: Rect);
				{ Return a rectangle giving the cells that are visible }

				FUNCTION TCellsView.IsCellVisible(aCell: GridCell): BOOLEAN;
				{ Tell whether the specified cell is visible or not }

				PROCEDURE TCellsView.PositionAtCell(aCell: GridCell);
				{ Redraw the views so that aCell is at the top left corner of the grid }

				PROCEDURE TCellsView.ReSelect(cellRegion: RgnHandle);
				{ Set the current selection to the given cells }

				PROCEDURE TCellsView.ReSelectCell(aCell: GridCell);
				{ Set the current selection to the given cell }

				PROCEDURE TCellsView.ScrollSelectionIntoView(redraw: BOOLEAN); OVERRIDE;
				{ Scroll so that at least the top left cell of the selection is visible }

				PROCEDURE TCellsView.SetCell(aCell: GridCell);

				PROCEDURE TCellsView.SetColWidth(aCol: INTEGER;
												numOfCols: INTEGER;
												aWidth: INTEGER); OVERRIDE;

				PROCEDURE TCellsView.SetSelection(cellsToSelect: RgnHandle;
												  extendSelection, highlight,
					select: BOOLEAN); OVERRIDE;
				{ Set the current selections to the specified cells. Adjust the entry and
				  coordinate views accordingly. }

				{ Printing OVERRIDES }

				FUNCTION TCellsView.DoBreakFollowing(vhs: VHSelect;
													 prevBreak: VCoordinate;
													 VAR Automatic: BOOLEAN): VCoordinate; OVERRIDE;

{$IFC qDebug}
				PROCEDURE TCellsView.DoDrawPageBreak(vhs: VHSelect;
													 whichBreak: INTEGER;
													 loc: VCoordinate;
													 Automatic: BOOLEAN); OVERRIDE;
				{ For debugging printing only, draw lines showing the page breaks. }
{$ENDC}

				PROCEDURE TCellsView.GetPrintExtent(VAR printExtent: VRect); OVERRIDE;
				{ Return the area to print, depending on whether the command was Print Selection
				  or Print. }

				{ Clipboard handling }

				FUNCTION TCellsView.ContainsClipType(aType: ResType): BOOLEAN; OVERRIDE;

				PROCEDURE TCellsView.WriteCalcScrap(calcScrap: Handle);

				PROCEDURE TCellsView.WriteTextScrap(textScrap: Handle);

				PROCEDURE TCellsView.WriteToDeskScrap; OVERRIDE;

				{ Inspecting }

				PROCEDURE TCellsView.Fields(obj: TObject); OVERRIDE;

				END;									{ TCellsView }

{--------------------------------------------------------------------------------------------------}
			TRowsView			= OBJECT (TTextGridView)
			{ TRowsView is the view used to show the row numbers. Click in this view
			  to select an entire row or group of rows. }

				fCalcDocument:		TCalcDocument;

				PROCEDURE TRowsView.IRes(itsDocument: TDocument;
										 itsSuperView: TView;
										 VAR itsParams: Ptr); OVERRIDE;
				{ Create the view from a resource template}

				PROCEDURE TRowsView.AdornRow(aRow: INTEGER;
											 area: VRect); OVERRIDE;
				{ Draw a row delimiter in the given area }

				PROCEDURE TRowsView.DoMouseCommand(VAR themouse: VPoint;
												   event: TToolboxEvent;
												   hysteresis: Point); OVERRIDE;

				PROCEDURE TRowsView.DoSetCursor(localPoint: VPoint;
											   cursorRgn: RgnHandle); OVERRIDE;

				PROCEDURE TRowsView.DrawCell(aCell: GridCell;
											 aRect: VRect); OVERRIDE;

				PROCEDURE TRowsView.Fields(obj: TObject); OVERRIDE;

				END;									{ TRowsView }

{--------------------------------------------------------------------------------------------------}
			TColumnsView		= OBJECT (TTextGridView)
			{ TColumnsView is used to show the column headers, represented by the letters
			  A-BL. Click in this view to select an entire column or group of columns. }

				fCalcDocument:		TCalcDocument;

				PROCEDURE TColumnsView.IRes(itsDocument: TDocument;
											itsSuperView: TView;
											VAR itsParams: Ptr); OVERRIDE;
				{ Create the view from a resource template}

				PROCEDURE TColumnsView.AdornCol(aCol: INTEGER;
												area: VRect); OVERRIDE;
				{ Draw a column delimiter in the given area }

				PROCEDURE TColumnsView.CoordToString(coord: INTEGER;
													 VAR theString: Str255);

				PROCEDURE TColumnsView.DoMouseCommand(VAR themouse: VPoint;
													  event: TToolboxEvent;
													  hysteresis: Point); OVERRIDE;

				PROCEDURE TColumnsView.DoSetCursor(localPoint: VPoint;
												  cursorRgn: RgnHandle); OVERRIDE;

				PROCEDURE TColumnsView.DrawCell(aCell: GridCell;
												aRect: VRect); OVERRIDE;

				PROCEDURE TColumnsView.Fields(obj: TObject); OVERRIDE;

				PROCEDURE TColumnsView.ReSelect(cellRegion: RgnHandle);

				END;									{ TColumnsView }

{--------------------------------------------------------------------------------------------------}
			TCalcPrintHandler	= OBJECT (TStdPrintHandler)

				fCmdNumber: 		cmdNumber;

				PROCEDURE TCalcPrintHandler.CalcViewPerPage(VAR amtPerPage: VPoint); OVERRIDE;

				PROCEDURE TCalcPrintHandler.DoMenuCommand ( aCmdNumber: CmdNumber ); OVERRIDE;
				{ Overridden to handle Print Selection. }

				PROCEDURE TCalcPrintHandler.DoSetupMenus; OVERRIDE;
				{ Overridden to handle Print Selection. }

				END;									{ TCalcPrintHandler }

{--------------------------------------------------------------------------------------------------}
			TEntryView			= OBJECT (TTEView)
			{ TEntryView uses TextEdit to display and edit the selected cell's formula. }

				fCalcDocument:		TCalcDocument;
				fTouched:			BOOLEAN;
				fTEditing:			BOOLEAN;			{ TRUE: TextEdit mode }
				fFirstEdit: 		BOOLEAN;			{ TRUE: undo to previous string }
				fOldString: 		Str255; 			{ previous string, for undo/redo }

				PROCEDURE TEntryView.IRes(itsDocument: TDocument;
										  itsSuperView: TView;
										  VAR itsParams: Ptr); OVERRIDE;

				PROCEDURE TEntryView.DoKeyCommand(event: TToolboxEvent); OVERRIDE;

				FUNCTION TEntryView.DoMakeTypingCommand(ch: Char): TTETypingCommand; OVERRIDE;
				{ Actually returns a TCalcTypingCommand }

				PROCEDURE TEntryView.DoMouseCommand(VAR themouse: VPoint;
													event: TToolboxEvent;
													hysteresis: Point); OVERRIDE;

				PROCEDURE TEntryView.DoSetupMenus; OVERRIDE;

				PROCEDURE TEntryView.Draw(area: VRect); OVERRIDE;
				
				FUNCTION TEntryView.WantToBecomeTarget : BOOLEAN; OVERRIDE;

				PROCEDURE TEntryView.EditMode(editing: BOOLEAN);
				{ Enter or leave TextEdit mode.  In TextEdit mode (fTEditing=TRUE),
				  the Undo command causes an undo of the latest TextEdit action;
				  in non-TextEdit mode, Undo brings back the previous string (fOldString) }

				PROCEDURE TEntryView.Fields(obj: TObject); OVERRIDE;

				PROCEDURE TEntryView.GetAsString(VAR theString: Str255);
				{ Return the current text as a string }

				PROCEDURE TEntryView.BecameApplicationTarget; OVERRIDE;
				{ Called when this object becomes the current target  }

				PROCEDURE TEntryView.ResignedApplicationTarget; OVERRIDE;
				{ Called when this object ceases to be the current target  }

				PROCEDURE TEntryView.SetEditMode;
				{ Enter TextEdit mode and clear the current text }

				PROCEDURE TEntryView.SetToString(theString: Str255);
				{ Set the current text to theString }

				PROCEDURE TEntryView.SwapStrings;
				{ Exchange the current text with that saved in fOldString }

				END;									{ TEntryView }

{--------------------------------------------------------------------------------------------------}
			TCoordView			= OBJECT (TView)
			{ TCoordView is the view used to display the coordinates of the cell which is
			  currently selected. }

				fCalcDocument:		TCalcDocument;

				PROCEDURE TCoordView.IRes(itsDocument: TDocument;
										  itsSuperView: TView;
										  VAR itsParams: Ptr); OVERRIDE;

				PROCEDURE TCoordView.Draw(area: VRect); OVERRIDE;
			   { Draws the view seen in the window.  Every nonblank view MUST override this method }

				PROCEDURE TCoordView.Fields(obj: TObject); OVERRIDE;

				END;									{ TCoordView }

{--------------------------------------------------------------------------------------------------}
			TColumn 			= OBJECT (TObject)

				fNumber:			INTEGER;
				fFormat:			FormatRecord;
				fOldFormat: 		FormatRecord;		{ save original format for Undo }
				fWidth: 			INTEGER;

				PROCEDURE TColumn.Initialize; OVERRIDE;
				{ Put the object into a known state from which it may be safely FREEd.
				 (and hopefully a usable state) }

				PROCEDURE TColumn.IColumn(number: INTEGER);

				PROCEDURE TColumn.ReadFromDisk(aFile: TFile);

				PROCEDURE TColumn.WriteToDisk(aFile: TFile);

				PROCEDURE TColumn.ReadFromScrap(theScrap: Handle;
												VAR scrapOffset: LONGINT);

				PROCEDURE TColumn.WriteToScrap(theScrap: Handle;
											   VAR scrapOffset: LONGINT);

				PROCEDURE TColumn.Fields(obj: TObject); OVERRIDE;

				END;									{ TColumn }

{--------------------------------------------------------------------------------------------------}
			TRow				= OBJECT (TObject)

				fNumber:			INTEGER;

				PROCEDURE TRow.Initialize; OVERRIDE;
				{ Put the object into a known state from which it may be safely FREEd.
				 (and hopefully a usable state) }

				PROCEDURE TRow.IRow(number: INTEGER);

				PROCEDURE TRow.ReadFromDisk(aFile: TFile);

				PROCEDURE TRow.WriteToDisk(aFile: TFile);

				PROCEDURE TRow.ReadFromScrap(theScrap: Handle;
											 VAR scrapOffset: LONGINT);

				PROCEDURE TRow.WriteToScrap(theScrap: Handle;
											VAR scrapOffset: LONGINT);

				PROCEDURE TRow.Fields(obj: TObject); OVERRIDE;

				END;									{ TRow }

{--------------------------------------------------------------------------------------------------}
			TRowList			= OBJECT (TSortedList)
				PROCEDURE TRowList.IRowList;
				{ Init the list procedurally }

				FUNCTION TRowList.Compare(item1, item2: TObject): CompareResult; OVERRIDE;
				{ Compares rows based on row number }

				FUNCTION TRowList.GetRow(r: RowNumber): TRow;
				{ Returns the row whose number is r }
				END;									{ TRowList }

{--------------------------------------------------------------------------------------------------}
			TColumnList 		= OBJECT (TSortedList)
				PROCEDURE TColumnList.IColumnList;
				{ Init the list procedurally }

				FUNCTION TColumnList.Compare(item1, item2: TObject): CompareResult; OVERRIDE;
				{ Compares rows based on row number }

				FUNCTION TColumnList.GetColumn(c: ColumnNumber): TColumn;
				{ Returns the column whose number is c }
				END;									{ TColumnList }

{--------------------------------------------------------------------------------------------------}
			TCellList			= OBJECT (TSortedList)
				PROCEDURE TCellList.ICellList;
				{ Init the list procedurally }

				FUNCTION TCellList.Compare(item1, item2: TObject): CompareResult; OVERRIDE;
				{ Compares cells based on row and column number }

				FUNCTION TCellList.GetCell(r: RowNumber;
										   c: ColumnNumber): TCell;
				{ Returns the cell whose row,column is r,c }
				END;									{ TCellList }

{--------------------------------------------------------------------------------------------------}
			TCell				= OBJECT (TObject)
			{ TCell describes one element of the spreadsheet grid. }
				fDeleted:			BOOLEAN;			{ Whether the cell has been deleted }
				fCalcDocument:		TCalcDocument;		{ The document I belong to }

				fRow:				RowNumber;			{ Which row I'm in }
				fColumn:			ColumnNumber;		{ Which column I'm in }

				fKind:				KindOfCell; 		{ What kind of cell: text, value, empty,
														 error }
				fError: 			EvalResult; 		{ Error code from my latest evaluation }
				fEvaluating:		BOOLEAN;			{ flag for preventing self reference
														 loops }
				fValue: 			ValueType;			{ My value represented as a number }
				fValueString:		ValueString;		{ My value represented as a string, for
														 display }
				fFormula:			Str255; 			{ My contents as typed in by the user }
				fMarked: 			BOOLEAN;			{ !!!RCR For change notification }

				PROCEDURE TCell.Initialize; OVERRIDE;
				{ Put the object into a known state from which it may be safely FREEd.
				 (and hopefully a usable state) }

				PROCEDURE TCell.ICell(owningDocument: TCalcDocument;
									  r: RowNumber;
									  c: ColumnNumber);
				{ Initialize the cell's fields }

				FUNCTION TCell.Clone: TObject; OVERRIDE;
				{ Create a new cell which is a copy of this cell, except for dependents and
				  references }

				PROCEDURE TCell.CopyContents(sourceCell: TCell);
				{ Copy contents of sourceCell }

				PROCEDURE TCell.Free; OVERRIDE;
				{ Free my lists and then free me }

				PROCEDURE TCell.EvaluateFormula(updateDependencies: BOOLEAN; VAR cellChanged: BOOLEAN);
				{ Calculate my value by evaluating fFormula. If updateDependencies is TRUE, add me
				  to the list of dependents of any cells referenced in my formula. Return TRUE in
				  cellChanged if my fValue or fKind has changed }

				PROCEDURE TCell.GetAsString(VAR theString: Str255);
				{ Return my formula string }

				PROCEDURE TCell.GetValueAsString(VAR theString: Str255);
				{ Return a string representation of my value }

				PROCEDURE TCell.invalidate;
				{ Cause a cell to be marked invalid (in need of re-drawing). }

				FUNCTION TCell.IsEmpty: BOOLEAN;
				{ Return TRUE if I have no formula or value }

				PROCEDURE TCell.SetDeleteState(deleted: BOOLEAN);
				{ Mark a cell as deleted or not deleted }

				PROCEDURE TCell.SetToString(theString: Str255);
				{ Set my formula to theString and recalculate my value if necessary }

				PROCEDURE TCell.Recalculate(forceAutomatic: BOOLEAN;
											updateDependencies: BOOLEAN);
				{ Re-evaluate my formula string. Notify dependent cells of the change }

				PROCEDURE TCell.DoUpdate(VAR theChange: ChangeID;
											changedObject: TObject;
											changedBy: TObject); OVERRIDE;
				{ Update this cell in response to a change in a cell referenced in this
				cell's formula }
				
				FUNCTION  TCell.GetNoOfDependentCells: ArrayIndex;
				{ Returns the size of the dependent list }

				FUNCTION  TCell.GetNoOfReferencedCells: ArrayIndex;
				{ Returns the size of the dependent list }
				
				PROCEDURE TCell.EachReferencedCell(PROCEDURE DoToCell(aCell: TCell));
				{ Perform DoToCell on each cell referenced by this one }
				
				PROCEDURE TCell.EachDependentCell(PROCEDURE DoToCell(aCell: TCell));
				{ Perform DoToCell on each cell dependent on this one }
				
				PROCEDURE TCell.AddDependentCell(aCell: TCell);
				{ Add aCell to this cell's list of dependents }
			
				PROCEDURE TCell.RemoveDependentCell(aCell: TCell);
				{ Remove aCell from this cell's list of dependents }
				
				PROCEDURE TCell.ValueToString;
				{ Fill fValueString with the string representation of my value }

				FUNCTION TCell.GetDiskSize(infoRecordOnly: BOOLEAN): INTEGER;

				PROCEDURE TCell.ReadFromDisk(aFile: TFile);

				PROCEDURE TCell.WriteToDisk(aFile: TFile);

				PROCEDURE TCell.ReadFromScrap(theScrap: Handle;
											  VAR scrapOffset: LONGINT);

				PROCEDURE TCell.WriteToScrap(theScrap: Handle;
											 VAR scrapOffset: LONGINT);

				{ Inspecting }

				PROCEDURE TCell.Fields(obj: TObject); OVERRIDE;

				PROCEDURE TCell.GetInspectorName(VAR inspectorName: Str255); OVERRIDE;

				END;									{ TCell }

{--------------------------------------------------------------------------------------------------}
			TCalcSelectCommand	= OBJECT (TCellSelectCommand)
			{ TCalcSelectCommand is a command object created to handle mouse movement when you
			  click in a TCellsView. }

				fCalcDocument:		TCalcDocument;

				PROCEDURE TCalcSelectCommand.ICalcSelectCommand(itsDocument: TCalcDocument;
																itsView: TGridView;
																VAR themouse: VPoint;
																theShiftKey, theCmdKey: BOOLEAN);

				PROCEDURE TCalcSelectCommand.ComputeNewSelection(VAR clickedCell: GridCell);
					OVERRIDE;

				END;									{ TCalcSelectCommand }

{--------------------------------------------------------------------------------------------------}
			TRowSelector		= OBJECT (TCalcSelectCommand)
			{ TRowSelector is a command object created to handle mouse movement when you click
			  in a TRowsView. }

				fCellSelector:		TCalcSelectCommand; { command to handle cells view selection }

				PROCEDURE TRowSelector.IRowSelector(itsDocument: TCalcDocument;
													itsView: TGridView;
													VAR themouse: VPoint;
													theShiftKey, theCmdKey: BOOLEAN);
				{ Create a TCalcSelectCommand to handle selection in the cells view in parallel
				  with selection in our view, the rows view }

				PROCEDURE TRowSelector.Free; OVERRIDE;
				{ Dispose of our cell selector }

				PROCEDURE TRowSelector.ComputeAnchorCell(VAR clickedCell: GridCell); OVERRIDE;
				PROCEDURE TRowSelector.ComputeNewSelection(VAR clickedCell: GridCell); OVERRIDE;
				PROCEDURE TRowSelector.DoIt; OVERRIDE;
				{ Perform these operations for the cells view as well as for the rows view }

				FUNCTION TRowSelector.TrackMouse(aTrackPhase: TrackPhase;
												 VAR anchorPoint, previousPoint, nextPoint: VPoint;
												 mouseDidMove: BOOLEAN): TTracker; OVERRIDE;

				END;									{ TRowSelector }

{--------------------------------------------------------------------------------------------------}
			TColumnSelector 	= OBJECT (TCalcSelectCommand)
			{ TColumnSelector is a command object created to handle mouse movement when you
			  click in a TColumnsView. }

				fCellSelector:		TCalcSelectCommand; { command to handle cells view selection }

				PROCEDURE TColumnSelector.IColumnSelector(itsDocument: TCalcDocument;
														  itsView: TGridView;
														  VAR themouse: VPoint;
														  theShiftKey, theCmdKey: BOOLEAN);
				{ Create a TCalcSelectCommand to handle selection in the cells view in parallel with
				  selection in our view, the column headers }

				PROCEDURE TColumnSelector.Free; OVERRIDE;
				{ Dispose of our cell selector }

				PROCEDURE TColumnSelector.ComputeAnchorCell(VAR clickedCell: GridCell); OVERRIDE;
				PROCEDURE TColumnSelector.ComputeNewSelection(VAR clickedCell: GridCell); OVERRIDE;
				PROCEDURE TColumnSelector.DoIt; OVERRIDE;
				{ Perform these operations for the cells view as well as for the column headers }

				FUNCTION TColumnSelector.TrackMouse(aTrackPhase: TrackPhase;
													VAR anchorPoint, previousPoint,
													nextPoint: VPoint;
													mouseDidMove: BOOLEAN): TTracker; OVERRIDE;

				END;									{ TColumnSelector }

{--------------------------------------------------------------------------------------------------}
			TColumnFormatter	= OBJECT (TCommand)
			{ TColumnFormatter is a command object created to change the style or justification
			  of a column. It is not undoable. }

				fCalcDocument:		TCalcDocument;
				fSelection: 		RgnHandle;			{ columns selected for formatting }

				PROCEDURE TColumnFormatter.IFormatter(itsDocument: TCalcDocument;
													  itsCommand: INTEGER);

				PROCEDURE TColumnFormatter.Free; OVERRIDE;

				PROCEDURE TColumnFormatter.DoIt; OVERRIDE;
				PROCEDURE TColumnFormatter.UndoIt; OVERRIDE;
				PROCEDURE TColumnFormatter.RedoIt; OVERRIDE;

				PROCEDURE TColumnFormatter.Fields(obj: TObject); OVERRIDE;

				END;									{ TColumnFormatter }

{--------------------------------------------------------------------------------------------------}
			TColumnSizer		= OBJECT (TTracker)
			{ TColumnSizer is a command object created to handle mouse movement when you click
			  on the boundary between two columns. }

				fCalcDocument:		TCalcDocument;
				fCellsView: 		TCellsView;
				fLeftEdge:			INTEGER;

				fColumn:			TColumn;
				fNewWidth:			INTEGER;
				fOldWidth:			INTEGER;			{ remember previous width for Undo }

				PROCEDURE TColumnSizer.IColumnSizer(itsDocument: TCalcDocument;
													c: ColumnNumber);

				FUNCTION TColumnSizer.TrackMouse(aTrackPhase: TrackPhase;
												 VAR anchorPoint, previousPoint, nextPoint: VPoint;
												 mouseDidMove: BOOLEAN): TTracker; OVERRIDE;

				PROCEDURE TColumnSizer.TrackFeedback(aTrackPhase: TrackPhase;
													 anchorPoint: VPoint;
													 previousPoint: VPoint;
													 nextPoint: VPoint;
													 mouseDidMove: BOOLEAN;
													 turnItOn: BOOLEAN); OVERRIDE;

				PROCEDURE TColumnSizer.TrackConstrain(aTrackPhase: TrackPhase;
													  anchorPoint: VPoint;
													  previousPoint: VPoint;
													  VAR nextPoint: VPoint;
													  mouseDidMove: BOOLEAN); OVERRIDE;

				PROCEDURE TColumnSizer.DoIt; OVERRIDE;

				PROCEDURE TColumnSizer.UndoIt; OVERRIDE;

				PROCEDURE TColumnSizer.RedoIt; OVERRIDE;

				PROCEDURE TColumnSizer.SetColumnWidth(newWidth: INTEGER);

				PROCEDURE TColumnSizer.Fields(obj: TObject); OVERRIDE;

				END;									{ TColumnSizer }

{--------------------------------------------------------------------------------------------------}
			TCalcTypingCommand	= OBJECT (TTETypingCommand)
			{ We define our own typing command in order to remember which cell the typing was
			  performed in. Then if the user makes another selection we can undo/redo the
			  correct cell. }

				fCellsView: 		TCellsView;
				fTargetCell:		GridCell;			{ cell selected when command was created }

				PROCEDURE TCalcTypingCommand.ITETypingCommand(itsTEView: TTEView;
															  itsFirstChar: Char); OVERRIDE;
				{ Initialize the command object. Save the current cell selection for later
				  Undo/Redo }

				PROCEDURE TCalcTypingCommand.AddCharacter(aChar: Char); OVERRIDE;
				{ Overridden to keep text visible when it overflows the entry view's box }

				PROCEDURE TCalcTypingCommand.UndoIt; OVERRIDE;
				{ Undo the typing in the target cell }

				PROCEDURE TCalcTypingCommand.RedoIt; OVERRIDE;
				{ Redo the typing in the target cell }

				PROCEDURE TCalcTypingCommand.Fields(obj: TObject); OVERRIDE;

				END;									{ TCalcTypingCommand }

{--------------------------------------------------------------------------------------------------}
			TCellEditCommand	= OBJECT (TCommand)
			{ TCellEditCommand is a command object created to handle the cCopy, cCut, and cClear
			  menu commands. }

				fCalcDocument:		TCalcDocument;
				fSelection: 		RgnHandle;			{ cells selected when command was created }

				PROCEDURE TCellEditCommand.ICellEditCommand(itsDocument: TCalcDocument;
															itsCommand: INTEGER);
				{ Initialize the command object. Save the current cell selection for later
				  Undo/Redo }

				PROCEDURE TCellEditCommand.Free; OVERRIDE;
				{ Clean up our stuff }

				PROCEDURE TCellEditCommand.DoIt; OVERRIDE;
				{ Edit the currently selected cells }

				PROCEDURE TCellEditCommand.UndoIt; OVERRIDE;
				{ Undo the edit of the cells }

				PROCEDURE TCellEditCommand.RedoIt; OVERRIDE;
				{ Redo the edit of the cells }

				PROCEDURE TCellEditCommand.Commit; OVERRIDE;
				{ Commit the edit of the cells }

				PROCEDURE TCellEditCommand.CopySelection;
				{ Copy the currently selected cell(s) into the clipboard }

				PROCEDURE TCellEditCommand.DeleteSelection;
				{ Delete the currently selected cell(s) }

				PROCEDURE TCellEditCommand.RestoreSelection;
				{ Restore the deleted cells }

				PROCEDURE TCellEditCommand.ReSelect;
				{ Change the selection back to what it was when this command was created }

				PROCEDURE TCellEditCommand.Fields(obj: TObject); OVERRIDE;

				END;									{ TCellEditCommand }

{--------------------------------------------------------------------------------------------------}
			TCellPasteCommand	= OBJECT (TCommand)
			{ TCellPasteCommand is a command object created to handle the cPaste menu command. }

				fClipDocument:		TCalcDocument;
				fCalcDocument:		TCalcDocument;
				fSelection: 		RgnHandle;			{ cells selected when command was created }
				fReplacedCells: 	TList;				{ the cells we pasted over }

				PROCEDURE TCellPasteCommand.ICellPasteCommand(itsDocument: TCalcDocument);
				{ Initialize the command object }

				PROCEDURE TCellPasteCommand.Free; OVERRIDE;
				{ We don' need this command no more }

				PROCEDURE TCellPasteCommand.DoIt; OVERRIDE;
				{ Paste to the currently selected cells. Save this selection for Undo/Redo. }

				PROCEDURE TCellPasteCommand.UndoIt; OVERRIDE;
				{ Undo the paste of the cells }

				PROCEDURE TCellPasteCommand.RedoIt; OVERRIDE;
				{ Redo the paste of the cells }

				PROCEDURE TCellPasteCommand.UpdateViews;
				{ Redisplay the affected cells }

				PROCEDURE TCellPasteCommand.Fields(obj: TObject); OVERRIDE;

				END;									{ TCellPasteCommand }

	IMPLEMENTATION
{$I UCalc.inc1.p}
END.
