{[a-,body+,h-,o=100,r+,rec+,t=4,u+,#+,j=20/57/1$,n+]}
{ UBetterFeedbackCmd.p }
{ Copyright  1988-1991 by Apple Computer, Inc. All rights reserved.}

{[f-]}
{	This unit attempts to provide smooth flicker-free feedback during command tracking.

		  T H E O R Y	O F   O P E R A T I O N

	By synchronizing drawing to the vertical retrace interrupt, this unit attempts
	to improve the feedback that mouse trackers provide.

	Ideally, this behavior would be part of MacApp's TApplication.TrackMouse method.
	What TApplication.TrackMouse should really do is:
		a) wait for the vertical retrace interrupt of the device that the view most intersects
		b) erase the old tracker feedback by calling tracker.TrackFeedback(, turn It Off, )
		c) immediately draw the new feedback by calling tracker.TrackFeedback(, turn It on, )

	However, with the current architecture, this is impossible. TApplication.TrackMouse currently
	does the following:
		a) call tracker.TrackFeedback(, turn It Off, ) to erase the previous tracker feedback
		b) call tracker.TrackMouse(trackMove, ) to tell the tracker the track phase
		c) call tracker.TrackFeedback(, turn It On, ) to draw the new tracker feedback

	UBetterFeedbackCmd provides a way to work within the current architecture to provide a close
	attempt at smooth flicker-free feedback. To take advantage of this unit, create a
	subclass of TBetterFeedbackCmd. Then, OVERRIDE PROCEDURE TBetterFeedbackCmd.TrackFeedback()
	and as the first statement of this method call:
			INHERITED TrackFeedback(anchorPoint, nextPoint, turnItOn, mouseDidMove);
	so that TBetterFeedbackCmd.TrackFeedback can synch to the vertical retrace interrupt.
	Then the tracker should go ahead and draw/erase the feedback.

	For an example, see DrawShapes: TShapeSketcher, TShapeDragger, and TShapeSelector.

}
{[f+]}

UNIT UBetterFeedbackCmd;

	INTERFACE

		USES
			{
			 List units defining any types or constants that are required for your interface
			section (e.g. MacAppTypes or QuickDraw)
			 Also list units defining the classes that you are subclassing from in the interface
			section.  DO NOT list units for classes that you are only referencing
			(e.g. instance vars, parameter types or function results).
			 Also list the units used by the above (in their interface section).
			}
			
		Types, MacAppTypes, Memory, UPascalObject, UObject, UList, AppleEvents, UEvent, UCommand, Editions, Scrap, Dialogs
		 , UEvtHandler, UApplication, UDocument, Balloons, UAdorners, UView, UWindow, UBehavior, UViewBehavior, UPrintHandler, UFailure,
		 UGeometry, OSUtils, UMacAppUniversal, UMacAppUtilities, Menus, UMenuMgr, UPatch, UMacAppGlobals,

		{  Building Blocks }
		PrintTraps, UPrinting, UGridView, UTEView, UControl, UDialog, UPopup;
			
		CONST

			kInstall			= TRUE; 				{ pass to BetterFeedback: install our VBL? }
			kBetterFeedbackDesired = TRUE;				{ pass to IBetterFeedbackCmd: desire better
														 feedback? }

		TYPE
			{
			 Declare as external the classes that you are only referencing
			(e.g. instance vars, parameter types or function results) that are not covered by
			the USES in the interface section.
			}
			TDocument			= OBJECT; EXTERNAL;
			TScroller			= OBJECT; EXTERNAL;
			TView				= OBJECT; EXTERNAL;

			TBetterFeedbackCmd	= OBJECT (TTracker)

				fBetterFeedbackInstalled: BOOLEAN;		{ are the synchronization routines in? }

				fBetterFeedbackDesired: BOOLEAN;		{ want the synchronization routines? }

				PROCEDURE TBetterFeedbackCmd.IBetterFeedbackCmd(itsCmdNumber: CmdNumber;
																itsDocument: TDocument;
																itsView: TView;
																itsScroller: TScroller;
																itsMouse: VPoint;
																betterFeedbackDesired: BOOLEAN);

				PROCEDURE TBetterFeedbackCmd.Free; OVERRIDE;

				{ for tracking }
				PROCEDURE TBetterFeedbackCmd.BetterFeedback(install: BOOLEAN);

				PROCEDURE TBetterFeedbackCmd.WaitBetterFeedback;

				FUNCTION TBetterFeedbackCmd.TrackMouse(aTrackPhase: TrackPhase; VAR anchorPoint,
													   previousPoint, nextPoint: VPoint;
													   mouseDidMove: BOOLEAN): TTracker; OVERRIDE;

				PROCEDURE TBetterFeedbackCmd.TrackFeedback(aTrackPhase: TrackPhase;
														   anchorPoint: VPoint;
														   previousPoint: VPoint;
														   nextPoint: VPoint;
														   mouseDidMove: BOOLEAN;
														   turnItOn: BOOLEAN); OVERRIDE;

				{ Inspecting }

				PROCEDURE TBetterFeedbackCmd.Fields(obj: TObject); OVERRIDE;

				END;

	IMPLEMENTATION

{$I UBetterFeedbackCmd.inc1.p}

END.
