{$P}
{[a-,body+,h-,o=100,r+,rec+,t=4,u+,#+,j=20/57/1$,n+]}
{ UDrawShapes.inc1.p}
{ Copyright  1985 - 1991 by Apple Computer, Inc.  All rights reserved.}

USES
	UViewServer;

CONST
	kRainbowArrow		= 140;

	kToolsPaletteRSRCID = 1007; 						{Resource id for the floating window with
														 tools palette }
	kPatternsPaletteRSRCID = 1008;						{Resource id for the floating window with
														 Patterns palette }
	kToolsPaletteWidth	= 41;							{Width of the palette}
	kToolsPaletteHeight = 160;							{Height of the palette}

	kMinWidth			= 20;							{minimum width of new shapes}
	kMinHeight			= 20;							{minimum height of new shapes}
	{ The above two constants define the minimum size a newly-sketched shape
	  must become before it is considered a legitimate attempt to draw }

	kStaggerAmount		= 16;							{Amount to stagger windows by}

	kColorMenuBar		= 131;							{Menu bar for a color system}
	kNonColorMenuBar	= 132;							{Menu bar for a black & white system}

	kPickerPrompt		= 256;							{'STR ' resource for Color Picker}

	cChangeShade		= 1012; 						{Buzz command for "Undo Shade Change"}
	cChangeColor		= 1013; 						{Buzz command for "Undo Color Change"}
	cChangeTool 		= 1014; 						{Buzz command for "Undo Tool Change"}

	kNoToolSelection	= - 1;

VAR
	gPat:				ARRAY [1..kTotalPatterns] OF Pattern;

	{ prototype shapes for the palette }

	gShapesArray:		ARRAY [1..kShapesInPalette] OF TShape;

	gArwBitMap: 		BitMap; 						{bitmap used to draw the arrow in palette}

	gClipMargin:		Point;							{the top & left margins to use when
														 displaying shapes in the Clipboard}
	gPasteReplacesSelection: BOOLEAN;					{Tells whether PASTE should REPLACE the
														 existing selection, or instead simply add
														 new shapes without replacement. Default:
														 FALSE; change its value by using the "More
														 Debug" menu, obtainable by typing
														 command-D}

	gConstrainDrags:	BOOLEAN;						{Whether dragging shapes with the mouse
														 should be constrained so that nothing
														 overlaps the view's borders. Default:
														 TRUE; change its value by using the "More
														 Debug" menu, obtainable by typing
														 command-D}

	gStaggerCount:		INTEGER;						{For SimpleStagger}

	gRainbowArrow:		CCrsrHandle;

	gToolsMenu: 		TToolsMenu;
	gPatternsMenu:		TPatternsMenu;

	gBetterFeedback:	BOOLEAN;						{ TRUE to invoke BetterFeedback routines }

	gToolsPaletteWindow: TWindow;
	gPatternsWindow:	TWindow;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE RotatePat(VAR pat: Pattern);

	VAR
		temp: byte;
		index: integer;

	BEGIN
	temp := pat[0];
	FOR index := 0 TO 6 DO
		pat[index] := pat[index + 1];
	pat[7] := temp;
	END;

{--------------------------------------------------------------------------------------------------}
{$S AInit}

PROCEDURE TShapeApplication.Initialize; OVERRIDE;

	BEGIN
	INHERITED Initialize;

	{ This has to be done here since TApplication.Initialize plays with this field }
	IF qNeedsColorQD | gConfiguration.hasColorQD THEN
		gApplication.fMBarDisplayed := kColorMenuBar
	ELSE
		gApplication.fMBarDisplayed := kNonColorMenuBar;
	END;

{--------------------------------------------------------------------------------------------------}
{$S AInit}

PROCEDURE TShapeApplication.IShapeApplication;			{Initialize the application}

	VAR
		i:					INTEGER;
		r:					Rect;
		box:				TBox;
		circle: 			TCircle;
		hBox:				THeavyBox;
		toolsMenu:			TToolsMenu;
		PatternMenu:		TPatternsMenu;
		patListHdl: 		Handle;
		hState: 			SignedByte;
		numPatterns:		INTEGER;
		startAddress:		LONGINT;
		patternPtr:			^Pattern;

	BEGIN

	IApplication(kDocType); 							{Generic initialization}

	{ copy patterns into global array of patterns }
	patListHdl := GetResource('PAT#', 128);
	IF patListHdl = NIL THEN
		BEGIN
		gPat[1] := white;
		gPat[2] := ltGray;
		gPat[3] := gray;
		gPat[4] := dkGray;
		gPat[5] := black;
		FOR i := 6 TO kTotalPatterns DO
			gPat[i] := black;
		END
	ELSE
		BEGIN
		hState := HGetState(patListHdl);
		IF LockHandleHigh(patListHdl) = SignedByte(0) THEN;
		numPatterns := Min(kTotalPatterns, IntegerHandle(patListHdl)^^); { first word in pattern
																		  list is array size }
{		patternPtr := POINTER(Ord4(patListHdl^) + Sizeof(INTEGER));
		FOR i := 1 TO numPatterns DO
			BEGIN
			gPat[i] := patternPtr^;
			patternPtr := POINTER(Ord4(patternPtr) + Sizeof(Pattern));
			END;
}
		
		startAddress := ORD (patListHdl^) + sizeof(INTEGER); { the address where the patterns start }
		FOR i := 1 TO numPatterns DO
			BlockMove(POINTER(startAddress + (i - 1) * sizeof(Pattern)), @gPat[i], sizeof(Pattern));
		HSetState(patListHdl, hState);
		END;

	{ now create the floating palette window }
	gToolsPaletteWindow := gViewServer.NewTemplateWindow(kToolsPaletteRSRCID, NIL);

	{ now create the floating Patterns window }
	gPatternsWindow := gViewServer.NewTemplateWindow(kPatternsPaletteRSRCID, NIL);

	{ Install our custom tools menu }
	New(toolsMenu);
	toolsMenu.IToolsMenu;
	gToolsMenu := toolsMenu;

	{ Install our custom pattern menu }
	New(PatternMenu);
	PatternMenu.IPatternsMenu;
	gPatternsMenu := PatternMenu;

	{ Set the standard margins to use in the clipboard }
	SetPt(gClipMargin, 16, 16);

	SetRect(r, 10, 50, 28, 70); 						{Define the prototype shapes}
	New(box);
	box.IBox(r, IDBox);
	gShapesArray[IDBox] := box;

	OffSetRect(r, 0, 40);
	New(circle);
	circle.ICircle(r, IDCircle);
	gShapesArray[IDCircle] := circle;

	OffSetRect(r, 0, 40);
	New(hBox);
	hBox.IHeavyBox(r, IDhBox);
	gShapesArray[IDhBox] := hBox;

	WITH gArwBitMap DO									{Define the arrow bitmap to be drawn in the
														 palette}
		BEGIN
		rowBytes := 2;
		SetRect(bounds, 0, 0, 16, 16);
		baseAddr := @arrow.data;
		END;

	gPasteReplacesSelection := FALSE;
	gConstrainDrags := TRUE;
	gStaggerCount := 0;

	IF qNeedsColorQD | gConfiguration.hasColorQD THEN
		BEGIN
		{ Unlike GetCursor, GetCCursor makes a copy of the color cursor
		  resource.  Therefore, you should make one call to GetCCursor
		  and multiple calls to SetCCursor }
		gRainbowArrow := GetCCursor(kRainbowArrow);
		FailNil(gRainbowArrow);
		END;
	gBetterFeedback := kBetterFeedbackDesired;

	IF qTemplateViews & gDeadStripSuppression THEN
		BEGIN
		IF Member(TObject(NIL), TShapeView) THEN;
		IF Member(TObject(NIL), TToolsPalette) THEN;
		IF Member(TObject(NIL), TPatternsPalette) THEN;
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$IFC qDebug}
{$S ASelCommand}

PROCEDURE TShapeApplication.DoCommandKey(event: TEvent); OVERRIDE;

{ This illustrates how to have a 'Command-key-only' command, i.e. a command
  which is NOT in a menu, but rather only available when the user types
  the 'command' key and another key concurrently.  In this example, the
  user presses 'Command-D' (the D can be in upper or lower case) to
  request that the special 'more debug' menu be put up (or be taken down
  if it was already up) }

	BEGIN
	CASE UprChar(event.fCharacter) OF
		'D':
			DoMenuCommand(cToggleMoreDebugMenu);

		OTHERWISE
			INHERITED DoCommandKey(event);
	END;
	END;
{$ENDC}

{--------------------------------------------------------------------------------------------------}
{$S AOpen}

FUNCTION TShapeApplication.DoMakeDocument(itsCmdNumber: CmdNumber; itsFile: TFile): TDocument;
{ NB: Not used to create the document for a shape view in the Clipboard }

	VAR
		shapeDocument:		TShapeDocument;

	BEGIN
	New(shapeDocument);
	shapeDocument.IShapeDocument(itsFile);
	DoMakeDocument := shapeDocument;
	END;

{--------------------------------------------------------------------------------------------------}
{$IFC qDebug}
{$S ASelCommand}

PROCEDURE TShapeApplication.DoMenuCommand(aCmdNumber: CmdNumber); OVERRIDE;

	BEGIN
	CASE aCmdNumber OF

		cPasteReplacesSelection:
			gPasteReplacesSelection := NOT gPasteReplacesSelection;

		cConstrainDrags:
			gConstrainDrags := NOT gConstrainDrags;

		cToggleMoreDebugMenu:
			BEGIN
			IF GetMHandle(mMoreDebug) = NIL THEN		{menu not currrently up--put it up}
				InsertMenu(MAGetMenu(mMoreDebug), 0)
			ELSE										{menu currently up -- take it down}
				DeleteMenu(mMoreDebug);
			InvalidateMenuBar;							{ Get it redrawn }
			END;

		OTHERWISE
			INHERITED DoMenuCommand(aCmdNumber);
	END;												{Case}
	END;
{$ENDC}

{--------------------------------------------------------------------------------------------------}
{$IFC qDebug}
{$S ARes}

PROCEDURE TShapeApplication.DoSetupMenus; OVERRIDE;
{The only menu commands handled here are the following two debugging commands:}

	BEGIN
	INHERITED DoSetupMenus;
	EnableCheck(cPasteReplacesSelection, TRUE, gPasteReplacesSelection);
	EnableCheck(cConstrainDrags, TRUE, gConstrainDrags);
	END;
{$ENDC}

{--------------------------------------------------------------------------------------------------}
{$IFC qDebug}
{$S ADebug}

PROCEDURE TShapeApplication.IdentifySoftware; OVERRIDE;

	BEGIN
	WriteLn('DrawShapes Source date: 6 June 86; Compiled on: ', COMPDATE, ' @ ', COMPTIME);
	INHERITED IdentifySoftware;
	END;
{$ENDC}

{--------------------------------------------------------------------------------------------------}
{$S AClipboard}
FUNCTION TShapeApplication.MakeViewForAlienClipboard: TView; OVERRIDE;
{ Launch a view to represent the data found in the Clipboard at
  application start-up time, or when returning from an excursion
  to Switcher, or when returning from a Desk Accessory }

	VAR
		offset: 			LONGINT;
		clipShapeView:		TShapeView;
		clipShapeDoc:		TShapeDocument;
		clipShapes: 		ShapesOnClipboard;
		aNewShape:			TShape;
		i:					INTEGER;
		err:				LONGINT;
		perm:				BOOLEAN;
		fi: 				FailInfo;

	PROCEDURE HdlFailure(error: OSErr; message: LONGINT);

		BEGIN
		Handle(clipShapes) := DisposeIfHandle(clipShapes);

		clipShapeDoc := TShapeDocument(FreeIfObject(clipShapeDoc));
		END;

	BEGIN
	clipShapes := NIL;

	{Before doing anything else, make sure the scrap contains shapes}
	IF GetScrap(NIL, kShapeClipType, offset) > 0 THEN	{found my kind of data }
		BEGIN
		New(clipShapeDoc);
		clipShapeDoc.IShapeDocument(NIL);

		CatchFailures(fi, HdlFailure);
		New(clipShapeView);
		clipShapeView.IShapeView(clipShapeDoc, NIL, TRUE);

		clipShapeDoc.fShapeView := clipShapeView;

		clipShapes := ShapesOnClipboard(NewPermHandle(0));
		FailSpaceIsLow;

		perm := PermAllocation(TRUE);					{Don't allow GetScrap to use temp space}
		err := GetScrap(Handle(clipShapes), kShapeClipType, offset);
		perm := PermAllocation(perm);					{Restore perm allocation setting}

  { Only a negative result indicates an error--FailOSErr considers
	any non-zero result an error.}
		IF err < 0 THEN
			FailOSErr(err);

		FOR i := 0 TO clipShapes^^.theNumberOfShapes - 1 DO
			BEGIN
			aNewShape := TShape(gShapesArray[clipShapes^^.theShapes[i].theId].Clone);
			WITH aNewShape, clipShapes^^.theShapes[i] DO
				BEGIN
				fPattern := thePattern;
				fColor := theColor;
				fExtentRect := theRect;
				END;
			clipShapeDoc.AddShape(aNewShape);
			END;

		Success(fi);
		Handle(clipShapes) := DisposeIfHandle(clipShapes);

		MakeViewForAlienClipboard := clipShapeView;
		END
	ELSE
		MakeViewForAlienClipboard := INHERITED MakeViewForAlienClipboard;
	END;

{--------------------------------------------------------------------------------------------------}
{$S AOpen}

PROCEDURE TShapeDocument.IShapeDocument(itsFile: TFile);

	VAR
		fi: 				FailInfo;

	PROCEDURE HdlNewList(error: OSErr; message: LONGINT);

		BEGIN
		Free;
		END;

	BEGIN
	fShapeView := NIL;
	fToolsPalette := NIL;
	fShapeList := NIL;									{Just in case IFileBasedDocument fails}
	IFileBasedDocument(	itsFile,
						kDocType,
						kSignature,
						kScrapType,
						kUsesDataFork,
						kUsesRsrcFork,
						NOT kDataOpen,
					   NOT kRsrcOpen);

	CatchFailures(fi, HdlNewList);						{ In case NewList fails.}
	fShapeList := NewList;
	Success(fi);

{$IFC qDebug}
	fShapeList.SetEltType('TShape');
{$ENDC}

	fSavePrintInfo := TRUE;

	fReopening := FALSE;
	fFiltering := FALSE;
	fReplaceCommand := NIL;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TShapeDocument.AddShape(shape: TShape);

	BEGIN
	fShapeList.InsertLast(shape);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TShapeDocument.DeleteShape(shape: TShape);
{ Doesn't work for shape still belonging to a command
  (i.e., not yet committed to the document }

	BEGIN
	fShapeList.Delete(shape);
	shape := TShape(FreeIfObject(shape));
	END;

{--------------------------------------------------------------------------------------------------}
{$S AOpen}

PROCEDURE TShapeDocument.DoMakeViews(forPrinting: BOOLEAN);

	VAR
		shapeView:			TShapeView;
		toolsPalette:		TToolsPalette;
		aWindow:			TWindow;
		aDocState:			DocState;
		minSize:			Point;
		maxSize:			Point;

	PROCEDURE CreateProceduralShapeView;
		{ CreateProceduralShapeView used when creating views procedurally to create the shapes view
		in both printing & non-printing cases }

		BEGIN
		New(shapeView);
		shapeView.IShapeView(SELF, toolsPalette, FALSE);

		fShapeView := shapeView;
		END;

	PROCEDURE RestoreWindow;
		{ RestoreWindow restores the window & scroller using the settings in the documents fDocState
		field }

		BEGIN
		aDocState := fDocState;
		WITH aDocState DO
			BEGIN
			aWindow.Locate(theLocation, FALSE);
			aWindow.Resize(theSize, FALSE);
			END;
		aWindow.ForceOnScreen;
		WITH aDocState.theScrollPosition DO
			fShapeView.fScroller.SetLocalOrigin(aDocState.theScrollPosition, FALSE);
		END;

	BEGIN
	IF forPrinting THEN
		BEGIN
		IF qTemplateViews THEN
			BEGIN
			shapeView := TShapeView(gViewServer.DoCreateViews(SELF, NIL, kShapeViewRSRCID, gZeroVPt));
			fShapeView := shapeView;
			END
		ELSE
			BEGIN
			toolsPalette := NIL;
			CreateProceduralShapeView;
			END;
		END 											{ this is the end of the "forPrinting=TRUE"
														 case }
	ELSE
		BEGIN
		IF qTemplateViews THEN
			BEGIN
			aWindow := gViewServer.NewTemplateWindow(kShapeWindowRSRCID, SELF);

			fToolsPalette := TToolsPalette(aWindow.FindSubView('PLTT'));
			FailNil(fToolsPalette);

			fShapeView := TShapeView(aWindow.FindSubView('SHAP'));
			FailNil(fShapeView);

			fShapeView.fToolsPalette := fToolsPalette;
			END
		ELSE
			BEGIN
			New(toolsPalette);

			toolsPalette.IToolsPalette(SELF, NIL);
			fToolsPalette := toolsPalette;

			CreateProceduralShapeView;

			aWindow := gViewServer.NewPaletteWindow(kShapeWindowRSRCID, kWantHScrollBar, kWantVScrollBar, SELF,
										fShapeView, fToolsPalette, kToolsPaletteWidth,
					   kLeftPalette);
			END;

		fShapeView.fScroller := fShapeView.GetScroller(TRUE);

		IF fReopening THEN
			RestoreWindow
		ELSE
			BEGIN
			aWindow.AdaptToScreen;
			aWindow.SimpleStagger(kStaggerAmount, kStaggerAmount, gStaggerCount);
			END;

		{ set window's resize limits so it can't become wider than the shapeview's edge }
		WITH aWindow.fResizeLimits DO
			BEGIN
			minSize := topLeft;
			maxSize := botRight;
			END;
		WITH maxSize DO
			h := Min(fShapeView.fSize.h + fToolsPalette.fSize.h + kSBarSizeMinus1, h);
		aWindow.SetResizeLimits(minSize, maxSize);
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TShapeDocument.DoSetupMenus;

	VAR
		i:					INTEGER;

	BEGIN
	INHERITED DoSetupMenus;

	FOR i := cArrow TO cHBox DO
		Enable(i, TRUE);
	END;												{ DoSetupMenus }

{--------------------------------------------------------------------------------------------------}
{$S ASelCommand}

PROCEDURE TShapeDocument.DoMenuCommand(aCmdNumber: CmdNumber); OVERRIDE;

	VAR
		aWindow:			TWindow;
		activeDocument: 	TDocument;
		activePalette:		TToolsPalette;

	BEGIN
	INHERITED DoMenuCommand(aCmdNumber);
	END;

{--------------------------------------------------------------------------------------------------}
{$S AWriteFile}

PROCEDURE TShapeDocument.DoNeedDiskSpace(itsFile: TFile;
															VAR dataForkBytes, rsrcForkBytes: LONGINT);

	BEGIN
	{ get Print record requirements }
	INHERITED DoNeedDiskSpace(itsFile, dataForkBytes, rsrcForkBytes);

	dataForkBytes := dataForkBytes + fShapeList.GetSize * sizeof(ShapeData);
	rsrcForkBytes := rsrcForkBytes + kRsrcTypeOverhead + kRsrcOverhead + sizeof(DocState);
	END;

{
Doc file has the following format:
  Data Fork:
	(a)  kSizePrintInfo (120 bytes) => PrintInfo
	(b)  The rest => The shapes themselves

 Resource Fork:
	(a)  SIZEOF(DocState) => DocState (rsrc type: 'DSTA', number: 1)
}

{--------------------------------------------------------------------------------------------------}
{$S AReadFile}

PROCEDURE TShapeDocument.DoRead(aFile: TFile; forPrinting: BOOLEAN);

	VAR
		i:					INTEGER;
		id: 				INTEGER;
		count:				LONGINT;
		newShape:			TShape;
		docStateHandle: 	HDocState;

	BEGIN
	INHERITED DoRead(aFile, forPrinting);				{read print info stuff}

	IF aFile.IsRsrcForkOpen THEN
		BEGIN
		docStateHandle := HDocState(GetResource(kDocRsrcKind, kDocStateID));
		IF docStateHandle <> NIL THEN
			BEGIN
			fDocState := docStateHandle^^;
			fReopening := TRUE;
			END;
		END
	ELSE
		BEGIN
{$IFC qDebug}
		ProgramBreak('Resource fork doesn''t exist for saved file');
{$ENDC}
		Failure(1, 0);
		END;

	FOR i := 1 TO fDocState.theNumberOfShapes DO
		BEGIN
		count := sizeof(INTEGER);
		FailOSErr(aFile.ReadData(@id, count));

		IF (id >= 1) & (id <= kShapesInPalette) THEN
			BEGIN
			newShape := TShape(gShapesArray[id].Clone);

			newShape.ReadFromFile(aFile);

			AddShape(newShape);
			END
{$IFC qDebug}
		ELSE
			WriteLn('Ignored invalid shape ID = ', id: 1, ' shape #: ', i: 1)
{$ENDC} ;
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S AWriteFile}

PROCEDURE TShapeDocument.DoWrite(aFile: TFile; makingCopy: BOOLEAN);

	VAR
		vhs:				VHSelect;
		aDocState:			DocState;
		count:				LONGINT;
		window: 			TWindow;
		numberOfShapes: 	INTEGER;
		dummyRect:			Rect;
		docStateHandle: 	HDocState;

	PROCEDURE WriteShape(shape: TShape);

		BEGIN
		shape.WriteToFile(aFile);
		END;

	BEGIN
	INHERITED DoWrite(aFile, makingCopy);				{write print info stuff}

	{ Call SurveyShapes just to get number of shapes }
	SurveyShapes(FALSE, numberOfShapes, dummyRect);
	window := TWindow(fWindowList.First);

	docStateHandle := HDocState(NewPermHandle(sizeof(DocState)));

	WITH docStateHandle^^ DO
		BEGIN
		theLocation := window.fLocation;
		theSize := window.fSize;
		theScrollPosition := fShapeView.fScroller.fTranslation;
		theNumberOfShapes := numberOfShapes;
		END;

	AddResource(Handle(docStateHandle), kDocRsrcKind, kDocStateID, 'Doc State');
	FailOSErr(ResError);

	EachVirtualShapeDo(WriteShape);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TShapeDocument.EachShapeDo(PROCEDURE DoThis(shape: TShape));
{ Iterates through the list of shapes. We have a separate method for this
  to hide the actual implementation of the shape list structure. }

	BEGIN
	fShapeList.Each(DoThis);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TShapeDocument.EachPotentialShapeDo(PROCEDURE
											  DoThis(shape: TShape));
{ Iterates through all the shapes in document plus any 'pastee' shapes
  which may have been added by a not-yet-committed Paste.}

	BEGIN
	EachShapeDo(DoThis);
	IF fReplaceCommand <> NIL THEN
		fReplaceCommand.EachNewShapeDo(DoThis);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TShapeDocument.EachVirtualShapeDo(PROCEDURE
											DoThis(shape: TShape));
{ EachVirtualShape iterates through only those shapes that appear
  to be present at the moment to the USER, given the
  UNDO/REDO status of the last command. Thus it iterates
  through some but possibly not all of the shapes in the
  document, and possibly also through not-yet-in-the-document pastees }

{--------------------------------------------------------------------------------------------------}

	PROCEDURE MaybeDoThis(shape: TShape);

		BEGIN
		IF (NOT fFiltering) | (NOT shape.fWasSelected) THEN
			DoThis(shape);
		END;

	BEGIN
	EachShapeDo(MaybeDoThis);
	IF fReplaceCommand <> NIL THEN
		fReplaceCommand.EachNewShapeDo(DoThis);
	END;

{--------------------------------------------------------------------------------------------------}
{$S AFields}

PROCEDURE TShapeDocument.Fields(obj: TObject); OVERRIDE;

	BEGIN
	WITH obj DO
		BEGIN
		DoToField(AtStr('TShapeDocument'), NIL, bClass);
		DoToField(AtStr('fShapeView'), @fShapeView, bObject);
		DoToField(AtStr('fToolsPalette'), @fToolsPalette, bObject);
		DoToField(AtStr('fShapeList'), @fShapeList, bObject);
		DoToField(AtStr('fDocState.theNumberOfShapes'), @fDocState.theNumberOfShapes, bInteger);
		DoToField(AtStr('fDocState.theWindowLocation'), @fDocState.theLocation, bRect);
		DoToField(AtStr('fDocState.theWindowSize'), @fDocState.theSize, bRect);
		DoToField(AtStr('fDocState.theScrollPosition'), @fDocState.theScrollPosition, bPoint);
		DoToField(AtStr('fReopening'), @fReopening, bBoolean);
		DoToField(AtStr('fReplaceCommand'), @fReplaceCommand, bObject);
		DoToField(AtStr('fFiltering'), @fFiltering, bBoolean);
		END;
	INHERITED Fields(obj);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

FUNCTION TShapeDocument.FirstSelectedShapeThat(FUNCTION
											   TestSelectedShape(aShape: TShape): BOOLEAN): TShape;

	VAR
		aShape: 			TShape;

	FUNCTION TestShape(aShape: TObject): BOOLEAN;

		BEGIN
		IF TShape(aShape).fIsSelected THEN
			TestShape := TestSelectedShape(TShape(aShape))
		ELSE
			TestShape := FALSE;
		END;

	BEGIN
	aShape := TShape(fShapeList.FirstThat(TestShape));
	IF (aShape = NIL) & (fReplaceCommand <> NIL) THEN
		aShape := fReplaceCommand.FirstShapeThat(TestSelectedShape);
	FirstSelectedShapeThat := aShape;
	END;

{--------------------------------------------------------------------------------------------------}
{$S AClose}

PROCEDURE TShapeDocument.Free; OVERRIDE;

	BEGIN
	FreeData;
	fShapeList := TList(FreeIfObject(fShapeList));

	INHERITED Free;
	END;

{--------------------------------------------------------------------------------------------------}
{$S AClose}

PROCEDURE TShapeDocument.FreeData;

	PROCEDURE DoToShape(aShape: TShape);

		BEGIN
		aShape := TShape(FreeIfObject(aShape));
		END;

	BEGIN
	IF fShapeList <> NIL THEN
		BEGIN
		EachShapeDo(DoToShape);
		fShapeList.DeleteAll;
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TShapeDocument.SurveyShapes(selecteesOnly: BOOLEAN; VAR numberOfShapes: INTEGER;
									  VAR combinedExtent: Rect);

	PROCEDURE UnionizeShapes(shape: TShape);

		BEGIN
		IF shape.fIsSelected | NOT selecteesOnly THEN
			BEGIN
			numberOfShapes := numberOfShapes + 1;
			IF numberOfShapes > 1 THEN
{$Push} {$h-}
				UnionRect(shape.fExtentRect, combinedExtent, combinedExtent)
{$Pop}

			ELSE
				combinedExtent := shape.fExtentRect;
			END;
		END;

	BEGIN
	numberOfShapes := 0;
	combinedExtent := gZeroRect;
	EachVirtualShapeDo(UnionizeShapes);
	END;

{--------------------------------------------------------------------------------------------------}
{$S AOpen}

PROCEDURE TToolsPalette.IToolsPalette(itsDocument: TDocument; itsSuperview: TView);

	VAR
		itsSize:			VPoint;

	BEGIN
	SetVPt(itsSize, kToolsPaletteWidth, kToolsPaletteHeight);
	IView(itsDocument, itsSuperview, gZeroVPt, itsSize, sizeFixed, sizeFixed);

	fCurrTool := 0;
	fOldTool := 0;
	fSelectedTool := 0;
	fCurrSelection := kNoToolSelection;

	BuildChoiceArray;
	END;

{--------------------------------------------------------------------------------------------------}
{$IFC qTemplateViews}
{$S AOpen}

PROCEDURE TToolsPalette.IRes(itsDocument: TDocument; itsSuperview: TView;
	VAR itsParams: Ptr); OVERRIDE;

	BEGIN
	INHERITED IRes(itsDocument, itsSuperview, itsParams);

	fCurrTool := 0;
	fOldTool := 0;
	fSelectedTool := 0;
	fCurrSelection := kNoToolSelection;

	BuildChoiceArray;
	END;

{$ENDC}

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TToolsPalette.Activate(entering: BOOLEAN); OVERRIDE;

	BEGIN
	IF NOT entering THEN
		fCurrSelection := kNoToolSelection;

	INHERITED Activate(entering);
	END;

{--------------------------------------------------------------------------------------------------}
{$S AOpen}

PROCEDURE TToolsPalette.BuildChoiceArray;

	VAR
		top:				INTEGER;
		i:					INTEGER;
		r:					Rect;

	BEGIN
	top := 0;
	FOR i := 0 TO kShapesInPalette DO					{Define the palette choices}
		BEGIN
		SetRect(r, 0, top, kToolsPaletteWidth - 1, top + kToolsPaletteWidth - 1);
		fChoiceArray[i] := r;
		top := top + kToolsPaletteWidth - 1;
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TToolsPalette.DoHighlightSelection(fromHL, toHL: HLState);

	PROCEDURE Toggle;

		VAR
			r:					Rect;

		BEGIN
		r := fChoiceArray[fCurrSelection];
		UseSelectionColor;
		InvertRect(r);
		END;

	BEGIN
	IF (fCurrTool >= 0) & (fCurrTool <= kShapesInPalette) THEN
		CASE toHL OF
			hlOn:
				IF fCurrSelection <> fCurrTool THEN 	{ if it's not already selected }
					BEGIN
					{ if another tool is selected, turn it off }
					IF fCurrSelection <> kNoToolSelection THEN
						Toggle;
					fCurrSelection := fCurrTool;
					Toggle;
					END;
			hlOff:
				IF fCurrSelection = fCurrTool THEN		{ if it's already selected, deselect it }
					BEGIN
					Toggle;
					fCurrSelection := kNoToolSelection;
					END;
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ASelCommand}

PROCEDURE TToolsPalette.DoMouseCommand(VAR theMouse: VPoint; event: TToolboxEvent;
									   hysteresis: Point);

	VAR
		aWindow:			TWindow;
		aToolSelectCmd: 	TToolSelectCmd;
		documentToolsPalette: TToolsPalette;
		menuToolsPalette:	TToolsPalette;
		floatingToolsPalette: TToolsPalette;

	BEGIN
	New(aToolSelectCmd);

	documentToolsPalette := NIL;
	aWindow := gApplication.GetActiveWindow;
	IF (aWindow <> NIL) & aWindow.IsShown THEN
		documentToolsPalette := TToolsPalette(aWindow.FindSubView('PLTT'));

	menuToolsPalette := NIL;
	IF (gToolsMenu <> NIL) THEN
		menuToolsPalette := TToolsPalette(gToolsMenu.FindSubView('MTUL'));

	floatingToolsPalette := NIL;
	IF (gToolsPaletteWindow <> NIL) & gToolsPaletteWindow.IsShown THEN
		floatingToolsPalette := TToolsPalette(gToolsPaletteWindow.FindSubView('TPLT'));

	aToolSelectCmd.IToolSelectCmd(SELF, documentToolsPalette, menuToolsPalette,
								  floatingToolsPalette, theMouse);

	{ the tracker will get tracked immediately if this is the tools view in the toolsmenu }
	PostCommand(aToolSelectCmd);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ASelCommand}

PROCEDURE TToolsPalette.TrackFeedback(aTrackPhase: TrackPhase;
											  anchorPoint: VPoint;
											  previousPoint: VPoint;
											  nextPoint: VPoint;
											  mouseDidMove: BOOLEAN;
											  turnItOn: BOOLEAN); OVERRIDE;

	VAR
		r:					Rect;
		qdPt:				Point;
		i:					INTEGER;

	PROCEDURE FrameTool(toolRect: Rect);

		BEGIN
		PenNormal;
		PenMode(patXOr);
		PenSize(2, 2);
		FrameRect(r);
		END;

	BEGIN
	IF mouseDidMove THEN
		BEGIN
		IF Focus THEN;
		qdPt := ViewToQDPt(nextPoint);
		FOR i := 0 TO kShapesInPalette DO
			BEGIN
			r := fChoiceArray[i];
			IF PtInRect(qdPt, r) THEN
				BEGIN
				FrameTool(r);
				IF turnItOn THEN
					fSelectedTool := i					{ remember the Tool currently selected }
				ELSE
					fSelectedTool := 0; 				{ no Tool currently selected }
				LEAVE;
				END;
			END;
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TToolsPalette.DoSetCursor(localPoint: VPoint; cursorRgn: RgnHandle); OVERRIDE;

	VAR
		qdExtent:			Rect;

	BEGIN
	SetCursor(arrow);
	GetQDExtent(qdExtent);
	RectRgn(cursorRgn, qdExtent);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TToolsPalette.Draw(area: VRect);

	VAR
		i:					INTEGER;
		r:					Rect;

	BEGIN
	PenSize(1, 1);
	MoveTo(fSize.h - 1, 0);
	Line(0, fSize.v);
	FOR i := 0 TO kShapesInPalette DO
		BEGIN
		r := fChoiceArray[i];
		FrameRect(r);

		IF i = 0 THEN
			BEGIN
			SetRect(r, 14, 12, 30, 28);
			CopyBits(gArwBitMap, thePort^.portBits, gArwBitMap.bounds, r, srcOR, NIL);
			END
		ELSE
			gShapesArray[i].Draw;
		END;

	INHERITED Draw(area);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TToolsPalette.SelectNewTool(whichTool: INTEGER);

	BEGIN
	IF IsShown THEN
		BEGIN
		IF Focus THEN;
		DoHighlightSelection(hlOn, hlOff);
		fCurrTool := whichTool;
		DoHighlightSelection(hlOff, hlOn);
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S AFields}

PROCEDURE TToolsPalette.Fields(obj: TObject); OVERRIDE;

	BEGIN
	WITH obj DO
		BEGIN
		DoToField(AtStr('TToolsPalette'), NIL, bClass);
		DoToField(AtStr('fCurrTool'), @fCurrTool, bInteger);
		END;
	INHERITED Fields(obj);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ASelCommand}

PROCEDURE TToolSelectCmd.IToolSelectCmd(theToolsPalette, theDocumentToolsPalette,
										theMenuToolsPalette, theFloatingToolsPalette: TToolsPalette;
										theMouse: VPoint);

	BEGIN
	ITracker(cChangeTool, NIL, theToolsPalette, NIL, theMouse);
	fCausesChange := FALSE;
	fTool := 0; 										{ the tools that the user selects with this
														 cmd }
	fExitTracking := FALSE;
	fMenuToolsPalette := theMenuToolsPalette;
	fDocumentToolsPalette := theDocumentToolsPalette;
	fFloatingToolsPalette := theFloatingToolsPalette;
	fViewConstrain := FALSE;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

FUNCTION TToolSelectCmd.TrackMouse(aTrackPhase: TrackPhase; VAR anchorPoint, previousPoint,
								   nextPoint: VPoint; mouseDidMove: BOOLEAN): TTracker; OVERRIDE;

	BEGIN
	TrackMouse := SELF;
	IF gTrackingInMenu THEN
		BEGIN
		{ quit tracking when the mouse leaves the view
		  - if gTrackingInMenu is TRUE, view constrain is set to FALSE, so nextPoint CAN leave the view
		  - all other times, view constrain is TRUE, so nextPoint WON'T leave the view }
		IF (fView <> NIL) & (fView.IsShown) THEN
			fExitTracking := NOT fView.ContainsMouse(nextPoint);

		{ if gTrackingInMenu is TRUE, then fExitTracking might get set to TRUE above meaning that the user
		is either returning to the menubar or trying to tear off this menu, so simply return NIL when
		we're done tracking and the tracker will be freed by TApplication.TrackMouse }
		IF (aTrackPhase = trackRelease) & fExitTracking THEN
			TrackMouse := NIL;
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TToolSelectCmd.TrackFeedback(aTrackPhase: TrackPhase;
											  anchorPoint: VPoint;
											  previousPoint: VPoint;
											  nextPoint: VPoint;
											  mouseDidMove: BOOLEAN;
											  turnItOn: BOOLEAN);

	BEGIN
	{ all we really want to do here is let the view give us some feedback, so...}
	IF (fView <> NIL) & (fView.IsShown) & mouseDidMove THEN
		fView.TrackFeedback(aTrackPhase, anchorPoint, previousPoint, nextPoint, mouseDidMove, turnItOn);

	{ When we're selecting a pattern (ie turning it on) then this command needs to know
	about the new selection. When we're deselecting a pattern (ie turning it off) then this command
	MUST ignore the deselection since MacApp's tracking will turn off the current selection as part
	of its default tracking behavior even when the user has made a valid selection in the view. }
	IF turnItOn THEN
		fTool := TToolsPalette(fView).fSelectedTool;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

FUNCTION TToolSelectCmd.IsDoneTracking: BOOLEAN; OVERRIDE;

	BEGIN
	IsDoneTracking := NOT StillDown | fExitTracking;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TToolSelectCmd.DoIt; OVERRIDE;

	BEGIN
	{ tell the TToolsPalette in the menu which tool is selected }
	IF (fMenuToolsPalette <> NIL) THEN
		BEGIN
		fMenuToolsPalette.fOldTool := fMenuToolsPalette.fCurrTool;
		fMenuToolsPalette.fCurrTool := fTool;
		END;

	{ tell the TToolsPalette in the floating window which tool is selected }
	IF (fFloatingToolsPalette <> NIL) THEN
		BEGIN
		fFloatingToolsPalette.fOldTool := fFloatingToolsPalette.fCurrTool;
		fFloatingToolsPalette.SelectNewTool(fTool);
		END;

	{ tell the TToolsPalette in the document window which tool is selected }
	IF (fDocumentToolsPalette <> NIL) THEN
		BEGIN
		fDocumentToolsPalette.fOldTool := fDocumentToolsPalette.fCurrTool;
		fDocumentToolsPalette.SelectNewTool(fTool);
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TToolSelectCmd.RedoIt; OVERRIDE;

	BEGIN
	{ tell the TToolsPalette in the menu which tool is selected }
	IF (fMenuToolsPalette <> NIL) THEN
		fMenuToolsPalette.fCurrTool := fTool;

	{ tell the TToolsPalette in the floating window which tool is selected }
	IF (fFloatingToolsPalette <> NIL) THEN
		fFloatingToolsPalette.SelectNewTool(fTool);

	{ tell the TToolsPalette in the document window which pattern is selected }
	IF (fDocumentToolsPalette <> NIL) THEN
		fDocumentToolsPalette.SelectNewTool(fTool);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TToolSelectCmd.UndoIt; OVERRIDE;

	BEGIN
	{ tell the TToolsPalette in the menu which tool *was* selected }
	IF (fMenuToolsPalette <> NIL) THEN
		fMenuToolsPalette.fCurrTool := fMenuToolsPalette.fOldTool;

	{ tell the TToolsPalette in the floating window which tool *was* selected }
	IF (fFloatingToolsPalette <> NIL) THEN
		fFloatingToolsPalette.SelectNewTool(fFloatingToolsPalette.fOldTool);

	{ tell the TToolsPalette in the document window which tool *was* selected }
	IF (fDocumentToolsPalette <> NIL) THEN
		fDocumentToolsPalette.SelectNewTool(fDocumentToolsPalette.fOldTool);
	END;

{--------------------------------------------------------------------------------------------------}
{$S AOpen}

PROCEDURE TShapeView.IShapeView(itsDocument: TShapeDocument; itsPalette: TToolsPalette;
								forClipboard: BOOLEAN);

	VAR
		itsLocation:		VPoint;
		itsSize:			VPoint;
		aHandler:			TStdPrintHandler;
		aDocState:			DocState;
		sd: 				SizeDeterminer;

	BEGIN
	fDragging := FALSE;
	fToolsPalette := itsPalette;
	SetVPt(itsSize, kMaxCoord, kMaxCoord);
	IF forClipboard THEN
		sd := sizeVariable
	ELSE
		sd := sizeFillPages;
	IView(itsDocument, NIL, gZeroVPt, itsSize, sd, sd);
	fScroller := NIL;

	fShapeDocument := itsDocument;

{$IFC FALSE}											{!!! Need to handle this}
	IF forClipboard THEN
		fWouldMakePICTScrap := TRUE;
{$ENDC}

	IF NOT forClipboard THEN
		BEGIN
		New(aHandler);
		aHandler.IStdPrintHandler(itsDocument, SELF, NOT kSquareDots, { does not have square dots }
								  kFixedSize,			{ horizontal page size is fixed }
								  kFixedSize);			{ vertical page size is fixed }
		END;

	fClickPt := gZeroPt;								{plausible starting value}
	END;

{--------------------------------------------------------------------------------------------------}
{$IFC qTemplateViews}
{$S AOpen}

PROCEDURE TShapeView.IRes(itsDocument: TDocument; itsSuperview: TView;
	VAR itsParams: Ptr); OVERRIDE;

	VAR
		aHandler:			TStdPrintHandler;

	BEGIN
	fDragging := FALSE;
	fScroller := NIL;
	fToolsPalette := NIL;

	INHERITED IRes(itsDocument, itsSuperview, itsParams);

	fShapeDocument := TShapeDocument(itsDocument);

	New(aHandler);
	aHandler.IStdPrintHandler(itsDocument, SELF, NOT kSquareDots, { does not have square dots }
							  kFixedSize,				{ horizontal page size is fixed }
							  kFixedSize);				{ vertical page size is fixed }

	fClickPt := gZeroPt;								{plausible starting value}
	
	SELF.AddAdorner(gSelectionAdorner, kViewAdornPriority, FALSE);
	END;

{$ENDC}

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TShapeView.CalcMinSize(VAR minSize: VPoint);

	VAR
		aRect:				Rect;
		numberOfShapes: 	INTEGER;

	BEGIN
	fShapeDocument.SurveyShapes(FALSE, numberOfShapes, aRect);
	SetVPt(minSize, Max(100, aRect.right), Max(100, aRect.bottom));
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

FUNCTION TShapeView.ContainsClipType(aType: ResType): BOOLEAN;

	BEGIN
	ContainsClipType := (aType = kShapeClipType);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TShapeView.Deselect;

	PROCEDURE DeselShape(shape: TShape);

		BEGIN
		shape.fIsSelected := FALSE;
		END;

	BEGIN
	DoHighlightSelection(hlOn, hlOff);
	fShapeDocument.EachPotentialShapeDo(DeselShape);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TShapeView.DoHighlightSelection(fromHL, toHL: HLState);

	PROCEDURE HiliteShape(shape: TShape);

		BEGIN
		IF shape.fIsSelected & (NOT fDragging) THEN
			shape.Highlight(fromHL, toHL);
		END;

	BEGIN
	fShapeDocument.EachVirtualShapeDo(HiliteShape);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ASelCommand}

PROCEDURE TShapeView.DoMenuCommand(aCmdNumber: CmdNumber);

	VAR
		recolorCmd: 		TRecolorCmd;
		reshadeCmd: 		TReshadeCmd;
		shapeCutCopyCommand: TShapeCutCopyCommand;
		shapePasteCommand:	TShapePasteCommand;
		shapeClearCommand:	TShapeClearCommand;
		menu, item: 		INTEGER;
		pMCEntry:			MCEntryPtr;
		theColor:			RGBColor;
		pickerPrompt:		StringHandle;

	PROCEDURE SelectIt(aShape: TShape);

		BEGIN
		IF NOT aShape.fIsSelected THEN
			BEGIN
			aShape.fIsSelected := TRUE;
			aShape.Highlight(hlOff, hlOn);
			END;
		END;

	FUNCTION GetShapeColor(aShape: TShape): BOOLEAN;

		BEGIN
		GetShapeColor := TRUE;
		END;

	BEGIN
	CASE aCmdNumber OF
		cCut, cCopy:
			BEGIN
			New(shapeCutCopyCommand);
			shapeCutCopyCommand.IShapeCutCopyCommand(aCmdNumber, SELF);
			PostCommand(shapeCutCopyCommand);
			END;

		cPaste:
			BEGIN
			New(shapePasteCommand);
			shapePasteCommand.IShapePasteCommand(SELF);
			PostCommand(shapePasteCommand);
			END;

		cClear:
			BEGIN
			New(shapeClearCommand);
			shapeClearCommand.IShapeClearCommand(SELF);
			PostCommand(shapeClearCommand);
			END;

{$IFC qDebug}
		cRecalcExtent:
			AdjustSize;
{$ENDC}

		cSelectAll:
			BEGIN
			IF Focus THEN;								{At least try to focus}
			fShapeDocument.EachVirtualShapeDo(SelectIt);
			END;

		cPickColor:
			BEGIN
			theColor := fShapeDocument.FirstSelectedShapeThat(GetShapeColor).fColor;
			pickerPrompt := GetString(kPickerPrompt);
			FailNil(pickerPrompt);
			IF GetColor(gZeroPt, pickerPrompt^^, theColor, theColor) THEN
				BEGIN
				New(recolorCmd);
				recolorCmd.IRecolorCmd(theColor, SELF);
				PostCommand(recolorCmd);
				END;
			END;

		cBetterFeedback:
			gBetterFeedback := NOT gBetterFeedback;

		OTHERWISE
			BEGIN
			CmdToMenuItem(aCmdNumber, menu, item);
			IF menu = mColor THEN
				BEGIN
				New(recolorCmd);
				pMCEntry := GetMCEntry(menu, item);
				theColor := pMCEntry^.mctRGB2;			{the MC entry can move}
				recolorCmd.IRecolorCmd(theColor, SELF);
				PostCommand(recolorCmd);
				END
			ELSE
				INHERITED DoMenuCommand(aCmdNumber);
			END;

	END;												{Case}
	END;

{--------------------------------------------------------------------------------------------------}
{$S ASelCommand}

PROCEDURE TShapeView.DoMouseCommand(VAR theMouse: VPoint; event: TToolboxEvent;
									hysteresis: Point);

	VAR
		qdPt:				Point;
		palette:			TToolsPalette;
		protoShape: 		TShape;
		shapeSketcher:		TShapeSketcher;
		shapeUnderMouse:	TShape;
		shapeSelector:		TShapeSelector;
		shapeDragger:		TShapeDragger;
		fi: 				FailInfo;

	PROCEDURE HdlInitCmdFailed(error: OSErr; message: LONGINT);

		BEGIN
		protoShape := TShape(FreeIfObject(protoShape));
		END;

	PROCEDURE CheckShape(aShape: TShape);

		BEGIN
{$Push} {$h-}
		IF PtInRect(qdPt, aShape.fExtentRect) THEN
			shapeUnderMouse := aShape;
{$Pop}
		END;

	BEGIN												{ DoMouseCommand }
	palette := fToolsPalette;
	qdPt := ViewToQDPt(theMouse);
	fClickPt := qdPt;
	IF palette.fCurrTool > 0 THEN						{draw mode}
		BEGIN
		FailSpaceIsLow; 								{ Make sure we aren't low on memory }

		Deselect;

		{Clone appropriate shape}

		protoShape := TShape(gShapesArray[palette.fCurrTool].Clone);

		CatchFailures(fi, HdlInitCmdFailed);
		{ Make sure cloning the shape left us with enough memory to continue.}
		FailSpaceIsLow;

		New(shapeSketcher);
		shapeSketcher.IShapeSketcher(SELF, protoShape, theMouse, event.fOptionKey);
		Success(fi);
		PostCommand(shapeSketcher);
		END 											{draw mode}
	ELSE
		BEGIN											{select mode}
		shapeUnderMouse := NIL;
		fShapeDocument.EachVirtualShapeDo(CheckShape);

		IF shapeUnderMouse = NIL THEN					{area select}
			BEGIN
			IF NOT event.fShiftKey THEN
				Deselect;
			New(shapeSelector);
			shapeSelector.IShapeSelector(cMouseCommand, SELF, theMouse);
			PostCommand(shapeSelector);
			END 										{area select}

		ELSE
			BEGIN										{shape select/move/...}

			IF NOT (shapeUnderMouse.fIsSelected | event.fShiftKey) THEN
				Deselect;

			IF event.fShiftKey THEN
				BEGIN
				shapeUnderMouse.fIsSelected := NOT shapeUnderMouse.fIsSelected;
				IF shapeUnderMouse.fIsSelected THEN
					shapeUnderMouse.Highlight(hlOff, hlOn)
				ELSE
					shapeUnderMouse.Highlight(hlOn, hlOff);
				END
			ELSE IF NOT shapeUnderMouse.fIsSelected THEN
				BEGIN
				shapeUnderMouse.fIsSelected := TRUE;
				DoHighlightSelection(hlOff, hlOn);
				END;

			IF shapeUnderMouse.fIsSelected THEN
				BEGIN
				New(shapeDragger);
				shapeDragger.IShapeDragger(SELF, theMouse);
				PostCommand(shapeDragger);
				END;
			END;										{shape select/move/...}
		END;											{Select mode}
	END;												{ DoMouseCommand }

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TShapeView.DoSetCursor(localPoint: VPoint; cursorRgn: RgnHandle); OVERRIDE;

	VAR
		cursorSet:			BOOLEAN;
		qdExtent:			Rect;
		shapeExtent:		Rect;
		qdPoint:			Point;

	PROCEDURE TestShape(shape: TShape);

		BEGIN
{$Push} {$h-}
		IF PtInRect(qdPoint, shape.fExtentRect) THEN
{$Pop}
			BEGIN
			UseROMMap(TRUE);
			SetCursor(GetCursor(plusCursor)^^);
			shapeExtent := shape.fExtentRect;			{RectRgn may move memory}
			RectRgn(cursorRgn, shapeExtent);
			cursorSet := TRUE;

 {can't exit from the middle of an Each, because TList
 doesn't allow it; should really use FirstWhich;
 no harm in skipping the Exit, just takes longer}
			{* Exit(DoSetCursor); *}
			END;
		END;

	BEGIN												{ DoSetCursor }
	qdPoint := ViewToQDPt(localPoint);
	IF fToolsPalette.fCurrTool = 0 THEN 				{selection}
		BEGIN
		cursorSet := FALSE;
		fShapeDocument.EachVirtualShapeDo(TestShape);
		IF NOT cursorSet THEN
			IF qNeedsColorQD | gConfiguration.hasColorQD THEN
			{set cursor to color arrow}
				SetCCursor(gRainbowArrow)
			ELSE
				SetCursor(arrow);
		END
	ELSE
		BEGIN
		UseROMMap(TRUE);
		SetCursor(GetCursor(crossCursor)^^);
		GetQDExtent(qdExtent);
		RectRgn(cursorRgn, qdExtent);
		END;
	END;												{ DoSetCursor }

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TShapeView.DoSetupMenus;

	VAR
		i:					INTEGER;
		anySelection:		BOOLEAN;
		anyShapes:			BOOLEAN;
		haveMemory: 		BOOLEAN;
		aMenuHandle:		MenuHandle;
		item:				INTEGER;
		itemName:			Str255;

	PROCEDURE TestShapes(theShape: TShape);

		BEGIN
		anySelection := anySelection | theShape.fIsSelected;
		anyShapes := anyShapes | (NOT fShapeDocument.fFiltering) | (NOT theShape.fWasSelected);
		END;

	BEGIN

	INHERITED DoSetupMenus;

	anySelection := FALSE;
	anyShapes := FALSE;

	haveMemory := NOT MemSpaceIsLow;
	{ Find out if we are low on memory.  If we are then we'll disable all
	  memory-intensive commands. }

	fShapeDocument.EachVirtualShapeDo(TestShapes);
	{ This checks every virtual shape--could be made faster. }

	{ enable the Pattern menu if there is a selection }
	Enable(cPatterns, anySelection);

	IF anySelection & (qNeedsColorQD | gConfiguration.hasColorQD) THEN
		BEGIN
		{ Enable each of the Color menu items, if the Color menu is present }
		aMenuHandle := GetMHandle(mColor);
		IF aMenuHandle <> NIL THEN
			FOR item := 1 TO CountMItems(aMenuHandle) DO
				BEGIN
 { There can be more than 31 menu entries with scrolling menus,
   but trying to enable an item with number > 31 is bad news.
   If the menu itself is enabled (which it will be in MacApp
   if any of the first 31 items is enabled), then the extras
   will always be enabled. }
				{Don't enable line separators.}
				GetItem(aMenuHandle, item, itemName);
				IF (item <= 31) & (itemName <> '-') THEN
					EnableItem(aMenuHandle, item);
				END;
		END;

{$IFC qDebug}
	Enable(cRecalcExtent, TRUE);
{$ENDC}
	Enable(cCut, anySelection & haveMemory);
	Enable(cCopy, anySelection & haveMemory);
	IF haveMemory THEN
		gClipboardMgr.CanPaste(kShapeClipType);
	Enable(cClear, anySelection);

	Enable(cSelectAll, anyShapes);
	EnableCheck(cBetterFeedback, TRUE, gBetterFeedback);
	END;												{ DoSetupMenus }

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TShapeView.Draw(area: VRect);

	VAR
		QDArea: 			Rect;

	PROCEDURE DrawShape(shape: TShape);

		VAR
			r:					Rect;

		BEGIN
{$Push} {$h-}
		IF NOT (fDragging & shape.fIsSelected) & SectRect(shape.fExtentRect, QDArea, r) THEN
			shape.Draw;
{$Pop}
		END;

	BEGIN
	ViewToQDRect(area, QDArea);
	fShapeDocument.EachVirtualShapeDo(DrawShape);		{draw the shapes}

	INHERITED Draw(area);
	END;

{--------------------------------------------------------------------------------------------------}
{$S AFields}

PROCEDURE TShapeView.Fields(obj: TObject); OVERRIDE;

	BEGIN
	WITH obj DO
		BEGIN
		DoToField(AtStr('TShapeView'), NIL, bClass);
		DoToField(AtStr('fDragging'), @fDragging, bBoolean);
		DoToField(AtStr('fToolsPalette'), @fToolsPalette, bObject);
		DoToField(AtStr('fClickPt'), @fClickPt, bPoint);
		DoToField(AtStr('fShapeDocument'), @fShapeDocument, bObject);
		DoToField(AtStr('fScroller'), @fScroller, bObject);
		END;
	INHERITED Fields(obj);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TShapeView.InvalShape(aShape: TShape);

	VAR
		r:					Rect;

	BEGIN
	r := aShape.fExtentRect;
	InsetRect(r, - 2, - 2);
	InvalidateRect(r);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TShapeView.RestoreSelection;

	PROCEDURE DoRestore(aShape: TShape);

		VAR
			wantInval:			BOOLEAN;

		BEGIN
		wantInval := aShape.fIsSelected | aShape.fWasSelected;
		aShape.fIsSelected := aShape.fWasSelected;
		IF wantInval THEN
			InvalShape(aShape);
		END;

	BEGIN
	IF Focus THEN;
	Deselect;
	fShapeDocument.EachPotentialShapeDo(DoRestore);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TShapeView.SaveSelection(andInval: BOOLEAN);

	PROCEDURE DoToShape(aShape: TShape);

		BEGIN
		WITH aShape DO
			fWasSelected := fIsSelected;
		IF andInval & aShape.fWasSelected THEN
			InvalShape(aShape);
		END;

	BEGIN
	IF Focus THEN;
	fShapeDocument.EachPotentialShapeDo(DoToShape);
	END;

{--------------------------------------------------------------------------------------------------}
{$S AClipboard}

PROCEDURE TShapeView.WriteToDeskScrap;

	VAR
		err:				LONGINT;
		clipShapes: 		ShapesOnClipboard;
		count:				INTEGER;
		bBox:				Rect;
		i:					INTEGER;
		outline:			Rect;
		aShapeDatum:		ShapeData;
		wantBytes:			INTEGER;

	PROCEDURE CopyToDeskScrap(aShape: TShape);

		BEGIN
		WITH aShapeDatum DO
			BEGIN
			theId := aShape.id;
			thePattern := aShape.fPattern;
			theColor := aShape.fColor;
			theRect := aShape.fExtentRect;
			END;
		clipShapes^^.theShapes[i] := aShapeDatum;
		i := i + 1;
		END;

	BEGIN
 { Write the PICT scrap first in case we will be unable to write both
   the PICT scrap and our own scrap type.}
	INHERITED WriteToDeskScrap; 						{Generate PICT-type scrap}

	fShapeDocument.SurveyShapes(FALSE, count, bBox);	{count shapes}
	wantBytes := sizeof(INTEGER) + sizeof(Rect) + (count * sizeof(ShapeData));
	{??? call CanAllocate, and if can't get enough, put up alert?}
	clipShapes := ShapesOnClipboard(NewPermHandle(wantBytes));

	WITH clipShapes^^ DO
		BEGIN
		theBoundingBox := bBox;
		theNumberOfShapes := count;
		END;
	i := 0;
	fShapeDocument.EachShapeDo(CopyToDeskScrap);
	err := gClipboardMgr.PutDeskScrapData(kShapeClipType, Handle(clipShapes));
	Handle(clipShapes) := DisposeIfHandle(clipShapes);

	FailOSErr(err);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TShape.Initialize; OVERRIDE;

	BEGIN
	INHERITED Initialize;

	fID := 0;
	fExtentRect := gZeroRect;
	fPattern := 1;
	fOldPattern := 1;
	fColor := gRGBWhite;
	fIsSelected := FALSE;
	fWasSelected := FALSE;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TShape.IShape(itsExtent: Rect; itsID: INTEGER);

	BEGIN
	IObject;
	fExtentRect := itsExtent;
	fID := itsID;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TShape.Draw;

	BEGIN
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TShape.DrawOutline;

	BEGIN
{$IFC qDebug}
	ProgramBreak('TShape.DrawOutline called!');
{$ENDC}
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TShape.EachHandleDo(PROCEDURE DoThis(Handle: Rect; handVHS: VHSelect;
											   handTopOrLeft: BOOLEAN));

	VAR
		r:					Rect;
		extent: 			Rect;

	BEGIN
	SetRect(r, - 2, - 2, 2, 2);
	extent := fExtentRect;

	WITH extent DO
		BEGIN
		OffSetRect(r, left, (top + bottom) DIV 2);
		DoThis(r, h, TRUE); 							{left}

		OffSetRect(r, right - left, 0);
		DoThis(r, h, FALSE);							{right}

		OffSetRect(r, - (right - left) DIV 2, top - ((top + bottom) DIV 2));
		DoThis(r, v, TRUE); 							{top}

		OffSetRect(r, 0, bottom - top);
		DoThis(r, v, FALSE);							{bottom}
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S AFields}

PROCEDURE TShape.Fields(obj: TObject); OVERRIDE;

	BEGIN
	WITH obj DO
		BEGIN
		DoToField(AtStr('TShape'), NIL, bClass);
		DoToField(AtStr('fID'), @fID, bInteger);
		DoToField(AtStr('fExtentRect'), @fExtentRect, bRect);
		DoToField(AtStr('fPattern'), @fPattern, bInteger);
		DoToField(AtStr('fOldPattern'), @fOldPattern, bInteger);
		DoToField(AtStr('fColor'), @fColor, bRGBColor);
		DoToField(AtStr('fOldColor'), @fOldColor, bRGBColor);
		DoToField(AtStr('fIsSelected'), @fIsSelected, bBoolean);
		DoToField(AtStr('fWasSelected'), @fWasSelected, bBoolean);
		END;
	INHERITED Fields(obj);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TShape.Highlight(fromHL, toHL: HLState);

	PROCEDURE PaintHandle(Handle: Rect; handVHS: VHSelect; handTopOrLeft: BOOLEAN);

		BEGIN
		PaintRect(Handle);
		END;

	BEGIN
	{!!!???### egregious hack by SRF.  Fix before shipment Just needed any view reference }
	gApplication.GetFrontWindow.SetHLPenState(fromHL, toHL);
	EachHandleDo(PaintHandle);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

FUNCTION TShape.id: INTEGER;

	BEGIN
	id := fID;
	END;

{--------------------------------------------------------------------------------------------------}
{$S AReadFile}

PROCEDURE TShape.ReadFromFile(aFile: TFile);

	VAR
		data:				ShapeData;
		count:				LONGINT;

	BEGIN
	{ DoRead has already read in the id; read the rest of the data }
	count := sizeof(ShapeData) - sizeof(INTEGER);
	FailOSErr(aFile.ReadData(@data.theRect, count));

	WITH data DO
		BEGIN
		fExtentRect := theRect;
		fPattern := thePattern;
		fColor := theColor;
		fIsSelected := theSelected;
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S AWriteFile}

PROCEDURE TShape.WriteToFile(aFile: TFile);

	VAR
		data:				ShapeData;
		count:				LONGINT;

	BEGIN
	WITH data DO
		BEGIN
		theId := id;
		theRect := fExtentRect;
		thePattern := fPattern;
		theColor := fColor;
		theSelected := fIsSelected;
		END;
	count := sizeof(ShapeData);
	FailOSErr(aFile.WriteData(@data, count));
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TBox.IBox(itsExtent: Rect; itsID: INTEGER);

	BEGIN
	IShape(itsExtent, itsID);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TBox.Draw;

	VAR
		itsExtent:			Rect;
		itsColor:			RGBColor;

	BEGIN
	PenNormal;
	itsExtent := fExtentRect;

	IF qNeedsColorQD | gConfiguration.hasColorQD THEN
		BEGIN
		{Get the color of the menu item representing the shape's color}
		itsColor := fColor;
		RGBForeColor(itsColor);
		FillRect(itsExtent, gPat[fPattern]);
		ForeColor(blackColor);
		END
	ELSE
		FillRect(itsExtent, gPat[fPattern]);

	DrawOutline;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TBox.DrawOutline;

	VAR
		itsExtent:			Rect;

	BEGIN
	PenSize(1, 1);
	itsExtent := fExtentRect;							{FrameRect may move memory.}
	FrameRect(itsExtent);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE THeavyBox.IHeavyBox(itsExtent: Rect; itsID: INTEGER);

	BEGIN
	IBox(itsExtent, itsID);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE THeavyBox.Draw;

	CONST
		s					= '4K';

	VAR
		itsExtent:			Rect;
		wid:				INTEGER;
		r:					Rect;
		x:					INTEGER;

	BEGIN
	INHERITED Draw;

	itsExtent := fExtentRect;

	PenSize(2, 2);
	FrameRect(itsExtent);
	PenSize(1, 1);

	TextFont(geneva);
	TextFace([]);
	TextSize(9);
	TextMode(srcOR);

	wid := StringWidth(s);

	WITH itsExtent DO
		x := (left + right) DIV 2;

	SetRect(r, x - 7, itsExtent.bottom - 14, x + 7, itsExtent.bottom - 4);
	EraseRect(r);
	MADrawString(AtStr(s), r, teCenter);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TCircle.ICircle(itsExtent: Rect; itsID: INTEGER);

	BEGIN
	IShape(itsExtent, itsID);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TCircle.Draw;

	VAR
		itsExtent:			Rect;
		itsColor:			RGBColor;

	BEGIN
	PenNormal;
	itsExtent := fExtentRect;							{FillOval may move memory}

	IF qNeedsColorQD | gConfiguration.hasColorQD THEN
		BEGIN
		{Get the color of the menu item representing the shape's color}
		itsColor := fColor;
		RGBForeColor(itsColor);
		FillOval(itsExtent, gPat[fPattern]);
		ForeColor(blackColor);
		END
	ELSE
		FillOval(itsExtent, gPat[fPattern]);

	DrawOutline;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TCircle.DrawOutline;

	VAR
		itsExtent:			Rect;

	BEGIN
	PenSize(1, 1);
	itsExtent := fExtentRect;							{FrameOval may move memory}
	FrameOval(itsExtent);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ASelCommand}

PROCEDURE TShapeCommand.IShapeCommand(itsCmdNumber: CmdNumber; itsShapeView: TShapeView;
									  theMouse: VPoint; betterFeedbackDesired: BOOLEAN);

	BEGIN
	IBetterFeedbackCmd(itsCmdNumber, itsShapeView.fShapeDocument, itsShapeView,
					   itsShapeView.fScroller, theMouse, betterFeedbackDesired);
	fShapeView := itsShapeView;
	fShapeDocument := itsShapeView.fShapeDocument;
	END;

{--------------------------------------------------------------------------------------------------}
{$S AFields}

PROCEDURE TShapeCommand.Fields(obj: TObject); OVERRIDE;

	BEGIN
	WITH obj DO
		BEGIN
		DoToField(AtStr('TShapeCommand'), NIL, bClass);
		DoToField(AtStr('fShapeView'), @fShapeView, bObject);
		DoToField(AtStr('fShapeDocument'), @fShapeDocument, bObject);
		END;
	INHERITED Fields(obj);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ASelCommand}

PROCEDURE TShapeSelector.IShapeSelector(itsCmdNumber: CmdNumber; itsShapeView: TShapeView;
										theMouse: VPoint);

	BEGIN
	IShapeCommand(itsCmdNumber, itsShapeView, theMouse, gBetterFeedback);
	fCausesChange := FALSE;
	fCanUndo := FALSE;
	fBounds := gZeroRect;

	{$Push}{$H-}
	
	StuffHex(@fAnts, '7C3E1F8FC7E3F1F8');
	{$Pop}
	fLastMarch := 0;
	fFeedBackIsOn := FALSE;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TShapeSelector.DoIt; OVERRIDE;

	VAR
		shapeView:			TShapeView;

	PROCEDURE TestShape(shape: TShape);

		BEGIN
{$Push} {$h-}
		IF RectsNest(fBounds, shape.fExtentRect) THEN
{$Pop}
			BEGIN
			IF shape.fIsSelected THEN
				shape.Highlight(hlOn, hlOff)
			ELSE
				shape.Highlight(hlOff, hlOn);
			shape.fIsSelected := NOT shape.fIsSelected;
			END;
		END;

	BEGIN
	shapeView := fShapeView;
	shapeView.fShapeDocument.EachVirtualShapeDo(TestShape);
	END;

{--------------------------------------------------------------------------------------------------}
{$S AFields}

PROCEDURE TShapeSelector.Fields(obj: TObject); OVERRIDE;

	BEGIN
	WITH obj DO
		BEGIN
		DoToField(AtStr('TShapeSelector'), NIL, bClass);
		DoToField(AtStr('fBounds'), @fBounds, bRect);
		DoToField(AtStr('fShiftKey'), @fShiftKey, bBoolean);
		END;
	INHERITED Fields(obj);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TShapeSelector.TrackFeedback(aTrackPhase: TrackPhase;
											  anchorPoint: VPoint;
											  previousPoint: VPoint;
											  nextPoint: VPoint;
											  mouseDidMove: BOOLEAN;
											  turnItOn: BOOLEAN);

	VAR
		p:					Pattern;
		r:					Rect;

	BEGIN
	INHERITED TrackFeedback(aTrackPhase, anchorPoint, previousPoint, nextPoint, mouseDidMove, turnItOn);
	IF (fView <> NIL) & (fFeedBackIsOn <> turnItOn) & (mouseDidMove | (TickCount - fLastMarch >= 4)) THEN
		BEGIN
		Pt2Rect(fView.ViewToQDPt(anchorPoint), fView.ViewToQDPt(nextPoint), r);
		p := fAnts;
		if turnItOn THEN
			begin
			RotatePat(p);
			fAnts := p;
			fLastMarch := TickCount;
			END;

		PenPat(p);
		FrameRect(r);									{ draw/erase }
		fFeedBackIsOn := turnItOn;
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

FUNCTION TShapeSelector.TrackMouse(aTrackPhase: TrackPhase; VAR anchorPoint, previousPoint,
								   nextPoint: VPoint; mouseDidMove: BOOLEAN): TTracker;

	VAR
		r:					Rect;
		qdAnchor, qdPrevious: Point;

	BEGIN
	TrackMouse := INHERITED TrackMouse(aTrackPhase, anchorPoint, previousPoint, nextPoint,
									   mouseDidMove);
	qdAnchor := fShapeView.ViewToQDPt(anchorPoint);
	qdPrevious := fShapeView.ViewToQDPt(previousPoint);
	Pt2Rect(qdAnchor, qdPrevious, r);
	fBounds := r;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ASelCommand}

PROCEDURE TShapeDragger.IShapeDragger(aShapeView: TShapeView; theMouse: VPoint);

	VAR
		bounds: 			Rect;
		numberOfShapes: 	INTEGER;

	BEGIN
	IShapeCommand(cMoveShape, aShapeView, theMouse, gBetterFeedback);

	aShapeView.fShapeDocument.SurveyShapes(TRUE, numberOfShapes, bounds);
	aShapeView.fDragging := FALSE;
	fBounds := bounds;

	fConstrainsMouse := gConstrainDrags;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TShapeDragger.DoIt;

	BEGIN
	MoveBy(fDeltaH, fDeltaV);
	END;

{--------------------------------------------------------------------------------------------------}
{$S AFields}

PROCEDURE TShapeDragger.Fields(obj: TObject); OVERRIDE;

	BEGIN
	WITH obj DO
		BEGIN
		DoToField(AtStr('TShapeDragger'), NIL, bClass);
		DoToField(AtStr('fBounds'), @fBounds, bRect);
		DoToField(AtStr('fDeltaH'), @fDeltaH, bInteger);
		DoToField(AtStr('fDeltaV'), @fDeltaV, bInteger);
		END;
	INHERITED Fields(obj);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TShapeDragger.MoveBy(deltaH, deltaV: INTEGER);

	PROCEDURE MoveShape(shape: TShape);

		BEGIN
		IF shape.fIsSelected THEN
			BEGIN
{$Push} {$h-}
			OffSetRect(shape.fExtentRect, deltaH, deltaV);
{$Pop}
			fShapeView.InvalShape(shape);
			END;
		shape.fWasSelected := shape.fIsSelected;
		END;

	BEGIN
	IF fShapeView.Focus THEN;
	fShapeDocument.EachShapeDo(MoveShape);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TShapeDragger.RedoIt;

	BEGIN
	fShapeView.RestoreSelection;

	MoveBy(fDeltaH, fDeltaV);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TShapeDragger.TrackConstrain(aTrackPhase: TrackPhase;
											  anchorPoint: VPoint;
											  previousPoint: VPoint;
											  VAR nextPoint: VPoint;
											  mouseDidMove: BOOLEAN); OVERRIDE;

	VAR
		vhs:				VHSelect;
		temp:				INTEGER;

	BEGIN
	FOR vhs := v TO h DO
		BEGIN
		temp := anchorPoint.vh[vhs] - fBounds.topLeft.vh[vhs];
		nextPoint.vh[vhs] := Max(temp, nextPoint.vh[vhs]);

		temp := fShapeView.fSize.vh[vhs] - (fBounds.botRight.vh[vhs] - anchorPoint.vh[vhs]);
		nextPoint.vh[vhs] := Min(temp, nextPoint.vh[vhs]);
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TShapeDragger.TrackFeedback(aTrackPhase: TrackPhase;
											  anchorPoint: VPoint;
											  previousPoint: VPoint;
											  nextPoint: VPoint;
											  mouseDidMove: BOOLEAN;
											  turnItOn: BOOLEAN);

	VAR
		aRect:				Rect;
		delta:				Point;

	BEGIN
	INHERITED TrackFeedback(aTrackPhase, anchorPoint, previousPoint, nextPoint, mouseDidMove, turnItOn);
	IF mouseDidMove & fShapeView.fDragging THEN
		BEGIN
		delta.h := nextPoint.h - anchorPoint.h;
		delta.v := nextPoint.v - anchorPoint.v;

		aRect := fBounds;
		OffSetRect(aRect, delta.h, delta.v);

		FrameRect(aRect);								{ draw/erase it }
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

FUNCTION TShapeDragger.TrackMouse(aTrackPhase: TrackPhase; VAR anchorPoint, previousPoint,
								  nextPoint: VPoint; mouseDidMove: BOOLEAN): TTracker;

	PROCEDURE EraseShape(shape: TShape);

		BEGIN
		IF shape.fIsSelected THEN
			fShapeView.InvalShape(shape);
		END;

	BEGIN
	TrackMouse := INHERITED TrackMouse(aTrackPhase, anchorPoint, previousPoint, nextPoint,
									   mouseDidMove);

	IF aTrackPhase = trackRelease THEN					{set up for moving the shape(s)}
		BEGIN
		IF fShapeView.fDragging THEN					{actually did move}
			BEGIN
			fDeltaH := previousPoint.h - anchorPoint.h;
			fDeltaV := previousPoint.v - anchorPoint.v;

			fShapeView.fDragging := FALSE;
			END
		ELSE
			TrackMouse := NIL;
		END

	ELSE IF aTrackPhase = trackMove THEN
		IF mouseDidMove THEN
			IF NOT fShapeView.fDragging THEN			{this is first move}
				BEGIN
				fShapeView.DoHighlightSelection(hlOn, hlOff);
				fShapeDocument.EachVirtualShapeDo(EraseShape);
				fShapeView.fDragging := TRUE;
				fShapeView.GetWindow.Update;
				IF fShapeView.Focus THEN;				{UpdateEvent changes the focus - restore it}
				END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TShapeDragger.UndoIt;

	BEGIN
	fShapeView.RestoreSelection;

	MoveBy( - fDeltaH, - fDeltaV);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ASelCommand}

PROCEDURE TReshadeCmd.IReshadeCmd(itsCmdNumber: INTEGER; itsShapeView: TShapeView;
								  thePatternsPalette, menuPatternsPalette,
								  floatingPatternsPalette: TPatternsPalette; theMouse: VPoint);

	{	trackingPatternsPalette is the view we're tracking in
		floatingPatternsPalette is the view that's in the floating window }

	BEGIN
	IF itsShapeView <> NIL THEN
		ITracker(cChangeShade, itsShapeView.fDocument, thePatternsPalette, NIL, theMouse)
	ELSE
		ITracker(cChangeShade, NIL, thePatternsPalette, NIL, theMouse);
	fShapeView := itsShapeView;
	fShapeDocument := itsShapeView.fShapeDocument;
	fMenuPatternsPalette := menuPatternsPalette;
	fFloatingPatternsPalette := floatingPatternsPalette;
	fPattern := itsCmdNumber - cPatterns;
	fExitTracking := FALSE;
	fViewConstrain := FALSE;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

FUNCTION TReshadeCmd.TrackMouse(aTrackPhase: TrackPhase; VAR anchorPoint, previousPoint,
								nextPoint: VPoint; mouseDidMove: BOOLEAN): TTracker; OVERRIDE;

	BEGIN
	TrackMouse := SELF;
	IF gTrackingInMenu THEN
		BEGIN
		{ quit tracking when the mouse leaves the view
		  - if gTrackingInMenu is TRUE, view constrain is set to FALSE, so nextPoint CAN leave the view
		  - all other times, view constrain is TRUE, so nextPoint WON'T leave the view }
		IF (fView <> NIL) & (fView.IsShown) THEN
			fExitTracking := NOT fView.ContainsMouse(nextPoint);

		{ if gTrackingInMenu is TRUE, then fExitTracking might get set to TRUE above meaning that the user
		is either returning to the menubar or trying to tear off this menu, so simply return NIL when
		we're done tracking and the tracker will be freed by TApplication.TrackMouse }
		IF (aTrackPhase = trackRelease) & fExitTracking THEN
			TrackMouse := NIL;
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TReshadeCmd.TrackFeedback(aTrackPhase: TrackPhase;
											  anchorPoint: VPoint;
											  previousPoint: VPoint;
											  nextPoint: VPoint;
											  mouseDidMove: BOOLEAN;
											  turnItOn: BOOLEAN);

	BEGIN
	{ all we really want to do here is let the view give us some feedback, so...}
	IF (fView <> NIL) & (fView.IsShown) & mouseDidMove THEN
		fView.TrackFeedback(aTrackPhase, anchorPoint, previousPoint, nextPoint, mouseDidMove, turnItOn);

	{ When we're selecting a pattern (ie turning it on) then this command needs to know
	about the new selection. When we're deselecting a pattern (ie turning it off) then this command
	MUST ignore the deselection since MacApp's tracking will turn off the current selection as part
	of its default tracking behavior even when the user has made a valid selection in the view. }
	IF turnItOn THEN
		fPattern := TPatternsPalette(fView).fSelectedPattern;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

FUNCTION TReshadeCmd.IsDoneTracking: BOOLEAN; OVERRIDE;

	BEGIN
	IsDoneTracking := NOT StillDown | fExitTracking;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TReshadeCmd.DoIt;

	PROCEDURE ReshadeShape(shape: TShape);

		BEGIN
		IF shape.fIsSelected THEN
			BEGIN
			shape.fOldPattern := shape.fPattern;
			shape.fPattern := fPattern;
			fShapeView.InvalShape(shape);
			END;
		shape.fWasSelected := shape.fIsSelected;
		END;

	BEGIN
	{ tell the patterns view in the menu which pattern is selected }
	IF (fMenuPatternsPalette <> NIL) THEN
		BEGIN
		fMenuPatternsPalette.fOldPattern := fMenuPatternsPalette.fCurrPattern;
		fMenuPatternsPalette.fCurrPattern := fPattern;
		END;

	{ tell the floating window which pattern is selected }
	IF (fFloatingPatternsPalette <> NIL) THEN
		BEGIN
		fFloatingPatternsPalette.fOldPattern := fFloatingPatternsPalette.fCurrPattern;
		fFloatingPatternsPalette.SelectNewPattern(fPattern);
		END;

	IF (fShapeDocument <> NIL) THEN
		BEGIN
		IF (fShapeView <> NIL) & fShapeView.Focus THEN;
		fShapeDocument.EachShapeDo(ReshadeShape);
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TReshadeCmd.RedoIt;

	PROCEDURE ReshadeShape(shape: TShape);

		BEGIN
		IF shape.fWasSelected THEN
			BEGIN
			shape.fOldPattern := shape.fPattern;
			shape.fPattern := fPattern;
			fShapeView.InvalShape(shape);
			END;
		shape.fIsSelected := shape.fWasSelected;
		END;

	BEGIN
	{ tell the floating window which pattern is selected }
	IF (fFloatingPatternsPalette <> NIL) THEN
		fFloatingPatternsPalette.SelectNewPattern(fPattern);

	IF fShapeView <> NIL THEN
		BEGIN
		IF fShapeView.Focus THEN;
		fShapeView.Deselect;
		fShapeDocument.EachShapeDo(ReshadeShape);
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TReshadeCmd.UndoIt;

	PROCEDURE ReshadeShape(shape: TShape);

		BEGIN
		IF shape.fWasSelected THEN
			BEGIN
			shape.fPattern := shape.fOldPattern;
			fShapeView.InvalShape(shape);
			END;
		shape.fIsSelected := shape.fWasSelected;
		END;

	BEGIN
	{ tell the floating window which pattern *was* selected }
	IF (fFloatingPatternsPalette <> NIL) THEN
		fFloatingPatternsPalette.SelectNewPattern(fFloatingPatternsPalette.fOldPattern);

	IF fShapeView <> NIL THEN
		BEGIN
		IF fShapeView.Focus THEN;
		fShapeView.Deselect;
		fShapeDocument.EachShapeDo(ReshadeShape);
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ASelCommand}

PROCEDURE TRecolorCmd.IRecolorCmd(itsColor: RGBColor; itsShapeView: TShapeView);

	BEGIN
	IShapeCommand(cChangeColor, itsShapeView, gZeroVPt, NOT kBetterFeedbackDesired);
	fColor := itsColor;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TRecolorCmd.DoIt;

	PROCEDURE RecolorShape(shape: TShape);

		BEGIN
		IF shape.fIsSelected THEN
			BEGIN
			shape.fOldColor := shape.fColor;
			shape.fColor := fColor;
			fShapeView.InvalShape(shape);
			END;
		shape.fWasSelected := shape.fIsSelected;
		END;

	BEGIN
	IF fShapeView.Focus THEN;
	fShapeDocument.EachShapeDo(RecolorShape);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TRecolorCmd.RedoIt;

	PROCEDURE RecolorShape(shape: TShape);

		BEGIN
		IF shape.fWasSelected THEN
			BEGIN
			shape.fOldColor := shape.fColor;
			shape.fColor := fColor;
			fShapeView.InvalShape(shape);
			END;
		shape.fIsSelected := shape.fWasSelected;
		END;

	BEGIN
	IF fShapeView.Focus THEN;
	fShapeView.Deselect;
	fShapeDocument.EachShapeDo(RecolorShape);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TRecolorCmd.UndoIt;

	PROCEDURE RecolorShape(shape: TShape);

		BEGIN
		IF shape.fWasSelected THEN
			BEGIN
			shape.fColor := shape.fOldColor;
			fShapeView.InvalShape(shape);
			END;
		shape.fIsSelected := shape.fWasSelected;
		END;

	BEGIN
	IF fShapeView.Focus THEN;
	fShapeView.Deselect;
	fShapeDocument.EachShapeDo(RecolorShape);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TShapeReplaceCommand.Commit; OVERRIDE;

	PROCEDURE HandleIt(aShape: TShape);

		BEGIN
		IF aShape.fWasSelected THEN
			fShapeDocument.DeleteShape(aShape);
		END;

	BEGIN
	IF fShapeDocument.fFiltering THEN
		fShapeDocument.EachShapeDo(HandleIt);
	fShapeDocument.fFiltering := FALSE;
	fShapeDocument.fReplaceCommand := NIL;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TShapeReplaceCommand.EachNewShapeDo(PROCEDURE
											  DoThis(shape: TShape));

	BEGIN
	END;

{--------------------------------------------------------------------------------------------------}
{$ARes}

FUNCTION TShapeReplaceCommand.FirstShapeThat(FUNCTION
											 TestShape(aShape: TShape): BOOLEAN): TShape;

	BEGIN
	FirstShapeThat := NIL;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TShapeReplaceCommand.RedoIt; OVERRIDE;

	BEGIN
	IF fShapeDocument.fFiltering THEN
		fShapeView.RestoreSelection;
	fShapeDocument.fReplaceCommand := SELF;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TShapeReplaceCommand.UndoIt; OVERRIDE;

	BEGIN
	fShapeView.RestoreSelection;
	fShapeDocument.fReplaceCommand := NIL;
	fShapeDocument.fFiltering := FALSE;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ASelCommand}

PROCEDURE TShapeSketcher.IShapeSketcher(aShapeView: TShapeView; protoShape: TShape;
										theMouse: VPoint; constrain: BOOLEAN);

	BEGIN
	IShapeCommand(cNewShape, aShapeView, theMouse, gBetterFeedback);
	fShape := protoShape;
	fConstrainsMouse := constrain;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TShapeSketcher.Free;

	BEGIN
	fShape := TShape(FreeIfObject(fShape));

	INHERITED Free;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TShapeSketcher.DoIt;

	VAR
		pMCEntry:			MCEntryPtr;

	BEGIN
	fShape.fIsSelected := TRUE;
	fShape.fPattern := 1 + ABS(Random MOD kTotalPatterns);
	IF qNeedsColorQD | gConfiguration.hasColorQD THEN
	{ Pick a random color from the Color menu }
		BEGIN
		pMCEntry := GetMCEntry(mColor, ABS(Random MOD CountMItems(GetMHandle(mColor))) + 1);
		fShape.fColor := pMCEntry^.mctRGB2;
		END
	ELSE
		fShape.fColor := gRGBBlack; 					{ Set color to black }

	fShapeDocument.fReplaceCommand := SELF;
	fShapeView.InvalShape(fShape);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TShapeSketcher.UndoIt;

	BEGIN
	INHERITED UndoIt;
	fShapeView.InvalShape(fShape);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TShapeSketcher.RedoIt;

	BEGIN
	IF fShapeView.Focus THEN;
	fShapeView.Deselect;
	INHERITED RedoIt;
	fShape.fIsSelected := TRUE;
	fShapeView.InvalShape(fShape);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TShapeSketcher.Commit;

	BEGIN
	fShapeDocument.AddShape(fShape);					{ Add the shape to the list }
	{ Set this field to NIL to prevent Free from freeing it}
	fShape := NIL;
	INHERITED Commit;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TShapeSketcher.EachNewShapeDo(PROCEDURE DoThis(shape: TShape)); OVERRIDE;

	BEGIN
	DoThis(fShape); 									{ there's only one shape }
	END;

{--------------------------------------------------------------------------------------------------}
{$ARes}

FUNCTION TShapeSketcher.FirstShapeThat(FUNCTION TestShape(aShape: TShape): BOOLEAN): TShape;
	OVERRIDE;

	BEGIN
	IF TestShape(fShape) THEN							{ there's only one shape to test }
		FirstShapeThat := fShape
	ELSE
		FirstShapeThat := NIL;
	END;

{--------------------------------------------------------------------------------------------------}
{$S AFields}

PROCEDURE TShapeSketcher.Fields(obj: TObject); OVERRIDE;

	BEGIN
	WITH obj DO
		BEGIN
		DoToField(AtStr('TShapeSketcher'), NIL, bClass);
		DoToField(AtStr('fShape'), @fShape, bRect);
		END;
	INHERITED Fields(obj);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TShapeSketcher.TrackConstrain(aTrackPhase: TrackPhase;
											  anchorPoint: VPoint;
											  previousPoint: VPoint;
											  VAR nextPoint: VPoint;
											  mouseDidMove: BOOLEAN); OVERRIDE;

	VAR
		dh: 				INTEGER;
		dv: 				INTEGER;
		absDh:				INTEGER;
		absDv:				INTEGER;
		delta:				INTEGER;

	BEGIN
	dh := nextPoint.h - anchorPoint.h;
	dv := nextPoint.v - anchorPoint.v;
	absDh := ABS(dh);
	absDv := ABS(dv);
	delta := Min(absDh, absDv);
	IF dh < 0 THEN
		dh := - delta
	ELSE
		dh := delta;
	IF dv < 0 THEN
		dv := - delta
	ELSE
		dv := delta;
	nextPoint.h := anchorPoint.h + dh;
	nextPoint.v := anchorPoint.v + dv;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TShapeSketcher.TrackFeedback(aTrackPhase: TrackPhase;
											  anchorPoint: VPoint;
											  previousPoint: VPoint;
											  nextPoint: VPoint;
											  mouseDidMove: BOOLEAN;
											  turnItOn: BOOLEAN);

	BEGIN
	INHERITED TrackFeedback(aTrackPhase, anchorPoint, previousPoint, nextPoint, mouseDidMove, turnItOn);
	IF mouseDidMove & (fShape <> NIL) THEN
		fShape.DrawOutline; 							{ draw/erase }
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

FUNCTION TShapeSketcher.TrackMouse(aTrackPhase: TrackPhase; VAR anchorPoint, previousPoint,
								   nextPoint: VPoint; mouseDidMove: BOOLEAN): TTracker;

	VAR
		aVRect: 			VRect;
		r:					Rect;
		size:				Point;
		bigEnough:			BOOLEAN;

	BEGIN
	TrackMouse := INHERITED TrackMouse(aTrackPhase, anchorPoint, previousPoint, nextPoint,
									   mouseDidMove);

	IF aTrackPhase = trackRelease THEN					{create the new shape}
		BEGIN
		bigEnough := FALSE;

		size.h := nextPoint.h - anchorPoint.h;
		size.v := nextPoint.v - anchorPoint.v;
		IF ABS(size.h) >= kMinWidth THEN
			IF ABS(size.v) >= kMinHeight THEN
				bigEnough := TRUE;

		IF NOT bigEnough THEN
			BEGIN
			fShape := TShape(FreeIfObject(fShape));

			TrackMouse := NIL;
			END;
		END
	ELSE
		BEGIN
		Pt2VRect(anchorPoint, nextPoint, aVRect);
		VRectToRect(aVRect, r); 						{ okay because we know the view's size is in
														 QD dimensions }
		fShape.fExtentRect := r;
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ASelCommand}

PROCEDURE TShapeCutCopyCommand.IShapeCutCopyCommand(itsCmdNumber: CmdNumber;
													itsShapeView: TShapeView);

	BEGIN
	IShapeCommand(itsCmdNumber, itsShapeView, gZeroVPt, NOT kBetterFeedbackDesired);
	fChangesClipboard := TRUE;
	fCausesChange := itsCmdNumber = cCut;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TShapeCutCopyCommand.DoIt;

	VAR
		succeeded:			BOOLEAN;
		clipShapeView:		TShapeView;
		clipDoc:			TShapeDocument;
		selTopLeft: 		Point;
		count:				INTEGER;
		outline:			Rect;
		fi: 				FailInfo;

{--------------------------------------------------------------------------------------------------}

	PROCEDURE CopyToClip(shape: TShape);

		VAR
			aNewShape:			TShape;
			r:					Rect;

		BEGIN
		IF shape.fIsSelected THEN
			BEGIN
			aNewShape := TShape(shape.Clone);
			WITH aNewShape DO
				BEGIN
				fIsSelected := FALSE;
				fWasSelected := FALSE;
				END;
			r := aNewShape.fExtentRect;
			OffSetRect(r, gClipMargin.h - outline.left, gClipMargin.v - outline.top);
			aNewShape.fExtentRect := r;

			clipDoc.AddShape(aNewShape);
			END;
		END;

	PROCEDURE HdlFailure(error: OSErr; message: LONGINT);

		BEGIN
		clipDoc := TShapeDocument(FreeIfObject(clipDoc));
		END;

	BEGIN												{ TShapeCutCopyCommand.DoIt }
	fShapeView.SaveSelection(fID = cCut);		{don't inval if it's just COPY}
	New(clipDoc);
	clipDoc.IShapeDocument(NIL);

	CatchFailures(fi, HdlFailure);
	fShapeDocument.SurveyShapes(TRUE, count, outline);

	New(clipShapeView);
	clipShapeView.IShapeView(clipDoc, NIL, TRUE);

	{ Set fShapeView since the doc will NOT be told to DoMakeViews }
	clipDoc.fShapeView := clipShapeView;

	IF fID = cCut THEN
		IF fShapeView.Focus THEN;

	fShapeDocument.EachShapeDo(CopyToClip);
	clipShapeView.AdjustSize;

	{ Make sure the Cut left us with enough memory to continue. }
	FailSpaceIsLow;
	Success(fi);

	gClipboardMgr.ClaimClipboard(clipShapeView);

	fShapeDocument.fFiltering := (fID <> cCopy);

	fShapeDocument.fReplaceCommand := SELF;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TShapeCutCopyCommand.RedoIt;

	BEGIN
	fShapeDocument.fFiltering := (fID <> cCopy);
	INHERITED RedoIt;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ASelCommand}

PROCEDURE TShapePasteCommand.IShapePasteCommand(itsShapeView: TShapeView);

	VAR
		fi: 				FailInfo;

	PROCEDURE HdlFailure(error: OSErr; message: LONGINT);

		BEGIN
		IF FreeIfObject(SELF) <> NIL THEN;
		END;

	BEGIN
	fPasteList := NIL;									{ So free works when we can't allocated it }
	IShapeCommand(cPaste, itsShapeView, gZeroVPt, NOT kBetterFeedbackDesired);
	CatchFailures(fi, HdlFailure);
	fPasteList := NewList;
	Success(fi);
{$IFC qDebug}
	fPasteList.SetEltType('TShape');
{$ENDC}
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TShapePasteCommand.Commit; OVERRIDE;

	VAR
		theShape:			TShape;

	BEGIN
	IF gPasteReplacesSelection THEN
		INHERITED Commit;								{Deletes old selectees from the document}

 { The following loop transfers shapes from fPasteList to the document's
   shape list, in such a way that fPasteList is shrunk while the
   document's shape list is grown.  This helps prevent running out
   of memory when committing a Paste command. }
	theShape := TShape(fPasteList.First);
	WHILE theShape <> NIL DO
		BEGIN
		fPasteList.Delete(theShape);
		fShapeDocument.AddShape(theShape);
		theShape := TShape(fPasteList.First);
		END;

	fShapeDocument.fReplaceCommand := NIL;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TShapePasteCommand.DoIt; OVERRIDE;

	VAR
		whereToPaste:		Point;
		noOfShapes: 		INTEGER;
		translation:		VPoint;
		t:					INTEGER;
		vhs:				VHSelect;
		extent: 			Rect;
		scrollerExtent: 	VRect;

	PROCEDURE PasteShape(clipShape: TShape);

		VAR
			aShape: 			TShape;

		BEGIN
		aShape := TShape(clipShape.Clone);
		WITH aShape DO
			BEGIN
			fIsSelected := TRUE;
			fWasSelected := TRUE;
{$Push} {$h-}
			OffSetRect(fExtentRect, whereToPaste.h, whereToPaste.v);
{$Pop}
			END;
		fPasteList.InsertLast(aShape);
		fShapeView.InvalShape(aShape);
		END;

	BEGIN
{$IFC qDebug}
	IF NOT Member(gClipboardMgr.fClipView, TShapeView) THEN
		ProgramBreak('Attempt to paste a non-TShapeView clipboard');
{$ENDC}

 { The next section figures out where the pasted shapes should be placed
   in the view. Lovely, isn't it? }
	IF gPasteReplacesSelection THEN
		BEGIN
		{ If we're replacing shapes, then paste the new shapes starting at
		  the top-left corner of the replaced shapes. Otherwise, start
		  at the last clicked point in the view }
		fShapeDocument.SurveyShapes(TRUE, noOfShapes, extent);
		IF noOfShapes > 0 THEN
			whereToPaste := extent.topLeft
		ELSE
			whereToPaste := fShapeView.fClickPt;
		END
	ELSE
		BEGIN
		fShapeView.fSuperView.GetExtent(scrollerExtent);
		FOR vhs := v TO h DO
			WITH scrollerExtent.topLeft DO
				BEGIN
				{ temp var "t" needed because Code Generator finds the
				  following expression too complex }
				t := (scrollerExtent.botRight.vh[vhs] + {scrollerExtent.} vh[vhs] -
					 gClipboardMgr.fClipView.fSize.vh[vhs]) DIV 2;
				whereToPaste.vh[vhs] := Max(			{translation.} vh[vhs], t);
				END;
		END;
	SubPt(gClipMargin, whereToPaste);

	fShapeView.SaveSelection(gPasteReplacesSelection);
	fShapeView.Deselect;

	TShapeView(gClipboardMgr.fClipView).fShapeDocument.EachShapeDo(PasteShape);

	fShapeDocument.fFiltering := gPasteReplacesSelection;
	fShapeDocument.fReplaceCommand := SELF;
	fShapeView.AdjustSize;								{Make sure all the Pasted shapes can be
														 seen}
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TShapePasteCommand.Free; OVERRIDE;

	BEGIN
	fPasteList := TList(FreeListIfObject(fPasteList));	{Free the TList object itself}

	INHERITED Free;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TShapePasteCommand.EachNewShapeDo(PROCEDURE
											DoThis(shape: TShape));

	BEGIN
	fPasteList.Each(DoThis);
	END;

{--------------------------------------------------------------------------------------------------}
{$ARes}

FUNCTION TShapePasteCommand.FirstShapeThat(FUNCTION
										   TestShape(aShape: TShape): BOOLEAN): TShape;

	BEGIN
	FirstShapeThat := TShape(fPasteList.FirstThat(TestShape));
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TShapePasteCommand.UndoIt; OVERRIDE;

{--------------------------------------------------------------------------------------------------}

	PROCEDURE BeInvalidated(theShape: TShape);

		BEGIN
		fShapeView.InvalShape(theShape);
		END;

	BEGIN
	INHERITED UndoIt;
	EachNewShapeDo(BeInvalidated);
	IF fShapeView.Focus THEN;
	fShapeDocument.fFiltering := FALSE;
	fShapeView.AdjustSize;								{In case we replaced shapes when we Pasted}
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TShapePasteCommand.RedoIt; OVERRIDE;

	PROCEDURE BeSelected(shape: TShape);

		BEGIN
		shape.fIsSelected := TRUE;
		fShapeView.InvalShape(shape);
		END;

	BEGIN
	IF fShapeView.Focus THEN;
	fShapeView.Deselect;
	EachNewShapeDo(BeSelected); 						{Invalidate all the newly-added shapes}
	fShapeDocument.fFiltering := gPasteReplacesSelection;
	INHERITED RedoIt;
	fShapeView.AdjustSize;								{Make sure size reflects the Paste}
	END;

{--------------------------------------------------------------------------------------------------}
{$S ASelCommand}

PROCEDURE TShapeClearCommand.IShapeClearCommand(itsShapeView: TShapeView);

	BEGIN
	IShapeCommand(cClear, itsShapeView, gZeroVPt, NOT kBetterFeedbackDesired);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TShapeClearCommand.DoIt;

	BEGIN
	fShapeView.SaveSelection(TRUE); 					{TRUE means invalidate the shapes}
	fShapeDocument.fFiltering := TRUE;
	fShapeDocument.fReplaceCommand := SELF;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ADoCommand}

PROCEDURE TShapeClearCommand.RedoIt;

	BEGIN
	fShapeDocument.fFiltering := TRUE;
	INHERITED RedoIt;
	END;

{--------------------------------------------------------------------------------------------------}
{$S AInit}

PROCEDURE TPatternsMenu.IPatternsMenu;

	VAR
		aPatternsPalette:	TPatternsPalette;

	BEGIN
	ITearOffMenuView(mPatterns, kWPatternsChoice, kHPatternsChoice, gPatternsWindow);

	New(aPatternsPalette);
	aPatternsPalette.IPatternsPalette(SELF);
	aPatternsPalette.fIdentifier := 'MPLT'; 			{ identifies the patterns palette in the
														 menu }
	END;

{--------------------------------------------------------------------------------------------------}
{$S AInit}

PROCEDURE TToolsMenu.IToolsMenu;

	VAR
		aToolsPalette:		TToolsPalette;

	BEGIN
	ITearOffMenuView(mTools, kToolsPaletteWidth, kToolsPaletteHeight, gToolsPaletteWindow);

	New(aToolsPalette);
	aToolsPalette.IToolsPalette(NIL, SELF);
	aToolsPalette.fIdentifier := 'MTUL';
	END;

{--------------------------------------------------------------------------------------------------}
{$S AOpen}

PROCEDURE TPatternsPalette.IPatternsPalette(itsSuperview: TView);

	VAR
		itsSize:			VPoint;

	BEGIN
	SetVPt(itsSize, kWPatternsChoice, kHPatternsChoice);
	IView(NIL, itsSuperview, gZeroVPt, itsSize, sizeFixed, sizeFixed);

	fCurrPattern := 0;									{ the pattern currently selected by user }
	fSelectedPattern := 0;								{ the pattern selected by user during
														 tracking }
	fHLDesired := hlOn;

	BuildChoiceArray;
	BuildFramesArray;
	END;

{--------------------------------------------------------------------------------------------------}
{$IFC qTemplateViews}
{$S AOpen}

PROCEDURE TPatternsPalette.IRes(itsDocument: TDocument; itsSuperview: TView;
	VAR itsParams: Ptr); OVERRIDE;

	VAR
		r:					Rect;
		choiceID:			INTEGER;
		i:					INTEGER;
		j:					INTEGER;

	BEGIN
	INHERITED IRes(itsDocument, itsSuperview, itsParams);

	fCurrPattern := 0;									{ the pattern currently selected by user }
	fSelectedPattern := 0;								{ the pattern selected by user during
														 tracking }
	fHLDesired := hlOn;

	BuildChoiceArray;
	BuildFramesArray;
	END;

{$ENDC}

{--------------------------------------------------------------------------------------------------}
{$S AOpen}

PROCEDURE TPatternsPalette.BuildChoiceArray;

	VAR
		r:					Rect;
		choiceID:			INTEGER;
		i:					INTEGER;
		j:					INTEGER;

	BEGIN
	{ Create the Pattern choice rectangles that hold the Patterns }
	SetRect(r, 0, 0, kWPatternsCell, kHPatternsCell);
	choiceID := 1;
	FOR i := 1 TO kPatternsAcross DO					{all the rectangles across}
		BEGIN
		FOR j := 1 TO kPatternsDown DO					{all the rectangles down}
			BEGIN
			fChoiceArray[choiceID] := r;
			OffSetRect(r, 0, kHPatternsCell + kFrameWidth);
			choiceID := choiceID + 1;
			END;
		{ move rectangle over one column }
		SetRect(r, r.right + kFrameWidth, 0, r.right + kFrameWidth + kWPatternsCell,
				kHPatternsCell);
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S AInit}

PROCEDURE TPatternsPalette.BuildFramesArray;

	VAR
		i:					INTEGER;
		theMenuColors:		MenuColors;
		tempRect:			Rect;

	BEGIN
	{ build the vertical borders between the patterns }
	tempRect := gZeroRect;
	tempRect.bottom := kHPatternsChoice;
	tempRect.left := kWPatternsCell;
	tempRect.right := kWPatternsCell + kFrameWidth;
	FOR i := 1 TO kVerticalFrames DO
		BEGIN
		fFramesArray[i] := tempRect;
		OffSetRect(tempRect, kWPatternsCell + kFrameWidth, 0);
		END;

	{ build the horizontal borders between the patterns }
	tempRect := gZeroRect;
	tempRect.right := kWPatternsChoice;
	tempRect.top := kHPatternsCell;
	tempRect.bottom := kHPatternsCell + kFrameWidth;
	FOR i := 1 TO kHorizontalFrames DO
		BEGIN
		fFramesArray[i + kVerticalFrames] := tempRect;
		OffSetRect(tempRect, 0, kHPatternsCell + kFrameWidth);
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TPatternsPalette.TrackFeedback(aTrackPhase: TrackPhase;
											  anchorPoint: VPoint;
											  previousPoint: VPoint;
											  nextPoint: VPoint;
											  mouseDidMove: BOOLEAN;
											  turnItOn: BOOLEAN);

	VAR
		r:					Rect;
		qdPt:				Point;
		i:					INTEGER;

	PROCEDURE TurnOffPattern(patternRect: Rect; whichPattern: INTEGER);

		BEGIN
		{ only turn it off if it's already on }
		IF fSelectedPattern = whichPattern THEN
			BEGIN
			PenNormal;
			FillRect(patternRect, gPat[whichPattern]);	{ draw pattern }
			InsetRect(patternRect, - 1, - 1);			{ draw frame }
			FrameRect(patternRect);
			FrameCurrPattern;							{ frame for current pattern }

			fSelectedPattern := 0;						{ remember the pattern currently selected }
			END;
		END;

	PROCEDURE TurnOnPattern(patternRect: Rect; whichPattern: INTEGER);

		BEGIN
		{ only turn it on if it's not already on }
		IF fSelectedPattern <> whichPattern THEN
			BEGIN
			PenNormal;
			PenMode(patXOr);
			InsetRect(patternRect, - 1, - 1);
			PenSize(2, 2);
			FrameRect(patternRect);

			fSelectedPattern := whichPattern;			{ remember the pattern currently selected }
			END;
		END;

	BEGIN
	IF Focus THEN;
	qdPt := ViewToQDPt(nextPoint);
	FOR i := 1 TO kTotalPatterns DO
		BEGIN
		r := fChoiceArray[i];
		IF PtInRect(qdPt, r) THEN
			BEGIN
			IF turnItOn THEN
				TurnOnPattern(r, i)
			ELSE
				TurnOffPattern(r, i);
			LEAVE;
			END;
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TPatternsPalette.FrameCurrPattern;
{ frame the currently selected pattern }

	VAR
		r:					Rect;

	BEGIN
	IF fCurrPattern <> 0 THEN							{ frame selected pattern, if any }
		BEGIN
		r := fChoiceArray[fCurrPattern];

		PenMode(patCopy);
		InsetRect(r, - 1, - 1);
		FrameRect(r);									{ frame it }
		InsetRect(r, - 3, - 3); 						{ outer white frame }
		PenSize(3, 3);
		PenPat(white);
		FrameRect(r);
		InsetRect(r, - 1, - 1); 						{ outer black frame }
		PenSize(1, 1);
		PenPat(black);
		FrameRect(r);
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TPatternsPalette.DoHighlightSelection(fromHL, toHL: HLState);

	VAR
		r:					Rect;
		area:				VRect;

	BEGIN
	IF (fCurrPattern >= 1) & (fCurrPattern <= kTotalPatterns) THEN
		CASE toHL OF
			hlOff:
				BEGIN
				r := fChoiceArray[fCurrPattern];
				InsetRect(r, - 5, - 5);
				QDToViewRect(r, area);
				Draw(area);
				END;
			hlOn:
				FrameCurrPattern;
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TPatternsPalette.SelectNewPattern(whichPattern: INTEGER);

	BEGIN
	IF IsShown THEN
		BEGIN
		IF Focus THEN;
		DoHighlightSelection(hlOn, hlOff);
		fCurrPattern := whichPattern;
		DoHighlightSelection(hlOff, hlOn);
		END;
	END;

{--------------------------------------------------------------------------------------------------}
{$S ASelCommand}

PROCEDURE TPatternsPalette.DoMouseCommand(VAR theMouse: VPoint; event: TToolboxEvent;
										  hysteresis: Point);

	VAR
		aReshadeCmd:		TReshadeCmd;
		aShapeView: 		TShapeView;
		aWindow:			TWindow;
		menuPatternsView:	TPatternsPalette;
		floatingPatternsView: TPatternsPalette;

	BEGIN
	New(aReshadeCmd);

	{ find the active TShapeView }
	aShapeView := NIL;
	aWindow := gApplication.GetActiveWindow;
	IF (aWindow <> NIL) THEN
		aShapeView := TShapeView(aWindow.FindSubView('SHAP'));

	menuPatternsView := NIL;
	IF (gPatternsMenu <> NIL) THEN
		menuPatternsView := TPatternsPalette(gPatternsMenu.FindSubView('MPLT'));

	floatingPatternsView := NIL;
	IF (gPatternsWindow <> NIL) & gPatternsWindow.IsShown THEN
		floatingPatternsView := TPatternsPalette(gPatternsWindow.FindSubView('PPLT'));

	aReshadeCmd.IReshadeCmd(cPatterns, aShapeView, SELF, menuPatternsView, floatingPatternsView,
							theMouse);

	PostCommand(aReshadeCmd);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TPatternsPalette.DoSetCursor(localPoint: VPoint; cursorRgn: RgnHandle); OVERRIDE;

	VAR
		qdExtent:			Rect;

	BEGIN
	SetCursor(arrow);
	GetQDExtent(qdExtent);
	RectRgn(cursorRgn, qdExtent);
	END;

{--------------------------------------------------------------------------------------------------}
{$S ARes}

PROCEDURE TPatternsPalette.Draw(area: VRect);

	VAR
		i:					INTEGER;
		QDArea, r, tempRect, drawRect: Rect;

	BEGIN
	ViewToQDRect(area, QDArea);
	{ draw the frames }
	FOR i := 1 TO kTotalFrames DO
		BEGIN
		r := fFramesArray[i];
		IF SectRect(r, QDArea, drawRect) THEN
			FrameRect(drawRect);
		END;

	{ draw the Patterns }
	FOR i := 1 TO kTotalPatterns DO
		BEGIN
		r := fChoiceArray[i];
		IF SectRect(r, QDArea, drawRect) THEN
			FillRect(drawRect, gPat[i]);
		END;

	INHERITED Draw(area);
	END;

{--------------------------------------------------------------------------------------------------}
{$S AFields}

PROCEDURE TPatternsPalette.Fields(obj: TObject); OVERRIDE;

	VAR
		i:					INTEGER;
		str:				Str255;

	BEGIN
	WITH obj DO
		BEGIN
		DoToField(AtStr('TPatternsPalette'), NIL, bClass);
		DoToField(AtStr('fCurrPattern'), @fCurrPattern, bInteger);
		FOR i := 1 TO kTotalPatterns DO
			BEGIN
			NumToString(i, str);
			DoToField(AtStr(Concat('fChoiceArray[', str, ']')), @fChoiceArray[i], bRect);
			END;
		END;
	INHERITED Fields(obj);
	END;
