// UAppleEvents.h
// Copyright 1984-1991 Apple Computer, Inc.All rights reserved.

#ifndef __UAPPLEEVENTS__
#define __UAPPLEEVENTS__

#ifndef __PASCALDEFS__
#include "PascalDefs.h"
#endif

#ifndef __APPLEEVENTS__
#include "AppleEvents.h"
#endif

#ifndef __EDITIONS__
#include "Editions.h"
#endif

#ifndef __UOBJECT__
#include "UObject.h"
#endif


// Forward and external classes
class TDynamicArray;

class TAppleEvent : public TObject
{
public:

	AppleEvent fMessage;						// Created by IAppleEvent, Set from InitFromMessage
	long	fSendingMode;						// kAENoReply, kAEQueueReply, or kAEWaitReply
	short	fPriority;							// kAENormalPriority or kAEHighPriority
	long	fTimeoutVal;						// How long should we wait for a response?
	Boolean	fFreeMessage;						// False if initialized from InitFromMessage.
												// We let the AppleEvent manager take care of it.

	virtual pascal void IAppleEvent(const AEEventClass itsEventClass,
									const AEEventID itsEventID,
									const AEAddressDesc& itsAddress,
									long itsSendingMode);


	// Creates a new AppleEvent message and stores it in fMessage.Supplies
	// defaults for return ID, transaction ID, and sending priority

	virtual pascal void InitFromMessage(const AppleEvent& theMessage);
	// Stores theMessage in fMessage.This may be a message or a reply.

	virtual pascal void Free();

	virtual pascal void Send();
	// Send fMessage by the indicated fSending mode and with fPriority.

	virtual pascal void GetAddress(AEAddressDesc& theAddress);
	// Return the target address of the message

	virtual pascal short GetPriority();
	// Return the sending priority of the message

	virtual pascal short GetReturnID();
	// Return the ID assigned to the message

	virtual pascal long GetTimeoutVal();
	// Return the timeout value (in ticks) used when sending the message

	virtual pascal long GetTransactionID();
	// Return the ID of the transaction the message belongs to

	virtual pascal void SetAddress(const AEAddressDesc& theAddress);
	// Assign the target address of the message

	virtual pascal void SetPriority(short thePriority);
	// Assign the priority with which the message will be sent

	virtual pascal void SetReturnID(short returnID);
	// Assign a new return ID to the message

	virtual pascal void SetTimeoutVal(long theTimeoutVal);
	// Assign the time (in ticks) to wait for reply if sending kAEWaitReply

	virtual pascal void SetTransactionID(long transactionID);
	// Assign a transaction ID to the message

	// READING - The following methods retrieve theData from fMessage based on the keyword
	virtual pascal void ReadInteger(const AEKeyword theKey,
									short& theData);
									
	virtual pascal void ReadLong(const AEKeyword theKey,
								 long& theData);
								 
	virtual pascal void ReadString(const AEKeyword theKey,
								 Str255& theData);
								 
	virtual pascal void ReadSectionHandle(const AEKeyword theKey,
										  SectionHandle& theData);

	virtual pascal void ReadPtrList(const AEKeyword theKey,
									const DescType theType,
									TDynamicArray* theData);

	virtual pascal void ReadHandleList(const AEKeyword theKey,
									   const DescType theType,
									   TDynamicArray* theData);


	// Allow raw read access to a parameter of the message

	virtual pascal void ReadParameter(const AEKeyword theKey,
									  const DescType desiredType,
									  AEDesc& theData);


	virtual pascal void ReadParameterPtr(const AEKeyword theKey,
										 const DescType desiredType,
										 DescType actualType,
										 Ptr theData,
										 long maximumSize,
										 long& actualSize);

	// WRITING - The following methods store theData in fMessage with theKey
	virtual pascal void WriteInteger(const AEKeyword theKey,
									 short theData);
									 
	virtual pascal void WriteLong(const AEKeyword theKey,
								  long theData);
								  
	virtual pascal void WriteString(const AEKeyword theKey,
								 	const Str255& theData);
								 
	virtual pascal void WriteSectionHandle(const AEKeyword theKey,
										   SectionHandle theData);
										   
	virtual pascal void WritePtrList(const AEKeyword theKey,
									 const DescType theType,
									 TDynamicArray* theData);

	virtual pascal void WriteHandleList(const AEKeyword theKey,
										const DescType theType,
										TDynamicArray* theData);

	// Allow raw access to a parameter of the message

	virtual pascal void WriteParameter(const AEKeyword theKey,
									   const AEDesc& theData);

	virtual pascal void WriteParameterPtr(const AEKeyword theKey,
										  const DescType typeCode,
										  Ptr theData,
										  long dataSize);

	// There will be more reading/writing methods as we support more types

	virtual pascal void DeleteParameter(const AEKeyword theKey);
	// Delete the specified parameter from the message

	virtual pascal void Fields(TObject* obj);
	// Used by the Inspector and the Debugger to display the contents of this class's fields
};


extern pascal void InitUAppleEvents();
// Loads essential tables and starts up support for appleevents

extern pascal OSErr AppleEventsDispatch(const AppleEvent& message,
										const AppleEvent& reply,
										long info);
// The general dispatch function which is passed AEInit.  Forwards into gApplication->DispatchAppleEvents

#endif


