// UApplication.h
// Copyright  1984-1991 by Apple Computer Inc.	All rights reserved.

#ifndef __UAPPLICATION__
#define __UAPPLICATION__

#ifndef __PASCALDEFS__
#include <PascalDefs.h>
#endif

#ifndef __GEOMETRY__
#include <Geometry.h>
#endif

#ifndef __UGEOMETRY__
#include <UGeometry.h>
#endif

#ifndef __DIALOGS__
#include <Dialogs.h>
#endif

#ifndef __UEVTHANDLER__
#include <UEvtHandler.h>
#endif

#ifndef __UCOMMAND__
#include <UCommand.h>
#endif

#ifndef __UITERATOR__
#include <UIterator.h>
#endif

#ifndef __ULIST__
#include <UList.h>
#endif


//----------------------------------------------------------------------------------------
// Forward and external classes
//----------------------------------------------------------------------------------------

class TDocument;
class TFile;
class TView;
class TWindow;
class TEvent;
class TToolboxEvent;
class TApplication;


//----------------------------------------------------------------------------------------
// CDocumentIterator
//----------------------------------------------------------------------------------------

class CDocumentIterator : public CObjectIterator
{
public:
	CDocumentIterator(TApplication* itsApplication,
					  ArrayIndex itsLowBound, ArrayIndex itsHighBound,
					  Boolean itsForward);

	CDocumentIterator(TApplication* itsApplication, Boolean itsForward);

	CDocumentIterator(TApplication* itsApplication);
	
	TDocument* CurrentDocument();
		// returns the current document
	
	TDocument* FirstDocument();
		// return the first document in the iteration

	TDocument* NextDocument();
		// advances the iteration and then returns the document
};


//----------------------------------------------------------------------------------------
//  TEventList: A list for keeping TEvents ordered by priority
//----------------------------------------------------------------------------------------

class TEventList : public TSortedList
{
public:
	virtual pascal void IEventList();
		// Initialize the EventList procedurally.

	virtual pascal short Compare(TObject* item1,
								 TObject* item2);
		// Compares Two TEvents based on their priority

	virtual pascal void Insert(TObject* item);
		// Overridden to guarantee insertion even in low memory and to preserve arrival
        // sequence within "Compare" ordering.

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields.

};


//----------------------------------------------------------------------------------------
// Typedefs for functions used to do interation.
//----------------------------------------------------------------------------------------

typedef pascal void (*DoToVoidType)(void* staticLink);

typedef pascal void (*DoToEvtHandlerType)(TEvtHandler* anEvtHandler,
										  void* staticLink);
										  
typedef pascal void (*DoToDocumentType)(TDocument* doc,
										void* staticLink);
										
typedef pascal void (*DoToWindowType)(TWindow* aWindow,
									  void* staticLink);
									  
typedef pascal void (*DoToWindowPtrType)(WindowPtr theWMgrWindow,
										 void* staticLink);

typedef pascal Boolean (*TestWindowType)(TWindow* aWindow,
										 void* staticLink);
										 
typedef pascal Boolean (*TestWindowPtrType)(WindowPtr theWMgrWindow,
											void* staticLink);


//----------------------------------------------------------------------------------------
// TApplication: The purpose of this class is to implement the main event loop that all
// Macintosh applications must have. There is only one instance of TApplication. One thing
// that TApplication does is to interpret the raw events that are posted and convert them
// into higher-level events. For example, it converts a click in the menu bar into a 'menu
// event' that contains the menu ID/ item number that was chosen by the user.
//----------------------------------------------------------------------------------------

class TApplication : public TEvtHandler
{
public:
	TEventList* fEventList;						// Events/Commands that were posted for
                                                // execution
	
	Boolean fLaunchWithNewDocument;				// TRUE (the default) to create a new
                                                // untitled document when the application
                                                // is launched without a document.
												
    TCommand* fLastCommand; 					// the last command done or undone by the 
												// user
	
	Boolean fAllowApplicationToSleep;			// Whether the application should be
                                                // allowed to "sleep" and thus share
                                                // processor time, when getting events

	ProcessSerialNumber fApplicationPSN;		// This application's process serial
                                                // number.

	IdlePhase fIdlePhase;						// The current idle phase

	Boolean fAlwaysTrackCursor;					// set this to TRUE when you want the
                                                // application to track the cursor even if
                                                // it doesn't stray outside gCursorRgn.
												
	Boolean fAppDone;							// set this to TRUE when you want the
                                                // application to terminate

	short fClickCount;							// number of 'saved up' mouse clicks;
                                                // handled in TApplication.DispatchEvent,
                                                // ObeyEvent & ObeyMouseDown. If a
                                                // mouseDown, the value is that returned
                                                // by the call to
                                                // TApplication.CountClicks. If a mouseUp,
                                                // the value is left alone. Otherwise, set
                                                // to 0. Also set to 0 is the 2 clicks
                                                // were in different parts ofthe window.
												
	short fLastClickPart;						// the list window part clicked in-used
                                                // for double click detection
												
	Point fLastMsePt;							// coordinates of mouse in last event
                                                // passed to TApplication.CountClicks
												
	long fLastUpTime;							// time of last mouse up event passed to
                                                // TApplication.ObeyEvent

	long fNextSpaceMsg;							// time when next low space message should
                                                // be displayed
												
	long fLowSpaceInterval;						// If >= 0, the frequency (in Ticks) with
                                                // which MacApp displays a low space
                                                // alert. (Defaults to kLowSpaceInterval.)
                                                // If < 0, MacApp doesn't display an
                                                // alert.
												
	short fMainEventMask;						// Event mask used in main event loop.
                                                // Initialized by InitUMacApp.
												
	Boolean fSysWindowActive;					// TRUE if the front window is a system
                                                // window
	
	Boolean fUndoState;							// are we in undo or redo
	
	CmdNumber fUndoCmd;
	
	TList* fDocList;							// list of documents
	
	TList* fFreeWindowList;						// list of free-standing (documentless)
                                                // windows
	
	TEvtHandler* fHeadCohandler;				// head of linked list of global
                                                // co-handlers

	Boolean fInBackground;						// True if app is currently in background
	
	TEvtHandler* fTarget;						// the TEvtHandler that gets the first
                                                // chance at DoCommand, DoSetupMenu,
                                                // DoKeyCommand, Idle; should never be NIL
                                                // -- If you do not want your own target
                                                // set this to the application object
												
	OSType fMainFileType;						// principal file type opened/printed by
                                                // application; set in
                                                // TApplication.IApplication; by default,
                                                // TApplication.SFGetFilters returns a
                                                // list of just this
												
	short fEventLevel;							// A count of the number of nested calls
                                                // to PollToolboxEvent.
												
	short fMBarDisplayed;						// menus that are read in and installed in
                                                // menu bar
												
	short fMBarHierarchical;					// menus that pop up when a menu item is
                                                // choosen
	
	short fMBarNotDisplayed;					// menus that are read in but not
                                                // installed
	
	RgnHandle fSleepRgn;						// the region that will be passed to
                                                // WaitNextEvent as the sleep region.
                                                // Computed from the cursor and help
                                                // regions
												
	RgnHandle fCursorRgn;						// the current cursor region
	
	RgnHandle fHelpRgn;							// the current help region


	//------------------------------------------------------------------------------------
	// Initialization and Free
	//------------------------------------------------------------------------------------

	virtual pascal void Initialize();
		// Puts a newly created TApplication into a known state. Subclasses should
        // override it and call INHERITED. Typically used to put the object into a state
        // where it can be safely FREED.

	virtual pascal void IApplication(const OSType itsMainFileType);
		// itsMainFileType should be the the 4-byte file type that this application reads
        // writes. We require this here so that programmers do not have to explicitly
        // override TApplication.SFGetParms and supply the type there. That is the only
        // place where the main file type is used.

	virtual pascal Boolean ChooseApplication(AEAddressDesc& theAddress);
		// Choose a running application using the PPCBrowser and store the address in an
        // AppleEvent address

	virtual pascal void GetInspectorName(Str255& inspectorName);
		// Used to get this class's name by the Inspector.

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields, in this case the global variables.


	//------------------------------------------------------------------------------------
	// Event Handling
	//------------------------------------------------------------------------------------

	virtual pascal TToolboxEvent* GetEvent(short eventMask, long sleep, 
											RgnHandle sleepRgn);
		// Calls WaitNextEvent (or GetNextEvent) with the given event mask, sleep, and
        // sleep region parameters. You can override this if you have an alternate source
        // for events.

	virtual pascal void HandleAlienEvent(TEvent* event);
		// This method is called upon to deal with 'application-specific' events

	virtual pascal void MainEventLoop();
		// Loop on PollEvent until the application is done

	virtual pascal void DispatchEvent(TEvent* event);

	virtual pascal void DoAppleCommand(CmdNumber aCmdNumber, const AppleEvent& message,
									   const AppleEvent& reply);
		// Field the Required AppleEvents

	virtual pascal OSErr DispatchAppleEvent(const AppleEvent& message,
											const AppleEvent& reply,
											long info);
		// Called by AppleEventsDispatch to "DoAppleCommand" Default behavior is to call
        // fTarget.DoAppleCommand

	virtual pascal void DoEvent(EvtNumber eventNumber,
								TEvtHandler* source,
								TEvent* event);
		// When this is called by MacApp, theEvent is the event to be processed.

	virtual pascal void PostDoEvent(TEvent* event);

	virtual pascal void HandleActivateEvent(TToolboxEvent* event);
		// Handles activate (and deactivate) events. Calls the window's Activate method,
        // and invalidates the menubar if activating the window.

	virtual pascal void HandleDiskEvent(TToolboxEvent* event);
		// Handles disk events. Calls DIBadMount if the high word of theEvent's message is
        // <> noErr

	virtual pascal void HandleHighLevelEvent(TToolboxEvent* event);
		// Process a high level event.Default behavior is to assume it is an AppleEvent

	virtual pascal void KeyEventToComponents(TToolboxEvent* event);
		// re-extracts components of an event in a script manager compatible way.

	virtual pascal void HandleKeyDownEvent(TToolboxEvent* event);
		// Handles key down events. If the command key is down, then this calls
        // CommandKey. Otherwise it calls fTarget.DoKeyCommand.

	virtual pascal void HandleMouseDown(TToolboxEvent* event);
		// Called when a mouse down occurs; this cases on whereMouseDown and does the
        // right thing.

	virtual pascal TTracker* TrackMouse(const VPoint& theMouse, Point hysteresis,
										TTracker* theCommand);
		// Responsible for mouse tracking. !!! Candidate for eventhandler method?

	virtual pascal void HandleMouseUp(TToolboxEvent* event);
		// Sets fLastUpTime. Returns NIL.

	virtual pascal void HandleSystemEvent(TToolboxEvent* event);
		// Handles "system" events, in particular suspend/ resume events and mouse moved
        // events. Calls RegainControl if switching in, AboutToLoseControl if switching
        // out, or TrackCursor if it is a mouse-moved event.

	virtual pascal void HandleUpdateEvent(TToolboxEvent* event);
		// Handles window update events. Calls UpdateEvent for the window indicate by the
        // event.

	virtual pascal Boolean IsDeskAccessory(WindowPtr aWMgrWindow);
		// Returns true if the given Window Manager window is a desk accessory. We assume
        // that if the window has a negative windowKind then it is a desk accessory. If
        // the process Manager is present (or required) always returns FALSE.

	virtual pascal Boolean IsFrontProcess();
		// Returns true if the application is currently the front process.

	virtual pascal void MakeFrontProcess();
		// Makes this application the front process.

	virtual pascal void OpenDeskAccessory(const Str255& deskAccName);
		// Attempts to open the desk accessory whose name is deskAccName. Called from
        // TApplication.MenuEvent when an Apple menu item is chosen, and the item has a
        // negative command number.

	virtual pascal void PollEvent(Boolean allowApplicationToSleep);
		// Polls for events/commands and processes them.

	virtual pascal void PollToolboxEvent(Boolean allowApplicationToSleep);
		// Polls for events calling Idle before waitNextEvent if there are no events
        // pending and the application is allowed to go to sleep
        // (kAllowApplicationToSleep), calls HandleEvent with the event, if there is one.
        // and then idles if necessary.

	virtual pascal void Run();
		// call MainEventLoop

	virtual pascal void UpdateAllWindows();
		// Process all update events in the queue.

	virtual pascal Boolean InModalState();
		// Returns true if the front window is modal.

	virtual pascal Boolean InModalMenuState();
		// Returns true if the front window does not allow menu commands.


	//------------------------------------------------------------------------------------
	// Target and Cohandlers
	//------------------------------------------------------------------------------------

	virtual pascal void InstallCohandler(TEvtHandler* aCohandler,
										 Boolean addIt);
		// Used to add (addIt = TRUE) or delete (addIt = FALSE) a cohandler to/ from the
        // global list of cohandlers

	virtual pascal TWindow* GetFrontWindow();
		// Returns the window object reference for the frontmost application window
        // (whether active or not. Returns nil if the FrontWindow window is not a window
        // object

	virtual pascal TWindow* GetActiveWindow();
		// Returns the window object reference for the active application window.Returns
        // nil if the FrontWindow window is not a window object

	virtual pascal void SetTarget(TEvtHandler* newTarget);
		// Sets the current target (the head of the target chain) for the application.

	virtual pascal TEvtHandler* GetTarget();
		// Returns the current target


	//------------------------------------------------------------------------------------
	// Finder Requests
	//------------------------------------------------------------------------------------

	virtual pascal Boolean CanOpenDocument(CmdNumber itsCmdNumber, TFile* aFile);
		// Simulates the filtering done by Std File; this is only called when opening/
        // printing documents from the finder; when using Std File, Std File does the
        // filtering.

	virtual pascal void HandleFinderRequest();
		// If the AppleEvents Manager is not required or present: Gets the info from the
        // finder about what files to open/ print and opens/ prints them.
	

	//------------------------------------------------------------------------------------
	// Opening and Printing Documents
	//------------------------------------------------------------------------------------

	virtual pascal TDocument* AlreadyOpen(TFile* aFile);
		// Given a TFile, if that document is already opened, returns the TDocument.
        // Otherwise returns NIL.

	virtual pascal Boolean ChooseDocument(CmdNumber itsCmdNumber, TList** aFileList);
		// Call this to make a Std File Get call; returns TRUE is user selected a file.

	virtual pascal TDocument* DoMakeDocument(CmdNumber itsCmdNumber, TFile* itsFile);
		// Must be overridden. Based on itsCmdNumber create a document object of the
        // appropriate kind. If itsFile <> NIL it is attached to the document

	virtual pascal TFile* DoMakeFile(CmdNumber itsCmdNumber);
		// Default behavior is to return an instance of TFile.May be overridden to create
        // a file of your own special type.

	virtual pascal CmdNumber KindOfDocument(CmdNumber itsCmdNumber, TFile* itsFile);
		// Given a cmd number and a specification of the file, return the cmd number to
        // pass to DoMakeDocument that indicates the type of document to make; Default is
        // to return itsCmdNumber. If you have multiple document types, a good convention
        // is to return the cmd numbers assigned to create new documents of each kind.
        // aFile will be NIL if we are creating a brand new document, rather than opening
        // an existing document.

	virtual pascal void OpenNew(CmdNumber itsCmdNumber);
		// Called when the application is opened (itsCmdNumber = cFinderNew), or when NEW
        // is chosen from menu (itsCmdNumber = cNew). If you do not want to create a new
        // document when the user opens the application, override this and do nothing if
        // itsCmdNumber = cFinderNew.

	virtual pascal void OpenOld(CmdNumber itsOpenCmd, TFile* aFile);
		// Called when opening an existing document from finder (itsCmdNumber =
        // cFinderOpen) or if OPEN is chosen from menu (itsCmdNumber = cOpen).

	virtual pascal Boolean PrintDocument(TFile* aFile);
		// Called to print a document from the finder. Returns TRUE if the user did not
        // cancel printing of the rest of the documents.

	virtual pascal void SFGetParms(CmdNumber itsCmdNumber,
								   ProcPtr& fileFilter,
								   TypeListHandle typeList,
								   short& dlgID,
								   Point& where,
								   ProcPtr& dlgHook,
								   ProcPtr& modalFilter,
								   Ptr& activeList,
								   ProcPtr& activateProc,
								   Ptr yourDataPtr);
		// Called to return all the parameters that should be passed to SFPGetFile or
        // CustomGetFile, and CanOpenDocument; when called, typeList is a valid handle to
        // a 0-length block. typeList = list of just the main file type supported by the
        // application. (If type list ends up with nothing in it, then all file types are
        // supported.Defaults to:
		// 		dlgID = getDlgID or sfGetDialogID;
		// 		where = (100, 100) or (-1, -1);
		// 		fileFilter = NIL;
		// 		dlgHook = NIL;
		// 		modalFilter = NIL;
		// 		activeList := NIL;
		// 		activateProc := NIL;
		// 		yourDataPtr := NIL;


	//------------------------------------------------------------------------------------
	// Termination
	//------------------------------------------------------------------------------------

	virtual pascal void Close();
		// Called when the user chooses Quit from the menu, and tries to save all the open
        // documents. If all succeed the application terminates. Signals Failure with
        // error = noErr and message = msgCancelled if user cancels


	//------------------------------------------------------------------------------------
	// Hierarchy
	//------------------------------------------------------------------------------------

	virtual pascal void AddDocument(TDocument* aNewDocument);
		// Add another document to my list of documents, and make it the current one

	virtual pascal void AddFreeWindow(TWindow* aWindow);
		// Add the window to the free-window list

	virtual pascal void DeleteDocument(TDocument* docToDelete);
		// Delete a document from my list of documents

	virtual pascal void DeleteFreeWindow(TWindow* windowToDelete);
		// Delete a window from my list of free windows

	virtual pascal void EachActiveWindow(DoToWindowType DoToWindow, void* staticLink);
		// Iterates over active windows. i.e. floating windows and the active document
        // window

	virtual pascal void EachFreeWindow(DoToWindowType DoToWindow, void* staticLink);
		// Perform DoToWindow on each TWindow object in the list fFreeWindowList

	virtual pascal void ForAllDocumentsDo(DoToDocumentType DoToDoc, void* staticLink);
		// Perform the given procedure on all open documents currently owned by the
        // application

	virtual pascal void ForAllWindowsDo(DoToWindowType DoToWindow, void* staticLink);
		// Perform the given procedure on all windows known to the application, both
        // document and free windows.
	

	//------------------------------------------------------------------------------------
	// Window Manager Windows
	//------------------------------------------------------------------------------------
	
	virtual pascal void SelectWMgrWindow(WindowPtr aWMgrWindow);
		// "Selects" aWMgrWindow by calling the toolbox routine SelectWindow. When any
        // window is selected, it goes through here. We never call SelectWindow directly.

	virtual pascal WindowPtr GetRsrcWindow(Ptr storage, short rsrcId,
										   Boolean& isResizable,
										   Boolean& isClosable);
		// Get a Window Manger window resource whose resource id is rsrId. Return
        // isResizable and isClosable based upon the 'WIND' resource.

	virtual pascal TWindow* WMgrToWindow(WindowPtr aWMgrWindow);
		// Returns the window object that represents the given Window Manager window, or
        // NIL if there is no window object.
	

	//------------------------------------------------------------------------------------
	// Idle Events
	//------------------------------------------------------------------------------------
	
	virtual pascal long GetWaitTicks(Boolean allowApplicationToSleep);
		// Compute the "wait ticks" to be passed to GetEvent

	virtual pascal void Idle(IdlePhase phase);
		// Parcels out idle time to the DoIdle method in the target and cohandler chains
        // of TEvtHandlers. If your application needs to do something at idle time then
        // override DoIdle instead of this method.


	//------------------------------------------------------------------------------------
	// Cursor and Help
	//------------------------------------------------------------------------------------
	
	virtual pascal void ExcludeWindowRegions(RgnHandle deskTopRgn,
											 TestWindowType TestWindow,
											 void* staticLink);
		// Removes from the given region the visRgn of all windows for which TestWindow
        // returns TRUE. Used by GetDefaultCursorRgn and GetDefaultHelpRgn

	virtual pascal RgnHandle GetSleepRgn();
		// Return the "sleep region" to be passed to GetEvent

	virtual pascal void GetDefaultCursorRgn(Point globalMouse, RgnHandle cursorRgn);
		// Computes the cursor region for the case where no window has claimed the cursor

	virtual pascal void GetDefaultHelpRgn(Point globalMouse,  RgnHandle helpRgn);
		// Computes the help region for the case where no window has claimed the cursor

	virtual pascal void DoSetCursor(Point globalMouse, RgnHandle cursorRgn);
		// Sets the cursor to an arrow and computes the region for which it is claimed.
        // Called when no view has set the cursor

	virtual pascal void TrackCursor(Point globalMouse);
		// Allows windows/ views to set the cursor and cursor region via TView.HandleCursor
        // and TView.DoSetCursor. If no region is set, the cursor is set to an arrow and
        // the largest possible default cursor region is calculated.

	virtual pascal TView* HandleCursor(Point globalMouse, RgnHandle cursorRgn);

	virtual pascal void DoShowHelp(Point globalMouse, RgnHandle helpRgn);
		// !!!

	virtual pascal void TrackHelp(Point globalMouse);
		// !!!

	virtual pascal TView* HandleHelp(Point globalMouse, RgnHandle helpRgn);

	virtual pascal void InvalidateMouseRegions();
		// Invalidates the cursor and help regions

	virtual pascal void InvalidateCursorRgn();
		// Sets the cursor region to the empty region

	virtual pascal void InvalidateHelpRgn();
		// Sets the help region to the empty region

	virtual pascal Boolean IsCursorRgnInvalid();
		// Returns TRUE if the cursor region needs to be recalculated

	virtual pascal Boolean IsHelpRgnInvalid();
		// Returns TRUE if the help region needs to be recalculated
	

	//------------------------------------------------------------------------------------
	// Keyboard events
	//------------------------------------------------------------------------------------
	
	virtual pascal void DoKeyCommand(TToolboxEvent* event);
		// Handle keydown events.


	//------------------------------------------------------------------------------------
	// Menu Events
	//------------------------------------------------------------------------------------
	
	virtual pascal void DoCommandKey(TToolboxEvent* event);
		// Called when a keyDown event is received and the command key is down.

	virtual pascal void MenuEvent(long menuItem);
		// Given a value returned by MenuKey or MenuSelect, figure out the command number
        // that was chosen, have the application create a command object, and post it.

	virtual pascal void SetUndoText(Boolean cmdDone, CmdNumber aCmdNumber);
		// Called to setup the Undo menu item; if cmdDone is TRUE it reads Undo, otherwise
        // it reads Redo. aCmdNumber indicates the command that goes after the Undo/ Redo.

	virtual pascal void SetupTheMenus();
		// Initiates the process of enabling & checking menu items.


	//------------------------------------------------------------------------------------
	// Opening and Closing Windows
	//------------------------------------------------------------------------------------
	
	virtual pascal void CloseWMgrWindow(WindowPtr aWMgrWindow);
		// Called when user closes a window either with a menu item or with a GoAway box.
        // Signals Failure with error = noErr and message = msgCancelled if user cancels


	//------------------------------------------------------------------------------------
	// Command Management
	//------------------------------------------------------------------------------------
	
	virtual pascal void CommitLastCommand();
		// Commits the last command by calling its Commit method.

	virtual pascal TCommand* GetLastCommand();
		// Returns the last undoable command.The command returned is not yet committed

	virtual pascal TEvent* RetrieveAnEvent(); // Override
		// Retrieves a previously posted event/ command from the queue. Nil if there are no
        // queued commands.

	virtual pascal void ProcessEvent(TEvent* event);
		// Processes the given event from the queue of event objects. The event may be a
        // Toolbox event or a command object. Calls TEvent methods for three phases of
        // event processing: PreProcess, Process and PostProcess. Also accounts for
        // commands which track the mouse */

	virtual pascal void PerformCommand(TCommand* command);	// Override
		// Performs the given command taking care of UNDO/ REDO and clipboard related
        // details. Tracks the command first, if necessary. This DOES NOT check to see if
        // command is NIL.

	virtual pascal void PostAnEvent(TEvent* event);	// Override
		// Called to post an event or command to a queue for later processing. Default is
        // to hand off to fNextHandler

	virtual pascal void PostCommand(TCommand* command);	// Override
		// Called to post a command to the queue for later execution.


	//------------------------------------------------------------------------------------
	// Double/Triple Clicks
	//------------------------------------------------------------------------------------
	
	virtual pascal short CountClicks(TToolboxEvent* event, short whereMouseDown);
		// Called by TApplication.HandleMouseDown. Returns the number of clicks that can
        // be considered multiple clicks (e.g. if it returns 1 the mouse down should be
        // treated as a single click, or possibly the first click of a multi-click
        // sequence. If it returns 2 the click should be considered a double-click, etc. A
        // click is considered part of a multi-click sequence if the mouse down was within
        // the proper time range of the previous mouse up, and was within the proper
        // number of pixels of the last mouse down. TApplication.HandleMouseDown sets
        // theClickCount of its EventInfo record to the result of this function.
        // aPDownEvent is a pointer to the mouse down event. whereMouseDown is the result
        // of FindWindow on the event.

	virtual pascal TView* MakeViewForAlienClipboard();
		// Override this to create a view to handle your application's specific data
        // types. Return NIL (the default) to have the clipboard manager create a default
        // view.


	//------------------------------------------------------------------------------------
	// Command Handlers
	//------------------------------------------------------------------------------------
	
	virtual pascal void DoMenuCommand(CmdNumber aCmdNumber);
		// Handles the application specific menu commands.

	virtual pascal void DoSetupMenus();

	virtual pascal void AboutToLoseControl(Boolean convertClipboard);
		// Called when about to activate a Desk Accessory, switch out to another
        // application, or Terminate. convertClipboard is generally true, except it will
        // be FALSE if we get a System event that does not require Clipboard conversion

	virtual pascal void RegainControl(Boolean checkClipboard);
		// Called when switching in or when leaving a desk accessory


	//------------------------------------------------------------------------------------
	// Error Alerts
	//------------------------------------------------------------------------------------

	virtual pascal void Beep(short duration);
		// Call this method when you want to beep at the user. Sound manager users may
        // have to override this method to integrate its functionality in with your
        // application

	virtual pascal void ShowError(OSErr error, long message);
		// Calls ErrorAlert to display an error alert.

	virtual pascal void SpaceIsLow();
		// Called when space is low, at intervals of kLowSpaceInterval ticks. Displays an
        // alerting informing the user that memory space is low. Override this if you wish
        // to take more appropriate action.


	//------------------------------------------------------------------------------------
	// Miscellaneous
	//------------------------------------------------------------------------------------

	virtual pascal void DoShowAboutApp();
		// Called to show the "About the App" information

	virtual pascal void InvalidateFocus();
		// Called to indicate that a view is unfocused when you don't know which view was focused.

	virtual pascal Boolean IsHelpEnabled();
		// Returns TRUE if the Help Manager is present and the user has enabled help

	virtual pascal void ActivateBusyCursor(Boolean entering);
		// Called when the busy cursor mechanism should be activated or deactivated.

	virtual pascal Boolean AlertFilter(DialogPtr theDialog, EventRecord& theEvent,
									   short& itemHit);
									   
	virtual pascal void TApplication::DoMakeViewServer(void);
		// instantiates the global view server								   

};


//----------------------------------------------------------------------------------------
// commands that are created in response to common operations and will forward to the
// appropriate action
//----------------------------------------------------------------------------------------

//----------------------------------------------------------------------------------------
// TDebugCommand: Causes the MacApp Debugger to be entered
//----------------------------------------------------------------------------------------

#if qDebug
class TDebugCommand : public TNoChangesCommand
{
public:
	virtual pascal void IDebugCommand(CmdNumber itsCmdNumber);
		// Initialize the DebugCommand procedurally.

	virtual pascal void DoIt();
		// Cause the MacApp Debugger to be entered

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields.
};
#endif


//----------------------------------------------------------------------------------------
// TInspectorCommand: Tells the inspector to make a new window
//----------------------------------------------------------------------------------------

#if qInspector
class TInspectorCommand : public TNoChangesCommand
{
public:
	virtual pascal void IInspectorCommand(CmdNumber itsCmdNumber);
		// Initialize the DebugCommand procedurally.

	virtual pascal void DoIt();
		// Tell the inspector to make a new window

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields.
};
#endif


//----------------------------------------------------------------------------------------
// TNewDocCommand: Tells the application open a new document.
//----------------------------------------------------------------------------------------

class TNewDocCommand : public TNoChangesCommand
{
public:
	virtual pascal void INewDocCommand(CmdNumber itsCmdNumber);
		// Initialize the NewDocCommand procedurally.

	virtual pascal void DoIt();
		// tell the application open a new document.

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields.
};


//----------------------------------------------------------------------------------------
// TFilesCommand: Retrives a list of files from an AppleEvent Abstract class for
// TODocCommand and TPDocCommand.
//----------------------------------------------------------------------------------------

class TFilesCommand : public TAppleCommand
{
public:
	TList* fFileList;
	
	virtual pascal void IFilesCommand(CmdNumber itsCmdNumber, TList* theDocuments);
		// Initialize the command procedurally.

	virtual pascal void InitFromAppleEvent(CmdNumber itsCmdNumber,
										   TDocument* itsDocument,
										   TView* itsView,
										   const AppleEvent& itsMessage,
										   const AppleEvent& itsReply);
		// Initialize the filelist from the information contained in the AppleEvent.

	virtual pascal void Initialize();
		// Default initialization of some command fields.

	virtual pascal void Free();
		// Free the list of file alias handles.

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields
};


//----------------------------------------------------------------------------------------
// TODocCommand: Accepts a list of file aliases and opens each of them
//----------------------------------------------------------------------------------------

class TODocCommand : public TFilesCommand
{
public:
	virtual pascal void IODocCommand(CmdNumber itsCmdNumber, TList* theDocuments);
		// Initialize the ODocCommand procedurally.
		
	virtual pascal void DoIt();
		// Open each file found in fFileList.

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields
};


//----------------------------------------------------------------------------------------
// TPDocCommand: Accepts a list of file aliases and prints each of them
//----------------------------------------------------------------------------------------

class TPDocCommand : public TFilesCommand
{
public:
	virtual pascal void IPDocCommand(CmdNumber itsCmdNumber, TList* theDocuments);
		// Initialize the PDocCommand procedurally.

	virtual pascal void DoIt();
		// Print each file found in fFileList.

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields
};


//----------------------------------------------------------------------------------------
// TQuitCommand: Tells the application to quit.
//----------------------------------------------------------------------------------------

class TQuitCommand : public TNoChangesCommand
{
public:
	virtual pascal void IQuitCommand(CmdNumber itsCmdNumber);
		// Initialize the QuitCommand procedurally.

	virtual pascal void DoIt();
		// tell the application to quit.

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields.
};


//----------------------------------------------------------------------------------------
// TUndoRedoCommand: Tells the application to undo/redo the last undoable command.
//----------------------------------------------------------------------------------------

class TUndoRedoCommand : public TNoChangesCommand
{
public:
	virtual pascal void IUndoRedoCommand(CmdNumber itsCmdNumber);
		// Initialize the UndoRedoCommand procedurally.

	virtual pascal void DoIt();
		// tell the application to undo/redo.
	
	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields.
};


//----------------------------------------------------------------------------------------
// TAboutAppCommand: Tells the application to show its about box.
//----------------------------------------------------------------------------------------

class TAboutAppCommand : public TNoChangesCommand
{
public:
	virtual pascal void IAboutAppCommand(CmdNumber itsCmdNumber);
		// Initialize the AboutAppCommand procedurally.

	virtual pascal void DoIt();
		// tell the application to DoShowAboutApp.

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields.
};


//----------------------------------------------------------------------------------------
// TEventCommand: Retrieve Events and process them.
//----------------------------------------------------------------------------------------

class TEventCommand : public TNoChangesCommand
{
public:
	virtual pascal void IEventCommand(CmdNumber itsCmdNumber);
		// Initialize the EventCommand procedurally.

	virtual pascal void DoIt();
		// Retrieve and process an event

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields.
};


//----------------------------------------------------------------------------------------
// Globals defined by this unit
//----------------------------------------------------------------------------------------

extern TApplication* gApplication;		// the application object

extern Boolean gInitialized;			// Set to TRUE at the end of IApplication

extern AEAddressDesc gServerAddress;	// Address of the AppleEvent Server. Initialized
                                        // to send to ourselves.

#endif


