// UCPlusTool.h
// Copyright  1988-1991 by Apple Computer Inc.  All rights reserved.

//----------------------------------------------------------------------------------------
// Simple framework for an MPW tool. NOTE - tools cannot currently be built with -debug
//----------------------------------------------------------------------------------------

#ifndef __UCPLUSTOOL__
#define __UCPLUSTOOL__

#ifndef __PASCALDEFS__
#include "PascalDefs.h"
#endif

#ifndef __UOBJECT__
#include "UObject.h"
#endif

//----------------------------------------------------------------------------------------
// Constants
//----------------------------------------------------------------------------------------

extern const char* Version;
const unsigned long kErrorMarker = '### ';

// Keyword IDs. Negative numbers reserved for the framework
const short kwP = -1;
const short kwNoP = -2;
const short kwT = -3;
const short kwNoT = -4;
const short kwHelp = -5;


//----------------------------------------------------------------------------------------
// Forward and external classes
//----------------------------------------------------------------------------------------

class TAssociation;


//----------------------------------------------------------------------------------------
// TCPlusTool
//----------------------------------------------------------------------------------------

class TCPlusTool : public TObject
{
public:

	TAssociation* fKeyWordList;					// keywords to this command
	
	Str255 fProgName;							// Program's file name
	
	Boolean fInterrupted;						// True ==> interrupted (Cmd "." pressed)
	
	short fCursorCount;
	
	enum
	{											// for our spinning cursor
		RC_Normal, RC_ParmErrs, RC_DontMatch, RC_Abort
	} fRetCode;									// Return codes
	
	Boolean fProgress;							// true for progress request
	
	Boolean fTime;								// true for elapsed time request
	
	long fStartTicks;							// tickcount at start of tool
	
	long fStartDateTime;						// Date/ Time at start of tool
	
	short fArgc;								// number of command line arguments
	
	char** fArgv;								// command line arguments. If the tool is
												// written in pascal this should be NIL
	
	short fArgvIndex;							// current argument being processed

	virtual pascal void Initialize(void);
		// Puts a newly created TCPlusTool into a known state. Subclasses should override it and
		// call INHERITED. Typically used to put the object into a state where it can be
		// safely FREED.

	virtual pascal void ICPlusTool(short argc, char** argv);
		// If the tool is written in pascal, argv should be passed NIL. Otherwise you
        // should pass the argc and argv passed into your main.
		
	virtual pascal void InstallKeyWord(const Str255& keyword, short kw);
	
	virtual pascal Boolean LookupKeyword(const Str255& keyword, short& kw);
	
	virtual pascal void InstallKeyWords(void);
	
	virtual pascal void DoProcessFileArg(const Str255& arg);
	
	virtual pascal void DoProcessOptionArg(short kw);
	
	virtual pascal void DoShowUsage(void);
	
	virtual pascal void DoStartProgress(void);
	
	virtual pascal void DoToolAction(void);
	
	virtual pascal void GetNextArg(Str255& nextArg);
	
	virtual pascal void ProcessArg(const Str255& arg);
	
	virtual pascal void Run(void);
	
	virtual pascal void Stop(const Str255& msg);
	
	virtual pascal void SyntaxError(const Str255& suffix);
};


//----------------------------------------------------------------------------------------
// Unit intialization
//----------------------------------------------------------------------------------------

extern pascal void InitUCPlusTool();


//----------------------------------------------------------------------------------------
// Globals defined by this unit
//----------------------------------------------------------------------------------------

extern TCPlusTool* gCPlusTool;					// The tool

#endif


