// UClipboardMgr.h
// Copyright  1984-1991 by Apple Computer Inc. All rights reserved.

#ifndef __UCLIPBOARDMGR__
#define __UCLIPBOARDMGR__

#ifndef __PASCALDEFS__
#include <PascalDefs.h>
#endif

#ifndef __SCRAP__
#include <Scrap.h>
#endif

#ifndef __UEVTHANDLER__
#include <UEvtHandler.h>
#endif

//----------------------------------------------------------------------------------------
// Forward and external class declarations.
//----------------------------------------------------------------------------------------

class TView;
class TWindow;


//----------------------------------------------------------------------------------------
// TClipboardMgr: This object is responsible for interaction with the scrap manager.
//----------------------------------------------------------------------------------------

class TClipboardMgr : public TEvtHandler
{
public:
	
	ScrapStuff fOldScrapStuff;					// the last ScrapStuff record used in
                                                // tracking the scrap

	ScrapStuff fNewScrapStuff;					// the current ScrapStuff record used in
                                                // tracking the scrap

	ResType fPrefClipType;

	Boolean fGotClipType;						// True if we can paste the data in the
                                                // clipboard

	Boolean fClipClaimed;						// Used by PerformCommand & ClaimClipboard
                                                // to determine, if DoIt of a cut/copy cmd
                                                // fails, whether the Clipboard had
                                                // already been claimed by the new command
                                                // or not

	TView* fClipOrphanage;						// A view to represent the Clipboard when
                                                // the application can't

	TView* fClipUndoView;						// The view previously installed in the
                                                // Clipboard

	TView* fClipView;							// The view currently installed in the
                                                // Clipboard

	TWindow* fClipWindow;						// The window holding the Clipboard
                                                // display

	Boolean fClipWrittenToDeskScrap;			// True if the clipboard view has been
                                                // written to the desk scrap


	//------------------------------------------------------------------------------------
	// Initialize, I<Method>.
	//------------------------------------------------------------------------------------

	virtual pascal void Initialize();

	virtual pascal void IClipboardMgr();
	
	
	//------------------------------------------------------------------------------------
	// Rest of methods.
	//------------------------------------------------------------------------------------

	virtual pascal void Close();

	virtual pascal void Fields(TObject* obj);

	virtual pascal void Launch();

	virtual pascal long GetDataToPaste(Handle aDataHandle, ResType& dataType);
		// Returns the number of bytes in the paste data. Will signal Failure if it gets
        // an error.

	virtual pascal TWindow* MakeClipboardWindow();
		// Called by to make the clipboard window. ClipView is the view to be initially
        // installed in the clipboard--it is fClipOrphanage.

	virtual pascal TView* MakeViewForAlienClipboard();
		// Calls the application's MakeViewForAlienClipboard.If it returns a view then
        // that view is returned to our caller else it returns fClipOrphanage, which is
        // capable of displaying scrap of type TEXT or PICT. Override this to create a
        // view to handle other types of scrap data.

	virtual pascal OSErr PutDeskScrapData(ResType aResType, Handle aDataHandle);
		// Call this from your TCommand method 'WriteToDeskScrap' (for a Command which
        // changes the Clipboard) to write out data to the actual Desk Scrap. The return
        // code from the Scrap Manager is returned as the function value -- will be noErr
        // unless something went wrong. This procedure leaves aDataHandle UNLOCKED. Rather
        // than calling this, you can call the ToolBox routine PutScrap yourself

	virtual pascal void AbandonUndoClipboard();
		// Called when the undo clipboard is no longer needed. Calls
        // fClipUndoView.FreeFromClipboard and sets fClipUndoView to NIL.

	virtual pascal void AboutToLoseControl(Boolean convertClipboard);

	virtual pascal void AbsorbScrapStuff();
		// Called to retrieve the current InfoScrap record from low memory. This will be
        // used to determine if the scrap has changed.

	virtual pascal void CheckDeskScrap();
		// Checks to see if the desk scrap has changed by calling AbsorbScrapStuff and
        // comparing the current scrap change count with the previous change count. If the
        // scrap has changed, then the current clipboard becomes the undo clipboard and
        // ReadFromDeskScrap is called to read in the new desk scrap.

	virtual pascal void ClaimClipboard(TView* clipView);
		// Makes clipView the new view displayed by the Clipboard. Typically called by a
        // Cut or Copy command.

	virtual pascal void CanPaste(ResType aClipType);
		// Call this in your SetUpMenus code to register an ability to paste a particular
        // type of Clipboard data

	virtual pascal void DoSetupMenus();
		// Handles the show/hide clipboard command and enabling of paste

	virtual pascal void DoMenuCommand(CmdNumber aCmdNumber);
		// Handles the show/hide clipboard command

	virtual pascal void ReadFromDeskScrap();
		// Called at launch and when the desk scrap changes. Calls
        // MakeViewForAlienClipboard, then ClaimClipboard

	virtual pascal void RegainControl(Boolean checkClipboard);
		// Called when switching in or when leaving a desk accessory

	virtual pascal void SetClipView(TView* clipView);
		// Installs the given view into the clipboard window.

	virtual pascal void SwapClipViews();
		// Swaps the undo clipboard view with the current clipboard view. Used during an
        // Undo or Redo.

};


//----------------------------------------------------------------------------------------
// Definitions for externally available variables and functions.
//----------------------------------------------------------------------------------------

extern TClipboardMgr* gClipboardMgr;

extern pascal void InitUClipboardMgr();
	// called to initialize the clipboard manager

#endif


