// UDeskScrapView.h
// Copyright 1984-1991 Apple Computer, Inc.All rights reserved.

#ifndef __UDESKSCRAPVIEW__
#define __UDESKSCRAPVIEW__

#ifndef __PASCALDEFS__
#include <PascalDefs.h>
#endif

#ifndef __UGEOMETRY__
#include <UGeometry.h>
#endif

#ifndef __UVIEW__
#include <UView.h>
#endif

//----------------------------------------------------------------------------------------
// Forward and external class declarations
//----------------------------------------------------------------------------------------

class TDocument;


//----------------------------------------------------------------------------------------
// TDeskScrapView: The default view in the Clipboard, able to display only standard text
// and pictures.
//----------------------------------------------------------------------------------------

class TDeskScrapView : public TView
{
public:
	Boolean fHavePicture;						// are we holding a PICT

	Boolean fHaveText;							// is there any TEXT

	short fScrapCount;							// scrapCount of the scrap represented by
                                                // this view.

	Handle fDataHandle;							// if non-NIL, will be either a PicHandle
                                                // to my picture or a Handle to my Text


	//------------------------------------------------------------------------------------
	// Initialize and Free
	//------------------------------------------------------------------------------------

	virtual pascal void Initialize();
		// Puts a newly created TDeskScrapView into a known state. Subclasses should
        // override it and call INHERITED. Typically used to put the object into a state
        // where it can be safely FREED.

	virtual pascal void IDeskScrapView();
		// Initialize the deskscrap view procedurally.

	virtual pascal void IRes(TDocument* itsDocument,
							 TView* itsSuperView,
							 Ptr& itsParams);
		// Initialize the deskscrap view from a resource template.

	virtual pascal void Free();
		// Does nothing, don't ever want to free the ClipOrphange.


	//------------------------------------------------------------------------------------
	// Other methods
	//------------------------------------------------------------------------------------

	virtual pascal void CheckScrapContents();
		// Checks the contents of the deskscrap.

	virtual pascal void CalcMinSize(VPoint& minSize);
		// Responsible for calculating the correct size for the deskscrap view.

	virtual pascal void Draw(const VRect& area);
		// handles the drawing of the contents of the deskscrap, which by default is
        // either a PICT or text.

	virtual pascal void SuperViewChangedSize(const VPoint& delta, Boolean invalidate);
		// If our superview changes size then call AdjustSize & ForceRedraw.

	virtual pascal void WriteToDeskScrap();
		// This view represents data that is already written to the deskscrap, so this
        // does nothing.


	//------------------------------------------------------------------------------------
	// Inspecting
	//------------------------------------------------------------------------------------

	virtual pascal void GetInspectorName(Str255& inspectorName);
		// Used by the Inspector to get the name of this class.

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields.

};


//----------------------------------------------------------------------------------------
// Externally available global variables.
//----------------------------------------------------------------------------------------

extern Boolean gVarClipPicSize;
	// if TRUE, Pictures in the Clipboard are treated as variable size, depending on the
    // window size; if FALSE (default), then pictures in the Clipboard are drawn and
    // pasted ??? actual size

#endif


