// UDialog.h
// Copyright  1988-1991 by Apple Computer Inc.All rights reserved.

#ifndef __UDIALOG__
#define __UDIALOG__

#ifndef __PASCALDEFS__
#include <PascalDefs.h>
#endif

#ifndef __UGEOMETRY__
#include <UGeometry.h>
#endif

#ifndef __UCONTROL__
#include <UControl.h>
#endif

#ifndef __UTEVIEW__
#include <UTEView.h>
#endif

#ifndef __UCOMMAND__
#include <UCommand.h>
#endif

//----------------------------------------------------------------------------------------
// Forward and External classe declarations
//----------------------------------------------------------------------------------------

class TAssociation;
class TDocument;
class TTEView;
class TScroller;
class TToolboxEvent;
class TEditText;
class TDialogTEView;


//----------------------------------------------------------------------------------------
// Some constants
//----------------------------------------------------------------------------------------

const int kPreferColor = True;					// argument to IIcon

const int kControlOn = True;					// argument to ICheckBox and IRadio

const int kFlash = True;						// Constants for TDialogView.Dismiss.

const int kDontFlash = False;					// Constants for TDialogView.Dismiss.

const short kMaxTEWidth = 688;					// Maximum width for TEditText autoscroll

const short phInvalidValue = 300;				// Alert used when an invalid value is
                                                // entered
												
const short kDialogTEViewID = 300;				// The ID of the 'view' resource for the
                                                // "floating" TEView

const short kInvalidValueReasons = 300;			// String list used for invalid value
                                                // alert
												
const short kErrorHandled = -1;					// Don't post an alert, the user handled
                                                // it
												
const long kValidValue = 0;						// Not looked up in the string list.

const long kInvalidValue = 1;

const long kValueTooSmall = 2;

const long kValueTooLarge = 3;

const long kNonNumericCharacters = 4;

const long kTooManyCharacters = 5;

const long kNoOfDefaultReasons = 5;


typedef pascal void(* DoToEditTextType)(TEditText* , void* staticLink);
	// Interations procedure typedef


//----------------------------------------------------------------------------------------
// DialogViewTemplate
//----------------------------------------------------------------------------------------

struct DialogViewTemplate
{
	IDType defaultItem;
	IDType cancelItem;
};

typedef DialogViewTemplate* DialogViewTemplatePtr;


//----------------------------------------------------------------------------------------
// TDialogView: Mimics normal toolbox dialogs providing behavior for handling default
// buttons, param text, currently selected edit text and other behavior typically
// associated with dialogs.
//----------------------------------------------------------------------------------------

class TDialogView : public TView
{
public:
	IDType fDefaultItem;						// The identifier of the default item

	IDType fCancelItem;							// The identifier of item that cancels
                                                // dialog

	IDType fDismisser;							// ID of view that caused dismissal

	
	virtual pascal void Initialize();
		// Puts a newly created object into a known state. Subclasses should override it
        // and call INHERITED. Typically used to put the object into a state where it can
        // be safely FREED.

	virtual pascal void IDialogView(TDocument* itsDocument,
									TView* itsSuperView,
									const VPoint& itsLocation,
									const VPoint& itsSize,
									SizeDeterminer itsHSizeDet,
									SizeDeterminer itsVSizeDet,
									IDType itsDefItemID,
									IDType itsCancelItemID);
		// Initialization method for TDialogView.

	virtual pascal void IRes(TDocument* itsDocument,
							 TView* itsSuperView,
							 Ptr& itsParams);
		// For initializing a dialog view from a resource template.

	virtual pascal TObject* Clone();
		// calls INHERITED Clone and then clones owned objects

	virtual pascal void WRes(ViewRsrcHandle theResource, Ptr& itsParams);
		// For writing a dialog view out to a resource.

	virtual pascal void WriteRes(ViewRsrcHandle theResource, Ptr& itsParams);
		// For writing a dialog view to a resource, works in conjunction with tbe previous
        // method.

	virtual pascal void Free();
		// !!! Calls inherited Free.

	virtual pascal Boolean CanDismiss(IDType dismissing);
		// Used to check whether the dialog view can be dismissed or not by checking if
        // the view is enabled, then check if the currently selected edit text is valid.

	virtual pascal void Close();
		// Called to close a dialog view.

	virtual pascal void DismissDialog(IDType dismisser);
		// The method responsible for actually dismissing a dialog.

	virtual pascal void DoCommandKey(TToolboxEvent* event);
		// Called when a keyDown event is received and the command key is down.

	virtual pascal void DoKeyCommand(TToolboxEvent* event);
		// This method handles all key presses other than a command key press, which is
        // handled by the previous method.

	virtual pascal void DoOpen();
		// If when opening the dialog the current target is an edit text field of the
        // dialog, then select it.

	virtual pascal Boolean DeselectCurrentEditText();
		// Deselects the current edit text, this is called when tabbing between edit text
        // fields.

	virtual pascal void DoSelectEditText(TEditText* theEditText, Boolean selectChars);
		// Selects theEditText, is selectChars then all the characters are selected
        // otherwise the insertion point is merely placed at the start of the field.


	virtual pascal TEditText* GetCurrentEditText();
		// Returns the currently selected EditText, or NIL

	virtual pascal void EachEditText(DoToEditTextType DoToEditText, void* staticLink);
		// Iterates over all the edit text items in the dialog calling DoToEditText for
        // each item.

	virtual pascal TView* GetDialogView();
		// Returns a reference to the dialog view.

	virtual pascal void DoEvent(EvtNumber eventNumber,
								TEvtHandler* source,
								TEvent* event);
		// If the default or cancel key are pressed then dismisses the dialog, otherwise
        // passes the message along the chain. If the hit was an edit text item then
        // deselects the item. Otherwise if the hit is on some other control and it
        // dismisses the dialog then dismiss it.

	virtual pascal void Open();
		// Called to open a dialog.

	virtual pascal IDType PoseModally();
		// Displays the dialog in a modal fashion waiting for a response, which is
        // returned as the IDType of the button hit that causes the dialog to be
        // dismissed.

	virtual pascal void SelectEditText(IDType itsIdentifier, Boolean selectChars);
		// If itsIdentifier can be found then the field is selected, selectChars if true
        // means that all the text will be selected as opposed to the insertion point
        // merely being placed in the field.

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields.
};


//----------------------------------------------------------------------------------------
// ButtonTemplate
//----------------------------------------------------------------------------------------

struct ButtonTemplate
{
	Str255 itsLabel;							// Actually, variable length
};

typedef ButtonTemplate* ButtonTemplatePtr;


//----------------------------------------------------------------------------------------
// TButton: Mimics a typical toolbox button as used in a dialog.
//----------------------------------------------------------------------------------------

class TButton : public TCtlMgr
{
public:
	virtual pascal void Initialize();
		// Puts a newly created object into a known state. Subclasses should override it
        // and call INHERITED. Typically used to put the object into a state where it can
        // be safely FREED.

	virtual pascal void IButton(TView* itsSuperView,
								const VPoint& itsLocation,
								const VPoint& itsSize,
								SizeDeterminer itsHSizeDet,
								SizeDeterminer itsVSizeDet,
								const Str255& itsLabel);
		// Initializes the button setting the fDefChoice field to mButtonHit.

	virtual pascal void IRes(TDocument* itsDocument,
							 TView* itsSuperView,
							 Ptr& itsParams);
		// Initialize the view from a resource template.

	virtual pascal TObject* Clone();
		// calls INHERITED Clone and then clones owned objects

	virtual pascal void WRes(ViewRsrcHandle theResource, Ptr& itsParams);
		// Write this object out as a view resource.

	virtual pascal void WriteRes(ViewRsrcHandle theResource, Ptr& itsParams);
		// Set up the type and signature of this object and call WRes.

	virtual pascal void DoEvent(EvtNumber eventNumber,
								TEvtHandler* source,
								TEvent* event);
		// If the hit is mButtonHit then flash the button provided you didn't hit
        // yourself.

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields.
};


//----------------------------------------------------------------------------------------
// CheckBoxTemplate
//----------------------------------------------------------------------------------------

struct CheckBoxTemplate
{
	Boolean isOn;
	Str255 itsLabel;							// Actually, variable length
};

typedef CheckBoxTemplate* CheckBoxTemplatePtr;


//----------------------------------------------------------------------------------------
// TCheckBox: Provides the functionality of a toolbox checkbox.
//----------------------------------------------------------------------------------------

class TCheckBox : public TCtlMgr
{
public:
	virtual pascal void Initialize();
		// Puts a newly created object into a known state. Subclasses should override it
        // and call INHERITED. Typically used to put the object into a state where it can
        // be safely FREED.

	virtual pascal void ICheckBox(TView* itsSuperView,
								  const VPoint& itsLocation,
								  const VPoint& itsSize,
								  SizeDeterminer itsHSizeDet,
								  SizeDeterminer itsVSizeDet,
								  const Str255& itsLabel,
								  Boolean isTurnedOn);
		// Initialize the checkbox setting fDefChoice to mCheckBoxHit.

	virtual pascal void IRes(TDocument* itsDocument,
							 TView* itsSuperView,
							 Ptr& itsParams);
		// Initialize the view from a resource template.

	virtual pascal TObject* Clone();
		// calls INHERITED Clone and then clones owned objects

	virtual pascal void WRes(ViewRsrcHandle theResource, Ptr& itsParams);
		// Write this object out as a view resource.

	virtual pascal void WriteRes(ViewRsrcHandle theResource, Ptr& itsParams);
		// Set up the type and signature of this object and call WRes.

	virtual pascal void DoEvent(EvtNumber eventNumber,
								TEvtHandler* source,
								TEvent* event);
		// If the hit is mCheckBoxHit then toggle the checkbox.

	virtual pascal Boolean IsOn();
		// Is the checkbox currently checked.

	virtual pascal void SetState(Boolean state, Boolean redraw);
		// Sets the state of the checkbox to state and if redraw is TRUE redraws it.

	virtual pascal void Toggle(Boolean redraw);
		// Toggles the state of the checkbox redraw it if redraw is TRUE.

	virtual pascal void ToggleIf(Boolean matchState, Boolean redraw);
		// Toggles the state of the checkbox if matchState is TRUE redrawing if redraw is
        // TRUE.

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields.
};


//----------------------------------------------------------------------------------------
// RadioTemplate
//----------------------------------------------------------------------------------------

struct RadioTemplate
{
	Boolean isOn;
	Str255 itsLabel;							// Actually, variable length
};

typedef RadioTemplate* RadioTemplatePtr;


//----------------------------------------------------------------------------------------
// TRadio: Provides the functionality of a toolbox radio button.
//----------------------------------------------------------------------------------------

class TRadio : public TCtlMgr
{
public:
	virtual pascal void Initialize();
		// Puts a newly created object into a known state. Subclasses should override it
        // and call INHERITED. Typically used to put the object into a state where it can
        // be safely FREED.

	virtual pascal void IRadio(TView* itsSuperView,
							   const VPoint& itsLocation,
							   const VPoint& itsSize,
							   SizeDeterminer itsHSizeDet,
							   SizeDeterminer itsVSizeDet,
							   const Str255& itsLabel,
							   Boolean isTurnedOn);
		// Initalize the radio button procedurally.

	virtual pascal void IRes(TDocument* itsDocument,
							 TView* itsSuperView,
							 Ptr& itsParams);
		// Initialize the view from a resource template.

	virtual pascal TObject* Clone();
		// calls INHERITED Clone and then clones owned objects

	virtual pascal void WRes(ViewRsrcHandle theResource, Ptr& itsParams);
		// Write the object out as a view resource.

	virtual pascal void WriteRes(ViewRsrcHandle theResource, Ptr& itsParams);
		// Set up the type and signature of this object and call WRes.

	virtual pascal void DoEvent(EvtNumber eventNumber,
								TEvtHandler* source,
								TEvent* event);
		// If the hit is mRadioHit then toggle the radio button.

	virtual pascal Boolean IsOn();
		// Is the radio button currently on.

	virtual pascal void SetState(Boolean state, Boolean redraw);
		// Sets the state of the radio button to state, redrawing if redraw is TRUE.

	virtual pascal void Toggle(Boolean redraw);
		// Toggles the state of the radio button, redrawing if redraw is TRUE.

	virtual pascal void ToggleIf(Boolean matchState, Boolean redraw);
		// Only toggles the state of the radio button if matchState is TRUE, redrawing if
        // redraw is TRUE.

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields.
};


//----------------------------------------------------------------------------------------
// ClusterTemplate
//----------------------------------------------------------------------------------------

struct ClusterTemplate
{
	Str255 itsLabel;							// Actually, variable length
};

typedef ClusterTemplate* ClusterTemplatePtr;


//----------------------------------------------------------------------------------------
// TCluster: Provides a mechanism whereby radio buttons can be grouped in clusters, such
// that when one of the buttons is selected the others are all deselected. Also can be
// utilized for providing labelled adornments around groups of dialog items.
//----------------------------------------------------------------------------------------

class TCluster : public TControl
{
public:
	short fRsrcID;								// resource ID of the cluster's label

	short fIndex;								// currently selected item

	StringHandle fDataHandle;					// the cluster's label


	virtual pascal void Initialize();
		// Puts a newly created object into a known state. Subclasses should override it
        // and call INHERITED. Typically used to put the object into a state where it can
        // be safely FREED.

	virtual pascal void ICluster(TView* itsSuperView,
								 const VPoint& itsLocation,
								 const VPoint& itsSize,
								 SizeDeterminer itsHSizeDet,
								 SizeDeterminer itsVSizeDet,
								 short itsRsrcID,
								 short itsIndex);
		// Initialize the cluster procedurally.

	virtual pascal void IRes(TDocument* itsDocument,
							 TView* itsSuperView,
							 Ptr& itsParams);
		// Initialize the view template.

	virtual pascal TObject* Clone();
		// calls INHERITED Clone and then clones owned objects

	virtual pascal void WRes(ViewRsrcHandle theResource, Ptr& itsParams);
		// Write the object out as a view resource.

	virtual pascal void WriteRes(ViewRsrcHandle theResource, Ptr& itsParams);
		// Set up the type and signature of this object and call WRes.

	virtual pascal void Free();
		// Release the label and call inherited free.

	virtual pascal void Draw(const VRect& area);
		// Frame the cluster if there is a label, then draw the label.

	virtual pascal void DrawLabel(const VRect& area);
		// Draw the label.

	virtual pascal void DoEvent(EvtNumber eventNumber,
								TEvtHandler* source,
								TEvent* event);
		// Responsible for resetting the radio buttons so that the last selected one is
        // the only one selected.

	virtual pascal void GetLabel(Str255& theLabel);
		// Returns in theLable the text in the clusters label.

	virtual pascal void ReleaseLabel();
		// Releases fDataHandle which is a string handle containing the text for the
        // label.

	virtual pascal IDType ReportCurrent();
		// Returns the IDType of the currently selected item.

	virtual pascal void SetLabel(const Str255& theLabel, Boolean redraw);
		// Sets the clusters label to be theLabel, redrawing if redraw is TRUE.

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields.
};


//----------------------------------------------------------------------------------------
// IconTemplate
//----------------------------------------------------------------------------------------

struct IconTemplate
{
	Boolean preferColor;
	short rsrcID;
};

typedef IconTemplate* IconTemplatePtr;


//----------------------------------------------------------------------------------------
// TIcon: Provides an icon that can behave as a button.
//----------------------------------------------------------------------------------------

class TIcon : public TControl
{
public:
	Boolean fPreferColor;						// tells us how to GET the icon - GetCIcon
                                                // or GetIcon

	Boolean fIsColor;							// tells us how to DRAW or FREE the icon

	short fRsrcID;

	Handle fDataHandle;


	virtual pascal void Initialize();
		// Puts a newly created object into a known state. Subclasses should override it
        // and call INHERITED. Typically used to put the object into a state where it can
        // be safely FREED.

	virtual pascal void IIcon(TView* itsSuperView,
							  const VPoint& itsLocation,
							  const VPoint& itsSize,
							  SizeDeterminer itsHSizeDet,
							  SizeDeterminer itsVSizeDet,
							  short itsRsrcID,
							  Boolean preferColor);
		// Initialize the view template.

	virtual pascal void IRes(TDocument* itsDocument,
							 TView* itsSuperView,
							 Ptr& itsParams);
		// Initialize the view template.

	virtual pascal TObject* Clone();
		// calls INHERITED Clone and then clones owned objects

	virtual pascal void WRes(ViewRsrcHandle theResource, Ptr& itsParams);
		// Write the object out as a view resource.

	virtual pascal void WriteRes(ViewRsrcHandle theResource, Ptr& itsParams);
		// Set up the type and signature of this object and call WRes.

	virtual pascal void Free();
		// Release the icon then call inherited Free.

	virtual pascal void Draw(const VRect& area);
		// Resonsible for drawing the icon and any required adornments.

	virtual pascal void ReleaseIcon();
		// Releases the handle to the icon.

	virtual pascal void SetIcon(Handle theIcon, Boolean redraw);
		// Sets fDataHandle to theIcon, redrawing if redraw is TRUE.

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields.
};


//----------------------------------------------------------------------------------------
// SmallIconTemplate
//----------------------------------------------------------------------------------------

struct SmallIconTemplate
{
	short rsrcID;
};

typedef struct SmallIconTemplate *SmallIconTemplatePtr;


//----------------------------------------------------------------------------------------
// TSmallIcaon
//----------------------------------------------------------------------------------------

class TSmallIcon : public TControl
{
public:
	short fRsrcID;

	Handle fDataHandle;

	virtual pascal void Initialize();
		// Puts a newly created object into a known state. Subclasses should override it
        // and call INHERITED. Typically used to put the object into a state where it can
        // be safely FREED.

	virtual pascal void ISmallIcon(TView* itsSuperView,
								   const VPoint& itsLocation,
								   const VPoint& itsSize,
								   SizeDeterminer itsHSizeDet,
								   SizeDeterminer itsVSizeDet,
								   short itsRsrcID);

	virtual pascal void IRes(TDocument* itsDocument,
							 TView* itsSuperView,
							 Ptr& itsParams);

	virtual pascal TObject* Clone();
		// calls INHERITED Clone and then clones owned objects

	virtual pascal void WRes(ViewRsrcHandle theResource, Ptr& itsParams);

	virtual pascal void WriteRes(ViewRsrcHandle theResource, Ptr& itsParams);

	virtual pascal void Draw(const VRect& area);

	virtual pascal void ReleaseSmallIcon();

	virtual pascal void SetSmallIcon(short theSmallIcon, Boolean redraw);

	virtual pascal void Fields(TObject* obj);
};


//----------------------------------------------------------------------------------------
// PatternTemplate
//----------------------------------------------------------------------------------------

struct PatternTemplate
{
	Boolean preferColor;
	short rsrcID;
};

typedef PatternTemplate* PatternTemplatePtr;


//----------------------------------------------------------------------------------------
// TPattern: Provides an object that can display a pattern and can also behave as a button
// if required.
//----------------------------------------------------------------------------------------

class TPattern : public TControl
{
public:
	Boolean fPreferColor;						// tells us how to GET the pattern

	Boolean fIsColor;							// tells us how to DRAW or FREE the
                                                // pattern

	short fRsrcID;

	Handle fDataHandle;


	virtual pascal void Initialize();
		// Puts a newly created object into a known state. Subclasses should override it
        // and call INHERITED. Typically used to put the object into a state where it can
        // be safely FREED.

	virtual pascal void IPattern(TView* itsSuperView,
								 const VPoint& itsLocation,
								 const VPoint& itsSize,
								 SizeDeterminer itsHSizeDet,
								 SizeDeterminer itsVSizeDet,
								 short itsRsrcID,
								 Boolean preferColor);
		// Initialize the pattern procedurally.

	virtual pascal void IRes(TDocument* itsDocument,
							 TView* itsSuperView,
							 Ptr& itsParams);
		// Initialize the view template.

	virtual pascal TObject* Clone();
		// calls INHERITED Clone and then clones owned objects

	virtual pascal void WRes(ViewRsrcHandle theResource, Ptr& itsParams);
		// Write the object out as a view resource.

	virtual pascal void WriteRes(ViewRsrcHandle theResource, Ptr& itsParams);
		// Set up the type and signature of this object and call WRes.

	virtual pascal void Free();
		// Release the pattern then call inherited Free.

	virtual pascal void Draw(const VRect& area);
		// Responsible for drawing the pattern and any required adornments.

	virtual pascal void ReleasePattern();
		// Releases the pattern by disposing fDataHandle.

	virtual pascal void SetPattern(Handle thePattern, Boolean redraw);
		// Sets fDataHandle to thePattern, redrawing if redraw is TRUE.

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields.
};


//----------------------------------------------------------------------------------------
// PictureTemplate
//----------------------------------------------------------------------------------------

struct PictureTemplate
{
	short rsrcID;
};

typedef PictureTemplate* PictureTemplatePtr;


//----------------------------------------------------------------------------------------
// TPicture: Provides an object that can display a PICT and if required behave like a
// button.
//----------------------------------------------------------------------------------------

class TPicture : public TControl
{
public:
	short fRsrcID;

	PicHandle fDataHandle;


	virtual pascal void Initialize();
		// Puts a newly created object into a known state. Subclasses should override it
        // and call INHERITED. Typically used to put the object into a state where it can
        // be safely FREED.

	virtual pascal void IPicture(TView* itsSuperView,
								 const VPoint& itsLocation,
								 const VPoint& itsSize,
								 SizeDeterminer itsHSizeDet,
								 SizeDeterminer itsVSizeDet,
								 short itsRsrcID);
		// Initialize the picture procedurally.

	virtual pascal void IRes(TDocument* itsDocument,
							 TView* itsSuperView,
							 Ptr& itsParams);
		// Initialize the view from a resource template.

	virtual pascal TObject* Clone();
		// calls INHERITED Clone and then clones owned objects

	virtual pascal void WRes(ViewRsrcHandle theResource, Ptr& itsParams);
		// Write the object out as a view resource.

	virtual pascal void WriteRes(ViewRsrcHandle theResource, Ptr& itsParams);
		// Set up the type and signature of this object and call WRes.

	virtual pascal void Free();
		// Release the picture then call inherited Free.

	virtual pascal void Draw(const VRect& area);
		// Responsible for drawing the PICT and any required adornments.

	virtual pascal void ReleasePicture();
		// Releases the PICT by disposing of fDataHandle.

	virtual pascal void SetPicture(PicHandle thePicture, Boolean redraw);
		// Sets fDataHandle to thePicture, redrawing if redraw is TRUE.

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields.
};


//----------------------------------------------------------------------------------------
// StaticTextTemplate
//----------------------------------------------------------------------------------------

struct StaticTextTemplate
{
	short just;
	Str255 data;								// Actually, variable length
};

typedef StaticTextTemplate* StaticTextTemplatePtr;


//----------------------------------------------------------------------------------------
// TStaticText: Provides an object that represents a toolbox static text item.
//----------------------------------------------------------------------------------------

class TStaticText : public TControl
{
public:
	short fRsrcID;								// ID of the string list that the string
                                                // is in.

	short fIndex;								// Index number for the string from the
                                                // above list.

	StringHandle fDataHandle;					// A handle to the actual string.

	short fJust;								// Jusitification of the text in the
                                                // field.

	Boolean fAutoWrap;							// TRUE to word wrap lines at the view
                                                // boundary otherwise lines wrap at
                                                // Carriage return.


	virtual pascal void Initialize();
		// Puts a newly created object into a known state. Subclasses should override it
        // and call INHERITED. Typically used to put the object into a state where it can
        // be safely FREED.

	virtual pascal void IStaticText(TView* itsSuperView,
									const VPoint& itsLocation,
									const VPoint& itsSize,
									SizeDeterminer itsHSizeDet,
									SizeDeterminer itsVSizeDet,
									short itsRsrcID,
									short itsIndex);
		// Initialize the static text field procedurally.

	virtual pascal void IRes(TDocument* itsDocument,
							 TView* itsSuperView,
							 Ptr& itsParams);
		// Initialize the view template.

	virtual pascal TObject* Clone();
		// calls INHERITED Clone and then clones owned objects

	virtual pascal void WRes(ViewRsrcHandle theResource, Ptr& itsParams);
		// Write the object out as a view resource.

	virtual pascal void WriteRes(ViewRsrcHandle theResource, Ptr& itsParams);
		// Set up the type and signature of this object and call WRes.

	virtual pascal void Free();
		// Releases the text and then calls inherited Free.

	virtual pascal void DoSubstitution(Str255& theText);
		// Calls ReplaceText so that any paramTxt items can be handled if needed.

	virtual pascal void Draw(const VRect& area);
		// Draws the text by calling ImageText.

	virtual pascal void GetText(Str255& theText);
		// If fDataHandle not NIL then returns the text, otherwise returns the null
        // string.

	virtual pascal void ImageText(Ptr text,
								  long length,
								  const VRect& box,
								  short just);
		// Draws the text.

	virtual pascal void ChangeWrap(Boolean newAutoWrap, Boolean redraw);
		// Changes auto-wrapping behavior, redrawing if requested

	virtual pascal void ReleaseText();
	// Releases the text by disposing of fDataHandle.

	virtual pascal void SetJustification(short theJust, Boolean redraw);
		// Sets the justification of the text to theJust, redrawing if required.

	virtual pascal void SetText(const Str255& theText, Boolean redraw);
		// Sets the text to theText, redrawing if required.

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields.
};


//----------------------------------------------------------------------------------------
// EditTextTemplate
//----------------------------------------------------------------------------------------

struct EditTextTemplate
{
	short maxChars;
	ControlCharSet controlChars;
};

typedef EditTextTemplate* EditTextTemplatePtr;


//----------------------------------------------------------------------------------------
// TEditText: Implements an edit text field.
//----------------------------------------------------------------------------------------

class TEditText : public TStaticText
{
public:
	short fMaxChars;

	TDialogTEView* fTEView;

	ControlCharSet fControlChars;				// Control chars accepted in text


	virtual pascal void Initialize();
		// Puts a newly created object into a known state. Subclasses should override it
        // and call INHERITED. Typically used to put the object into a state where it can
        // be safely FREED.

	virtual pascal void IEditText(TView* itsSuperView,
								  const VPoint& itsLocation,
								  const VPoint& itsSize,
								  short itsMaxChars);
		// Initialize the edit text item procedurally.

	virtual pascal void IRes(TDocument* itsDocument,
							 TView* itsSuperView,
							 Ptr& itsParams);
		// Initialize the view from a resource template.

	virtual pascal TObject* Clone();
		// calls INHERITED Clone and then clones owned objects

	virtual pascal void WRes(ViewRsrcHandle theResource, Ptr& itsParams);
		// Write the object out as a view resource.

	virtual pascal void WriteRes(ViewRsrcHandle theResource, Ptr& itsParams);
		// Set up the type and signature of this object and call WRes.

	virtual pascal void Free();
		// Un-installs the floating TEView, sets fTEView to NIL then calls inherited Free.

	virtual pascal void DoSubstitution(Str255& theText);
		// Default is to do nothing.

	virtual pascal void GetText(Str255& theText);
		// If TEView not NIL then returns the extracted text, otherwise calls inherited GetText.

	virtual pascal void ImageText(Ptr text,
								  long length,
								  const VRect& box,
								  short just);
		// Sets up the text to the parameters passed. Text is drawn in a 'box' , the text
        // is 'length' long.

	virtual pascal void BecameWindowTarget();			// Override
	virtual pascal void ResignedWindowTarget();			// Override
	virtual pascal void BecameApplicationTarget();		// Override
	virtual pascal void ResignedApplicationTarget();	// Override
	
	virtual pascal Boolean WantToBecomeTarget();

	virtual pascal long WillingToResignTarget();
		// !!! */

	virtual pascal void RestartEdit(const Str255& restartText);
		// This method resets the text of the view, as well as selecting the text. We go
        // to some extra trouble to ensure that this method doesn't do any drawing,
        // defering to window update time. The reasoning is that this method usually gets
        // called right after displaying an alert. The alert may have obscured this view,
        // thus including it in the window's update region. If we drew in this method we
        // would get flashing as this view would get redraw at update time. Unfortunately
        // TextEdit isn't very cooperative so we set the clipping to an empty rect to
        // ensure that TEActivate's drawing isn't seen.

	virtual pascal void ChangeWrap(Boolean newAutoWrap, Boolean redraw);
		// Changes auto-wrapping behavior, redrawing if requested.Forwards into the
        // floating TE if necessary.

	virtual pascal void SetJustification(short theJust, Boolean redraw);
		// Sets the justification of the text to theJust, redrawing is required.

	virtual pascal void SetSelection(short selStart,
									 short selEnd,
									 Boolean redraw);
		// Sets the selection in the field starting at selStart and ending at selEnd,
        // redrawing if required.

	virtual pascal void SetText(const Str255& theText, Boolean redraw);
		// Sets the displayed text to theText, redrawing if required.

	virtual pascal void StartEdit(Boolean selectChars);
		// Install the selection, revealing the selection if required.

	virtual pascal void MakeVisible();
		// Reveal the EditText, if scrolled out of view

	virtual pascal Boolean Deselect();
		// Valdates and and deselects the edit text, if possible

	virtual pascal void StopEdit();
		// Extracts the string from the TEView, then clears out the TEView before calling
        // SELF.SetText.

	virtual pascal void InstallFloatingTEView(Boolean selectChars);
		// Attach the "floating" TDialogTEView to this EditText

	virtual pascal void RemoveFloatingTEView();
		// Remove the "floating" TDialogTEView from this EditText.

	virtual pascal long Validate();
		// By default just returns TRUE.

	virtual pascal void TargetValidationFailed(long reason);	// Override
		// Displays an alert when validation fails. */

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields.
};


//----------------------------------------------------------------------------------------
// NumberTextTemplate
//----------------------------------------------------------------------------------------

struct NumberTextTemplate
{
	long value;
	long minimum;
	long maximum;
};

typedef NumberTextTemplate* NumberTextTemplatePtr;


//----------------------------------------------------------------------------------------
// TNumberText: This subclass of TEditText that allows the handling of integer numbers.
// Range checking will be performed on the numbers entered to see if they are in the range
// fMinimum to fMaximum.
//----------------------------------------------------------------------------------------

class TNumberText : public TEditText
{
public:
	long fMinimum;

	long fMaximum;


	virtual pascal void Initialize();
		// Puts a newly created object into a known state. Subclasses should override it
        // and call INHERITED. Typically used to put the object into a state where it can
        // be safely FREED.

	virtual pascal void INumberText(TView* itsSuperView,
									const VPoint& itsLocation,
									const VPoint& itsSize,
									long itsValue,
									long itsMinimum,
									long itsMaximum);
		// Initialize the number text item procedurally.

	virtual pascal void IRes(TDocument* itsDocument,
							 TView* itsSuperView,
							 Ptr& itsParams);
		// Initialize the view template.

	virtual pascal void WRes(ViewRsrcHandle theResource, Ptr& itsParams);
		// Write the object out as a view resource.

	virtual pascal void WriteRes(ViewRsrcHandle theResource, Ptr& itsParams);
		// Set up the type and signature of this object and call WRes.

	virtual pascal long GetValue();
		// Returns the current value by getting the string and converting it to a number.

	virtual pascal void SetValue(long newValue, Boolean redraw);
		// Sets the current value by converting the number to a string and then calling SetText.

	virtual pascal long Validate();
		// Does a range check on the number and if it is out of range returns the
        // appropriate error code.

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields.
};


//----------------------------------------------------------------------------------------
// TDialogTEView: Provides a floating TEView that is placed over edit & number text fields
// when entering text.
//----------------------------------------------------------------------------------------

class TDialogTEView : public TTEView
{
public:
	TEditText* fEditText;						// The edit text field to which the
                                                // floating TEView is currently tied.

	TScroller* fScroller;						// The scroller that I am in for
                                                // mouseability.


	virtual pascal void Initialize();
		// Puts a newly created object into a known state. Subclasses should override it
        // and call INHERITED. Typically used to put the object into a state where it can
        // be safely FREED.

	virtual pascal void IDialogTEView(TDocument* itsDocument,
									  TView* itsSuperView,			
									  const VPoint& itsLocation,
									  const VPoint& itsSize,
									  SizeDeterminer itsHDeterminer,
									  SizeDeterminer itsVDeterminer,
									  const VRect& itsInset,
									  const TextStyle& itsTextStyle,
									  short itsJustification,
									  Boolean itsStyleType,
									  Boolean itsAutoWrap);
		// For Initialization of procedurally created TEView.

	virtual pascal void IRes(TDocument* itsDocument,
							 TView* itsSuperView,
							 Ptr& itsParams);
		// Initialize a TTEView via a 'view' resource.

	virtual pascal TObject* Clone();
		// calls INHERITED Clone and then clones owned objects

	virtual pascal void Free();
		// Free associated components and then free self.

	virtual pascal void InstallEditText(TEditText* theEditText, Boolean selectChars);
		// Installs the EditText adding SELF as a subview to EditText, then gets the text
        // from the EditText item and installs it in SELF.

	virtual pascal void ComputeSize(VPoint& newSize);
		// accounts for case in which horizontal determiner is sizeVariable should be
        // added to TTEView !!!

	virtual pascal void BecameWindowTarget();	// Override
	virtual pascal void ResignedWindowTarget();	// Override
	virtual pascal void BecameApplicationTarget();	// Override
	virtual pascal void ResignedApplicationTarget();	// Override

	virtual pascal long WillingToResignTarget();	// Override
		// !!!

	virtual pascal void TargetValidationFailed(long reason);	// Override

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields.
	};


//----------------------------------------------------------------------------------------
// TFloatingTEManager: Acts as a "server" for the "floating" TDialogTEViews installed in
// EditTexts. A one-view cache is maintained
//----------------------------------------------------------------------------------------

class TFloatingTEManager : public TObject
{
public:
	TDialogTEView* fCache;

	
	virtual pascal void Initialize();
		// Puts a newly created object into a known state. Subclasses should override it
        // and call INHERITED. Typically used to put the object into a state where it can
        // be safely FREED.

	virtual pascal void Free();
		// Frees the cache, and calls INHERITED Free

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields.

	virtual pascal TDialogTEView* AquireFloatingTE();
		// Returns a "floating" TDialogTEView to install in an EditText. If the cache is
        // empty, one is created

	virtual pascal TDialogTEView* MakeDialogTEView();

	virtual pascal void ReleaseFloatingTE(TDialogTEView* aDialogTEView);
		// Places the specified TDialogTEView in the cache, or frees it if the cache is
        // full
};


//----------------------------------------------------------------------------------------
// Externally available function declaration.
//----------------------------------------------------------------------------------------

extern pascal void InitUDialog();
	// This routine must be called if UDialog is used so that all the necessary
    // initialization can be performed. Sets the global flag 'gUDialogInitialized' after
    // the initalization is performed.

extern pascal TDialogTEView* AquireFloatingTEView();
	// Called by EditTexts to obtain a "floating" TDialogTEView to install. Procedural
    // interface to TFloatingTEManager.AquireFloatingTE

extern pascal void ReleaseFloatingTEView(TDialogTEView* aDialogTEView);
	// Called by EditTexts when deselecting. Procedural interface to
    // TFloatingTEManager.ReleaseFloatingTE

extern pascal void MAParamText(const Str255& keyStr, const Str255& valueStr);
	// Used to create paramTxt items for a dialog. Builds an association with keyStr and
    // valueStr. All instances of keyStr in the dialog will be replaced with valueStr when
    // the dialog is displayed.

extern pascal void MAReplaceText(Str255& theText);
	// If there are any param text items in the theText passed to this method then they
    // are replaced with the appropriate values.


//----------------------------------------------------------------------------------------
// Externally available variable declarations.
//----------------------------------------------------------------------------------------

extern TFloatingTEManager* gFloatingTEManager;

#endif


