// UDocument.h
// Copyright  1984-1991 by Apple Computer Inc.	All rights reserved.

#ifndef __UDOCUMENT__
#define __UDOCUMENT__

#ifndef __PASCALDEFS__
#include <PascalDefs.h>
#endif

#ifndef __UEVTHANDLER__
#include <UEvtHandler.h>
#endif

#ifndef __UCOMMAND__
#include <UCommand.h>
#endif

#ifndef __UIterator__
#include <UIterator.h>
#endif


//----------------------------------------------------------------------------------------
// Forward and External classe declarations
//----------------------------------------------------------------------------------------

class TDesignator;
class TFile;
class TList;
class TNoChangesCommand;
class TPrintHandler;
class TView;
class TWindow;
class TDesignator;
class TStream;


typedef pascal void(* DoToWindType)(TWindow* aWindow, void* staticLink);
	// Iteration typedefs


//----------------------------------------------------------------------------------------
// CWindowIterator
//----------------------------------------------------------------------------------------

class CWindowIterator : public CObjectIterator
{
public:
	CWindowIterator(TDocument* itsDocument,
					ArrayIndex itsLowBound, ArrayIndex itsHighBound,
					Boolean itsForward);

	CWindowIterator(TDocument* itsDocument, Boolean itsForward);

	CWindowIterator(TDocument* itsDocument);
	
	TWindow* CurrentWindow(void);
		// returns the current window
	
	TWindow* FirstWindow(void);
		// return the first window in the iteration

	TWindow* NextWindow(void);
		// advances the iteration and then returns the window
};


//----------------------------------------------------------------------------------------
// TDocument: Corresponds to a Finder document. Manages document data in files and in main
// memory.
//----------------------------------------------------------------------------------------

class TDocument : public TEvtHandler
{
public:
	Str255 fTitle;								// Title of the document

	TList* fWindowList;							// list of windows belonging to this
                                                // document

	TList* fViewList;							// list of views belonging to document

	long fChangeCount;							// master count of changes since last Save

	Boolean fSavePrintInfo;						// if TRUE for a document saved on disk,
                                                // the 'print info' record of the
                                                // fDocPrintHandler will be written out to
                                                // the data fork before other writing
                                                // takes place

	Boolean fSharePrintInfo;					// if TRUE, then all printHandlers
                                                // associated with views belonging to the
                                                // same document will share the same
                                                // 'print info' record

	Handle fPrintInfo;							// if non-NIL, this is a handle to a
                                                // 120-byte 'print info' record

	Boolean fReopenAlert;						// whether to give an alert if user
                                                // reopens doc

	Boolean fCommitOnSave;						// commit the last command before saving
                                                // the document, if it affected the
                                                // document. This defaults to TRUE, but
                                                // carefully written applications could
                                                // set this FALSE.

	TDesignator* fUserSelection;				// When there is a single user-selection
                                                // for a document this is it.


	virtual pascal void Initialize();
		// Puts a newly created TDocument into a known state. Subclasses should override
        // it and call INHERITED. Typically used to put the object into a state where it
        // can be safely FREED.

	virtual pascal void IDocument();
		// Initialization method for TDocument.

	virtual pascal void Free();
		// This does not call FreeData by default, since you may need to control the order
        // in which things are freed. Your override of TDocument.Free can call FreeData if
        // convenient.

	virtual pascal void FreeData();
		// Called when a document is reverted, in order to free it's data objects. You may
        // also wish to call this from you document's Free method, if convenient.

	virtual pascal void FreeFromClipboard();
		// Called to free a Clipboard document.Simply calls Free.

	virtual pascal void GetInspectorName(Str255& inspectorName);
		// Called by the Inspector to get the name of this class.

	virtual pascal void Fields(TObject* obj);
		// Called by the Inspector and the Debugger to display the contents of this
        // class's fields.


	//------------------------------------------------------------------------------------
	// Opening / Printing Documents
	//------------------------------------------------------------------------------------

	virtual pascal Boolean AlreadyOpen(TFile* aFile);
		// TRUE if this document is already open

	virtual pascal void DoInitialState();
		// Called for 'New', 'Revert' to blank, Open Tool. Default is to do nothing; You
        // may need to do further initialization that you would not do in case of an Open

	virtual pascal void DoMakeWindows();
		// Should take the views created by DoMakeViews and create windows and frames to
        // display them.

	virtual pascal void DoMakeViews(Boolean forPrinting);
		// Create all necessary views for this document (based on forPrinting flag) and
        // store into fields of your document. The document will then be sent either
        // DoMakeWindows or Print.

	virtual pascal void AttachPrintHandler(TPrintHandler* itsPrintHandler);
		// Creates a TPrintMenuBehavior and adds it to this document

	virtual pascal void DetachPrintHandler(TPrintHandler* itsPrintHandler);
		// Deletes a TPrintMenuBehavior that's been added to this document

	virtual pascal void OpenAgain(CmdNumber itsCmdNumber, TDocument* openingDoc);
		// Called if the user tries to open the same document twice. openingDoc will be
        // the document object being opened. Default brings SELF to the front and calls
        // Failure(noErr, 0), which will abort opening the document. (The goal is to
        // prevent the user from having 2 copies of the same document open, in which case
        // they could make incompatible changes to both.) You might override this if you
        // want to open a second window on the same document. In the ideal case, you
        // should make both windows display exactly the same data, and changes made to one
        // should be reflected in the other. You could also override this to create a
        // read-only copy of the document.

	virtual pascal short PoseSaveDialog();
		// If the document has been changed (fChangeCount > 0), then the user is asked to
        // save the document. PoseSaveDialog returns cancel, kYesButton or kNoButton,
        // depending on the button chosen by the user. If the document has not been
        // changed then kNoButton is returned.

	virtual pascal void ReadDocument(Boolean forPrinting);
		// Called to read an existing file for display or printing.

	virtual pascal void ShowWindows();
		// Called when opening the document initially; default is to call OpenWindow for
        // all windows that have fFlags.openInitially TRUE

	virtual pascal void UntitledName(Str255& noName);
		// Called to supply a name for an untitled document. If this returns '' then the
        // windows are not renamed from the value specified in their titles.


	//------------------------------------------------------------------------------------
	// Saving Documents
	//------------------------------------------------------------------------------------

	virtual pascal long GetChangeCount();
		// Gets the document change count

	virtual pascal void SetChangeCount(long newChangeCount);
		// Sets the document change count

	virtual pascal void SaveDocument(CmdNumber itsCmdNumber);
		// Try to save the document to disk

	virtual pascal void SaveAgain(CmdNumber itsCmdNumber, TDocument* savingDoc);
		// Called when the user tries to Save As or Save a Copy In and specifies the name
        // of a document that is already opened. The default case calls
        // Failure(errSaveAgain, msgSaveFailed) to display an error. (The goal is to
        // prevent the user from having 2 copies of the same document open, in which case
        // they might make incompatible changes to both copies.) savingDoc is the document
        // being saved


	//------------------------------------------------------------------------------------
	// Closing Documents
	//------------------------------------------------------------------------------------

	virtual pascal void Close();
		// Close a document and free it. Signals Failure with error = noErr and message =
        // msgCancelled if user cancels. ??? NOTE: Must never be called for a document
        // related to a view in the Clipboard.


	//------------------------------------------------------------------------------------
	// Revert
	//------------------------------------------------------------------------------------

	virtual pascal void RevertDocument();
		// IF fSaveExists is TRUE then reverts to the file saved on disk, otherwise merely
        // resets the print handler and calls doInitialState. Calls FreeData to free any
        // existing data and commits any currently active command.

	virtual pascal void Abandon();
		// Called when a document is closed and changes are being abandoned.For memory
        // based documents the default behaviour of doing nothing is OK.For disk based
        // documents that change the original copy this is the place to override to put
        // back the original contents if you still have it available somewhere (a scratch
        // file?)

	virtual pascal void ShowReverted();
		// Called after Revert to show brand new document; default tells each view to
        // ShowReverted


	//------------------------------------------------------------------------------------
	// Miscellaneous
	//------------------------------------------------------------------------------------

	virtual pascal void DoWriteData(const OSType aScrapType,
											  TDesignator* aDesignator,
											  TStream* aStream);
		// Given a designator and a stream, writes the designated data to the stream.

	virtual pascal void DoReadData(const OSType aScrapType,
											  TDesignator* aDesignator,
											  TStream* aStream, long length);
		// Given a designator and a stream, read the designated data from the stream.


	//------------------------------------------------------------------------------------
	// designator management methods
	//------------------------------------------------------------------------------------

	virtual pascal TDesignator* GetUserSelection();
		// Returns the designator for the user selection (if any).

	virtual pascal void SetUserSelection(TDesignator* newSelection);
		// Sets the designator for the user selection (if any).

	virtual pascal void UserSelectionChanged();
		// Call this whenever the user's selection changes. Subclasses should implement
        // the appropriate behavior for their corresponding user selection TDesignator.

	virtual pascal void RevealSelection(TDesignator* aSelection);
		// OVERRIDE this; bring this designation into user's view

	virtual pascal void GetTitle(Str255& aTitle);
		// Returns the name of the fFile or '' if the file does not exist

	virtual pascal void SetTitle(const Str255& aTitle);
		// Does fFile.SetName to aTitle and calls SetTitleForDoc for each window of the
        // document.

	virtual pascal void Changed(ChangeID theChange, TObject* changedBy);
		// Updates fChangeCount. Calls INHERITED Changed to notify dependents


	//------------------------------------------------------------------------------------
	// Command Handlers
	//------------------------------------------------------------------------------------

	virtual pascal void DoMenuCommand(CmdNumber aCmdNumber);
		// Handles the default menu commands associated with a document and must be
        // overridden to handle any additional items.

	virtual pascal void DoSetupMenus();
		// Setup the documents menus, this handles the default menus and must be
        // overridden for any additional items.


	//------------------------------------------------------------------------------------
	// Hierarchy
	//------------------------------------------------------------------------------------

	virtual pascal void AddView(TView* aView);
		// Adds the view to the end of the document's view list.

	virtual pascal void AddWindow(TWindow* aWindow);
		// Adds the window to the end of the document's window list.

	virtual pascal void CloseView(TView* aView);
		// Called to close a view or window associated with a document. Document gets to
        // decide whether to really close the view or not and whether to close itself or
        // not.Does nothing if the view is not associated with the doc.

	virtual pascal void DeleteView(TView* viewToDelete);
		// Deletes the view from the document's view list.

	virtual pascal void DeleteWindow(TWindow* windowToDelete);
		// Deletes the window from the document's window list.

	virtual pascal void ForAllViewsDo(pascal void(* DoToView)(TView* aView,
									void* staticLink), void* staticLink);
		// Performs DoToView on every view in every window belonging to the document. If
        // you want to deal with all views belonging to the document use fViewList.Each.

	virtual pascal void ForAllWindowsDo(DoToWindType DoToWind, void* staticLink);
		// Perform the supplied proc to each window belonging to the document

private:
	short OpenWindowCount(void);
};


//----------------------------------------------------------------------------------------
// TSaveDocCommand: Tells the document to save itself.
//----------------------------------------------------------------------------------------

class TSaveDocCommand : public TNoChangesCommand
{
public:
	TDocument* fChangedDocument;				// the document changed by this command

	virtual pascal void Initialize();
		// Puts a newly created TSaveDocCommand into a known state. Subclasses should
        // override it and call INHERITED. Typically used to put the object into a state
        // where it can be safely FREED.

	virtual pascal void ISaveDocCommand(CmdNumber itsCmdNumber, TDocument* itsDocument);
		// Initialize the SaveDocCommand procedurally.

	virtual pascal void DoIt();
		// tell the document to save.

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields.
};


//----------------------------------------------------------------------------------------
// TRevertDocCommand: Tells the document to revert after checking with the user.
//----------------------------------------------------------------------------------------

class TRevertDocCommand : public TNoChangesCommand
{
public:
	TDocument* fChangedDocument;				// the document changed by this command

	virtual pascal void Initialize();
		// Puts a newly created TRevertDocCommand into a known state. Subclasses should
        // override it and call INHERITED. Typically used to put the object into a state
        // where it can be safely FREED.

	virtual pascal void IRevertDocCommand(CmdNumber itsCmdNumber, TDocument* itsDocument);
		// Initialize the RevertDocCommand procedurally.

	virtual pascal void DoIt();
		// tell the document to revert if the user says OK.

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields.
};


//----------------------------------------------------------------------------------------
// Externally available global methods.
//----------------------------------------------------------------------------------------

extern short gNumUntitled;
	// The number to assign to the next Untitled document (assuming that the app provides
    // a template for filling in the number. You must change STR# resource with ID 0/ index
    // 3 to read (for example) Untitled-<<<#>>>.

#endif


