// UEditionDocument.h
// Copyright 1984-1991 Apple Computer, Inc.All rights reserved.

#ifndef __UEDITIONDOCUMENT__
#define __UEDITIONDOCUMENT__

#ifndef __PASCALDEFS__
#include <PascalDefs.h>
#endif

#ifndef __UGEOMETRY__
#include <UGeometry.h>
#endif

#ifndef __EDITIONS__
#include <Editions.h>
#endif

#ifndef __UFILEBASEDDOCUMENT__
#include <UFileBasedDocument.h>
#endif

#ifndef __UADORNERS__
#include <UAdorners.h>
#endif

#ifndef __UITERATOR__
#include <UIterator.h>
#endif


//----------------------------------------------------------------------------------------
// Edition Manager menu command constants
//----------------------------------------------------------------------------------------

const short cCreatePublisher = 601;				// the user has selected "Create
                                                // Publisher" from the Edit menu

const short cSubscribeTo = 602;					// the user has selected Subscribe to from
                                                // the Edit menu

const short cOptions = 603;						// the user has selected a publisher in
                                                // the document and then selected
                                                // Publisher Options from the Edit menu,
                                                // OR, the user has selected a subscriber
                                                // in the document and then selected
                                                // Subscriber Options from the Edit menu

const short cBorders = 604;						// the user has selected Show/ Hide Borders
                                                // from the Edit menu

const short cStopAllEditions = 605;				// the user has selected Stop all Editions
                                                // from the Edit menu


//----------------------------------------------------------------------------------------
// Edition Manager command constants
//----------------------------------------------------------------------------------------

const short cCancelPublisher = 606;				// the user canceled this publisher

const short cCancelSubscriber = 607;			// the user canceled this subscriber


//----------------------------------------------------------------------------------------
// Forward and external class declarations
//----------------------------------------------------------------------------------------

class TDesignator;
class TFile;
class TList;
class TPublisher;
class TSection;
class TSubscriber;
class TView;
class TEditionDocument;


//----------------------------------------------------------------------------------------
// Typedefs
//----------------------------------------------------------------------------------------

typedef pascal void (*DoToSectionType)(TSection* aSection, void* staticLink);

typedef pascal void (*DoToPublisherType)(TPublisher* aPublisher, void* staticLink);

typedef pascal void (*DoToSubscriberType)(TSubscriber* aSubscriber, void* staticLink);

typedef pascal Boolean (*TestSectionType)(TSection* aSection, void* staticLink);

																			
//----------------------------------------------------------------------------------------
// CSectionIterator
//----------------------------------------------------------------------------------------

class CSectionIterator : public CObjectIterator
{
public:
	CSectionIterator(TEditionDocument* itsDocument,
					 ArrayIndex itsLowBound, ArrayIndex itsHighBound,
					 Boolean itsForward);

	CSectionIterator(TEditionDocument* itsDocument, Boolean itsForward);

	CSectionIterator(TEditionDocument* itsDocument);
	
	TSection* CurrentSection(void);
		// returns the current section
	
	TSection* FirstSection(void);
		// Resets the iteration to begin again and return the first section in the iteration

	TSection* NextSection(void);
		// advances the iteration and then returns the section
};


//----------------------------------------------------------------------------------------
// TFileBasedDocument
//----------------------------------------------------------------------------------------

class TEditionDocument : public TFileBasedDocument
{
public:
	TList* fSectionList;						//  list of TSection objects

	Boolean fStopAllEditions;					//  FALSE by default

	Boolean fShowSectionBorders;				//  TRUE by default

	Boolean fNewPublishers;						//  are there any new publishers?

	short fNextEditionNumber;					//  number used in "Untitled-n"

	OSType fEditionCreator;						// creator for publisher edition container
                                                // file


	//------------------------------------------------------------------------------------
	// constructor/destructor methods
	//------------------------------------------------------------------------------------

	virtual pascal void Initialize();

	virtual pascal void IEditionDocument(TFile* itsFile,
										 const OSType itsFileType,
										 const OSType itsCreator,
										 const OSType itsScrapType,
										 Boolean usesDataFork,
										 Boolean usesRsrcFork,
										 Boolean keepsDataOpen,
										 Boolean keepsRsrcOpen,
										 const OSType editionCreator);
		// initializes this object

	virtual pascal void Free();
		// frees this object

	virtual pascal void FreeData();
		// frees the fSectionList


	//------------------------------------------------------------------------------------
	// saving documents
	//------------------------------------------------------------------------------------

	virtual pascal void Close();
		// For the case in which the document's change count = 0, but there are "dirty"
        // sections, need to ask the user whether to save the document.

	virtual pascal short PoseNewPublishersAlert();
		// Warns the user that he is about to close a document that hasn't been saved in
        // which he has created a new publisher. The user hasn't modified the document's
        // data.

	virtual pascal short PoseMultPublishersAlert(TSection* aSection);
		// Warns the user that he is creating multiple publishers to the same edition.

	virtual pascal short PoseSavingMultPublishersAlert(TSection* aSection);
		// Warns the user that he is saving multiple publishers to the same edition.

	virtual pascal void Abandon();
		// Calls TSection.Delete on all dirty sections.

	virtual pascal void DoNeedDiskSpace(TFile* itsFile,
										long& dataForkBytes,
										long& rsrcForkBytes);
		// Bytes required to store SAVE file(s) on disk. When called, both parameters are
        // set to 0 for you.

	virtual pascal void DoWrite(TFile* aFile, Boolean makingCopy);
		// writes out the sections and edition doc settings

	virtual pascal void DoWriteSettings(TFile* aFile);
		// writes out the edition doc settings (Show/ Hide Border, Stop All Editions)

	virtual pascal void DoRead(TFile* aFile, Boolean forPrinting);
		// reads in the sections and edition doc settings

	virtual pascal void DoReadSettings(TFile* aFile);
		// reads in the edition doc settings (Show/ Hide Border, Stop All Editions)

	virtual pascal void DoReadPublisher(TFile* aFile, short theID);
		// reads in a given publisher

	virtual pascal void DoReadSubscriber(TFile* aFile, short theID);
		// reads in a given subscriber

	virtual pascal Boolean IsAnySectionDirty();
		// returns TRUE if any section is dirty


	//------------------------------------------------------------------------------------
	// Accessors
	//------------------------------------------------------------------------------------

	virtual pascal void SetSectionBorders(Boolean state, Boolean invalidate);
		// sets the value of fShowSectionBorders to state and invalidates the document's windows
        // if "invalidate" is TRUE

	virtual pascal void SetStopAllEditions(Boolean state);
		// sets the value of fStopAllEditions to state and reads in any new subscribers if "state"
        // is TRUE

	virtual pascal OSType GetEditionCreatorSignature();
		// returns the creator signature used for the edition container file


	//------------------------------------------------------------------------------------
	// menu management methods
	//------------------------------------------------------------------------------------

	virtual pascal void DoMenuCommand(CmdNumber aCmdNumber);

	virtual pascal void DoSetupMenus();

	virtual pascal Boolean CanPublishSelection();

	virtual pascal Boolean CanSubscribe();


	//------------------------------------------------------------------------------------
	// edition manager support
	//------------------------------------------------------------------------------------

	virtual pascal void DoNewPublisher();

	virtual pascal TPublisher* DoMakePublisher(TDesignator* itsDesignator,
											   SectionHandle itsSectionHandle,
											   short itsSectionID);


	virtual pascal void DoNewPublisherDialog(NewPublisherReply& reply);

	virtual pascal void DoMakePreview(TDesignator* aDesignator,
												  FormatType& previewFormat,
												  Handle& preview);
	
	virtual pascal void DoNewSubscriber();

	virtual pascal TSubscriber* DoMakeSubscriber(TDesignator* itsDesignator,
												 SectionHandle itsSectionHandle,
												 short itsSectionID);

	virtual pascal void DoNewSubscriberDialog(NewSubscriberReply& reply);

	virtual pascal void DoSectionOptions();
		// poses the publisher or subscriber options dialog depending on what
        // GetSelectedSection returns

	virtual pascal FormatType GetPublishPreference();
		// return 'PICT' to publish 'PICT' first, return 'TEXT' to publish 'TEXT' first

	virtual pascal SignedByte GetSubscriberFormatsMask();

	virtual pascal void GetNextEditionName(Str63& editionName);

	virtual pascal void DoPublisherOptions(TPublisher* aPublisher);

	virtual pascal void DoPublisherOptionsDialog(SectionOptionsReply& reply);

	virtual pascal void DoSubscriberOptions(TSubscriber* aSubscriber);

	virtual pascal void DoSubscriberOptionsDialog(SectionOptionsReply& reply);


	//------------------------------------------------------------------------------------
	// section management
	//------------------------------------------------------------------------------------

	virtual pascal void AddSection(TSection* aSection);
		// call aSection.Register; this method should mark document as changed so that
        // when saving the document, the user saves this section

	virtual pascal void CancelSection(TSection* aSection, Boolean cancel);
		// calls aSection.CancelSection(cancel);

	virtual pascal void EachCanceledSectionDo(DoToSectionType DoToCancelledSection,
											  void* staticLink);
		// iterates over section list calling DoToCanceledSection on each canceled section

	virtual pascal void EachPublisherDo(DoToPublisherType DoToPublisher,
										void* staticLink);
		// iterates over section list calling DoToPublisher on each publisher object,
        // filtering out canceled sections

	virtual pascal void EachSectionDo(DoToSectionType DoToSection,
									  void* staticLink);
		// iterates over section list calling DoToSection on each object, filtering out
        // canceled sections

	virtual pascal void EachSubscriberDo(DoToSubscriberType DoToSubscriber,
										 void* staticLink);
		// iterates over section list calling DoToSubscriber on each subscriber object,
        // filtering out canceled sections

	virtual pascal TSection* FirstSectionThat(TestSectionType TestSection,
											  void* staticLink);
		// Call TestSection on each section, until it returns TRUE.

	virtual pascal TSection* GetSectionObject(SectionHandle aSectionHandle);
		// maps a SectionHandle to a TSection object

	virtual pascal TSection* GetSelectedSection();
		// This returns the currently selected section, if any. Returns NIL if > 1 section
        // is currently selected.

	virtual pascal short GetUniqueSectRsrcID();
		// returns a unique 'sect' resource id for use as both the section ID and as the
        // rsrc ID

	virtual pascal Boolean IsBorderShown(TSection* aSection);
		// returns TRUE if (borders are shown) or (the user selection contains aSection)

	virtual pascal Boolean IsSectionSelected(TSection* aSection);
		// returns TRUE if the given section is in the user selection

	virtual pascal void RemoveSection(TSection* aSection);
		// this method should mark document as changed so that document removes this
        // section when saved (e.g. sipAlways)


	//------------------------------------------------------------------------------------
	// inspecting
	//------------------------------------------------------------------------------------

	virtual pascal void Fields(TObject* obj);
};


//----------------------------------------------------------------------------------------
// TCancelPublishedCmd: TCancelPublisherCmd is generated when the user selects Cancel
// Publisher from the Publisher options dialog [this command should be undoable in case
// the user changes his mind]. This command cancels a publisher. Note: keep the publisher
// *data* intact
//----------------------------------------------------------------------------------------

class TCancelPublisherCmd : public TCommand
{
public:
	TPublisher* fPublisher;						// the publisher that will be canceled

	virtual pascal void ICancelPublisherCmd(CmdNumber itsCmdNumber,
											TDocument* itsDocument,
											TPublisher* itsPublisher);
		// initialize this command

	virtual pascal void DoIt();
		// call fDocument.CancelPublisher(fPublisher, kCancel)

	virtual pascal void UndoIt();
		// call fDocument.CancelPublisher(fPublisher, kUnCancel)
};


//----------------------------------------------------------------------------------------
// TCancelSubscriberCmd: TCancelSubscriberCmd is generated when the user selects Cancel
// Subscriber from the Subscriber options dialog [this command should be undoable in case
// the user changes his mind]. This command cancels a subscriber. Note: keep the
// subscriber *data* intact in the document
//----------------------------------------------------------------------------------------

class TCancelSubscriberCmd : public TCommand
{
public:
	TSubscriber* fSubscriber;					// the subscriber that will be canceled

	virtual pascal void ICancelSubscriberCmd(CmdNumber itsCmdNumber,
											 TDocument* itsDocument,
											 TView* itsView,
											 TSubscriber* itsSubscriber);
		// initialize this command

	virtual pascal void DoIt();
		// call fDocument.CancelSubscriber(fSubscriber, kCancel)

	virtual pascal void UndoIt();
		// call fDocument.CancelSubscriber(fSubscriber, kUnCancel)
};


//----------------------------------------------------------------------------------------
// TSectionBorderCmd: TSectionBorderCmd is an undoable command which changes the setting
// for the show/ hide border option for the current document when the user chooses Hide/
// Show Borders from the Edit menu
//----------------------------------------------------------------------------------------

class TSectionBorderCmd : public TCommand
{
public:
	Boolean fShowSectionBorders;				// value to set

	virtual pascal void ISectionBorderCmd(CmdNumber itsCmdNumber, TDocument* itsDocument);
		// initialize this command

	virtual pascal void DoIt();
		// tell the document to show or hide the borders for all sections

	virtual pascal void RedoIt();
		// call DoIt

	virtual pascal void UndoIt();
		// tell the document to undo show or hide the borders for all sections
};


//----------------------------------------------------------------------------------------
// TStopAllEditionsCmd: TStopAllEditionsCmd is an undoable command gets posted when the
// user chooses Stop all Editions from the Edit menu. This command toggles the setting for
// the documents fStopAllEditions field.
//----------------------------------------------------------------------------------------

class TStopAllEditionsCmd : public TCommand
{
public:
	Boolean fStopAllEditions;					// value to set

	virtual pascal void IStopAllEditionsCmd(CmdNumber itsCmdNumber, 
											TDocument* itsDocument);
		// initialize this command

	virtual pascal void DoIt();
		// tell the document to toggle the state of its fStopAllEditions flag

	virtual pascal void RedoIt();
		// tell the document to toggle the state of its fStopAllEditions flag

	virtual pascal void UndoIt();
		// tell the document to toggle the state of its fStopAllEditions flag
};


//----------------------------------------------------------------------------------------
// TSectionAdorner: abstract superclass for section adorners
//----------------------------------------------------------------------------------------

class TSectionAdorner : public TAdorner
{
public:
	RgnHandle fBorderRgn;						// the region for the section border

	TSection* fSection;							// the section this adorner adorns

	virtual pascal void Initialize();
		// sets fBorderRgn & fSection to NIL

	virtual pascal void ISectionAdorner(IDType itsID,
										TSection* itsSection,
										RgnHandle itsBorderRgn);

		// initializes the section adorner with priority kLowAdornPriority

	virtual pascal void Free();
		// frees fBorderRgn

	virtual pascal void InvalidateAdorner(TView* itsView); // Override
		// Invalidates the adorner in the TAdornerMgr' TView.

	virtual pascal void DrawBorder(const Pattern& whichPattern);	
		// Draws the adorner's border in the TAdornerMgr' TView.

};


//----------------------------------------------------------------------------------------
// TSubscriberAdorner: A specialized adorner for imaging subscriber borders
//----------------------------------------------------------------------------------------

class TSubscriberAdorner : public TSectionAdorner
{
public:
	virtual pascal void ISubscriberAdorner(IDType itsID,
										   TSubscriber* itsSubscriber,
										   RgnHandle itsBorderRgn);
		// initializes the subscriber adorner

	virtual pascal void DoHighlightSelection(TView* itsView,
											 const VRect& area,
											 HLState fromHL,
											 HLState toHL); // Override
		// Adorns the selection before the view's DoHighlightSelection.
};


//----------------------------------------------------------------------------------------
// TPublisherAdorner
//----------------------------------------------------------------------------------------

class TPublisherAdorner : public TSectionAdorner
{
public:
	virtual pascal void IPublisherAdorner(IDType itsID,
										  TPublisher* itsPublisher,
										  RgnHandle itsBorderRgn);
		// initializes the publisher adorner

	virtual pascal void DoHighlightSelection(TView* itsView,
											 const VRect& area,
											 HLState fromHL,
											 HLState toHL); // Override
		// Adorns the selection before the view's DoHighlightSelection.

};


//----------------------------------------------------------------------------------------
// Externally avaibable routines
//----------------------------------------------------------------------------------------

extern pascal void InitUEditionDocument();
	// call this routine to initialize this unit.


#endif


