// UEvent.h
// Copyright 1984-1991 Apple Computer, Inc.All rights reserved.

#ifndef __UEVENT__
#define __UEVENT__

#ifndef __PASCALDEFS__
#include <PascalDefs.h>
#endif

#ifndef __EVENTS__
#include <Events.h>
#endif

#ifndef __UOBJECT__
#include <UObject.h>
#endif


//----------------------------------------------------------------------------------------
// Constants
//----------------------------------------------------------------------------------------

const short kPriorityLowest = 127;
	// Low priority commands are considered last

const short kPriorityLow = kPriorityLowest - 32;
	// Low priority commands are considered last

const short kPriorityNormal = 64;
	// Normal priority: command default priority

const short kPriorityHigh = kPriorityNormal - 32;
	// High priority commands take precedence

const short kPriorityHighest = 0;
	// High priority commands take precedence


//----------------------------------------------------------------------------------------
// Typedefs
//----------------------------------------------------------------------------------------

typedef char EventPriority;
	//  Event priorities


//----------------------------------------------------------------------------------------
// Forward and external class declarations
//----------------------------------------------------------------------------------------

class TEvtHandler;


//----------------------------------------------------------------------------------------
// TEvent
//----------------------------------------------------------------------------------------
class TEvent : public TObject
{
public:
	EvtNumber fID;								// Can be a many-to-one mapping of fID to
                                                // TEvent

	TEvtHandler* fSource;

	TEvtHandler* fHandler;						//  the handler for this event

	Boolean fAffectsMenus;						// True means the menus must be setup
                                                // after the event

	EventPriority fPriority;					// How this event should be prioritized if
                                                // it has do compete on a prioritized
                                                // basis for execution. kNormalPriority is
                                                // the Default. Most programs won't have
                                                // to mess with this.


	virtual pascal void Initialize();

	virtual pascal void IEvent(EvtNumber itsEventNumber,
							   TEvtHandler* itsSource,
							   TEvtHandler* itsHandler);


	virtual pascal Boolean ShouldFreeOnCompletion();

	virtual pascal Boolean IsReadyToExecute();

	virtual pascal Boolean IsRecurring();


	virtual pascal TEvtHandler* GetHandler();

	virtual pascal void Process();

	virtual pascal void ReportEvent();

	virtual pascal void Fields(TObject* obj);
};


//----------------------------------------------------------------------------------------
// TToolboxEvent
//----------------------------------------------------------------------------------------

class TToolboxEvent : public TEvent
{
public:
	EventRecord fEventRecord;					// The original packed ToolBox event
                                                // record

	Boolean fBtnState;							// bits found in the modifiers field of an
                                                // EventRecord

	Boolean fCmdKey;							// Was Command key depressed?

	Boolean fShiftKey;							// Was Shift key depressed?

	Boolean fAlphaLock;							// Was Alpha Lock depressed?

	Boolean fOptionKey;							// Was Option key depressed?

	Boolean fControlKey;						// Was Control key depressed?

	Boolean fAutoKey;							// TRUE if this was an auto key event

	short fCharacter;							// IF a key event, the char that the event
                                                // translates to. What about 16 bit
                                                // chars???

	short fKeyCode;								// IF a key event the virtual key code
                                                // that the event translates to

	short fClickCount;							// 0 = event was not a mouse down; 1-N = #
                                                // of multiple clicks


	virtual pascal void IToolboxEvent(TEvtHandler* itsHandler);

	virtual pascal void ReportEvent();

	virtual pascal void Fields(TObject* obj);

	virtual pascal void HaveEvent(const EventRecord& theEvent);
};


#endif


