// UFailure.h
// Copyright  1984-1991 by Apple Computer Inc. All rights reserved.

#ifndef __UFAILURE__
#define __UFAILURE__

#ifndef __PASCALDEFS__
#include "PascalDefs.h"
#endif

#ifndef __MACAPPTYPES__
#include "MacAppTypes.h"
#endif


//----------------------------------------------------------------------------------------
// This macro can be called on any variable to keep it out of a register. It is
// used specifically in exception handling. Cludge to be used till volatile or
// C++ exception handling is implemented.
//----------------------------------------------------------------------------------------

#define VOLATILE(a) ((void) &a)


//----------------------------------------------------------------------------------------
// Max and min error number constants
//----------------------------------------------------------------------------------------

const short minErr = -32768;
const short maxErr = 32767;


//----------------------------------------------------------------------------------------
// FailInfo
//----------------------------------------------------------------------------------------

class FailInfo
{
public:
	long regs[12];
	short flags;
	short error;
	long message;
	long failA6;
	long failPC;
	FailInfo* nextInfo;
#if qDebug
	long whoPC;
#endif

public:
	FailInfo();

	inline Boolean Try();

	inline void ReSignal();

	inline void Success();
};

typedef struct FailInfo *FailInfoPtr;


//----------------------------------------------------------------------------------------
// Global variable declarations
//----------------------------------------------------------------------------------------

#if qDebug
extern Boolean gAskAboutAlloc;
extern Boolean gAskFailure;
#endif

extern FailInfoPtr gTopHandler;


//----------------------------------------------------------------------------------------
// Global function declarations
//----------------------------------------------------------------------------------------

pascal void Assertion(const Boolean condition, const Str255& description);

pascal long BuildMessage(short lowWord, short highWord) = { 0x2e9f };

typedef pascal void(* CatchFailuresType)(short e, long m, void* staticLink);

pascal void CatchFailures(FailInfo& fi, CatchFailuresType Handler, void* staticLink);

typedef pascal void(* DoToHandlerType)(FailInfoPtr fiPtr, void* staticLink);

pascal void EachFailureHandlerDo(DoToHandlerType DoToHandler, void* staticLink);

pascal void Failure(short error, long message);

pascal void FailMemError();

pascal void FailResError();

pascal void FailNewMessage(short error, long oldMessage, long newMessage);

pascal void FailNIL(void* p);

pascal void FailNILResource(Handle r);

pascal void FailOSErr(short error);

pascal Boolean HandlerExists(FailInfoPtr testFailInfoPtr);

pascal void Success(FailInfo& fi);

pascal void ProgramBreak(const Str255& grievance);

pascal void ProgramReport(const Str255& grievance, const Boolean breakInDebugger);

extern "C" {
extern Boolean Try(FailInfo& fi);
}

//----------------------------------------------------------------------------------------
// FailInfo inline method definitions
//----------------------------------------------------------------------------------------

inline FailInfo::FailInfo() { }

inline Boolean FailInfo::Try() { return ::Try(*this); }

inline void FailInfo::ReSignal() { ::Failure(error, message); }

inline void FailInfo::Success() { ::Success(*this); }


#endif


