#ifndef __UFILE__
#define __UFILE__

// UFile.h
// Copyright 1984-1991 Apple Computer, Inc.All rights reserved.

#ifndef __PASCALDEFS__
#include <PascalDefs.h>
#endif

#ifndef __ALIASES__
#include <Aliases.h>
#endif

#ifndef __UOBJECT__
#include <UObject.h>
#endif


enum StationeryState { isStationery, notStationery, checkStationery };


//----------------------------------------------------------------------------------------
// TFile
//----------------------------------------------------------------------------------------

class TFile : public TObject
{
public:


	FSSpec fFileSpec;							// volume/ directory/filename

	long fModDate;								// file mod date when last read/ saved

	StationeryState fStationery;				// is this Stationery?

	OSType fFileType;							// file type

	OSType fCreator;							// creator ID

	ScriptCode fScriptTag;						// The script the file was created with

	Boolean fUsesDataFork;						// The file uses its data fork

	Boolean fUsesRsrcFork;						// The file uses its rsrc fork

	SignedByte fDataPermission;					// Permissions used when opening the data
                                                // fork

	SignedByte fRsrcPermission;					// Permissions used when opening the rsrc
                                                // fork

	Boolean fKeepDataOpen;						// Will the data fork be left open?

	Boolean fKeepRsrcOpen;						// Will the rsrc fork be left open?

	short fDataRefNum;							// Reference for reading/ writing to data
                                                // fork

	short fRsrcRefNum;							// Reference for reading/ writing to rsrc
                                                // fork


	virtual pascal void Initialize();
		// Put the file into a known state

	virtual pascal void IFile();
		// Initialization method for TFile

	virtual pascal void Free();

	virtual pascal OSErr CloseFile(Boolean forceClose);
		// Close the file.

	virtual pascal OSErr CloseDataFork(Boolean forceClose);
		// Close the data fork.Called by CloseFile

	virtual pascal OSErr CloseRsrcFork(Boolean forceClose);
		// Close the rsrc fork. Called by CloseFile

	virtual pascal OSErr CreateFile();
		// Create the file

	virtual pascal OSErr CreateDataFork();
		// Create the data fork of the file.Called by CreateFile

	virtual pascal OSErr CreateRsrcFork();
		// Create the rsrc fork of the file.Called by CreateFile

	virtual pascal OSErr DeleteFile();
		// Close the file and delete it.

	virtual pascal OSErr OpenFile();
		// Open the file with the fDataPermission and fRsrcPermission

	virtual pascal OSErr OpenDataFork(SignedByte permission);
		// Open the data fork of the file.Called by OpenFile

	virtual pascal OSErr OpenRsrcFork(SignedByte permission);
		// Open the rsrc fork of the file.Called by OpenFile


	//------------------------------------------------------------------------------------
	// file attributes
	//------------------------------------------------------------------------------------

	virtual pascal void DefineFile(const OSType itsFileType,
								   const OSType itsCreator,
								   Boolean usesDataFork,
								   Boolean usesRsrcFork,
								   Boolean keepsDataOpen,
								   Boolean keepsRsrcOpen);
		// Initializes the fields of TFile used for data and resource manipulation

	virtual pascal OSErr GetAlias(AliasHandle& alias);
		// Return the file specification as an alias handle

	virtual pascal OSErr GetCatInfo(CInfoPBRec& cInfo);
		// Return the catalog information about the this file

	virtual pascal long GetCreationDate();
		// Date the file was created

	virtual pascal OSErr GetDataLength(long& length);
		// Return the length of the data fork

	virtual pascal OSErr GetDataMark(long& mark);
		// Return the position of the mark in the data fork

	virtual pascal long GetDirID();
		// Return the directory ID stored in the FSSpec

	virtual pascal OSErr GetFileCreator(OSType& creator);
		// Return the creator of the file as found in the finder info

	virtual pascal OSErr GetFileInfo(HParamBlockRec& pb);
		// Return all the information the file manager provides

	virtual pascal void GetFileSpec(FSSpec& theFileSpec);
		// Return its file specification record

	virtual pascal OSErr GetFileType(OSType& fileType);
		// Return the type of the file as found in the finder info

	virtual pascal OSErr GetFinderInfo(FInfo& fndrInfo);
		// Return finder information about the file

	virtual pascal long GetModificationDate();
		// Return the date the file was last modified

	virtual pascal void GetName(Str63& name);
		// Return the name of the file

	virtual pascal OSErr GetPhysicalSize(long& dataSize, long& rsrcSize);
		// Returns the physical size in bytes of the data and resource fork

	virtual pascal OSErr GetScript(ScriptCode& theScript);
		// Returns the script code as stored in the extended finder info


	//----------------------------------------------------------------------------------------
	// Volume attributes
	//----------------------------------------------------------------------------------------

	virtual pascal OSErr FlushVolume();
		// Flush the volume to force any changes to take place

	virtual pascal OSErr GetBlockSize(long& blockSize);
		// Return the size of allocated blocks on the volume

	virtual pascal OSErr GetFreeBlocks(long& freeBlocks);
		// Return the number of free blocks on the volume

	virtual pascal OSErr GetVolumeInfo(HParamBlockRec& pb);
		// Return all information provided by the file manager on the volume

	virtual pascal OSErr GetVolumeName(Str63& name);
		// Return the name of the volume

	virtual pascal short GetVolRefNum();
		// Return the volume reference number stored in the FSSpec

	virtual pascal OSErr ExchangeFiles(TFile* aFile);
		// Exchange the contents of self with the contents of another file maintaining the
        // file id

	virtual pascal void Identify(const FSSpec& theFile);
		// Identify the file by its FSSpec.If the file has already opened a file, Identify
        // will attempt to close the file before re-identifying the file

	virtual pascal void IdentifyWithScript(const FSSpec& theFile, ScriptCode theScript);
		// Identify the file by its FSSpec and indiciate its script

	virtual pascal OSErr IdentifyByAlias(AliasHandle alias);
		// Similar to TFile.Identify except the file is identified by an alias

	virtual pascal void IdentifyByFile(TFile* aFile);
		// Make this file reference the same file as aFile.Also sets fScriptTag equal to
        // aFile.fScriptTag

	virtual pascal OSErr IdentifyByTrio(short volRefNum, long dirID, const Str63& name);
		// Identify a file by volrefnum, dir ID, and filename.If dirID is zero, it then
        // volRefNum is assumed to be a working directory and will be resolved to a
        // directory ID

	virtual pascal Boolean IsDataForkOpen();
		// True if the data fork of the file is open

	virtual pascal Boolean IsModified();
		// Checks fModDate to determine if file changed

	virtual pascal Boolean IsRsrcForkOpen();
		// True if the resource fork of the file is open

	virtual pascal Boolean IsSameFile(TFile* aFile);
		// Does this file refer to the same file as aFile?

	virtual pascal OSErr IsVolumeLocked(Boolean& locked);
		// Returns TRUE if the volume is hardware or software locked

	virtual pascal Boolean HasDataFork();
		// Returns TRUE if file has a data fork

	virtual pascal Boolean HasRsrcFork();
		// Returns TRUE if file has a resource fork

	virtual pascal OSErr MakeStationery();
		// Make this file stationery

	virtual pascal void Modified();
		// Update fModDate to the value stored by the finder

	virtual pascal OSErr MoveAndRename(const FSSpec& dest);
		// Move the file to the new destination and give it a new name

	virtual pascal OSErr ReadData(void* buffer, long& count);
		// Read count bytes from the data fork into buffer

	virtual pascal OSErr RenameFile(Str63& newName);
		// Rename the file to the specified name

	virtual pascal OSErr SetCatInfo(CInfoPBRec& cInfo);
		// Set the catalog information about this file.The file reference portion of the
        // record is supplied by fFileSpec.

	virtual pascal OSErr SetDataLength(long length);
		// Set the length of the data fork

	virtual pascal OSErr SetDataMark(long mark, short fromWhere);
		// Set the position of the mark in the data fork

	virtual pascal OSErr SetFileInfo(HParamBlockRec& pb);
		// Set the file information about this file.The file reference portion of the
        // record is supplied by fFileSpec.

	virtual pascal void SetName(const Str63& newName);
		// Sets the name field of the filespec but doesn't rename the file

	virtual pascal void SetPermissions(SignedByte dataPermission,
									   SignedByte rsrcPermission);
		// Set the fDataPermission and fRsrcPermission fields of the file

	virtual pascal OSErr SetScript(ScriptCode theScript);
		// Sets the script code stored in the extended finder info

	virtual pascal OSErr UpdateResource();
		// Update the resource referenced by fRsrcRefNum

	virtual pascal short UseResource();
		// Returns the current resource file and set itself as the current file

	virtual pascal OSErr WriteData(void* buffer, long& count);
		// Write count bytes from buffer to the data fork

	virtual pascal void Fields(TObject* obj);
		// Called by the Inspector and the Debugger to display the contents of this
        // class's fields.
};


//----------------------------------------------------------------------------------------
// Global function declarations
//----------------------------------------------------------------------------------------

extern pascal TFile* NewFile();
	// A convenience function. Create a TFile, initialize it, and return a reference to
    // it. Signals Failure if it cannot allocate the object.


#endif


