// UFloatWindow.h
// Copyright 1984-1991 Apple Computer, Inc.All rights reserved.

#ifndef __UFLOATWINDOW__
#define __UFLOATWINDOW__

//----------------------------------------------------------------------------------------
// Theory of Operation
//  
// UFloatWindow a number of services to support floating windows, including the
// TFloatWindow base class which provides floating window capabilities to MacApp windows.
// Floating windows are distinguished from other windows in the application heap by the
// value of the windowKind field of the windowPeek record. The windowKind values 128-999
// are reserved by MacApp. For now, windowKind values of 128 are used to indicate that a
// window is a floating window.
//
// To use this unit, call InitUFloatWindow from your main program (i.e.
// M<your>Application.p) after calling InitUMacApp. This routine ensures that the class
// TFloatWindow isn't dead-stripped by the linker, sets gUFloatWindowInitialized to TRUE,
// and installs patches to the Window Manager which provide floating window services.
// N.B.: The MacApp team wishes to thank the HyperCard team for the use of their window
// manager patches to support floating window operation.
//
// We don't need to ensure that the window manager patches are removed when the
// application terminates because we use the UPatch mechanism to patch the traps and on
// the way out, CleanUpMacApp calls UnpatchAll which unpatches all traps patched with
// UPatch. This is important in non-MultiFinder worlds (MultiFinder automatically removes
// the application patches in MF worlds when the application either quits or is switched
// out in a context switch).
//
// To create a floating window:

// For template views, it's business as usual: specify the name of the window class,
// "TFloatWindow", in the window's view description resource and call NewTemplateWindow as
// usual. For non-template views, call this unit's NewFloatPaletteWindow, instead of
// NewPaletteWindow; call this unit's NewFloatSimpleWindow, instead of NewSimpleWindow;
// and call this unit's NewFloatTWindow, instead of NewTWindow.
//----------------------------------------------------------------------------------------


#ifndef __PASCALDEFS__
#include "PascalDefs.h"
#endif

#ifndef __UWINDOW__
#include "UWindow.h"
#endif


//----------------------------------------------------------------------------------------
// Foward and external class declarations
//----------------------------------------------------------------------------------------

class TDocument;


//----------------------------------------------------------------------------------------
// TFloatWindow
//----------------------------------------------------------------------------------------

class TFloatWindow : public TWindow
{
public:
	virtual pascal void IFloatWindow(TDocument* itsDocument,
									 WindowPtr itsWMgrWindow,
									 Boolean canResize,
									 Boolean canClose,
									 Boolean disposeOnFree);

	virtual pascal void IRes(TDocument* itsDocument,
							 TView* itsSuperView,
							 Ptr& itsParams);

	virtual pascal WindowPtr GetBehindWindowPtr();
		// returns WindowPtr(-1)

	virtual pascal TEvtHandler* GetNextHandler();
		// !!!
};


//----------------------------------------------------------------------------------------
// Global function declaraions
//----------------------------------------------------------------------------------------

extern pascal void InitUFloatWindow();
	// Unit initialization

extern pascal void MAClipAbove(WindowPtr window);

extern pascal void SendSystemWindowsToBack();


//----------------------------------------------------------------------------------------
// Global variable declarations
//----------------------------------------------------------------------------------------

extern Boolean gUFloatWindowInitialized;


#endif


