// UMacAppGlobals.h
// Copyright  1984-1991 by Apple Computer Inc.	All rights reserved.

#ifndef __UMACAPPGLOBALS__
#define __UMACAPPGLOBALS__

#ifndef __PASCALDEFS__
#include <PascalDefs.h>
#endif

#ifndef __DIALOGS__
#include <Dialogs.h>
#endif

#ifndef __UPATCH__
#include <UPatch.h>
#endif

#ifndef __UFAILURE__
#include <UFailure.h>
#endif

#ifndef __UOBJECT__
#include <UObject.h>
#endif


//----------------------------------------------------------------------------------------
//  Menu constants
//----------------------------------------------------------------------------------------

const short kMBarDisplayed = 128;				// Default 'MBAR' id of the menus that are
                                                // read in and installed in menu bar

const short kMBarNotDisplayed = 129;			// Default 'MBAR' id of the menus that are
                                                // read in but not installed

const short kMBarHierarchical = 130;			// Default 'MBAR' id of the menus that pop
                                                // up when a menu item is choosen

const short kMBarDebug = 900;					// Default 'MBAR' id of the Debug menus

const short kMBarDebugHierarchical = 901;		// Default 'MBAR' id of the Debug
                                                // Hierarchial menus


//----------------------------------------------------------------------------------------
// Command Numbers: Negative numbers and positive numbers up to 999 are reserved by
// MacApp.
//
// Not all of these are caught by MacApp:
//
// "<!>" means MacApp catches it.
// "<&>" means TEView catches it.
// "<->" means the application must catch it if it is used as a command number in any
// 			menu.

//----------------------------------------------------------------------------------------


//----------------------------------------------------------------------------------------
// Apple-menu commands
//----------------------------------------------------------------------------------------

const short cAboutApp = 1;						// <!> "About <appname>"


//----------------------------------------------------------------------------------------
// File-menu filing commands
//----------------------------------------------------------------------------------------

const short cNew = 10;							// <!> "New" (??? reserve a range of NEW
                                                // commands ???)

const short cNewLast = 19;

const short cOpen = 20;							// <!> "Open " (??? reserve a range of
                                                // OPEN commands ???)

const short cOpenLast = 29;

const short cSave = 30;							// <!> "Save"

const short cClose = 31;						// <!> "Close"

const short cSaveAs = 32;						// <!> "Save as"

const short cSaveCopy = 33;						// <!> "Save a Copy in "

const short cRevert = 34;						// <!> "Revert" (to previous version)

const short cShowClipboard = 35;				// <!> "Show Clipboard"/ "Hide Clipboard"

const short cQuit = 36;							// <!> "Quit"


//----------------------------------------------------------------------------------------
// Edit-menu commands: For the following command numbers, we must guarantee that <command
// number> - cEditBase = <appropriate number to pass to SystemEdit>.This relationship is
// enforced in TApplication.IApplication.
//----------------------------------------------------------------------------------------

const short cEditBase = 101;					// start of standard editing commands

const short cUndo = 101;						// <!> "Undo <command>"/ "Redo <command>"

const short cEditSep = 102;						// line separating UNDO from CUT

const short cCut = 103;							// <&> "Cut"

const short cCopy = 104;						// <&> "Copy"

const short cPaste = 105;						// <&> "Paste"

const short cClear = 106;						// <-> "Clear"

const short cEditLast = cClear;

const short cSelectAll = 110;					// <&> "Select All"

const short cTyping = 120;						// for use in a TTypingCommand

const short cMouseCommand = 121;				// generic mouse command

const short cStyleChange = 130;					// "Menu" command for style change

const short cRememberStyle = 131;				// "Menu" command for remembering present
                                                // style


//----------------------------------------------------------------------------------------
// Finder pseudo-commands
//----------------------------------------------------------------------------------------

const short cFinderNew = 40;					// <!> The user selected the tool icon in
                                                // the Finder and chose "Open"

const short cFinderPrint = 41;					// <!> The user selected document icons in
                                                // the Finder and chose "Print"

const short cFinderOpen = 42;					// <!> The user selected document icons in
                                                // the Finder and chose "Open"

const short cFinderQuit = 43;					// <!> The user selected "ShutDown" and
                                                // the the Finder sends a 'quit'
                                                // AppleEvent'


//----------------------------------------------------------------------------------------
// EDITION MANAGER AppleEvent COMMAND CONSTANTS
//----------------------------------------------------------------------------------------

const short cSectionRead = 50;					// subscribers should read themselves
                                                // afresh from disk

const short cSectionWrite = 51;					// publishers should write themselves to
                                                // an edition file

const short cSectionScroll = 52;				// the user selects Open Publisher for a
                                                // subscriber

const short cSectionCancel = 53;				// cancel this section in a document


//----------------------------------------------------------------------------------------
// File-menu printing commands
//----------------------------------------------------------------------------------------

const short cPrFileBase = 176;					// Command numbers between cPrFileBase and
const short cPrFileMax = 195;					// cPrFileMax are reserved for printing.

const short cPageSetup = 176;					// <!> "Page Setup"

const short cPrintOne = 177;					// <!> "Print One"

const short cPrint = 178;						// <!> "Print"

const short cPrintToFile = 179;					// <!> "Print to file"

const short cPrintSpoolFile = 190;				// <-> "Print spooled file"

const short cPrViewBase = 201;					// Command numbers between cPrViewBase and
                                                // cPrViewMax are printing commands
                                                // applied to a displayed view in the
                                                // fTarget chain

const short cPrViewMax = 250;

const short cShowBorders = 199;					// <!> Toggle. "Show view borders"--also
                                                // usable outside Debug menu


//----------------------------------------------------------------------------------------
// Zooming commands
//----------------------------------------------------------------------------------------

const short cReduce50 = 301;					// <-> "Reduce 50%"

const short cReduceToFit = 302;					// <-> "Reduce to Fit"

const short cShowFullSize = 303;				// <-> "Show Full Size"


//----------------------------------------------------------------------------------------
// Control tracking
//----------------------------------------------------------------------------------------

const short cTrackingControl = 400;


//----------------------------------------------------------------------------------------
// Debug-menu commands. Flag toggled by "cFoo" is called "gFoo" for any "Foo"
//----------------------------------------------------------------------------------------

const short cExperimenting = 901;				// <!> Toggle. "Enable experimental
                                                // features"

const short cReportMenuChoices = 902;			// <!> Toggle. "Trace menu commands"--
                                                // WriteLn menu choices

const short cIntenseDebugging = 904;			// <!> Toggle. "Intense Debugging"

const short cTraceSetupMenus = 905;				// <!> Toggle. "Allow Trace of Menu
                                                // Setups" if gTrace is also on

const short cTraceIdle = 906;					// <!> Toggle. "Allow Trace during Idle"
                                                // if gTrace is also on

const short cDebugPrinting = 912;				// <!> Toggle. "Debug Printing"

const short cReportEvt = 914;					// <!> Toggle. "Report Events"--WriteLn
                                                // events received from queue

const short cDoFirstClick = 915;				// <!> Toggle. "'Do First Click' for this
                                                // window"

const short cVarClipPicSize = 916;				// <!> Toggle. "Scale Pictures in
                                                // Clipboard to Window"

const short cRefreshFrontWindow = 917;			// <!> Action. "Refresh Front Window"

const short cNewInspectorWindow = 918;			// <!> Action. "Open new inspector window"

const short cModalToggle = 919;					// <!> Toggle. "Make front window modal/
                                                // modeless"

const short cDebugWind = 913;					// <!> Action. "Show Debug Window"

const short cEnterMacAppDebugger = 920;			// <!> Action. "Enter the MacApp Debugger"

const short cSetSysJust = 921;					// <!> Action. "Switch system
                                                // justification"

const short cDebugFont = 922;

const short cDebugSize = 923;

const short cToggleFlag = 924;					// Hierarchial Flags menu


//----------------------------------------------------------------------------------------
// MacAppBuzz - StringResource
//----------------------------------------------------------------------------------------

const short kIDBuzzString = 250;				// std list of buzzwords stored as 'STR#'
                                                // resources. This string list is reserved
                                                // for MacApp; you should choose a
                                                // different resource ID for your strings.


//----------------------------------------------------------------------------------------
// Indices for strings in the buzz-string resource
//----------------------------------------------------------------------------------------

const short bzSaveAs = 1;						// 'Save This Document As:'

const short bzSaveCopy = 2;						// 'Save a Copy In:'

const short bzShowClip = 3;						// 'Show Clipboard'

const short bzHideClip = 4;						// 'Hide Cllpboard'

const short bzUndo = 5;							// 'Undo x'

const short bzRedo = 6;							// 'Redo x'

const short bzCantDraw = 7;						// 'Unable to draw contents of window'

const short bzUntitled = 8;						// name of untitled document; if blank
                                                // then this is taken from the window
                                                // title template; if this includes the
                                                // string '<<<>>>' then MacApp will
                                                // substitute a number for the string
                                                // (e.g., if the buzz string contains
                                                // 'Untitled-<<<>>>' then untitled windows
                                                // will be named 'Untitled-1', Untitled-2,

const short bzClosing = 9;						// 'closing'

const short bzQuitting = 10;					// 'quitting'

const short bzCantUndo = 11;					// 'Cant Undo'

const short bzSaveAnyways = 12;					// 'save'

const short bzRevertAnyways = 13;				// 'revert'

const short bzHideBorders = 14;					// 'Hide Borders'

const short bzShowBorders = 15;					// 'Show Borders'

const short bzSubscriberOptions = 16;			// 'Subscriber Options'

const short bzPublisherOptions = 17;			// 'Publisher Options'


//----------------------------------------------------------------------------------------
// DebugBuzz - StringResource
//----------------------------------------------------------------------------------------

const short kDebugBuzzStrings = 251;			// Debug buzzwords STR# resource

const short bzMakeModal = 1;					// 'Make Front Window Modal'

const short bzMakeModeless = 2;					// 'Make Front Window Modeless'

const short bzDoFirstClick = 3;					// 'Do First Click For This Window'

const short bzDontDoFirstClick = 4;				// 'Don't Do First Click For This Window'

const short bzSetRightSysJust = 5;				// Set system to right justification

const short bzSetLeftSysJust = 6;				// Set system to left justification


//----------------------------------------------------------------------------------------
// Resource ID's
//
// Negative numbers and numbers up to 127 are reserved by Apple
// Positive numbers from 128 up to 999 are reserved by MacApp
//----------------------------------------------------------------------------------------

const short kDefaultCredits = 1001;				// People responsible for the application

const short kIDClipWindow = 200;				// window displaying the Clipboard

const short kScrollBarId = 202;					// for when we create scrollers from
                                                // resources

const short kDebugParamsID = 300;				// debug window

const short kAEDispatchTable = 400;				// AppleEvents dispatch table


//----------------------------------------------------------------------------------------
// View Identifiers: These follow the same reservation rules as resource types
//----------------------------------------------------------------------------------------

const short kDefaultWindowID = 1001;			// Window view resource installed by
                                                // default in TDocument

const short kDefaultViewID = 1002;				// View resource installed by default in
                                                // TDocument

const unsigned long kIDDefaultView = 'DFLT';	// View ID of default view

const unsigned long kIDClipView = 'CLIP';		// identifier of the clipboard "orhanage"


//----------------------------------------------------------------------------------------
// Object Signatures: These follow the same reservation rules as resource types The
// signature is used to search the signature table and create an object "by signature"If
// the signature's entry in the table has been updated with a new class then an object of
// that class will be created.
//----------------------------------------------------------------------------------------

const unsigned long kStdDocument = 'docu';
const unsigned long kStdTracker = 'trak';
const unsigned long kStdList = 'list';

const unsigned long kStdButton = 'butn';
const unsigned long kStdCheckBox = 'chkb';
const unsigned long kStdCluster = 'clus';
const unsigned long kStdControl = 'cntl';
const unsigned long kStdDefaultView = 'dflt';
const unsigned long kStdDialogView = 'dlog';
const unsigned long kStdEditText = 'edit';
const unsigned long kStdGridView = 'grid';
const unsigned long kStdIcon = 'icon';
const unsigned long kStdNumberText = 'nmbr';
const unsigned long kStdPattern = 'patn';
const unsigned long kStdPicture = 'pict';
const unsigned long kStdPopup = 'popp';
const unsigned long kStdRadio = 'radb';
const unsigned long kStdScroller = 'scrl';
const unsigned long kStdSScrollBar = 'sbar';
const unsigned long kStdStaticText = 'stat';
const unsigned long kStdTEView = 'tevw';
const unsigned long kStdTextGridView = 'txtg';
const unsigned long kStdTextListView = 'lstg';
const unsigned long kStdView = 'view';
const unsigned long kStdWindow = 'wind';


//----------------------------------------------------------------------------------------
// Choices: Negative numbers and positive numbers up to 999 are reserved by MacApp
//----------------------------------------------------------------------------------------

const short mOKHit = 1;
const short mCancelHit = 2;
const short mButtonHit = 3;
const short mCheckBoxHit = 4;
const short mClusterHit = 5;
const short mEditTextHit = 6;
const short mIconHit = 7;
const short mListItemHit = 8;
const short mListScrollBarHit = 9;
const short mPictureHit = 10;
const short mPopupHit = 11;
const short mRadioHit = 12;
const short mStaticTextHit = 13;
const short mHScrollBarHit = 14;
const short mVScrollBarHit = 15;
const short mEditTabKey = 16;
const short mEditReturnKey = 17;
const short mEditEnterKey = 18;
const short mPatternHit = 19;
const short mControlHit = 20;
const short mCancelKey = 21;
const short mDefaultKey = 22;
const short mSmallIconHit = 23;
const short mBecameWindowTarget = 24;
const short mResignedWindowTarget = 25;
const short mBecameApplicationTarget = 26;
const short mResignedApplicationTarget = 27;

const unsigned long kNoIdentifier = '    ';
const short kNoTemplate = -1;
const short kNoResource = -1;


//----------------------------------------------------------------------------------------
// Lists
//----------------------------------------------------------------------------------------

const Boolean kIterateForward = True;

const Boolean kIterateBackward = !kIterateForward;

const long kEmptyIndex = 0;						// Value to use when no valid index is
                                                // available Indexes are always positive

typedef long ArrayIndex;						// At least always positive ( in this
                                                // universe ) !!! kEmptyIndex..MaxLong
                                                // would be nice


//----------------------------------------------------------------------------------------
// Miscellaneous
//----------------------------------------------------------------------------------------

const unsigned long kStdMainFileType = 'TEXT';	// the file type identifier used by
                                                // default

const short kYesButton = 1;						// the 'Yes' button in various dialogs

const short kNoButton = 3;						// the 'No' button in various dialogs

const short kLowSpaceInterval = 2 * 60 * 60;	// default low space msg interval (2
                                                // seconds)

const short kStdScrollUnit = 16;				// default value of fScrollUnit used by
                                                // scrollers

const short kSBarSize = 16;

const short kSBarSizeMinus1 = kSBarSize - 1;

const short kStdSzSBar = 16;					// width/ height of a standard vertical/
                                                // horizontal scroll bar

const short kStdSzMinus1SBar = kStdSzSBar - 1;

const short kStdStaggerAmount = 16;				// standard # of pixels to stagger windows

const long kMaxIdleTime = MAXLONGINT;			// Default tick count sent to
                                                // WaitNextEvent


//----------------------------------------------------------------------------------------
// Numbers to pass to SystemEdit
//----------------------------------------------------------------------------------------

const short kSysUndo = 0;
const short kSysCut = 2;
const short kSysCopy = 3;
const short kSysPaste = 4;
const short kSysClear = 5;


#if qDebug
const short kRsrcCheckInterval = 100;
#endif

const Boolean kBuild = True;					// Pass to TWindow.BuildWindowRgns

const Boolean kAllowApplicationToSleep = True;	// Pass to TApplication.PollToolboxEvent
                                                // if the application is allowed to sleep
                                                // if there are no events pending.


//----------------------------------------------------------------------------------------
// Printing
//----------------------------------------------------------------------------------------

const Boolean kSquareDots = True;				// Square dots in Printing

const Boolean kFixedSize = True;				// Fixed Page Size in Printing

const short kPrintInfoSize = 120;				// size in bytes of printInfo record


//----------------------------------------------------------------------------------------
// size in bytes of the overhead for a resource file; kRsrcFileOverhead is added in
// TDocument.DoNeedDiskSpace if the document uses the rsrc fork. You should add
// kRsrcTypeOverhead for each unique type you use plus kRsrcOverhead for each resoruce you
// use plus the total bytes in all the resources plus the length of all the names for
// named resources.
//----------------------------------------------------------------------------------------

const short kRsrcFileOverhead = 256 + 30;		// fixed part of resource map

const short kRsrcTypeOverhead = 8;				// overhead for each resource type

const short kRsrcOverhead = 16;					// overhead for each resource

const short kViewRsrcExpandAmt = 1024;			// ??? Is 1K enough for expansion, or too
                                                // much maybe???

//----------------------------------------------------------------------------------------
// debugging info
//----------------------------------------------------------------------------------------
const short kDebugFont = 4;						// Font for Debug Window (monaco)
const short kDebugSize = 9;						// Font size for Debug Window

//----------------------------------------------------------------------------------------
// argument of TDocument.DoMakeViews
//----------------------------------------------------------------------------------------
const Boolean kForDisplay = False;
const Boolean kForPrinting = !kForDisplay;

//----------------------------------------------------------------------------------------
// arguments of TScroller.IScroller and TViewServer.NewPaletteWindow
//----------------------------------------------------------------------------------------
const Boolean kWantHScrollBar = True;
const Boolean kWantVScrollBar = True;

//----------------------------------------------------------------------------------------
// arguments of TDocument.Save
//----------------------------------------------------------------------------------------
const Boolean kAskForFilename = True;
const Boolean kMakingCopy = True;
const Boolean kSwitchToTarget = !kMakingCopy;

//----------------------------------------------------------------------------------------
// argument of TApplication.SetUndoText
//----------------------------------------------------------------------------------------
const Boolean kShowUndo = True;	
const Boolean kShowRedo = False;	
const Boolean kShowCantUndo = kShowUndo;

//----------------------------------------------------------------------------------------
// argument of TDocument.IDocument
//----------------------------------------------------------------------------------------
const Boolean kUsesDataFork = True;
const Boolean kUsesRsrcFork = True;	
const Boolean kDataOpen = True;	
const Boolean kRsrcOpen = True;	

//----------------------------------------------------------------------------------------
// argument of TDocument.OpenDocFile
//----------------------------------------------------------------------------------------
const Boolean kKeepingOpen = True;	
const Boolean kReadingFile = True;	

const Boolean kVisible = True;	
const Boolean kInvisible = False;	

//----------------------------------------------------------------------------------------
// arguments to routines with a 'redraw' parameter
//----------------------------------------------------------------------------------------
const Boolean kRedraw = True;
const Boolean kDontRedraw = False;

//----------------------------------------------------------------------------------------
// arguments to routines with an 'invalidate' parameter
//----------------------------------------------------------------------------------------
const Boolean kInvalidate = True;
const Boolean kDontInvalidate = False;


//----------------------------------------------------------------------------------------
// Declare as external the classes that you are only referencing (e.g. instance vars,
// parameter types or function results) that are not covered by the USES in the interface
// section.
//----------------------------------------------------------------------------------------

class TView;

class TWindow;

class TDocument;

class TPrintHandler;

class TDynamicArray;


//----------------------------------------------------------------------------------------
// SignatureRec: Used by the signature mapper
//----------------------------------------------------------------------------------------

struct SignatureRec
{
	IDType itsSignature;
	ObjClassID itsClassID;
};

typedef SignatureRec* SignatureRecPtr;


//----------------------------------------------------------------------------------------
// Globacl variable declarations
//----------------------------------------------------------------------------------------

extern TextStyle gApplicationStyle;
	// Application's default text style

extern Boolean gCouldPrint;
	// whether Printer code is accessible to the application
	
#if qExperimentalAndUnsupported
extern Boolean gEnableDoubleBuffering;
	// TRUE to enable automatic double buffering when drawing/ scrolling views
#endif

extern Boolean gOldChooserFlag;
	// The state of the chooser alert flag when the application started up.

extern TDynamicArray* gSignatures;
	// Standard object signatures and their classIDs

extern TextStyle gSystemStyle;
	// System's default text style

extern WindowPtr gWorkPort;
	// Pointer to a window that is created during initialization, but never shown or
    // enumerated.

extern RgnHandle gTempRgn;
	// gTempRgn is a temporary RgnHandle created in InitUMacApp. When debugging: before
    // using gTempRgn, call UseTempRgn and when done call DoneWithTempRgn; this ensures
    // that 2 routines do not try to use gTempRgn and the same time

extern RgnHandle gTempRgn2;
	// gTempRgn is a temporary RgnHandle created in InitUMacApp. When debugging: before
    // using gTempRgn, call UseTempRgn and when done call DoneWithTempRgn; this ensures
    // that 2 routines do not try to use gTempRgn and the same time

#if qDebug
extern Boolean gBusyTempRgn;
	// Is gTempRgn in use?

extern Str255 gUsedBy;
	// The routine using gTempRgn

extern Boolean gBusyTempRgn2;
	// Is gTempRgn in use?

extern Str255 gUsedBy2;
	// The routine using gTempRgn
#endif


//----------------------------------------------------------------------------------------
// These are really private but, they're here in case access is needed.
//----------------------------------------------------------------------------------------

extern StringHandle pCopyright;

extern TrapPatch pETSPatch;					// patch for ExitToShell

extern FailInfo pFi;						// Outermost failure handler


//----------------------------------------------------------------------------------------
// Debugging
//----------------------------------------------------------------------------------------

#if qDebug
pascal void DoneWithTempRgn();
	// Indicates that gTempRgn is no longer in use. Call this only if qDebug is true.

pascal void DoneWithTempRgn2();
	// Indicates that gTempRgn is no longer in use. Call this only if qDebug is true.

pascal void EntDebugger(Boolean entering);
	// Call to enter the MacApp Debugger

pascal void UseTempRgn(const Str255& byWhom);
	// Call this when you are about to use gTempRgn and qDebug is true. Used with
    // DoneWithTempRgn will prevent you from trying to use gTempRgn from two places at the
    // same time.

pascal void UseTempRgn2(const Str255& byWhom);
	// Call this when you are about to use gTempRgn and qDebug is true. Used with
    // DoneWithTempRgn will prevent you from trying to use gTempRgn from two places at the
    // same time.
#endif qDebug

pascal long LookupSymbol(Str255& sym);	
	// Call to see if anyone knows a reference for the given symbol


//----------------------------------------------------------------------------------------
// Windows
//----------------------------------------------------------------------------------------

pascal WindowPtr FreeIfWMgrWindow(WindowPtr w,Boolean dispose);
	// Calls DisposeWindow if dispose is true, else calls CloseWindow. Does nothing if w
    // is nil. Returns NIL for convenient assignment back to the reference passed in.

pascal Boolean ParseTitleTemplate(Str255& itsTemplate,
								  short& preDocname,
								  short& constTitle);
	// Used in TWindow.IWindow to parse the template for window titles. Returns TRUE if it
    // changed the template. preDocname is start of document name in string; constChars is
    // total # characters of window title that are constants. (These values are fixed
    // regardless of the document name.) If preDocname is 0, then the entire title is
    // constant.

pascal Boolean SubstituteInTitle(Str255& title,
										const Str255& newStuff,
										short preDocname,
										short constTitle);
// Substitutes newStuff into the template as parsed by ParseTitleTemplate; returns TRUE if
// a substitution was made.


//----------------------------------------------------------------------------------------
// Object Creation Utilities
//----------------------------------------------------------------------------------------

pascal void RegisterStdType(const Str255& typeName, IDType signature);
	// Associates the given class with a signature.

pascal ObjClassID GetClassIDFromSignature(IDType signature);
	// Returns the classID associated with a given signature, or kNILClass

pascal ObjClassID DetermineClassID(IDType signature, const MAName& className);
	// Returns the classID associated with the given class name, if className is not
    // empty. Returns the classID associated with the given signature, if className is
    // empty

pascal TObject* NewStdObject(IDType signature);
	// Returns the prototype object for the given signature.

pascal TObject* NewObjectBySignature(IDType signature, const MAName& className);
	// Returns an instance of class "className" if className is non-empty. Returns the
    // prototype object for the given signature if className is empty


//----------------------------------------------------------------------------------------
// Miscellaneous
//----------------------------------------------------------------------------------------

pascal void DoAlertKeyDown(DialogPtr theDialog, short itemNo);
	// Given a dialog pointer and button item, simulate pressing the button

pascal void DoInitUMacApp(void);
	// Should be called only by InitUMacApp. It is here because this doesn't need to be
    // universal code but InitUMacApp does.

pascal void GetAlertButtonTitle(DialogPtr theDialog, short itemNo, Str255& theTitle);
	// Given a dialog pointer and a button item, return the title of that button

pascal Boolean CompareAlertKeysToItem(DialogPtr theDialog, Str31& theChars, short& itemHit);
	// Compares the buffered multi-byte chars to the first character of each button title
    // 1st button in alert (by convention == "OK"). 2nd button in alert (by convention ==
    // "Cancel"). 3rd button in alert (by convention == "No")

pascal Boolean MacAppAlertFilter(DialogPtr theDialog,
										EventRecord& theEvent,
										short& itemHit);
	// Default alert filter used throughout MacApp. Lets you answer yes and no to dialogs
    // with keystrokes. Handles "cmd-." and escape. If the alert has not been called in a
    // failure sequence then the application also handles update and activate events and
    // idles.

pascal void ApplicationBeep();
	// If gApplication is not NIL just calls gApplication.Beep else it does a SysBeep.
    // Removes a circular reference for UFailure.

pascal DialogPtr GetNewCenteredDialog(short dialogID,
											 Ptr dStorage,
											 WindowPtr behind);
	// The same as GetNewDialog except it centers the dialog on the screen and sets the
    // cursor to an arrow.

pascal void GetTextStyleFontInfo(const TextStyle& theTextStyle,
										FontInfo& theFontInfo,
										short& theFontHeight);
	// Returns the FontInfo record for the font/ face/ size specified in theTextStyle.Uses a
    // temporary port so you don't have to. Also returns the font's line height.

pascal void ExitMacApp();
	// Call this if for some reason you want to immediately exit the application. It calls
    // gApplication.Terminate, cleans up some other internal stuff, and then calls
    // ExitToShell. (Normally, you would not call this, because MacApp takes care of
    // terminating the application.)

pascal void InstallIfPrintHandler(TPrintHandler* aPrintHandler, TView* aView);
	// Install a clone of aPrintHandler into the view if aPrintHandler is not the same as
    // the null print handler. (Typically if printing is initialized.)This lets us install
    // a print handler if printing is being used in the application and do NOTHING if it
    // is not.

pascal RgnHandle MakeNewRgn();
	// Calls NewRgn, then FailNIL

pascal void CleanupMacApp();
	// Called as the final step of TApplication.Run

#endif


