// UMacAppUtilities.h
// Copyright  1984-1991 by Apple Computer Inc. All rights reserved.

#ifndef __UMACAPPUTILITIES__
#define __UMACAPPUTILITIES__

#ifndef __PASCALDEFS__
#include <PascalDefs.h>
#endif

#ifndef __UGEOMETRY__
#include <UGeometry.h>
#endif

#ifndef __TEXTEDIT__
#include <TextEdit.h>
#endif

#ifndef __UMACAPPUNIVERSAL__
#include <UMacAppUniversal.h>
#endif

#ifndef __ULOMEM__
#include <ULoMem.h>
#endif

#ifndef __UITERATOR__
#include <UIterator.h>
#endif

#ifndef __EDITIONS__
#include <Editions.h>
#endif


//----------------------------------------------------------------------------------------
// Global constants declarations. Build options.
//----------------------------------------------------------------------------------------

extern const char* kCopyright;

#if !qDebug
const int gPreCondition = False;
const int gPostCondition = False;
const int gExperimenting = False;
const int gReportMenuChoices = False;
const int gIntenseDebugging = False;
const int gDebugPrinting = False;
const int gReportEvt = False;
const int gAssumeFocused = False;
const int gShowInvalidations = False;
const int gShowCursorRegion = False;
const int gShowHelpRegion = False;
const int gShowSleepRegion = False;
#endif

#if qDebug
extern Boolean gPreCondition;
extern Boolean gPostCondition;
extern Boolean gAssumeFocused;
extern Boolean gDebugPrinting;
extern Boolean gExperimenting;
extern Boolean gIntenseDebugging;
extern Boolean gReportEvt;
extern Boolean gReportMenuChoices;
extern short gRsrcCheck;
extern Boolean gShowCursorRegion;
extern Boolean gShowHelpRegion;
extern Boolean gShowInvalidations;
extern Boolean gShowSleepRegion;
extern Boolean gTraceIdle;
#endif


//----------------------------------------------------------------------------------------
// Global variable declarations.
//----------------------------------------------------------------------------------------

extern short gApplicationRefNum;

extern ConfigRecord gConfiguration;

typedef char BoolStrType[6];

extern BoolStrType gBoolString[2];

extern ProcPtr gFieldToStrRtn;

extern ProcPtr gFieldToCountRtn;

extern Boolean gToolBoxInitialized;

extern Boolean gUDialogInitialized;

extern Boolean gUGridViewInitialized;

extern Boolean gUPrintingInitialized;

extern Boolean gUTEViewInitialized;

extern Ptr gStrippedAddress;

extern Boolean gDeadStripSuppression;

extern TEHandle gMATextBoxTE;

extern WordBreakProcPtr gTEDefaultWordBreak;

extern VHSelect gOrthogonal[2];

extern Rect gZeroRect;

extern Point gZeroPt;

extern VPoint gZeroVPt;

extern VRect gZeroVRect;

extern RGBColor gRGBBlack;

extern RGBColor gRGBWhite;

extern RGBColor gRGBRed;

extern RGBColor gRGBGreen;

extern RGBColor gRGBBlue;


//----------------------------------------------------------------------------------------
// CWMgrIterator: A simple iterator for window lists.  Used by EachWMgrWindowDo.
//----------------------------------------------------------------------------------------

class CWMgrIterator : public CIterator
{
private:
	WindowPtr fCurrentWindow;

public:
	CWMgrIterator(void);

	virtual Boolean More(void);					// override
		// Returns TRUE if there are more elements to iterate over

	virtual void Reset(void);					// override
		// Resets the iterator to begin again

	inline WindowPtr CurrentWMgrWindow(void);
		// returns the current window

	inline WindowPtr FirstWMgrWindow(void);
		// Resets the iterator to begin again and returns the first window in the window list

	inline WindowPtr NextWMgrWindow(void);
		// increments and then returns the window in the window list

protected:
	virtual void Advance(void);					// override
		// Advances the iteration
};


//----------------------------------------------------------------------------------------
// CWMgrIterator inline method definitions.
//----------------------------------------------------------------------------------------

inline CWMgrIterator::CWMgrIterator()
{
	fCurrentWindow = GetWindowList();
}

inline WindowPtr CWMgrIterator::CurrentWMgrWindow()
{
	return fCurrentWindow;						// Always return the current window
}

inline WindowPtr CWMgrIterator::FirstWMgrWindow()
{
	this->Reset();
	return fCurrentWindow;						// Always return the first window
}

inline WindowPtr CWMgrIterator::NextWMgrWindow()
{
	this->Advance();
	return fCurrentWindow;
}


//----------------------------------------------------------------------------------------
// Global assembly inline function definitions.
//----------------------------------------------------------------------------------------

pascal long GetParmBlockPtr() = { 0x2e88 };

pascal long GetA5() = { 0x2e8d };
	
pascal Ptr GetCurStackFramePtr() = { 0x2e8e };
	
pascal Ptr GetCurStackTop() = { 0x2e8f };
	
pascal Ptr PRStr(const Str255& astr) = { 0x2e9f };

pascal const Str255& AtStr(const Str255& astr) = { 0x2e9f };

pascal MANamePtr AtMAName(const MAName& astr) = { 0x2e9f };
													 
pascal long IntMultiply(short x, short y) = { 0x301f, 0xc1df, 0x2e80 };

//----------------------------------------------------------------------------------------
// Global function declarations
//----------------------------------------------------------------------------------------

pascal Boolean CompareMultiByteChars(const Str31& first,
									 const Str31& second,
									 Boolean caseSens);
	// Special case single byte characters and allow case insensitive comparisons

pascal short CompareStrings(const Str255& first,
							const Str255& second);

pascal void CopyStr255(const Str255& fmStr,
					   Ptr toAddr);

pascal void EachWMgrWindowDo(pascal void(* DoToWMgrWindow)(WindowPtr theWMgrWindow,
														   void* staticLink),
							 void* staticLink);

pascal WindowPtr FindWindowBefore(WindowPtr theWindow);

pascal void DefaultSize(short& theSize);

pascal Handle DisposeIfHandle(Handle aHandle);

pascal PicHandle DisposeIfPicHandle(PicHandle aPicHandle);

pascal Ptr DisposeIfPtr(Ptr aPtr);

pascal RgnHandle DisposeIfRgnHandle(RgnHandle aRgnHandle);

pascal SectionHandle DisposeIfSectionHandle(SectionHandle aSectionHandle);

pascal Boolean EqualBlocks(Ptr first,
						   Ptr second,
						   short theSize);

pascal void FieldToString(const Str255& fieldLabel,
						  void* theData,
						  short fieldType,
						  pascal void(* ProcessField)(const Str255& theLabel,
													  const Str255& theString,
													  void* theData,
													  short theType,
													  void* staticLink),
						  void* staticLink);
						  
pascal void FieldToCount(short& count, short fieldType);

pascal short GetActualJustification(short justification);

pascal void GetDeskTopRegion(RgnHandle deskTopRgn);

pascal short GetFontNum(const Str255& fontName);

pascal void GetIfColor(RGBColor& aColor);

pascal void GetIfBkColor(RGBColor& aColor);

pascal void GetPortTextStyle(TextStyle& theTextStyle);

pascal void GetPortFontInfo(short fontNum, Str255& fontName, short& fontSize);

pascal Boolean IsColorPort(GrafPtr port);

pascal Boolean IsAResource(Handle h);

pascal Boolean IsHandle(Handle h);

pascal Boolean IsHandleLocked(Handle h);

#if qDebug
	pascal Boolean IsHandlePurged(Handle h);
#else
	inline Boolean IsHandlePurged(Handle h)
	{
		return (!*h);
	}
#endif

pascal short GetWindowVariant(WindowPtr theWindow);

pascal short LengthRect(const Rect& r, VHSelect vhs);

pascal SignedByte LockHandleHigh(Handle h);

pascal VHSelect LongerSide(Rect& r);

pascal VHSelect LongerVSide(VRect& r);

pascal void LIntToHex(long decNumber, Str31& hexNumber, short noOfDigits);

pascal short MAGetFontInfo(FontInfo& theFontInfo);

pascal short MAUseResFile(short refNum);

pascal void MATextBox(Ptr text,
					  long itsLength,
					  const Rect& box,
					  short itsJust,
					  Boolean autoWrap,
					  ProcPtr wordBreak,
					  Boolean eraseFirst,
					  Boolean spaceForCaret);

pascal void MADrawString(const Str255& s,
						 const Rect& box,
						 short justification);

long Max(long a, long b);

long Min(long a, long b);

pascal long MinMax(long MinVal,
				   long expression,
				   long MaxVal);

pascal void NumberToHex(long theNumber,
						Str255& hexString,
						short hexDigits);

pascal Boolean IsCommandKeyDown();

pascal Boolean IsControlKeyDown();

pascal Boolean IsOptionKeyDown();

pascal Boolean IsShiftKeyDown();

pascal long PinOnRect(const Rect& theRect, Point thePt);

pascal void PinOnVRect(const VRect& theRect, const VPoint& thePt, VPoint& thePin);

pascal long StripLong(void* address);

pascal void PointerToHex(long theNumber, Str255& hexString, short hexDigits);

pascal Boolean RectsNest(const Rect& outer, const Rect& inner);

pascal Boolean VRectsNest(const VRect& outer, const VRect& inner);

pascal long RoundUp(long aNumber, short aModulus);

pascal short SetKeyScript(short newKeyScript);

pascal void SetIfColor(const RGBColor& aColor);

pascal void SetIfBkColor(const RGBColor& aColor);

pascal void SetPortTextStyle(const TextStyle& theTextStyle);

pascal void SetTextStyle(TextStyle& theTextStyle,
						 short theFont,
						 /* Style */
						 short theStyle,
						 short theSize,
						 const RGBColor& theColor);

pascal void StdFieldToString(const Str255& fieldLabel,
							 void* theData,
							 short fieldType,
							 pascal void(* ProcessField)(const Str255& theLabel,
														 const Str255& theString,
														 void* theData,
														 short theType,
														 void* staticLink),
							 void* staticLink);

pascal void StdFieldToCount(short& count,
							short fieldType);

pascal short UprChar(short ch);

pascal void UprStr255(Str255& s);

pascal void UprMAName(MAName& s);

pascal short LowerChar(short ch);

pascal void LowerStr255(Str255& s);

pascal void UseSelectionColor();

pascal void UseROMMap(Boolean resLoad);

pascal void WithApplicationResFileDo(pascal void(* DoWithResFile)(void* staticLink),
									 void* staticLink);

pascal long NumBlocks(long numBytes, long blkSize);

pascal Boolean CanReadLn();

pascal Boolean CanWriteLn();

pascal short ReadInteger(const Str255& prompt);

pascal Boolean ReadYesNo(const Str255& prompt);

pascal Boolean VerboseIsHandle(Handle h);

pascal void WritePt(Point pt);

pascal void WritePtr(long val);

pascal void WriteRect(const Rect& r);

pascal void WriteBoolean(Boolean b);

pascal void WriteVPt(const VPoint& pt);

pascal void WriteVRect(const VRect& r);

pascal void WrLblPt(const Str255& aLabel, Point pt);

pascal void WrLblPtr(const Str255& aLabel, long val);

pascal void WrLblRect(const Str255& aLabel, const Rect& r);

pascal void WrLblBoolean(const Str255& aLabel, Boolean b);

pascal void WrLblVPt(const Str255& aLabel, const VPoint& pt);

pascal void WrLblVRect(const Str255& aLabel, const VRect& r);

pascal void WriteSig(IDType theID);

pascal void WrLblSig(const Str255& theLabel, IDType theID);

pascal void WrLblHandleContents(const Str255& aLabel, Handle theHandle);

pascal void WriteHandleContents(Handle theHandle);

pascal void WriteHexInt(short theInt);

pascal void WrLblHexInt(const Str255& theLabel, short theInt);

pascal void WriteHexLongint(long theLongint);

pascal void WrLblHexLongint(const Str255& theLabel, long theLongint);

#endif


