#ifndef __UPASCALOBJECT__
#define __UPASCALOBJECT__

//  UPascalObject.h
//  Copyright  1984-1991 Apple Computer, Inc. All rights reserved.

#ifndef __PASCALDEFS__
#include <PascalDefs.h>
#endif

#ifndef __MACAPPTYPES__
#include <MacAppTypes.h>
#endif

#ifndef __MEMORY__
#include <Memory.h>
#endif


//----------------------------------------------------------------------------------------
// Failure type Constants
//----------------------------------------------------------------------------------------

const short kFailNone = 1;
const short kFailAbstract = 2;
const short kFailCoercion = 3;
const short kFailMethNotFound = 4;

extern const char* kInvalidObj;					// return value from LookupObjName if not
                                                // an object

const short kNilClass = 0;						// Value for superclass of the root object
                                                //


//----------------------------------------------------------------------------------------
// Forward and external class declarations
//----------------------------------------------------------------------------------------

class TObject;


//----------------------------------------------------------------------------------------
// Typedefs
//----------------------------------------------------------------------------------------

typedef long ChangeID;

#if qModelFar
typedef long ObjClassID;						// Object Class identifier. The first 4
                                                // bytes of each object contain the class
                                                // of the object as an ObjClassID.
#else
typedef short ObjClassID;						// Object Class identifier. The first two
                                                // bytes of each object contain the class
                                                // of the object as an ObjClassID.
#endif

typedef ObjClassID *ObjClassIDPtr, **ObjClassIDHandle;

typedef TObject* TObjectPtr;					// basic pointer to the object structure,
                                                // same as "this"

typedef pascal TObjectPtr (*ObjAllocateType)(ObjClassID classID);

typedef pascal void (*ObjDisposeType)(TObject* obj);

typedef pascal Boolean (*IsObjectType)(TObject* obj);

typedef pascal void (*DoToClassType)(ObjClassID theClass, void* staticLink);


//----------------------------------------------------------------------------------------
// ObjProcs: Structure containing pointers to bottleneck routine for the object system.
//----------------------------------------------------------------------------------------

struct ObjProcs
{
	ObjAllocateType AllocateProc;				// Routine to allocate objects
	ObjDisposeType DisposeProc;					// Routine to dispose objects
	IsObjectType IsObjectProc;					// Routine to test objectHood
};


//----------------------------------------------------------------------------------------
// Global variable declarations
//----------------------------------------------------------------------------------------

extern ObjProcs gObjProcs;						// Actual bottlenecks for object system.
                                                // Set in InitUObject


//----------------------------------------------------------------------------------------
// Global function declarations
//----------------------------------------------------------------------------------------

pascal Boolean AllocateObjectsFromPerm(Boolean allocateFromPerm);
	// Set to TRUE to make object allocation calls use Permanent memory (the default). Set
    // to false to make object allocation calls use temporary memory (if they cannot be
    // allowed to fail).Returns old state

pascal Boolean DisciplineMethodCalls(Boolean discipline);
	// Set to true to discipline all method calls that go through the dispatcher. Function
    // returns old setting.

pascal void EachClassDo(DoToClassType DoToClass, void* staticLink);
	// Calls DoToClass for each class in the application

pascal void EachSubClassDo(ObjClassID testClass,
						   DoToClassType DoToClass,
						   void* staticLink);
	// Calls DoToClass for subclass of the test class. Uses EachClassDo, so the order of
    // iteration is not based on the class hierarchy but on the sequence of class IDs in
    // the class table, Sorry

pascal void EachSuperClassDo(ObjClassID testClass,
									DoToClassType DoToClass,
									void* staticLink);
	// Calls DoToClass for superclass of the test class. Order of iteration is in order of
    // ancestry.

pascal void FailNonObject(TObject* obj);
	// called by dispatcher to discipline an object. Invokes failure if parameter is not
    // IsObject

pascal TObject* FreeIfObject(TObject* obj);
	// IF obj <> NIL THEN obj.Free; useful for freeing an object that might sometimes be
    // NIL. NIL Function result is handy for assigning back to the reference just freed
    // (to help allieviate dangling pointerciples)

pascal Size GetClassSizeFromID(ObjClassID classID);
	// Given an object id, return the class's instantiation size.

pascal void GetClassNameFromID(ObjClassID classID, MAName& clName);
	// Given an object id, return the object's name. ??? add error checking ???

pascal ObjClassID GetClassIDFromName(const MAName& clName);
	// Given an object name, return its id.

pascal ObjClassID GetClassID(TObject* obj);
	// Given an object, return the object's ClassID.

pascal ObjClassID GetSuperClassID(ObjClassID objID);
	// returns immediate superclass of class. Returns kNilClass for TObject. If we ever
    // get MI this will have to be an enumerator.

pascal void InitUObject();
	// Essential one time initialization for this unit

pascal Boolean IsObject(TObject* obj);
	// Debugging check. Returns true if obj references a real object, false if obj is NIL
    // or a non-object reference. This is not an absloutely sure-fire test of objectness,
    // but does validate the following: (1) is it a handle? (2) is the handle
    // non-purgeable? (3) is the class ID valid? (4) is the handle size at least as large
    // as the class size?

pascal Boolean IsClassIDMemberClass(ObjClassID testClass, ObjClassID superClass);
	// tests the testClass for MEMBERship in the superclass

pascal Boolean IsMemberClassID(TObject* obj, ObjClassID objID);
	// Returns true if obj references the class or a subclass of the class represented by
    // objID.

pascal void OBJFail(short error);
	// Entered due (presumably) to some object failing a test for objectness. There are
    // two codes which result in a special message: kFailCoercion, which means that an
    // object could not be coerced to a class due to not being a subclass of that class;
    // and kFailMethNotFound, which means a method was called for a class, but that method
    // is not defined for the class. Other codes are allowed but are not given special
    // treatment. In qDebug mode a ProgramBreak preceeds the Failure.

pascal TObject* NewObjectByClassId(ObjClassID classID);
	// Creates an object of the given class id

pascal TObject* NewObjectByClassName(const MAName& className);
	// Creates an object of the given class name

pascal Boolean VerboseIsObject(TObject* obj);
	// Debugging check. Indicate whether the parameter appears to really be an object and
    // emit diagnostics if it is not

pascal void SetStdObjProcs(ObjProcs& theObjProcs);
	// Sets theObjProcs to point to the standard object system bottleneck procs

#endif


