// UProjFileHandler.h
// Copyright 1984-1991 Apple Computer, Inc. All rights reserved.

#ifndef __UPROJFILEHANDLER__
#define __UPROJFILEHANDLER__

#ifndef __PASCALDEFS__
#include <PascalDefs.h>
#endif

#ifndef __UFILEHANDLER__
#include <UFileHandler.h>
#endif


//----------------------------------------------------------------------------------------
// Projector ckid resource record.
//----------------------------------------------------------------------------------------

struct ProjectorRecord
{
	long checksum;
	long locationID;
	short version;
	short checkoutState;
	SignedByte branch;
	SignedByte Modified;
	long unused;
	long checkoutTime;
	long modDate;
	long etcetera;
};

typedef ProjectorRecord *ProjectorPtr, **ProjectorHandle;


//----------------------------------------------------------------------------------------
// ProjectorStatus
//----------------------------------------------------------------------------------------

enum ProjectorStatus
{
	readOnly, readWrite, modReadOnly, notInProject
};


//----------------------------------------------------------------------------------------
// Forward and external class declarations
//----------------------------------------------------------------------------------------

class TFileBasedDocument;
class TFile;


//----------------------------------------------------------------------------------------
// TProjFileHandler
//----------------------------------------------------------------------------------------

class TProjFileHandler : public TFileHandler
{
public:
	ProjectorStatus fProjStatus;

	Boolean fWriteckid;							// True if fFile status was modified

	virtual pascal void Initialize();

	virtual pascal void IProjFileHandler(TFileBasedDocument* itsDocument,
										 TFile* itsFile,
										 const OSType itsFileType,
										 const OSType itsCreator,
										 Boolean usesDataFork,
										 Boolean usesRsrcFork,
										 Boolean keepsDataOpen,
										 Boolean keepsRsrcOpen);
		// Initialization method for TProjectorFile.

	virtual pascal void DoRead(Boolean forPrinting);
		// Check for a CKID resource in the file and set fProjStatus accordingly

	virtual pascal void DoWrite(TFile* aFile, Boolean makingCopy);
		// If fWriteckid is TRUE, then write the projector status out as modified readonly

	virtual pascal ProjectorStatus GetProjectorStatus();
		// Return the projector read/ write status of its file

	virtual pascal void ModifyReadOnly();
		// Set the projector status of its file as modified read-only

	virtual pascal void Fields(TObject* obj);
		// Called by the Inspector and the Debugger to display the contents of this
        // class's fields.
};


#endif


