// USectionMgr.h
// Copyright 1984-1991 Apple Computer, Inc. All rights reserved.

#ifndef __USECTIONMGR__
#define __USECTIONMGR__

#ifndef __PASCALDEFS__
#include <PascalDefs.h>
#endif

#ifndef __DIALOGS__
#include <Dialogs.h>
#endif

#ifndef __EDITIONS__
#include <Editions.h>
#endif

#ifndef __UEVTHANDLER__
#include <UEvtHandler.h>
#endif

#ifndef __UCOMMAND__
#include <UCommand.h>
#endif

#ifndef __ULIST__
#include <UList.h>
#endif

#ifndef __UBEHAVIOR__
#include <UBehavior.h>
#endif


//----------------------------------------------------------------------------------------
// External and forward class declarations
//----------------------------------------------------------------------------------------

class TDocument;
class TPublisher;
class TSection;
class TSubscriber;
class TView;


//----------------------------------------------------------------------------------------
// TSectionMgr: This object is responsible for interaction with the Edition manager
//----------------------------------------------------------------------------------------

class TSectionMgr : public TBehavior
{
public:

	virtual pascal void ISectionMgr();
		// initialize

	virtual pascal void DoAppleCommand(CmdNumber aCmdNumber,
									   const AppleEvent& message,
									   const AppleEvent& reply);
		// responds to the edition manager apple events

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields, in this case the global variables.
};


//----------------------------------------------------------------------------------------
// TSectionCommand: an abstract superclass used as the base class for the edition manager
// apple event command classes
//----------------------------------------------------------------------------------------

class TSectionCommand : public TAppleCommand
{
public:
	TSection* fSection;							// section object for this apple event
                                                // command

	virtual pascal void ISectionCommand(CmdNumber itsCmdNumber,
										TDocument* itsDocument,
										TView* itsView,
										TSection* itsSection);
		// initialize this command

	virtual pascal void InitFromAppleEvent(CmdNumber itsCmdNumber,
										   TDocument* itsDocument,
										   TView* itsView,
										   const AppleEvent& itsMessage,
										   const AppleEvent& itsReply);
		// initialize this command from the AppleEvent stream

	virtual pascal TSection* GetSectionObject(SectionHandle aSectionHandle);
	// maps a section handle to a section object

};


//----------------------------------------------------------------------------------------
// TSectionReadEventCmd
//----------------------------------------------------------------------------------------

class TSectionReadEventCmd : public TSectionCommand
{
public:

	virtual pascal void ISectionReadEventCmd(CmdNumber itsCmdNumber,
											 TDocument* itsDocument,
											 TView* itsView,
											 TSubscriber* itsSubscriber);
		// initialize this command

	virtual pascal void DoIt();
		// call TSubscriber(fSection).Subscribe
};


//----------------------------------------------------------------------------------------
// TSectionWriteEventCmd
//----------------------------------------------------------------------------------------

class TSectionWriteEventCmd : public TSectionCommand
{
public:

	virtual pascal void ISectionWriteEventCmd(CmdNumber itsCmdNumber,
											  TDocument* itsDocument,
											  TView* itsView,
											  TPublisher* itsPublisher);
		// initialize this command

	virtual pascal void DoIt();
		// call TPublisher(fSection).Publish
};


//----------------------------------------------------------------------------------------
// TSectionScrollEventCmd
//----------------------------------------------------------------------------------------

class TSectionScrollEventCmd : public TSectionCommand
{
public:


	virtual pascal void ISectionScrollEventCmd(CmdNumber itsCmdNumber,
											   TDocument* itsDocument,
											   TView* itsView,
											   TSection* itsSection);
		// initialize this command

	virtual pascal void DoIt();
		// scrolls the document to the publisher
};


//----------------------------------------------------------------------------------------
// TSectionCancelEventCmd
//----------------------------------------------------------------------------------------

class TSectionCancelEventCmd : public TSectionCommand
{
public:

	virtual pascal void ISectionCancelEventCmd(CmdNumber itsCmdNumber,
											   TDocument* itsDocument,
											   TView* itsView,
											   TSection* itsSection);
		// initialize this command

	virtual pascal void DoIt();
		// cancels this section
};


//----------------------------------------------------------------------------------------
// Global function declarations
//----------------------------------------------------------------------------------------

extern pascal void InitUSectionMgr();
	// Initialize the section manager and attach it to the application. Note that the
    // application should already have been instantiated and initialized when this
    // procedure is called.

#endif


