// UTEDocument.h
// Copyright 1984-1991 Apple Computer, Inc. All rights reserved.

#ifndef __UTEDOCUMENT__
#define __UTEDOCUMENT__

//----------------------------------------------------------------------------------------
// This separate unit for TE Documents is still an example and not a supported Building
// Block. It provides a backing document for those times when you want to write your
// TEView to external media. Remember, since its not a building block right now the
// structure is subject to more than the usual amount of change. Let us know what you
// think.
//----------------------------------------------------------------------------------------

#ifndef __PASCALDEFS__
#include <PascalDefs.h>
#endif

#ifndef __UFILEBASEDDOCUMENT__
#include <UFileBasedDocument.h>
#endif

#ifndef __UTEVIEW__
#include <UTEView.h>
#endif




const unsigned long kTextFileType = 'TEXT';		// Standard file-type code for saved disk
                                                // files -- uses standard text files 

const unsigned long kScrapType = 'TEXT';		// Standard scrap-type code for
                                                // WriteToScrap

const short kWindowRsrcID = 1004;				// 'view' template for a DemoText window 

const short kViewRsrcID = 1005;					// 'view' template for a DemoText view 


//----------------------------------------------------------------------------------------
// Menu numbers 
//----------------------------------------------------------------------------------------

const short mFont = 10;


//----------------------------------------------------------------------------------------
// Command numbers 
//----------------------------------------------------------------------------------------

const short cWidthSuperView = 601;				// View-width-determination commands 
const short cWidthView = 602;
const short cWidthOnePage = 603;

const short cHeightSuperView = 604;				// View-height-determination commands 
const short cHeightPages = 605;
const short cHeightText = 606;
const short cHeightConst = 607;

const short cJustForceLeft = 608;				// Justification commands 
const short cJustCenter = 609;
const short cJustRight = 610;
const short cJustSystem = 611;

//  Command numbering scheme:
//  styles	= 1000
//  sizes	= 1100
//  just	= 1200
//  fonts	= 1300
//  hiers	= 1400
//  colors	= 1500


//----------------------------------------------------------------------------------------
// Command numbers for typestyle attributes 
//----------------------------------------------------------------------------------------

const short cPlainText = 1001;
const short cBold = 1002;
const short cItalic = 1003;
const short cUnderline = 1004;
const short cOutline = 1005;
const short cShadow = 1006;
const short cCondense = 1007;
const short cExtend = 1008;


//----------------------------------------------------------------------------------------
// Command numbers for font-size commands 
//----------------------------------------------------------------------------------------

const short cSizeChange = 1100;
const short cSizeBase = 1100;
const short cSizeMin = 1109;
const short cSizeMax = 1196;
const short cSizeOther = 1197;

//----------------------------------------------------------------------------------------
// 1101-1197 reserved for font sizes 1-97 pts. 
//----------------------------------------------------------------------------------------

const short cSizeGrow = 1198;
const short cSizeShrink = 1199;


//----------------------------------------------------------------------------------------
// Command numbers to cover other stylistic changes 
//----------------------------------------------------------------------------------------

const short cJustChange = 1200;
const short cFontChange = 1300;


//----------------------------------------------------------------------------------------
// Command numbers for the hierarchial menu 
//----------------------------------------------------------------------------------------

const short cStyle = 1401;
const short cSize = 1402;
const short cFont = 1403;
const short cColor = 1404;


//----------------------------------------------------------------------------------------
// Command numbers for changing colors 
//----------------------------------------------------------------------------------------

const short cColorChange = 1500;
const short cColorText = 1501;
const short cColorBackground = 1502;


//----------------------------------------------------------------------------------------
// Constant for staggering windows 
//----------------------------------------------------------------------------------------

const short kStaggerAmount = 16;


//----------------------------------------------------------------------------------------
// Constant for amount to relative size text selection 
//----------------------------------------------------------------------------------------

const short kRelSizeAmount = 4;


//----------------------------------------------------------------------------------------
// Constants for the text specs resource 
//----------------------------------------------------------------------------------------

const unsigned long kTextSpecsRsrcType = 'SPEC';
const short kTextSpecsRsrcID = 1;


//----------------------------------------------------------------------------------------
// Constants for the text style resource 
//----------------------------------------------------------------------------------------

const unsigned long kTextStyleRsrcType = 'STYL';
const short kStylesRsrcID = 1;
const short kElementsRsrcID = 2;


//----------------------------------------------------------------------------------------
// Constants for the prompts string list 
//----------------------------------------------------------------------------------------
const short kPromptsRsrcID = 1005;
const short kColTextPrompt = 1;
const short kColBackPrompt = 2;

const short kFileTooBig = 1000;					// The 'File is too large' alert 


const short kHierDisplayedMBar = 131;			// Displayed menus on hier. menu system 
const short kNonHierDisplayedMBar = 128;		// Displayed menus on non-hier. system 

const short kHierMenuOffset = 1000;				// Offset added to non-hier menu cmds to
                                                // get the right number 


//----------------------------------------------------------------------------------------
// TextSpecs: Additional document specifications
//----------------------------------------------------------------------------------------

struct TextSpecs
{												
	Str255 theTextFont;
	Style theTextFace;
	short theTextSize;
	RGBColor theTextColor;
	short theJustification;						// record justification 
	RGBColor theBackColor;						// Window's background color 
};

typedef TextSpecs *TextSpecsPtr, **TextSpecsHdl;


//----------------------------------------------------------------------------------------
// External and forward class declarations
//----------------------------------------------------------------------------------------

class TTEView;
class TFile;


//----------------------------------------------------------------------------------------
// TTEDocument
//----------------------------------------------------------------------------------------

class TTEDocument : public TFileBasedDocument
{
public:
	Handle fDocText;							// The text owned by the document 

	TEStyleHandle fStyles;						// Style handle, if any 

	STHandle fElements;							// Handle to element array, if any 

	TTEView* fTEView;							// The view which displays the text 

	TextSpecs fTextSpecs;						// Specifies properties of the text 


	//------------------------------------------------------------------------------------
	// Initialization and freeing 
	//------------------------------------------------------------------------------------

	virtual pascal void Initialize();
	
	virtual pascal void ITEDocument(TFile* itsFile,
									const OSType itsFileType,
									const OSType itsCreator,
									const OSType itsScrapType,
									Boolean usesDataFork,
									Boolean usesRsrcFork,
									Boolean keepsDataOpen,
									Boolean keepsRsrcOpen);
		// Initialization method for TTEDocument. 

	virtual pascal void Free();

	virtual pascal void DoInitialState();

	virtual pascal void FreeData();


	//------------------------------------------------------------------------------------
	// Filing 
	//------------------------------------------------------------------------------------

	virtual pascal void DoNeedDiskSpace(TFile* itsFile,
										long& dataForkBytes,
										long& rsrcForkBytes);

	virtual pascal void DoRead(TFile* aFile, Boolean forPrinting);

	virtual pascal void DoWrite(TFile* aFile, Boolean makingCopy);


	//------------------------------------------------------------------------------------
	// Making views and windows 
	//------------------------------------------------------------------------------------

	virtual pascal void DoMakeViews(Boolean forPrinting);


	//------------------------------------------------------------------------------------
	// Command processing 
	//------------------------------------------------------------------------------------

	virtual pascal void DoMenuCommand(CmdNumber aCmdNumber);

	virtual pascal void DoSetupMenus();


	//------------------------------------------------------------------------------------
	// Auxiliary routines 
	//------------------------------------------------------------------------------------

	virtual pascal void ChangeBackColor(const RGBColor& newColor);

	virtual pascal void SetSpecStyle();

	virtual pascal void ShowReverted();
		// When the user reverts a document, this is called after reading the old document
        // and before displaying it. Causes the reverted font specs to be installed. 


	//------------------------------------------------------------------------------------
	// Inspecting 
	//------------------------------------------------------------------------------------

	virtual pascal void Fields(TObject* obj);

private:
	void DoJustChange(CmdNumber aCmdNumber);

	void DoStyleChange(CmdNumber aCmdNumber);

	void DoColTextChange();

	void DoColBackChange();

	void InstallChangedDeterminer(VHSelect vhs, SizeDeterminer sd);

	void DoWidthChange(CmdNumber aCmdNumber);

	void DoHeightChange(CmdNumber aCmdNumber);
};


class TJustCommand : public TCommand
{
public:
	TTEView* fTEView;

	short fOldJust;

	short fNewJust;


	virtual pascal void Initialize();
		// Puts a newly created TJustCommand into a known state. Subclasses should
        // override it and call INHERITED. Typically used to put the object into a state
        // where it can be safely FREED. 

	virtual pascal void IJustCommand(TTEView* itsTEView, short itsNewJust);

	virtual pascal void DoIt();

	virtual pascal void RedoIt();

	virtual pascal void UndoIt();

	virtual pascal void Fields(TObject* obj);
};


//----------------------------------------------------------------------------------------
// Global variable declarations
//----------------------------------------------------------------------------------------

extern TextSpecs gDefaultSpecs;
	// Text specs as default for new documents 

#endif


