// UTEView.p
// Copyright  1986-1991 Apple Computer, Inc. All rights reserved.

#ifndef __UTEVIEW__
#define __UTEVIEW__

#ifndef __PASCALDEFS__
#include "PascalDefs.h"
#endif

#ifndef __UGEOMETRY__
#include "UGeometry.h"
#endif

#ifndef __TEXTEDIT__
#include "TextEdit.h"
#endif

#ifndef __UVIEW__
#include "UView.h"
#endif

#ifndef __UCOMMAND__
#include "UCommand.h"
#endif



const short kUnlimited = MAXINT;				/*  The maximum number of characters in the
												  fText of a TTEView object  */

const int kWithStyle = True;					/*  Parameters to TTEView.ITEView  */
const int kWithoutStyle = False;
const int kSaveCurrentChars = True;				/*  Parameter to ITECommand  */

const short Chr00 = 0;							/*  Needed for ControlCharSet  */
const short Chr1F = 31;

const short kMinAhead = 96;						/*  Default value for fMinAhead  */



typedef long ControlCharSet;


struct TEViewTemplate
{
	Boolean itsStyleType:1;
	Boolean itsAutoWrap:1;
	Boolean itsAcceptsChanges:1;
	Boolean itsFreesText:1;
	short filler1:12;
	short itsKeyCmdNumber;
	short itsMaxChars;
	Rect itsInset;
	short itsJustification;
	Style itsTextFace;
	short itsTextSize;
	RGBColor itsTextColor;
	Str255 itsFontName;							/*  Actually a variable length P-String  */
};


typedef TEViewTemplate* TEViewTemplatePtr;


class TDocument;

class TScroller;

class TToolboxEvent;


class TTECommand;

class TTETypingCommand;

class TTEStyleCommand;


class TTEView : public TView
{
public:
	/*  TTEView is a view subclass representing a
	  TextEdit record. TextEdit is the simple
	  text-editing facility built into the
	  Macintosh ROM. The purpose of a TEView is
	  to make TextEdit function properly in a
	  MacApp environment, for things like
	  scrolling, printing, page breaks, and
	  command handling  */

	TEHandle fHTE;								/*  Handle to the actual TextEdit object  */

	Handle fText;								/*  The text in the TEHandle  */
	Handle fSavedTEHandle;						/*  Saved handle from TENew  */

	VRect fInset;								/*  Amount to inset text from edges of view  */

	CmdNumber fKeyCmdNumber;					/*  Will be used as the string number for
												  "Undo Typing"  */

	short fMaxChars;							/*  Maximum number of chars to accept into
												  fText. Default is MAXINT; stuff to
												  different value if you want to limit max
												  chars to fewer than MAXINT  */

	long fLastHeight;							/*  Last checked height of record. If the
												  record is stylish, this represents the
												  height in pixels. For old-style records,
												  equals number of lines.  */

	long fLastWidth;							/*  Last checked width of record. Not currently
												  used If the TE is styled, this represents the
												  width in pixels.  */

	TTETypingCommand* fTypingCommand;			/*  The current TE Typing command relating to
												  me, if any  */

	TextStyle fTextStyle;						/*  Current style of text  */

	short fJustification;						/*  Current justification of text record  */

	Boolean fAcceptsChanges;					/*  Set to FALSE to have text which will not
												  accept any change, such as text on the
												  Clipboard, or perhaps received mail  */
	Boolean fStyleType;							/*  Set to kWithStyle if record is styled  */
	Boolean fAutoWrap;							/*  Set to FALSE for line wrapping at CR's
												  only  */
	Boolean fFreeText;							/*  Determines if fText should be freed on
												  Free.  */
	Boolean fSpecsChanged;						/*  Something recently happened which could
												  effect font/style/size/color menu item
												  updating. Should be reset to FALSE when
												  application has taken the appropriate
												  action.  */
	short fLastPageBreak;						/*  Caches last page break computed.  */
	short fLastLine;							/*  Last line of text of the last page break  */
	ControlCharSet fControlChars;				/*  Control chars accepted in text  */
	short fMinAhead;							/*  Minimum amount by which to autoscroll ahead
												  when scrolling selection into view.(When typing
												  etc.  */
	/*  Stuff for keystroke selection control  */
	short fSelAnchor;							/* current selection "anchor" */
	Boolean fUpDown;							/* used to keep track of horizontal position
												*/
	short fUpDownH;								/* for repeated Up- and/or Down-arrowing  */

	/*  Initialize and Free  */

	virtual pascal void Initialize();
	/*  Puts a newly created TTEView into a known state. Subclasses should
	  override it and call INHERITED. Typically used to put the object into a state where
	  it can be safely FREED.  */

	virtual pascal void ITEView(TDocument* itsDocument,
								TView* itsSuperView,
								const VPoint& itsLocation,
								const VPoint& itsSize,
								SizeDeterminer itsHDeterminer,
								SizeDeterminer itsVDeterminer,
								const VRect& itsInset,
								const TextStyle& itsTextStyle,
								short itsJustification,
								Boolean itsStyleType,
								Boolean itsAutoWrap);/*  OK to be NIL, if view will
												  belong to a documentless window
												*/
	/*  The view in which this view is
	  contained  */
	/*  Location in its superview  */
	/*  Size of view  */
	/*  How to width of the view
	  is to be determined  */
	/*  How the height of the
	  view is to be determined
	*/
	/*  Amount to inset text  */
	/*  Initial text style  */
	/*  Its justification  */
	/*  Whether or not record is styled  */
	/*  FALSE if newline occurs at CR only
	*/

	/*  For Initialization of procedurally created TEView.  */

	virtual pascal void IRes(TDocument* itsDocument,
							 TView* itsSuperView,
							 Ptr& itsParams);


	/*  Initialize a TTEView via a 'view' resource.  */

	virtual pascal TObject* Clone();
	/*  calls INHERITED Clone and then clones owned objects  */

	virtual pascal void WRes(ViewRsrcHandle theResource,
							 Ptr& itsParams);

	/*  Write the object out to a view resource.  */

	virtual pascal void WriteRes(ViewRsrcHandle theResource,
								 Ptr& itsParams);

	/*  Set up the type and signature of this object and call WRes.  */

	virtual pascal void Free();
	/*  Frees the TERecord and optionally fText, then frees SELF  */

	virtual pascal void MakeTERecord();
	/*  Called with grafport properly set, to create the actual TE Record  */

	/*  Target management  */
	virtual pascal Boolean WantToBecomeTarget();

	/*  Commands and Menus  */

	virtual pascal void DoKeyCommand(TToolboxEvent* event);
	/*  Handles keystrokes  */

	virtual pascal TTECommand* DoMakeEditCommand(CmdNumber aCmdNumber);
	/*  Make a command for handling edit menu stuff  */

	virtual pascal TTEStyleCommand* DoMakeStyleCommand(const TextStyle& aStyle,
													   CmdNumber itsCmdNumber,
													   short itsMode);


	/*  Make a style change command  */

	virtual pascal TTETypingCommand* DoMakeTypingCommand(short ch);
	/*  Make a typing command for handling keystrokes  */

	virtual pascal void DoMenuCommand(CmdNumber aCmdNumber);
	/*  Handles menu commands  */

	virtual pascal void DoMouseCommand(VPoint& theMouse,
									   TToolboxEvent* event,
									   Point hysteresis);


	/*  Handles mousepresses  */

	virtual pascal void DoneTyping();
	/*  No further typing can occur for the current typing command  */

	virtual pascal void DoSetupMenus();
	/*  Sets up menus  */

	virtual pascal void SetActive(Boolean state);
	/*  Calls TEActivate or TEDeactivate  */

	virtual pascal void Activate(Boolean entering);
	/* !!!RCR */

	virtual pascal void TTEView::DoHighlightSelection(HLState fromHL,
											HLState toHL);	// Override

	virtual pascal void BecameWindowTarget();	// Override
	virtual pascal void ResignedWindowTarget();	// Override
	virtual pascal void BecameApplicationTarget();	// Override
	virtual pascal void ResignedApplicationTarget();// Override

	virtual pascal void SetEnable(Boolean state);
	/*  Resets the view's idle  */

	/*  Screen Display  */

	virtual pascal Boolean DoIdle(IdlePhase phase);
	/*  Blinks the insertion point  */

	virtual pascal void Draw(const VRect& area);
	/*  Draw the text in a frame or on the printed page  */

	virtual pascal void ShowReverted();
	/*  Make sure line starts are correct before redisplay  */

	/*  Size Changes  */

	virtual pascal void BeInPort(GrafPtr itsPort);
	/*  Now that a grafport is established for the view, tells TextEdit about it  */

	virtual pascal void BeInScroller(TScroller* itsScroller);
	/*  Gives us a chance to set the scroll parameters  */

	virtual pascal void CalcMinSize(VPoint& minSize);
	/*  Compute the minimum size of the view  */

	virtual pascal long CalcRealHeight();
	/*  Calculate true height of record, including last character if it is a carriage
	  return  */

	virtual pascal long CalcRealWidth();
	/*  Calculate true width of record.Only supported for NON-styled TE Records right now  */

	virtual pascal void ComputeSize(VPoint& newSize);
	/*  Compute the actual size of the view  */

	virtual pascal void Resize(const VPoint& newSize,
							   Boolean invalidate);

	/*  Installs the new size for both the View and TextEdit  */

	virtual pascal void SynchView(Boolean redraw);
	/*  Keeps view metrics in synch after TextEdit operation. If redraw is true then
	  ScrollSelectionIntoView is called to make sure the selection is visible. */

	virtual pascal void OffsetToPt(short offset, VPoint& itsPoint);
	/*  Converts offset in text to view coordinates.  */

	virtual pascal short OffsetToLine(short offset);
	/*  Returns the line number that contains the offset  */

	virtual pascal short OffsetToLineStart(short offset);
	/*  Returns the offset of the start of the line that contains the offset  */

	virtual pascal short OffsetToLineEnd(short offset);
	/*  Returns the offset of the end of the line that contains the offset  */

	virtual pascal short PtToOffset(const VPoint& aPoint);// %% Point to aPoint
	/*  Converts a view point to an offset in the text.  */

	virtual pascal Boolean WordBounds(short charPos,
									  short& wordStart,
									  short& wordEnd);

	/*  charPos is the offset of a specific character in the text
	  Returns:	TRUE iff that character is part of a word
	  wordStart = offset of first character of the word
	  wordEnd= offset of final character of the word
	  Note: these are CHARACTER offsets, not CURSOR POSITIONS. */

	/*  Clipboard  */

	/*  NOTE -- These methods apply only to a TEView installed as the view in the Clipboard
	*/

	virtual pascal Boolean ContainsClipType(ResType aType);
	/*  Determines whether the indicated clipboard type can be produced by the view  */

	virtual pascal long GivePasteData(Handle aDataHandle,
									  ResType dataType);

	/*  Supplies data to be PASTED by some requesting PASTE command  */

	virtual pascal void WriteToDeskScrap();
	/*  Produces TEXT data for the Desk Scrap when called upon to do so  */

	/*  Miscellaneous  */

	virtual pascal void AutoScrolling(Boolean doScrolling);
	/*  Set TERecord auto scrolling to given value via TEAutoScroll  */

	virtual pascal Boolean ClikLoop();
	/*  Forwarded to from the global routine: ClickLoopForTTEView which was set as the
	  TE's ClikLoop.Typically handles autoscrolling for the view.  */

	virtual pascal void CalcSelLoc(VRect& selectionRect);
	/*  Calculates the location of the current Selection  */

	virtual pascal void SetSelection(short NewSelStart,
									 short NewSelEnd,
									 Boolean redraw);

	/* !!! Comment this  */
	virtual pascal void GetSelectionString(Str255& selection);// %%% AMB
	/*  Returns the selected text as a Str255. If the selection is
	  longer than 255 characters, the first 255 characters are returned  */

	virtual pascal void ChangeWrap(Boolean newAutoWrap,
								   Boolean redraw);
	/*  Changes auto-wrapping behavior, redrawing if requested  */

	virtual pascal Boolean ContinuousStyle(short firstChar,
										   short lastChar,
										   short& mode,
										   TextStyle& aStyle);


	/*  Returns TRUE if the style of the characters within the given range is wholly
	  continuous. The style attributes to check are specified by mode, which has the same
	  meaning as in the TextEdit call TESetStyle. Attributes which are continuous are
	  returned in aStyle, with the appropriate bits set in mode. Font style attribute
	  bits are individually checked across the selection range, and returned set to 1 if
	  that attribute is continuous.  */

	virtual pascal void ExtractStyles(TEStyleHandle& theStyles,
									  STHandle& theElements);

	/*  Extract handles to style information for a TextEdit record, presumably for the
	  purpose of storing the data to disk. Note that line heights can be recalculated,
	  and as such don't need to be stored.  */

	virtual pascal Handle ExtractText();
	/*  For sake of completeness. Extract handle to text, presumably for the purpose of
	  storing the text externally (perhaps to disk).  */

	virtual pascal void RecalcText();
	/*  Tells TextEdit to recompute linestarts  */

	virtual pascal void ScrollSelectionIntoView();
	/*  Scrolls the selection into view -- called after Commands are done or undone If auto
	  scrolling is on, calls TESelView to do the right thing  */

	virtual pascal void SetJustification(short newJust,
										 Boolean redraw);

	/*  Sets the TERecord to the given justification. If redraw is true then the view is
	  redrawn by calling ForceRedraw. Otherwise it is assumed the called will get the
	  view redrawn.  */

	virtual pascal void SetOneStyle(short theStart,
									short theEnd,
									short theMode,
									const TextStyle& theStyle,
									Boolean redraw);


	/*  Sets a text style continuously over a range. In unstylish TERecords, the style is
	  set across the entire record. If redraw is true then the text is redrawn.  */

	virtual pascal void SetText(const Str255& theText);// %%% AMB
	/*  Convenience routine, to save from having to create a handle just to pass to
	  StuffText. Note that if fSavedTEHandle already holds a saved handle, it will be
	  disposed before StuffText is called.  */

	virtual pascal Boolean SpaceForStyles(long rangeStart,
										  long rangeEnd);
	/*  Returns TRUE if there is enough memory to hold the styles over the given range in
	  the TERecord  */

	virtual pascal void StuffStyles(TEStyleHandle theStyles,
									STHandle theElements);

	/*  Installs style information, presumably fetched from disk, into the TextEdit record.
	*/											/*  Installs text into the TextEdit
												  record.Note that this destroys any
												  existing style information -- StuffStyles
												  should be called if styles are to be
												  attached to this text information.  */

	virtual pascal void StuffText(Handle theText);
	/*  Installs text into the TextEdit record. Note that this destroys any existing style
	  information -- StuffStyles should be called if styles are to be attached to this
	  text information  */

	virtual pascal void StuffTERects(const Rect& newTERect);
	/*  installs "newTERect" as the dest & view rect of the TextEdit record  */

	/*  Printing  */

	virtual pascal VCoordinate DoBreakFollowing(VHSelect vhs,
												VCoordinate prevBreak,
												Boolean& automatic);


	/*  Computes the next page-break following 'prevBreak' in direction given by 'vhs'  */

	virtual pascal void DoCalcViewPerPage(VPoint& viewPerPage);
	/*  Computes how much of the view is to be allocated to each printed page  */

	virtual pascal void DoSetPageOffset(const VPoint& coord);
	/*  At printing time, determine the location of the 'interior' (body)
	  of the page  */

	virtual pascal void GetPrintExtent(VRect& printExtent);
	/*  Print extent is the view's extent minus the insets.  */

	/*  Debugging  */

	virtual pascal void Fields(TObject* obj);
	/*  Used by the Inspector and the Debugger to display the contents of this class's
	  fields.  */

};


/* -------------------------------------------------------------------------------------------------- */


extern ProcPtr gDefClikLoopProc;				// %%% AMB
/*  Standard TextEdit click loop routine  */

/*  The following are considered private, but appear in the interface in case you need to
  override a method that uses one of these.  */

extern TTEView* pCurrTEView;					// %%% AMB
/*  For the benefit of ClikLoopForTTEView only  */
#if qDebug
extern Boolean pTEIntenseDebugging;		// %%% AMB
#endif

/* -------------------------------------------------------------------------------------------------- */
pascal void InitUTEView();
/*  Initialize TEView unit.  */

pascal void SetSelect(short theStart,
							 short theEnd,
							 TEHandle hTE);

/*  For those times when we don't want the selection range hilited when changed  */

pascal Boolean ClickLoopForTTEView();
/*  Called from TextEdit when the mouse is down forwards to TTEView.ClikLoop so that
  autoscrolling may be done.  */

pascal void DumpTERecord(TEHandle aTEH);
/*  writes salient information about the TERecord out to the Debug window  */

#endif


