// UTabTEView.h
// Copyright 1984-1991 Apple Computer, Inc. All rights reserved.

#ifndef __UTABTEVIEW__
#define __UTABTEVIEW__

#ifndef __PASCALDEFS__
#include <PascalDefs.h>
#endif

#ifndef __UGEOMETRY__
#include <UGeometry.h>
#endif

#ifndef __UTEVIEW__
#include <UTEView.h>
#endif

//----------------------------------------------------------------------------------------
//  Tabs for TE Views.
//  by Steve Friedrich
//  Adapted from code generously supplied by Tom Dowdy @ Apple
// 
//  KNOWN LIMITATIONS
//  only works for styled text but not with styles
//  slow on full recalcs when linewrap is true
//  won't print
//----------------------------------------------------------------------------------------



//----------------------------------------------------------------------------------------
// pretty pictures for "ShowInvisibles" 
//----------------------------------------------------------------------------------------

const short kVisibleTAB = 0;

const short kVisibleSpace = 0;

const short kVisibleCR = 0;

const short kDefaultTabSize = 24;				// the default tab size in pixels 

const short cIncTabs = 1410;

const short cDecTabs = 1411;

const short cShowInvs = 1412;


//----------------------------------------------------------------------------------------
// OldTEHooksRec: Routines that can be hooked in TE
//----------------------------------------------------------------------------------------

struct OldTEHooksRec
{
	ProcPtr DrawHook;
	ProcPtr WidthHook;
	ProcPtr HitTestHook;
	ProcPtr EOLHook;
};


//----------------------------------------------------------------------------------------
// Forward and external class declarations
//----------------------------------------------------------------------------------------

class TDocument;
class TToolboxEvent;


//----------------------------------------------------------------------------------------
// TTabTEView
//----------------------------------------------------------------------------------------

class TTabTEView : public TTEView
{
public:
	short fTabSize;

	Boolean fShowInvisibles;

	CQDProcs fOldCQDProcs;						// currently effective grafprocs 

	CQDProcsPtr fHookedCQDProcsPtr;				// Points to a hooked version of currently
                                                // effective grafprocs. 

	OldTEHooksRec fOldTEHooksRec;				// previous hooks returned by TECustomHook


	virtual pascal void IRes(TDocument* itsDocument,
							 TView* itsSuperView,
							 Ptr& itsParams);


	virtual pascal void BeInPort(GrafPtr itsPort);

	virtual pascal void SetTabSize(short newSize, Boolean redraw);

	virtual pascal void ShowInvisibles(Boolean showEm, Boolean redraw);

	virtual pascal short TETextLength(Ptr theText,
									  short theLength,
									  short theWidthBefore,
									  Point& numer,
									  Point& denom,
									  FontInfo& info);

	virtual pascal void DoMouseCommand(VPoint& theMouse,
									   TToolboxEvent* event,
									   Point hysteresis);

	virtual pascal void DoSetupMenus();

	virtual pascal void DoMenuCommand(CmdNumber aCmdNumber);

	virtual pascal void Fields(TObject* obj);

	virtual pascal short TETxMeas(short theLength,
								  Ptr theText,
								  Point& numer,
								  Point& denom,
								  FontInfo& info);

	virtual pascal void TEDrawText(short theLength,
								   Ptr theText,
								   Point numer,
								   Point denom);

	virtual pascal void WithHookedGrafProcsDo(pascal void(* whatToDo)(void* staticLink),
											  void* staticLink);
};


//----------------------------------------------------------------------------------------
// TSetTabCommand
//----------------------------------------------------------------------------------------

class TSetTabCommand : public TCommand
{
public:
	short fOldTabSetting;

	short fNewTabSetting;


	virtual pascal void ISetTabCommand(TView* itsView,
									   CmdNumber itsCmdNumber,
									   short tabSetting);

	virtual pascal void DoIt();
		// sets new tabs, then calls ForceRedraw 

	virtual pascal void RedoIt();
		// calls DoIt 

	virtual pascal void UndoIt();
		// restores previous tab setting 

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields. 
};


//----------------------------------------------------------------------------------------
// TShowInvsCommand
//----------------------------------------------------------------------------------------

class TShowInvsCommand : public TCommand
{
public:
	Boolean fInvsWereShown;


	virtual pascal void IShowInvsCommand(TView* itsView,
										 CmdNumber itsCmdNumber,
										 Boolean invsWereShown);

	virtual pascal void DoIt();
		// sets new tabs, then calls ForceRedraw 

	virtual pascal void RedoIt();
		// calls DoIt 

	virtual pascal void UndoIt();
		// restores previous tab setting 

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields. 
};


//----------------------------------------------------------------------------------------
// Global function declarations
//----------------------------------------------------------------------------------------

extern pascal void InitUTabTEView();


#endif


