// UTearOffMenuView.h
// Copyright 1984-1991 Apple Computer, Inc. All rights reserved.

#ifndef __UTEAROFFMENUVIEW__
#define __UTEAROFFMENUVIEW__

#ifndef __PASCALDEFS__
#include <PascalDefs.h>
#endif

#ifndef __UGEOMETRY__
#include <UGeometry.h>
#endif

#ifndef __UMENUVIEW__
#include <UMenuView.h>
#endif

#ifndef __UCOMMAND__
#include <UCommand.h>
#endif


//----------------------------------------------------------------------------------------
// Constants
//----------------------------------------------------------------------------------------

const short kTearOffMargin = 10;				// the slop around the displayed menu
                                                // within which we don't tear-off; see
                                                // Apple Human Interface Guidelines p. 91;
                                                // note: some programs (eg. HyperCard) use
                                                // a 15 pixel slop value 

const short kTearOffOffset = 3;					// pixel offset from pointer to gray rgn
                                                // being dragged around 


//----------------------------------------------------------------------------------------
// Forward and external class declarations
//----------------------------------------------------------------------------------------

class TWindow;
class TTearOffMenuView;


//----------------------------------------------------------------------------------------
// TShowTearOffWindowCmd: moves, shows and selects a TTearOffMenuView 
//----------------------------------------------------------------------------------------

class TShowTearOffWindowCmd : public TNoChangesCommand
{
public:
	TWindow* fTearOffWindow;

	Point fWhere;


	virtual pascal void Initialize();
		// Puts a newly created TShowTearOffWindowCmd into a known state. Subclasses
        // should override it and call INHERITED. Typically used to put the object into a
        // state where it can be safely FREED. 

	virtual pascal void IShowTearOffWindowCmd(TWindow* tearOffWindow, Point where);

	virtual pascal void DoIt();

	virtual pascal void Fields(TObject* obj);
};


//----------------------------------------------------------------------------------------
// TTearOffTracker: abstract superclass providing basic tear-off menu tracking: tracks the
// grey outline of the menu around the screen 
//----------------------------------------------------------------------------------------

class TTearOffTracker : public TTracker
{
public:
	TTearOffMenuView* fTearOffMenu;

	RgnHandle fTearOffWindowOutline;

	Boolean fExitTracking;

	Point fOrigin;

	virtual pascal void Initialize();
		// Puts a newly created TTearOffTracker into a known state. Subclasses should
        // override it and call INHERITED. Typically used to put the object into a state
        // where it can be safely FREED. 

	virtual pascal void ITearOffTracker(Point hitPt, TTearOffMenuView* tearOffMenu);

	virtual pascal void Free();

	virtual pascal void DoIt();

	virtual pascal void TrackFeedback(TrackPhase aTrackPhase,
										const VPoint& anchorPoint,
										const VPoint& previousPoint,
										const VPoint& nextPoint,
										Boolean mouseDidMove,
										Boolean turnItOn);

	virtual pascal TTracker* TrackMouse(TrackPhase aTrackPhase,
										VPoint& anchorPoint,
										VPoint& previousPoint,
										VPoint& nextPoint,
										Boolean mouseDidMove);

	virtual pascal Boolean IsDoneTracking();

	virtual pascal void Fields(TObject* obj);
};


//----------------------------------------------------------------------------------------
// TTearOffMenuView: abstract superclass providing basic tear-off menu behavior 
//----------------------------------------------------------------------------------------

class TTearOffMenuView : public TMenuView
{
public:
	RgnHandle fTearOffTrackingRgn;

	TTearOffTracker* fTearOffTracker;

	TWindow* fTearOffWindow;

	virtual pascal void Initialize();
		// Puts a newly created TTearOffMenuView into a known state. Subclasses should
        // override it and call INHERITED. Typically used to put the object into a state
        // where it can be safely FREED. 

	virtual pascal void ITearOffMenuView(short rsrcID,
										 short menuWidth,
										 short menuHeight,
										 TWindow* tearOffWindow);

	virtual pascal void InstallTearOffTracker();
		// For the sake of efficiency when tracking a tear-off, we'll create the tear-off
        // tracker and keep it with us, so we don't need to allocate a new tracker every
        // time we tear off a menu. If you want a different tracker in your subclass of
        // TTearOffMenuView, then override this method. 

	virtual pascal void Free();

	virtual pascal void HandleChooseMessage(short message,
											MenuHandle theMenu,
											Rect& menuRect,
											Point hitPt,
											short& whichItem);
		// initiates tracking if mouse is in fTearOffTrackingRgn 

	virtual pascal RgnHandle GetTearOffTrackingRgn(const Rect& menuRect);
		// accessor for fTearOffTrackingRgn: a rgn that represents the desktop less the
        // menu rect (with some buffer area around the menu rect). This is useful for
        // testing whether tracking should occur because the mouse just might be in the
        // menu bar (which isn't in the desktop & isn't in the menu rect) and so tracking
        // should not occur while the mouse is in the menu bar (MenuSelect does that
        // tracking just fine). 

	virtual pascal void Fields(TObject* obj);
};


#endif


