// UView.h
//  Copyright  1984-1991 by Apple Computer Inc.	All rights reserved.

#ifndef __UVIEW__
#define __UVIEW__

#ifndef __PASCALDEFS__
#include <PascalDefs.h>
#endif

#ifndef __UGEOMETRY__
#include <UGeometry.h>
#endif

#ifndef __BALLOONS__
#include <Balloons.h>
#endif

#ifndef __UEVTHANDLER__
#include <UEvtHandler.h>
#endif

#ifndef __UADORNERS__
#include <UAdorners.h>
#endif

#ifndef __UITERATOR__
#include <UIterator.h>
#endif


extern const unsigned char kFrame;				// Defines that a view be completely framed.


//----------------------------------------------------------------------------------------
// External and forward class declarations
//----------------------------------------------------------------------------------------

class TDesignator;
class TDocument;
class TList;
class TPrintHandler;
class TScroller;
class TWindow;
class TAdornment;
class TAdorner;
class TEvent;
class TToolboxEvent;


//----------------------------------------------------------------------------------------
// SizeDeterminer: Tells how a view's size is to be determined; specified separately in
// each dimension.
//----------------------------------------------------------------------------------------

enum SizeDeterminer
{
	sizeSuperView,								// View is the same size as its superview

	sizeRelSuperView,							// View size is relative to the
                                                // superview's size

	sizePage,									// View to be the size of one page

	sizeFillPages,								// View to grow upward to fill an exact
                                                // number of pages

	sizeVariable,								// View size fluctuates according to
                                                // app-specific criteria

	sizeFixed
};

enum ViewTemplateSelector { ViewType, ViewIncludeAt };


//----------------------------------------------------------------------------------------
// ViewTemplate
//----------------------------------------------------------------------------------------

struct ViewTemplate
{
	IDType itsParentID;
	IDType thisViewID;
	SimpleVPoint itsLocation;
	SimpleVPoint itsSize;
	SizeDeterminer itsVSizeDet:3;
	SizeDeterminer itsHSizeDet:3;
	Boolean isShown:1;
	Boolean isEnabled:1;
	Byte filler:8;
	IDType itsSignature;
	union
	{
		struct
		{
			SimpleStr255 itsType;
		} u0, _ViewType;
		struct
		{										//  Actually variable length
			short includeRsrcID;
			SimpleVPoint itsSubViewOffset;
		} u1, _ViewIncludeAt;
	};
};

typedef ViewTemplate* ViewTemplatePtr;


//----------------------------------------------------------------------------------------
// ViewResource
//----------------------------------------------------------------------------------------

struct ViewResource
{
	short numViews;
	ViewTemplate theViews;						// Actually, open-ended
};

typedef ViewResource *ViewRsrcPtr, **ViewRsrcHandle;


//----------------------------------------------------------------------------------------
// ViewPortInfo: Set by GetFocus; used by SetFocus. GetFocus/ SetFocus are used to save and
// restore the focus behind MacApp's back.
//----------------------------------------------------------------------------------------

struct ViewPortInfo
{												// !!!???###SRF silly intermediate name
	Point org;
	GrafPtr port;
	RgnHandle clip;
	Boolean isValid;							// TRUE if FocusRec is valid
};


//----------------------------------------------------------------------------------------
// FocusRec
//----------------------------------------------------------------------------------------

struct FocusRec
{
	Boolean printing;
	Boolean drawingPictScrap;
	TView* drawingPictScrapView;
	TView* focusedView;
	ViewPortInfo itsViewPortInfo;
};


//----------------------------------------------------------------------------------------
// Typedefs for declaring procedural parameters.
//----------------------------------------------------------------------------------------

typedef pascal void (*DoToViewType)(TView* aView, void* staticLink);
									
typedef pascal Boolean (*TestViewType)(TView* aView, void* staticLink);


//----------------------------------------------------------------------------------------
// CSubViewIterator
//----------------------------------------------------------------------------------------

class CSubViewIterator : public CObjectIterator
{
public:
	CSubViewIterator(TView* itsView,
					 ArrayIndex itsLowBound, ArrayIndex itsHighBound,
					 Boolean itsForward);

	CSubViewIterator(TView* itsView, Boolean itsForward);

	CSubViewIterator(TView * itsView);
	
	TView* CurrentSubView(void);
		// returns the current subview
	
	TView* FirstSubView(void);
		// return the first subview in the iteration

	TView* NextSubView(void);
		// advances the iteration and then returns the subview
};


//----------------------------------------------------------------------------------------
// TView: Renders an image of a document or other data, and identifies mouse-clicked
// objects.
//----------------------------------------------------------------------------------------

class TView : public TEvtHandler
{
public:
	TView* fSuperView;							// the current superview

	TList* fSubViews;							// subviews of this view

	TDocument* fDocument;						// the associated document if any.

	VPoint fLocation;							// the location in superview coordinates

	VPoint fSize;								// the size of this view

	VPoint fTranslation;						// translation of coordinates required to
                                                // account for the origin of the view's
                                                // local coordinate system.

	SizeDeterminer fSizeDeterminer[2];			// how this view's size is to be
                                                // determined

	HLState fHLDesired;							// the hilighting that is desired for this
                                                // view
	
	Boolean	fHandlesCursor;						// TRUE if the view tracks the mouse for
                                                // cursor setting

	Boolean	fLetsSubViewsHandleCursor;			// TRUE if subviews should be given a
                                                   // chance to set the cursor

	Boolean	fHandlesHelp;						// TRUE if the view tracks the mouse for
                                                // showing help balloons

	Boolean	fLetsSubViewsHandleHelp;			// TRUE if subviews should be given a
                                                // chance to display help

	short fCursorID;

	short fHelpID;

	Boolean fShown;								// is this view to be shown

	TPrintHandler* fPrintHandler;				// the associated print handler if any.

	Point fQDOrigin;							// the computed QD origin for the port to
                                                // account for the view's location.

	VPoint fViewToQDOffset;						// the computed translation of coordinates
                                                // necessary to account for adjustment of
                                                // large views down to QD space.

	VRect fVisibleExtent;						// The extent of the view that could be
                                                // "visible" through all its superviews
                                                // (not taking clipping into account

	TAdornment* fViewAdornment;					// manages a list of adorners


	//------------------------------------------------------------------------------------
	// Creation/ Destruction Methods
	//------------------------------------------------------------------------------------

	virtual pascal void Initialize();
		// Puts a newly created TView into a known state. Subclasses should override it
        // and call INHERITED. Typically used to put the object into a state where it can
        // be safely FREED.

	virtual pascal void IView(TDocument* itsDocument,
							  TView* itsSuperView,
							  const VPoint& itsLocation,
							  const VPoint& itsSize,
							  SizeDeterminer itsHSizeDet,
							  SizeDeterminer itsVSizeDet);
		// Initializes a view procedurally.

	virtual pascal void IRes(TDocument* itsDocument,
							 TView* itsSuperView,
							 Ptr& itsParams);
		// Initialize aview from a resource template.

	virtual pascal TObject* Clone();
		// Calls INHERITED Clone and then clones owned objects.

	virtual pascal void WRes(ViewRsrcHandle theResource, Ptr& itsParams);
		// For writing a view out to a resource.

	virtual pascal void WriteRes(ViewRsrcHandle theResource, Ptr& itsParams);
		// For writing a view to a resource works in conjunction with tbe previous method.
        //

	virtual pascal void Free();
		// Frees its subviews, then if its superview is not NIL removes itself from the
        // superviews list of subviews and if its document is not NIL removes itself from
        // the documents list of views. If it has a print handler then it frees it, then
        // calls inherited Free. Frees its fViewAdornment.

	virtual pascal void FreeFromClipboard();
		// If its document is not NIL then calls the document's FreeFromClipboard method,
        // otherwise calls Free.


	//------------------------------------------------------------------------------------
	// Coordinate Conversion Methods
	//------------------------------------------------------------------------------------

	virtual pascal void QDToViewPt(Point qdPoint, VPoint& viewPt);
		// Convert a Quickdraw point to a view point.

	virtual pascal void QDToViewRect(const Rect& qdRect, VRect& viewRect);
		// Convert a Quickdraw rect to a view rect.

	virtual pascal Point ViewToQDPt(const VPoint& viewPt);
		// Convert a view point to a QuickDraw point.

	virtual pascal void ViewToQDRect(const VRect& viewRect, Rect& qdRect);
		// Convert a view rect to a Quickdraw rect.

	virtual pascal void GetLocalOffsetInSuper(VPoint& localOffsetInSuper);
		// Returns the location of the origin for this view coordinate systemin the
        // superview's coordinate system

	virtual pascal void SetLocalOrigin(const VPoint& newOrigin, Boolean redraw);
		// Sets the origin of the view's local coordinate system. Redraws if requested.

	virtual pascal void LocalToSuper(VPoint& thePoint);
		// Given a point in local view coordinates, returns the same point in its
        // superview's coordinate system.

	virtual pascal void LocalToSuperVRect(VRect& viewRect);
		// Given a rect in local view coordinates, returns the same rect in its
        // superview's coordinate system.

	virtual pascal void LocalToSuperRgn(RgnHandle& theRgn);
		// Given a region in local view coordinates, returns the same region in its
        // superview's coordinate system.

	virtual pascal void SuperToLocal(VPoint& thePoint);
		// Given a point in superview coordinates, returns the same point in the view's
        // own coordinate system.

	virtual pascal void SuperToLocalVRect(VRect& viewRect);
		// Given a rect in superview coordinates, returns the same rect in the view's own
        // coordinate system.

	virtual pascal void SuperToLocalRgn(RgnHandle& theRgn);
		// Given a region in superview coordinates, returns the same region inthe view's
        // own coordinate system.

	virtual pascal void LocalToWindow(VPoint& thePoint);
		// Given a point in local view coordinates, returns the same point in its window's
        // (or most super superview's) coordinate system.

	virtual pascal void WindowToLocal(VPoint& thePoint);
		// Given a point in window (or most super superview) coordinates, returns the same
        // point in the view's own coordinate system.


	//------------------------------------------------------------------------------------
	// SubView Management Methods
	//------------------------------------------------------------------------------------

	virtual pascal void EachSubView(DoToViewType DoToSubView, void* staticLink);
		// Iterate over all the views subviews passing each in turn to the procedure
        // "DoToSubView".

	virtual pascal TView* FirstSubViewThat(TestViewType TestSubView, void* staticLink);
		// Iterate over this views subviews and return the first subview that meets
        // 'TestSubView'.

	virtual pascal TView* LastSubViewThat(TestViewType TestSubView, void* staticLink);
		// Iterate over this views subviews and return the last subview that meets
        // "TestSubView".

	virtual pascal void TraverseSubViewsPreOrder(DoToViewType DoToSubView, void* staticLink);
		// Perform a pre-order traversal of all the view's subviews passing each in turn
        // to the procedure "DoToSubView".

	virtual pascal void TraverseSubViewsPostOrder(DoToViewType DoToSubView, void* staticLink);
		// Perform a post-order traversal of all the view's subviews passing each in turn
        // to the procedure "DoToSubView".

	virtual pascal void AddSubView(TView* theSubView);
		// Add 'theSubView' to our list of subviews.

	virtual pascal void RemoveSubView(TView* theSubView);
		// Remove 'theSubView' from our list of subviews.

	virtual pascal void MakeFirstSubView(TView* theSubView);
		// Make 'theSubView ' the first subview in our list.

	virtual pascal void MakeLastSubView(TView* theSubView);
		// Make 'theSubView' the last subview in our list.

	virtual pascal short CountSubViews();
		// Return the number of subviews in our view.

	virtual pascal TView* FindSubView(IDType itsIdentifier);
		// Find subview with identifier 'itsIdentifier' in our list of subviews.


	//------------------------------------------------------------------------------------
	// Open/ Close/Activate Methods
	//------------------------------------------------------------------------------------

	virtual pascal void Activate(Boolean entering);
		// Focuses and then highlights any current selection before calling all its
        // subviews to activate themselves.

	virtual pascal void Open();
		// Calls each of its subviews to Open themselves.

	virtual pascal void Close();
		// Commits the last command before calling each of its subviews to Close
        // themselves.

	virtual pascal void BeInScroller(TScroller* itsScroller);
		// If 'itsScroller' is not NIL then sets its scroll limits toits fSize.

	virtual pascal void BeInPort(GrafPtr itsPort);
		// If its print handler is not NIL, calls DoCheckPrinter. Then calls each of its
        // subviews to BeInPort.

	virtual pascal void ShowReverted();
		// First calls AdjustSize before forcing a redraw of the view, then call
        // ShowReverted for each of its subviews.


	//------------------------------------------------------------------------------------
	// Size Methods
	//------------------------------------------------------------------------------------

	virtual pascal void AdjustSize();
		// Calls ComputeSize, if the size has changed resizes itself and then calls
        // DoPagination so that any page breaks can be recalculated.

	virtual pascal void CalcMinSize(VPoint& minSize);
		// Calculate the minimum size of SELF.

	virtual pascal void ComputeSize(VPoint& newSize);
		// Called by MacAppto get the size of the view computed.

	virtual pascal void Resize(const VPoint& newSize, Boolean invalidate);
		// Resizes SELF to 'width' & 'height', and forces an invalidation if 'invalidate'
        // is TRUE.

	virtual pascal void SuperViewChangedSize(const VPoint& delta, Boolean invalidate);
		// Called when our superview changes size.

	virtual pascal void SubViewChangedSize(TView* theSubView, const VPoint& delta);
		// Called when one of our subviews changes size.


	//------------------------------------------------------------------------------------
	// Location Methods
	//------------------------------------------------------------------------------------

	virtual pascal void Locate(const VPoint& newLoc, Boolean invalidate);
		// Moves our view to coordinate 'h' & 'v' invalidating if 'invalidate' is TRUE.

	virtual pascal void SubViewMoved(TView* theSubView);
		// Called when a subview is moved.

	virtual pascal void SuperViewMoved(Boolean invalidate);
		// Called when our superview is moved.


	//------------------------------------------------------------------------------------
	// Focusing Methods
	//------------------------------------------------------------------------------------

	virtual pascal Boolean Focus();
		// Attempts to set up the drawing environment for this view. That consists of
        // port, origin, gLongOffset and clipping. Returns FALSE if not able to focus. One
        // reason focus could fail is if no superview is able to supply a port (ie not in
        // a window and not printing or drawing pict scrap). Focus will return true,
        // however, if the drawing environment can be setup but none of the view is
        // currently visible due to clipping or scrolling by a superview. This lets you
        // assemble view hierarchies and manipulate views without showing them to the
        // user, but requires that you test IsVisible if you MUST know if the view is in
        // fact visible. (MacApp will never ask a view to draw unless it is visible)

	virtual pascal Boolean FocusOnSuperView();
		// Focuses on our superview.

	virtual pascal void InvalidateFocus();
		// Indicates that the view's focus is no longer valid and must be recomputed.

	virtual pascal Boolean IsFocused();
		// Check to see if we are currently focused.

	virtual pascal void ClipFurtherTo(const Rect& r, short hDeltaOrg, short vDeltaOrg);
		// Set clipping to: (<current clipping> INTERSECT r) OFFSET-BY (hDeltaOrg,
        // vDeltaOrg).

	virtual pascal void OffsetAndClipFurtherTo(const Rect& r, 
											   short hDeltaOrg,
											   short vDeltaOrg);
		// Set clipping to: OFFSET-BY (hDeltaOrg, vDeltaOrg) and (<current clipping>
        // INTERSECT r).

	virtual pascal void AssumeFocused();
		// Causes a ProgramBreak if the focused view isn't SELF

	virtual pascal void UpdateCoordinates();
		// Indicates that the view's computed offsets are no longer valid and must be
        // recomputed.Calls updateCoordinates for all subviews.


	//------------------------------------------------------------------------------------
	// Adornment Methods
	//------------------------------------------------------------------------------------

	virtual pascal void AddAdorner(TAdorner* anAdorner,
									AdornPriority itsPriority,
								   Boolean invalidate);
		// adds an adorner to the view; invalidates the adorner if "invalidate" is TRUE

	virtual pascal void DeleteAdorner(TAdorner* anAdorner, Boolean invalidate);
		// removes an adorner from the view; invalidates the adorner if "invalidate" is
        // TRUE

	virtual pascal void DeleteAdornerByID(IDType itsID, Boolean invalidate);
		// removes an adorner from the view; invalidates the adorner if "invalidate" is
        // TRUE

	virtual pascal void Dim();
		// dims the current image in the view

	virtual pascal void GetAdornExtent(VRect& itsAdornExtent);
		// returns the adorn extent for adorning this view

	virtual pascal void Hilite();
		// hilites the current image in the view by inverting its extent

	virtual pascal TAdornment* MakeAdornment();
		// creates and returns a TAdornment


	//------------------------------------------------------------------------------------
	// Drawing Methods
	//------------------------------------------------------------------------------------

	virtual pascal void DoHighlightSelection(HLState fromHL, HLState toHL);
		// Currently does nothing. Must be overridden.

	virtual pascal void Draw(const VRect& area);
		// Currently does nothing. Must be overridden.

	virtual pascal void DrawContents();
		// Causes the views contents to be drawn and that of any of its subviews.

	virtual pascal TDesignator* GetUserSelection();
		// Returns the designator for the user selection (if any).Defaults to returning
        // the user selection of the document (if any)

	virtual pascal void SetUserSelection(TDesignator* newSelection);
		// Sets the designator for the user selection (if any). Defaults to setting the
        // user selection of the document (if any)

	virtual pascal void UserSelectionChanged();
		// Call this whenever the user's selection changes. Defaults to calling the
        // UserSelectionChanged method of the document (if any).

	virtual pascal Boolean HasPendingUpdate();
		// Returns TRUE if there is a pending update event for this view, FALSE if not.

	virtual pascal void Update();
		// If its window is not NIL then does an update by calling DrawContents.
	

	//------------------------------------------------------------------------------------
	// Validation/ Invalidation Methods
	//------------------------------------------------------------------------------------

	virtual pascal void ForceRedraw();
		// Invalidates the entire view forcing a redraw.

	virtual pascal void InvalidateVRect(const VRect& badVRect);
		// Invalidates a Quickdraw rect by calling InvalidateRect.

	virtual pascal void InvalidateRect(const Rect& badRect);
		// Invalidates a View rect by calling InvalidateRgn.

	virtual pascal void InvalidateRgn(RgnHandle badRgn);
		// Intersects badRgn with the visible extent then calls DoInvalidateRgn with the
        // result. We can only call InvalidRgn on a grafPort belonging to a window, if
        // your code calls InvalidRgn on a non-window grafPort, it will die miserably. So
        // views forward up their superview chain until they reach a view which overrides
        // DoInvalidateRgn (usually a window) which then makes the actual InvalidRgn call
        //

	virtual pascal void DoInvalidateRgn(RgnHandle badRgn);
		// Invalidates a Region by passing it to the superview

	virtual pascal void ValidateVRect(const VRect& goodVRect);
		// Validates a view rect by calling ValidateRect.

	virtual pascal void ValidateRect(const Rect& goodRect);
		// Validates a Quickdraw rect by calling ValidateRgn.

	virtual pascal void ValidateRgn(RgnHandle goodRgn);
		// Intersects goodRgn with the visible extent then calls DoValidateRgn with the
        // result. We can only call ValidRgn on a grafPort belonging to a window, if your
        // code calls ValidRgn on a non-window grafPort, it will die miserably. So views
        // forward up their superview chain until they reach a view which overrides
        // DoValidateRgn (usually a window) which then makes the actual ValidRgn call

	virtual pascal void DoValidateRgn(RgnHandle goodRgn);
		// Validates a Region by passing it to the superview

	virtual pascal void RevealRect(const VRect& rectToReveal,
								   const VPoint& minToSee,
								   Boolean redraw);
		// Scrolls a rect in to view, with 'minToSee' being the point which will end up at
        // the bottom of the view, i.e.the minimum amount to be seen and 'rectToReveal'
        // the rect being scrolled.

	virtual pascal void RevealTop(Boolean redraw);
		// Scroll the top of a rect into view.

	virtual pascal void RevealBottom(Boolean redraw);
		// Scroll the bottom of a rect into view.


	//------------------------------------------------------------------------------------
	// Mouse Handling Methods
	//------------------------------------------------------------------------------------

	virtual pascal Boolean ContainsMouse(const VPoint& theMouse);
		// Is the view point 'theMouse' currently in our view.

	virtual pascal Boolean HandleMouseDown(const VPoint& theMouse,
										   TToolboxEvent* event,
										   Point hysteresis);
		// The method responsible for handling a mouse down in the view, calls
        // DoMouseCommand to handle it.

	virtual pascal void DoMouseCommand(VPoint& theMouse,
									   TToolboxEvent* event,
									   Point hysteresis);
		// Posts a simple tracker which by default calls back tracking methods of this
        // view. There are two possibilities here: 1) Override DoMouseCommand to post
        // commands that represent more complex, even undoable actions.That command may
        // either let its tracking call back the view's tracking methods (the default)
        // which must be overridden to do anything useful or it may simply do all the
        // tracking itself. This mostly depends on whether different commands returned
        // from the view would have different tracking behavior.For example this is the
        // approach taken by DrawShapes. 2) Override the view's tracking methods and let
        // the default tracker them.For example this is the approach taken by TControl.


	//------------------------------------------------------------------------------------
	// Cursor and Help Handling Methods
	//------------------------------------------------------------------------------------

	virtual pascal void ExcludeSubViewRegions(RgnHandle aRegion,
											  TestViewType TestSubView,
											  void* staticLink);
		// Reduces aRegion by the extents of all subviews for which TestSubView returns
        // TRUE. Used by TView.GetDefaultCursorRgn and TView.GetDefaultHelpRgn

	virtual pascal Boolean HandlesCursor();
		// Returns TRUE, if this view tracks the mouse so that it can claim the cursor.
        // The default returns TRUE. If FALSE, subviews may still get a chance to set the
        // cursor, as determined by TView.SubViewsHandleCursor.
				
	virtual pascal Boolean LetsSubViewsHandleCursor();
		// Returns TRUE if this view should allow subviews a chance to set the cursor. If
        // TRUE, the subviews themselves may still return FALSE for HandlesCursor. By
        // setting both values false, cursor tracking can be turned off for an entire
        // window, such as the clipboard window.

	virtual pascal Boolean HandlesHelp();
		// Returns TRUE, if this view tracks the mouse so that it can show help. The
        // default returns TRUE. If FALSE, subviews may still get a chance to show help,
        // as determined by TView.SubViewsHandleHelp.

	virtual pascal Boolean LetsSubViewsHandleHelp();
		// Returns TRUE if this view should allow subviews a chance to show help. The
        // subviews themselves may still return FALSE for HandlesHelp. By setting both
        // values false, help can be turned off for an entire view hierarchy

	virtual pascal TView* HandleCursor(const VPoint& theMouse, RgnHandle cursorRgn);
		// Handles the setting of the cursor by calling our DoSetCursor method and the
        // same For all our subviews.

	virtual pascal TView* HandleHelp(const VPoint& theMouse, RgnHandle helpRgn);
		// !!!

	virtual pascal void DoSetCursor(const VPoint& localPoint, RgnHandle cursorRgn);
		// Can be overridden to set the cursor and compute a cursor region. The default
        // sets the cursor to an arrow and uses TView.GetDefaultCursorRgn to compute the
        // largest appropriate cursor region

	virtual pascal void GetHelpParameters(const VPoint& localPoint,
										  RgnHandle helpRgn,
										  HMMessageRecord& helpMessage,
										  Point& localQDTip,
										  Rect& localQDRect,
										  short& balloonVariant);
		// Determine the parameters needed for showing a help balloon for this view. The
        // default gets the data from a resource of type 'hrct' with id fHelpID.

	virtual pascal void DoShowBalloon(const VPoint& localPoint,
									  RgnHandle helpRgn,
									  const HMMessageRecord& helpMessage,
									  Point localQDTip,
									  const Rect& localQDRect,
									  short balloonVariant);

	virtual pascal void DoShowHelp(const VPoint& localPoint, RgnHandle helpRgn);
		// !!!

	virtual pascal void GetDefaultCursorRgn(const VPoint& localPoint, 
											RgnHandle cursorRgn);
		// Returns the largest appropriate cursor region, called by DoSetCursor.

	virtual pascal void GetDefaultHelpRgn(const VPoint& localPoint, RgnHandle helpRgn);
		// !!!


	//------------------------------------------------------------------------------------
	// Miscellaneous Methods
	//------------------------------------------------------------------------------------

	virtual pascal TView* GetDialogView();
		// If our superview is not NIL then calls its GetDialogView method and returns the
        // result, otherwise returns NIL.

	virtual pascal GrafPtr GetGrafPort();
		// Returns a pointer to our Graf Port.

	virtual pascal TScroller* GetScroller(Boolean immediateSuperView);
		// Returns a reference to our scroller.

	virtual pascal TView* GetRootView();
		// Returns a reference to the root view.

	virtual pascal TWindow* GetWindow();
		// Returns a reference to the window that we are in, if any.

	virtual pascal void GetExtent(VRect& itsExtent);
		// Returns a view rect the size of our extent in 'itsExtent'.

	virtual pascal void GetQDExtent(Rect& qdExtent);
		// Returns a Quickdraw rect the size of our extent in 'qdExtent'.

	virtual pascal void GetFrame(VRect& itsFrame);
		// Returns in'itsFrame' a VRect that is the boundary of our view in superview
        // coordinates. This is calculated by adding its size to its location.

	virtual pascal void GetVisibleRect(VRect& visRect);
		// Returns a rect enclosing the visible portion of the view in view coordinates.

	virtual pascal void GetVisibleQDRect(Rect& visQDRect);
		// Returns a rect enclosing the visible portion of the view in QD coordinates.

	virtual pascal void GetDrawableRect(VRect& drawRect);
		// Returns a rect enclosing the drawable portion of the view in view coordinates.
        //

	virtual pascal void GetDrawableQDRect(Rect& drawQDRect);
		// Returns a rect enclosing the drawable portion of the view in QD coordinates.

	virtual pascal short GetPartCode(const VPoint& theMouse);
		// tells which part of a view, if any, the mouse button was pressed in. Returns:
        // inContent = 3, inDrag = 4, inGrow = 5, inGoAway = 6, or inDesk = 0 (ie. none of
        // the above),depending on where theMouse is in the view

	virtual pascal void ViewEnable(Boolean state, Boolean redraw);
		// Enables & Redraws the view based on 'state' & 'redraw'.

	virtual pascal void Show(Boolean state, Boolean redraw);
		// If 'state' is not fShown then set fShown to TRUE and call ForceRedraw if
        // 'redraw' is TRUE. Otherwise invalidate our focus and set fShown to 'state'.

	virtual pascal Boolean IsShown();
		// Checks to see if our view is currently shown.

	virtual pascal Boolean IsVisible();
		// Returns TRUE if any portion of the view is currently visible when focused

	virtual pascal Boolean IsDrawable();
		// Returns TRUE if any portion of the view is currently drawable when focused

	virtual pascal Boolean IsActive();
		// Returns TRUE if the view is in an "active" superview (typically a window)

	
	//------------------------------------------------------------------------------------
	// Target Management
	//------------------------------------------------------------------------------------

	virtual pascal void SetTarget(TEvtHandler* newTarget);
		// Set the target for this view's layer (i.e. window). Forwards to fSuperView

	virtual pascal void BeTarget();
		// Attempt to make the receiver the target


	//------------------------------------------------------------------------------------
	// Clipboard Handling
	//------------------------------------------------------------------------------------

	virtual pascal PicHandle AsPict();
		// Returns a PicHandle containing a rendering of the view and its subviews.

	virtual pascal Boolean ContainsClipType(ResType aType);
		// Checks to see if the clipboard contains the type 'aType'.

	virtual pascal long GivePasteData(Handle aDataHandle, ResType dataType);
		// Gets the scrap data of the type 'dataType', returns any errors as function
        // result.

	virtual pascal void WriteToDeskScrap();
		// Writes to desk scrap.


	//------------------------------------------------------------------------------------
	// PrintingMethods
	//------------------------------------------------------------------------------------

	virtual pascal void AttachPrintHandler(TPrintHandler* itsPrintHandler);
		// Called by the PrintHandler during its initialization, NOT directly by you

	virtual pascal VCoordinate DoBreakFollowing(VHSelect vhs,
												VCoordinate prevBreak,
												Boolean& automatic);
		// Returns the location of the page break which follows the page break located at
        // 'prevBreak', in direction vhs; returns automatic = TRUE if the page-break is
        // thought of as an 'automatic' rather than a 'manual' (user-specified) one. Note
        // that page-breaks in dimension 'v' are drawn as vertical lines, those in
        // dimension 'h' as horizontal lines

	virtual pascal void DoCalcPageStrips(VPoint& pageStrips);
		// Computes the number of horizontal and vertical page strips.

	virtual pascal void DoCalcViewPerPage(VPoint& viewPerPage);
		// Determine how much of the view normally goes into each printed page, in each
        // dimension.

	virtual pascal void DoCheckPrinter();
		// Check whether printer (if relevant) has changed, and if so, take appropriate
        // action

	virtual pascal void DoDrawPrintFeedback(const VRect& area);
		// Draw page-breaks, page-numbers, view-borders, rulers, etc.

	virtual pascal void DoDrawPageBreak(VHSelect vhs,
										short whichBreak,
										VCoordinate loc,
										Boolean automatic);
		// Draw one page break.

	virtual pascal void DoPagination();
		// Recompute the dividing lines between areas of the view which will be mapped
        // into different printed pages

	virtual pascal void DoPrinterChanged();
		// The metrics relating to printer use have changed; absorb the information and
        // react

	virtual pascal void DoSetPageOffset(const VPoint& coord);
		// Only reimplement in very unusual circumstances involving Tall-UnAdjusted text
        // printing etc.

	virtual pascal void GetPrintExtent(VRect& printExtent);
		// Returns the part of the view that is to be printed. The default is to return
        // the view's extent.

	virtual pascal void PageInteriorChanged(const VRect& newInterior);
		// Allows a view which is printable to react to the changing of the page-interior
        // rectangle


	//------------------------------------------------------------------------------------
	// Inspecting Methods
	//------------------------------------------------------------------------------------

	virtual pascal void Fields(TObject* obj);
		// Called by the Inspector and the Debugger to display the contents of this
        // class's fields.

	virtual pascal void GetInspectorName(Str255& inspectorName);
		// Called by the Inspector to get the name of this class.


	//------------------------------------------------------------------------------------
	// View Tracking Methods
	//------------------------------------------------------------------------------------

	virtual pascal void GoAwayByUser(const VPoint& theMouse);
		// Provides tracking for the GoAway box (or other indicator) and calls CloseByUser
        // if go away was indicated.

	virtual pascal void MoveByUser(const VPoint& theMouse);
		// Provides tracking for the Title bar (or other drag area) and calls Locate if
        // the location changed.

	virtual pascal void ResizeByUser(const VPoint& theMouse);
		// Provides tracking for the Resize box (or other indicator) and calls Resize if
        // the size changed.

	virtual pascal void ZoomByUser(const VPoint& theMouse, short partCode);
		// Provides tracking for the Zoom box (or other indicator) and calls Zoom if zoom
        // was indicated.


	//------------------------------------------------------------------------------------
	// Tracker Tracking Methods
	//------------------------------------------------------------------------------------

	virtual pascal Boolean IsDoneTracking();
		// Indicates whether the Tracker is through tracking.(the command is also through
        // tracking if you return NIL from TrackMouse).Defaults to NOT StillDown. Most
        // programs won't have to mess with this. NOTE you still have to deal with queued
        // events if you change the criteria for TRUE.

	virtual pascal void TrackConstrain(TrackPhase aTrackPhase,
										const VPoint& anchorPoint,
									  	const VPoint& previousPoint,
									    VPoint& nextPoint,
										Boolean mouseDidMove);
		// Override this if your Tracker calls back to the view (the Default) an you want
        // to constrain the mouse point to a grid, force drawing a square, etc. This is
        // called only if fConstrainsMouse is TRUE.

	virtual pascal void TrackFeedback(TrackPhase aTrackPhase,
										const VPoint& anchorPoint,
										const VPoint& previousPoint,
										const VPoint& nextPoint,
										Boolean mouseDidMove,
										Boolean turnItOn);
		// Override this if your Tracker calls back to the view (the Default) an you want
        // to display any tracking feedback

	virtual pascal void TrackMouse(TrackPhase aTrackPhase,
										VPoint& anchorPoint,
										VPoint& previousPoint,
										VPoint& nextPoint,
										Boolean mouseDidMove);
		// Override this if your Tracker calls back to the view (the Default) an you want
        // to display any tracking feedback


	//------------------------------------------------------------------------------------
	// View hierarchy management
	//------------------------------------------------------------------------------------
	
	virtual pascal void AddedASubView(TView* newSubView);
		// Called by AddSubView to notify this view that a subview was added. Calls
        // fSuperView.AddedASubView so that the notification is passed on to all
        // superviews

	virtual pascal void RemovedASubView(TView* theSubView);
		// Called by RemoveSubView to notify this view that a subview was removed. Calls
        // fSuperView.RemovedASubView so that the notification is passed on to all
        // superviews

	virtual pascal void BeInSuperView(TView* itsSuperView);
		// Called to notify this view that it has become a subview of theSuperView.
        // theSuperView can be NIL. Called by TView.AddSubView and TView.RemoveSubView

	virtual pascal void SetHLPenState(HLState fromHL, HLState toHL);
		// Set the pen state for highlighting by XOR, given the highlight transition. You
        // can use this if you use Paint<X> or Frame<X> or line drawing for your
        // highlighting.
		//
		// ??? Add a pattern parameter or make a helper routine that has a pattern
        // parameter ???
};


//----------------------------------------------------------------------------------------
// Global variable declarations.
//----------------------------------------------------------------------------------------

extern TView* gFocusedView;						
	// the view that is currently focused
	
extern Boolean gDrawingPictScrap;				
	// TRUE if a view's Draw routine is being called in order to create PICT data in the
    // Desk Scrap; your View.Draw can check this routine, and insert PictComments as
    // appropriate if it wishes to have them in the Picture in the Desk Scrap
  
extern TView* gDrawingPictScrapView;			
	// the view being currently drawn in the PICT scrap ??? this stuff is a good candidate
    // for a small change to the view architecture to more easily allow providing
    // alternate drawing environments

extern Boolean gPrinting;						
	// true if currently Printing

extern TPrintHandler* gCurrPrintHandler;		
	// If printing, this is set to the print handler. Nil if not printing.

extern TPrintHandler* gNullPrintHandler;		
	// handles printing-relating messages for views which don't print

extern VPoint gPageOffset;
	// offset in view of page being printed

extern TPrintHandler* gPrintHandler;
	// a global print-handler object for use in some standard printing-related activities;
    // this is initialized to be just a reference to gNullPrintHandler, but if you call
    // InitPrinting, that will install a non-trivial print-handler here

extern IDType gWResSignature;					
	// Signature used by TView.WRes

extern Str255 gWResType;						
	// used by TView.WRes Type used by TView.WRes

//----------------------------------------------------------------------------------------
// Global function declarations.
//----------------------------------------------------------------------------------------

pascal ViewRsrcHandle NewViewRsrc(Ptr& p);
	// Convenience routine to create a new view resource template handle. Passes back in
    // "p" the pointer to the first template entry.

pascal void DoneViewRsrc(Handle viewRsrc, long lastPtr);
	// Cuts back the handle to the proper length, based on lastPtr which should be the
    // address of the next template entry.

pascal Ptr ExpandPtr(Handle viewRsrc, Ptr& p, long offset);
	// Bumps size of view resource if necessary to add a new template. Amount to bump is
    // set to at least kViewRsrcExpandAmt over the current size. "p" is offset to point to
    // the next available position for a new template. Returns the old value of p before
    // it was updated.

pascal Ptr ExpandPtrWStr(Handle viewRsrc, Ptr& p, long offset, const long len);
	// Bumps size of view resource if necessary to add a new template which ends with a
    // variable length string, the length of which is passed in len. Returns the old value
    // of p before it was updated.

pascal void OffsetPtr(Ptr& p, long offset);
	// Adds offset to p, forcing word alignment.

pascal void OffsetPtrWStr(Ptr& p, long offset);
	// Adds offset to p, taking length of trailing variable length string into account,
    // forcing word alignment.

pascal void GetFocus(FocusRec& theFocusRec);
	// gets the current focus into a focusrec. The theFocusRec.itsPortInfo.clipRgn must be
    // a valid rgn.Useful for saving and restoring the focus behind MacApp's back.

pascal void SetFocus(const FocusRec& theFocusRec);
	// sets the current focus from a focus rec

pascal void GetViewPortInfo(ViewPortInfo& theViewPortInfo);
	// gets the current PortInfo into a ViewPortInfo. theViewPortInfo.clipRgn must be a
    // valid rgn.

pascal void SetViewPortInfo(const ViewPortInfo& theViewPortInfo);
	// sets the current PortInfo from a ViewPortInfo

pascal Boolean IsRectVisible(const Rect& r);
	// Determine if a rect is visible in the current grafport.

pascal Boolean IsRectDrawable(const Rect& r);
	// Determine if any portion of a rect intersects the ports visRgn AND clipRgn.

pascal void VisibleRect(Rect& r);
	// trims the supplied rect down to its intersection with the port's visRgn

pascal void DrawableRect(Rect& r);
	// trims the supplied rect down to its intersection with the port's visRgn AND
    // clipRgn.

pascal Boolean IsPtVisible(Point pt);
	// Determine if a pt intersects the ports visRgn

pascal Boolean IsPtDrawable(Point pt);
	// Determine if a pt intersects the ports visRgn AND clipRgn.

pascal void PtAndRgn(Point aPoint, RgnHandle aRegion);
	// Adds the given point to the given region

pascal void VRectAndRgn(const VRect& theVRect, RgnHandle theRgn);
	// Call this procedure to add theVRect to theRgn.

pascal void WriteFocus();
	// Writes information about the current focus to the output


#endif


