// UViewServer.h
// Copyright  1984-1991 by Apple Computer Inc.	All rights reserved.

#ifndef __UVIEWSERVER__
#define __UVIEWSERVER__

#ifndef __PASCALDEFS__
#include "PascalDefs.h"
#endif

#ifndef __UGEOMETRY__
#include "UGeometry.h"
#endif

#ifndef __BALLOONS__
#include "Balloons.h"
#endif

#ifndef __UEVTHANDLER__
#include "UEvtHandler.h"
#endif

#ifndef __UADORNERS__
#include "UAdorners.h"
#endif

#ifndef __UITERATOR__
#include "UIterator.h"
#endif

//----------------------------------------------------------------------------------------
// argument of NewPaletteWindow
//----------------------------------------------------------------------------------------
const VHSelect kLeftPalette = hSel;
const VHSelect kTopPalette = vSel;	

// --------------------------------------------------------------------------------------------------
// TViewServer is responsible for creating views, either procedurally or from templates.
//----------------------------------------------------------------------------------------

class TViewServer : public TObject
{
public:

	//------------------------------------------------------------------------------------
	// Initializer, Pascal I<Method> and Free.
	//------------------------------------------------------------------------------------

	virtual pascal void Initialize();		// override 
	/*  Puts a newly created TViewServer into a known state. Subclasses should
	  override it and call INHERITED. Typically used to put the object into a state where
	  it can be safely FREED.  */

	virtual pascal void IViewServer();
		// Initializes the TViewServer. 

	virtual pascal void Free();				// override 
		// Sets the global gViewServer to NIL. 

	//------------------------------------------------------------------------------------
	// Procedural View Creation
	//------------------------------------------------------------------------------------

	virtual pascal TWindow* NewPaletteWindow(short itsRsrcID,
									 Boolean wantHScrollBar,
									 Boolean wantVScrollBar,
									 TDocument* itsDocument,
									 TView* itsMainView,
									 TView* itsPaletteView,
									 short sizePalette,
									 VHSelect whichWay);
		// Used for creating MacDraw-like windows with 1 non-scrolling palette along the
		// left edge (whichWay = h), or a non-scrolling status area at the top of the window
		// (whichWay = v), and a main view that may or may not scroll. Signals Failure if the
		// window could not be created.
	
	virtual pascal TWindow* NewSimpleWindow(short itsRsrcID,
									Boolean wantHScrollBar,
									Boolean wantVScrollBar,
									TDocument* itsDocument,
									TView* itsView);
		// Used for creating simple windows that contain 1 view and may or may not scroll.
		// Signals Failure if the window could not be created.
	
	virtual pascal TWindow* NewTWindow(short itsRsrcID, TDocument* itsDocument);
		// Used for creating a window object. Used within NewPaletteWindow &
		// NewSimpleWindow. SIgnals Failure if the window could not be created.
	
	//------------------------------------------------------------------------------------
	// Template View Creation
	//------------------------------------------------------------------------------------

	virtual pascal TWindow* NewTemplateWindow(short viewRsrcID, TDocument* itsDocument);
		// Used for creating a window using a 'view' template.

	virtual pascal TView* DoCreateViews(TDocument* itsDocument,
										TView* parentView,
										short itsRsrcID,
										const VPoint& subViewOffset);
		// This is the method responsible for creating view hierarchies from resource
        // templates. If fNextHandler is not NIL it calls its DoCreateViews method.
        // Otherwise it creates the desired view heirarchy.

	virtual pascal TView* CreateAView(TDocument* itsDocument,
									  TView* itsSuperView,
									  Ptr& itsParams);
		// For creating a single view from a resource template view. If fNextHandler is
        // not NIL calls its CreateAView method, otherwise it proceeds to create the
        // desired view.

	//------------------------------------------------------------------------------------
	// Debugging Support
	//------------------------------------------------------------------------------------

private:
#if qDebugMsg
	virtual pascal void TViewServer::ReportTemplate(const ViewTemplate& theViewData);
		// Writes description of view to std err.
#endif

};


// --------------------------------------------------------------------------------------------------
extern TViewServer* gViewServer;				// the globally accessible view server


#endif


