// UWindow.h
// Copyright 1984-1991 Apple Computer, Inc. All rights reserved.

#ifndef __UWINDOW__
#define __UWINDOW__

#ifndef __PASCALDEFS__
#include <PascalDefs.h>
#endif

#ifndef __UGEOMETRY__
#include <UGeometry.h>
#endif

#ifndef __WINDOWS__
#include <Windows.h>
#endif

#ifndef __UVIEW__
#include <UView.h>
#endif

#ifndef __UCOMMAND__
#include <UCommand.h>
#endif



//----------------------------------------------------------------------------------
// Constants
//----------------------------------------------------------------------------------

const short kFloatWindowKind = 128;				// the windowKind of floating windows 

const short CurDeKind = 2594;					// lo memory: the windowKind associated
                                                // with curDeactive 


//----------------------------------------------------------------------------------
// WindowTemplate
//----------------------------------------------------------------------------------

struct WindowTemplate
{
	short procID;
	Boolean hasGoAway:1;
	Boolean resizable:1;
	Boolean isModal:1;
	Boolean doFirstClick:1;
	Boolean freeOnClosing:1;
	Boolean disposeOnFree:1;
	Boolean closesDocument:1;
	Boolean openInitially:1;
	Boolean mustAdaptToScreen:1;
	Boolean stagger:1;
	Boolean mustForceOnScreen:1;
	Boolean vertCenter:1;
	Boolean horzCenter:1;
	char filler:3;
	IDType targetID;
	Str255 title;								// Actually variable length 
};

typedef WindowTemplate* WindowTemplatePtr;


//----------------------------------------------------------------------------------
// External and forward class declarations
//----------------------------------------------------------------------------------

class TDocument;
class TToolboxEvent;


//----------------------------------------------------------------------------------
// TWindow: Corresponds to a desktop Window. The outermost of a nested set of views.
//----------------------------------------------------------------------------------

class TWindow : public TView
{
public:
	WindowPtr fWMgrWindow;						// the window manager window Ptr 

	short fProcID;								// the proc ID of the window 

	Boolean fUpdating;							// True when TWindow.Update has control.
                                                // Changes behaviour of TWindow.Focus to
                                                // clip TO the update region instead of
                                                // clipping OUT the update region 

	Rect fMoveBounds;							// bounds over which the window may be
                                                // moved 

	Rect fResizeLimits;							// limits over which the window may be
                                                // resized 

	TEvtHandler* fTarget;						// when this window is activated; the
                                                // TEvtHandler to SetTarget on 

	IDType fTargetID;							// corresponding ID to fTarget 

	short fPreDocname;							// amount of window title that precedes
                                                // the document name 

	short fConstTitle;							// amount of the title name that is
                                                // constant 

	Boolean fIsActive;							// is this window active 

	Boolean fIsResizable;						// is this window resizeable 

	Boolean fIsClosable;						// is this window closable 

	Boolean fFreeOnClosing;						// should this window free itself when it
                                                // closes 

	Boolean fDisposeOnFree;						// should the window manager window be
                                                // disposed when this window frees 

	Boolean fClosesDocument;					// does this window close the document 

	Boolean fOpenInitially;						// should this window be opened when its
                                                // document is asked to show its windows 

	Boolean fIsModal;							// should this window behave modally 

	Boolean fDoFirstClick;						// should a click in this window not only
                                                // select it if it is inactive but also be
                                                // sent to its contents 

	Boolean fMustAdapt;							// does this window require adaption to
                                                // the screen 

	Boolean fMustHorzCenter;					// does this window require centering
                                                // horizontally 

	Boolean fMustVertCenter;					// does this window require centering
                                                // vertically 

	Boolean fMustStagger;						// does this window require staggering 

	Boolean fMustForceOnScreen;					// does this window require forcing on the
                                                // screen 

	Boolean fAdapted;							// has this window's size ever been
                                                // adapted to the screen 

	Boolean fHorzCentered;						// has this window been centered
                                                // horizontally 

	Boolean fVertCentered;						// has this window been centered
                                                // vertically 

	Boolean fStaggered;							// has this window been staggered 

	Boolean fForcedOnScreen;					// has this window been forced on the
                                                // screen 

	Boolean fFloats;							// does this window float above
                                                // non-floating windows (NOT SUPPORTED IN
                                                // 2.0) 

	Boolean fDismissed;							// Has the dialog been dismissed? 

	Boolean fHideOnSuspend;						// does this window get hid on suspend? 

	Boolean fWasHiddenOnSuspend;				// TRUE if this window was hid on suspend

	Point fContRgnInset;						// topleft inset of cont rgn in struc rgn

	Point fContDifference;						// total amount the content is less than
                                                // and offset into the structure (Accounts
                                                // for title bar, etc.) 

	Boolean fGenerateActivates;					// a flag indicating whether or not
                                                // TWindow.Show should generate the
                                                // necessary activate events 


	//------------------------------------------------------------------------------------
	// Creation/ Destruction Methods 
	//------------------------------------------------------------------------------------

	virtual pascal void Initialize();
		// Puts a newly created TWindow into a known state. Subclasses should override it
        // and call INHERITED. Typically used to put the object into a state where it can
        // be safely FREED. 

	virtual pascal void IWindow(TDocument* itsDocument,
								WindowPtr itsWMgrWindow,
								Boolean canResize,
								Boolean canClose,
								Boolean disposeOnFree);
		// Initialize a window procedurally. 

	virtual pascal void IRes(TDocument* itsDocument,
							 TView* itsSuperView,
							 Ptr& itsParams);
		// Initialize a window from a resource template. 

	virtual pascal WindowPtr GetBehindWindowPtr();
		// return the windowPtr in the windowlist behind which this window gets created 

	virtual pascal void WRes(ViewRsrcHandle theResource, Ptr& itsParams);
		// For writing a window out to a resource. 

	virtual pascal void WriteRes(ViewRsrcHandle theResource, Ptr& itsParams);
		// For writing a TWindow to a resource, works in conjunction with the previous
        // method. 

	virtual pascal void Free();
		// Frees the window if fDisposeOnFree is TRUE, also deletes the window from the
        // document if there is one, otherwise deletes it from the applications list of
        // free windows. 


	//------------------------------------------------------------------------------------
	// Open/Close Methods 
	//------------------------------------------------------------------------------------

	virtual pascal void Open();
		// Makes the window visible by showing it before calling inherited Open. 

	virtual pascal void Close();
		// Calls inherited Close and the hides & deactivates the window. 

	virtual pascal void CloseByUser();
		// If the window is shown and it can be closed handles the closing. 

	virtual pascal void GoAwayByUser(const VPoint& theMouse);
		// Responsible for calling CloseByUser. 


	//------------------------------------------------------------------------------------
	// Activation Methods 
	//------------------------------------------------------------------------------------

	virtual pascal void Activate(Boolean entering);
		// Activate the window. 


	//------------------------------------------------------------------------------------
	// Size/ Location Methods 
	//------------------------------------------------------------------------------------

	virtual pascal void Locate(const VPoint& newLoc, Boolean invalidate);
		// If fWMgrWindow is not NIL then MoveWindow to relocate the window. 

	virtual pascal void MoveByUser(const VPoint& theMouse);
		// Handles the user moving the window by dragging the title bar, calls DragWindow
        // to handle movement. 

	virtual pascal void ResizeByUser(const VPoint& theMouse);
		// Handles the user resizing the window by dragging the growbox, calls GrowWindow
        // to handle movement and Resize to resize the window. 

	virtual pascal void Resize(const VPoint& newSize, Boolean invalidate);
		// Used to resize the window by setting its width & height to the parameters
        // passed in. 

	virtual pascal void Zoom(short partCode);
		// Responsible for actually handling the zooming of a window when the user clicks
        // in the zoombox. 

	virtual pascal void ZoomByUser(const VPoint& theMouse, short partCode);
		// handles the user clicking in the zoombox to zoom the window, calls Zoom. 


	//------------------------------------------------------------------------------------
	// Focusing Methods 
	//------------------------------------------------------------------------------------

	virtual pascal Boolean Focus();
		// Attempts to set the Focus to the window, returns TRUE if it succeeds, FALSE if
        // not. 

	virtual pascal Boolean FocusOnSuperView();
		// Attempts to set the Focus to our superview, returning TRUE if it succeeds or
        // FALSE if not. 


	//------------------------------------------------------------------------------------
	// Drawing Methods 
	//------------------------------------------------------------------------------------

	virtual pascal void DrawResizeIcon();
		// Draws the grow icon by calling the toolbox routine DrawGrowIcon. 


	//------------------------------------------------------------------------------------
	// Mouse Handling Methods 
	//------------------------------------------------------------------------------------

	virtual pascal Boolean HandleMouseDown(const VPoint& theMouse,
										   TToolboxEvent* event,
										   Point hysteresis);
		// Handles mouse downs in the window. 

	virtual pascal short GetPartCode(const VPoint& theMouse);
		// determines what part of the window theMouse is in 

	virtual pascal Boolean HasPendingUpdate();
		// Returns TRUE if there is a pending update event for this window, FALSE if not.

	virtual pascal void Update();
		// Handle the update region and drawcontents on SELF. 

	virtual pascal void DoInvalidateRgn(RgnHandle badRgn);
		// If a WMgrWindow is available calls InvalRgn 

	virtual pascal void DoValidateRgn(RgnHandle goodRgn);
		// If a WMgrWindow is available calls ValidRgn 


	//------------------------------------------------------------------------------------
	// Menu Handling Methods 
	//------------------------------------------------------------------------------------

	virtual pascal Boolean AllowsMenuAccess();
		// By default returns TRUE. 

	virtual pascal void DoSetupMenus();
		// If the window is not modal calls inherited DoSetupMenus. 

	virtual pascal void DoMenuCommand(CmdNumber aCmdNumber);
		// Handles closing the window. 


	//------------------------------------------------------------------------------------
	// Target and view management 
	//------------------------------------------------------------------------------------

	virtual pascal void SetTarget(TEvtHandler* newTarget);
		// Set fTarget to 'newTarget' and if we are the active window then call
        // gApplication.SetTarget. 

	virtual pascal TEvtHandler* GetTarget();
		// Return the current fTarget of this window 

	virtual pascal void RemovedASubView(TView* theSubView);
		// Overridden to fix up the target, if the subview removed was the target 


	//------------------------------------------------------------------------------------
	// Cursor Handling 
	//------------------------------------------------------------------------------------

	virtual pascal Boolean HandlesCursor(); // OVERRIDE
		// Returns TRUE if fHandlesCursor is TRUE, and the window is active, or
        // fDoFirstClick is TRUE 

	virtual pascal Boolean LetsSubViewsHandleCursor(); // OVERRIDE

	virtual pascal Boolean HandlesHelp(); // OVERRIDE
		// Returns the value of fDoesHandleHelp 

	virtual pascal Boolean LetsSubViewsHandleHelp(); // OVERRIDE


	//------------------------------------------------------------------------------------
	// Misc. Methods 
	//------------------------------------------------------------------------------------

	virtual pascal void AboutToLoseControl();
		// called when the application is about to lose control 

	virtual pascal void RegainControl();
		// called when the application is regaining control 

	virtual pascal void Show(Boolean state, Boolean redraw);
		// Called to make the window visible or not based on 'state'. 

	virtual pascal Boolean IsShown();
		// Returns TRUE if the window is currently shown. 

	virtual pascal Boolean IsActive();
		// Returns TRUE if the window is currently active. 

	virtual pascal void Select();
		// Called to select the window. 

	virtual pascal void SetTitle(const Str255& newTitle); 
		// Sets the windows title to 'newTitle'. 

	virtual pascal void GetTitle(Str255& theTitle); 
		// Gets the current title of the window. 

	virtual pascal void SetTitleForDoc(const Str255& newDocTitle);// %%% AMB
		// Called when the documents name is known or changed 

	virtual pascal void AdaptToScreen();
		// Resizes the window to fit the screen on which it is being displayed. 

	virtual pascal void ForceOnScreen();
		// ForceOnScreen gaurantees that some minimal drag area is accessible to the user,
        // to be dragged to the desired location. 

	virtual pascal void Center(Boolean horizontally,
							   Boolean vertically,
							   Boolean forDialog);
		// Centers the window on the screen either horizontally, vertically or both. 

	virtual pascal void SimpleStagger(short dh, short dv, short& counter);
		// Handles the staggering of windows as they are opened. 

	virtual pascal GrafPtr GetGrafPort();
		// Returns the windows GrafPtr. 

	virtual pascal GDHandle GetMaxIntersectedDevice(Rect& screenRect);
		// Returns the screenRect of the most intersected device & its GDHandle, or if CQD
        // isn't available it returns GetGrayRgn intersected with screenbits.bounds and
        // NIL for the GDHandle. 

	virtual pascal TWindow* GetWindow();
		// Returns SELF. 

	virtual pascal void SetResizeLimits(Point itsMinSize, Point itsMaxSize);
		// Used to set the limits to which the window can be resized. 

	virtual pascal void InstallDocument(TDocument* itsDocument);
		// Sets our fDocument field to 'itsDocument' and then installs the SELF in the
        // document, if the document is not NIL. 

	virtual pascal Boolean IsDraggable(const Rect& whichRect);
		// Returns TRUE if any of the corner points of whichRect are draggable. 

	virtual pascal Boolean IsHiddenOnSuspend();
		// Returns TRUE if this window gets hidden when the application is suspended. 

	virtual pascal Boolean IsDismissed();
		// Returns TRUE if this dialog has been dismissed. 

	virtual pascal void Dismiss();
		// dismissed the dialog that is currently being posed modally 

	virtual pascal Boolean BuildWindowRgns(Boolean build);
		// IF build=kBuild THEN ensures that the window regions are valid; sets
        // fContRgnInset, fContDifference. Cleans up if necessary, when done with window
        // regions. Returns the old state of the window regions. 

	virtual pascal void PoseModally();
		// poses this window in a modal state, called (usually) by TDialogView.PoseModally


	//------------------------------------------------------------------------------------
	// Target management 
	//------------------------------------------------------------------------------------

	virtual pascal void DoKeyCommand(TToolboxEvent* event);
		// Overridden to handle the tab key 


	//------------------------------------------------------------------------------------
	// Inspecting Methods 
	//------------------------------------------------------------------------------------

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields. 

	virtual pascal void GetInspectorName(Str255& inspectorName);
		// Used by the Inspector to get the name of this class. 

	virtual pascal void GetLocationAdjustment(Point& delta);

	virtual pascal void GetStandardStateFrame(const VRect& boundingRect,
											   VRect& stdFrame);

	virtual pascal void GetUserStateFrame(const VRect& boundingRect,
										   VRect& userFrame);
};


//----------------------------------------------------------------------------------------
// TCloseWindowCommand: Tells the application to close a window
//----------------------------------------------------------------------------------------

class TCloseWindowCommand : public TNoChangesCommand
{
public:
	virtual pascal void ICloseWindowCommand(CmdNumber itsCmdNumber, TWindow* itsWindow);
		// Initialize the CloseWindowCommand procedurally. 

	virtual pascal void DoIt();
		// tell the application to close a window. 

	virtual pascal void Fields(TObject* obj);
		// Used by the Inspector and the Debugger to display the contents of this class's
        // fields. 
};


//----------------------------------------------------------------------------------------
// Global function declarations
//----------------------------------------------------------------------------------------

extern Boolean IsDocumentWindow(WindowPtr aWindow);

pascal Boolean IsDialog(WindowPtr window);

pascal Boolean IsFloatWindow(WindowPtr window);

pascal WindowPtr GetLastFloatingWindowPtr();

pascal WindowPtr MAGetActiveWindow();

pascal WindowPtr MAFrontWindow();


//----------------------------------------------------------------------------------------
// Global variable declarations
//----------------------------------------------------------------------------------------

extern Rect gStdWMoveBounds;					// standard boundsRect to pass to
                                                // DragWindow Toolbox routine 

extern Rect gStdWSizeRect;						// standard sizeRect to pass to GrowWindow
                                                // Toolbox routine 

extern Rect gStdWScreenRect;					// Used to force a window onto the screen.
                                                // If the window is not in the GrayRgn,
                                                // then at least one corner of the window
                                                // must lie within gStdWScreenRect. 

extern short gStdStaggerCount;					// Used to stagger windows created from
                                                // templates 

#endif


