{[a-,body+,h-,o=100,r+,rec+,t=4,u+,#+,j=20/57/1$,n-]}
{ UAdorners.p }
{ Copyright  1986-1991 by Apple Computer, Inc. All rights reserved.}

UNIT UAdorners;

	INTERFACE

		USES
			{  requirements for this unit's interface. }
			Types, MacAppTypes,
			{}
			Memory, UPascalObject, UObject,
			{}
			QuickDraw, UDesignator, AppleEvents, UEvtHandler, UList;

		CONST
			kLowestAdornPriority 	= 127;				{ Low priority adorners are considered last
														 }
			kLowAdornPriority		= kLowestAdornPriority - 32; { Low priority adorners are considered last
														 }
			kViewAdornPriority 	= 64;					{ View priority is special case }
			kHighAdornPriority= kViewAdornPriority - 32; { High priority adorners take precedence }
			kHighestAdornPriority	= 0;				{ High priority adorners take precedence }

			kFreeOnDeletion		= TRUE;
			kDontFreeOnDeletion = TRUE;
			
			kDimAdorner			= 'dima';
			kDrawAdorner		= 'draw';
			kEraseAdorner		= 'eras';
			kFrameAdorner		= 'fram';
			kHiliteAdorner		= 'hili';
			kLineBottomAdorner	= 'lnbo';
			kLineLeftAdorner	= 'lnlf';
			kLineRightAdorner	= 'lnrt';
			kLineTopAdorner		= 'lntp';
			kOvalAdorner		= 'oval';
			kPrintAdorner		= 'prnt';
			kResizeIconAdorner	= 'resz';
			kRRectAdorner		= 'rrct';
			kSelectionAdorner	= 'sele';

		TYPE
		
			TList				= OBJECT; EXTERNAL;
			TView				= OBJECT; EXTERNAL;

			AdornPriority 	= kHighestAdornPriority..kLowestAdornPriority; { Command priorities }
{--------------------------------------------------------------------------------------------------}
			TAdornment			= OBJECT; FORWARD;

			TAdorner			= OBJECT (TObject)
			{ abstract superclass for adorners which are basically dumb redering algorithms }

				fID:				IDType;	{ The ID of this adorner, to distinguish it from other instances }
				fFreeOnDeletion:	BOOLEAN; { Should the adorner be freed when removed from an adornment }
				{ === constructor/destructor === }
				PROCEDURE TAdorner.Initialize; OVERRIDE;
				{ sets fAdornPriority to kLowestAdornPriority }

				PROCEDURE TAdorner.IAdorner(itsID: IDType; freeOnDeletion: BOOLEAN);
				{ initializes the adorner }
				
				PROCEDURE TAdorner.AssociateWithAdornment(itsAdornment: TAdornment; associate: BOOLEAN);
				{ Associate this adorner with an adornment }

				{ === rendering ==== }
				PROCEDURE TAdorner.DrawContents(itsView: TView; area: VRect);
				{ Calls SELF.Draw }
				
				PROCEDURE TAdorner.Draw(itsView: TView; area: VRect);
				{ Draws the adorner before the view's Draw. }

				PROCEDURE TAdorner.DrawLine(vhs: VHSelect; lh, lv, lto: INTEGER);
				{ Utility routine for adorners - draws a line. }

				PROCEDURE TAdorner.DoHighlightSelection(itsView: TView; area: VRect; fromHL, toHL: HLState);
				{ Adorns the selection before the view's DoHighlightSelection. }

				PROCEDURE TAdorner.InvalidateAdorner(itsView: TView);
				{ Invalidates the adorner in its view. }
				
				PROCEDURE TAdorner.ViewChangedSize ( itsView: TView;
													delta: VPoint;
													invalidate: BOOLEAN );
				{ This method is called when the owning view changes size }
				
				END;

{--------------------------------------------------------------------------------------------------}

			TDimAdorner	= OBJECT (TAdorner)
			
				PROCEDURE TDimAdorner.Initialize; OVERRIDE;
				{ Initializes this adorner with ID kDimAdorner and fFreeOnDeletion FALSE }
				
				PROCEDURE TDimAdorner.IDimAdorner;
				{ Initializes this adorner with ID kDimAdorner and fFreeOnDeletion FALSE }

				PROCEDURE TDimAdorner.Draw(itsView: TView; area: VRect); OVERRIDE;
				{ calls itsView.Dim }
				END;

{--------------------------------------------------------------------------------------------------}

			TDrawAdorner	= OBJECT (TAdorner)

				PROCEDURE TDrawAdorner.Initialize; OVERRIDE;
				{ Initializes this adorner with ID kDrawAdorner and fFreeOnDeletion FALSE }
			
				PROCEDURE TDrawAdorner.IDrawAdorner;
				{ Initializes this adorner with ID kDrawAdorner and fFreeOnDeletion FALSE }

				PROCEDURE TDrawAdorner.Draw(itsView: TView; area: VRect); OVERRIDE;
				{ calls itsView.Draw }
				END;

{--------------------------------------------------------------------------------------------------}

			TEraseAdorner	= OBJECT (TAdorner)

				PROCEDURE TEraseAdorner.Initialize; OVERRIDE;
				{ Initializes this adorner with ID kEraseAdorner and fFreeOnDeletion FALSE }
			
				PROCEDURE TEraseAdorner.IEraseAdorner;
				{ Initializes this adorner with ID kEraseAdorner and fFreeOnDeletion FALSE }

				PROCEDURE TEraseAdorner.Draw(itsView: TView; area: VRect); OVERRIDE;
				{ calls itsView.Draw }
				END;

{--------------------------------------------------------------------------------------------------}

			THiliteAdorner	= OBJECT (TAdorner)
			
				PROCEDURE THiliteAdorner.Initialize; OVERRIDE;
				{ Initializes this adorner with ID kHiliteAdorner and fFreeOnDeletion FALSE }

				PROCEDURE THiliteAdorner.IHiliteAdorner;
				{ Initializes this adorner with ID kHiliteAdorner and fFreeOnDeletion FALSE }

				PROCEDURE THiliteAdorner.Draw(itsView: TView; area: VRect); OVERRIDE;
				{ calls itsView.Hilite }
				END;

{--------------------------------------------------------------------------------------------------}

			TPrintAdorner	= OBJECT (TAdorner)
			
				PROCEDURE TPrintAdorner.Initialize; OVERRIDE;
				{ Initializes this adorner with ID kPrintAdorner and fFreeOnDeletion FALSE }

				PROCEDURE TPrintAdorner.IPrintAdorner;
				{ Initializes this adorner with ID kPrintAdorner and fFreeOnDeletion FALSE }

				PROCEDURE TPrintAdorner.DoHighlightSelection(itsView: TView; area: VRect; fromHL, toHL: HLState); OVERRIDE;
				{ calls itsView.DoHighlightSelection(fromHL, toHL) }
				END;

{--------------------------------------------------------------------------------------------------}

			TResizeIconAdorner	= OBJECT (TAdorner)
			
				PROCEDURE TResizeIconAdorner.Initialize; OVERRIDE;
				{ Initializes this adorner with ID kResizeIconAdorner and fFreeOnDeletion FALSE }

				PROCEDURE TResizeIconAdorner.IResizeIconAdorner;
				{ Initializes this adorner with ID kResizeIconAdorner and fFreeOnDeletion FALSE }

				PROCEDURE TResizeIconAdorner.DoHighlightSelection(itsView: TView; area: VRect; fromHL, toHL: HLState); OVERRIDE;
				{ calls TWindow(itsView).DrawResizeIcon }
				END;
{--------------------------------------------------------------------------------------------------}

			TSelectionAdorner	= OBJECT (TAdorner)
			
				PROCEDURE TSelectionAdorner.Initialize; OVERRIDE;
				{ Initializes this adorner with ID kSelectionAdorner and fFreeOnDeletion FALSE }

				PROCEDURE TSelectionAdorner.ISelectionAdorner;
				{ Initializes this adorner with ID kSelectionAdorner and fFreeOnDeletion FALSE }

				PROCEDURE TSelectionAdorner.DoHighlightSelection(itsView: TView; area: VRect; fromHL, toHL: HLState); OVERRIDE;
				{ calls itsView.DoHighlightSelection(fromHL, toHL) }
				END;

{--------------------------------------------------------------------------------------------------}

			TFrameAdorner	= OBJECT (TAdorner)
			
				PROCEDURE TFrameAdorner.IFrameAdorner(itsID: IDType; freeOnDeletion: BOOLEAN);
				{ initializes this adorner with itsView }

				PROCEDURE TFrameAdorner.Draw(itsView: TView; area: VRect); OVERRIDE;
				{ frames the extent of itsView }
				END;
{--------------------------------------------------------------------------------------------------}

			TLineBottomAdorner	= OBJECT (TAdorner)
			
				PROCEDURE TLineBottomAdorner.ILineBottomAdorner(itsID: IDType; freeOnDeletion: BOOLEAN);
				{ initializes this adorner with itsView }

				PROCEDURE TLineBottomAdorner.Draw(itsView: TView; area: VRect); OVERRIDE;
				{ draws bottom line }
				END;

{--------------------------------------------------------------------------------------------------}

			TLineLeftAdorner	= OBJECT (TAdorner)
			
				PROCEDURE TLineLeftAdorner.ILineLeftAdorner(itsID: IDType; freeOnDeletion: BOOLEAN);
				{ initializes this adorner with itsView }

				PROCEDURE TLineLeftAdorner.Draw(itsView: TView; area: VRect); OVERRIDE;
				{ draws left line }
				END;

{--------------------------------------------------------------------------------------------------}

			TLineRightAdorner	= OBJECT (TAdorner)
			
				PROCEDURE TLineRightAdorner.ILineRightAdorner(itsID: IDType; freeOnDeletion: BOOLEAN);
				{ initializes this adorner with itsView }

				PROCEDURE TLineRightAdorner.Draw(itsView: TView; area: VRect); OVERRIDE;
				{ draws right line }
				END;

{--------------------------------------------------------------------------------------------------}

			TLineTopAdorner	= OBJECT (TAdorner)
			
				PROCEDURE TLineTopAdorner.ILineTopAdorner(itsID: IDType; freeOnDeletion: BOOLEAN);
				{ initializes this adorner with itsView }

				PROCEDURE TLineTopAdorner.Draw(itsView: TView; area: VRect); OVERRIDE;
				{ draws top line }
				END;

{--------------------------------------------------------------------------------------------------}

			TOvalAdorner	= OBJECT (TAdorner)
			
				PROCEDURE TOvalAdorner.IOvalAdorner(itsID: IDType; freeOnDeletion: BOOLEAN);
				{ initializes this adorner with itsView }

				PROCEDURE TOvalAdorner.Draw(itsView: TView; area: VRect); OVERRIDE;
				{ draws oval adorner }
				END;

{--------------------------------------------------------------------------------------------------}

			TRRectAdorner	= OBJECT (TAdorner)
			
				PROCEDURE TRRectAdorner.IRRectAdorner(itsID: IDType; freeOnDeletion: BOOLEAN);
				{ initializes this adorner with itsView }

				PROCEDURE TRRectAdorner.Draw(itsView: TView; area: VRect); OVERRIDE;
				{ adorns itsView with a round rectangle }
				END;


{--------------------------------------------------------------------------------------------------}

			AdornerElement = RECORD
				priority:	AdornPriority;
				adorner:	TAdorner;
			END;
			AdornerElementPtr = ^AdornerElement;

			TAdornerList		= OBJECT (TSortedDynamicArray)	{ A list for keeping TAdorner objects ordered by
														 priority }
				PROCEDURE TAdornerList.IAdornerList;
				{ Initialize the AdornerList procedurally. }

				FUNCTION  TAdornerList.CompareElements(Element1, Element2: UNIV Ptr): CompareResult; OVERRIDE;
				{ Compares Two TAdorner objects based on their priority }
	
				PROCEDURE TAdornerList.AddAdorner(itsAdorner: TAdorner;
													itsPriority: AdornPriority);
				{ Add the adorner to the list }

				FUNCTION  TAdornerList.AdornerAt(index: ArrayIndex): TAdorner;

				PROCEDURE TAdornerList.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields. }

				END;

{--------------------------------------------------------------------------------------------------}

			TAdornment			= OBJECT (TAdorner)

				fView:				TView;
				fAdorners:			TAdornerList;		{ Sorted List of TAdorner objects }

				fPenSize:			Point;				{ pen size for adorners to use }
				fDrawShadow:		Boolean;			{ Draw a shadow on the result of all the adorners }

				{ === constructor/destructor === }
				PROCEDURE TAdornment.Initialize; OVERRIDE;
				{ sets fAdorners  to NIL }

				PROCEDURE TAdornment.IAdornment(itsID: IDType;
												itsView: TView;
												itsPenSize: Point);
				{ initializes the adorner manager, allocate an adorner list }

				PROCEDURE TAdornment.Free; OVERRIDE;
				{ calls FreeList on fAdorners }

				{ === adorner mgmt ==== }
				PROCEDURE TAdornment.AddAdorner(anAdorner: TAdorner;
												itsPriority:  AdornPriority;
												invalidate: BOOLEAN);
				{ adds an adorner to the adorner manager's adorner list }

				PROCEDURE TAdornment.DeleteAdorner(anAdorner: TAdorner;
												invalidate: BOOLEAN);
				{ removes an adorner from the adorner manager's adorner list }

				PROCEDURE TAdornment.DeleteAdornerByID(itsID: IDType;
											  	invalidate: BOOLEAN);
				{ removes an adorner from the adorner manager's adorner list }

				FUNCTION TAdornment.AdornerAt(i: ArrayIndex): TAdorner;
				{ returns the i'th adorner in the list }
				FUNCTION TAdornment.AdornerWithID(itsID: IDType): TAdorner;
				
				{ === rendering ==== }
				PROCEDURE TAdornment.PreDraw(itsView: TView; area: VRect); 
				{ Prepares for shadowed adornment }
				
				PROCEDURE TAdornment.DrawContents(itsView: TView; area: VRect); OVERRIDE;
				{ Calls PreDraw, Draw and PostDraw }
				PROCEDURE TAdornment.Draw(itsView: TView; area: VRect); OVERRIDE;
				{ iterates over fAdorners calling Draw on each adorner }
				PROCEDURE TAdornment.PostDraw(itsView: TView; area: VRect); 
				{ Draws shadow }
				
				PROCEDURE TAdornment.DoHighlightSelection(itsView: TView; area: VRect; 
															fromHL, 
															toHL: HLState); OVERRIDE;
				{ iterates over fAdorners calling DoPreHighlightSelection on each adorner }

				PROCEDURE TAdornment.ViewChangedSize ( 	itsView: TView;
														delta: VPoint;
														invalidate: BOOLEAN ); OVERRIDE;
				{ Iterates over the Adorners calling their ViewChangedSize method }
				
				END;

{--------------------------------------------------------------------------------------------------}

			FUNCTION NewStdAdorner(signature: IDType;
								   className: MAName;
								   itsID: IDType;
								   freeOnDeletion: BOOLEAN) : TAdorner;
			{ Creates and initializes an adorner }

{--------------------------------------------------------------------------------------------------}

			PROCEDURE InitUAdorners;
			{ creates system-supplied adorners }

{--------------------------------------------------------------------------------------------------}

		VAR
			{$Push} 									{$J+}
			{ Global, pre-allocated, shared adorners }
			
			gDimAdorner:			TDimAdorner;			{ calls the view's Dim method }
			gDrawAdorner:			TDrawAdorner;			{ calls the view's Draw method }
			gEraseAdorner:			TEraseAdorner;			{ erases the area passed in the Draw method }
			gHiliteAdorner:			THiliteAdorner;			{ calls the view's Hilite method }
			gPrintAdorner:			TPrintAdorner;			{ calls the view's DoDrawPrintFeedback method }
			gResizeIconAdorner:		TResizeIconAdorner;		{ calls the TWindow's DrawResizeIcon method }
			gSelectionAdorner:		TSelectionAdorner;		{ calls the view's DoHighlightSelection method }

			{ === Convenient RGB color settings === }
			gRGBVeryLtGray:				RGBColor;		{ Greyscale very light gray				}
			gRGBLtGray:					RGBColor;		{ Greyscale light gray					}
			gRGBGray:					RGBColor;		{ Greyscale regular gray				}
			gRGBDkGray:					RGBColor;		{ Greyscale dark gray					}
			{$Pop}
END.
