{[a-,body+,h-,o=100,r+,rec+,t=4,u+,#+,j=20/57/1$,n+]}
{Copyright  1988-1991 by Apple Computer Inc.  All rights reserved.}

UNIT UAppleEvents;

	INTERFACE

		USES
			{
			 List units defining any types or constants that are required for your interface
			section (e.g. MacAppTypes or QuickDraw)
			 Also list units defining the classes that you are subclassing from in the interface
			section.  DO NOT list units for classes that you are only referencing
			(e.g. instance vars, parameter types or function results).
			 Also list the units used by the above (in their interface section).
			}
			Types, MacAppTypes, Memory, UPascalObject, UObject, AppleEvents, Aliases, Editions;

		TYPE
			{
			 Declare as external the classes that you are only referencing
			(e.g. instance vars, parameter types or function results) that are not covered by
			the USES in the interface section.
			}
			TDynamicArray		= OBJECT; EXTERNAL;

			TAppleEvent 		= OBJECT (TObject)

				fMessage:			AppleEvent; 		{ Created by IAppleEvent, Set from
														 InitFromMessage }
				fSendingMode:		LongInt;			{ kAENoReply, kAEQueueReply, or kAEWaitReply
														 }
				fPriority:			Integer;			{ kAENormalPriority or kAEHighPriority }
				fTimeoutVal:		LongInt;			{ How long should we wait for a response? }
				fFreeMessage:		Boolean;			{ False if intialized from InitFromMessage }
														{ We let the AppleEvent manager take }
														{ care of it. }

				PROCEDURE TAppleEvent.IAppleEvent(itsEventClass: AEEventClass;
												  itsEventID: AEEventID; itsAddress: AEAddressDesc;
												  itsSendingMode: LongInt);
				{ Creates a new AppleEvent message and stores it in fMessage.  Supplies }
				{ defaults for return ID, transaction ID, and sending priority }

				PROCEDURE TAppleEvent.InitFromMessage(theMessage: AppleEvent);
				{ Stores theMessage in fMessage.  This may be a message or a reply. }

				PROCEDURE TAppleEvent.Free; OVERRIDE;

				PROCEDURE TAppleEvent.Send;
				{ Send fMessage by the indicated fSending mode and with fPriority. }

				{ GET and SET attributes }

				PROCEDURE TAppleEvent.GetAddress(theAddress: AEAddressDesc);
				{ Return the target address of the message }

				FUNCTION TAppleEvent.GetPriority: Integer;
				{ Return the sending priority of the message }

				FUNCTION TAppleEvent.GetReturnID: Integer;
				{ Return the ID assigned to the message }

				FUNCTION TAppleEvent.GetTimeoutVal: LongInt;
				{ Return the timeout value (in ticks) used when sending the message }

				FUNCTION TAppleEvent.GetTransactionID: LongInt;
				{ Return the ID of the transaction the message belongs to }

				PROCEDURE TAppleEvent.SetAddress(theAddress: AEAddressDesc);
				{ Assign the target address of the message }

				PROCEDURE TAppleEvent.SetPriority(thePriority: Integer);
				{ Assign the priority with which the message will be sent }

				PROCEDURE TAppleEvent.SetReturnID(returnID: Integer);
				{ Assign a new return ID to the message }

				PROCEDURE TAppleEvent.SetTimeoutVal(theTimeoutVal: LongInt);
				{ Assign the time (in ticks) to wait for reply if sending kAEWaitReply }

				PROCEDURE TAppleEvent.SetTransactionID(transactionID: LongInt);
				{ Assign a transaction ID to the message }

				{ READING }
				{ The following retrieve theData from fMessage based on the keyword }

				PROCEDURE TAppleEvent.ReadInteger(theKey: AEKeyword; VAR theData: UNIV Integer);
				PROCEDURE TAppleEvent.ReadLong(theKey: AEKeyword; VAR theData: UNIV LongInt);
				PROCEDURE TAppleEvent.ReadString(theKey: AEKeyword; VAR theData: Str255);
				PROCEDURE TAppleEvent.ReadSectionHandle(theKey: AEKeyword;
														VAR theData: SectionHandle);
				PROCEDURE TAppleEvent.ReadPtrList(theKey: AEKeyword; theType: DescType;
												  theData: TDynamicArray);
				PROCEDURE TAppleEvent.ReadHandleList(theKey: AEKeyword; theType: DescType;
													 theData: TDynamicArray);

				{ Allow raw read access to a parameter of the message }

				PROCEDURE TAppleEvent.ReadParameter(theKey: AEKeyword; desiredType: DescType;
													VAR theData: AEDesc);

				PROCEDURE TAppleEvent.ReadParameterPtr(theKey: AEKeyword; desiredType: DescType;
													   VAR actualType: DescType; theData: Ptr;
													   maximumSize: LongInt;
													   VAR actualSize: LongInt);

				{ WRITING }
				{ The following methods store theData in fMessage with theKey }
				PROCEDURE TAppleEvent.WriteInteger(theKey: AEKeyword; theData: UNIV Integer);
				PROCEDURE TAppleEvent.WriteLong(theKey: AEKeyword; theData: UNIV LongInt);
				PROCEDURE TAppleEvent.WriteString(theKey: AEKeyword; theData: Str255);
				PROCEDURE TAppleEvent.WriteSectionHandle(theKey: AEKeyword; theData: SectionHandle);
				PROCEDURE TAppleEvent.WritePtrList(theKey: AEKeyword; theType: DescType;
												   theData: TDynamicArray);

				PROCEDURE TAppleEvent.WriteHandleList(theKey: AEKeyword; theType: DescType;
													  theData: TDynamicArray);

				{ Allow raw access to a parameter of the message }

				PROCEDURE TAppleEvent.WriteParameter(theKey: AEKeyword; theData: AEDesc);

				PROCEDURE TAppleEvent.WriteParameterPtr(theKey: AEKeyword; typeCode: DescType;
														theData: Ptr; dataSize: LongInt);

				{ There will be more reading/writing methods as we support more types }

				PROCEDURE TAppleEvent.DeleteParameter(theKey: AEKeyword);
				{ Delete the specified parameter from the message }

				PROCEDURE TAppleEvent.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields}

				END;

		PROCEDURE InitUAppleEvents;
		{ Loads essential tables and starts up support for appleevents }

		FUNCTION AppleEventsDispatch(message, reply: AppleEvent;
									 info: LONGINT): OSErr;
		{ The general dispatch function which is passed AEInit. }
END.
