{[a-,body+,h-,o=100,r+,rec+,t=4,u+,#+,j=20/57/1$,n+]}
{ UApplication.p }
{ Copyright  1984-1991 by Apple Computer Inc.	All rights reserved. }

UNIT UApplication;

	INTERFACE

		USES
			{}
			Types, MacAppTypes, Memory, UPascalObject, UObject, Editions, AppleEvents, UEvtHandler, UEvent, UCommand, UList,
			Dialogs;

		TYPE

			TDocument			= OBJECT; EXTERNAL;
			TFile				= OBJECT; EXTERNAL;
			TView				= OBJECT; EXTERNAL;
			TWindow 			= OBJECT; EXTERNAL;
			TEvent				= OBJECT; EXTERNAL;
			TToolboxEvent		= OBJECT; EXTERNAL;

			TEventList		= OBJECT (TSortedList)	{ A list for keeping TEvents (including TCommands) ordered by
														 priority }
				PROCEDURE TEventList.IEventList;
				{ Initialize the EventList procedurally. }

				FUNCTION TEventList.Compare(item1, item2: TObject): INTEGER; OVERRIDE;
				{ Compares Two TEvents based on their priority }

				PROCEDURE TEventList.Insert(item: TObject); OVERRIDE;
				{ Overridden to guarantee insertion even in low memory and to preserve arrival
				sequence within "Compare" ordering. }

				PROCEDURE TEventList.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields. }

				END;


{--------------------------------------------------------------------------------------------------}

			TApplication		= OBJECT (TEvtHandler)	{ The purpose of TApplication is to
														 implement the main event loop that all
														 Macintosh applications must have. There is
														 only one instance of TApplication. One
														 thing that TApplication does is to
														 interpret the raw events that are posted
														 and convert them into higher-level events.
														 For example, it converts a click in the
														 menu bar into a 'menu event' that contains
														 the menu ID/item number that was chosen by
														 the user. }


				fEventList:		TEventList;				{ Events/Commands that were posted for execution }
				fLaunchWithNewDocument: BOOLEAN;		{ TRUE (the default) to create a new
														 untitled document when the application is
														 launched without a document. }
				fLastCommand:		TCommand;			{ the last command done or undone by the
														 user }
				fAllowApplicationToSleep: BOOLEAN;		{ Whether the application should be allowed
														 to "sleep" and thus share processor time,
														 when getting events }

				fApplicationPSN:	ProcessSerialNumber;{ This application's process serial number. }

				fIdlePhase: 		IdlePhase;			{ The current idle phase }

				fAlwaysTrackCursor: BOOLEAN;			{ set this to TRUE when you want the
														 application to track the cursor even if it
														 doesn't stray outside gCursorRgn. }
				fAppDone:			BOOLEAN;			{ set this to TRUE when you want the
														 application to terminate }

				fClickCount:		INTEGER;			{ number of 'saved up' mouse clicks; handled
														 in TApplication.DispatchEvent, ObeyEvent &
														 ObeyMouseDown. If a mouseDown, the value
														 is that returned by the call to
														 TApplication.CountClicks. If a mouseUp,
														 the value is left alone. Otherwise, set to
														 0. Also set to 0 is the 2 clicks were in
														 different parts ofthe window. }
				fLastClickPart: 	INTEGER;			{ the list window part clicked in-used for
														 double click detection }
				fLastMsePt: 		Point;				{ coordinates of mouse in last event passed
														 to TApplication.CountClicks }
				fLastUpTime:		LONGINT;			{ time of last mouse up event passed to
														 TApplication.ObeyEvent }

				fNextSpaceMsg:		LONGINT;			{ time when next low space message should be
														 displayed }
				fLowSpaceInterval:	LONGINT;			{ If >= 0, the frequency (in Ticks) with
														 which MacApp displays a low space alert.
														 (Defaults to kLowSpaceInterval.) If < 0,
														 MacApp doesn't display an alert. }
				fMainEventMask: 	INTEGER;			{ Event mask used in main event loop.
														 Initialized by InitUMacApp. }
				fSysWindowActive:	BOOLEAN;			{ TRUE if the front window is a system
														 window }
				fUndoState: 		BOOLEAN;			{ are we in undo or redo }
				fUndoCmd:			CmdNumber;			{}
				fDocList:			TList;				{ list of documents }
				fFreeWindowList:	TList;				{ list of free-standing (documentless)
														 windows }
				fHeadCohandler: 	TEvtHandler;		{ head of linked list of global co-handlers
														 }
				fInBackground:		BOOLEAN;			{ True if app is currently in background }
				fTarget:			TEvtHandler;		{ the TEvtHandler that gets the first chance
														 at DoCommand, DoSetupMenu, DoKeyCommand,
														 Idle; should never be NIL -- If you do not
														 want your own target set this to the
														 application object }
				fMainFileType:		OSType; 			{ principal file type opened/printed by
														 application; set in
														 TApplication.IApplication; by default,
														 TApplication.SFGetFilters returns a list
														 of just this }
				fEventLevel:		INTEGER;			{ A count of the number of nested calls to
														 PollToolboxEvent. }
				fMBarDisplayed: 	INTEGER;			{ menus that are read in and installed in
														 menu bar }
				fMBarHierarchical:	INTEGER;			{ menus that pop up when a menu item is
														 choosen }
				fMBarNotDisplayed:	INTEGER;			{ menus that are read in but not installed }
				fSleepRgn: 			RgnHandle;			{ the region that will be
														 passed to WaitNextEvent as the sleep
														 region. Computed from the cursor
														 and help regions }
				fCursorRgn: 		RgnHandle;			{ the current cursor region }
				fHelpRgn: 			RgnHandle;			{ the current help region}
												
														
				
									  { Init & Free }
				PROCEDURE TApplication.Initialize; OVERRIDE;
				{ Puts a newly created TApplication into a known state. Subclasses should
				override it and call INHERITED. Typically used to put the object into a state where
				it can be safely FREED. }

				PROCEDURE TApplication.IApplication(itsMainFileType: OSType);
				{ itsMainFileType should be the the 4-byte file type that this application
				reads/writes. We require this here so that programmers do not have to explicitly
				override TApplication.SFGetParms and supply the type there. That is the only place
				where the main file type is used. }

				FUNCTION TApplication.ChooseApplication(VAR theAddress: AEAddressDesc): Boolean;
				{ Choose a running application using the PPCBrowser and store the address in }
				{ an AppleEvent address }

				PROCEDURE TApplication.GetInspectorName(VAR inspectorName: Str255); OVERRIDE;
				{ Used to get this class's name by the Inspector. }

				PROCEDURE TApplication.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields, in this case the global variables. }

				{ Event Handling }
				FUNCTION TApplication.GetEvent(eventMask: INTEGER; sleep: LONGINT;
											   sleepRgn: RgnHandle): TToolboxEvent;
				{ Calls WaitNextEvent (or GetNextEvent) with the given event mask, sleep, and sleep
				region parameters. You can override this if you have an alternate source for
				events. }

				PROCEDURE TApplication.HandleAlienEvent(event: TEvent);
				{ This method is called upon to deal with 'application-specific' events }

				PROCEDURE TApplication.MainEventLoop;
				{ Loop on PollEvent until the application is done }

				PROCEDURE TApplication.DispatchEvent(event: TEvent);

				PROCEDURE TApplication.DoAppleCommand(aCmdNumber: CmdNumber; message: AppleEvent;
													  reply: AppleEvent); OVERRIDE;
				{ Field the Required AppleEvents }

				FUNCTION TApplication.DispatchAppleEvent(message, reply: AppleEvent; info: LONGINT): OSErr;
				{ Called by AppleEventsDispatch to "DoAppleCommand" }
				{ Default behavior is to call fTarget.DoAppleCommand }

				PROCEDURE TApplication.DoEvent(eventNumber: EvtNumber;
													source: TEvtHandler;
													event: TEvent); OVERRIDE;
				{ When this is called by MacApp, theEvent is the event to be processed. }

				PROCEDURE TApplication.PostDoEvent(event: TEvent);

				PROCEDURE TApplication.HandleActivateEvent(event: TToolboxEvent);
				{ Handles activate (and deactivate) events. Calls the window's Activate method, and
				invalidates the menubar if activating the window. }

				PROCEDURE TApplication.HandleDiskEvent(event: TToolboxEvent);
				{ Handles disk events. Calls DIBadMount if the high word of theEvent's message is <>
				noErr }

				PROCEDURE TApplication.HandleHighLevelEvent(event: TToolboxEvent);
				{ Process a high level event.  Default behavior is to assume it is an AppleEvent }

				PROCEDURE TApplication.KeyEventToComponents(event: TToolboxEvent); OVERRIDE;
				{ re-extracts components of an event in a script manager compatible way. }

				PROCEDURE TApplication.HandleKeyDownEvent(event: TToolboxEvent);
				{ Handles key down events. If the command key is down, then this calls CommandKey.
				Otherwise it calls fTarget.DoKeyCommand. }

				PROCEDURE TApplication.HandleMouseDown(event: TToolboxEvent);
				{ Called when a mouse down occurs; this cases on whereMouseDown and does the right
				thing. }

				FUNCTION TApplication.TrackMouse(theMouse: VPoint; hysteresis: Point;
												 theCommand: TTracker): TTracker;

				{ Responsible for mouse tracking. !!! Candidate for eventhandler method? }

				PROCEDURE TApplication.HandleMouseUp(event: TToolboxEvent);
				{ Sets fLastUpTime. Returns NIL. }

				PROCEDURE TApplication.HandleSystemEvent(event: TToolboxEvent);
				{ Handles "system" events, in particular suspend/resume events and mouse moved
				events. Calls RegainControl if switching in, AboutToLoseControl if switching out,
				or TrackCursor if it is a mouse-moved event.  }

				PROCEDURE TApplication.HandleUpdateEvent(event: TToolboxEvent);
				{ Handles window update events. Calls UpdateEvent for the window indicate by the
				event. }

				FUNCTION TApplication.IsDeskAccessory(aWMgrWindow: WindowPtr): BOOLEAN;
				{ Returns true if the given Window Manager window is a desk accessory. We assume that
				if the window has a negative windowKind then it is a desk accessory. If the process
				Manager is present (or required) always returns FALSE.  }

				FUNCTION TApplication.IsFrontProcess: BOOLEAN;
				{ Returns true if the application is currently the front process. }

				PROCEDURE TApplication.MakeFrontProcess;
				{ Makes this application the front process. }

				PROCEDURE TApplication.OpenDeskAccessory(deskAccName: Str255);
				{ Attempts to open the desk accessory whose name is deskAccName. Called from
				TApplication.MenuEvent when an Apple menu item is chosen, and the item has a
				negative command number. }

				PROCEDURE TApplication.PollEvent(allowApplicationToSleep: BOOLEAN);
				{ Polls for events/commands and processes them. }

				PROCEDURE TApplication.PollToolboxEvent(allowApplicationToSleep: BOOLEAN);
				{ Polls for events calling Idle before waitNextEvent if there are no events pending
				and the application is allowed to go to sleep (kAllowApplicationToSleep),
				calls HandleEvent with the event, if there is one. and then idles if necessary. }

				PROCEDURE TApplication.Run;
				{ call MainEventLoop }

				PROCEDURE TApplication.UpdateAllWindows;
				{ Process all update events in the queue. }

				FUNCTION TApplication.InModalState: BOOLEAN;
				{ Returns true if the front window is modal. }

				FUNCTION TApplication.InModalMenuState: BOOLEAN;
				{ Returns true if the front window does not allow menu commands. }

				{ Target and Cohandlers }
				PROCEDURE TApplication.InstallCohandler(aCohandler: TEvtHandler; addIt: BOOLEAN);
				{ Used to add (addIt = TRUE) or delete (addIt = FALSE) a cohandler to/from the global
				list of cohandlers }

				FUNCTION TApplication.GetFrontWindow: TWindow;
				{ Returns the window object reference for the frontmost application window (whether
				active or not. Returns nil if the FrontWindow window is not a window object }

				FUNCTION TApplication.GetActiveWindow: TWindow;
				{ Returns the window object reference for the active application window.  Returns nil
				if the FrontWindow window is not a window object }

				PROCEDURE TApplication.SetTarget(newTarget: TEvtHandler);
				{ Sets the current target (the head of the target chain) for the application. }

				FUNCTION  TApplication.GetTarget: TEvtHandler;
				{ Returns the current target }
				
				{ Finder Requests }
				FUNCTION TApplication.CanOpenDocument(itsCmdNumber: CmdNumber; aFile: TFile): BOOLEAN;
				{ Simulates the filtering done by Std File; this is only called when opening/printing
				documents from the finder; when using Std File, Std File does the filtering. }


{$IFC NOT qNeedsAppleEventMgr}
				PROCEDURE TApplication.HandleFinderRequest;
				{ If the AppleEvents Manager is not required or present:
				Gets the info from the finder about what files to open/print and opens/prints them.
				}
{$ENDC}

				{ Opening / Printing Documents }
				FUNCTION TApplication.AlreadyOpen(aFile: TFile):
												  TDocument;
				{ Given a name/volRefnum/dirID, if that document is already opened, returns the TDocument.
				Otherwise returns NIL. }

				FUNCTION TApplication.ChooseDocument(itsCmdNumber: CmdNumber;
													 VAR aFileList: TList):
													 BOOLEAN;
				{ Call this to make a Std File Get call; returns TRUE is user selected a file. }

				FUNCTION TApplication.DoMakeDocument(itsCmdNumber: CmdNumber;
														itsFile: TFile): TDocument;
				{ Must be overridden. Based on itsCmdNumber create a document object of the
				appropriate kind. If itsFile <> NIL it is attached to the document }

				FUNCTION TApplication.DoMakeFile(itsCmdNumber: CmdNumber): TFile;
				{ Default behavior is to return an instance of TFile.  May be overridden to create
				a file of your own special type. }

				FUNCTION TApplication.KindOfDocument(itsCmdNumber: CmdNumber;
													 itsFile: TFile):
													 CmdNumber;
				{ Given a cmd number and a specification of the file, return the cmd number to pass
				to DoMakeDocument that indicates the type of document to make; Default is to return
				itsCmdNumber. If you have multiple document types, a good convention is to return
				the cmd numbers assigned to create new documents of each kind. aFile
				will be NIL if we are creating a brand new document, rather than opening an existing
				document. }

				PROCEDURE TApplication.OpenNew(itsCmdNumber: CmdNumber);
				{ Called when the application is opened (itsCmdNumber = cFinderNew), or when NEW is
				chosen from menu (itsCmdNumber = cNew). If you do not want to create a new document
				when the user opens the application, override this and do nothing if itsCmdNumber =
				cFinderNew. }

				PROCEDURE TApplication.OpenOld(itsOpenCmd: CmdNumber;
											   aFile: TFile);
				{ Called when opening an existing document from finder (itsCmdNumber = cFinderOpen)
				or if OPEN is chosen from menu (itsCmdNumber = cOpen). }

				FUNCTION TApplication.PrintDocument(aFile: TFile): BOOLEAN;
				{ Called to print a document from the finder. Returns TRUE if the user did not cancel
				printing of the rest of the documents. }

				PROCEDURE TApplication.SFGetParms(itsCmdNumber: CmdNumber; VAR fileFilter: ProcPtr;
												  typeList: TypeListHandle; VAR dlgID: INTEGER;
												  VAR where: Point; VAR dlgHook,
												  modalFilter: ProcPtr; VAR activeList: Ptr;
												  VAR activateProc: ProcPtr;
												  yourDataPtr: UNIV Ptr);
				{ Called to return all the parameters that should be passed to SFPGetFile or
				CustomGetFile, and CanOpenDocument; when called, typeList is a valid handle to
				a 0-length block. typeList = list of just the main file type supported by the application. (If type
				list ends up with nothing in it, then all file types are supported.  Defaults to:
				dlgID = getDlgID or sfGetDialogID;
				where = (100, 100) or (-1, -1);
				fileFilter = NIL;
				dlgHook = NIL;
				modalFilter = NIL;
				activeList := NIL;
				activateProc := NIL;
				yourDataPtr := NIL; }

				{ Termination }
				PROCEDURE TApplication.Close;
				{ Called when the user chooses Quit from the menu, and tries to save all the open
				documents. If all succeed the application terminates. Signals Failure with error =
				noErr and message = msgCancelled if user cancels }

				{ Hierarchy }
				PROCEDURE TApplication.AddDocument(aNewDocument: TDocument);
				{ Add another document to my list of documents, and make it the current one }

				PROCEDURE TApplication.AddFreeWindow(aWindow: TWindow);
				{ Add the window to the free-window list }

				PROCEDURE TApplication.DeleteDocument(docToDelete: TDocument);
				{ Delete a document from my list of documents }

				PROCEDURE TApplication.DeleteFreeWindow(windowToDelete: TWindow);
				{ Delete a window from my list of free windows }

				PROCEDURE TApplication.EachActiveWindow(PROCEDURE DoToWindow(aWindow: TWindow));
				{ Iterates over active windows. i.e. floating windows and the active document window }
				
				PROCEDURE TApplication.EachFreeWindow(PROCEDURE
													  DoToWindow(aWindow: TWindow));
				{ Perform DoToWindow on each TWindow object in the list fFreeWindowList }

				PROCEDURE TApplication.ForAllDocumentsDo(PROCEDURE
														 DoToDoc(aDocument: TDocument));
				{ Perform the given procedure on all open documents currently owned by the
				application }

				PROCEDURE TApplication.ForAllWindowsDo(PROCEDURE
													   DoToWind(aWindow: TWindow));
				{ Perform the given procedure on all windows known to the application, both
				document and free windows. }

				{ Window Manager Windows }
				PROCEDURE TApplication.SelectWMgrWindow(aWMgrWindow: WindowPtr);
				{ "Selects" aWMgrWindow by calling the toolbox routine SelectWindow. When any window
				is selected, it goes through here. We never call SelectWindow directly.}

				FUNCTION TApplication.GetRsrcWindow(storage: Ptr; rsrcId: INTEGER; VAR isResizable,
													isClosable: BOOLEAN): WindowPtr;
				{ Get a Window Manger window resource whose resource id is rsrId. Return isResizable
				and isClosable based upon the 'WIND' resource. }

				FUNCTION TApplication.WMgrToWindow(aWMgrWindow: WindowPtr): TWindow;
				{ Returns the window object that represents the given Window Manager window, or NIL
				if there is no window object. }

				{ Idle Events }
				FUNCTION TApplication.GetWaitTicks(allowApplicationToSleep: BOOLEAN): LONGINT;
				{ Compute the "wait ticks" to be passed to GetEvent }

				PROCEDURE TApplication.Idle(phase: IdlePhase);
				{ Parcels out idle time to the DoIdle method in the target and cohandler chains of
				TEvtHandlers. If your application needs to do something at idle time then override
				DoIdle instead of this method. }

				{ Cursor and Help }
				PROCEDURE TApplication.ExcludeWindowRegions(deskTopRgn: RgnHandle; 
															FUNCTION TestWindow(aWindow: TWindow): BOOLEAN);
				{ Removes from the given region the visRgn of all windows for which TestWindow
				returns TRUE. Used by GetDefaultCursorRgn and GetDefaultHelpRgn }
															
				FUNCTION TApplication.GetSleepRgn: RgnHandle;
				{ Return the "sleep region" to be passed to GetEvent }

				PROCEDURE TApplication.GetDefaultCursorRgn(globalMouse: Point;cursorRgn: RgnHandle);
				{ Computes the cursor region for the case where no window has claimed the cursor }

				PROCEDURE TApplication.GetDefaultHelpRgn(globalMouse: Point;helpRgn: RgnHandle);
				{ Computes the help region for the case where no window has claimed the cursor }

				PROCEDURE TApplication.TrackCursor(globalMouse: Point);
				{ Allows windows/views to set the cursor and cursor region via TView.HandleCursor
				and TView.DoSetCursor. If no region is set, the cursor is set to an arrow and the
				largest possible default cursor region is calculated. }

				FUNCTION  TApplication.HandleCursor(globalMouse: Point; cursorRgn: RgnHandle): TView;
				{ !!! }

				PROCEDURE TApplication.DoSetCursor(globalMouse: Point;
													cursorRgn: RgnHandle );
				{ Sets the cursor to an arrow and computes the region for which it is claimed.
				Called when no view has set the cursor }
				
				PROCEDURE TApplication.TrackHelp(globalMouse: Point);
				{ !!! }
				FUNCTION  TApplication.HandleHelp(globalMouse: Point; helpRgn: RgnHandle): TView;
				{ !!! }

				PROCEDURE TApplication.DoShowHelp(globalMouse: Point;
													helpRgn: RgnHandle );
				{ !!! }
				
				PROCEDURE TApplication.InvalidateMouseRegions;
				{ Invalidates the cursor and help regions }

				PROCEDURE TApplication.InvalidateCursorRgn;
				{ Sets the cursor region to the empty region }

				PROCEDURE TApplication.InvalidateHelpRgn;
				{ Sets the help region to the empty region }
				
				FUNCTION TApplication.IsCursorRgnInvalid : BOOLEAN;
				{ Returns TRUE if the cursor region needs to be recalculated }

				FUNCTION TApplication.IsHelpRgnInvalid : BOOLEAN;
				{ Returns TRUE if the help region needs to be recalculated }

				{ Keyboard events }
				PROCEDURE TApplication.DoKeyCommand(event: TToolboxEvent); OVERRIDE;
				{ Handle keydown events. }

				{ Menu Events }
				PROCEDURE TApplication.DoCommandKey(event: TToolboxEvent); OVERRIDE;
				{ Called when a keyDown event is received and the command key is down. }

				PROCEDURE TApplication.MenuEvent(menuItem: LONGINT);
				{ Given a value returned by MenuKey or MenuSelect, figure out the command number that
				was chosen, have the application create a command object, and post it. }

				PROCEDURE TApplication.SetUndoText(cmdDone: BOOLEAN; aCmdNumber: CmdNumber);
				{ Called to setup the Undo menu item; if cmdDone is TRUE it reads Undo, otherwise it
				reads Redo. aCmdNumber indicates the command that goes after the Undo/Redo. }

				PROCEDURE TApplication.SetupTheMenus;
				{ Initiates the process of enabling & checking menu items. }

				{ Opening and Closing Windows }
				PROCEDURE TApplication.CloseWMgrWindow(aWMgrWindow: WindowPtr);
				{ Called when user closes a window either with a menu item or with a GoAway box.
				Signals Failure with error = noErr and message = msgCancelled if user cancels }

				{ Command Management }
				PROCEDURE TApplication.CommitLastCommand; OVERRIDE;
				{ Commits the last command by calling its Commit method. }

				FUNCTION TApplication.GetLastCommand: TCommand; OVERRIDE;
				{ Returns the last undoable command.  The command returned is not yet committed.}

				FUNCTION TApplication.RetrieveAnEvent: TEvent; OVERRIDE;
				{ Retrieves a previously posted event/command from the queue.	Nil if there are 
				no queued commands. }

				PROCEDURE TApplication.ProcessEvent(event: TEvent);
				{  Processes the given event from the queue of event objects. The event may be a 
				Toolbox event or a command object. 
				Calls TEvent methods for three phases of event processing: PreProcess, Process and
				PostProcess. Also accounts for commands which track the mouse }

				PROCEDURE TApplication.PerformCommand(command: TCommand); OVERRIDE;
				{ Performs the given command taking care of UNDO/REDO and clipboard related details.
				Tracks the command first, if necessary.
				This DOES NOT check to see if command is NIL. }

				PROCEDURE TApplication.PostAnEvent(event: TEvent); OVERRIDE;
				{ Called to post an event or command to a queue for later processing.
	 			 Default is to hand off to fNextHandler }

				PROCEDURE TApplication.PostCommand(command: TCommand); OVERRIDE;
				{ Called to post a command to the queue for later execution. }

				{ Double/Triple Clicks }
				FUNCTION TApplication.CountClicks(event: TToolboxEvent;
												  whereMouseDown: INTEGER): INTEGER;
				{ Called by TApplication.HandleMouseDown. Returns the number of clicks that can be
				considered multiple clicks (e.g. if it returns 1 the mouse down should be treated
				as a single click, or possibly the first click of a multi-click sequence. If it
				returns 2 the click should be considered a double-click, etc. A click is considered
				part of a multi-click sequence if the mouse down was within the proper time range
				of the previous mouse up, and was within the proper number of pixels of the last
				mouse down. TApplication.HandleMouseDown sets theClickCount of its EventInfo record
				to the result of this function. aPDownEvent is a pointer to the mouse down event.
				whereMouseDown is the result of FindWindow on the event. }

				FUNCTION TApplication.MakeViewForAlienClipboard: TView;
				{ Override this to create a view to handle your application's specific data types.
				Return NIL (the default) to have the clipboard manager create a default view. }

				{ Command Handlers }
				PROCEDURE TApplication.DoMenuCommand(aCmdNumber: CmdNumber); OVERRIDE;
				{ Handles the application specific menu commands. }

				PROCEDURE TApplication.DoSetupMenus; OVERRIDE;

				PROCEDURE TApplication.AboutToLoseControl(convertClipboard: BOOLEAN);
				{ Called when about to activate a Desk Accessory, switch out to another application,
				or Terminate. convertClipboard is generally true, except it will be FALSE if we get
				a System event that does not require Clipboard conversion }

				PROCEDURE TApplication.RegainControl(checkClipboard: BOOLEAN);
				{ Called when switching in or when leaving a desk accessory }
		
				{ Error Alerts }

				PROCEDURE TApplication.Beep(duration: INTEGER);
				{ Call this method when you want to beep at the user. Sound manager users may have to
				override this method to integrate its functionality in with your application }

				PROCEDURE TApplication.ShowError(error: OSErr; message: LONGINT);
				{ Calls ErrorAlert to display an error alert. }

				PROCEDURE TApplication.SpaceIsLow;
				{ Called when space is low, at intervals of kLowSpaceInterval ticks. Displays an
				alerting informing the user that memory space is low. Override this if you wish to
				take more appropriate action. }

				{ Miscellaneous }

				PROCEDURE TApplication.DoShowAboutApp;
				{ Called to show the "About the App" information }
				
				PROCEDURE TApplication.InvalidateFocus;
				{ Called to indicate that a view is unfocused when you don't know which view
				was focused. }
				
				FUNCTION TApplication.IsHelpEnabled: BOOLEAN;
				{ Returns TRUE if the Help Manager is present and the user has enabled help }

				PROCEDURE TApplication.ActivateBusyCursor(entering: BOOLEAN);
				{ Called when the busy cursor mechanism should be activated or deactivated. }

				FUNCTION TApplication.AlertFilter(theDialog: DialogPtr;
										   VAR theEvent: EventRecord;
										   VAR itemHit: INTEGER): BOOLEAN;

				PROCEDURE TApplication.DoMakeViewServer;
				{ instantiates the global view server }

				END;

{--------------------------------------------------------------------------------------------------}
			{ commands that are created in response to common operations and will forward to the
			appropriate action }

{$IFC qDebug}
			TDebugCommand		= OBJECT (TNoChangesCommand) { Causes the MacApp Debugger to be
															  entered }
				PROCEDURE TDebugCommand.IDebugCommand(itsCmdNumber: CmdNumber);
				{ Initialize the DebugCommand procedurally. }

				PROCEDURE TDebugCommand.DoIt; OVERRIDE;
				{ Cause the MacApp Debugger to be entered }

				PROCEDURE TDebugCommand.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields. }

				END;
{$ENDC}

{--------------------------------------------------------------------------------------------------}
{$IFC qInspector}
			TInspectorCommand	= OBJECT (TNoChangesCommand) { Tells the inspector to make a new
															  window }
				PROCEDURE TInspectorCommand.IInspectorCommand(itsCmdNumber: CmdNumber);
				{ Initialize the DebugCommand procedurally. }

				PROCEDURE TInspectorCommand.DoIt; OVERRIDE;
				{ Tell the inspector to make a new window }

				PROCEDURE TInspectorCommand.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields. }

				END;
{$ENDC}

{--------------------------------------------------------------------------------------------------}
			TNewDocCommand		= OBJECT (TNoChangesCommand) { Tells the application open a new
															  document. }
				PROCEDURE TNewDocCommand.INewDocCommand(itsCmdNumber: CmdNumber);
				{ Initialize the NewDocCommand procedurally. }

				PROCEDURE TNewDocCommand.DoIt; OVERRIDE;
				{ tell the application open a new document. }

				PROCEDURE TNewDocCommand.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields. }

				END;

{--------------------------------------------------------------------------------------------------}
			TFilesCommand		= OBJECT (TAppleCommand)	{ Retrives a list of files from an AppleEvent }
															{ Abstract class for TODocCommand and TPDocCommand. }

				fFileList: 		TList;

				PROCEDURE TFilesCommand.IFilesCommand(itsCmdNumber: CmdNumber;
													theDocuments: TList);
				{ Initialize the ODocCommand procedurally. }

				PROCEDURE TFilesCommand.InitFromAppleEvent(itsCmdNumber: CmdNumber;
														  itsDocument: TDocument;
														  itsView: TView;
														  itsMessage: AppleEvent;
														  itsReply: AppleEvent); OVERRIDE;
				{ Initialize the ODocCommand from the information contained in the 'odoc' AppleEvent. }

				PROCEDURE TFilesCommand.Initialize; OVERRIDE;
				{ Default initialization of some command fields. }

				PROCEDURE TFilesCommand.Free; OVERRIDE;
				{ Free the list of file alias handles. }

				PROCEDURE TFilesCommand.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields}

				END;

{--------------------------------------------------------------------------------------------------}
			TODocCommand		= OBJECT (TFilesCommand)	{ Accepts a list of file aliases and opens each of them }

				PROCEDURE TODocCommand.IODocCommand(itsCmdNumber: CmdNumber;
													theDocuments: TList);
				{ Initialize the ODocCommand procedurally. }

				PROCEDURE TODocCommand.DoIt; OVERRIDE;
				{ Open each file found in fFileList. }

				PROCEDURE TODocCommand.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields}

				END;
{--------------------------------------------------------------------------------------------------}
			TPDocCommand		= OBJECT (TFilesCommand)	{ Accepts a list of file aliases and prints each of them }

				PROCEDURE TPDocCommand.IPDocCommand(itsCmdNumber: CmdNumber;
													theDocuments: TList);
				{ Initialize the ODocCommand procedurally. }

				PROCEDURE TPDocCommand.DoIt; OVERRIDE;
				{ Open each file found in fFileList. }

				PROCEDURE TPDocCommand.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields}

				END;
{--------------------------------------------------------------------------------------------------}
			TQuitCommand		= OBJECT (TNoChangesCommand) { Tells the application to quit. }
				PROCEDURE TQuitCommand.IQuitCommand(itsCmdNumber: CmdNumber);
				{ Initialize the QuitCommand procedurally. }

				PROCEDURE TQuitCommand.DoIt; OVERRIDE;
				{ tell the application to quit. }

				PROCEDURE TQuitCommand.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields. }

				END;

{--------------------------------------------------------------------------------------------------}
			TUndoRedoCommand	= OBJECT (TNoChangesCommand) { Tells the application to undo/redo
															  the last undoable command }
				PROCEDURE TUndoRedoCommand.IUndoRedoCommand(itsCmdNumber: CmdNumber);
				{ Initialize the UndoRedoCommand procedurally. }

				PROCEDURE TUndoRedoCommand.DoIt; OVERRIDE;
				{ tell the application to undo/redo. }
				PROCEDURE TUndoRedoCommand.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields. }

				END;

{--------------------------------------------------------------------------------------------------}
			TAboutAppCommand	= OBJECT (TNoChangesCommand) { Tells the application to show its
															  about box. }
				PROCEDURE TAboutAppCommand.IAboutAppCommand(itsCmdNumber: CmdNumber);
				{ Initialize the AboutAppCommand procedurally. }

				PROCEDURE TAboutAppCommand.DoIt; OVERRIDE;
				{ tell the application to DoShowAboutApp. }

				PROCEDURE TAboutAppCommand.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields. }

				END;

{--------------------------------------------------------------------------------------------------}

			TEventCommand		= OBJECT (TNoChangesCommand) { Retrieve Events and process them. }

				PROCEDURE TEventCommand.IEventCommand(itsCmdNumber: CmdNumber);
				{ Initialize the EventCommand procedurally. }

				PROCEDURE TEventCommand.DoIt; OVERRIDE;
				{ Retrieve and process an event }

				PROCEDURE TEventCommand.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields. }

				END;
		VAR
			{$Push} 									{$J+}
			gApplication:		TApplication;			{ the application object }
			gInitialized:		BOOLEAN;				{ Set to TRUE at the end of IApplication }
			gServerAddress: 	AEAddressDesc;			{ Address of the AppleEvent Server. }
			{ Initialized to send to ourselves. }
			{$Pop}
END.
