{[a-,body+,h-,o=100,r+,rec+,t=4,u+,#+,j=20/57/1$,n+]}
{ UCommand.p }
{ Copyright  1984-1991 by Apple Computer Inc.	All rights reserved. }

UNIT UCommand;

	INTERFACE

		USES
			Types, MacAppTypes,
			{}
			Memory, UPascalObject, UObject,
			{}
			AppleEvents,
			UEvent;


TYPE
			TDocument			= OBJECT; EXTERNAL;
			TView				= OBJECT; EXTERNAL;
			TScroller			= OBJECT; EXTERNAL;
			TAppleEvent			= OBJECT; EXTERNAL;

			CommandPriority 	= EventPriority; 		{ Command priorities }

			TCommand			= OBJECT (TEvent)		{ Handles a user command, including undoing
														 and mouse-tracking if required. Usually
														 created by the part of the Application
														 with the greatest specific knowledge about
														 the action to take in response to events
														 and passed back to be performed at several
														 well defined places.}
				fView:				TView;				{ The view affected by this command, and in
														 which tracking takes place, if any. }
				fChangedObject:		TObject;			{ object to notify of changes }
				fCmdDone:			BOOLEAN;			{ TRUE if the last "execution" message sent
														 to the command was DoIt or RedoIt }
				fCanUndo:			BOOLEAN;			{ Defaults to TRUE }
				fCausesChange:		BOOLEAN;			{ Defaults to TRUE; Marks document changed
														 when command is done }
				fChangesClipboard:	BOOLEAN;			{ Defaults to FALSE. Set to true for command
														 subclasses representing CUT and/ or COPY
														 commands which change the Clipboard. }
				fFreeOnCompletion:	BOOLEAN;			{ TRUE to cause the command to be freed on
														 completion (the default). Completion is
														 after DoIt for non-undoable commands and
														 Commit for undoable commands. }
				fReadyToExecute:	BOOLEAN;			{ TRUE (the default) will enable this
														 command to be returned from any queues
														 that may be holding it. Most programs
														 won't have to mess with this. }
				fRecurring:			BOOLEAN;			{ FALSE (the default) will enable this
														 command to be removed from any command
														 queue that holds it because it is only
														 executed once. }

								{ Init & Free }
				PROCEDURE TCommand.Initialize;OVERRIDE;
				{ Puts a newly created command into a known state. Subclasses should
				override it and call INHERITED. Typically used to put the object into a state where
				it can be safely FREED. }

				PROCEDURE TCommand.ICommand(itsCmdNumber: CmdNumber;
											notify: TObject;
											itsView: TView);
				{ Initialize a command procedurally. }

				{ Inspecting }
				PROCEDURE TCommand.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields. }

				{ command processing }
				
				PROCEDURE TCommand.Process; OVERRIDE;
				
				{ command processing - usually overridden }

				PROCEDURE TCommand.Commit;
				{ Stub }

				PROCEDURE TCommand.DoIt; 
				{ Stub }

				PROCEDURE TCommand.RedoIt;
				{ Stub }

				PROCEDURE TCommand.UndoIt;
				{ Stub }

				{ Scheduling }
				FUNCTION TCommand.IsReadyToExecute: BOOLEAN; OVERRIDE;
				{ Returns TRUE (the Default) when the command is ready to execute.	Override to base
				the execution of the command on more sophisticated criteria.
				Most programs won't have to mess with this.. }

				FUNCTION TCommand.ShouldFreeOnCompletion: BOOLEAN; OVERRIDE;
				{ Overridden to return the value stored in fFreeOnCompletion }
				
				FUNCTION TCommand.IsRecurring: BOOLEAN; OVERRIDE;
				{ Overridden to return the value stored in fRecurring }
				
				{ Change notification }
				FUNCTION TCommand.GetChangeID : ChangeID;
				{ Returns a ChangeID for TApplication.PerformCommand to pass to Changed.
				Returns fID as a default. }
				END;

{--------------------------------------------------------------------------------------------------}
			TNoChangesCommand	= OBJECT (TCommand) 	{ Doesn't "change" the document and is not
														 itself undoable. Thus it doesn't flush the
														 last command that did. Useful for
														 deferring actions out to the
														 PerformCommand bottleneck where the stack
														 is unwound and the heap is unloaded for
														 maximum elbow room. }
				PROCEDURE TNoChangesCommand.INoChangesCommand(itsCmdNumber: CmdNumber;
															  notify: TObject;
															  itsView: TView);

				PROCEDURE TNoChangesCommand.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields. }

				END;


{--------------------------------------------------------------------------------------------------}
			TTracker			= OBJECT (TCommand)	{ Handles a user command, including undoing
														 and mouse-tracking if required. Usually
														 created by the part of the Application
														 with the greatest specific knowledge about
														 the action to take in response to events
														 and passed back to be performed at several
														 well defined places.}
				fTracksMouse:		BOOLEAN;			{ TRUE if the command should be tracked. }
				fInitialPt: 		VPoint;				{ Where to track from (usually where the
														 mouse went down).  In coordinates of the
														 fView (that's being tracked) else if the
														 fView is nil in global coordinates. }

				fConstrainsMouse:	BOOLEAN;			{ Defaults to FALSE; if you set to TRUE then
														 TrackConstrain will be called to constrain
														 the mouse. }
				fViewConstrain: 	BOOLEAN;			{ Defaults to TRUE, which means constrain
														 mouse to view limits }
				fTrackNonMovement:	BOOLEAN;			{ Defaults to FALSE, which means don't track
														 mouse unless it moves. }
				fScroller:			TScroller;			{ The scroller that handles auto-scrolling }

								{ Init & Free }
				PROCEDURE TTracker.Initialize; OVERRIDE;
				{ Puts a newly created TTracker into a known state. Subclasses should
				override it and call INHERITED. Typically used to put the object into a state where
				it can be safely FREED. }

				PROCEDURE TTracker.ITracker(itsCmdNumber: CmdNumber;
											notify: TObject;
											itsView: TView;
											itsScroller: TScroller;
											itsMouse: VPoint);
				{ Initialize a command procedurally. }

				{ Inspecting }
				PROCEDURE TTracker.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields. }
				
				{ Command Processing }
				
				PROCEDURE TTracker.Process; OVERRIDE;
				{ Overridden to call TApplication::TrackMouse. }

				{ Mouse Tracking }

				FUNCTION TTracker.IsDoneTracking: BOOLEAN;
				{ Indicates whether the Command is through tracking.  (the command is also through
				tracking if you return NIL from TrackMouse).  Defaults to fView.IsDoneTracking.
				Most programs won't have to mess with this.
				NOTE you still have to deal with queued events if you change the criteria for TRUE. }

				PROCEDURE TTracker.TrackConstrain(aTrackPhase: TrackPhase;
											  anchorPoint: VPoint;
											  previousPoint: VPoint;
											  VAR nextPoint: VPoint;
											  mouseDidMove: BOOLEAN);
				{ Override this if you want to constrain the mouse point to a grid, force drawing a
				square, etc. This is called only if fConstrainsMouse is TRUE. }

				PROCEDURE TTracker.TrackFeedback(aTrackPhase: TrackPhase;
											  anchorPoint: VPoint;
											  previousPoint: VPoint;
											  nextPoint: VPoint;
											  mouseDidMove: BOOLEAN;
											  turnItOn: BOOLEAN);
				{ Default is: IF NOT mouseDidMove THEN FrameRect(<rect formed by anchorPoint and
				nextPoint>). Before this is called the pen is set to PenNormal and the PenMode to
				PatXOR. }

				FUNCTION TTracker.TrackMouse(aTrackPhase: TrackPhase;
											  VAR anchorPoint: VPoint;
											  VAR previousPoint: VPoint;
											  VAR nextPoint: VPoint;
											  mouseDidMove: BOOLEAN): TTracker;
				{ This does the mouse tracking. The default returns SELF. Override it to force
				gridding, to return a different type of command, or to perform the command on mouse
				release instead of using the usual DoIt approach. All points are in local
				coordinates. If aTrackPhase = trackPress then all 3 points will be the same. If
				aTrackPhase = trackRelease then nextPoint will be the coordinate in the mouseUp
				event (if a mouseUp event is found), otherwise the same point as previousPoint.
				Generally, you will ignore nextPoint and just look at previousPoint. mouseDidMove
				will be TRUE if aTrackPhase = trackPress or trackRelease; otherwise, it will
				indicate if nextPoint = previousPoint. If you change anchorPoint, the new value
				will be passed to you next time. The value of nextPoint at the time the routine
				exits will be passed to you as previousPoint the next time the routine is called.
				You can change nextPoint if you wish. Usually, however, you will do gridding in the
				TrackConstrain method. }

				PROCEDURE TTracker.AutoScroll(delta: VPoint);
				{ Implements autoscrolling by calling fScroller.ScrollBy. }

				END;

{--------------------------------------------------------------------------------------------------}
			TNoChangesTracker	= OBJECT (TTracker) 	{ Doesn't "change" the document and is not
														 itself undoable. Thus it doesn't flush the
														 last command that did. Useful for
														 deferring actions out to the
														 PerformCommand bottleneck where the stack
														 is unwound and the heap is unloaded for
														 maximum elbow room. }
				PROCEDURE TNoChangesTracker.INoChangesTracker(itsCmdNumber: CmdNumber;
															  notify: TObject;
															  itsView: TView;
															  itsScroller: TScroller;
															  itsMouse: VPoint);

				PROCEDURE TNoChangesTracker.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields. }

				END;


{--------------------------------------------------------------------------------------------------}
			{ TAppleCommand can be used to both service an AppleEvent or send an AppleEvent. }
			{ It should also be used for all commands that can be invoked by an AppleEvent }
			{ _OR_ some user event (DoMenuCommand).  A good example is the 'odoc' command. }
			{ The 'odoc' command's DoIt method simply opens a list of files.  TAppleCommand }
			{ lets that list come from an AppleEvent or from some user action. }
			
			TAppleCommand		= OBJECT (TCommand)

				fMessage:			TAppleEvent;
				fReply: 			TAppleEvent;

				PROCEDURE TAppleCommand.Initialize; OVERRIDE;
				{ Puts a newly created TAppleCommand into a known state. Subclasses should
				override it and call INHERITED. Typically used to put the object into a state where
				it can be safely FREED. }

				PROCEDURE TAppleCommand.IAppleCommand(itsCmdNumber: CmdNumber;
													  itsDocument: TDocument;
													  itsView: TView);

				PROCEDURE TAppleCommand.InitFromAppleEvent(itsCmdNumber: CmdNumber;
														   itsDocument: TDocument;
														   itsView: TView;
														   itsMessage: AppleEvent;
														   itsReply: AppleEvent);


				PROCEDURE TAppleCommand.Free; OVERRIDE;
				
				procedure TAppleCommand.ReportError(error: OSErr; message: LONGINT);
				{ Report any errors in the default reply. }

				PROCEDURE TAppleCommand.SendIt;

				PROCEDURE TAppleCommand.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields}

				END;
END.
