{[a-,body+,h-,o=100,r+,rec+,t=4,u+,#+,j=20/57/1$,n+]}
{ UControl.p }
{ Copyright  1984-1991 by Apple Computer Inc.	All rights reserved. }

UNIT UControl;

	INTERFACE

		USES
			{}
			Types, MacAppTypes, Memory, UPascalObject, UObject, AppleEvents, UEvtHandler, Balloons, UView, UEvent, UCommand
			, Controls, TextEdit;

		TYPE
			TScroller			= OBJECT; EXTERNAL;
			TDocument			= OBJECT; EXTERNAL;
			TToolboxEvent		= OBJECT; EXTERNAL;
			TAdornment			= OBJECT; EXTERNAL;

			ControlTemplate 	= PACKED RECORD
				itsAdornment:		CntlAdornment;
				filler1:			Byte;
				itsPenSize: 		Point;
				isSizable:			BOOLEAN;
				isDimmed:			BOOLEAN;
				isHilited:			BOOLEAN;
				canDismiss: 		BOOLEAN;
				filler2:			0..4095;
				itsInset:			Rect;
				itsTextFace:		Style;
				itsTextSize:		INTEGER;
				itsTextColor:		RGBColor;
				itsFontName:		Str255; 			{Actually a variable length P-String}
				END;
			ControlTemplatePtr	= ^ControlTemplate;

			TControl			= OBJECT (TView)		{ An abstract class that represents a
														 control. }
				fDefChoice: 		INTEGER;			{ the choice to automatically pass up the
														 line via DoChoice }
				fHilite:			BOOLEAN;			{ is this view hilited }
				fDimmed:			BOOLEAN;			{ is this view dimmed }
				fSizeable:			BOOLEAN;			{ is the control portion of this view
														 sizeable }
				fDismissesDialog:	BOOLEAN;			{ can this view dismiss it's dialogview }
				fAdornment: 		CntlAdornment;		{ the adornment to be drawn in addition to
														 the view }
				fPenSize:			Point;				{ pen size to be used in adorning }
				fInset: 			VRect;				{ amount to inset the control portion of the
														 view }
				fTextStyle: 		TextStyle;			{ the TextStyle to use when rendering this
														 view }

				PROCEDURE TControl.Initialize; OVERRIDE;
				{ Puts a newly created TControl into a known state. Subclasses should
				override it and call INHERITED. Typically used to put the object into a state where
				it can be safely FREED. }

				PROCEDURE TControl.IControl(itsSuperView: TView;
											itsLocation, itsSize: VPoint;
											itsHSizeDet, itsVSizeDet: SizeDeterminer);
				{ Initialize a control procedurally. }

				PROCEDURE TControl.IRes(itsDocument: TDocument;
										itsSuperView: TView;
										VAR itsParams: Ptr); OVERRIDE;
				{ Initialize a control from a resource template. }

				FUNCTION TControl.Clone: TObject; OVERRIDE;
				{ calls INHERITED Clone and then clones owned objects }
	
				PROCEDURE TControl.WRes(theResource: ViewRsrcHandle;
										VAR itsParams: Ptr); OVERRIDE;
				{ For writing a control view out to a resource. }

				PROCEDURE TControl.WriteRes(theResource: ViewRsrcHandle;
											VAR itsParams: Ptr); OVERRIDE;
				{ For writing a control view to a resource, works in conjunction with tbe previous
				method. }

				PROCEDURE TControl.ComputeSize(VAR newSize: VPoint); OVERRIDE;
				{ Called to compute the control's size. }

				FUNCTION TControl.ContainsMouse(theMouse: VPoint): BOOLEAN; OVERRIDE;
				{ Returns TRUE if 'theMouse is currently in the controls area. }

				PROCEDURE TControl.ControlArea(VAR theArea: VRect);
				{ Returns in 'theArea' the controls active area. }

				PROCEDURE TControl.Dim; OVERRIDE;
				{ dims the current image in the TControl }

				PROCEDURE TControl.DimState(state, redraw: BOOLEAN);
				{ If we are not currently dimmed then set fDimmed to 'state.and if 'redraw' is TRUE
				then call DrawContents. }

				PROCEDURE TControl.DoMouseCommand(VAR theMouse: VPoint;
												 event: TToolboxEvent;
												 hysteresis: Point); OVERRIDE;
				{ Creates a control tracker command, which is posted. }

				FUNCTION TControl.Focus: BOOLEAN; OVERRIDE;
				{ Calls inherited Focus and if its TRUE then sets the port's text style and returns
				TRUE, if it can't be focused returns FALSE. }

				PROCEDURE TControl.Flash;
				{ Causes a control to flash with an 8 tick delay. }

				PROCEDURE TControl.Hilite; OVERRIDE;
				{ hilites the current image in the TControl }

				PROCEDURE TControl.HiliteState(state, redraw: BOOLEAN);
				{ If our fHilite is not 'state' the set it to 'state' before calling Hilite. Note:
				hilite is only called if 'redraw is TRUE and we can Focus. }

				PROCEDURE TControl.Inset(dh, dv: INTEGER;
										 redraw: BOOLEAN);
				{ If we are sizable then offset fInset to 'dh' & 'dv' redrawing if 'redraw' is TRUE.
				}

				FUNCTION TControl.IsDimmed: BOOLEAN;
				{ Returns TRUE if the control is currently dimmed. }

				PROCEDURE TControl.GetAdornExtent(VAR itsAdornExtent: VRect); OVERRIDE;
				{ returns the adorn extent for adorning this view }

				FUNCTION TControl.MakeAdornment:TAdornment; OVERRIDE;
				{ creates and returns a TAdornment }

				PROCEDURE TControl.SetAdornment(newAdornment: CntlAdornment;
												redraw: BOOLEAN);
				{ Set fAdornment to 'newAdornment' redrawing if 'redraw' is TRUE. }

				PROCEDURE TControl.SetInset(newInset: VRect;
											redraw: BOOLEAN);
				{ Set fInset to 'newInset' redrawing if 'redraw' is TRUE. }

				PROCEDURE TControl.InstallColor(theColor: RGBColor;
												redraw: BOOLEAN);
				{ Sets the color field of our text style record to 'theColor', if 'redraw' then call
				DrawContents. }

				PROCEDURE TControl.InstallTextStyle(theTextStyle: TextStyle;
													redraw: BOOLEAN);
				{ Sets fTextStyle to 'theTextStyle', if 'redraw' then call DrawContents. }

				PROCEDURE TControl.Resize(newSize: VPoint;
										  invalidate: BOOLEAN); OVERRIDE;
				{ Calls inherited Resize before calling ForceRedraw. }

				PROCEDURE TControl.TrackFeedback(aTrackPhase: TrackPhase;
											  anchorPoint: VPoint;
											  previousPoint: VPoint;
											  nextPoint: VPoint;
											  mouseDidMove: BOOLEAN;
											  turnItOn: BOOLEAN); OVERRIDE;
				{ Doesn't use the default feedback }

				PROCEDURE TControl.TrackMouse(aTrackPhase: TrackPhase;
											  VAR anchorPoint: VPoint;
											  VAR previousPoint: VPoint;
											  VAR nextPoint: VPoint;
											  mouseDidMove: BOOLEAN); OVERRIDE;
				{ Called back from tracking command. Handles tracking of the mouse and then calls
				DoChoice on TrackRelease. }

				FUNCTION TControl.Validate: LONGINT;
				{ By default returns 'noErr'. }

				PROCEDURE TControl.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields. }

				END;

{--------------------------------------------------------------------------------------------------}
			TCtlMgr 			= OBJECT (TControl) 	{ An abstract class that represents a
														 toolbox Control Manager control. }
				fCMgrControl:		ControlHandle;		{ the Control Manager Control that is being
														 managed }
				fBitsToShift:		INTEGER;			{ # bits shifted to convert between short
														 and long value. }
				fLongVal:			VCoordinate;		{ the current value }
				fLongMin:			VCoordinate;		{ the minimum setting }
				fLongMax:			VCoordinate;		{ the maximum setting }

				PROCEDURE TCtlMgr.Initialize; OVERRIDE;
				{ Puts a newly created TCtlMgr into a known state. Subclasses should
				override it and call INHERITED. Typically used to put the object into a state where
				it can be safely FREED. }

				PROCEDURE TCtlMgr.ICtlMgr(itsSuperView: TView;
										  itsLocation, itsSize: VPoint;
										  itsHSizeDet, itsVSizeDet: SizeDeterminer;
										  itsTitle: Str255;
										  itsVal, itsMin, itsMax: LONGINT;
										  itsProcID: INTEGER);
				{ Initialize procedurally. }

				PROCEDURE TCtlMgr.IRes(itsDocument: TDocument;
									   itsSuperView: TView;
									   VAR itsParams: Ptr); OVERRIDE;
				{ Initialize from a resource template. }

				FUNCTION TCtlMgr.Clone: TObject; OVERRIDE;
				{ calls INHERITED Clone and then clones owned objects }
	
				PROCEDURE TCtlMgr.WriteRes(theResource: ViewRsrcHandle;
										   VAR itsParams: Ptr); OVERRIDE;
				{ For writing a CtlMgr view to a resource. }

				PROCEDURE TCtlMgr.Free; OVERRIDE;
				{ Set the size of control to zero call DisposeControl, then  call inherited Free. }

				PROCEDURE TCtlMgr.BeInPort(itsPort: GrafPtr); OVERRIDE;
				{ Sets the control to be in 'itsPort', by setting the contrlOwner field of the
				ControlHandle. }

				PROCEDURE TCtlMgr.CreateCMgrControl(itsBounds: VRect;
													itsTitle: Str255;
													itsVal, itsMin, itsMax: LONGINT;
													itsProcID: INTEGER);
				{ Used to create a new control, calls NewControl to create the control. }

				PROCEDURE TCtlMgr.DimState(state, redraw: BOOLEAN); OVERRIDE;
				{ Set the dim state to 'state' and redrawing it if 'redraw'. }

				PROCEDURE TCtlMgr.DoMouseCommand(VAR theMouse: VPoint;
												event: TToolboxEvent;
												hysteresis: Point); OVERRIDE;
				{ Handles mouse downs in the control by calling TestControl and DoChoice }

				PROCEDURE TCtlMgr.Draw(area: VRect); OVERRIDE;
				{ Draws the control by calling the toolbox routine ShowControl. }

				{ Methods to manipulate the TCtlMgr values }
				FUNCTION TCtlMgr.GetLongMax: VCoordinate;
				{ Returns the current fLongMax value. }

				FUNCTION TCtlMgr.GetLongMin: VCoordinate;
				{ Returns the current fLongMin value. }

				FUNCTION TCtlMgr.GetLongVal: VCoordinate;
				{ Returns the current fLongVal value. }

				PROCEDURE TCtlMgr.SetLongMax(itsMax: VCoordinate;
												redraw: BOOLEAN);
				{ Sets fLongMax to 'itsMax', redrawing if required. }

				PROCEDURE TCtlMgr.SetLongMin(itsMin: VCoordinate;
												redraw: BOOLEAN);
				{ Sets flongMin to 'itsMin', redrawing if required. }

				PROCEDURE TCtlMgr.SetLongVal(itsVal: VCoordinate;
												redraw: BOOLEAN);
				{ Sets fLongVal to 'itsVal' redrawing if required. }

				PROCEDURE TCtlMgr.SetLongValues(itsVal, itsMin, itsMax: VCoordinate;
												   redraw: BOOLEAN);
				{ Sets all of the values at once, redrawing if required. }

				{ Methods to manipulate the Control Manager control values }
				FUNCTION TCtlMgr.GetMax: INTEGER;
				{ Returns the maximum value of the Control Manager control. }

				FUNCTION TCtlMgr.GetMin: INTEGER;
				{ Returns the minimum value of the Control Manager control. }

				PROCEDURE TCtlMgr.GetText(VAR theText: Str255);
				{ Returns the controls title, if any. }

				FUNCTION TCtlMgr.GetVal: INTEGER;
				{ Returns the current value of the Control Manager control. }

				FUNCTION TCtlMgr.GetVariant: INTEGER;
				{ Returns the variant control value for the Control Manager control. }

				PROCEDURE TCtlMgr.SetMax(itsMax: INTEGER;
										 redraw: BOOLEAN);
				{ While focused sets the maximum value of the Control Manager control to 'itsMax'. }

				PROCEDURE TCtlMgr.SetMin(itsMin: INTEGER;
										 redraw: BOOLEAN);
				{ While focused sets the minimum value of the Control Manager control to 'itsMin'. }

				PROCEDURE TCtlMgr.SetText(itsText: Str255;
										  redraw: BOOLEAN);
				{ Sets the controls title to 'itsText', redrawing if required. }

				PROCEDURE TCtlMgr.SetVal(newVal: INTEGER;
										 redraw: BOOLEAN);
	   { While focused sets the Control Manager controls value to 'newVal', redrawing if required. }

				PROCEDURE TCtlMgr.SetValues(itsVal, itsMin, itsMax: INTEGER;
											redraw: BOOLEAN);
{ While focused sets all the values of the Control Manager control, at once, redrawing if required. }

				PROCEDURE TCtlMgr.HiliteState(state, redraw: BOOLEAN); OVERRIDE;
				{ Set the hilite state to 'state', redrawing if required.If hilite state is currently
				'state' then does nothing. }

				FUNCTION TCtlMgr.IsCMgrVisible: BOOLEAN;
				{ Returns TRUE if the control is currently visible, FALSE if not. }

				PROCEDURE TCtlMgr.Resize(newSize: VPoint;
										 invalidate: BOOLEAN); OVERRIDE;
				{ If the control is resizable calls the toolbox routines MoveControl & SizeControl
				before calling inherited Resize. }

				PROCEDURE TCtlMgr.SetCMgrVisibility(beVisible: BOOLEAN);
				{ Sets the visibility of the control to 'isVisible'. }

				PROCEDURE TCtlMgr.WhileFocused(PROCEDURE DoToControl;
											   redraw: BOOLEAN);
				{ While focused call "DoToControl", redrawing if required. }

				PROCEDURE TCtlMgr.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields. }
				END;

{--------------------------------------------------------------------------------------------------}
			ScrollBarTemplate	= RECORD
				value:				LONGINT;
				minimum:			LONGINT;
				maximum:			LONGINT;
				END;
			ScrollBarTemplatePtr = ^ScrollBarTemplate;

			TScrollBar			= OBJECT (TCtlMgr)		{ A subclass of TCtlMgr that maps between
														 longint values for our view coordinates
														 and the integers of a toolbox scrollbar. }
				fDirection: 		VHSelect;			{ is this a horizontal or vertical scrollbar
														 }

				PROCEDURE TScrollBar.Initialize; OVERRIDE;
				{ Puts a newly created TScrollBar into a known state. Subclasses should
				override it and call INHERITED. Typically used to put the object into a state where
				it can be safely FREED. }

				PROCEDURE TScrollBar.IScrollBar(itsSuperView: TView;
												itsLocation, itsSize: VPoint;
												itsHSizeDet, itsVSizeDet: SizeDeterminer;
												itsDirection: VHSelect;
												itsVal, itsMin, itsMax: LONGINT);
				{ Initialize a scrollbar procedurally. }

				PROCEDURE TScrollBar.IRes(itsDocument: TDocument;
										  itsSuperView: TView;
										  VAR itsParams: Ptr); OVERRIDE;
				{ Initialize a scrollbar from a resource template. }

				FUNCTION TScrollBar.Clone: TObject; OVERRIDE;
				{ calls INHERITED Clone and then clones owned objects }
	
				PROCEDURE TScrollBar.WRes(theResource: ViewRsrcHandle;
										  VAR itsParams: Ptr); OVERRIDE;
				{ For writing a dialog view out to a resource. }

				PROCEDURE TScrollBar.WriteRes(theResource: ViewRsrcHandle;
											  VAR itsParams: Ptr); OVERRIDE;
				{ For writing a scrollbar to a resource, works in conjunction with tbe previous
				method. }

				PROCEDURE TScrollBar.DeltaValue(delta: VCoordinate);
				{ Does range checking on 'delta' checking it against fLongMin & fLongMax for the
				scrollbar. }

				PROCEDURE TScrollBar.DoMouseCommand(VAR theMouse: VPoint;
												   event: TToolboxEvent;
												   hysteresis: Point); OVERRIDE;
				{ Handles the mouse down in the various parts of the scrollbar. }

				PROCEDURE TScrollBar.TrackScrollBar(partCode: INTEGER);
				{ Tracks the scrollbar by calling DeltaValue. }

				PROCEDURE TScrollBar.ActionProc(partCode: INTEGER);
				{ Forwarded to from the global routine: ActionProcForTScrollBar which was set as the
				ScrollBar's action proc.  Typically forwards again to TrackScrollBar after bounds
				checking and Updates before returning to the ScrollBar. }

				{ Inspecting }

				PROCEDURE TScrollBar.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents to this class's
				fields. }
				END;

{--------------------------------------------------------------------------------------------------}
			TList					= OBJECT; EXTERNAL;
			TScrollerScrollBar 		= OBJECT (TScrollBar)	{ A subclass of TScrollBar that is
														 associated with one or more scrollers. }
				fScrollers: 		TList;				{ the associated scrollers if any }

				PROCEDURE TScrollerScrollBar.Initialize; OVERRIDE;
				{ Puts a newly created TScrollerScrollBar into a known state. Subclasses should
				override it and call INHERITED. Typically used to put the object into a state where
				it can be safely FREED. }

				PROCEDURE TScrollerScrollBar.IScrollerScrollBar(itsSuperView: TView;
												  itsLocation, itsSize: VPoint;
												  itsHSizeDet, itsVSizeDet: SizeDeterminer;
												  itsDirection: VHSelect;
												  itsMax: LONGINT;
												  itsScroller: TScroller);
				{ Initialize the scrollbar procedurally. }

				PROCEDURE TScrollerScrollBar.IRes(itsDocument: TDocument;
										   itsSuperView: TView;
										   VAR itsParams: Ptr); OVERRIDE;
				{ Initialize the scrollbar from a resource template. }

				FUNCTION TScrollerScrollBar.Clone: TObject; OVERRIDE;
				{ calls INHERITED Clone and then clones owned objects }
				
				PROCEDURE TScrollerScrollBar.WriteRes(theResource: ViewRsrcHandle;
											   VAR itsParams: Ptr); OVERRIDE;
				{ For writing an sscrollbar out to a resource. }

				PROCEDURE TScrollerScrollBar.Free; OVERRIDE;
				{ If the scrollers are not NIL then first detach the scrollbars from them before
				freeing the scrollers, then call inherited Free. }

				PROCEDURE TScrollerScrollBar.AttachScroller(itsScroller: TScroller);
				{ IF 'itsScroller' is not NIL then add it to our list and attach ourselves to it. }

				PROCEDURE TScrollerScrollBar.BeInPort(itsPort: GrafPtr); OVERRIDE;
				{ Call inherited BeInPort and then set our visibility based on the state of the
				window we are in. }

				PROCEDURE TScrollerScrollBar.Activate(entering: BOOLEAN); OVERRIDE;
				{ If 'entering' is TRUE then show ourselves, otherwise just call Draw to frame our
				area. }

				PROCEDURE TScrollerScrollBar.DoMouseCommand(VAR theMouse: VPoint;
													event: TToolboxEvent;
													hysteresis: Point); OVERRIDE;
				{ Handles mouse downs in the scrollbar. }

				PROCEDURE TScrollerScrollBar.Draw(area: VRect); OVERRIDE;
				{ Draws a frame around the controls area before calling inherited Draw. }

				PROCEDURE TScrollerScrollBar.TrackScrollBar(partCode: INTEGER); OVERRIDE;
				{ Called to handle tracking of the scrollbar by calling DeltaValue. }

				{ Inspecting }

				PROCEDURE TScrollerScrollBar.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields. }
				END;

{--------------------------------------------------------------------------------------------------}
			TControlTracker 	= OBJECT (TNoChangesTracker) { a Tracker adapted for tracking
															  controls. }
				fControl:			TControl;			{ the object of our desire }

				PROCEDURE TControlTracker.Initialize; OVERRIDE;
				{ Puts a newly created TControlTracker into a known state. Subclasses should
				override it and call INHERITED. Typically used to put the object into a state where
				it can be safely FREED. }

				PROCEDURE TControlTracker.IControlTracker(theControl: TControl;
														  itsMouse: VPoint);
				{ Initialize the tracker procedurally. }

				{ Inspecting }
				PROCEDURE TControlTracker.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields. }

				END;
END.
