{$P}
{[a-,body+,h-,o=100,r+,rec+,t=4,u+,#+,j=20/57/1$,n-]}
{Copyright  1988-1991 by Apple Computer Inc.  All rights reserved.}

UNIT UDependencies;

{ 	
	The dependency relationships are stored in a single array of triples.
	This is not space-efficient , but each entry is only ten bytes, so you should 
	be able to have hundreds of dependencies with no problem.
	
	The array is sorted, so we can iterate over just the necessary triples, by
	doing a binary search for a matching triple, and then iterating forward and
	backward from there.
	
}

	INTERFACE

		USES
			{}
			Types, MacAppTypes,
			{}
			Memory, UPascalObject, UObject,
			{}
			UList;

		CONST
		
			kNotifierGreater 	= kItem1GreaterThanItem2 + 2; { Values returned by TDependencies.CompareElements }
			kNotifierLess 	= kItem1LessThanItem2 - 2;
			kChangeGreater 	= kItem1GreaterThanItem2 + 1;
			kChangeLess		= kItem1LessThanItem2 - 1;
			kNoLabel		= MAXLONGINT - 1;

		TYPE 
			DependencyLabels = 1..kNoLabel; 	{ The valid range of labels }
			LabelRange = -kNoLabel..kNoLabel;	{ Used internally because we flip the label's sign to
													mark an object }
	
			DependencyPtr = ^Dependency;
			Dependency = RECORD
				notifier:  			TObject;
				dependent: 			TObject;
				dependencyLabel:	LabelRange;
			END;
	
			TDependencies = OBJECT(TSortedDynamicArray)	{	An array for storing the dependency relationships }
			
				PROCEDURE TDependencies.IDependencies;
				{ Initialize the array }
						
				{ Insertion and Deletion }
			
				PROCEDURE TDependencies.AddDependency(notifier : TObject;
													dependent : TObject;
													dependencyLabel: DependencyLabels);
				{ Register an object ("dependent") as dependent on the specified change in 
				another object ("Notifier") }
				
				PROCEDURE TDependencies.InsertDependencyBefore(index : ArrayIndex;
															notifier : TObject;
															dependent : TObject;
															dependencyLabel: DependencyLabels);
				{ Add a dependency relation to the array }
				
				PROCEDURE TDependencies.RemoveDependency(notifier : TObject;
														dependent : TObject;
														dependencyLabel: DependencyLabels);
				{ Remove a dependency relation from the array if it is present }
				
				
				{ Searching }
				FUNCTION  TDependencies.FindDependency(notifier : TObject;
													dependent : TObject;
													dependencyLabel: DependencyLabels;
													VAR index : ArrayIndex) : BOOLEAN;
				{ Returns TRUE if the specified dependency is in the array. The position of
				the dependency is returned in "index". If the dependency is not present the
				position at which it should be inserted is returned in "index"  }
								
				{ Debugging }
				PROCEDURE TDependencies.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields. }	
				
				PROCEDURE TDependencies.DynamicFields(obj: TObject); OVERRIDE;	
				{ Used by the Inspector and the Debugger to display the contents of this class's
				dynamic area. }	
				
			END;

			TDependenciesByNotifier = OBJECT(TDependencies)
				FUNCTION  TDependenciesByNotifier.CompareElements(Element1,Element2 : UNIV Ptr) : CompareResult; OVERRIDE;
				{ Orders the elements of the array by the ordinal value of the notifier.}

				FUNCTION TDependenciesByNotifier.FindNotifier(notifier : TObject;
															VAR index : ArrayIndex) : BOOLEAN;

				PROCEDURE TDependenciesByNotifier.EachDependencyWithNotifier(notifier : TObject;
								PROCEDURE DoToDependency(index : ArrayIndex;
															notifier : TObject;
															dependent : TObject;
															dependencyLabel: DependencyLabels));
				
				PROCEDURE TDependenciesByNotifier.EachDependentDo(notifier : TObject;
													PROCEDURE DoToDependent(aDependent : TObject));
				{ Calls DoToDependent on each of the objects which is dependent on the specified
				change in the object "Notifier" }
			END;
			
			TDependenciesByDependent = OBJECT(TDependencies)
				FUNCTION  TDependenciesByDependent.CompareElements(Element1,Element2 : UNIV Ptr) : CompareResult; OVERRIDE;
				{ Orders the elements of the array by the ordinal value of the dependent.}

				FUNCTION TDependenciesByDependent.FindDependent(dependent : TObject;
													VAR index : ArrayIndex) : BOOLEAN;

				PROCEDURE TDependenciesByDependent.EachDependencyWithDependent(dependent : TObject;
											PROCEDURE DoToDependency(index : ArrayIndex;
																		notifier : TObject;
																		dependent : TObject;
																		dependencyLabel: DependencyLabels));
				
				PROCEDURE TDependenciesByDependent.EachNotifierDo(dependent : TObject;
													PROCEDURE DoToNotifier(aNotifier : TObject));
				{ Calls DoToNotifier on each of the objects on which the specified dependent is
				dependent }				
			END;
			
{--------------------------------------------------------------------------------------------------}

	{ Procedural Interface to this unit }
	
	PROCEDURE AddDependentOf(notifier :  TObject; 
							dependent : TObject;
							dependencyLabel: DependencyLabels);
	{ Register an object ("dependent") as a dependent of another object ("Notifier"),
		with specified label (can be kNoLabel) }
	
	PROCEDURE RemoveDependentOf(notifier :  TObject; 
								dependent : TObject;
								dependencyLabel: DependencyLabels);
	{ Remove the dependency relation if it is present }

	PROCEDURE RemoveDependencies(anObject :  TObject);
	{ Remove all dependencies containing the object, either as Notifier or dependent.
	Called by TObject.Free. }
	
	PROCEDURE EachDependentOf(notifier : TObject;
					PROCEDURE DoToDependent(aDependent : TObject));
	{ Calls DoToDependent on each of the objects which is dependent on the specified
	change in the object "Notifier" }

	PROCEDURE EachNotifierOf(dependent : TObject;
					PROCEDURE DoToNotifier(aNotifier : TObject));
	{ Calls DoToNotifier on each of the objects on which the specified dependent
	depends }
	
	PROCEDURE MarkObject(anObject: TObject;state: BOOLEAN);
	{ Used to mark or unmark an object as needing updating. Used during a 
	change/update phase }
	
	FUNCTION  IsObjectMarked(anObject: TObject): BOOLEAN;
	{ Returns TRUE if the specified object is currently marked }
END.