{[a-,body+,h-,o=100,r+,rec+,t=4,u+,#+,j=20/57/1$,n+]}
{ UDesignator.p }
{ Copyright  1984-1991 by Apple Computer Inc.	All rights reserved. }

{
	Unit: UDesignator

	OVERVIEW
	========
	TDesignator is designed as an abstract superclass which designates a portion of the document.
	Each specialized document will require its own specialized designator. Specialized subclasses
	of TDesignator are required to designate portions of different documents.

	CLASS HIERARCHY
	===============
	TDesignator					abstract superclass for designators
		TLinearDesignator		a linear data designator (e.g. for text models)
		TRectDesignator			a rectangular designator (e.g. for spreadsheet models)
		TRgnDesignator			a region designator (e.g. for discontiguous designations)

	USAGE
	=====
	TDesignator is used with the Edition Manager support that is built into
	MacApp. Whenever a TSection object is instantiated, it requires a TDesignator object that
	lets the TSection object determine which portion of the document is being designated as
	a section, whether a publisher or a subscriber.
}

UNIT UDesignator;

	INTERFACE

		USES
			{  requirements for this unit's interface. }
			Types, MacAppTypes,
			{}
			Memory, UPascalObject, UObject,
			{}
			QuickDraw;

		CONST
			{ C O N T A I N M E N T }
			kNotContained		= 0;					{ designator is not contained }
			kPartiallyContained = 1;					{ designator is partially contained }
			kFullyContained		= 2;					{ designator is fully contained }

			{ O B J E C T  S I G N A T U R E S }
			{ These follow the same reservation rules as resource types }
			{ The signature is used to search the signature table and create
			an object "by signature"  If the signature's entry in the table has
			been updated with a new class then an object of that class will be created. }

			kLinearDesignator	= 'ldsg';
			kVRectDesignator 	= 'vdsg';
			kRgnDesignator 		= 'rdsg';
			
		TYPE
			{ C O N T A I N M E N T }
			ContainmentType		= kNotContained..kFullyContained;

			{ D E S I G N A T O R S }
			TDocument = object; external;
			TStream = object; external;

			{ TDesignator is designed as an abstract superclass. Use subclasses of TDesignator. }
			TDesignator 		= OBJECT (TObject)

				fDocument:			TDocument;			{ the document this designator refers to }

				{ === contructors/destructors === }
				PROCEDURE TDesignator.IDesignator(itsDocument: TDocument);
				{ initialize this object }

				PROCEDURE TDesignator.Initialize; OVERRIDE;

				PROCEDURE TDesignator.InitFromStream(itsDocument: TDocument; aStream: TStream);
				{ initializes this object from a stream specification }

				{ === data accessors === }
				FUNCTION TDesignator.GetSignature: IDType;
				{ returns the signature for this class }

				FUNCTION TDesignator.GetSize: Size;
				{ returns the size of this designation }

				{ === saving === }
				PROCEDURE TDesignator.WriteToStream(aStream: TStream);
				{ OVERRIDE this; writes specification of designator to stream }

				FUNCTION TDesignator.GetSpecificationSize: Size;
				{ OVERRIDE this; returns the size in bytes required to save the specification }

				{ === inspecting === }
				PROCEDURE TDesignator.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields}

				{ === containment === }
				FUNCTION TDesignator.IsContained(aDesignator: TDesignator): ContainmentType;
				{ OVERRIDE this; returns a ContainmentType indicating to what extent aDesignator
				is contained in SELF }

				END;

			{ TLinearDesignator is designed to designate any linear portion of data that may be
			designated by a start position and an end position. }
			TLinearDesignator = OBJECT (TDesignator)
		
				fStartPos: Longint;						{ offset to start }
				fEndPos: Longint;						{ offset to end }
		
				{ === contructors/destructors === }
				PROCEDURE TLinearDesignator.ILinearDesignator(itsDocument: TDocument; startPos,
												  endPos: Longint);
				{ initialize this object }

				PROCEDURE TLinearDesignator.Initialize; OVERRIDE;

				PROCEDURE TLinearDesignator.InitFromStream(itsDocument: TDocument; aStream: TStream); OVERRIDE;
				{ initializes this object from a stream specification }

				{ === data accessors === }
				FUNCTION TLinearDesignator.GetSignature: IDType; OVERRIDE;
				{ returns the signature for this class }

				FUNCTION TLinearDesignator.GetSize: Size; OVERRIDE;
				{ returns the size of this designation }

				{ === saving === }
				FUNCTION TLinearDesignator.GetSpecificationSize: Size; OVERRIDE;
				{ OVERRIDE this; returns the size in bytes required to save the specification }

				PROCEDURE TLinearDesignator.WriteToStream(aStream: TStream); OVERRIDE;
				{ OVERRIDE this; writes specification of designator to stream }

				{ === inspecting === }
				PROCEDURE TLinearDesignator.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields}

				{ === containment === }
				FUNCTION TLinearDesignator.IsContained(aDesignator: TDesignator): ContainmentType; OVERRIDE;
				{ OVERRIDE this; returns a ContainmentType indicating to what extent aDesignator
				is contained in SELF }

				END;

			{ TRectDesignator is designed to designate any rectangular portion of data that may be
			designated by top, left, bottom, and right VCoordinates. }
			TVRectDesignator = OBJECT (TDesignator)
		
				fDesignation: VRect;						{ rect representing designation }
		
				{ === contructors/destructors === }
				PROCEDURE TVRectDesignator.IVRectDesignator(itsDocument: TDocument;
														  itsDesignation: VRect);
				{ initialize this object }

				PROCEDURE TVRectDesignator.Initialize; OVERRIDE;

				PROCEDURE TVRectDesignator.InitFromStream(itsDocument: TDocument; aStream: TStream); OVERRIDE;
				{ initializes this object from a stream specification }

				{ === data accessors === }
				FUNCTION TVRectDesignator.GetSignature: IDType; OVERRIDE;
				{ returns the signature for this class }

				FUNCTION TVRectDesignator.GetSize: Size; OVERRIDE;
				{ returns the size of this designation }

				{ === saving === }
				FUNCTION TVRectDesignator.GetSpecificationSize: Size; OVERRIDE;
				{ OVERRIDE this; returns the size in bytes required to save the specification }

				PROCEDURE TVRectDesignator.WriteToStream(aStream: TStream); OVERRIDE;
				{ OVERRIDE this; writes specification of designator to stream }

				{ === inspecting === }
				PROCEDURE TVRectDesignator.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields}

				{ === containment === }
				FUNCTION TVRectDesignator.IsContained(aDesignator: TDesignator): ContainmentType; OVERRIDE;
				{ OVERRIDE this; returns a ContainmentType indicating to what extent aDesignator
				is contained in SELF }

				END;

			{ TRgnDesignator is designed to designate any portion of data that may be
			designated by a region. The caller must create the region, and the designator
			optionally frees the region. In general, the TStream passed into this object
			should be a THandleStream. }
			TRgnDesignator = OBJECT (TDesignator)
		
				fDesignation:	RgnHandle;				{ rgn representing designation }
				
				fFreeRgnOnFree:	BOOLEAN;				{ whether to free the RgnHandle when this
														designator is freed }
		
				{ === contructors/destructors === }
				PROCEDURE TRgnDesignator.IRgnDesignator(itsDocument: TDocument;
														itsDesignation: RgnHandle;
														freeRgnOnFree: BOOLEAN);
				{ initialize this object }

				PROCEDURE TRgnDesignator.Initialize; OVERRIDE;
				{ Initializes this object }
				
				PROCEDURE TRgnDesignator.InitFromStream(itsDocument: TDocument; aStream: TStream); OVERRIDE;
				{ initializes this object from a stream specification }

				PROCEDURE TRgnDesignator.Free; OVERRIDE;
				{ frees fDesignation and calls INHERITED Free }

				FUNCTION TRgnDesignator.Clone: TObject; OVERRIDE;
				{ calls INHERITED Clone, creates a copy of fDesignation rgn }

				{ === data accessors === }
				FUNCTION TRgnDesignator.GetSignature: IDType; OVERRIDE;
				{ returns the signature for this class }

				FUNCTION TRgnDesignator.GetSize: Size; OVERRIDE;
				{ returns the size of this designation }

				{ === saving === }
				FUNCTION TRgnDesignator.GetSpecificationSize: Size; OVERRIDE;
				{ OVERRIDE this; returns the size in bytes required to save the specification }

				PROCEDURE TRgnDesignator.WriteToStream(aStream: TStream); OVERRIDE;
				{ OVERRIDE this; writes specification of designator to stream }

				{ === inspecting === }
				PROCEDURE TRgnDesignator.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields}

				{ === containment === }
				FUNCTION TRgnDesignator.IsContained(aDesignator: TDesignator): ContainmentType; OVERRIDE;
				{ OVERRIDE this; returns a ContainmentType indicating to what extent aDesignator
				is contained in SELF }

				END;
		
		PROCEDURE InitUDesignator;
		{ initializes the designator unit, registering the standard types recognized by this unit }
END.
