{[a-,body+,h-,o=100,r+,rec+,t=4,u+,#+,j=20/57/1$,n+]}
{UDialog.p}
{Copyright  1988-1991 by Apple Computer Inc.  All rights reserved.}

UNIT UDialog;

	INTERFACE

		USES
			{}
			Types, MacAppTypes, Memory, UPascalObject, UObject, AppleEvents, UEvtHandler, Balloons, UView, UEvent, UCommand
			, Controls, TextEdit, UControl, UTEView;

		CONST
			kPreferColor		= TRUE; 				{ argument to IIcon }
			kControlOn			= TRUE; 				{ argument to ICheckBox and IRadio }

			{ Constants for TDialogView.Dismiss }
			kFlash				= TRUE; 				{ Constants for TDialogView.Dismiss. }
			kDontFlash			= FALSE;				{ Constants for TDialogView.Dismiss. }

			kMaxTEWidth 		= $2B0; 				{ Maximum width for TEditText autoscroll }

			phInvalidValue		= 300;					{ Alert used when an invalid value is
														 entered }
			kDialogTEViewID 	= 300; 					{ The ID of the 'view' resource for
														the "floating" TEView }

			kInvalidValueReasons = 300; 				{ String list used for invalid value alert }
			kErrorHandled		= - 1;					{ Don't post an alert, the user handled it }
			kValidValue 		= 0;					{ Not looked up in the string list. }
			kInvalidValue		= 1;
			kValueTooSmall		= 2;
			kValueTooLarge		= 3;
			kNonNumericCharacters = 4;
			kTooManyCharacters	= 5;
			kNoOfDefaultReasons = 5;

		TYPE

			DialogViewTemplate	= RECORD
				defaultItem:		IDType;
				cancelItem: 		IDType;
				END;
			DialogViewTemplatePtr = ^DialogViewTemplate;

			TAssociation	 	= OBJECT;	EXTERNAL;
			TDocument		 	= OBJECT;	EXTERNAL;
			TTEView			 	= OBJECT;	EXTERNAL;
			TScroller		 	= OBJECT;	EXTERNAL;
			TToolboxEvent		= OBJECT; 	EXTERNAL;

			TEditText	 		= OBJECT;	FORWARD;
			TDialogTEView		= OBJECT;	FORWARD;

			TDialogView 		= OBJECT (TView)		{ Mimics normal toolbox dialogs providing
														 behavior for handling default buttons,
														 param text, currently selected edit text
														 and other behavior typically associated
														 with dialogs. }
				fDefaultItem:		IDType; 			{ The identifier of the default item }
				fCancelItem:		IDType; 			{ The identifier of item that cancels dialog
														 }
				fDismisser: 		IDType; 			{ ID of view that caused dismissal }

				PROCEDURE TDialogView.Initialize; OVERRIDE;
				{ Puts a newly created TDialogView into a known state. Subclasses should
				override it and call INHERITED. Typically used to put the object into a state where
				it can be safely FREED. }

				PROCEDURE TDialogView.IDialogView(itsDocument: TDocument;
												  itsSuperView: TView;
												  itsLocation: VPoint;
												  itsSize: VPoint;
												  itsHSizeDet: SizeDeterminer;
												  itsVSizeDet: SizeDeterminer;
												  itsDefItemID: IDType;
												  itsCancelItemID: IDType);
				{ Initialization method for TDialogView. }

				PROCEDURE TDialogView.IRes(itsDocument: TDocument;
										   itsSuperView: TView;
										   VAR itsParams: Ptr); OVERRIDE;
				{ For initializing a dialog view from a resource template. }

				FUNCTION TDialogView.Clone: TObject; OVERRIDE;
				{ calls INHERITED Clone and then clones owned objects }

				PROCEDURE TDialogView.WRes(theResource: ViewRsrcHandle;
										   VAR itsParams: Ptr); OVERRIDE;
				{ For writing a dialog view out to a resource. }

				PROCEDURE TDialogView.WriteRes(theResource: ViewRsrcHandle;
											   VAR itsParams: Ptr); OVERRIDE;
				{ For writing a dialog view to a resource, works in conjunction with tbe
				previous method. }

				PROCEDURE TDialogView.Free; OVERRIDE;
				{!!! Calls inherited Free. }

				FUNCTION TDialogView.CanDismiss(dismissing: IDType): BOOLEAN;
				{ Used to check whether the dialog view can be dismissed or not by checking if the
				view is enabled, then check if the currently selected edit text is valid. }

				PROCEDURE TDialogView.Close; OVERRIDE;
				{ Called to close a dialog view. }

				PROCEDURE TDialogView.DismissDialog(dismisser: IDType);
				{ The method responsible for actually dismissing a dialog. }
				
				PROCEDURE TDialogView.DoCommandKey(event: TToolboxEvent); OVERRIDE;
				{ Called when a keyDown event is received and the command key is down. }

				PROCEDURE TDialogView.DoKeyCommand(event: TToolboxEvent); OVERRIDE;
				{ This method handles all key presses other than a command key press, which is
				handled by the previous method. }

				PROCEDURE TDialogView.DoOpen;
				{ If when opening the dialog the current target is an edit text field of the dialog,
				then select it. }

				FUNCTION TDialogView.DeselectCurrentEditText: BOOLEAN;
				{ Deselects the current edit text, this is called when tabbing between edit text
				fields. }

				PROCEDURE TDialogView.DoSelectEditText(theEditText: TEditText;
													   selectChars: BOOLEAN);
				{ Selects theEditText, is selectChars then all the characters are selected otherwise
				the insertion point is merely placed at the start of the field. }
				
				FUNCTION  TDialogView.GetCurrentEditText: TEditText;
				{ Returns the currently selected EditText, or NIL }

				PROCEDURE TDialogView.EachEditText(PROCEDURE
												   DoToEditText(theEditText: TEditText));
				{ Iterates over all the edit text items in the dialog calling DoToEditText for
				each item. }

				FUNCTION TDialogView.GetDialogView: TView; OVERRIDE;
				{ Returns a reference to the dialog view. }

				PROCEDURE TDialogView.DoEvent(eventNumber: EvtNumber;
													source: TEvtHandler;
													event: TEvent); OVERRIDE;

				{ If the default or cancel key are pressed then dismisses the dialog, otherwise
				passes the message along the chain. If the hit was an edit text item then deselects
				the item. Otherwise if the hit is on some other control and it dismisses the dialog
				then dismiss it. }

				PROCEDURE TDialogView.Open; OVERRIDE;
				{ Called to open a dialog. }

				FUNCTION TDialogView.PoseModally: IDType;
				{ Displays the dialog in a modal fashion waiting for a response, which is returned
				as the IDType of the button hit that causes the dialog to be dismissed. }

				PROCEDURE TDialogView.SelectEditText(itsIdentifier: IDType;
													 selectChars: BOOLEAN);
				{ If itsIdentifier can be found then the field is selected, selectChars if true means
				that all the text will be selected as opposed to the insertion point merely being
				placed in the field. }

				PROCEDURE TDialogView.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields. }

				END;

			ButtonTemplate		= RECORD
				itsLabel:			Str255; 			{ Actually, variable length }
				END;
			ButtonTemplatePtr	= ^ButtonTemplate;

			TButton 			= OBJECT (TCtlMgr)		{ Mimics a typical toolbox button as used in
														 a dialog. }
				PROCEDURE TButton.Initialize; OVERRIDE;
				{ Puts a newly created TButton into a known state. Subclasses should
				override it and call INHERITED. Typically used to put the object into a state where
				it can be safely FREED. }

				PROCEDURE TButton.IButton(itsSuperView: TView;
										  itsLocation, itsSize: VPoint;
										  itsHSizeDet, itsVSizeDet: SizeDeterminer;
										  itsLabel: Str255);
				{ Initializes the button setting the fDefChoice field to mButtonHit. }

				PROCEDURE TButton.IRes(itsDocument: TDocument;
									   itsSuperView: TView;
									   VAR itsParams: Ptr); OVERRIDE;
				{ Initialize the view from a resource template. }

				FUNCTION TButton.Clone: TObject; OVERRIDE;
				{ calls INHERITED Clone and then clones owned objects }

				PROCEDURE TButton.WRes(theResource: ViewRsrcHandle;
									   VAR itsParams: Ptr); OVERRIDE;
				{ Write this object out as a view resource. }

				PROCEDURE TButton.WriteRes(theResource: ViewRsrcHandle;
										   VAR itsParams: Ptr); OVERRIDE;
				{ Set up the type and signature of this object and call WRes. }

				PROCEDURE TButton.DoEvent(eventNumber: EvtNumber;
													source: TEvtHandler;
													event: TEvent); OVERRIDE;
				{ If the hit is mButtonHit then flash the button provided you didn't hit yourself. }

				PROCEDURE TButton.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields. }

				END;

			CheckBoxTemplate	= RECORD
				isOn:				BOOLEAN;
				itsLabel:			Str255; 			{ Actually, variable length }
				END;
			CheckBoxTemplatePtr = ^CheckBoxTemplate;

			TCheckBox			= OBJECT (TCtlMgr)		{ Provides the functionality of a toolbox
														 checkbox. }
				PROCEDURE TCheckBox.Initialize; OVERRIDE;
				{ Puts a newly created TCheckBox into a known state. Subclasses should
				override it and call INHERITED. Typically used to put the object into a state where
				it can be safely FREED. }

				PROCEDURE TCheckBox.ICheckBox(itsSuperView: TView;
											  itsLocation, itsSize: VPoint;
											  itsHSizeDet, itsVSizeDet: SizeDeterminer;
											  itsLabel: Str255;
											  isTurnedOn: BOOLEAN);
				{ Initialize the checkbox setting fDefChoice to mCheckBoxHit. }

				PROCEDURE TCheckBox.IRes(itsDocument: TDocument;
										 itsSuperView: TView;
										 VAR itsParams: Ptr); OVERRIDE;
				{  Initialize the view from a resource template. }

				FUNCTION TCheckBox.Clone: TObject; OVERRIDE;
				{ calls INHERITED Clone and then clones owned objects }

				PROCEDURE TCheckBox.WRes(theResource: ViewRsrcHandle;
										 VAR itsParams: Ptr); OVERRIDE;
				{ Write this object out as a view resource. }

				PROCEDURE TCheckBox.WriteRes(theResource: ViewRsrcHandle;
											 VAR itsParams: Ptr); OVERRIDE;
				{ Set up the type and signature of this object and call WRes. }

				PROCEDURE TCheckBox.DoEvent(eventNumber: EvtNumber;
													source: TEvtHandler;
													event: TEvent); OVERRIDE;
				{ If the hit is mCheckBoxHit then toggle the checkbox. }

				FUNCTION TCheckBox.IsOn: BOOLEAN;
				{ Is the checkbox currently checked. }

				PROCEDURE TCheckBox.SetState(state, redraw: BOOLEAN);
				{ Sets the state of the checkbox to state and if redraw is TRUE redraws it. }

				PROCEDURE TCheckBox.Toggle(redraw: BOOLEAN);
				{ Toggles the state of the checkbox redraw it if redraw is TRUE. }

				PROCEDURE TCheckBox.ToggleIf(matchState, redraw: BOOLEAN);
				{ Toggles the state of the checkbox if matchState is TRUE redrawing if redraw is
				TRUE. }

				PROCEDURE TCheckBox.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields. }

				END;

			RadioTemplate		= RECORD
				isOn:				BOOLEAN;
				itsLabel:			Str255; 			{ Actually, variable length }
				END;
			RadioTemplatePtr	= ^RadioTemplate;

			TRadio				= OBJECT (TCtlMgr)		{ Provides the functionality of a toolbox
														 radio button. }
				PROCEDURE TRadio.Initialize; OVERRIDE;
				{ Puts a newly created TRadio into a known state. Subclasses should
				override it and call INHERITED. Typically used to put the object into a state where
				it can be safely FREED. }

				PROCEDURE TRadio.IRadio(itsSuperView: TView;
										itsLocation, itsSize: VPoint;
										itsHSizeDet, itsVSizeDet: SizeDeterminer;
										itsLabel: Str255;
										isTurnedOn: BOOLEAN);
				{ Initalize the radio button procedurally. }

				PROCEDURE TRadio.IRes(itsDocument: TDocument;
									  itsSuperView: TView;
									  VAR itsParams: Ptr); OVERRIDE;
				{  Initialize the view from a resource template. }

				FUNCTION TRadio.Clone: TObject; OVERRIDE;
				{ calls INHERITED Clone and then clones owned objects }

				PROCEDURE TRadio.WRes(theResource: ViewRsrcHandle;
									  VAR itsParams: Ptr); OVERRIDE;
				{ Write the object out as a view resource. }

				PROCEDURE TRadio.WriteRes(theResource: ViewRsrcHandle;
										  VAR itsParams: Ptr); OVERRIDE;
				{ Set up the type and signature of this object and call WRes. }

				PROCEDURE TRadio.DoEvent(eventNumber: EvtNumber;
													source: TEvtHandler;
													event: TEvent); OVERRIDE;
				{ If the hit is mRadioHit then toggle the radio button. }

				FUNCTION TRadio.IsOn: BOOLEAN;
				{ Is the radio button currently on. }

				PROCEDURE TRadio.SetState(state, redraw: BOOLEAN);
				{ Sets the state of the radio button to state, redrawing if redraw is TRUE. }

				PROCEDURE TRadio.Toggle(redraw: BOOLEAN);
				{ Toggles the state of the radio button, redrawing if redraw is TRUE. }

				PROCEDURE TRadio.ToggleIf(matchState, redraw: BOOLEAN);
				{ Only toggles the state of the radio button if matchState is TRUE, redrawing if
				redraw is TRUE. }

				PROCEDURE TRadio.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields. }

				END;

			ClusterTemplate 	= RECORD
				itsLabel:			Str255; 			{ Actually, variable length }
				END;
			ClusterTemplatePtr	= ^ClusterTemplate;

			TCluster			= OBJECT (TControl) 	{ Provides a mechanism whereby radio buttons
														 can be grouped in clusters, such that when
														 one of the buttons is selected the others
														 are all deselected. Also can be utilized
														 for providing labelled adornments around
														 groups of dialog items. }
				fRsrcID:			INTEGER;			{ resource ID of the cluster's label }
				fIndex: 			INTEGER;			{ currently selected item }
				fDataHandle:		StringHandle;		{ the cluster's label }

				PROCEDURE TCluster.Initialize; OVERRIDE;
				{ Puts a newly created TCluster into a known state. Subclasses should
				override it and call INHERITED. Typically used to put the object into a state where
				it can be safely FREED. }

				PROCEDURE TCluster.ICluster(itsSuperView: TView;
											itsLocation, itsSize: VPoint;
											itsHSizeDet, itsVSizeDet: SizeDeterminer;
											itsRsrcID, itsIndex: INTEGER);
				{ Initialize the cluster procedurally. }

				PROCEDURE TCluster.IRes(itsDocument: TDocument;
										itsSuperView: TView;
										VAR itsParams: Ptr); OVERRIDE;
				{  Initialize the view template. }

				FUNCTION TCluster.Clone: TObject; OVERRIDE;
				{ calls INHERITED Clone and then clones owned objects }

				PROCEDURE TCluster.WRes(theResource: ViewRsrcHandle;
										VAR itsParams: Ptr); OVERRIDE;
				{ Write the object out as a view resource. }

				PROCEDURE TCluster.WriteRes(theResource: ViewRsrcHandle;
											VAR itsParams: Ptr); OVERRIDE;
				{ Set up the type and signature of this object and call WRes. }

				PROCEDURE TCluster.Free; OVERRIDE;
				{ Release the label and call inherited free. }

				PROCEDURE TCluster.Draw(area: VRect); OVERRIDE;
				{ Frame the cluster if there is a label, then draw the label. }

				PROCEDURE TCluster.DrawLabel(area: VRect);
				{ Draw the label. }

				PROCEDURE TCluster.DoEvent(eventNumber: EvtNumber;
													source: TEvtHandler;
													event: TEvent); OVERRIDE;
				{ Responsible for resetting the radio buttons so that the last selected one is the only
				one selected. }

				PROCEDURE TCluster.GetLabel(VAR theLabel: Str255);
				{ Returns in theLable the text in the clusters label. }

				PROCEDURE TCluster.ReleaseLabel;
				{ Releases fDataHandle which is a string handle containing the text for the label. }

				FUNCTION TCluster.ReportCurrent: IDType;
				{ Returns the IDType of the currently selected item. }

				PROCEDURE TCluster.SetLabel(theLabel: Str255;
											redraw: BOOLEAN);
				{ Sets the clusters label to be theLabel, redrawing if redraw is TRUE. }

				PROCEDURE TCluster.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields. }
				END;

			IconTemplate		= RECORD
				preferColor:		BOOLEAN;
				rsrcID: 			INTEGER;
				END;
			IconTemplatePtr 	= ^IconTemplate;

			TIcon				= OBJECT (TControl) 	{ Provides an icon that can behave as a
														 button. }
				fPreferColor:		BOOLEAN;			{ tells us how to GET the icon - GetCIcon or GetIcon }
				fIsColor:			BOOLEAN;			{ tells us how to DRAW or FREE the icon }
				fRsrcID:			INTEGER;
				fDataHandle:		Handle;

				PROCEDURE TIcon.Initialize; OVERRIDE;
				{ Puts a newly created TIcon into a known state. Subclasses should
				override it and call INHERITED. Typically used to put the object into a state where
				it can be safely FREED. }

				PROCEDURE TIcon.IIcon(itsSuperView: TView;
									  itsLocation, itsSize: VPoint;
									  itsHSizeDet, itsVSizeDet: SizeDeterminer;
									  itsRsrcID: INTEGER;
									  preferColor: BOOLEAN);
				{  Initialize the view template. }

				PROCEDURE TIcon.IRes(itsDocument: TDocument;
									 itsSuperView: TView;
									 VAR itsParams: Ptr); OVERRIDE;
				{ Initialize the view template. }

				FUNCTION TIcon.Clone: TObject; OVERRIDE;
				{ calls INHERITED Clone and then clones owned objects }

				PROCEDURE TIcon.WRes(theResource: ViewRsrcHandle;
									 VAR itsParams: Ptr); OVERRIDE;
				{ Write the object out as a view resource. }

				PROCEDURE TIcon.WriteRes(theResource: ViewRsrcHandle;
										 VAR itsParams: Ptr); OVERRIDE;
				{ Set up the type and signature of this object and call WRes. }

				PROCEDURE TIcon.Free; OVERRIDE;
				{ Release the icon then call inherited Free. }

				PROCEDURE TIcon.Draw(area: VRect); OVERRIDE;
				{ Resonsible for drawing the icon and any required adornments. }

				PROCEDURE TIcon.ReleaseIcon;
				{ Releases the handle to the icon. }

				PROCEDURE TIcon.SetIcon(theIcon: Handle;
										redraw: BOOLEAN);
				{ Sets fDataHandle to theIcon, redrawing if redraw is TRUE. }

				PROCEDURE TIcon.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields. }
				END;

		SmallIconTemplatePtr = ^SmallIconTemplate;
		SmallIconTemplate	= RECORD
			rsrcID: 			INTEGER;
			END;

		TSmallIcon			= OBJECT (TControl)

			fRsrcID:			INTEGER;
			fDataHandle:		Handle;

			PROCEDURE TSmallIcon.Initialize; OVERRIDE;
			{ Puts a newly created TSmallIcon into a known state. Subclasses should
			override it and call INHERITED. Typically used to put the object into a state where
			it can be safely FREED. }

			PROCEDURE TSmallIcon.ISmallIcon(itsSuperView: TView;
											itsLocation, itsSize: VPoint;
											itsHSizeDet, itsVSizeDet: SizeDeterminer;
											itsRsrcID: INTEGER);

			PROCEDURE TSmallIcon.IRes(itsDocument: TDocument; itsSuperView: TView;
									  VAR itsParams: Ptr); OVERRIDE;

			FUNCTION TSmallIcon.Clone: TObject; OVERRIDE;
			{ calls INHERITED Clone and then clones owned objects }

			PROCEDURE TSmallIcon.WRes(theResource: ViewRsrcHandle; VAR itsParams: Ptr); OVERRIDE;

			PROCEDURE TSmallIcon.WriteRes(theResource: ViewRsrcHandle; VAR itsParams: Ptr); OVERRIDE;

			PROCEDURE TSmallIcon.Draw(area: VRect); OVERRIDE;

			PROCEDURE TSmallIcon.ReleaseSmallIcon;

			PROCEDURE TSmallIcon.SetSmallIcon(theSmallIcon: INTEGER; redraw: BOOLEAN);

			PROCEDURE TSmallIcon.Fields(obj: TObject); OVERRIDE;
			END;

			PatternTemplate 	= RECORD
				preferColor:		BOOLEAN;
				rsrcID: 			INTEGER;
				END;
			PatternTemplatePtr	= ^PatternTemplate;

			TPattern			= OBJECT (TControl) 	{ Provides an object that can display a
														 pattern and can also behave as a button if
														 required. }
				fPreferColor:		BOOLEAN;			{ tells us how to GET the pattern }
				fIsColor:			BOOLEAN;			{ tells us how to DRAW or FREE the pattern }
				fRsrcID:			INTEGER;
				fDataHandle:		Handle;

				PROCEDURE TPattern.Initialize; OVERRIDE;
				{ Puts a newly created TPattern into a known state. Subclasses should
				override it and call INHERITED. Typically used to put the object into a state where
				it can be safely FREED. }

				PROCEDURE TPattern.IPattern(itsSuperView: TView;
											itsLocation, itsSize: VPoint;
											itsHSizeDet, itsVSizeDet: SizeDeterminer;
											itsRsrcID: INTEGER;
											preferColor: BOOLEAN);
				{ Initialize the pattern procedurally. }

				PROCEDURE TPattern.IRes(itsDocument: TDocument;
										itsSuperView: TView;
										VAR itsParams: Ptr); OVERRIDE;
				{ Initialize the view template. }

				FUNCTION TPattern.Clone: TObject; OVERRIDE;
				{ calls INHERITED Clone and then clones owned objects }
	
				PROCEDURE TPattern.WRes(theResource: ViewRsrcHandle;
										VAR itsParams: Ptr); OVERRIDE;
				{ Write the object out as a view resource. }

				PROCEDURE TPattern.WriteRes(theResource: ViewRsrcHandle;
											VAR itsParams: Ptr); OVERRIDE;
				{ Set up the type and signature of this object and call WRes. }

				PROCEDURE TPattern.Free; OVERRIDE;
				{ Release the pattern then call inherited Free. }

				PROCEDURE TPattern.Draw(area: VRect); OVERRIDE;
				{ Responsible for drawing the pattern and any required adornments. }

				PROCEDURE TPattern.ReleasePattern;
				{ Releases the pattern by disposing fDataHandle. }

				PROCEDURE TPattern.SetPattern(thePattern: Handle;
											  redraw: BOOLEAN);
				{ Sets fDataHandle to thePattern, redrawing if redraw is TRUE. }

				PROCEDURE TPattern.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields. }
				END;

			PictureTemplate 	= RECORD
				rsrcID: 			INTEGER;
				END;
			PictureTemplatePtr	= ^PictureTemplate;

			TPicture			= OBJECT (TControl) 	{ Provides an object that can display a PICT
														 and if required behave like a button. }
				fRsrcID:			INTEGER;
				fDataHandle:		PicHandle;

				PROCEDURE TPicture.Initialize; OVERRIDE;
				{ Puts a newly created TPicture into a known state. Subclasses should
				override it and call INHERITED. Typically used to put the object into a state where
				it can be safely FREED. }

				PROCEDURE TPicture.IPicture(itsSuperView: TView;
											itsLocation, itsSize: VPoint;
											itsHSizeDet, itsVSizeDet: SizeDeterminer;
											itsRsrcID: INTEGER);
				{ Initialize the picture procedurally. }

				PROCEDURE TPicture.IRes(itsDocument: TDocument;
										itsSuperView: TView;
										VAR itsParams: Ptr); OVERRIDE;
				{ Initialize the view from a resource template. }

				FUNCTION TPicture.Clone: TObject; OVERRIDE;
				{ calls INHERITED Clone and then clones owned objects }
	
				PROCEDURE TPicture.WRes(theResource: ViewRsrcHandle;
										VAR itsParams: Ptr); OVERRIDE;
				{ Write the object out as a view resource. }

				PROCEDURE TPicture.WriteRes(theResource: ViewRsrcHandle;
											VAR itsParams: Ptr); OVERRIDE;
				{ Set up the type and signature of this object and call WRes. }

				PROCEDURE TPicture.Free; OVERRIDE;
				{ Release the picture then call inherited Free. }

				PROCEDURE TPicture.Draw(area: VRect); OVERRIDE;
				{ Responsible for drawing the PICT and any required adornments. }

				PROCEDURE TPicture.ReleasePicture;
				{ Releases the PICT by disposing of fDataHandle. }

				PROCEDURE TPicture.SetPicture(thePicture: PicHandle;
											  redraw: BOOLEAN);
				{ Sets fDataHandle to thePicture, redrawing if redraw is TRUE. }

				PROCEDURE TPicture.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields. }
				END;

			StaticTextTemplate	= RECORD
				just:				INTEGER;
				data:				Str255; 			{ Actually, variable length }
				END;
			StaticTextTemplatePtr = ^StaticTextTemplate;

			TStaticText 		= OBJECT (TControl) 	{ Provides an object that represents a
														 toolbox static text item. }
				fRsrcID:			INTEGER;			{ ID of the string list that the string is
														 in. }
				fIndex: 			INTEGER;			{ Index number for the string from the above
														 list. }
				fDataHandle:		StringHandle;		{ A handle to the actual string. }
				fJust:				INTEGER;			{ Jusitification of the text in the field. }
				fAutoWrap:			BOOLEAN;			{ TRUE to word wrap lines at the view boundary
														otherwise lines wrap at Carriage return. }

				PROCEDURE TStaticText.Initialize; OVERRIDE;
				{ Puts a newly created TStaticText into a known state. Subclasses should
				override it and call INHERITED. Typically used to put the object into a state where
				it can be safely FREED. }

				PROCEDURE TStaticText.IStaticText(itsSuperView: TView;
												  itsLocation, itsSize: VPoint;
												  itsHSizeDet, itsVSizeDet: SizeDeterminer;
												  itsRsrcID, itsIndex: INTEGER);
				{ Initialize the static text field procedurally. }

				PROCEDURE TStaticText.IRes(itsDocument: TDocument;
										   itsSuperView: TView;
										   VAR itsParams: Ptr); OVERRIDE;
				{ Initialize the view template. }

				FUNCTION TStaticText.Clone: TObject; OVERRIDE;
				{ calls INHERITED Clone and then clones owned objects }
	
				PROCEDURE TStaticText.WRes(theResource: ViewRsrcHandle;
										   VAR itsParams: Ptr); OVERRIDE;
				{ Write the object out as a view resource. }

				PROCEDURE TStaticText.WriteRes(theResource: ViewRsrcHandle;
											   VAR itsParams: Ptr); OVERRIDE;
				{ Set up the type and signature of this object and call WRes. }

				PROCEDURE TStaticText.Free; OVERRIDE;
				{ Releases the text and then calls inherited Free. }

				PROCEDURE TStaticText.DoSubstitution(VAR theText: Str255);
				{ Calls ReplaceText so that any paramTxt items can be handled if needed. }

				PROCEDURE TStaticText.Draw(area: VRect); OVERRIDE;
				{ Draws the text by calling ImageText. }

				PROCEDURE TStaticText.GetText(VAR theText: Str255);
				{ If fDataHandle not NIL then returns the text, otherwise returns the null string. }

				PROCEDURE TStaticText.ImageText(text: Ptr;
												length: LONGINT;
												box: VRect;
												just: INTEGER);
				{ Draws the text. }

				PROCEDURE TStaticText.ChangeWrap(newAutoWrap, redraw: BOOLEAN);
				{ Changes auto-wrapping behavior, redrawing if requested }

				PROCEDURE TStaticText.ReleaseText;
				{ Releases the text by disposing of fDataHandle. }

				PROCEDURE TStaticText.SetJustification(theJust: INTEGER;
													   redraw: BOOLEAN);
				{ Sets the justification of the text to theJust, redrawing if required. }

				PROCEDURE TStaticText.SetText(theText: Str255;
											  redraw: BOOLEAN);
				{ Sets the text to theText, redrawing if required. }

				PROCEDURE TStaticText.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields. }
				END;

			EditTextTemplate	= RECORD
				maxChars:			INTEGER;
				controlChars:		ControlCharSet;
				END;
			EditTextTemplatePtr = ^EditTextTemplate;

			TEditText			= OBJECT (TStaticText)	{ Implements an edit text field. }
				fMaxChars:			INTEGER;
				fTEView:			TDialogTEView;
				fControlChars:		ControlCharSet;		{ Control chars accepted in text }

				PROCEDURE TEditText.Initialize; OVERRIDE;
				{ Puts a newly created TEditText into a known state. Subclasses should
				override it and call INHERITED. Typically used to put the object into a state where
				it can be safely FREED. }

				PROCEDURE TEditText.IEditText(itsSuperView: TView;
											  itsLocation, itsSize: VPoint;
											  itsMaxChars: INTEGER);
				{ Initialize the edit text item procedurally. }

				PROCEDURE TEditText.IRes(itsDocument: TDocument;
										 itsSuperView: TView;
										 VAR itsParams: Ptr); OVERRIDE;
				{ Initialize the view from a resource template. }

				FUNCTION TEditText.Clone: TObject; OVERRIDE;
				{ calls INHERITED Clone and then clones owned objects }
	
				PROCEDURE TEditText.WRes(theResource: ViewRsrcHandle;
										 VAR itsParams: Ptr); OVERRIDE;
				{ Write the object out as a view resource. }

				PROCEDURE TEditText.WriteRes(theResource: ViewRsrcHandle;
											 VAR itsParams: Ptr); OVERRIDE;
				{ Set up the type and signature of this object and call WRes. }

				PROCEDURE TEditText.Free; OVERRIDE;
				{ Un-installs the floating TEView, sets fTEView to NIL then calls inherited Free. }

				PROCEDURE TEditText.DoSubstitution(VAR theText: Str255); OVERRIDE;
				{ Default is to do nothing. }

				PROCEDURE TEditText.GetText(VAR theText: Str255); OVERRIDE;
				{ If TEView not NIL then returns the extracted text, otherwise calls inherited
				GetText. }

				PROCEDURE TEditText.ImageText(text: Ptr;
											  length: LONGINT;
											  box: VRect;
											  just: INTEGER); OVERRIDE;
				{ Sets up the text to the parameters passed. Text is drawn in a 'box' , the text is
				'length' long. }

				PROCEDURE TEditText.BecameWindowTarget; OVERRIDE;
				PROCEDURE TEditText.ResignedWindowTarget; OVERRIDE;
				PROCEDURE TEditText.BecameApplicationTarget; OVERRIDE;
				PROCEDURE TEditText.ResignedApplicationTarget; OVERRIDE;
				
				FUNCTION TEditText.WantToBecomeTarget: BOOLEAN; OVERRIDE;
				FUNCTION  TEditText.WillingToResignTarget: LONGINT; OVERRIDE;

				
				PROCEDURE TEditText.RestartEdit(restartText: Str255);
				{ This method resets the text of the view, as well as selecting the text. We go to
				some extra trouble to ensure that this method doesn't do any drawing, defering to
				window update time. The reasoning is that this method usually gets called right
				after displaying an alert. The alert may have obscured this view, thus including it
				in the window's update region. If we drew in this method we would get flashing as
				this view would get redraw at update time. Unfortunately TextEdit isn't very
				cooperative so we set the clipping to an empty rect to ensure that TEActivate's
				drawing isn't seen. }

				PROCEDURE TEditText.ChangeWrap(newAutoWrap, redraw: BOOLEAN); OVERRIDE;
				{ Changes auto-wrapping behavior, redrawing if requested.  Forwards into
				the floating TE if necessary. }

				PROCEDURE TEditText.SetJustification(theJust: INTEGER;
													 redraw: BOOLEAN); OVERRIDE;
				{ Sets the justification of the text to theJust, redrawing is required. }

				PROCEDURE TEditText.SetSelection(selStart, selEnd: INTEGER;
												 redraw: BOOLEAN);
				{ Sets the selection in the field starting at selStart and ending at selEnd,
				redrawing if required. }

				PROCEDURE TEditText.SetText(theText: Str255;
											redraw: BOOLEAN); OVERRIDE;
				{ Sets the displayed text to theText, redrawing if required. }

				PROCEDURE TEditText.StartEdit(selectChars: BOOLEAN);
				{ Install the selection, revealing the selection if required. }
				
				PROCEDURE TEditText.MakeVisible;
				{ Reveal the EditText, if scrolled out of view }
				
				FUNCTION  TEditText.Deselect : BOOLEAN;
				{ Valdates and and deselects the edit text, if possible }

				PROCEDURE TEditText.StopEdit;
				{ Extracts the string from the TEView, then clears out the TEView before calling
				SELF.SetText. }

				PROCEDURE TEditText.InstallFloatingTEView(selectChars: BOOLEAN);
				{ Attach the "floating" TDialogTEView to this EditText }

				PROCEDURE TEditText.RemoveFloatingTEView;
				{ Remove the "floating" TDialogTEView from this EditText. }
				
				FUNCTION TEditText.Validate: LONGINT; OVERRIDE;
				{ By default just returns TRUE. }

				PROCEDURE TEditText.TargetValidationFailed(reason: LONGINT); OVERRIDE;
				{ Displays an alert when validation fails.}
				
				PROCEDURE TEditText.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields. }
				END;


			NumberTextTemplate	= RECORD
				value:				LONGINT;
				minimum:			LONGINT;
				maximum:			LONGINT;
				END;
			NumberTextTemplatePtr = ^NumberTextTemplate;

			TNumberText 		= OBJECT (TEditText)	{ This subclass of TEditText that allows the
														 handling of integer numbers. Range
														 checking will be performed on the numbers
														 entered to see if they are in the range
														 fMinimum to fMaximum. }
				fMinimum:			LONGINT;
				fMaximum:			LONGINT;

				PROCEDURE TNumberText.Initialize; OVERRIDE;
				{ Puts a newly created TNumberText into a known state. Subclasses should
				override it and call INHERITED. Typically used to put the object into a state where
				it can be safely FREED. }

				PROCEDURE TNumberText.INumberText(itsSuperView: TView;
												  itsLocation, itsSize: VPoint;
												  itsValue, itsMinimum, itsMaximum: LONGINT);
				{ Initialize the number text item procedurally. }

				PROCEDURE TNumberText.IRes(itsDocument: TDocument;
										   itsSuperView: TView;
										   VAR itsParams: Ptr); OVERRIDE;
				{ Initialize the view template. }

				PROCEDURE TNumberText.WRes(theResource: ViewRsrcHandle;
										   VAR itsParams: Ptr); OVERRIDE;
				{ Write the object out as a view resource. }

				PROCEDURE TNumberText.WriteRes(theResource: ViewRsrcHandle;
											   VAR itsParams: Ptr); OVERRIDE;
				{ Set up the type and signature of this object and call WRes. }

				FUNCTION TNumberText.GetValue: LONGINT;
				{ Returns the current value by getting the string and converting it to a number. }

				PROCEDURE TNumberText.SetValue(newValue: LONGINT;
											   redraw: BOOLEAN);
				{ Sets the current value by converting the number to a string and then calling
				SetText. }

				FUNCTION TNumberText.Validate: LONGINT; OVERRIDE;
				{ Does a range check on the number and if it is out of range returns the appropriate
				error code. }

				PROCEDURE TNumberText.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields. }
				END;

			TDialogTEView		= OBJECT (TTEView)		{ Provides a floating TEView that is placed
														 over edit & number text fields when
														 entering text. }
				fEditText:			TEditText;			{ The edit text field to which the floating
														 TEView is currently tied. }

				fScroller:			TScroller;			{ The scroller that I am in for mouseability. }

				PROCEDURE TDialogTEView.Initialize; OVERRIDE;
				{ Puts a newly created TDialogTEView into a known state. Subclasses should
				override it and call INHERITED. Typically used to put the object into a state where
				it can be safely FREED. }

				PROCEDURE TDialogTEView.IDialogTEView(itsDocument: TDocument; { OK to be NIL, if view will
																   belong to a documentless window
																   }
										  itsSuperView: TView; { The view in which this view is
																contained }
										  itsLocation: VPoint; { Location in its superview }
										  itsSize: VPoint; { Size of view }
										  itsHDeterminer: SizeDeterminer; { How to width of the view
																		   is to be determined }
										  itsVDeterminer: SizeDeterminer; { How the height of the
																		   view is to be determined
																		   }
										  itsInset: VRect; { Amount to inset text }
										  itsTextStyle: TextStyle; { Initial text style }
										  itsJustification: INTEGER; { Its justification }
										  itsStyleType: BOOLEAN; { Whether or not record is styled }
										  itsAutoWrap: BOOLEAN { FALSE if newline occurs at CR only
																}
										  );
				{ For Initialization of procedurally created TEView. }

				PROCEDURE TDialogTEView.IRes(itsDocument: TDocument;
									   itsSuperView: TView;
									   VAR itsParams: Ptr); OVERRIDE;
				{ Initialize a TTEView via a 'view' resource. }

				FUNCTION TDialogTEView.Clone: TObject; OVERRIDE;
				{ calls INHERITED Clone and then clones owned objects }
	
				PROCEDURE TDialogTEView.Free; OVERRIDE;
				{ Free associated components and then free self. }

				PROCEDURE TDialogTEView.InstallEditText(theEditText: TEditText;
														selectChars: BOOLEAN);
				{ Installs the EditText adding SELF as a subview to EditText, then gets the text
				from the EditText item and installs it in SELF. }

				PROCEDURE TDialogTEView.ComputeSize(VAR newSize: VPoint); OVERRIDE;
				{ accounts for case in which horizontal determiner is sizeVariable should be added
				to TTEView !!! }

				PROCEDURE TDialogTEView.BecameWindowTarget; OVERRIDE;
				PROCEDURE TDialogTEView.ResignedWindowTarget; OVERRIDE;
				PROCEDURE TDialogTEView.BecameApplicationTarget; OVERRIDE;
				PROCEDURE TDialogTEView.ResignedApplicationTarget; OVERRIDE;
			
				FUNCTION  TDialogTEView.WillingToResignTarget: LONGINT; OVERRIDE;

				PROCEDURE TDialogTEView.TargetValidationFailed(reason: LONGINT); OVERRIDE;

				PROCEDURE TDialogTEView.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields. }
				END;
				
		TFloatingTEManager = OBJECT(TObject)	{ Acts as a "server" for the "floating" 
												TDialogTEViews installed in EditTexts.
												A one-view cache is maintained }
				fCache: TDialogTEView;
				
				PROCEDURE TFloatingTEManager.Initialize; OVERRIDE;
				{ Puts the newly created TFloatingTEManager into a known state. Subclasses should
				override it and call INHERITED. Typically used to put the object into a state where
				it can be safely FREED. }
			
				PROCEDURE TFloatingTEManager.Free; OVERRIDE;
				{ Frees the cache, and calls INHERITED Free }
				
				PROCEDURE TFloatingTEManager.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields. }
				
				FUNCTION  TFloatingTEManager.AquireFloatingTE : TDialogTEView;
				{ Returns a "floating" TDialogTEView to install in an EditText. If the
				cache is empty, one is created }
				
				FUNCTION  TFloatingTEManager.MakeDialogTEView : TDialogTEView;
				
				PROCEDURE TFloatingTEManager.ReleaseFloatingTE(aDialogTEView: TDialogTEView);
				{ Places the specified TDialogTEView in the cache, or frees it if the
				cache is full }
				
		END;

		PROCEDURE InitUDialog;
		{ This routine must be called if UDialog is used so that all the necessary initialization can
		be performed. Sets the global flag 'gUDialogInitialized' after the initalization is
		performed. }

		FUNCTION AquireFloatingTEView : TDialogTEView;
		{ Called by EditTexts to obtain a "floating" TDialogTEView to install. Procedural
		interface to TFloatingTEManager.AquireFloatingTE }

		PROCEDURE ReleaseFloatingTEView(aDialogTEView: TDialogTEView);
		{ Called by EditTexts when deselecting. Procedural interface to 
		TFloatingTEManager.ReleaseFloatingTE }

		PROCEDURE MAParamText(keyStr, valueStr: Str255);
		{ Used to create paramTxt items for a dialog. Builds an association with keyStr
		and valueStr. All instances of keyStr in the dialog will be replaced with valueStr
		when the dialog is displayed. }

		PROCEDURE MAReplaceText(VAR theText: Str255);
		{ If there are any param text items in the theText passed to this method then they
		are replaced with the appropriate values. }

	VAR
		gFloatingTEManager: TFloatingTEManager;

END.
