{[a-,body+,h-,o=100,r+,rec+,t=4,u+,#+,j=20/57/1$,n+]}
{ UDocument.p }
{ Copyright  1984-1991 by Apple Computer Inc.	All rights reserved. }

UNIT UDocument;

	INTERFACE

		USES
			{}
			Types, MacAppTypes, Memory, UPascalObject, UObject, AppleEvents, UEvtHandler, UEvent, UCommand;

		CONST

			{ Constants for the print info resource }
			kPrintInfoRsrcType	= 'PRNT';
			kPrintInfoRsrcID	= 1;

		TYPE
			TDesignator 		= OBJECT; external;
			TFile				= OBJECT; external;
			TList				= OBJECT; external;
			TNoChangesCommand	= OBJECT; external;
			TPrintHandler		= OBJECT; external;
			TStream 			= OBJECT; external;
			TView				= OBJECT; external;
			TWindow 			= OBJECT; external;

			TDocument			= OBJECT (TEvtHandler)	{ Corresponds to a Finder document. Manages
														 document data in files and in main memory.
														 }
				fTitle: 			Str255; 			{ Title of the document }
				fWindowList:		TList;				{ list of windows belonging to this document
														 }
				fViewList:			TList;				{ list of views belonging to document }

				fChangeCount:		LONGINT;			{ master count of changes since last Save }
				fSavePrintInfo: 	BOOLEAN;			{ if TRUE for a document saved on disk, the
														 'print info' record of the
														 fDocPrintHandler will be written out to
														 the data fork before other writing takes
														 place }
				fSharePrintInfo:	BOOLEAN;			{ if TRUE, then all printHandlers associated
														 with views belonging to the same document
														 will share the same 'print info' record }
				fPrintInfo: 		Handle; 			{ if non-NIL, this is a handle to a 120-byte
														 'print info' record }

				fReopenAlert:		BOOLEAN;			{ whether to give an alert if user reopens
														 doc }
				fCommitOnSave:		BOOLEAN;			{ commit the last command before saving the
														 document, if it affected the document.
														 This defaults to TRUE, but carefully
														 written applications could set this FALSE.
														 }
				fUserSelection: 	TDesignator;		{ When there is a single user-selection for
														 a document this is it. }
								{ Init & Free }
				PROCEDURE TDocument.Initialize; OVERRIDE;
				{ Puts a newly created TDocument into a known state. Subclasses should
				override it and call INHERITED. Typically used to put the object into a state where
				it can be safely FREED. }

				PROCEDURE TDocument.IDocument;
				{ Initialization method for TDocument. }

				PROCEDURE TDocument.Free; OVERRIDE;
				{ This does not call FreeData by default, since you may need to control the order in
				which things are freed. Your override of TDocument.Free can call FreeData if
				convenient. }

				PROCEDURE TDocument.FreeData;
				{ Called when a document is reverted, in order to free it's data objects. You may
				also wish to call this from you document's Free method, if convenient. }

				PROCEDURE TDocument.FreeFromClipboard;
				{ Called to free a Clipboard document.Simply calls Free. }

				PROCEDURE TDocument.GetInspectorName(VAR inspectorName: Str255); OVERRIDE;
				{ Called by the Inspector to get the name of this class. }

				PROCEDURE TDocument.Fields(obj: TObject); OVERRIDE;
				{ Called by the Inspector and the Debugger to display the contents of this class's
				fields. }

				{ Opening / Printing Documents }
				FUNCTION TDocument.AlreadyOpen(aFile: TFile): BOOLEAN;
				{ TRUE if this document is already open }

				PROCEDURE TDocument.DoInitialState;
				{ Called for 'New', 'Revert' to blank, Open Tool. Default is to do nothing; You may
				need to do further initialization that you would not do in case of an Open }

				PROCEDURE TDocument.DoMakeWindows;
				{ Should take the views created by DoMakeViews and create windows and frames to
				display them. }

				PROCEDURE TDocument.DoMakeViews(forPrinting: BOOLEAN);
				{ Create all necessary views for this document (based on forPrinting flag) and store
				into fields of your document. The document will then be sent either DoMakeWindows
				or Print. }

				PROCEDURE TDocument.AttachPrintHandler(itsPrintHandler: TPrintHandler);
				{ Associates the print handler with this document }

				PROCEDURE TDocument.DetachPrintHandler(itsPrintHandler: TPrintHandler);
				{ dis-associates the print handler from this document }

				PROCEDURE TDocument.OpenAgain(itsCmdNumber: CmdNumber; openingDoc: TDocument);
				{ Called if the user tries to open the same document twice. openingDoc will be the
				document object being opened. Default brings SELF to the front and calls
				Failure(noErr, 0), which will abort opening the document. (The goal is to prevent
				the user from having 2 copies of the same document open, in which case they could
				make incompatible changes to both.) You might override this if you want to open a
				second window on the same document. In the ideal case, you should make both windows
				display exactly the same data, and changes made to one should be reflected in the
				other. You could also override this to create a read-only copy of the document. }

				FUNCTION TDocument.PoseSaveDialog: INTEGER;
				{ If the document has been changed (fChangeCount > 0), then the user is asked to save
				the document. PoseSaveDialog returns cancel, kYesButton or kNoButton, depending on
				the button chosen by the user. If the document has not been changed then kNoButton
				is returned. }

				PROCEDURE TDocument.ReadDocument(forPrinting: BOOLEAN);
				{ Called to read an existing file for display or printing. }

				PROCEDURE TDocument.ShowWindows;
				{ Called when opening the document initially; default is to call OpenWindow for all
				windows that have fFlags.openInitially TRUE }

				PROCEDURE TDocument.UntitledName(VAR noName: Str255);
				{ Called to supply a name for an untitled document. If this returns '' then the
				windows are not renamed from the value specified in their titles. }

				{ Saving Documents }
				FUNCTION TDocument.GetChangeCount: LONGINT;
				{ Gets the document change count }

				PROCEDURE TDocument.SetChangeCount(newChangeCount: LONGINT);
				{ Sets the document change count }

				PROCEDURE TDocument.SaveDocument(itsCmdNumber: CmdNumber);
				{ Try to save the document to disk }

				PROCEDURE TDocument.SaveAgain(itsCmdNumber: CmdNumber; savingDoc: TDocument);
				{ Called when the user tries to Save As or Save a Copy In and specifies the name of
				a document that is already opened. The default case calls Failure(errSaveAgain,
				msgSaveFailed) to display an error. (The goal is to prevent the user from having 2
				copies of the same document open, in which case they might make incompatible
				changes to both copies.) savingDoc is the document being saved }

				{ Closing Documents }
				PROCEDURE TDocument.Close;
				{ Close a document and free it. Signals Failure with error = noErr and message =
				msgCancelled if user cancels. ??? NOTE: Must never be called for a document related
				to a view in the Clipboard. }

				{ Revert }
				PROCEDURE TDocument.RevertDocument;
				{ IF fSaveExists is TRUE then reverts to the file saved on disk, otherwise merely
				resets the print handler and calls doInitialState. Calls FreeData to free any
				existing data and commits any currently active command. }

				PROCEDURE TDocument.Abandon;
				{ Called when a document is closed and changes are being abandoned.  For memory based
				documents the default behaviour of doing nothing is OK.  For disk based documents that
				change the original copy this is the place to override to put back the original
				contents if you still have it available somewhere (a scratch file?) }

				PROCEDURE TDocument.ShowReverted;
				{ Called after Revert to show brand new document; default tells each view to
				ShowReverted }

				{ === designator management methods === }
				FUNCTION TDocument.GetUserSelection: TDesignator;
				{ Returns the designator for the user selection (if any). }

				PROCEDURE TDocument.SetUserSelection(newSelection: TDesignator);
				{ Sets the designator for the user selection (if any). }

				PROCEDURE TDocument.UserSelectionChanged;
				{ Call this whenever the user's selection changes. Subclasses should implement the
				appropriate behavior for their corresponding user selection TDesignator. }

				PROCEDURE TDocument.RevealSelection(aSelection: TDesignator);
				{ OVERRIDE this; bring this designation into user's view }

				{ Miscellaneous }
				PROCEDURE TDocument.DoWriteData(aScrapType: OSType; aDesignator: TDesignator;
														  aStream: TStream);
				{ Given a designator and a stream, writes the designated data to the stream. }

				PROCEDURE TDocument.DoReadData(aScrapType: OSType; aDesignator: TDesignator;
														  aStream: TStream; length: LONGINT);
				{ Given a designator and a stream, read the designated data from the stream. }

				PROCEDURE TDocument.GetTitle(VAR aTitle: Str255);
				{ Returns the name of the fFile or '' if the file does not exist }

				PROCEDURE TDocument.SetTitle(aTitle: Str255);
				{ Does fFile.SetName to aTitle and calls SetTitleForDoc for each window of the
				document. }

				PROCEDURE TDocument.Changed(theChange: ChangeID; changedBy: TObject); OVERRIDE;
				{ Updates fChangeCount. Calls INHERITED Changed to notify dependents }

				{ Command Handlers }
				PROCEDURE TDocument.DoMenuCommand(aCmdNumber: CmdNumber); OVERRIDE;
				{ Handles the default menu commands associated with a document and must be overridden
				to handle any additional items. }

				PROCEDURE TDocument.DoSetupMenus; OVERRIDE;
				{ Setup the documents menus, this handles the default menus and must be overridden
				for any additional items. }

				{ Hierarchy }
				PROCEDURE TDocument.AddView(aView: TView);
				{ Adds the view to the end of the document's view list. }

				PROCEDURE TDocument.AddWindow(aWindow: TWindow);
				{ Adds the window to the end of the document's window list. }

				PROCEDURE TDocument.CloseView(aView: TView);
				{ Called to close a view or window associated with a document.
				Document gets to decide whether to really close the view or not and whether to
				close itself or not.  Does nothing if the view is not associated with the doc. }

				PROCEDURE TDocument.DeleteView(viewToDelete: TView);
				{ Deletes the view from the document's view list. }

				PROCEDURE TDocument.DeleteWindow(windowToDelete: TWindow);
				{ Deletes the window from the document's window list. }

				PROCEDURE TDocument.ForAllViewsDo(PROCEDURE
												  DoToView(aView: TView));
				{ Performs DoToView on every view in every window belonging to the document. If you
				want to deal with all views belonging to the document use fViewList.Each. }

				PROCEDURE TDocument.ForAllWindowsDo(PROCEDURE
													DoToWind(aWindow: TWindow));
				{ Perform the supplied proc to each window belonging to the document }

				END;

{--------------------------------------------------------------------------------------------------}
			TSaveDocCommand 	= OBJECT (TNoChangesCommand) { Tells the document to save. }

				fChangedDocument:	TDocument;			{ the document changed by this command }

				PROCEDURE TSaveDocCommand.Initialize; OVERRIDE;
				{ Puts a newly created TSaveDocCommand into a known state. Subclasses should
				override it and call INHERITED. Typically used to put the object into a state where
				it can be safely FREED. }

				PROCEDURE TSaveDocCommand.ISaveDocCommand(itsCmdNumber: CmdNumber;
														  itsDocument: TDocument);
				{ Initialize the SaveDocCommand procedurally. }

				PROCEDURE TSaveDocCommand.DoIt; OVERRIDE;
				{ tell the document to save. }

				PROCEDURE TSaveDocCommand.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields. }

				END;

{--------------------------------------------------------------------------------------------------}
			TRevertDocCommand	= OBJECT (TNoChangesCommand) { Tells the document to revert after
															  checking with the user }

				fChangedDocument:	TDocument;			{ the document changed by this command }

				PROCEDURE TRevertDocCommand.Initialize; OVERRIDE;
				{ Puts a newly created TRevertDocCommand into a known state. Subclasses should
				override it and call INHERITED. Typically used to put the object into a state where
				it can be safely FREED. }

				PROCEDURE TRevertDocCommand.IRevertDocCommand(itsCmdNumber: CmdNumber;
															  itsDocument: TDocument);
				{ Initialize the RevertDocCommand procedurally. }

				PROCEDURE TRevertDocCommand.DoIt; OVERRIDE;
				{ tell the document to revert if the user says OK. }

				PROCEDURE TRevertDocCommand.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields. }

				END;

		VAR
			{$Push} 									{$J+}
			gNumUntitled:		INTEGER;				{ The number to assign to the next Untitled
														 document (assuming that the app provides a
														 template for filling in the number. You
														 must change STR# resource with ID 0 /
														 index 3 to read (for example)
														 Untitled-<<<#>>>. }
			{$Pop}
END.
