{[a-,body+,h-,o=100,r+,rec+,t=4,u+,#+,j=20/57/1$,n+]}
{ UEditionDocument.p }
{ Copyright  1984-1991 by Apple Computer Inc.	All rights reserved. }

UNIT UEditionDocument;

	INTERFACE

		USES
			{}
			Types, MacAppTypes, Memory, UPascalObject, UObject, AppleEvents, UEvtHandler, UEvent, UCommand, UDocument,
			UFileBasedDocument, Editions, UList, UAdorners;

		CONST
			{---------------- EDITION MANAGER MENU COMMAND CONSTANTS ----------------}
			cCreatePublisher	= 601;					{ the user has selected "Create Publisher"
														 from the Edit menu }
			cSubscribeTo		= 602;					{ the user has selected Subscribe to from
														 the Edit menu }
			cOptions			= 603;					{ the user has selected a publisher in the
														 document and then selected Publisher
														 Options from the Edit menu, OR, the user
														 has selected a subscriber in the document
														 and then selected Subscriber Options
														 from the Edit menu }
			cBorders			= 604;					{ the user has selected Show/Hide Borders
														 from the Edit menu }
			cStopAllEditions	= 605;					{ the user has selected Stop all Editions
														 from the Edit menu }

			{---------------- EDITION MANAGER COMMAND CONSTANTS ----------------}
			cCancelPublisher	= 606;					{ the user canceled this publisher }
			cCancelSubscriber	= 607;					{ the user canceled this subscriber }

		TYPE
			TDesignator 		= OBJECT; external;
			TFile				= OBJECT; external;
			TList				= OBJECT; external;
			TPublisher			= OBJECT; external;
			TSection			= OBJECT; external;
			TSubscriber 		= OBJECT; external;
			TView		 		= OBJECT; external;

			TEditionDocument	= OBJECT (TFileBasedDocument)

				fSectionList:		TList;				{ list of TSection objects }

				fStopAllEditions:	BOOLEAN;			{ FALSE by default }

				fShowSectionBorders: BOOLEAN;			{ TRUE by default }

				fNewPublishers: 	BOOLEAN;			{ are there any new publishers? }

				fNextEditionNumber: INTEGER;			{ number used in "Untitled-n" }

				fEditionCreator:	OSType; 			{ creator for publisher edition container
														 file }

				{ === constructor/destructor methods === }
				PROCEDURE TEditionDocument.Initialize; OVERRIDE;

				PROCEDURE TEditionDocument.IEditionDocument(itsFile: TFile;
															itsFileType, itsCreator: OSType;
															itsScrapType: OSType;
															usesDataFork, usesRsrcFork: BOOLEAN;
															keepsDataOpen, keepsRsrcOpen: BOOLEAN;
															editionCreator: OSType);
				{ initializes this object }

				PROCEDURE TEditionDocument.Free; OVERRIDE;
				{ frees this object }

				PROCEDURE TEditionDocument.FreeData; OVERRIDE;
				{ frees the fSectionList }

				{ === inspecting === }
				PROCEDURE TEditionDocument.Fields(obj: TObject); OVERRIDE;

				{ === saving documents === }
				PROCEDURE TEditionDocument.Close; OVERRIDE;
				{ For the case in which the document's change count = 0, but there are "dirty"
				sections, need to ask the user whether to save the document. }

				FUNCTION TEditionDocument.PoseNewPublishersAlert: INTEGER;
				{ Warns the user that he is about to close a document that hasn't been saved in which
				he has created a new publisher. The user hasn't modified the document's data. }

				FUNCTION TEditionDocument.PoseMultPublishersAlert(aSection: TSection): INTEGER;
				{ Warns the user that he is creating multiple publishers to the same edition. }

				FUNCTION TEditionDocument.PoseSavingMultPublishersAlert(aSection: TSection): INTEGER;
				{ Warns the user that he is saving multiple publishers to the same edition. }

				PROCEDURE TEditionDocument.Abandon; OVERRIDE;
				{ Calls TSection.Delete on all dirty sections. }

				PROCEDURE TEditionDocument.DoNeedDiskSpace(itsFile: TFile;
															VAR dataForkBytes, rsrcForkBytes: LONGINT); OVERRIDE;
				{ Bytes required to store SAVE file(s) on disk. When called, both parameters are set
				to 0 for you. }

				PROCEDURE TEditionDocument.DoWrite(aFile: TFile; makingCopy: BOOLEAN); OVERRIDE;
				{ writes out the sections and edition doc settings }

				PROCEDURE TEditionDocument.DoWriteSettings(aFile: TFile);
				{ writes out the edition doc settings (Show/Hide Border, Stop All Editions) }

				PROCEDURE TEditionDocument.DoRead(aFile: TFile; forPrinting: BOOLEAN); OVERRIDE;
				{ reads in the sections and edition doc settings }

				PROCEDURE TEditionDocument.DoReadSettings(aFile: TFile);
				{ reads in the edition doc settings (Show/Hide Border, Stop All Editions) }

				PROCEDURE TEditionDocument.DoReadPublisher(aFile: TFile; theID: INTEGER);
				{ reads in a given publisher }

				PROCEDURE TEditionDocument.DoReadSubscriber(aFile: TFile; theID: INTEGER);
				{ reads in a given subscriber }

				FUNCTION TEditionDocument.IsAnySectionDirty: BOOLEAN;
				{ returns TRUE if any section is dirty }

				{ === accessors === }
				PROCEDURE TEditionDocument.SetSectionBorders(state, invalidate: BOOLEAN);
				{ sets the value of fShowSectionBorders to state and invalidates the document's windows
				if "invalidate" is TRUE }

				PROCEDURE TEditionDocument.SetStopAllEditions(state: BOOLEAN);
				{ sets the value of fStopAllEditions to state and reads in any new subscribers if
				"state" is TRUE }

				FUNCTION TEditionDocument.GetEditionCreatorSignature: OSType;
				{ returns the creator signature used for the edition container file }

				{ === menu management methods === }
				PROCEDURE TEditionDocument.DoMenuCommand(aCmdNumber: CmdNumber); OVERRIDE;
				PROCEDURE TEditionDocument.DoSetupMenus; OVERRIDE;
				FUNCTION TEditionDocument.CanPublishSelection: BOOLEAN;
				FUNCTION TEditionDocument.CanSubscribe: BOOLEAN;

				{ === edition manager support === }
				PROCEDURE TEditionDocument.DoNewPublisher;
				FUNCTION TEditionDocument.DoMakePublisher(itsDesignator: TDesignator;
														  itsSectionHandle: SectionHandle;
														  itsSectionID: INTEGER): TPublisher;
				PROCEDURE TEditionDocument.DoNewPublisherDialog(VAR reply: NewPublisherReply);

				PROCEDURE TEditionDocument.DoMakePreview(itsDesignator: TDesignator;
														 VAR previewFormat: FormatType;
														 VAR preview: Handle);

				PROCEDURE TEditionDocument.DoNewSubscriber;
				FUNCTION TEditionDocument.DoMakeSubscriber(itsDesignator: TDesignator;
														   itsSectionHandle: SectionHandle;
														   itsSectionID: INTEGER): TSubscriber;
				PROCEDURE TEditionDocument.DoNewSubscriberDialog(VAR reply: NewSubscriberReply);

				PROCEDURE TEditionDocument.DoSectionOptions;
				{ poses the publisher or subscriber options dialog depending on what GetSelectedSection returns }

				FUNCTION TEditionDocument.GetPublishPreference: FormatType;
				{ return 'PICT' to publish 'PICT' first, return 'TEXT' to publish 'TEXT' first }
				FUNCTION TEditionDocument.GetSubscriberFormatsMask: SignedByte;
				PROCEDURE TEditionDocument.GetNextEditionName(VAR editionName: Str63);
				PROCEDURE TEditionDocument.DoPublisherOptions(aPublisher: TPublisher);
				PROCEDURE TEditionDocument.DoPublisherOptionsDialog(VAR reply: SectionOptionsReply);
				PROCEDURE TEditionDocument.DoSubscriberOptions(aSubscriber: TSubscriber);
				PROCEDURE TEditionDocument.DoSubscriberOptionsDialog(VAR reply: SectionOptionsReply);

				{ === section management === }
				PROCEDURE TEditionDocument.AddSection(aSection: TSection);
				{ call aSection.Register; this method should mark document as changed so that when saving
				the document, the user saves this section }

				PROCEDURE TEditionDocument.CancelSection(aSection: TSection; cancel: BOOLEAN);
				{ calls aSection.CancelSection(cancel); }

				PROCEDURE TEditionDocument.EachCanceledSectionDo(PROCEDURE
																 DoToCanceledSection(aSection:
																					 TSection));
				{ iterates over section list calling DoToCanceledSection on each canceled section }

				PROCEDURE TEditionDocument.EachPublisherDo(PROCEDURE
														   DoToPublisher(aPublisher: TPublisher));
				{ iterates over section list calling DoToPublisher on each publisher object, filtering
				out canceled sections }

				PROCEDURE TEditionDocument.EachSectionDo(PROCEDURE
														 DoToSection(aSection: TSection));
				{ iterates over section list calling DoToSection on each object, filtering out canceled
				sections }

				PROCEDURE TEditionDocument.EachSubscriberDo(PROCEDURE
															DoToSubscriber(aSubscriber: TSubscriber)
															);
				{ iterates over section list calling DoToSubscriber on each subscriber object, filtering
				out canceled sections }

				FUNCTION TEditionDocument.FirstSectionThat(FUNCTION
														   TestSection(aSection: TSection): BOOLEAN)
														   : TSection;
				{ Call TestSection on each section, until it returns TRUE. }

				FUNCTION TEditionDocument.GetSectionObject(aSectionHandle: SectionHandle): TSection;
				{ maps a SectionHandle to a TSection object }

				FUNCTION TEditionDocument.GetSelectedSection: TSection;
				{ This returns the currently selected section, if any. Returns NIL if > 1 section
				is currently selected. }

				FUNCTION TEditionDocument.GetUniqueSectRsrcID: INTEGER;
		  		{ returns a unique 'sect' resource id for use as both the section ID and as the rsrc ID }

				FUNCTION TEditionDocument.IsBorderShown(aSection: TSection): BOOLEAN;
				{ returns TRUE if (borders are shown) or (the user selection contains aSection) }

				FUNCTION TEditionDocument.IsSectionSelected(aSection: TSection): BOOLEAN;
		  		{ returns TRUE if the given section is in the user selection }

				PROCEDURE TEditionDocument.RemoveSection(aSection: TSection);
				{ this method should mark document as changed so that document removes this section when
				saved (e.g. sipAlways) }

				END;

			{---------------- EDITION Document MENU COMMAND CLASSES ----------------}

			TCancelPublisherCmd = OBJECT (TCommand)
			{ TCancelPublisherCmd is generated when the user selects Cancel Publisher from the
			Publisher options dialog [this command should be undoable in case the user changes his
			mind]. This command cancels a publisher. }
			{ Note: keep the publisher *data* intact }

				fPublisher: 		TPublisher; 		{ the publisher that will be canceled }

				PROCEDURE TCancelPublisherCmd.ICancelPublisherCmd(itsCmdNumber: CmdNumber;
																  itsDocument: TDocument;
																  itsPublisher: TPublisher);
				{ initialize this command }

				PROCEDURE TCancelPublisherCmd.DoIt; OVERRIDE;
				{ call fDocument.CancelPublisher(fPublisher, kCancel) }

				PROCEDURE TCancelPublisherCmd.UndoIt; OVERRIDE;
				{ call fDocument.CancelPublisher(fPublisher, kUnCancel) }
				END;

{--------------------------------------------------------------------------------------------------}
			TCancelSubscriberCmd = OBJECT (TCommand)
			{ TCancelSubscriberCmd is generated when the user selects Cancel Subscriber from the
			Subscriber options dialog [this command should be undoable in case the user changes his
			mind]. This command cancels a subscriber. }
			{ Note: keep the subscriber *data* intact in the document }

				fSubscriber:		TSubscriber;		{ the subscriber that will be canceled }

				PROCEDURE TCancelSubscriberCmd.ICancelSubscriberCmd(itsCmdNumber: CmdNumber;
																	itsDocument: TDocument;
																	itsView: TView;
																	itsSubscriber: TSubscriber);
				{ initialize this command }

				PROCEDURE TCancelSubscriberCmd.DoIt; OVERRIDE;
				{ call fDocument.CancelSubscriber(fSubscriber, kCancel) }

				PROCEDURE TCancelSubscriberCmd.UndoIt; OVERRIDE;
				{ call fDocument.CancelSubscriber(fSubscriber, kUnCancel) }
				END;

{--------------------------------------------------------------------------------------------------}
			TSectionBorderCmd	= OBJECT (TCommand)
			{ TSectionBorderCmd is an undoable command which changes the setting for the
			show/hide border option for the current document when the user chooses Hide/Show Borders
			from the Edit menu }

				fShowSectionBorders: BOOLEAN;			{ value to set }

				PROCEDURE TSectionBorderCmd.ISectionBorderCmd(itsCmdNumber: CmdNumber;
															  itsDocument: TDocument);
				{ initialize this command }

				PROCEDURE TSectionBorderCmd.DoIt; OVERRIDE;
				{ tell the document to show or hide the borders for all sections }

				PROCEDURE TSectionBorderCmd.RedoIt; OVERRIDE;
				{ call DoIt }

				PROCEDURE TSectionBorderCmd.UndoIt; OVERRIDE;
				{ tell the document to undo show or hide the borders for all sections }
				END;

{--------------------------------------------------------------------------------------------------}
			TStopAllEditionsCmd = OBJECT (TCommand)
			{ TStopAllEditionsCmd is an undoable command gets posted when the user chooses
			Stop all Editions from the Edit menu. This command toggles the setting for the
			documents fStopAllEditions field. }

				fStopAllEditions:	BOOLEAN;			{ value to set }

				PROCEDURE TStopAllEditionsCmd.IStopAllEditionsCmd(itsCmdNumber: CmdNumber;
																  itsDocument: TDocument);
				{ initialize this command }

				PROCEDURE TStopAllEditionsCmd.DoIt; OVERRIDE;
				{ tell the document to toggle the state of its fStopAllEditions flag }

				PROCEDURE TStopAllEditionsCmd.RedoIt; OVERRIDE;
				{ tell the document to toggle the state of its fStopAllEditions flag }

				PROCEDURE TStopAllEditionsCmd.UndoIt; OVERRIDE;
				{ tell the document to toggle the state of its fStopAllEditions flag }
				END;

{--------------------------------------------------------------------------------------------------}

			TSectionAdorner 	= OBJECT (TAdorner)
			{ abstract superclass for section adorners }

				fBorderRgn: 		RgnHandle;					{ the region for the section border }
				
				fSection:			TSection;					{ the section this adorner adorns }

				PROCEDURE TSectionAdorner.Initialize; OVERRIDE;
				{ sets fBorderRgn & fSection to NIL }

				PROCEDURE TSectionAdorner.ISectionAdorner(itsID: IDType;
														  itsSection: TSection;
														  itsBorderRgn: RgnHandle);
				{ initializes the section adorner with priority kLowAdornPriority }

				PROCEDURE TSectionAdorner.Free; OVERRIDE;
				{ frees fBorderRgn }

				PROCEDURE TSectionAdorner.InvalidateAdorner(itsView: TView); OVERRIDE;
				{ Invalidates the adorner in the TAdornerMgr' TView. }

				PROCEDURE TSectionAdorner.DrawBorder(whichPattern: Pattern);
				{ Draws the adorner's border in the TAdornerMgr' TView. }

				END;

{--------------------------------------------------------------------------------------------------}

			TSubscriberAdorner	= OBJECT (TSectionAdorner)
			{ A specialized adorner for imaging subscriber borders. }

				PROCEDURE TSubscriberAdorner.ISubscriberAdorner(itsID: IDType;
														  		itsSubscriber: TSubscriber;
																itsBorderRgn: RgnHandle);
				{ initializes the subscriber adorner }

				PROCEDURE TSubscriberAdorner.DoHighlightSelection(itsView: TView; area: VRect; fromHL, toHL: HLState); OVERRIDE;
				{ Adorns the selection before the view's DoHighlightSelection. }

				END;

{--------------------------------------------------------------------------------------------------}

			TPublisherAdorner	= OBJECT (TSectionAdorner)

				PROCEDURE TPublisherAdorner.IPublisherAdorner(itsID: IDType;
														  	  itsPublisher: TPublisher;
															  itsBorderRgn: RgnHandle);
				{ initializes the publisher adorner }

				PROCEDURE TPublisherAdorner.DoHighlightSelection(itsView: TView; area: VRect; fromHL, toHL: HLState); OVERRIDE;
				{ Adorns the selection before the view's DoHighlightSelection. }

				END;

		   {-----------------------------GLOBAL ROUTINES-------------------------------------------}
		PROCEDURE InitUEditionDocument;
		{ call this routine to initialize this unit. }
END.
