{[a-,body+,h-,o=100,r+,rec+,t=4,u+,#+,j=20/57/1$,n-]}
{ UErrorMgr.p }
{ Copyright  1990-1991 by Apple Computer Inc.	All rights reserved. }

{[f-]}
(*
Theory courtesy of:
	Larry Goldman
	All Star Computer Services
	AppleLink: ALLSTAR
	Voice: (415) 282-0540


Using Error Message Tables

Part of MacApp's powerful failure handling facilities is the ability to give the user useful
alert messages. The MacApp Cookbook already explains how to use the common features of the
error reporting mechanism. But the msgLookup option is not explained.  Actually, this facility
is not difficult to use, and allows the program to report application-specific errors in the
same way as other machine errors.

Background

To use the failure handling mechanism for custom errors, an exception handler must first be
posted using the CatchFailures routine.  If an error is encountered, the Failure routine must
be called with application-defined parameter values. If, instead, the routine completes without
error, the exception handler is popped off the exception handler stack with the Success
routine.

The Failure routine takes two parameters: error, which is an INTEGER, and message, which is a
LONGINT. During exception processing, these parameters are passed eventually to the ErrorAlert
global routine. ErrorAlert decodes the message parameter to determine what alert to display and
what subsititions need to be made.

If the high word of the message is either msgLookup or msgAltRecovery, and if the message table
lookup is successful, then ErrorAlert will display MacApp's generic error alert, which displays
the alert message in the form:

	Could not ^2, because ^0. ^1.

The first substitution (^2) is called the "operation", or  opString, the ^0 substitution is
called the "reason", or errString, and the ^1 substitution is called the recovery.

Message Tables

If the high word of the message is msgLookup or msgAltRecovery, ErrorAlert uses message tables,
located in the application's resource file, to determine the text of each of the substitutions
that will be displayed in the "generic" error alert.

In both message cases, the low word of the message parameter is used to lookup the opString,
and the error parameter is used to lookup the errString. In the msgLookup case, the error
parameter is also used to determine the recovery. In the msgAltRecovery case, the low word of
the message parameter is used, instead, to determine recovery.

ErrorAlert uses the global procedure LookupErrString to search message tables for the text to
substitute into the alert.	LookupErrString first scans the MacApp message tables, and if the
message cannot be found, then application message tables are scanned.

Each message table has two parts in the resource file: an 'errs' resource which contains the
list of index ranges, and a 'STR#' list of messages that correspond to each index range. This
allows a single text message to cover a range of indexes, if necessary.

The index ranges are specified as an array of three integers: the first is the lowest index to
which the message applies, the next is the highest index to which the message applies, and the
last integer is the item in the string list that contains the text of the message to use.  The
first line of the index table, the one that begins with the whichList constant,  is used to
specifiy the resource number of the STR# list to use for this index table.

Here is a sample message index table:

resource 'errs' (1130, purgeable) { // application's operation table 
	{	whichList, 0, 2103; // indicates that STR# 2103 will be used by this table 
		-26001, -26001, 1;	// lowIndex, hiIndex, STR# item 
		-26002, -26002, 2;
		-26003, -26003, 3
	}
};

Here is the corresponding STR# list:

resource 'STR#' (2103, purgeable) { // application's operation strings 
	{
		"use the saved configuration";
		"add another character";
		"accept the number"
	}
};

The application's resource file normally contains three sets of message tables: one each for
the operation, error and recovery strings.	The resource numbers for the tables are fixed: the
application's operation table must have number 1130, the reason table must have number 1128 and
the recovery table must have number 1129.

================================================================================================
Recipe

To use application message tables:

1) Define constants for the application's message and error:

CONST
	msgCantAcceptChar	= msgLookup + -26002;
	{Note: Applications should use errors between -26000 and -29999.}
	errCantAddChar		= -26002;

Adding the predefined constant msgLookup to the message signals ErrorAlert to use the message
tables to find text to substitute into the generic alert.

2) Define the message tables in the application's resource file:

First, the operation table and its STR# list:

resource 'errs' (1130, purgeable) { // application's operation table 
	{	whichList, 0, 2103, // indicates that STR# 2103 will be used by this table 
		-26002, -26002, 1
	}
};

resource 'STR#' (2103, purgeable) { // application's operation strings 
	{
		"add another character"
	}
};

Next, the error  table and its STR# list:

resource 'errs' (1128, purgeable) { // application's reason table 
	{	whichList, 0, 2101;
		-26002, -26002, 1
	}
};


resource 'STR#' (2101, purgeable) { // application's error reason strings 
	{
		"only ^3 characters are allowed"
	}
};

Note that ^3 can be used in any message string. ErrorAlert always subsitutes the text in the
global variable gErrorParm3 for ^3, if present, in the message strings. Thus, gErrorParm3 can
be used by the application to further customize the alert according to the context of the
program at the time the error was encountered.

Finally, the recovery table and its STR# list:

resource 'errs' (1129, purgeable) { // application's recovery table 
	{	whichList, 0, 2102;
		-26002, -26002, 1
	}
};

resource 'STR#' (2102, purgeable) { // application's error recovery strings 
	{
		"Delete some characters before adding new ones."
	}
};

3) If, in the course of executing the program, an error is encountered, call Failure:

PROCEDURE TMyEditText.AddCharacter(ch: CHAR);
VAR
	fi	: FailInfo;

	PROCEDURE HdlFailure(error: INTEGER; message: LONGINT);
	BEGIN
		IF error = errCantAddChar THEN {if the error is recognized, fill in the param}
			NumToString(fMaxCharsAllowed, gErrorParm3);

		{Note: the error will be propagated automatically to exception handlers already
		posted on the exception handler stack.	Eventually, ErrorAlert will be called to
		display the message with the text located in the application's message tables.}

	END;

BEGIN
	CatchFailures(fi, HdlFailure);		{Setup an exception handler}
	IF NumOfChars = fMaxCharsAllowed THEN	{Assume fMaxCharsAllowed = 15, for
example}
		Failure(errCantAddChar, msgCantAcceptChar){Can't add a char, so signal failure.}
	ELSE
		INHERITED AddCharacter(ch); 	{Can accept char}
	Success(fi);					{Done with exception handler.}
END;

4) If the character could not be added, then the generic alert will appear on the screen with
the text:

	Could not add another character, because only 15 are allowed. Delete some characters
	before adding new ones.
*)
{[f+]}

UNIT UErrorMgr;

	INTERFACE

		USES
			{}
			Types, MacAppTypes,
			{}
			Dialogs;

		CONST
			{ A L E R T S }

			errReasonID 		= 128;					{ resource ID of errs resource describing
														 error messages }
			errRecoveryID		= 129;					{ resource ID of errs resource describing
														 recovery messages }
			errOperationsID 	= 130;					{ resource ID of errs resource containing
														 operation strings }
			errAppTable 		= 1000; 				{ added to the MacApp resource ID to get the
														 ID of an application error table }

			msgCmdErr			= 0;					{ when added to a command number, gives a
														 message value to display the alert "Could
														 not complete the  command " }
			msgAlert			= $FFFF0000;			{ when added to an alert number, gives a
														 message value to display that alert }
			msgLookup			= $FFFE0000;			{ when added to an INTEGER, gives a message
														 value that will look up the INTEGER in
														 table errOperationsID and display the
														 alert "Could not  because " }
			msgAltRecovery		= $FFFD0000;			{ same as msgLookup, except the INTEGER is
														 used instead of the error number to lookup
														 the recovery string }

			msgStrList			= 130 * $10000; 		{ Standard list of operation strings used by
														 MacApp * $10000 to plunk it into hiwrd }
			msgCancelled		= msgStrList;			{ used with error = noErr if user cancels
														 Save, Quit,  }
			msgInitFailed		= msgStrList + 1;
			msgSaveFailed		= msgStrList + 2;
			msgRevertFailed 	= msgStrList + 3;
			msgPrintFailed		= msgStrList + 4;
			msgNewFailed		= msgStrList + 5;
			msgOpenFailed		= msgStrList + 6;
			msgSaveAsFailed 	= msgStrList + 7;
			msgSaveCopyFailed	= msgStrList + 8;
			msgDrawFailed		= msgStrList + 9;
			msgImportClipFailed = msgStrList + 10;
			msgExportClipFailed = msgStrList + 11;

			{ error codes used in MacApp; applications should use errors between
			-25000 and -29999 }

			errSpooling 		= - 20000;				{ print spooling failed OSErr }
			errRevertFNF		= - 20001;				{ Revert failed because file was deleted }
			errFileChanged		= - 20002;				{ disk file modification date changed }
			errSaveAgain		= - 20003;				{ Save As / Save a Copy in an opened
														 document }
			errFTypeChanged 	= - 20004;				{ disk file type changed }
			errNoPrintDrvr		= - 20005;				{ print driver file not found }
			errNotMyType		= - 20006;				{ can't open this file type }
			errNotImplemented	= - 20007;				{ Signaled when a function is not yet
														 implemented }
			errMissingClass 	= - 20008;				{ Signaled when a class is missing. Usually
														 due to failure to dead-strip suppress when
														 creating objects any other way than by
														 "NEW" }
			errMissingSignature = - 20009;				{ Signaled when a signature has no associated class }

			{ Alerts 0 - 249 are reserved by MacApp. }

			phGenError			= 128;					{ 'Could not ^2, because ^0. ^1' }
			phCmdErr			= 129;					{ 'Could not complete the ^2 command
														 because ^0. ^1'. }
			phUnknownErr		= 130;					{ 'Could not complete your request because
														 ^0. ^1'; only used if no operation is
														 supplied. }
			phSaveChanges		= 131;					{ 'Save changes to the ^2 document ^0 before ^1?' }
			phRevert			= 132;					{ 'Revert to the last saved version of ^0?' }
			phFileChanged		= 133;					{ 'File changed since last save.' }
			phPurgeOld			= 134;					{ 'OK to purge old version before saving new
														 one?' }
			phReopenDoc 		= 135;					{ 'Document is already open.' }
			phSpaceIsLow		= 136;					{ 'Space is low' }
			phUnsupportedConfiguration = 137;			{ 'Can't start app because your machine is
														 too wimpy!' }
			phOfferReadOnly 	= 138;					{ 'Can't open for file for write. Open
														 read-only?' }

			phNewPublisherAlert	= 140;					{ Used by TEditionDocument when closing the
														document if there are new publishers but the
														document's DATA is unchanged. }

			phMultPublisherWrn	= 141;					{ Used by TEditionDocument when creating
														multiple publishers to the same
														edition container file. }

			phSavingMultPublisherWrn = 142;				{ Used by TEditionDocument when closing the
														document if there are multiple publishers to
														the same edition container file. }

			phTooManyChars		= 150;					{ 'Too Many Characters', used to reject
														 paste or keystrokes in UDialog and UTEView
														 }

			phStylesTooBig		= 151;					{ 'Too Many Styles', used to reject the
														 styles portion of a paste in UTEView }

			phAboutApp			= 201;					{ Can hold the 'About <appName>' message }

			phUnimplemented 	= 202;					{ 'This feature not yet implemented' }

		VAR
			{$Push} 									{$J+}
			gErrorParm3:		Str255; 				{ This is used as the last argument to
														 ParamText in ErrorAlert, if ErrorAlert
														 displays one of the standard alerts. (This
														 string will replace ^3 in those alerts.)
														 ErrorAlert also sets this to '' when
														 called. You can use this to parameterize
														 the automatic alerts that MacApp displays.
														 For example, TDocument.ReadFromFile sets
														 this to the document name. }
			gMacAppAlertFilter: ProcPtr;				{ the default filter proc. Will be
														 automatically set to point to
														 MacAppAlertFilter. Set this to nil in your
														 code if you don't want it or call it from
														 your own filter proc if you just want to
														 add behavior. Someday we will fix all this
														 to be Object based. }
			gInFilter:			BOOLEAN;				{ Flag that is set when we are in the
														 MacAppAlertFilter, used to indicate if we
														 should re-enter MacApp to handle updates
														 or not }
			gInhibitNestedHandling: BOOLEAN;			{ TRUE if Failure was executed and the
														 outer- most event loop has not been
														 reached or the ErrorAlert routing has not
														 yet shown the failure to the user. Also
														 may be set to TRUE to inhibit continued
														 operation of MacApp Activate/Update
														 handling and idling during MacAppAlerts. }
			{$Pop}

		PROCEDURE ErrorAlert(err: OSerr;
							 message: LONGINT);
		{ Displays an error alert box. The message displayed in the alert is constructed base upon
		err and message, as described in the "Failure Handling" section of the "Cookbook" chapter
		of the MacApp manual. }

		FUNCTION LookupErrString(value: INTEGER;
								 resourceID: INTEGER;
								 VAR str: Str255): BOOLEAN;
		{ Looks up an error value in an 'errs' resource and returns TRUE if found. resourceID should
		be that of a MacApp errs resource; we do the lookup first in errAppTable+resourceID, which
		applications can use to override or extend the MacApp table. If the value is not found, str
		will be empty. }

		FUNCTION MacAppAlert(alertID: INTEGER;
							 filterProc: ProcPtr): INTEGER;
		{ Does SetCursor(arrow) before calling Alert. }

		PROCEDURE StdAlert(alertID: INTEGER);
		{ no filterProc, reply ignored }

		FUNCTION CallAlertFilter(theDialog: DialogPtr;
								 VAR theEvent: EventRecord;
								 VAR itemHit: INTEGER;
								 theAlertFilter: ProcPtr): BOOLEAN;
			INLINE $205F,								{ MOVE.L (A7)+,A0 }
				   $4E90;								{ JSR (A0) }
		{ Lets you call the alert filter specified in theAlertFilter.  Useful for providing
		your own alert filter behaviour and then passing on to MacApp's }

		PROCEDURE NotYetImplemented(where: Str255);
		{ Signals the notYetImplemented failure which when caught puts up an alert saying
		"Not Yet Implemented" and in debug WriteLns where. }
END.
