{[a-,body+,h-,o=100,r+,rec+,t=4,u+,#+,j=20/57/1$,n-]}
{ UEvtHandler.p }
{ Copyright  1984-1991 by Apple Computer Inc.	All rights reserved. }

UNIT UEvtHandler;

	INTERFACE

		USES
			{}
			Types, MacAppTypes, Memory, UPascalObject, UObject, AppleEvents, UBehavior;

		TYPE

			IdlePhase			= (idleBegin, idleContinue, idleEnd);

			{ E V E N T S }

			TCommand			= OBJECT; EXTERNAL;
			TDocument			= OBJECT; EXTERNAL;
			TView				= OBJECT; EXTERNAL;
			TEvent				= OBJECT; EXTERNAL;
			TToolboxEvent		= OBJECT; EXTERNAL;
			TBehavior			= OBJECT; EXTERNAL;

			TEvtHandler 		= OBJECT (TObject)		{ The TEvtHandler type represents abstract
														 objects that handle certain kinds of
														 events: Key events: both key down and auto
														 key events, menu events: both enabling
														 menus and menu items and processing menu
														 commands, Read to/write from Disk, Idle
														 events: when there are no other events to
														 handle, Actual events: EvtHandlers -
														 through their method 'DoHandleEvent' may
														 intercept and handle actual ToolBox events
														 Termination: when the application quits
														 TEvtHandler are linked into a list with
														 the most specific object (usually a
														 selection) at the head of the list.
														 'TApplication.fTarget' contains the head
														 of the list. (When a window is
														 deactivated, this global variable is
														 cached in the window object, and retrieved
														 when it is later activated.) For these
														 kinds of events, the TApplication.fTarget
														 gets the first crack at handling the
														 event. The default implementation of the
														 methods of TEvtHandler is to pass the
														 event to the next element of the list. }

				fNextHandler:		TEvtHandler;		{ the next element of the list, or NIL }
				fBehavior:			TBehavior;			{ Attachment point for behaviors }
				fIdleFreq:			LONGINT;			{ the minimum number of ticks that can pass
														 before my DoIdle is called. 0 = call as
														 often as possible. n..kMaxIdleTime = call
														 every fIdleFreq ticks. }
				fLastIdle:			LONGINT;			{ the tick count the last time my DoIdle was
														 called. }

				fEnabled:			BOOLEAN;			{ is this eventHandler enabled. (does it
														 take clicks/keystrokes and other events? }
				fIdentifier:		IDType; 			{ the ID if any }

								{ Init & Free }
				PROCEDURE TEvtHandler.Initialize; OVERRIDE;
				{ Puts a newly created TEvtHandler into a known state. Subclasses should
				override it and call INHERITED. Typically used to put the object into a state where
				it can be safely FREED. }

				PROCEDURE TEvtHandler.IEvtHandler(itsNextHandler: TEvtHandler);
				{ Sets up the reference to the next handler by setting the fNextHandler field to
				'itsNextHandler', also sets the idle frequency and resets fLastIdle field to zero.
				}

				PROCEDURE TEvtHandler.Free; OVERRIDE;
				{ Before calling inherited Free, changes the target to the application if the next
				handler is NIL or the next handler if there is one. }

				{ Termination }
				PROCEDURE TEvtHandler.Terminate;
				{ Does nothing in this class. }

				{ Chainwalking }
				FUNCTION  TEvtHandler.GetNextHandler: TEvtHandler;
				{ Return the next handler in the chain }
				
				PROCEDURE TEvtHandler.EachHandler(PROCEDURE
												  DoToEvtHandler(anEvtHandler: TEvtHandler));
				{ Performs 'DoToEvtHandler' to aFirstHandler, then to its fNextHandler, etc., onward
				until the fNextHandler chain ends at NIL }

				FUNCTION TEvtHandler.FirstHandlerThat(FUNCTION
													  TestEvtHandler(anEvtHandler: TEvtHandler):
													  BOOLEAN): TEvtHandler;
				{ Calls TestEvtHandler for each event handler until TestEvtHandler returns true.
				Returns the TEvtHandler object for which TestEvtHandler returned true, or NIL if
				TestEvtHandler never returned true. }

				FUNCTION TEvtHandler.AddHandler(headOfChain: TEvtHandler): TEvtHandler;
				{ Adds SELF from the chain of handlers starting at headOfChain. Returns the new
				head of chain. }

				FUNCTION TEvtHandler.RemoveHandler(headOfChain: TEvtHandler): TEvtHandler;
				{ Removes SELF from the chain of handlers starting at headOfChain.	Returns the new
				head of chain (if self was the head then there _must_ be a new head. }

				{ Debugging }
				FUNCTION TEvtHandler.LookupSymbol(VAR sym: Str255): LONGINT;
				{ Called by the debugger to translate a symbol into a number; default passes to the
				fNextHandler. sym will be uppercased. Return of -1 means symbol was not found. If
				sym = '?' then writeln a list of available symbols. }

				PROCEDURE TEvtHandler.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields. }

				{ Events }
				FUNCTION TEvtHandler.NextIdle: LONGINT;
				{ Returns the number Tickcount for when the handler should next be idled. }

				PROCEDURE TEvtHandler.HandleIdle(phase: IdlePhase);
				{ Calls DoIdle if the handler is eligible. }

				FUNCTION TEvtHandler.DoIdle(phase: IdlePhase): BOOLEAN;
				{ Returns true if it frees itself so caller can know }

				FUNCTION TEvtHandler.DoCoHandlerEvent(event: TEvent): BOOLEAN;
				{ Handle the event and return TRUE if I want it, else I don't handle it, and return
				FALSE }
				
				PROCEDURE TEvtHandler.DoEvent(eventNumber: EvtNumber;
												source: TEvtHandler;
												event: TEvent);
				{ By default, asks the next handler to handle the event }

				PROCEDURE TEvtHandler.HandleEvent (eventNumber: EvtNumber;
													source: TEvtHandler;
													event: TEvent );
				{ Responsible for dispatching DoHandleEvent messages  }
				
				PROCEDURE TEvtHandler.SetEnable(state: BOOLEAN);
				{ Enables based on 'state' }

				FUNCTION TEvtHandler.IsEnabled: BOOLEAN;
				{ Returns the enabled state. }

				{ Double/Triple Clicks }
				FUNCTION TEvtHandler.DoMultiClick(lastDownPt, newDownPt: Point): BOOLEAN;
				{ Called by TApplication.CountClicks. Should return TRUE if the 2 points are close
				enough to be considered part of a double/triple click. (Both points are in global
				coordinates.) This is only called if the mouse down was within the proper time
				range of the previous mouse up. Default is to pass message to the nextEventHandler
				(if one exists), otherise to require that the sum of the x & y distances is <=
				gStdHysteresis. }

				{ Key Events }
				PROCEDURE TEvtHandler.DoKeyCommand(event: TToolboxEvent);
				{ Handles all keystrokes except those with the command key held down. }

				PROCEDURE TEvtHandler.HandleKeyCommand ( event: TToolboxEvent );
				{ Handles the dispatching of DoKeyCommand messages between itself
				and Behaviors	}
				
				PROCEDURE TEvtHandler.DoCommandKey(event: TToolboxEvent);
				{ Handles command-key combinations only. }

				PROCEDURE TEvtHandler.HandleCommandKey ( event: TToolboxEvent );
				{ Handles the dispatching of DoCommandKey messages between itself
				and Behaviors	}
				
				PROCEDURE TEvtHandler.KeyEventToComponents(event: TToolboxEvent);
				{ Intended to extract character components of an event in a script manager
				compatible way.  Default just forwards to next handler }

				{ MenuEvents }
				PROCEDURE TEvtHandler.DoMenuCommand(aCmdNumber: CmdNumber);
				{ If the handler can perform the command it does so by either performing
				the command directly or by posting a TCommand. }

				PROCEDURE TEvtHandler.HandleMenuCommand ( aCmdNumber: CmdNumber );
				{ Handles the dispatching of DoMenuCommand messages }
				
				PROCEDURE TEvtHandler.DoSetupMenus;
				{ Handles the setup of menus. }

				PROCEDURE TEvtHandler.HandleSetupMenus;
				{ Handles the dispatching of DoSetupMenus messages }
				
				{ AppleEvents }
				PROCEDURE TEvtHandler.DoAppleCommand(aCmdNumber: CmdNumber;
													 message: AppleEvent;
													 reply: AppleEvent);
				{ If the handler can perform the apple event command it does so by
				either performing the command directly or by posting a TCommand. }

				PROCEDURE TEvtHandler.HandleAppleCommand (	aCmdNumber: CmdNumber;
															 message: AppleEvent;
															 reply: AppleEvent);
				{ Handles the dispatching of DoAppleCommand messages }
				
				{ Help }

				PROCEDURE TEvtHandler.DoHelp(event: TToolboxEvent;
											 VAR message: UNIV LONGINT);
				{ If fNextHandler is not NIL calls its DoHelp method, otherwise returns NIL. }

				{ Target management }
				FUNCTION TEvtHandler.WantToBecomeTarget: BOOLEAN;
				{ Called to find out if this object wishes to become the target. Default
				returns FALSE }

				FUNCTION TEvtHandler.WillingToResignTarget: LONGINT;
				{ Called to find out if this object is willing to cease being the target. Default
				returns TRUE }
				
				PROCEDURE TEvtHandler.TargetValidationFailed(reason: LONGINT);

				PROCEDURE TEvtHandler.BecameWindowTarget;
				{ Called when this object becomes the current target of a window }

				PROCEDURE TEvtHandler.ResignedWindowTarget;
				{ Called when this object ceases to be the current target of a window }

				PROCEDURE TEvtHandler.BecameApplicationTarget;
				{ Called when this object becomes the current target  }

				PROCEDURE TEvtHandler.ResignedApplicationTarget;
				{ Called when this object ceases to be the current target  }
				
				{ Miscellaneous }
				PROCEDURE TEvtHandler.RemoveBehavior (theBehavior: TBehavior);
				{ This method is used to set the fBehavior field }

				PROCEDURE TEvtHandler.AddBehavior (theBehavior: TBehavior);
				{ Call to add theBehavior to this object }
				
				FUNCTION TEvtHandler.GetFirstBehavior:TBehavior;
				{ Call to retrieve the first behavior currently attached to EvtHandler }
				
				PROCEDURE TEvtHandler.SetIdleFreq(newIdleFreq: LONGINT);
				{ Call to set the handler's idling frequency. }

				{ Command Management }
				PROCEDURE TEvtHandler.CommitLastCommand;
				{ Commits the last command by calling its Commit method.
				Default is to hand off to fNextHandler }

				FUNCTION TEvtHandler.GetLastCommand: TCommand;
				{ Returns the last undoable command.  The command returned is not yet committed.
				Default is to hand off to fNextHandler }

				FUNCTION TEvtHandler.RetrieveAnEvent: TEvent;
				{ Retrieves a previously posted event/command from the queue.	Nil if there are 
				no queued commands. Default is to hand off to fNextHandler }

				PROCEDURE TEvtHandler.PerformCommand(command: TCommand);
				{ Called to perform the given command.
				Default is to hand off to fNextHandler }

				PROCEDURE TEvtHandler.PostAnEvent(event: TEvent);
				{ Called to post an event or command to a queue for later processing.
	 			 Default is to hand off to fNextHandler }

				PROCEDURE TEvtHandler.PostCommand(command: TCommand);
				{ Called to post a command to a queue for later execution. Left in for
	 			 compatibility - calls PostAnEvent }

				END;

		VAR
			{$Push} 									{$J+}
			gStdHysteresis: 	Point;					{ standard hysteresis used in
														 TCommand.ICommand }
			{$Pop}
END.
