{$P}
{[a-,body+,h-,o=100,r+,rec+,t=4,u+,#+,j=20/57/1$,n+]}
{Copyright  1988-1991 by Apple Computer Inc.  All rights reserved.}

UNIT UFile;

	INTERFACE

		USES
			{}
			Types, MacAppTypes,
			{}
			Memory, UPascalObject, UObject,
			{}
			Aliases, Script;

		TYPE
			StationeryState 	= (isStationery, notStationery, checkStationery);

			TFile				= OBJECT (TObject)

				fFileSpec:			FSSpec; 			{ volume/directory/filename }
				fModDate:			LONGINT;			{ file mod date when last read/saved }
				fStationery:		StationeryState;	{ is this Stationery? }
				fFileType:			OSType; 			{ file type }
				fCreator:			OSType; 			{ creator ID }
				fScriptTag:			ScriptCode;			{ The script the file was created with }
				fUsesDataFork:		Boolean;			{ The file uses its data fork }
				fUsesRsrcFork:		Boolean;			{ The file uses its rsrc fork }
				fDataPermission:	SignedByte;			{ Permissions used when opening the data fork }
				fRsrcPermission:	SignedByte;			{ Permissions used when opening the rsrc fork }
				fKeepDataOpen:		Boolean;			{ Will the data fork be left open? }
				fKeepRsrcOpen:		Boolean;			{ Will the rsrc fork be left open? }
				fDataRefNum:		INTEGER;			{ Reference for reading/writing to data fork }
				fRsrcRefNum:		INTEGER;			{ Reference for reading/writing to rsrc fork }

				PROCEDURE TFile.Initialize; OVERRIDE;
				{ Put the file into a known state }

				PROCEDURE TFile.IFile;
				{ Initialization method for TFile }
				
				PROCEDURE TFile.Free; OVERRIDE;
				
				FUNCTION TFile.CloseFile(forceClose: Boolean): OSErr;
				{ Close the file. }
				
				FUNCTION TFile.CloseDataFork(forceClose: Boolean): OSErr;
				{ Close the data fork.  Called by CloseFile }
				
				FUNCTION TFile.CloseRsrcFork(forceClose: Boolean): OSErr;
				{ Close the rsrc fork. Called by CloseFile }

				FUNCTION TFile.CreateFile: OSErr;
				{ Create the file }
				
				FUNCTION TFile.CreateDataFork: OSErr;
				{ Create the data fork of the file.  Called by CreateFile }

				FUNCTION TFile.CreateRsrcFork: OSErr;
				{ Create the rsrc fork of the file.  Called by CreateFile }

				FUNCTION TFile.DeleteFile: OSErr;
				{ Close the file and delete it. }
				
				FUNCTION TFile.OpenFile: OSErr;
				{ Open the file with the fDataPermission and fRsrcPermission }
				
				FUNCTION TFile.OpenDataFork(permission: SignedByte): OSErr;
				{ Open the data fork of the file.  Called by OpenFile }
				
				FUNCTION TFile.OpenRsrcFork(permission: SignedByte): OSErr;
				{ Open the rsrc fork of the file.  Called by OpenFile }

				{--------------- file attributes ------------------------- }

				PROCEDURE TFile.DefineFile(itsFileType, itsCreator: OSType;
											usesDataFork, usesRsrcFork: Boolean;
											keepsDataOpen, keepsRsrcOpen: Boolean);
				{ Initializes the fields of TFile used for data and resource manipulation }
				
				FUNCTION TFile.GetAlias(VAR alias: AliasHandle): OSErr;
				{ Return the file specification as an alias handle }
				
				FUNCTION TFile.GetCatInfo(VAR cInfo: CInfoPBRec): OSErr;
				{ Return the catalog information about the this file }

				FUNCTION TFile.GetCreationDate: LONGINT;
				{ Date the file was created }

				FUNCTION TFile.GetDataLength(VAR length: LONGINT): OSErr;
				{ Return the length of the data fork }

				FUNCTION TFile.GetDataMark(VAR mark: LONGINT): OSErr;
				{ Return the position of the mark in the data fork }

				FUNCTION TFile.GetDirID: LONGINT;
				{ Return the directory ID stored in the FSSpec }

				FUNCTION TFile.GetFileCreator(VAR creator: OSType): OSErr;
				{ Return the creator of the file as found in the finder info }

				FUNCTION TFile.GetFileInfo(VAR pb: HParamBlockRec): OSErr;
				{ Return all the information the file manager provides }
				
				PROCEDURE TFile.GetFileSpec(theFileSpec: FSSpec);
				{ Return its file specification record }

				FUNCTION TFile.GetFileType(VAR fileType: OSType): OSErr;
				{ Return the type of the file as found in the finder info }

				FUNCTION TFile.GetFinderInfo(VAR fndrInfo: FInfo): OSErr;
				{ Return finder information about the file }

				FUNCTION TFile.GetModificationDate: LONGINT;
				{ Return the date the file was last modified }

				PROCEDURE TFile.GetName(VAR name: STR255);
				{ Return the name of the file }

				FUNCTION TFile.GetPhysicalSize(VAR dataSize, rsrcSize: LONGINT): OSErr;
				{ Returns the physical size in bytes of the data and resource fork }
				
				FUNCTION TFile.GetScript(VAR theScript: ScriptCode): OSErr;
				{ Returns the script code as stored in the extended finder info }

				{--------------- Volume attributes ------------------------- }

				FUNCTION TFile.FlushVolume: OSErr;
				{ Flush the volume to force any changes to take place }
				
				FUNCTION TFile.GetBlockSize(VAR blockSize: LONGINT): OSErr;
				{ Return the size of allocated blocks on the volume }

				FUNCTION TFile.GetFreeBlocks(VAR freeBlocks: LONGINT): OSErr;
				{ Return the number of free blocks on the volume }

				FUNCTION TFile.GetVolumeInfo(VAR pb: HParamBlockRec): OSErr;
				{ Return all information provided by the file manager on the volume }

				FUNCTION TFile.GetVolumeName(VAR name: STR255): OSErr;
				{ Return the name of the volume }
				
				FUNCTION TFile.GetVolRefNum: INTEGER;
				{ Return the volume reference number stored in the FSSpec }

				FUNCTION TFile.ExchangeFiles(aFile: TFile): OSErr;
				{ Exchange the contents of self with the contents of another file maintaining
				the file id }

				PROCEDURE TFile.Identify(theFile: FSSpec);
				{ Identify the file by its FSSpec.  If the file has already opened a file,
				Identify will attempt to close the file before re-identifying the file }

				PROCEDURE TFile.IdentifyWithScript(theFile: FSSpec; theScript: ScriptCode);
				{ Identify the file by its FSSpec and indiciate its script }

				FUNCTION TFile.IdentifyByAlias(alias: AliasHandle): OSErr;
				{ Similar to TFile.Identify except the file is identified by an alias }

				PROCEDURE TFile.IdentifyByFile(aFile: TFile);
				{ Make this file reference the same file as aFile.  Also sets fScriptTag equal to
				aFile.fScriptTag }

				FUNCTION TFile.IdentifyByTrio(volRefNum: INTEGER; dirID: LONGINT; name: StringPtr): OSErr;
				{ Identify a file by volrefnum, dir ID, and filename.  If dirID is
				zero, it then volRefNum is assumed to be a working directory and
				will be resolved to a directory ID }
				
				FUNCTION TFile.IsDataForkOpen: Boolean;
				{ True if the data fork of the file is open }
				
				FUNCTION TFile.IsModified: Boolean;
				{ Checks fModDate to determine if file changed }

				FUNCTION TFile.IsRsrcForkOpen: Boolean;
				{ True if the resource fork of the file is open }

				FUNCTION TFile.IsSameFile(aFile: TFile): Boolean;
				{ Does this file refer to the same file as aFile? }

				FUNCTION TFile.IsVolumeLocked(VAR locked: Boolean): OSErr;
				{ Returns TRUE if the volume is hardware or software locked }
				
				FUNCTION TFile.HasDataFork: Boolean;
				{ Returns TRUE if file has a data fork }
				
				FUNCTION TFile.HasRsrcFork: Boolean;
				{ Returns TRUE if file has a resource fork }

				FUNCTION TFile.MakeStationery: OSErr;
				{ Make this file stationery }
				
				PROCEDURE TFile.Modified;
				{ Update fModDate to the value stored by the finder }

				FUNCTION TFile.MoveAndRename(dest: FSSpec): OSErr;
				{ Move the file to the new destination and give it a new name }

				FUNCTION TFile.ReadData(buffer: Ptr; VAR count: LONGINT): OSErr;
				{ Read count bytes from the data fork into buffer }

				FUNCTION TFile.RenameFile(newName: StringPtr): OSErr;
				{ Rename the file to the specified name }

				FUNCTION TFile.SetCatInfo(cInfo: CInfoPBRec): OSErr;
				{ Set the catalog information about this file.  The file reference portion
				of the record is supplied by fFileSpec. }

				FUNCTION TFile.SetDataLength(length: LONGINT): OSErr;
				{ Set the length of the data fork }

				FUNCTION TFile.SetDataMark(mark: LONGINT; fromWhere: INTEGER): OSErr;
				{ Set the position of the mark in the data fork }

				FUNCTION TFile.SetFileInfo(pb: HParamBlockRec): OSErr;
				{ Set the file information about this file.  The file reference portion
				of the record is supplied by fFileSpec. }
				
				PROCEDURE TFile.SetName(newName: StringPtr);
				{ Sets the name field of the filespec but doesn't rename the file }
				
				PROCEDURE TFile.SetPermissions(dataPermission, rsrcPermission: SignedByte);
				{ Set the fDataPermission and fRsrcPermission fields of the file }
				
				FUNCTION TFile.SetScript(theScript: ScriptCode): OSErr;
				{ Sets the script code stored in the extended finder info }

				FUNCTION TFile.UpdateResource: OSErr;
				{ Update the resource referenced by fRsrcRefNum }

				FUNCTION TFile.UseResource: INTEGER;
				{ Returns the current resource file and set itself as the current file }

				FUNCTION TFile.WriteData(buffer: Ptr; VAR count: LONGINT): OSErr;
				{ Write count bytes from buffer to the data fork }

				PROCEDURE TFile.Fields(obj: TObject); OVERRIDE;
				{ Called by the Inspector and the Debugger to display the contents of this class's
				fields. }

				END;

		FUNCTION NewFile: TFile;
		{ A convenience function. Create a TFile, initialize it, and return a reference to it.
		Signals Failure if it cannot allocate the object.}
END.
