{[a-,body+,h-,o=100,r+,rec+,t=4,u+,#+,j=20/57/1$,n+]}
{ UFileBasedDocument.p }
{ Copyright  1984-1991 by Apple Computer Inc.	All rights reserved. }

UNIT UFileBasedDocument;

	INTERFACE

		USES
			{}
			Types, MacAppTypes, Memory, UPascalObject, UObject, AppleEvents, UEvtHandler, UEvent, UCommand, UDocument;

		TYPE

			TFile				= OBJECT; external;
			TFileHandler		= OBJECT; external;

			TFileBasedDocument	= OBJECT (TDocument)	{ Corresponds to a Finder document. Manages
														 document data in files and in main memory.
														 }

				fFileHandler:		TFileHandler;		{ volume/directory/filename }

				fScrapType:			OSType;				{ document's private scrap type }

								{ Init & Free }
				PROCEDURE TFileBasedDocument.Initialize; OVERRIDE;
				{ Puts a newly created TDocument into a known state. Subclasses should
				override it and call INHERITED. Typically used to put the object into a state where
				it can be safely FREED. }

				PROCEDURE TFileBasedDocument.IFileBasedDocument(itsFile: TFile;
																itsFileType, itsCreator: OSType;
																itsScrapType: OSType;
																usesDataFork, usesRsrcFork: Boolean;
																keepsDataOpen, keepsRsrcOpen: Boolean);
				{ Initialization method for TFileBasedDocument. }

				FUNCTION TFileBasedDocument.DoMakeFileHandler(itsFile: TFile; itsFileType,
															  itsCreator: OSType; usesDataFork,
															  usesRsrcFork: Boolean; keepsDataOpen,
															  keepsRsrcOpen: Boolean): TFileHandler;
				{ Creates and returns a TFileHandler for this object. }

				PROCEDURE TFileBasedDocument.Free; OVERRIDE;
				{ This does not call FreeData by default, since you may need to control the order in
				which things are freed. Your override of TDocument.Free can call FreeData if
				convenient. }

				PROCEDURE TFileBasedDocument.Fields(obj: TObject); OVERRIDE;
				{ Called by the Inspector and the Debugger to display the contents of this class's
				fields. }

				{ Opening Documents }
				FUNCTION TFileBasedDocument.AlreadyOpen(aFile: TFile): BOOLEAN; OVERRIDE;
				{ True if this file is already open }

				PROCEDURE TFileBasedDocument.DoNeedDiskSpace(itsFile: TFile;
															VAR dataForkBytes, rsrcForkBytes: LONGINT);
				{ Bytes required to store SAVE file(s) on disk. When called, both parameters are set
				to 0 for you. }

				PROCEDURE TFileBasedDocument.DoRead(aFile: TFile; forPrinting: Boolean);
				{ The default behaviour is to just read the printinfo from the document's
				data-fork by calling DoReadPrintInfo (useful huh?) }

				PROCEDURE TFileBasedDocument.DoReadPrintInfo(aFile: TFile; forPrinting: Boolean);
				{ The default behaviour is to just read the printinfo from the document's
				data-fork (useful huh?) }

				PROCEDURE TFileBasedDocument.DoWrite(aFile: TFile; makingCopy: Boolean);
				{ makingCopy indicates whether this save is for
				making a copy of the document (vs. a regular Save or Save As); disk-based
				documents (especially) might optimize their behavior if this is TRUE. }

				PROCEDURE TFileBasedDocument.AboutToSaveFile(itsCmd: CmdNumber;
															 VAR makingCopy: Boolean);
				{ Called from file handler right after the SFPutFile (if any) and before deciding between SaveViaTemp &
				SaveInPlave. Applications can use the name in an alert and/or modify the makingCopy
				parameter if they do not want to have the document renamed. }

				PROCEDURE TFileBasedDocument.DoWritePrintInfo(aFile: TFile; makingCopy: Boolean);
				{ makingCopy indicates whether this save is for
				making a copy of the document (vs. a regular Save or Save As); disk-based
				documents (especially) might optimize their behavior if this is TRUE. The default
				behaviour is to just write the printinfo to the document's data-fork (useful huh?)}

				PROCEDURE TFileBasedDocument.ReadDocument(forPrinting: Boolean); OVERRIDE;
				{ Called to read an existing document for display or printing. }

				{ Saving Documents }
				PROCEDURE TFileBasedDocument.SaveDocument(itsCmdNumber: CmdNumber); OVERRIDE;
				{ Try to save the document to disk }

				PROCEDURE TFileBasedDocument.FileHasBeenSaved(newName: Str255);
				{ Called when a save successfully completes and we are switching to the new file
				(ie., not a Save a Copy In). }

				PROCEDURE TFileBasedDocument.SFPutParms(itsCmdNumber: CmdNumber;
														VAR prompt, defaultName: Str255;
														VAR dlgID: INTEGER;
														VAR where: Point;
														VAR dlgHook, modalFilter: ProcPtr;
														VAR activeList: Ptr;
														VAR activateProc: ProcPtr;
														VAR yourDataPtr: Ptr);
				{ Called to return all the parameters that should be passed to
				CustomPutFile or SFPutFile. Defaults to
					prompt gotten from resource file;
					defaultName not changed;
					dlgID = sfPutDialogID or putDlgID;
					where = (-1, -1) or (100, 100);
					dlgHook = NIL;
					modalFilter = NIL;
					activeList = NIL;
					activateProc = NIL;
					yourDataPtr = NIL.}

				{ Revert }
				PROCEDURE TFileBasedDocument.RevertDocument; OVERRIDE;
				{ IF fSaveExists is TRUE then reverts to the file saved on disk, otherwise merely
				resets the print handler and calls doInitialState. Calls FreeData to free any
				existing data and commits any currently active command. }

				{ Miscellaneous }
				PROCEDURE TFileBasedDocument.CheckFile(rsrcId, rsrcIndex: INTEGER;
													   reverting: Boolean);
				{ Check to see if the disk file has changed. If so put up alert phFileChanged after
				setting ^0 to fTitle and ^1 to the string specified by the rsrcId and rsrcIndex. If
				user cancels, signal Failure(noErr, msgCancelled). If reverting is TRUE, then I/O
				errors and nonmatching file types will cause a Failure. Otherwise, this is a Save
				and any I/O errors and nonmatching file type will be ignored. }

				PROCEDURE TFileBasedDocument.CloseFile;

				FUNCTION TFileBasedDocument.GetFileHandler: TFileHandler;
				{ accessor for fFileHandler }

				PROCEDURE TFileBasedDocument.SetTitle(aTitle: Str255); OVERRIDE;
				{ Does fFile.SetName to aTitle and calls SetTitleForDoc for each window of the
				document. }

				END;
END.
