{[a-,body+,h-,o=100,r+,rec+,t=4,u+,#+,j=20/57/1$,n+]}
{ UFileHandler.p }
{ Copyright  1984-1991 by Apple Computer Inc.	All rights reserved. }

UNIT UFileHandler;

	INTERFACE

		USES
			{}
			Types, MacAppTypes,
			{}
			Memory, UPascalObject, UObject,
			{}
			Files;

		TYPE

			TFileBasedDocument	= OBJECT; external;
			TFile				= OBJECT; external;

			SaveChoice			= (svtAlways, svtAskUser, sipAlways, sipAskUser);
			{ How to save a document.
			svtAlways: 	Always save with SaveViaTemp.  Never overwrite the existing file.
			svtAskUser:	Try to SaveViaTemp.  If not possible then ask before SaveInPlace.
			sipAlways:	Always save with SaveInPlace.  Never create a temporary file.
			sipAskUser: Always save with SaveInPlace, but only if user confirms.
			}

			TFileHandler		= OBJECT (TObject)

				fDocument:			TFileBasedDocument; { The controller of the model }

				fFile:				TFile;				{ volume/directory/filename }

				fFileExists:		BOOLEAN;			{ whether a disk file representing this
														 model exists }
				fHowToSave:			SaveChoice;			{ how to save files }

								{ Init & Free }
				PROCEDURE TFileHandler.Initialize; OVERRIDE;
				{ Puts a newly created TDocument into a known state. Subclasses should
				override it and call INHERITED. Typically used to put the object into a state where
				it can be safely FREED. }

				PROCEDURE TFileHandler.IFileHandler(itsDocument: TFileBasedDocument; itsFile: TFile;
													itsFileType, itsCreator: OSType; usesDataFork,
													usesRsrcFork: BOOLEAN; keepsDataOpen,
													keepsRsrcOpen: BOOLEAN);
				{ Initialization method for TFileHandler. }

				PROCEDURE TFileHandler.Free; OVERRIDE;

				PROCEDURE TFileHandler.Fields(obj: TObject); OVERRIDE;
				{ Called by the Inspector and the Debugger to display the contents of this class's
				fields. }

				{ Opening / Documents }
				FUNCTION TFileHandler.FileAlreadyOpen(aFile: TFile): BOOLEAN;
				{ True if the file for the model is already open }

				PROCEDURE TFileHandler.ReadFile(forPrinting: BOOLEAN);
				{ Called to read an existing file for display or printing. }

				PROCEDURE TFileHandler.DoRead(forPrinting: BOOLEAN);
				{ The default behaviour is to just read the printinfo from the document's
				data-fork by calling DoReadPrintInfo (useful huh?) }

				PROCEDURE TFileHandler.DoWrite(aFile: TFile; makingCopy: BOOLEAN);
				{ makingCopy indicates whether this save is for
				making a copy of the document (vs. a regular Save or Save As); disk-based
				documents (especially) might optimize their behavior if this is TRUE. }

				{ Saving Documents }
				PROCEDURE TFileHandler.AboutToSaveFile(itsCmd: CmdNumber; VAR makingCopy: BOOLEAN);
				{ Called right after the SFPutFile (if any) and before deciding between SaveViaTemp &
				SaveInPlave. Applications can use the name in an alert and/or modify the makingCopy
				parameter if they do not want to have the document renamed. }

				PROCEDURE TFileHandler.DoNeedDiskSpace(VAR dataForkBytes, rsrcForkBytes: LONGINT);
				{ Bytes required to store SAVE file(s) on disk. When called, both parameters are set
				to 0 for you. }

				PROCEDURE TFileHandler.CloseFile;
				{ Called to free in memory data associated with the currently opened document. The
				default is to call fFile.Close(TRUE) to force the file close }

				FUNCTION TFileHandler.GetSaveInfo(itsCmdNumber: CmdNumber; copyFInfo: BOOLEAN;
												  VAR cInfo: CInfoPBRec): BOOLEAN;
				{ Called to get the desired file info for saving the file. Default copies the info
				from the current file if fSaveExists AND copyFInfo, otherwise it just sets the file
				type and creator based on the fields of SELF. If an error occurs calling the file
				system, the cInfo record will not be filled in. Returns TRUE if it successfully
				gets the info from the file. }

				PROCEDURE TFileHandler.GetTempName(VAR fileName: Str255);
				{ Return a temporary name for saving the file. The filename is constructed in two
				parts: the first part is the document's name, or the application's name if the
				document is untitled. The second part, appended to the first part, is a pseudo-
				random number based on the tick count and number of seconds since boot. }

				PROCEDURE TFileHandler.RequestFileName(itsCmdNumber: CmdNumber; makingCopy: BOOLEAN;
													   aFile: TFile);
				{ Asks the user for a filename for the document. Calls TFileHandler.SFPutParms and then
				SFPPutFile. Fails if the user clicks the Cancel button to end the put file dialog.
				}

				PROCEDURE TFileHandler.SaveFile(itsCmdNumber: CmdNumber; askForFilename, copyFInfo,
												makingCopy: BOOLEAN);
				{ Try to save the document to disk; If askForFilename, put up a Std File box; If
				makingCopy, then this is for the Save a Copy In command, so we do not rename the
				document. This method calls FlushVol (which takes care of the requirements of
				SaveInPlace, and SaveViaTemp. }

				PROCEDURE TFileHandler.SaveInPlace(itsCmdNumber: CmdNumber; makingCopy,
												   copyFInfo: BOOLEAN; itsFile: TFile);
				{ Called to save a document in place. Default is to delete the existing file if NOT
				(fDataOpen OR fRsrcOpen). If you have disk based document, you have to decide
				whether to allow saving in place. makingCopy is TRUE if this save is making a copy
				of the document (vs. a regular Save or Save As) copyFInfo is TRUE if the file
				information should be copied to the new file Caller is responsible for calling
				FlushVol after this. }

				PROCEDURE TFileHandler.SaveViaTemp(itsCmdNumber: CmdNumber; makingCopy,
												   copyFInfo: BOOLEAN; itsFile: TFile);
				{ Called to save the document by creating a temporary copy of the file and renaming
				it. MacApp uses this method except when there is not enough disk space available.
				makingCopy is TRUE if this save is making a copy of the document (vs. a regular
				Save or Save As) copyFInfo is TRUE if the file information should be copied to the
				new file Caller is responsible for calling FlushVol after this. }

				{ Miscellaneous }
				FUNCTION TFileHandler.FileChanged(checkType: BOOLEAN): OSerr;
				{ Returns noErr if fSaveExists is FALSE or if the file's modification date matches
				fModDate. If checkType is TRUE then returns errFTypeChanged if the file type has
				changed. If the file exists but with a different modification date, returns
				errFileChanged. }

				FUNCTION TFileHandler.FileExists: BOOLEAN;
				{ Default returns the status of fFileExists }

				FUNCTION TFileHandler.HasRsrcFork: BOOLEAN;
				{ Returns TRUE if the file has a resource fork }

				FUNCTION TFileHandler.GetFile: TFile;
				{ Return the file reference by TFileHandler }

				PROCEDURE TFileHandler.GetFileName(VAR aName: Str255);
				{ Return the name of the file }

				PROCEDURE TFileHandler.SetFileName(aName: Str255);
				{ Default behavior calls fFile.SetName with aName }

				END;
END.
