{[a-,body+,h-,o=100,r+,rec+,t=4,u+,#+,j=20/57/1$,n-]}
{ UMenuView.p }
{ Copyright  1989-1991 by Apple Computer Inc. All rights reserved. }

{[f-]}
(*
		T H E O R Y   O F	O P E R A T I O N

This is Larry Rosenstein's excellent unit: UMenu, that supports an
object-oriented way of defining custom menus.

Instead of writing a menu defproc, you simply create a subtype of TMenuView
and override certain methods.  Not only is this easier to program,
but this unit takes care of some of the housekeeping that a menu
defproc would normally have to do.

Changes since MacApp 2.0: You can now add subviews to your subclass of TMenuView, and
these subviews can have their own trackers.
This is accomplished in HandleChooseMessage by creating a dummy mouse down
event and passing it to the TMenuView via the HandleMouseDown method which then
asks its subviews if any of them are interested in the mousedown (eg. to create
a tracker). The tracker should be posted via PostCommand, and the TMenuView override
of PostCommand will check to see if the command is a tracker, in which case it
will track it immediately.
*)
{[f+]}

UNIT UMenuView;

	INTERFACE

		USES
			{}
			Types, MacAppTypes, Memory, UPascalObject, UObject, AppleEvents, UEvtHandler, Balloons, UView, Menus;

		CONST
			kNoMenuItem 		= 0;					{ used to indicate that no item is selected
														 }

		TYPE

			MenuColors			= RECORD				{ a composite of the MCEntries }
				itemColor:			RGBColor;			{ the foreground color to draw the item with
														 (the color of the title if title or
														 menubar) }
				backgroundColor:	RGBColor;			{ the background color to draw the item with
														 (the color of the menubar if title or
														 menubar)}
				markColor:			RGBColor;			{ the foreground color to draw the mark with
														 (checkmarks, etc.) }
				commandColor:		RGBColor;			{ the foreground color to draw the command
														 key equivalent with (command-v for paste,
														 etc).}
				END;
			MenuColorsPtr		= ^MenuColors;
			MenuColorsHandle	= ^MenuColorsPtr;

			TEvent				= OBJECT; EXTERNAL;
			TDocument			= OBJECT; EXTERNAL;
			TToolboxEvent		= OBJECT; EXTERNAL;

			TMenuView				= OBJECT (TView)
				fMenuHandle:		MenuHandle; 		{ handle to menu itself }
				fFlashInterval: 	LONGINT;			{ Time (in ticks) between flashes of item
														 highlighting; default is -1 which means
														 only change highlighting w when item
														 changes }
				fNextFlash: 		LONGINT;			{ Used internally }
				fHighlighted:		BOOLEAN;			{ True if an item is highlighted }
				fBorder:			Rect;				{ Added to menuRect to get actual hit rect;
														 FindItem will not be called if the mouse
														 is in the border. }

								{ Initialization }

				PROCEDURE TMenuView.Initialize; OVERRIDE;
				{ Puts a newly created TMenuView into a known state. Subclasses should
				override it and call INHERITED. Typically used to put the object into a state where
				it can be safely FREED. }

				PROCEDURE TMenuView.IMenuView(rsrcID: INTEGER;
									  menuWidth, menuHeight: INTEGER);
				{ IMenu reads the menu and sets up the MenuHandle and defproc
				  so that the Menu Manager can communicate with the TMenuView object.
				 }

				{ Menu Definition; all these methods must be overridden }

				FUNCTION TMenuView.FindItem(hitPt: Point): INTEGER;
				{ This is called to convert a point to an item number. }

				PROCEDURE TMenuView.PostCommand(command: TCommand); OVERRIDE;
				{ ensure that trackers get tracked immediately }

				PROCEDURE TMenuView.Highlight(whichItem: INTEGER;
										  turnItOn: BOOLEAN);
				{ Given an item number as returned by FindItem, this is called
				to highlight the item.	This will not be called with
				whichItem = kNoMenuItem. }

				{ Other methods }

				PROCEDURE TMenuView.GeTMenuViewColors(theMenu, theItem: INTEGER;
											  VAR theMenuColors: MenuColors);
				{ Called to get the appropriate menu colors for the menu and item.
				If the menu and item are 0 then info is for the menubar.
				If item is 0 then info is for the menu title.
				Attempts to return best guess at colors based on all available info. }

				FUNCTION TMenuView.IsItemEnabled(item: INTEGER): BOOLEAN;
				{ Returns the enabled status of an item }

				PROCEDURE TMenuView.UpdateHighlight(oldItem, newItem: INTEGER);
				{ Called to update menu item highlighting. }

				{ Private }

				PROCEDURE TMenuView.HandleDefproc(message: INTEGER;
											  theMenu: MenuHandle;
											  VAR menuRect: Rect;
											  hitPt: Point;
											  VAR whichItem: INTEGER);
				{ This is called by the custom defproc; it does a CASE
				on the message and calls 1 or more of the methods above. }

				PROCEDURE TMenuView.HandleChooseMessage(message: INTEGER;
													theMenu: MenuHandle;
													VAR menuRect: Rect;
													hitPt: Point;
													VAR whichItem: INTEGER);
				{ Called by the message dispatcher when a choose message has been sent }

				PROCEDURE TMenuView.HandleDrawMessage(message: INTEGER;
												  theMenu: MenuHandle;
												  VAR menuRect: Rect;
												  hitPt: Point;
												  VAR whichItem: INTEGER);
				{ Called by the message dispatcher when a draw message has been sent }

				PROCEDURE TMenuView.HandleSizeMessage(message: INTEGER;
												  theMenu: MenuHandle;
												  VAR menuRect: Rect;
												  hitPt: Point;
												  VAR whichItem: INTEGER);
				PROCEDURE TMenuView.HandlePopUpMessage(message: INTEGER;
												   theMenu: MenuHandle;
												   VAR menuRect: Rect;
												   hitPt: Point;
												   VAR whichItem: INTEGER);
				{ Focusing Methods }

				FUNCTION TMenuView.Focus: BOOLEAN; OVERRIDE;

				FUNCTION TMenuView.FocusOnSuperView: BOOLEAN; OVERRIDE;

				{ Misc. Methods }

				FUNCTION TMenuView.GetGrafPort: GrafPtr; OVERRIDE;

				PROCEDURE TMenuView.CreateMouseDownEventInfo(hitPt: Point;
														 VAR anEventRecord: EventRecord;
														 event: TToolboxEvent);
				{ Called by HandleChooseMessage when creating a phony mousedown event which is then
				passed off to the subviews of TMenuView via HandleMouseDown }

				{ Debugging Methods }

				PROCEDURE TMenuView.Fields(obj: TObject); OVERRIDE;

				END;

		PROCEDURE InitUMenuView;
		{ Call this at the start of your program, before creating any TMenuView objects.
		This can signal Failure. }

		VAR
			{$Push} 									{$J+}
			gTrackingInMenu: BOOLEAN;
			{$Pop}
END.
