{[a-,body+,h-,o=100,r+,rec+,t=4,u+,#+,j=20/57/1$,n-]}
{ UObject.p }
{ Copyright  1984-1991  Apple Computer, Inc.  All rights reserved. }

UNIT UObject;

	INTERFACE

		USES
			{}
			Types, MacAppTypes,
			{}
			Memory,
			{}
			UPascalObject;

		TYPE
			TStream = OBJECT; EXTERNAL;
{--------------------------------------------------------------------------------------------------}
			{ Definition of the system's root object }

			TObject 			= OBJECT

			{ Change notification }

				PROCEDURE TObject.Changed(theChange: ChangeID;
										  changedBy: TObject);
				{ Informs this object that it has been changed. In MacApp, "theChange" will often contain a
				command number, and "changedBy" a command object. Marks all dependents (direct and indirect)
				as needing updating, asks SELF to "UpdateDependents", then unmarks all dependents. Called often,
				overridden rarely. }

				PROCEDURE TObject.DoUpdate(VAR theChange: ChangeID;
										   changedObject: TObject;
										   changedBy: TObject);
				{ Informs this object that an object on which it is dependent has been changed. By
				returning kNoChange in "theChange" the propagation of DoUpdate calls to dependents
				can be halted. Called rarely, overridden often. }

				PROCEDURE TObject.MarkDependents(state: BOOLEAN);
				{ Asks each of the dependents of this object to  "MarkRecursively" }

				PROCEDURE TObject.MarkRecursively(state: BOOLEAN);
				{ If it is not already marked, marks this object,  and recursively asks each dependent to
				mark itself }

				PROCEDURE TObject.SetMark(state: BOOLEAN);
				{ Marks (state=TRUE) or unmarks (state=FALSE) this object }

				FUNCTION TObject.IsMarked: BOOLEAN;
				{ Returns TRUE if this object is marked }

				PROCEDURE TObject.UpdateDependents(theChange: ChangeID;
												   changedObject: TObject;
												   changedBy: TObject);
				{ Asks each dependent to "HandleUpdate" }

				PROCEDURE TObject.UpdateNotifiers(theChange: ChangeID;
												  changedObject: TObject;
												  changedBy: TObject);
				{ Asks each object on which this object is dependent to "HandleUpdate" }

				PROCEDURE TObject.HandleUpdate(theChange: ChangeID;
											   changedObject: TObject;
											   changedBy: TObject);
				{ If SELF is marked as needing updating, update all objects on which
				SELF is dependent, update SELF ("DoUpdate"), then unmark SELF, and update
				all dependents of SELF }

				PROCEDURE TObject.AddDependent(dependent: TObject);
				{ Registers the specified object as a dependent of this object, so that it will
				be notified of changes in this object }

				PROCEDURE TObject.RemoveDependent(dependent: TObject);
				{ Remove the specified object from the list of dependents of this object }

				PROCEDURE TObject.RemoveAllDependencies;
				{ Used by TObject.Free to remove all dependency relations which involve
				this object, either as notifier or as dependent }

				PROCEDURE TObject.EachDependent(PROCEDURE
												DoToDependent(aDependent: TObject));
				{ Performs DoToDependent on every dependent of this object }

				PROCEDURE TObject.EachNotifier(PROCEDURE
											   DoToNotifier(aNotifier: TObject));
				{ Performs DoToNotifier on every object on which this object is dependent }

				FUNCTION TObject.Clone: TObject;
				{ Makes a duplicate copy of SELF. The default calls SELF.ShallowClone, which makes a
				literal copy of instance variables but does not attempt to clone owned objects. A
				subclass which owns other objects should override this to clone the owned objects
				and data structures as well. }

				{ Stream I/O }
				PROCEDURE TObject.ReadFrom(aStream: TStream);
				PROCEDURE TObject.WriteTo(aStream: TStream);

				PROCEDURE TObject.DoToField(fieldName: StringPtr;
											fieldAddr: Ptr;
											fieldType: INTEGER);
				{ Called by DoToField once for each instance variable in SELF. fieldName is a string
				representing the name of the variable. fieldAddr is the address of that variable.
				fieldType is one of the types defined in UMacAppUtilities.	}

				PROCEDURE TObject.Fields(obj: TObject);
				{ Calls obj.DoToField once for each instance variable in SELF.	Also, It is legal to
				use a local variable to hold data and pass the address of that variable to DoToField
				as if it were part of the object's instance variables. By convention, the first call
				to DoToField should have the class name as the fieldName, NIL as the fieldAddr, and
				bClass as the fieldType. Each subclass should override this method, particularly
				with qDebug on, dump its own information, then call INHERITED. }

				PROCEDURE TObject.DynamicFields(obj: TObject);
				{ Used by the Inspector and the Debugger to display the contents of this class's
				dynamic area. Defaults to nothing.	Override if your object uses the dynamic area
				See TList for an example. }

				PROCEDURE TObject.ForAllSubClassesDo(PROCEDURE
													 DoToSubClass(theClass: ObjClassID));
				{ Calls DoToSubClass for each subclass of SELF. The order of traversal is arbitrary;
				see EachSubClass global procedure. }

				PROCEDURE TObject.ForAllSuperClassesDo(PROCEDURE
													   DoToSubClass(theClass: ObjClassID));
				{ Calls DoToSubClass for each superclass of SELF. The order of traversal is the order
				of ancestry starting from the class of SELF; see EachSuperClass global procedure. }

				PROCEDURE TObject.Free;
				{ Called to dispose of an object. Gives object a chance to cleanup after itself.
				Default simply calls SELF.ShallowFree, which makes no attempt to free instance
				variables. Should be overridden by any class which allocates space or owns other
				objects in its instance variables.. Be sure to call INHERITED! }

				FUNCTION TObject.GetClass: ObjClassID;
				{ Returns the class ID of SELF. }

				PROCEDURE TObject.GetClassName(VAR clName: MAName);
				{ Returns the class name of SELF. }

				FUNCTION TObject.GetClassSize: Size;
				{ Returns the basic instantiation size. i.e. the size in bytes of a newly created
				object of this class }

				FUNCTION TObject.GetDynamicPtr: Ptr;
				{ Our objects have a dynamic area available. In 2.x it is at the end of the object
				handle but it may not always be so.  Returns a direct heap pointer to the start
				of the dynamic area.  Use with caution as this pointer can be invalidated if the
				heap compacts. }

				FUNCTION TObject.GetDynamicSize: Size;
				{ Our objects can have a dynamic area that can be allocated with a SetDynamicSize
				call.  Returns the size of the dynamic area in bytes. Preferred. }

				FUNCTION TObject.GetSuperClass: ObjClassID;
				{ Returns immediate superclass of class. Returns kNilClass for TObject. If we ever
				get MI this will have to be an enumerator. }

				PROCEDURE TObject.GetInspectorName(VAR inspectorName: Str255);
				{ Returns additional information useful to the inspector. For instance (ha ha!) a
				TDocument could give its title. }

				PROCEDURE TObject.Initialize;
				{ Call this to put a newly created object into a known state. Subclasses should
				override it and call INHERITED. Typically used to put the object into a state where
				it can be safely FREED. Defaults to nothing. }

				PROCEDURE TObject.IObject;
				{ The ancestral initializer. Should be called in the I<ClassName> chain. i.e. IView
				-> IEvtHandler -> IOBJECT }

				FUNCTION TObject.IsMemberClass(testClass: ObjClassID): BOOLEAN;
				{ True if I am a MEMBER of the testClass. i.e. the same class OR a subclass. }

				FUNCTION TObject.IsSameClass(testClass: ObjClassID): BOOLEAN;
				{ True if I belong to the immediate class. }

				FUNCTION TObject.Lock(lockIt: BOOLEAN): BOOLEAN;
				{ Locks down or unlocks an object AND its dynamic area if any. Returns old state.
				Since objects can float around in memory some occasions may arise where they need
				to be locked down (The Inspect method is an example: since addresses of instance
				variables are being passed, the object had better not move during the Inspect!)
				Typically the old state is saved in a local and re-applied when ascending the
				call chain. Don't forget to re-apply the old state from failure handlers.
				_PLEASE_ use this sparingly and don't just go locking stuff down willy-nilly just
				because you might be afraid of handles. }

				PROCEDURE TObject.SetDynamicSize(newSize: Size);
				{ Set this instance's Dynamic Area size in bytes.  The dynamic area starts out
				unallocated. Signals Failure on memory error.  The size applies only to the size
				of the Dynamic Area. Preferred. }

				FUNCTION TObject.ShallowClone: TObject;
				{ Lowest level method for copying an object; should not be overridden except in very
				unusual cases. Simply calls HandToHand to copy the object data. }

				PROCEDURE TObject.ShallowFree;
				{ Lowest level method for freeing an object; should not be overridden except in very
				unusual cases. Simply calls Dispose (the Pascal builtin) to free the object. }

				PROCEDURE TObject.SubClassResponsibility;
				{ Called from methods that MUST be overridden in a subclass.  Signals failure }
				END;

			TObjectPtrPtr			= ^TObject;
			TObjectPtrHandle		= ^TObjectPtrPtr;
END.
