{[a-,body+,h-,o=100,r+,rec+,t=4,u+,#+,j=20/57/1$,n-]}
{ UPascalObject.p }
{ Copyright  1984-1991  Apple Computer, Inc.  All rights reserved. }

UNIT UPascalObject;

	INTERFACE

		USES Types, MacAppTypes,
			{}
			 Memory;

		CONST
			kFailNone			= 1;
			kFailAbstract		= 2;
			kFailCoercion		= 3;
			kFailMethNotFound	= 4;

			kInvalidObj 		= '*Not an object*';	{ return value from LookupObjName if not an
														 object}
			kNilClass			= 0;					{ Value for superclass of the root object }

		TYPE
			TObject 			= OBJECT; EXTERNAL;

{$IFC qModelFarCode}
			ObjClassID			= LONGINT;				{ Object Class identifier. The first 4 bytes
														 of each object contain the class of the
														 object as an ObjClassID. }
{$ELSEC}
			ObjClassID			= kNilClass..16766; 	{ Object Class identifier. The first two
														 bytes of each object contain the class of
														 the object as an ObjClassID. }
{$EndC}

			ObjClassIDPtr		= ^ObjClassID;
			ObjClassIDHandle	= ^ObjClassIDPtr;

			ObjProcs			= RECORD				{ BottleNecks for object system }
				AllocateProc:		Ptr;				{ Routine to allocate objects }
				DisposeProc:		Ptr;				{ Routine to dispose objects }
				IsObjectProc:		Ptr;				{ Routine to test objectHood }
				END;

			ChangeID			= LONGINT;

		VAR
			{$Push}{$J+}
			gObjProcs:			ObjProcs;				{ Actual bottlenecks for object system. Set
														 in InitUObject }
			{$Pop}

		FUNCTION AllocateObjectsFromPerm(allocateFromPerm: BOOLEAN): BOOLEAN;
		{ Set to TRUE to make object allocation calls use Permanent memory (the default).
		Set to false to make object allocation calls use temporary memory (if they cannot be
		allowed to fail).  Returns old state }

		FUNCTION DisciplineMethodCalls(discipline: BOOLEAN): BOOLEAN;
		{ Set to true to discipline all method calls that go through the dispatcher. Function returns
		old setting. }

		PROCEDURE EachClassDo(PROCEDURE DoToClass(theClass: ObjClassID));
		{ Calls DoToClass for each class in the application }

		PROCEDURE EachSubClassDo(testClass: ObjClassID;
								 PROCEDURE DoToClass(theClass: ObjClassID));
		{ Calls DoToClass for subclass of the test class. Uses EachClassDo, so the order of iteration
		is not based on the class hierarchy but on the sequence of class IDs in the class table,
		Sorry }

		PROCEDURE EachSuperClassDo(testClass: ObjClassID;
								   PROCEDURE DoToClass(theClass: ObjClassID));
		{ Calls DoToClass for superclass of the test class. Order of iteration is in order of
		ancestry. }

		PROCEDURE FailNonObject(obj: UNIV TObject);
		{ called by dispatcher to discipline an object. Invokes failure if parameter is not IsObject
		}

		FUNCTION FreeIfObject(obj: TObject): TObject;
		{ IF obj <> NIL THEN obj.Free; useful for freeing an object that might sometimes be NIL.
		NIL Function result is handy for assigning back to the reference just freed (to help
		allieviate dangling pointerciples) }

		FUNCTION GetClassSizeFromID(classID: ObjClassID): Size;
		{ Given an object id, return the class's instantiation size. }

		PROCEDURE GetClassNameFromID(classID: ObjClassID;
									 VAR clName: MAName);
		{ Given an object id, return the object's name. ??? add error checking ??? }

		FUNCTION GetClassIDFromName(clName: MAName): ObjClassID;
		{ Given an object name, return its id. }

		FUNCTION GetClassID(obj: TObject): ObjClassID;
		{ Given an object, return the object's ClassID. }

		FUNCTION GetSuperClassID(objID: ObjClassID): ObjClassID;
		{ returns immediate superclass of class. Returns kNilClass for TObject. If we ever get MI
		this will have to be an enumerator. }

		PROCEDURE InitUObject;
		{ Essential one time initialization for this unit }

		FUNCTION IsObject(obj: UNIV TObject): BOOLEAN;
		{ Debugging check. Returns true if obj references a real object, false if obj is NIL or a
		non-object reference. This is not an absloutely sure-fire test of objectness, but does
		validate the following: (1) is it a handle? (2) is the handle non-purgeable? (3) is the
		class ID valid? (4) is the handle size at least as large as the class size? }

		FUNCTION IsClassIDMemberClass(testClass: ObjClassID;
									  superClass: ObjClassID): BOOLEAN;
		{ tests the testClass for MEMBERship in the superclass }

		FUNCTION IsMemberClassID(obj: TObject;
								 objID: ObjClassID): BOOLEAN;
	   { Returns true if obj references the class or a subclass of the class represented by objID. }

		PROCEDURE OBJFail(error: INTEGER);
		{ Entered due (presumably) to some object failing a test for objectness. There are two codes
		which result in a special message: kFailCoercion, which means that an object could not be
		coerced to a class due to not being a subclass of that class; and kFailMethNotFound, which
		means a method was called for a class, but that method is not defined for the class. Other
		codes are allowed but are not given special treatment. In qDebug mode a ProgramBreak
		preceeds the Failure. }

		FUNCTION NewObjectByClassId(classID: ObjClassID): TObject;
		{ Creates an object of the given class id }

		FUNCTION NewObjectByClassName(className: MAName): TObject;
		{ Creates an object of the given class name }

		FUNCTION VerboseIsObject(obj: UNIV TObject): BOOLEAN;
		{ Debugging check. Indicate whether the parameter appears to really be an object and emit
		diagnostics if it is not }

		PROCEDURE SetStdObjProcs(VAR theObjProcs: ObjProcs);
		{ Sets theObjProcs to point to the standard object system bottleneck procs }

END.
