{[a-,body+,h-,o=100,r+,rec+,t=4,u+,#+,j=20/57/1$,n+]}
{UPopup.p}
{Copyright  1988-1991 by Apple Computer Inc.  All rights reserved.}

{	OVERVIEW
	========
	This unit allows Popup menus to we instantiated which are managed by the Popup CDEF and MDEF
	that are included in System 7.0 and in the installer script for the Communications Toolbox.
	
	To use this Unit and the Popup CDEF with MacApp applications that run under System 6.x, the
	Popup CDEF and MDEF will need to be included in the application's resource fork.
	
	This unit actually extends the capabilities provided by the Popup CDEF as follows:
		allows for non resource-based MENUs (via a patch to GetResource)
		allows for hierarchical popups (via a patch to PopupMenuSelect)
	
	When a menu item from a CMNU menu is selected in the Popup, the Popup's DoEvent method then
	propagates the command number down the handler chain beginning with the Popup view:

			currentCommand := GetCurrentCommand;
			IF currentCommand <> cNoCommand THEN
				SELF.DoMenuCommand(currentCommand);

	It's still under debate whether the command should be propagated down the handler chain or
	down the target chain as follows:
				gApplication.GetTarget.DoMenuCommand(currentCommand)

	Note that this is a significant improvement over MacApp 2.0's popup support because the command
	number associated with a CMNU was not propagated down the handler chain.
	
	USAGE
	=====
	For Sys 6.0 support: include the Popup CDEF in the resource fork of your application with the
	following lines in your app.r file:
		include "Popup.rsrc";		/* included Popup CDEF and MDEF */
	
	Also, include either the CMNU or MENU resource in your application's resource fork.
	
	To handle the command number associated with the selected item in the popup, include code such
	as the following in the holding view (e.g. a TDialogView subclass) or in the document (this
	code was taken from DemoDialogs):

		PROCEDURE TTestDialogView.DoMenuCommand(aCmdNumber: CmdNumber); OVERRIDE;
		
			BEGIN
			CASE aCmdNumber OF
				cResetTextStyle:
					BEGIN
					END;
				cTextStyleChanged:
					BEGIN
					END;
				OTHERWISE
					INHERITED DoMenuCommand(aCmdNumber);
			END;
			END;
	
	LIMITATIONS
	===========
	The first item in the popup can't be a hierarchical menu. This is actually a bug in the Popup
	CDEF as it exists in System 7.0b1 and earlier versions (including the Comm Toolbox release). A
	bug report has been filed with SQA.
}

UNIT UPopup;

	INTERFACE

		USES
			{}
			Types, MacAppTypes, Memory, UPascalObject, UObject, AppleEvents, UEvtHandler, Balloons, UView, 
			UEvent, UCommand
			, Controls, TextEdit, UControl, Menus;

		CONST
			kUseAddResMenu	= TRUE;
			kDontUseAddResMenu	= NOT kUseAddResMenu;
			
		TYPE
		
			TDocument 	= OBJECT; EXTERNAL;

			TPopup 			= OBJECT (TCtlMgr)		{ Mimics a typical toolbox button as used in
														 a dialog. }

				fItemOffset:		INTEGER;
				fItsCurrentItem:	INTEGER;
				fTitleStyle:		INTEGER;			{ flags for both title's style & justification }
				fItsPopupMenuHandle:MenuHandle;
				fLastPopupResult:	Longint;

				{ === constructors/destructors === }
				PROCEDURE TPopup.Initialize; OVERRIDE;
				{ Puts a newly created TPopup into a known state. Subclasses should
				override it and call INHERITED. Typically used to put the object into a state where
				it can be safely FREED. }

				PROCEDURE TPopup.IPopup(itsSuperView: TView;
											itsLocation, itsSize: VPoint;
											itsHSizeDet, itsVSizeDet: SizeDeterminer;
											itsMenuID: INTEGER;				{ rsrc ID of 'MENU' or 'CMNU' rsrc }
											itsCurrentItem: INTEGER;
											itsItemOffset: INTEGER;			{ width of the popup title area }
											itsLabel: Str255;
											itsStyle: Style;				{ title's style }
											itsJust: INTEGER;				{ title's justification }
											useAddResMenu: BOOLEAN;			{ TRUE if want to AddResMenu (can use
																			this option iff menuID <> kNoResource) }
											useAddResMenuResType: ResType	{ used if useAddResMenu is TRUE });
				{ Initializes the popup. }

				FUNCTION TPopup.GetPopupTitleStyle(itsStyle: Style; itsJust: INTEGER): INTEGER;
				{ map itsStyle and itsJust to popupTitleStyle. }

				PROCEDURE TPopup.IRes(itsDocument: TDocument;
									   itsSuperView: TView;
									   VAR itsParams: Ptr); OVERRIDE;
				{ Initialize the view from a resource template. }

				PROCEDURE TPopup.CreateCMgrControl(itsBounds: VRect;
													itsTitle: Str255;
													itsVal, itsMin, itsMax: Longint;
													itsProcID: INTEGER); OVERRIDE;

				FUNCTION TPopup.Clone: TObject; OVERRIDE;
				{ calls INHERITED Clone and then clones owned objects }
				
				PROCEDURE TPopup.WRes(theResource: ViewRsrcHandle;
									   VAR itsParams: Ptr); OVERRIDE;
				{ Write this object out as a view resource. }

				PROCEDURE TPopup.WriteRes(theResource: ViewRsrcHandle;
										   VAR itsParams: Ptr); OVERRIDE;
				{ Set up the type and signature of this object and call WRes. }

				{ === imaging === }
				PROCEDURE TPopup.Draw(area: VRect); OVERRIDE;
				{ sets a patch to GetResource in certain cases }

				{ === tracking === }
				PROCEDURE TPopup.DoMouseCommand(VAR theMouse: VPoint;
												 event: TToolboxEvent;
												 hysteresis: Point); OVERRIDE;

				{ === accessors === }
				PROCEDURE TPopup.AttachMenuHandle(itsMenuHandle: MenuHandle);
				{ attaches itsMenuHandle to this popup }

				FUNCTION TPopup.GetMenuHandle: MenuHandle;
				{ returns the MenuHandle by accessing the control handle's private storage }
				
				FUNCTION TPopup.GetCurrentCommand: CmdNumber;
				{ For the currently selected menu item, returns its CmdNumber. }

				FUNCTION TPopup.GetCurrentItem : INTEGER;
				{ Returns the menu item of the last selected item. }

				PROCEDURE TPopup.GetItemText(item: INTEGER; VAR theText: Str255);
				{ Returns in "theText" the text for the specified item in the menu. }

				FUNCTION TPopup.GetMenuID : INTEGER;
				{ Returns the menu ID. }
				
				FUNCTION TPopup.GetNumberOfItems: INTEGER;
				{ Returns the number of items in the menu. }
				
				FUNCTION TPopup.GetRsrcID : INTEGER;
				{ Returns the resource ID, which is the same as the menuID. }

				PROCEDURE TPopup.GetTitle(VAR title: Str255);
				{ returns the popup's title }

				PROCEDURE TPopup.SetCurrentItem(item: INTEGER;
												redraw: BOOLEAN);
				{ Sets the currently selected item, redrawing if required. }
				
				PROCEDURE TPopup.SetPopup(itsMenuHandle: MenuHandle;
										  itsCurrentItem: INTEGER;
										  redraw: BOOLEAN);
				{ Sets the fields of the popup to contain theMenu, with currentItem selected,
				redrawing if required. }

				PROCEDURE TPopup.SetTitle(title: Str255);
				{ sets the popup's title }

				{ === inspector support === }
				PROCEDURE TPopup.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields. }

				END;
END.
