{[a-,body+,h-,o=100,r+,rec+,t=4,u+,#+,j=20/57/1$,n+]}
{ UPrintHandler.p }
{ Copyright  1984-1991 by Apple Computer Inc.	All rights reserved. }

UNIT UPrintHandler;

	INTERFACE

		USES
			{}
			Types, MacAppTypes, Memory, UPascalObject, UObject, AppleEvents, UBehavior, UViewBehavior;

		TYPE
			TView				= OBJECT; external;
			TDocument			= OBJECT; external;
			TPrintHandler		= OBJECT; forward;
			
			TPrintMenuBehavior	= OBJECT (TBehavior)
				fPrintHandler:		TPrintHandler;		{ the managed print handler }
				
				{ Initialization and termination }
				PROCEDURE TPrintMenuBehavior.Initialize; OVERRIDE;
				{ Puts a newly created TPrintMenuBehavior into a known state. Subclasses should
				override it and call INHERITED. Typically used to put the object into a state where
				it can be safely FREED. }
				
				PROCEDURE TPrintMenuBehavior.IPrintMenuBehavior (itsPrintHandler: TPrintHandler);
				{ initializes this behavior, and sets fPrintHandler to itsPrintHandler }

				PROCEDURE TPrintMenuBehavior.DoMenuCommand ( aCmdNumber: CmdNumber ); OVERRIDE;
				{ forwards into fPrintHandler's DoPrintCommand }
					
				PROCEDURE TPrintMenuBehavior.DoSetupMenus; OVERRIDE;
				{ forwards into fPrintHandler's DoSetupPrintMenus }

				END;
				

			TPrintHandler		= OBJECT (TViewBehavior)	{ Handles printing at a specified resolution
														 on behalf of a view. This class is really
														 a non-functional stub for programs that
														 don't use the printing building block. }
				fView:				TView;				{ The view whose printing is handled }
				fDocument:			TDocument;			{ the document printed by this handler }
				fDeviceRes: 		Point;				{ formal printer resolution, spots per inch
														 }
				fViewPerPage:		VPoint; 			{ The amount of the view that is visible on
														 a page }
				fFocusedPage:		longint;			{ The page number currently focused }

								{ Initialization and termination }
				PROCEDURE TPrintHandler.Initialize; OVERRIDE;
				{ Puts a newly created TStdPrintHandler into a known state. Subclasses should
				override it and call INHERITED. Typically used to put the object into a state where
				it can be safely FREED. }

				PROCEDURE TPrintHandler.IPrintHandler(itsView: TView);
				{ Initialize the printHandler, and associate it and 'itsView' with each other }

				PROCEDURE TPrintHandler.GetInspectorName(VAR inspectorName: Str255); OVERRIDE;
				{ Used by the inspector to get the name of this class. }

				PROCEDURE TPrintHandler.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields. }

				PROCEDURE TPrintHandler.FocusOnInterior;
				{ Stub }

				FUNCTION TPrintHandler.GetPrintInfo: Handle;
				{ Stub. Returns NIL in this case }

				PROCEDURE TPrintHandler.SetDefaultPrintInfo;
				{ Stub }

				{ Printing-related reformatting and computations }

				FUNCTION TPrintHandler.BreakFollowing(vhs: VHSelect; prevBreak: VCoordinate;
													  VAR automatic: BOOLEAN): VCoordinate;
				{ Returns the location of the page break which follows the page break located at
				'prevBreak', in direction vhs; returns automatic = TRUE if the page-break is
				thought of as an 'automatic' rather than a 'manual' (user-specified) one. Note that
				page-breaks in dimension 'v' are drawn as vertical lines, those in dimension 'h' as
				horizontal lines }

				PROCEDURE TPrintHandler.CalcPageStrips(VAR pageStrips: VPoint);
				{ Recalculate the number of strips of pages in each dimension }

				PROCEDURE TPrintHandler.CalcViewPerPage(VAR amtPerPage: VPoint);
			   { Computes the amount of view, in each dimension, to be allocated to a printed page }

				PROCEDURE TPrintHandler.CheckPrinter;
				{ See if there are changed printer-configuration parameters which need to be
				absorbed, and if so, absorb them }

				PROCEDURE TPrintHandler.LocatePageInterior(pageNumber: longint; VAR loc: VPoint);
				{ Decide where the top-left-most point of the the interior of the page should be }

				PROCEDURE TPrintHandler.PrinterChanged;
				{ The metrics relating to printer use have changed; absorb the information and react
				}

				PROCEDURE TPrintHandler.RedoPageBreaks;
				{ Recompute the dividing lines between areas of the view which will be mapped into
				different printed pages }

				PROCEDURE TPrintHandler.Reset;
				{ Resets the print handler to the default values }

				{ Printing-related commands }

				PROCEDURE TPrintHandler.Print(itsCmdNumber: CmdNumber; VAR proceed: BOOLEAN); {+}
				{ By default sets 'proceed' to TRUE and returns NIL. }

				FUNCTION TPrintHandler.SetupForFinder: BOOLEAN;
				{ Intended to set up the print handler for finder printing. }

				{ Printing-related screen feedback }

				PROCEDURE TPrintHandler.DrawPrintFeedback(area: VRect);
				{ Draw page-breaks, page-numbers, view-borders, rulers, etc. }

				PROCEDURE TPrintHandler.DrawPageBreak(vhs: VHSelect; whichBreak: longint;
													  loc: VCoordinate; automatic: BOOLEAN);
				{ Does nothing. }

				{ Actual printing }

				FUNCTION TPrintHandler.MaxPageNumber: longint;
				{ Returns the largest page number which could be reasonably printed, given the
				properties of the view; in cases like a SpreadSheet, this is not necessarily as
				large as the product (row strips) x (column strips) }

				PROCEDURE TPrintHandler.SetPageInterior(pageNumber: longint);
				{ Set up the pad-space Interior rectangle for the given page number }

				PROCEDURE TPrintHandler.SetPageOffset(coord: VPoint);
				{ Given the view coordinates of the top-left-most point of the view which is being
				mapped into the current page (in coord), this method's job is to compute the
				correct fRelOrigin field for the current page }

				END;
END.
