{[a-,body+,h-,o=100,r+,rec+,t=4,u+,#+,j=20/57/1$,n+]}
{ UScroller.p }
{ Copyright  1984-1991 by Apple Computer Inc.	All rights reserved. }

UNIT UScroller;

	INTERFACE

		USES
			{}
			Types, MacAppTypes, Memory, UPascalObject, UObject, AppleEvents, UEvtHandler, Balloons, UView;

TYPE

			TDocument			= OBJECT; EXTERNAL;
			TScrollerScrollBar	= OBJECT; EXTERNAL;
			TToolboxEvent		= OBJECT; EXTERNAL;

			ScrollerTemplate	= RECORD
				wantVSBar:			BOOLEAN;
				wantHSBar:			BOOLEAN;
				vertMax:			LONGINT;
				horzMax:			LONGINT;
				vScrollUnits:		INTEGER;
				hScrollUnits:		INTEGER;
				vConstrain: 		BOOLEAN;
				hConstrain: 		BOOLEAN;
				sBarOffsets:		Rect;
				END;
			ScrollerTemplatePtr = ^ScrollerTemplate;

			TScroller			= OBJECT (TView)		{ A view which can "scroll" its contents by
														 translating coordinates. }

				fScrollLimit:		VPoint; 			{ the 'limit' of the subviews that should be
														 scrollable }
				fMaxTranslation:	VPoint; 			{ the computed maximum amount of translation
														 taking into account the visible portion of
														 the view. This keeps us from scrolling the
														 bottom of a view to the top of a window }
				fScrollBars:		ARRAY [VHSelect] OF TScrollerScrollBar; { associated scroll bars if any
																	  }
				fScrollUnit:		VPoint;				{ increment of scroll }
				fConstrain: 		ARRAY [VHSelect] OF BOOLEAN; { should scrolling be constrained
																  to a multiple of my scrollunit }
				fSBarOffsets:		VRect;				{ how much to offset the associated scroll
														 bars from my bounds }
				fRespondsToFunctionKeys: BOOLEAN;		{ if it responds to PgUp/PgDn, etc. }

									 { Creation/Destruction Methods }

				PROCEDURE TScroller.Initialize; OVERRIDE;
				{ Puts a newly created TScroller into a known state. Subclasses should
				override it and call INHERITED. Typically used to put the object into a state where
				it can be safely FREED. }

				PROCEDURE TScroller.IScroller(itsSuperView: TView;
											  itsLocation, itsSize: VPoint;
											  itsHSizeDet, itsVSizeDet: SizeDeterminer;
											  itsMax: VPoint;
											  wantHorzSBar, wantVertSBar: BOOLEAN);
				{ Called to initialize a scroller procedurally. }

				PROCEDURE TScroller.CreateScrollBar(itsDirection: VHSelect);
				{ Called to create a scrollbar in 'itsDirection'. }

				PROCEDURE TScroller.IRes(itsDocument: TDocument;
										 itsSuperView: TView;
										 VAR itsParams: Ptr); OVERRIDE;
				{ Initialize a scroller from a resource template. }

				FUNCTION TScroller.Clone: TObject; OVERRIDE;
				{ calls INHERITED Clone and then clones owned objects }

				PROCEDURE TScroller.CreateTemplateScrollBar(itsDirection: VHSelect);
				{ Called to create a scrollbar in 'itsDirection' from a resource template. }

				PROCEDURE TScroller.WRes(theResource: ViewRsrcHandle;
										 VAR itsParams: Ptr); OVERRIDE;
				{ For writing a scroller view out to a resource. }

				PROCEDURE TScroller.WriteRes(theResource: ViewRsrcHandle;
											 VAR itsParams: Ptr); OVERRIDE;
				{ For writing a scroller to a resource, works in conjunction with tbe previous
				method. }

				PROCEDURE TScroller.Free; OVERRIDE;
				{ If there are attached scrollbars frees them before calling inherited Free. }

				{ SubView Management Methods }

				PROCEDURE TScroller.AddSubView(theSubView: TView); OVERRIDE;
				{ Calls inherited AddSubView and then calls 'theSubView' to BeInScroller passing SELF
				as the scroller to be in. }

				PROCEDURE TScroller.RemoveSubView(theSubView: TView); OVERRIDE;
				{ Calls BeInScroller with a parameter of NIL, for 'theSubView', before calling
				inherited RemoveSubView. }

				PROCEDURE TScroller.ForceRedraw; OVERRIDE;
				{ Invalidates the entire view forcing a redraw. }

				{ Size Methods }

				PROCEDURE TScroller.Resize(newSize: VPoint;
										   invalidate: BOOLEAN); OVERRIDE;
				{ Resizes the scroller to 'width' & 'height', redrawing if 'invalidate' is TRUE. }

				{ Location Methods }

				PROCEDURE TScroller.Locate(newLoc: VPoint;
										   invalidate: BOOLEAN); OVERRIDE;
				{ Calls inherited Locate before calling AdjustScrollbars. }

				PROCEDURE TScroller.AdjustScrollBars(invalidate: BOOLEAN);
				{ Resizes and re-Locates the scrollbars if required. }

				PROCEDURE TScroller.SubViewChangedSize(theSubView: TView;
													   delta: VPoint); OVERRIDE;
				{ Adjust the scroll limits based on the size change of the subviews. }

				PROCEDURE TScroller.SetScrollLimits(scrollLimit: VPoint;
													drawScrollBars: BOOLEAN);
				{ Sets the scroll of the attached scrollbars, if there are any. }

				PROCEDURE TScroller.SetScrollParameters(units: VPoint;
														horzConstraint, vertConstraint: BOOLEAN);
				{ Sets up the fScrollUnit and fConstrain fields of the scroller. }

				{ Scrolling Methods }

				PROCEDURE TScroller.HaveScrollBar(theScrollBar: TScrollerScrollBar;
												  direction: VHSelect);
				{ Sets the fScrollbar field in the given 'direction' to 'theScrollBar'. }

				PROCEDURE TScroller.ScrollDraw(delta: VPoint;
										   invalidate: BOOLEAN);
				{ Method responsible for actually doing the scrolling by calling moving bits. Called
				by DoScroll.  Invalidates newly revealed area if invalidate is true else updates
				the redraws the newly revealed area if invalidate is false.
				!!! NOTE this does not match the behaviour of other invalidate parameters because they
				interpret invalidate equals FALSE to neither draw nor invalidate.   Well we already
				know that the way that invalidate/redraw/don't-render control is handled needs _MAJOR_
				overhauling anyways Sigh! }

				PROCEDURE TScroller.DoScroll(delta: VPoint;
											 redraw: BOOLEAN);
				{ Method responsible for handling the scrolling, calls ScrollDraw if required. }

				FUNCTION TScroller.ScrollStep(vhs: VHSelect;
											  partCode: INTEGER): VCoordinate;
				{ Depending on the 'partCode' returns the amount to be scrolled. }

				FUNCTION TScroller.ScrollRelative(vhs: VHSelect;
												  sBarValue: VCoordinate): VCoordinate;
				{ Returns the relative amount to scroll. }

				PROCEDURE TScroller.ScrollBy(delta: VPoint;
											 redraw: BOOLEAN);
				{ Called to scroll by 'deltaH & 'deltaV'. }

				PROCEDURE TScroller.ScrollTo(scrollLoc: VPoint;
											 redraw: BOOLEAN);
				{ Scrolls to the required coordinates, calls SetLocalOrigin. }

				PROCEDURE TScroller.SetLocalOrigin(newOrigin: VPoint;
											 redraw: BOOLEAN); OVERRIDE;
				{ Sets the local origin by scrolling to the required coordinates, calls ScrollBy. }

				PROCEDURE TScroller.AutoScroll(viewPt: VPoint;
											   VAR delta: VPoint);
				{ Called during auto scrolling to return the delta to be scrolled. }

				PROCEDURE TScroller.RevealRect(rectToReveal: VRect;
											   minToSee: VPoint;
											   redraw: BOOLEAN); OVERRIDE;
				{ Shows 'rectToReveal' with the minimum amount to be seen set to 'minToSee' by
				calling ScrollBy. }

				{ Miscellaneous Methods }

				PROCEDURE TScroller.DoKeyCommand(event: TToolboxEvent); OVERRIDE;
				{ Handles when the Home, End, PageUp & PageDown keys are used. }

				FUNCTION TScroller.GetScroller(immediateSuperView: BOOLEAN): TScroller; OVERRIDE;
				{ Returns SELF. }

				{ Inspecting Methods }

				PROCEDURE TScroller.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields. }

				END;
END.
