{[a-,body+,h-,o=100,r+,rec+,t=4,u+,#+,j=20/57/1$,n+]}
{ USection.p }
{ Copyright  1984-1991 by Apple Computer Inc.	All rights reserved. }

{
	Unit: USection

	OVERVIEW
	========
	TSection is designed to provide Edition Manager support. TSection is an abstract superclass
	which references the main Edition Manager data structure (the SectionHandle), and a designator
	(designating which portion of the document is published/subscribed). Specialized subclasses
	of TSection are required to represet the TPublisher and TSubscriber.

	CLASS HIERARCHY
	===============
	TSection					abstract superclass for sections
		TPublisher				an Edition Manager publisher
		TSubscriber 			an Edition Manager subscriber
	
	TSectionStream				used for writing data to and reading data from an EditionContainerFile

	USAGE
	=====
	TPublisher/TSubscriber is designed to be used with the Edition Manager support that is built into
	MacApp. Whenever a portion of the document is publisher/subscribed, a TSection object is
	instantiated. The TSection object references a TDesignator.
}

UNIT USection;

	INTERFACE

		USES
			{}
			Types, MacAppTypes,
			{}
			Memory, UPascalObject, UObject, UStream,
			{}
			Editions, QuickDraw;

		CONST
			{---------------- CONSTANTS ----------------}
			kFound				= TRUE; 				{ the edition file was found }
			kNotFound			= FALSE;				{ the edition file wasn't found }

			{ passed into PROCEDURE TSection.CancelSection() }
			kCancel 			= TRUE; 				{ cancel the section }
			kDontCancel 		= FALSE;				{ don't cancel the section }

			kDesignatorResType	= 'DESG';				{ the ResType for saving a designator to a
														 resource }

			{---------------- Section Change ID's ----------------}
			cSubscriberChanged	= 610;					{ change id indicating that the subscriber
														 changed }
			cPublisherChanged	= 611;					{ change id indicating that the publisher
														 changed }

		TYPE

			TDesignator 		= OBJECT; external;
			TEditionDocument	= OBJECT; external;
			TFile		 		= OBJECT; external;
			TFileHandler		= OBJECT; external;

			{---------------- EDITION MANAGER SECTION CLASSES ----------------}

			{ TSection is an abstract superclass providing common support for sections - both
			subscribers and publishers. }
			TSection			= OBJECT (TObject)

				fDocument:			TEditionDocument;	{ the document that owns this section }

				fDesignator:		TDesignator;		{ the designated portion of the document }

				fSectionHandle: 	SectionHandle;		{ reference to the section record for this
														 edition }

				fRsrcID:			INTEGER;			{ rsrc id of stored form (needed for Read) }

				fLastEditionInfo:	EditionInfoRecord;	{ stored at various points for later use }

				fCanceled:			BOOLEAN;			{ whether this section is canceled }

				fDirty:				BOOLEAN;			{ section hasn't been saved }

				fFound: 			BOOLEAN;			{ whether this edition was found by
														 RegisterSection }

				fNewSection: 		BOOLEAN;			{ this is a brand spankin' new section }

				{ === contructors/destructors === }
				PROCEDURE TSection.ISection(itsDocument: TEditionDocument;
											itsDesignator: TDesignator;
											itsSectionHandle: SectionHandle; itsRsrcID: INTEGER);
				{ initializes the edition object }

				PROCEDURE TSection.Free; OVERRIDE;
				{ calls Unregister, disposes the SectionHandle and AliasHandle }

				{ === data accessors === }
				PROCEDURE TSection.AttachSectionHandle(itsSectionHandle: SectionHandle);
				{ attach a SectionHandle to this TSection object }

				PROCEDURE TSection.GetEditionName(VAR name: Str255);
				{ Returns the name of the edition container file. }

				FUNCTION TSection.GetFileHandler: TFileHandler;
				{ returns the TFileHandler of the document with which this section is associated }

				FUNCTION TSection.GetSectionType: SectionType;
				{ returns the fSectionHandle^^.kind }

				FUNCTION TSection.GetUpdateMode: UpdateMode;
				{ returns the fSectionHandle^^.mode }

				{ === saving/restoring === }
				PROCEDURE TSection.DoWrite(aFile: TFile; makingCopy: BOOLEAN);
				{ writes the section handle as a sect resource and the alias handle as a
				alis resource }

				PROCEDURE TSection.DoRead(aFile: TFile);
	 			{ reads the section handle from a sect resource and the alias handle from
				a alis resource }

				PROCEDURE TSection.DoNeedDiskSpace(VAR dataForkBytes, rsrcForkBytes: LONGINT);
				{ Bytes required to store saved sections on disk. When called, both parameters are
				set to 0 for you. }

				PROCEDURE TSection.Associate(newSectionDocument: FSSpecPtr);
				{ call this to associate the section handle with a new section document }

				{ === miscellaneous === }
				PROCEDURE TSection.Delete;
				{ removes the section resources }

				FUNCTION TSection.IsDirty: BOOLEAN;
				{ returns TRUE if this section is dirty }

				FUNCTION TSection.IsRegistered: BOOLEAN;
				{ returns TRUE if fRegistered & (IsRegisteredSection returns noErr) }
				
				FUNCTION TSection.IsSectionSelected: BOOLEAN;
		  		{ returns TRUE if the given section is in the user selection }

				PROCEDURE TSection.MarkDirty;
				{ marks this section as dirty so that it gets saved }

				FUNCTION TSection.Register: OSErr;
				{ calls RegisterSection on the Section Handle and sets fRegisteredSection }

				PROCEDURE TSection.UnRegister;
				{ calls UnRegisterSection on the Section Handle and sets fRegisteredSection }

				PROCEDURE TSection.UpdateEditionInfo;
				{ updates fEditionInfo for later use }

				PROCEDURE TSection.CancelSection(cancel: BOOLEAN);
				{ if cancel is TRUE then calls UnRegister and sets fCanceled to TRUE
				else if cancel is FALSE then calls Register and sets fCanceled to FALSE }

				PROCEDURE TSection.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields}
				END;

		   {---------------------------------------------------------------------------------------}
			TPublisher			= OBJECT (TSection)
				{ the publisher class - each instance of this class represents a part of the document
				that is published }

				{ === contructors/destructors === }
				PROCEDURE TPublisher.IPublisher(itsDocument: TEditionDocument;
												itsDesignator: TDesignator;
												itsSectionHandle: SectionHandle;
												itsRsrcID: INTEGER);
				{ initializes a publisher object }

				{ === change notification === }
				PROCEDURE TPublisher.DoUpdate(VAR theChange: ChangeID; changedObject: TObject;
											  changedBy: TObject); OVERRIDE;
				{ marks this publisher as changed calling MarkAsChanged }

				{ === saving/restoring === }
				PROCEDURE TPublisher.Publish;
				{ publishes this edition to the edition file }

				PROCEDURE TPublisher.DoWrite(aFile: TFile; makingCopy: BOOLEAN); OVERRIDE;
				{ calls INHERITTED Write
				if (mode is pumOnSave) & IsChanged then Publish }

				{ === miscellaneous === }
				PROCEDURE TPublisher.Delete; OVERRIDE;
				{ deletes the edition file; called when the doc is saved if the user canceled
				the publisher }

				PROCEDURE TPublisher.DeleteEditionFile;
				{ delete the edition container file }

				FUNCTION TPublisher.IsChanged: BOOLEAN;
				{ returns TRUE if the mdDate of the SectionRecord is newer than the mdDate of the
				edition on disk }

				PROCEDURE TPublisher.MarkAsChanged(mdDate: TimeStamp);
				{ marks this publisher as changed; should be called each time the user edits the data
				in the publisher }

				PROCEDURE TPublisher.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields}
				END;

		   {---------------------------------------------------------------------------------------}
			TSubscriber 		= OBJECT (TSection)
			{ the subscriber class - each instance represents a subscriber for this document }

				{ === contructors/destructors === }
				PROCEDURE TSubscriber.ISubscriber(itsDocument: TEditionDocument;
												  itsDesignator: TDesignator;
												  itsSectionHandle: SectionHandle;
												  itsRsrcID: INTEGER);
				{ initializes the subscriber object }

				{ === saving/restoring === }
				PROCEDURE TSubscriber.Subscribe;
				{ updates this subscriber from the edition on disk }

				PROCEDURE TSubscriber.SubscribeIfNewer;
				{ calls Subscribe if a newer edition is available }

				{ === miscellaneous === }
				FUNCTION TSubscriber.IsNewerEditionAvailable: BOOLEAN;
				{ return TRUE if there is a newer version of this edition on disk }

				PROCEDURE TSubscriber.OpenPublisher;
				{ calls GetEditionInfo & GotoPublisherSection }

				PROCEDURE TSubscriber.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields}
				END;

			TSectionStream 	= OBJECT (TStream)
			{ TSectionStream reads data from and writes data to an already open EditionContainerFile.
			 Should be enhanced to support streaming data concurrently of various OS Types (e.g. PICT,
			 TEXT, etc) rather than simply streaming one type at a time. }

				fEditionRefNum:		EditionRefNum;		{ refnum of EditionContainerFile }

				fEditionOSType:		OSType;				{ OSType to write to EditionContainerFile }

				fPosition:			LONGINT;			{ current position in the stream }

				fSize:				LONGINT;			{ current size of stream }

				PROCEDURE TSectionStream.Initialize; OVERRIDE;
				{ Puts a newly created TSectionStream into a known state. Subclasses should
				override it and call INHERITED. Typically used to put the object into a state where
				it can be safely FREED. }

				PROCEDURE TSectionStream.ISectionStream(itsEditionRefNum: EditionRefNum; itsEditionOSType: OSType);
				{ Initialize the TSectionStream. }

				PROCEDURE TSectionStream.Reset(itsEditionOSType: OSType; itsSize: LONGINT);
				{ Resets fPosition to 0, fEditionOSType to itsEditionOSType, and fSize to itsSize. }

				FUNCTION TSectionStream.GetPosition: LONGINT; OVERRIDE;
				{ Returns fPosition. }

				PROCEDURE TSectionStream.SetPosition(newPosition: LONGINT); OVERRIDE;
				{ Sets fPosition. }

				FUNCTION TSectionStream.GetSize: LONGINT; OVERRIDE;
				{ Returns fSize. }

				PROCEDURE TSectionStream.SetSize(newSize: LONGINT); OVERRIDE;
				{ Sets fSize. }

				PROCEDURE TSectionStream.ReadBytes(p: UNIV Ptr; count: LONGINT); OVERRIDE;
				{ Reads bytes from the Edition Container File. }

				PROCEDURE TSectionStream.WriteBytes(p: UNIV Ptr; count: LONGINT); OVERRIDE;
				{ Write bytes to the Edition Container File. }

				END;
END.
