{[a-,body+,h-,o=100,r+,rec+,t=4,u+,#+,j=20/57/1$,n+]}
{ UTECommands.p }
{ Copyright  1986-1991 Apple Computer, Inc.  All rights reserved. }

UNIT UTECommands;

	INTERFACE

		USES
			{}
			Types, MacAppTypes,
			{}
			Memory, UPascalObject, UObject,
			{}
			Events, AppleEvents, UEvtHandler,
			{}
			Balloons, UView,
			{}
			UEvent, UCommand,
			{}
			TextEdit,
			UTEView;

		TYPE

			TTECommand			= OBJECT (TCommand) 	{ A command that adds characters to, or
														 delete characters from, a TEView }

				fTEView:			TTEView;			{ The TEView operated on }

				fHTE:				TEHandle;			{ same as fTEView's fHTE; duplicated for
														 code efficiency }

				fOldStart:			INTEGER;			{ The beginning position of the selection at
														 the moment just before the command was
														 done. }
				fOldEnd:			INTEGER;			{ The end position of the selection at the
														 moment just before the command was done. }
				fOldText:			Handle; 			{ If fOldStart = fOldEnd, i.e., if old
														 selection had been an insertion point,
														 this will be NIL. Otherwise, provides a
														 temporary home for the characters
														 comprising the old selection }
				fOldStyles: 		StScrpHandle;

				fNewStart:			INTEGER;			{ The beginning location in the Text of the
														 new text that is added by the command, if
														 any. }
				fNewEnd:			INTEGER;			{ The ending location in the Text of the new
														 text that is added by the command, if any.
														 }
				fNewText:			Handle; 			{ A Handle to the characters added by the
														 command }
				fNewStyles: 		StScrpHandle;

				fPadding:			Handle; 			{ Handle to fill size between new and old.
														 This insures that we can always undo and
														 redo. }
				fTextPad:			INTEGER;			{ Size difference between New and Old text }
				fStylePad:			LONGINT;			{ Size difference between New and Old styles
														 }

								{ Initialize and Free }

				PROCEDURE TTECommand.Initialize; OVERRIDE;
				{ Puts a newly created TTECommand into a known state. Subclasses should
				override it and call INHERITED. Typically used to put the object into a state where
				it can be safely FREED. }

				PROCEDURE TTECommand.ITECommand(itsTEView: TTEView;
												itsCmdNumber: CmdNumber;
												itsSaveText: BOOLEAN);
				{ Initialize the command; if unsuccessful, exit is via Failure mechanism. }

				PROCEDURE TTECommand.Free; OVERRIDE;
				{ Free the text handles holding information needed for Undo/Redo, then Frees SELF }

				{ Command execution phase overrides }

				PROCEDURE TTECommand.DoIt; OVERRIDE;
				{ Focuses, then calls DoMainFunction }

				PROCEDURE TTECommand.RedoIt; OVERRIDE;
				{ Focuses, calls RestoreSelection to get selection right, then calls DoMainFunction
				to reinstate the changes done in DoIt which were undone by a preceding UndoIt }

				PROCEDURE TTECommand.UndoIt; OVERRIDE;
				{ Focuses, then dispatches to RemoveAdditions, ReviveDeletions, and RestoreSelection
				to accomplish the Undo }

				{ Command execution - Restoration of Selection -- called in both UNDO and REDO phases
				}

				PROCEDURE TTECommand.RestoreSelection;
				{ Set the Selection to be what it was just before the DO phase of the command was
				performed }

				{ Command execution - steps in DO and REDO phases }

				PROCEDURE TTECommand.BanishOldText;
				{ Remove text that was selected at the outset of the Do phase }

				PROCEDURE TTECommand.InstallNewText;
				{ Install the new text }

				PROCEDURE TTECommand.DoMainFunction;
				{ Forwards to the relevant methods for Do and Redo phases }

				{ Command execution - steps in UNDO phase }

				PROCEDURE TTECommand.RemoveAdditions;
				{ Remove any characters which were added by the DO phase of the command }

				PROCEDURE TTECommand.ReviveDeletions;
				{ Bring back the characters which were removed during the DO phase }

				{ Debugging }

				PROCEDURE TTECommand.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields. }

				END;

			TTECutCopyCommand	= OBJECT (TTECommand)	{ Command for handling Cut & Copy for the
														 TEView. }
				fClipCreated:		BOOLEAN;			{ Clipboard view created OK }

								{ Creation and Destruction }

				PROCEDURE TTECutCopyCommand.ITECutCopyCommand(itsTEView: TTEView;
															  itsCmdNumber: CmdNumber);
				{ Initializes the command }

				PROCEDURE TTECutCopyCommand.Free; OVERRIDE;
				{ Free the command }

				{ Command execution }

				PROCEDURE TTECutCopyCommand.DoIt; OVERRIDE;
				{ Launches a TEView for installation in the Clipboard, then calls DoMainFunction }

				PROCEDURE TTECutCopyCommand.ReviveDeletions; OVERRIDE;
				{ Bring back the characters which were removed during the DO phase; called during
				UNDO phase }

				{ Debugging }

				PROCEDURE TTECutCopyCommand.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields. }

				END;

			TTEPasteCommand 	= OBJECT (TTECommand)	{ Command that handles Pasting for the
														 TEView. }

			{ Initialization }
				PROCEDURE TTEPasteCommand.ITEPasteCommand(itsTEView: TTEView);
				{ Initialize the command; if unsuccessful, signalled by Failure mechanism }

				{ Debugging }

				PROCEDURE TTEPasteCommand.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields. }

				END;

			TTEStyleCommand 	= OBJECT (TTECommand)	{ Command for applying a new style to the
														 TEView. }

			{ These two fields are only used in non-styled TextEdit records }
				fMode:				INTEGER;			{ Mode for style change }
				fOldTextStyle:		TextStyle;			{ The original text style }
				fNewTextStyle:		TextStyle;			{ What we're replacing it with }

								{ Initialization }

				PROCEDURE TTEStyleCommand.ITEStyleCommand(itsTEView: TTEView;
														  itsNewStyle: TextStyle;
														  itsCmdNumber: CmdNumber;
														  itsMode: INTEGER);
				{ Initialize the command; if unsuccessful, signalled by Failure mechanism }

				{ Command execution }

				PROCEDURE TTEStyleCommand.InstallOneStyle(newStyl: TextStyle);
				{ Installs one style over entire record - for use with old TextEdit }

				PROCEDURE TTEStyleCommand.InstallManyStyles(newStyls: StScrpHandle);
				{ Installs styles over selection range - for use with styled TextEdit }

				PROCEDURE TTEStyleCommand.DoIt; OVERRIDE;
				{ Applies style to selection range }

				PROCEDURE TTEStyleCommand.UndoIt; OVERRIDE;
				{ Undoes the action performed by DoIt. }

				PROCEDURE TTEStyleCommand.RedoIt; OVERRIDE;
				{ Calls DoIt. }

				{ Debugging }
				PROCEDURE TTEStyleCommand.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields. }

				END;

			TTETypingCommand	= OBJECT (TTECommand)	{ The command that handles typing in a
														 TEView. }
				fCompleted: 		BOOLEAN;			{ Indicates whether further keystrokes will
														 be extensions to this command (FALSE) or
														 whether the command has already been
														 completed (TRUE) }
				fFirstChar: 		Char;				{ First character typed }

								{ Creation and destruction }

				PROCEDURE TTETypingCommand.ITETypingCommand(itsTEView: TTEView;
															itsFirstChar: Char);
				{ Initialize the command; if not successful, exit is via Failure mechanism }

				PROCEDURE TTETypingCommand.Free; OVERRIDE;
				{ Deallocate the command and dependent structures }

				{ Command processing }

				PROCEDURE TTETypingCommand.DoNormalChar(aChar: Char);
				{ Handle any typed character except a backspace }

				PROCEDURE TTETypingCommand.BkSpcLeft(theText: Handle;
													 curStart: INTEGER);
				{ Handle backspace to the left of the original selection }

				PROCEDURE TTETypingCommand.BkSpcRight(theText: Handle;
													  curStart: INTEGER);
				{ Handle backspace to the right of the original selection }

				PROCEDURE TTETypingCommand.FwdDelete(theText: Handle;
													 curStart, curEnd: INTEGER);
				{  Handle forward delete }

				PROCEDURE TTETypingCommand.AddCharacter(aChar: Char);
				{ Add another character to an already-launched TETypingCommand }

				PROCEDURE TTETypingCommand.DoIt; OVERRIDE;
				{ First processing for command; adds first character }

				PROCEDURE TTETypingCommand.RedoIt; OVERRIDE;
				{ Handles undoing forward backspace}

				PROCEDURE TTETypingCommand.UndoIt; OVERRIDE;
				{ Process undo typing }

				PROCEDURE TTETypingCommand.CompleteTyping;
				{ Mark as no more typing allowed and fix up style scrap, if any }

				{ Debugging }

				PROCEDURE TTETypingCommand.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields. }

				END;

{--------------------------------------------------------------------------------------------------}

		PROCEDURE DumpTECommand(theTTECommand: TTECommand);
		{ writes TTECommand info out to Debug window }

END.
