{[a-,body+,h-,o=100,r+,rec+,t=4,u+,#+,j=20/57/1$,n+]}
{=============================================================================
| Tabs for TE Views.
| by Steve Friedrich
| Adapted from code generously supplied by Tom Dowdy @ Apple
|
| KNOWN LIMITATIONS
|  only works for styled text but not with styles
|  slow on full recalcs when linewrap is true
|  won't print
}
{ Copyright  1986-1991 Apple Computer, Inc.  All rights reserved. }

UNIT UTabTEView;

	INTERFACE

		USES
			{}
			Types, MacAppTypes, Memory, UPascalObject, UObject, AppleEvents, UEvtHandler, Balloons, UView, 
			UEvent, UCommand
			, TextEdit, UTEView;

		CONST
			{ pretty pictures for "ShowInvisibles" }
			kVisibleTAB 		= '';
			kVisibleSpace		= '';
			kVisibleCR			= '';

			kDefaultTabSize 	= 24;					{ the default tab size in pixels }

			cIncTabs			= 1410;
			cDecTabs			= 1411;
			cShowInvs			= 1412;

		TYPE
			OldTEHooksRec		= RECORD				{ routines that can be hooked in TE }
				DrawHook:			ProcPtr;
				WidthHook:			ProcPtr;
				HitTestHook:		ProcPtr;
				EOLHook:			ProcPtr;
				END;

			TDocument			= OBJECT; EXTERNAL;
			TToolboxEvent		= OBJECT; EXTERNAL;

			TTabTEView			= OBJECT (TTEView)
				fTabSize:			integer;
				fShowInvisibles:	Boolean;
				fOldCQDProcs:		CQDProcs;			{ currently effective grafprocs }
				fHookedCQDProcsPtr: CQDProcsPtr;		{ Points to a hooked version of currently
														 effective grafprocs. }

				fOldTEHooksRec: 	OldTEHooksRec;		{ previous hooks returned by TECustomHook }
				

				PROCEDURE TTabTEView.IRes(itsDocument: TDocument;
										  itsSuperView: TView;
										  VAR itsParams: Ptr); OVERRIDE;
				PROCEDURE TTabTEView.BeInPort(itsPort: GrafPtr); OVERRIDE;

				PROCEDURE TTabTEView.SetTabSize(newSize: integer;
												redraw: Boolean);
				PROCEDURE TTabTEView.ShowInvisibles(showEm: Boolean;
													redraw: Boolean);
				FUNCTION TTabTEView.TETextLength(theText: Ptr;
												 theLength: integer;
												 theWidthBefore: integer;
												 VAR numer: Point;
												 VAR denom: Point;
												 VAR info: FontInfo): integer;
				PROCEDURE TTabTEView.DoMouseCommand(VAR theMouse: VPoint;
												   event: TToolboxEvent;
												   hysteresis: Point); OVERRIDE;
				PROCEDURE TTabTEView.DoSetupMenus; OVERRIDE;
				PROCEDURE TTabTEView.DoMenuCommand(aCmdNumber: CmdNumber); OVERRIDE;
				PROCEDURE TTabTEView.Fields(obj: TObject); OVERRIDE;
				FUNCTION TTabTEView.TETxMeas(theLength: integer;
											 theText: Ptr;
											 VAR numer: Point;
											 VAR denom: Point;
											 VAR info: FontInfo): integer;
				PROCEDURE TTabTEView.TEDrawText(theLength: integer;
												theText: Ptr;
												numer: Point;
												denom: Point);
				PROCEDURE TTabTEView.WithHookedGrafProcsDo(PROCEDURE whatToDo);
				END;

			TSetTabCommand		= OBJECT (TCommand)
				fOldTabSetting, fNewTabSetting: integer;
				PROCEDURE TSetTabCommand.ISetTabCommand(itsView: TView;
														itsCmdNumber: CmdNumber;
														tabSetting: integer);
				PROCEDURE TSetTabCommand.DoIt; OVERRIDE;
				{ sets new tabs, then calls ForceRedraw }

				PROCEDURE TSetTabCommand.RedoIt; OVERRIDE;
				{ calls DoIt }

				PROCEDURE TSetTabCommand.UndoIt; OVERRIDE;
				{ restores previous tab setting }

				{ Debugging }

				PROCEDURE TSetTabCommand.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields. }
				END;

			TShowInvsCommand	= OBJECT (TCommand)
				fInvsWereShown: 	Boolean;
				PROCEDURE TShowInvsCommand.IShowInvsCommand(itsView: TView;
															itsCmdNumber: CmdNumber;
															invsWereShown: Boolean);
				PROCEDURE TShowInvsCommand.DoIt; OVERRIDE;
				{ sets new tabs, then calls ForceRedraw }

				PROCEDURE TShowInvsCommand.RedoIt; OVERRIDE;
				{ calls DoIt }

				PROCEDURE TShowInvsCommand.UndoIt; OVERRIDE;
				{ restores previous tab setting }

				{ Debugging }

				PROCEDURE TShowInvsCommand.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields. }
				END;

		PROCEDURE InitUTabTEView;
		{ Initialize TabTEView unit. }
END.
