{[a-,body+,h-,o=100,r+,rec+,t=4,u+,#+,j=20/57/1$,n+]}
{ UView.p }
{ Copyright  1984-1991 by Apple Computer Inc.	All rights reserved. }

UNIT UView;

	INTERFACE

		USES
			Types, MacAppTypes, Memory, UPascalObject, UObject, Events, AppleEvents, UEvtHandler,
			Balloons, UAdorners;

		CONST
			kFrame				= [adnLineTop, adnLineLeft, adnLineBottom, adnLineRight]; { Constant
														defining that a view be completely framed. }

		TYPE
			TDesignator 		= OBJECT; EXTERNAL;
			TDocument			= OBJECT; EXTERNAL;
			TList				= OBJECT; EXTERNAL;
			TPrintHandler		= OBJECT; EXTERNAL;
			TScroller			= OBJECT; EXTERNAL;
			TWindow 			= OBJECT; EXTERNAL;
			TAdornment			= OBJECT; EXTERNAL;
			TAdorner			= OBJECT; EXTERNAL;
			TEvent				= OBJECT; EXTERNAL;
			TToolboxEvent		= OBJECT; EXTERNAL;

			{ V I E W  C O O R D I N A T E S }

			SizeDeterminer		=						{ Tells how a view's size is to be
														 determined; specified separately in each
														 dimension. }
			(sizeSuperView, 							{ View is the same size as its superview }
			 sizeRelSuperView,							{ View size is relative to the superview's
														 size }
			 sizePage,									{ View to be the size of one page }
			 sizeFillPages, 							{ View to grow upward to fill an exact
														 number of pages }
			 sizeVariable,								{ View size fluctuates according to
														 app-specific criteria }
			 sizeFixed);								{ No special default handling of size issues
														 }

			ViewTemplateSelector = (ViewType, ViewIncludeAt); { So C++ unions can have names }
			ViewTemplate		= PACKED RECORD
				itsParentID:		IDType;
				thisViewID: 		IDType;
				itsLocation:		VPoint;
				itsSize:			VPoint;
				itsVSizeDet:		SizeDeterminer;
				itsHSizeDet:		SizeDeterminer;
				isShown:			BOOLEAN;
				isEnabled:			BOOLEAN;
				filler: 			Byte;
				itsSignature:		IDType; 			{ Resource signature }
				CASE ViewTemplateSelector OF
					ViewType:
						(itsType:			 Str255);	{ Actually variable length }
					ViewIncludeAt:
						(includeRsrcID: 	 INTEGER;
						 itsSubViewOffset:	 VPoint);
				END;
			ViewTemplatePtr 	= ^ViewTemplate;

			ViewResource		= RECORD
				numViews:			INTEGER;
				theViews:			ViewTemplate;		{ Actually, open-ended }
				END;
			ViewRsrcPtr 		= ^ViewResource;
			ViewRsrcHandle		= ^ViewRsrcPtr; 		{ Preferred }

			{ Set by GetFocus; used by SetFocus. GetFocus/SetFocus are used to save and restore the focus
			behind MacApp's back. }
			ViewPortInfo			= RECORD	{ !!!???###SRF  silly intermediate name }
				org:				Point;
				port:				GrafPtr;
				clip:				RgnHandle;
				isValid:			BOOLEAN;			{ TRUE if FocusRec is valid }
				END;

			TView				= OBJECT; FORWARD;
			FocusRec			= RECORD
				printing:			BOOLEAN;
				drawingPictScrap:	BOOLEAN;
				drawingPictScrapView: TView;
				focusedView:		TView;
				itsViewPortInfo:		ViewPortInfo;
				END;

			TView				= OBJECT (TEvtHandler)	{ Renders an image of a document or other
														 data, and identifies mouse-clicked
														 objects. }
				fSuperView: 		TView;				{ the current superview }
				fSubViews:			TList;				{ subviews of this view }
				fDocument:			TDocument;			{ the associated document if any. }
				fLocation:			VPoint; 			{ the location in superview coordinates }
				fSize:				VPoint; 			{ the size of this view }
				fTranslation:		VPoint; 			{ translation of coordinates required
														to account for the origin of the view's
														local coordinate system. }

				fSizeDeterminer:	ARRAY [VHSelect] OF SizeDeterminer; { how this view's size is to
																		 be determined }
				fHLDesired: 		HLState;			{ the hilighting that is desired for this
														 view }
				
				fHandlesCursor:				BOOLEAN;	{ TRUE if the view tracks the mouse for
														 cursor setting }
				fLetsSubViewsHandleCursor: 	BOOLEAN;	{ TRUE if subviews should be given a chance
														to set the cursor }
				fHandlesHelp:				BOOLEAN;	{ TRUE if the view tracks the mouse for
														 showing help balloons }
				fLetsSubViewsHandleHelp:	BOOLEAN;	{ TRUE if subviews should be given a chance
														 to display help }
				fCursorID:			INTEGER;
				fHelpID:			INTEGER;
				fShown: 			BOOLEAN;			{ is this view to be shown }
				fPrintHandler:		TPrintHandler;		{ the associated print handler if any. }

				fQDOrigin:			Point; 				{ the computed QD origin for the port to
														account for the view's location. }
				fViewToQDOffset:	VPoint; 			{ the computed translation of coordinates
														necessary to account for adjustment of large
														views down to QD space. }
				fVisibleExtent:		VRect;				{ The extent of the view that could be
														"visible" through all its superviews (not
														taking clipping into account }
				fViewAdornment:			TAdornment;		{ manages a list of adorners }

								{ Creation/Destruction Methods }

				PROCEDURE TView.Initialize; OVERRIDE;
				{ Puts a newly created TView into a known state. Subclasses should
				override it and call INHERITED. Typically used to put the object into a state where
				it can be safely FREED. }

				PROCEDURE TView.IView(itsDocument: TDocument; itsSuperView: TView;
									  itsLocation: VPoint; itsSize: VPoint; itsHSizeDet,
									  itsVSizeDet: SizeDeterminer);
				{ Initializes a view procedurally. }

				PROCEDURE TView.IRes(itsDocument: TDocument; itsSuperView: TView;
									 VAR itsParams: Ptr);
				{ Initialize aview from a resource template. }

				FUNCTION TView.Clone: TObject; OVERRIDE;
				{ Calls INHERITED Clone and then clones owned objects. }

				PROCEDURE TView.WRes(theResource: ViewRsrcHandle; VAR itsParams: Ptr);
				{ For writing a view out to a resource. }

				PROCEDURE TView.WriteRes(theResource: ViewRsrcHandle; VAR itsParams: Ptr);
				{ For writing a view to a resource works in conjunction with tbe previous method. }

				PROCEDURE TView.Free; OVERRIDE;
				{ Frees its subviews, then if its superview is not NIL removes itself from the
				superviews list of subviews and if its document is not NIL removes itself from the
				documents list of views. If it has a print handler then it frees it, then calls
				inherited Free. Frees its fViewAdornment. }

				PROCEDURE TView.FreeFromClipboard;
				{ If its document is not NIL then calls the document's FreeFromClipboard method,
				otherwise calls Free. }

				{ Coordinate Conversion Methods }

				PROCEDURE TView.QDToViewPt(qdPoint: Point; VAR viewPt: VPoint);
				{ Convert a Quickdraw point to a view point. }

				PROCEDURE TView.QDToViewRect(qdRect: Rect; VAR viewRect: VRect);
				{ Convert a Quickdraw rect to a view rect. }

				FUNCTION TView.ViewToQDPt(viewPt: VPoint): Point;
				{ Convert a view point to a QuickDraw point. }

				PROCEDURE TView.ViewToQDRect(viewRect: VRect; VAR qdRect: Rect);
				{ Convert a view rect to a Quickdraw rect. }

				PROCEDURE TView.GetLocalOffsetInSuper(VAR localOffsetInSuper: VPoint);
				{ Returns the location of the origin for this view coordinate systemin the superview's
				coordinate system }

				PROCEDURE TView.SetLocalOrigin(newOrigin: VPoint; redraw: BOOLEAN);
				{ Sets the origin of the view's local coordinate system. Redraws if requested. }

				PROCEDURE TView.LocalToSuper(VAR thePoint: VPoint);
				{ Given a point in local view coordinates, returns the same point in its superview's
				coordinate system.}

				PROCEDURE TView.LocalToSuperVRect(VAR viewRect: VRect);
				{ Given a rect in local view coordinates, returns the same rect in its superview's
				coordinate system.}

				PROCEDURE TView.LocalToSuperRgn(VAR theRgn: RgnHandle);
				{ Given a region in local view coordinates, returns the same region in its superview's
				coordinate system.}

				PROCEDURE TView.SuperToLocal(VAR thePoint: VPoint);
				{ Given a point in superview coordinates, returns the same point in the view's own
				coordinate system. }

				PROCEDURE TView.SuperToLocalVRect(VAR viewRect: VRect);
				{ Given a rect in superview coordinates, returns the same rect in the view's own
				coordinate system. }

				PROCEDURE TView.SuperToLocalRgn(VAR theRgn: RgnHandle);
				{ Given a region in superview coordinates, returns the same region in  the view's own
				coordinate system. }

				PROCEDURE TView.LocalToWindow(VAR thePoint: VPoint);
				{ Given a point in local view coordinates, returns the same point in its window's (or
				most super superview's) coordinate system.}

				PROCEDURE TView.WindowToLocal(VAR thePoint: VPoint);
				{ Given a point in window (or most super superview) coordinates, returns the same
				point in the view's own coordinate system. }

				{ SubView Management Methods }

				PROCEDURE TView.EachSubView(PROCEDURE
											DoToSubView(theSubView: TView));
				{ Iterate over all the views subviews passing each in turn to the procedure
				"DoToSubView". }

				FUNCTION TView.FirstSubViewThat(FUNCTION
												TestSubView(theSubView: TView): BOOLEAN): TView;
				{ Iterate over this views subviews and return the first subview that meets
				'TestSubView'. }

				FUNCTION TView.LastSubViewThat(FUNCTION
											   TestSubView(theSubView: TView): BOOLEAN): TView;
				{ Iterate over this views subviews and return the last subview that meets
				"TestSubView". }

				PROCEDURE TView.TraverseSubViewsPreOrder(PROCEDURE
														 DoToSubView(theSubView: TView));
				{ Perform a pre-order traversal of all the view's subviews passing each
				in turn to the procedure "DoToSubView". }

				PROCEDURE TView.TraverseSubViewsPostOrder(PROCEDURE
														  DoToSubView(theSubView: TView));
				{ Perform a post-order traversal of all the view's subviews passing each
				in turn to the procedure "DoToSubView". }

				PROCEDURE TView.AddSubView(theSubView: TView);
				{ Add 'theSubView' to our list of subviews. }

				PROCEDURE TView.RemoveSubView(theSubView: TView);
				{ Remove 'theSubView' from our list of subviews. }

				PROCEDURE TView.MakeFirstSubView(theSubView: TView);
				{ Make 'theSubView ' the first subview in our list. }

				PROCEDURE TView.MakeLastSubView(theSubView: TView);
				{ Make 'theSubView' the last subview in our list. }

				FUNCTION TView.CountSubViews: INTEGER;
				{ Return the number of subviews in our view. }

				FUNCTION TView.FindSubView(itsIdentifier: IDType): TView;
				{ Find subview with identifier 'itsIdentifier' in our list of subviews. }

				{ Open/Close/Activate Methods }

				PROCEDURE TView.Activate(entering: BOOLEAN);
				{ Focuses and then highlights any current selection before calling all its subviews
				to activate themselves. }

				PROCEDURE TView.Open;
				{ Calls each of its subviews to Open themselves. }

				PROCEDURE TView.Close;
			   { Commits the last command before calling each of its subviews to Close themselves. }

				PROCEDURE TView.BeInScroller(itsScroller: TScroller);
				{ If 'itsScroller' is not NIL then sets its scroll limits to  its fSize. }

				PROCEDURE TView.BeInPort(itsPort: GrafPtr);
				{ If its print handler is not NIL, calls DoCheckPrinter. Then calls each of its
				subviews to BeInPort. }

				PROCEDURE TView.ShowReverted;
				{ First calls AdjustSize before forcing a redraw of the view, then call ShowReverted
				for each of its subviews. }

				{ Size Methods }

				PROCEDURE TView.AdjustSize;
				{ Calls ComputeSize, if the size has changed resizes itself and then calls
				DoPagination so that any page breaks can be recalculated. }

				PROCEDURE TView.CalcMinSize(VAR minSize: VPoint);
				{ Calculate the minimum size of SELF. }

				PROCEDURE TView.ComputeSize(VAR newSize: VPoint);
				{ Called by MacApp  to get the size of the view computed. }

				PROCEDURE TView.Resize(newSize: VPoint; invalidate: BOOLEAN);
				{ Resizes SELF to newSize, and forces an invalidation if 'invalidate' is
				TRUE. }

				PROCEDURE TView.SuperViewChangedSize(delta: VPoint; invalidate: BOOLEAN);
				{ Called when our superview changes size. }

				PROCEDURE TView.SubViewChangedSize(theSubView: TView; delta: VPoint);
				{ Called when one of our subviews changes size. }

				{ Location Methods }

				PROCEDURE TView.Locate(newLoc: VPoint; 
										invalidate: BOOLEAN);
				{ Moves our view to newLoc, invalidating if 'invalidate' is TRUE. }

				PROCEDURE TView.SubViewMoved(theSubView: TView);
				{ Called when a subview is moved. }

				PROCEDURE TView.SuperViewMoved(invalidate: BOOLEAN);
				{ Called when our superview is moved. }

				{ Focusing Methods }

				FUNCTION TView.Focus: BOOLEAN;
				{ Attempts to set up the drawing environment for this view. That consists of port,
				origin, gLongOffset and clipping. Returns FALSE if not able to focus. One reason
				focus could fail is if no superview is able to supply a port (ie not in a window
				and not printing or drawing pict scrap). Focus will return true, however, if the
				drawing environment can be setup but none of the view is currently visible due to
				clipping or scrolling by a superview. This lets you assemble view hierarchies and
				manipulate views without showing them to the user, but requires that you test
				IsVisible if you MUST know if the view is in fact visible. (MacApp will never ask a
				view to draw unless it is visible) }

				FUNCTION TView.FocusOnSuperView: BOOLEAN;
				{ Focuses on our superview. }

				PROCEDURE TView.InvalidateFocus;
				{ Indicates that the view's focus is no longer valid and must be recomputed. }

				FUNCTION TView.IsFocused: BOOLEAN;
				{ Check to see if we are currently focused. }

				PROCEDURE TView.ClipFurtherTo(r: Rect; hDeltaOrg, vDeltaOrg: INTEGER);
				{ Set clipping to: (<current clipping> INTERSECT r) OFFSET-BY (hDeltaOrg, vDeltaOrg).
				}

				PROCEDURE TView.OffsetAndClipFurtherTo(r: Rect; hDeltaOrg, vDeltaOrg: INTEGER);
				{ Set clipping to: OFFSET-BY (hDeltaOrg, vDeltaOrg) and (<current clipping> INTERSECT r).
				}

				PROCEDURE TView.AssumeFocused;
				{ Causes a ProgramBreak if the focused view isn't SELF }

				PROCEDURE TView.UpdateCoordinates;
				{ Indicates that the view's computed offsets are no longer valid and must be
				recomputed.  Calls updateCoordinates for all subviews. }

				{ Adornment Methods }

				PROCEDURE TView.AddAdorner(anAdorner: TAdorner; 
											itsPriority: AdornPriority;
											invalidate: BOOLEAN);
				{ adds an adorner to the view; invalidates the adorner if "invalidate" is TRUE }

				PROCEDURE TView.DeleteAdorner(anAdorner: TAdorner; invalidate: BOOLEAN);
				{ removes an adorner from the view; invalidates the adorner if "invalidate" is TRUE }

				PROCEDURE TView.DeleteAdornerByID(itsID: IDType;
											  invalidate: BOOLEAN);
				{ removes an adorner from the view; invalidates the adorner if "invalidate" is TRUE }

				PROCEDURE TView.Dim;
				{ dims the current image in the view }

				PROCEDURE TView.GetAdornExtent(VAR itsAdornExtent: VRect);
				{ returns the adorn extent for adorning this view }

				PROCEDURE TView.Hilite;
				{ hilites the current image in the view by inverting its extent }

				FUNCTION TView.MakeAdornment:TAdornment;
				{ creates and returns a TAdornment }

				{ Drawing Methods }

				PROCEDURE TView.DoHighlightSelection(fromHL, toHL: HLState);
				{ Currently does nothing. Must be overridden. }

				PROCEDURE TView.Draw(area: VRect);
				{ Currently does nothing. Must be overridden. }

				PROCEDURE TView.DrawContents;
				{ Causes the views contents to be drawn and that of any of its subviews. }

				FUNCTION TView.GetUserSelection: TDesignator;
				{ Returns the designator for the user selection (if any).  Defaults to returning
				the user selection of the document (if any) }

				PROCEDURE TView.SetUserSelection(newSelection: TDesignator);
				{ Sets the designator for the user selection (if any).	Defaults to setting
				the user selection of the document (if any) }

				PROCEDURE TView.UserSelectionChanged;
				{ Call this whenever the user's selection changes. Defaults to calling
				the UserSelectionChanged method of the document (if any). }

				FUNCTION TView.HasPendingUpdate: BOOLEAN;
				{ Returns TRUE if there is a pending update event for this view, FALSE if not. }

				PROCEDURE TView.Update;
				{ If its window is not NIL then does an update by calling DrawContents. }

				{ Validation/Invalidation Methods }

				PROCEDURE TView.ForceRedraw;
				{ Invalidates the entire view forcing a redraw. }

				PROCEDURE TView.InvalidateVRect(badVRect: VRect);
				{ Invalidates a Quickdraw rect by calling InvalidateRect. }

				PROCEDURE TView.InvalidateRect(badRect: Rect);
				{ Invalidates a View rect by calling InvalidateRgn. }

				PROCEDURE TView.InvalidateRgn(badRgn: RgnHandle);
				{ Intersects badRgn with the visible extent then calls DoInvalidateRgn with the result.
				We can only call InvalidRgn on a grafPort belonging to a window, if your code calls InvalidRgn
				on a non-window grafPort, it will die miserably. So views forward up
				their superview chain until they reach a view which overrides DoInvalidateRgn (usually a window)
				which then makes the actual InvalidRgn call }

				PROCEDURE TView.DoInvalidateRgn(badRgn: RgnHandle);
				{ Invalidates a Region by passing it to the superview }

				PROCEDURE TView.ValidateVRect(goodVRect: VRect);
				{ Validates a view rect by calling ValidateRect. }

				PROCEDURE TView.ValidateRect(goodRect: Rect);
				{ Validates a Quickdraw rect by calling ValidateRgn. }

				PROCEDURE TView.ValidateRgn(goodRgn: RgnHandle);
				{ Intersects goodRgn with the visible extent then calls DoValidateRgn with the result.
				We can only call ValidRgn on a grafPort belonging to a window, if your code calls ValidRgn
				on a non-window grafPort, it will die miserably. So views forward up
				their superview chain until they reach a view which overrides DoValidateRgn (usually a window)
				which then makes the actual ValidRgn call }

				PROCEDURE TView.DoValidateRgn(goodRgn: RgnHandle);
				{ Validates a Region by passing it to the superview }

				PROCEDURE TView.RevealRect(rectToReveal: VRect; minToSee: VPoint; redraw: BOOLEAN);
				{ Scrolls a rect in to view, with 'minToSee' being the point which will end up at the
				bottom of the view, i.e.the minimum amount to be seen and 'rectToReveal' the rect
				being scrolled. }

				PROCEDURE TView.RevealTop(redraw: BOOLEAN);
				{ Scroll the top of a rect into view. }

				PROCEDURE TView.RevealBottom(redraw: BOOLEAN);
				{ Scroll the bottom of a rect into view. }

				{ Mouse Handling Methods }

				FUNCTION TView.ContainsMouse(theMouse: VPoint): BOOLEAN;
				{ Is the view point 'theMouse' currently in our view. }

				FUNCTION TView.HandleMouseDown(theMouse: VPoint; event: TToolboxEvent;
											   hysteresis: Point): BOOLEAN;
				{ The method responsible for handling a mouse down in the view, calls DoMouseCommand
				to handle it. }

				PROCEDURE TView.DoMouseCommand(VAR theMouse: VPoint; event: TToolboxEvent;
											   hysteresis: Point);
				{ Posts a simple tracker which by default calls back tracking methods of this view.
				There are two possibilities here:
				1)	Override DoMouseCommand to post commands that represent more complex, even
				undoable actions.  That command may either let its tracking call back the view's
				tracking methods (the default) which must be overridden to do anything useful or it
				may simply do all the tracking itself.	This mostly depends on whether different
				commands returned from the view would have different tracking behavior.  For example
				this is the approach taken by DrawShapes.
				2) Override the view's tracking methods and let the default tracker them.  For example
				this is the approach taken by TControl. }

				{ Cursor and Help Handling Methods }
				PROCEDURE TView.ExcludeSubViewRegions(aRegion: RgnHandle; FUNCTION
													  TestSubView(aView: TView): BOOLEAN);
				{ Reduces aRegion by the extents of all subviews for which TestSubView returns
				TRUE. Used by TView.GetDefaultCursorRgn and TView.GetDefaultHelpRgn }

				FUNCTION TView.HandlesCursor: BOOLEAN;
				{ Returns TRUE, if this view tracks the mouse so that it can claim the cursor. 
				The default returns TRUE. If FALSE, subviews may still get a chance to set the
				cursor, as determined by TView.SubViewsHandleCursor. }
				
				FUNCTION TView.LetsSubViewsHandleCursor: BOOLEAN;
				{ Returns TRUE if this view should allow subviews a chance to set the cursor. 
				If TRUE, the subviews themselves may still return FALSE for HandlesCursor. By setting 
				both values false, cursor tracking can be turned off for an entire window, such 
				as the clipboard window. }

				FUNCTION TView.HandlesHelp: BOOLEAN;
				{ Returns TRUE, if this view tracks the mouse so that it can show help. 
				The default returns TRUE. If FALSE, subviews may still get a chance to show
				help, as determined by TView.SubViewsHandleHelp. }

				FUNCTION TView.LetsSubViewsHandleHelp: BOOLEAN;
				{ Returns TRUE if this view should allow subviews a chance to show help. 
				The subviews themselves may still return FALSE for HandleHelps. By setting 
				both values false, balloon help can be turned off for an entire view hierarchy }

				FUNCTION TView.HandleCursor(theMouse: VPoint; cursorRgn: RgnHandle): TView;
				{ Handles the setting of the cursor by calling our DoSetCursor method and the same
				For all our subviews. }

				FUNCTION TView.HandleHelp(theMouse: VPoint; helpRgn: RgnHandle): TView;
				{ !!! }

				PROCEDURE TView.DoSetCursor(localPoint: VPoint; cursorRgn: RgnHandle);
				{ Can be overridden to set the cursor and compute a cursor region.
				The default sets the cursor to an arrow and uses TView.GetDefaultCursorRgn
				to compute the largest appropriate cursor region }

				PROCEDURE TView.GetHelpParameters(localPoint: VPoint; helpRgn: RgnHandle;
												  VAR helpMessage: HMMessageRecord;
												  VAR localQDTip: Point; VAR localQDRect: Rect;
												  VAR balloonVariant: INTEGER);
				{ Determine the parameters needed for showing a help balloon for
				this view. The default gets the data from a resource of type 'hrct'
				with id fHelpID. }

				PROCEDURE TView.DoShowBalloon(localPoint: VPoint; helpRgn: RgnHandle;
											  helpMessage: HMMessageRecord; localQDTip: Point;
											  localQDRect: Rect; balloonVariant: INTEGER);

				PROCEDURE TView.DoShowHelp(localPoint: VPoint; helpRgn: RgnHandle);
				{ !!! }

				PROCEDURE TView.GetDefaultCursorRgn(localPoint: VPoint; cursorRgn: RgnHandle);
				{ Returns the largest appropriate cursor region, called by DoSetCursor. }

				PROCEDURE TView.GetDefaultHelpRgn(localPoint: VPoint; helpRgn: RgnHandle);
				{ !!! }
				{ Miscellaneous Methods }

				FUNCTION TView.GetDialogView: TView;	{ actually, TDialogView }
				{ If our superview is not NIL then calls its GetDialogView method and returns the
				result, otherwise returns NIL. }

				FUNCTION TView.GetGrafPort: GrafPtr;
				{ Returns a pointer to our Graf Port. }

				FUNCTION TView.GetScroller(immediateSuperView: BOOLEAN): TScroller;
				{ Returns a reference to our scroller. }

				FUNCTION TView.GetRootView: TView;
				{ Returns a reference to the root view. }

				FUNCTION TView.GetWindow: TWindow;
				{ Returns a reference to the window that we are in, if any. }

				PROCEDURE TView.GetExtent(VAR itsExtent: VRect);
				{ Returns a view rect the size of our extent in 'itsExtent'. }

				PROCEDURE TView.GetQDExtent(VAR qdExtent: Rect);
				{ Returns a Quickdraw rect the size of our extent in 'qdExtent'. }

				PROCEDURE TView.GetFrame(VAR itsFrame: VRect);
				{ Returns in'itsFrame' a VRect that is the boundary of our view in superview
				coordinates. This is calculated by adding its size to its location. }

				PROCEDURE TView.GetVisibleRect(VAR visRect: VRect);
				{ Returns a rect enclosing the visible portion of the view in view coordinates. }

				PROCEDURE TView.GetVisibleQDRect(VAR visQDRect: Rect);
				{ Returns a rect enclosing the visible portion of the view in QD coordinates. }

				PROCEDURE TView.GetDrawableRect(VAR drawRect: VRect);
				{ Returns a rect enclosing the drawable portion of the view in view coordinates. }

				PROCEDURE TView.GetDrawableQDRect(VAR drawQDRect: Rect);
				{ Returns a rect enclosing the drawable portion of the view in QD coordinates. }

				FUNCTION TView.GetPartCode(theMouse: VPoint): INTEGER;
				{ tells which part of a view, if any, the mouse button was pressed in. Returns:
				inContent = 3, inDrag = 4, inGrow = 5, inGoAway = 6, or inDesk = 0 (ie. none of the
				above),  depending on where theMouse is in the view }

				PROCEDURE TView.ViewEnable(state, redraw: BOOLEAN);
				{ Enables & Redraws the view based on 'state' & 'redraw'. }

				PROCEDURE TView.Show(state, redraw: BOOLEAN);
				{ If 'state' is not fShown then set fShown to TRUE and call ForceRedraw if 'redraw'
				is TRUE. Otherwise invalidate our focus and set fShown to 'state'. }

				FUNCTION TView.isShown: BOOLEAN;
				{ Checks to see if our view is currently shown. }

				FUNCTION TView.IsVisible: BOOLEAN;
				{ Returns TRUE if any portion of the view is currently visible when focused }

				FUNCTION TView.IsDrawable: BOOLEAN;
				{ Returns TRUE if any portion of the view is currently drawable when focused }

				FUNCTION TView.IsActive: BOOLEAN;
				{ Returns TRUE if the view is in an "active" superview (typically a window) }

				{ Target Management }
				
				PROCEDURE TView.SetTarget(newTarget: TEvtHandler);
				{ Set the target for this view's layer (i.e. window). Forwards to fSuperView }

				PROCEDURE TView.BeTarget;
				{ Attempt to make the receiver the target }
				
				{ Clipboard Handling }

				FUNCTION TView.AsPict: PicHandle;
				{ Returns a PicHandle containing a rendering of the view and its subviews. }

				FUNCTION TView.ContainsClipType(aType: ResType): BOOLEAN;
				{ Checks to see if the clipboard contains the type 'aType'. }

				FUNCTION TView.GivePasteData(aDataHandle: Handle; dataType: ResType): LONGINT;
			  { Gets the scrap data of the type 'dataType', returns any errors as function result. }

				PROCEDURE TView.WriteToDeskScrap;
				{ Writes to desk scrap. }

				{ PrintingMethods }

				PROCEDURE TView.AttachPrintHandler(itsPrintHandler: TPrintHandler);
				{ Called by the PrintHandler during its initialization, NOT directly by you }

				FUNCTION TView.DoBreakFollowing(vhs: VHSelect; prevBreak: VCoordinate;
												VAR automatic: BOOLEAN): VCoordinate;
				{ Returns the location of the page break which follows the page break located at
				'prevBreak', in direction vhs; returns automatic = TRUE if the page-break is
				thought of as an 'automatic' rather than a 'manual' (user-specified) one. Note that
				page-breaks in dimension 'v' are drawn as vertical lines, those in dimension 'h' as
				horizontal lines }

				PROCEDURE TView.DoCalcPageStrips(VAR pageStrips: VPoint);
				{ Computes the number of horizontal and vertical page strips. }

				PROCEDURE TView.DoCalcViewPerPage(VAR viewPerPage: VPoint);
				{ Determine how much of the view normally goes into each printed page, in each
				dimension. }

				PROCEDURE TView.DoCheckPrinter;
				{ Check whether printer (if relevant) has changed, and if so, take appropriate action
				}

				PROCEDURE TView.DoDrawPrintFeedback(area: VRect);
				{ Draw page-breaks, page-numbers, view-borders, rulers, etc. }

				PROCEDURE TView.DoDrawPageBreak(vhs: VHSelect; whichBreak: INTEGER;
												loc: VCoordinate; automatic: BOOLEAN);
				{ Draw one page break. }

				PROCEDURE TView.DoPagination;
				{ Recompute the dividing lines between areas of the view which will be mapped into
				different printed pages }

				PROCEDURE TView.DoPrinterChanged;
				{ The metrics relating to printer use have changed; absorb the information and react
				}

				PROCEDURE TView.DoSetPageOffset(coord: VPoint);
				{ Only reimplement in very unusual circumstances involving Tall-UnAdjusted text
				printing etc. }

				PROCEDURE TView.GetPrintExtent(VAR printExtent: VRect);
				{ Returns the part of the view that is to be printed. The default is to return the
				view's extent. }

				PROCEDURE TView.PageInteriorChanged(newInterior: VRect);
				{ Allows a view which is printable to react to the changing of the page-interior
				rectangle }

				{ Inspecting Methods }

				PROCEDURE TView.Fields(obj: TObject); OVERRIDE;
				{ Called by the Inspector and the Debugger to display the contents of this class's
				fields. }

				PROCEDURE TView.GetInspectorName(VAR inspectorName: Str255); OVERRIDE;
				{ Called by the Inspector to get the name of this class. }

				{ View Tracking Methods }

				PROCEDURE TView.GoAwayByUser(theMouse: VPoint);
				{ Provides tracking for the GoAway box (or other indicator) and calls
				CloseByUser if go away was indicated. }

				PROCEDURE TView.MoveByUser(theMouse: VPoint);
				{ Provides tracking for the Title bar (or other drag area) and calls
				Locate if the location changed. }

				PROCEDURE TView.ResizeByUser(theMouse: VPoint);
				{ Provides tracking for the Resize box (or other indicator) and calls
				Resize if the size changed. }

				PROCEDURE TView.ZoomByUser(theMouse: VPoint; partCode: INTEGER);
				{ Provides tracking for the Zoom box (or other indicator) and calls
				Zoom if zoom was indicated. }

				{ Tracker Tracking Methods }

				FUNCTION TView.IsDoneTracking: BOOLEAN;
				{ Indicates whether the Tracker is through tracking.  (the command is also through
				tracking if you return NIL from TrackMouse).  Defaults to NOT StillDown.
				Most programs won't have to mess with this.
				NOTE you still have to deal with queued events if you change the criteria for TRUE. }

				PROCEDURE TView.TrackConstrain(aTrackPhase: TrackPhase;
											  anchorPoint: VPoint;
											  previousPoint: VPoint;
											  VAR nextPoint: VPoint;
											  mouseDidMove: BOOLEAN);
				{ Override this if your Tracker calls back to the view (the Default) an you want to
				constrain the mouse point to a grid, force drawing a square, etc. This is called only
				if fConstrainsMouse is TRUE. }

				PROCEDURE TView.TrackFeedback(aTrackPhase: TrackPhase;
											  anchorPoint: VPoint;
											  previousPoint: VPoint;
											  nextPoint: VPoint;
											  mouseDidMove: BOOLEAN;
											  turnItOn: BOOLEAN);
				{ Override this if your Tracker calls back to the view (the Default) an you want to
				display any tracking feedback }

				PROCEDURE TView.TrackMouse(aTrackPhase: TrackPhase;
											  VAR anchorPoint: VPoint;
											  VAR previousPoint: VPoint;
											  VAR nextPoint: VPoint;
											  mouseDidMove: BOOLEAN);
				{ Override this if your Tracker calls back to the view (the Default) an you want to
				display any tracking feedback }

				{ View hierarchy management }
				PROCEDURE TView.AddedASubView(newSubView: TView);
				{ Called by AddSubView to notify this view that a subview was added.
				Calls fSuperView.AddedASubView so that the notification is passed on
				to all superviews }

				PROCEDURE TView.RemovedASubView(theSubView: TView);
				{ Called by RemoveSubView to notify this view that a subview was removed.
				Calls fSuperView.RemovedASubView so that the notification is passed on
				to all superviews }

				PROCEDURE TView.BeInSuperView(itsSuperView: TView);
				{ Called to notify this view that it has become a subview of theSuperView.
				theSuperView can be NIL. Called by TView.AddSubView and TView.RemoveSubView }

				PROCEDURE TView.SetHLPenState(fromHL, toHL: HLState);
				{ Set the pen state for highlighting by XOR, given the highlight transition. You can use this
				if you use Paint<X> or Frame<X> or line drawing for your highlighting. }
		   { ??? Add a pattern parameter or make a helper routine that has a pattern parameter ??? }
				END;

		VAR
			{$Push} 									{$J+}
			gFocusedView:		TView;					{ the view that is currently focused }
			gDrawingPictScrap:	BOOLEAN;				{ TRUE if a view's Draw routine is being
														 called in order to create PICT data in the
														 Desk Scrap; your View.Draw can check this
														 routine, and insert PictComments as
														 appropriate if it wishes to have them in
														 the Picture in the Desk Scrap }
			gDrawingPictScrapView: TView;				{ the view being currently drawn in the PICT
														 scrap ??? this stuff is a good candidate
														 for a small change to the view
														 architecture to more easily allow
														 providing alternate drawing environments }
			gPrinting:			BOOLEAN;				{ true if currently Printing }
			gCurrPrintHandler:	TPrintHandler;			{ If printing, this is set to the print
														 handler. Nil if not printing. }
			gNullPrintHandler:	TPrintHandler;			{ handles printing-relating messages for
														 views which don't print }
			gPageOffset:		VPoint; 				{ offset in view of page being printed }
			gPrintHandler:		TPrintHandler;			{ a global print-handler object for use in
														 some standard printing-related activities;
														 this is initialized to be just a reference
														 to gNullPrintHandler, but if you call
														 InitPrinting, that will install a
														 non-trivial print-handler here }
			gWResSignature: 	IDType; 				{ Signature used by TView.WRes }
			gWResType:			Str255; 				{ Type used by TView.WRes }
			{$Pop}

			{ V I E W	R E S O U R C E   U T I L I T I E S }

		FUNCTION NewViewRsrc(VAR p: UNIV Ptr): ViewRsrcHandle;
		{ Convenience routine to create a new view resource template handle. Passes back in "p" the
		pointer to the first template entry. }

		PROCEDURE DoneViewRsrc(viewRsrc: UNIV Handle; lastPtr: UNIV LONGINT);
		{ Cuts back the handle to the proper length, based on lastPtr which should be the address of
		the next template entry. }

		FUNCTION ExpandPtr(viewRsrc: UNIV Handle; VAR p: UNIV LONGINT; offset: LONGINT): Ptr;
		{ Bumps size of view resource if necessary to add a new template. Amount to bump is set to at
		least kViewRsrcExpandAmt over the current size. "p" is offset to point to the next
		available position for a new template. Returns the old value of p before it was updated. }

		FUNCTION ExpandPtrWStr(viewRsrc: UNIV Handle; VAR p: UNIV LONGINT; offset,
							   len: LONGINT): Ptr;
		{ Bumps size of view resource if necessary to add a new template which ends with a
		variable length string, the length of which is passed in len.
		Returns the old value of p before it was updated. }

		PROCEDURE OffsetPtr(VAR p: UNIV LONGINT; offset: LONGINT);
		{ Adds offset to p, forcing word alignment.}

		PROCEDURE OffsetPtrWStr(VAR p: UNIV LONGINT; offset: LONGINT);
		{ Adds offset to p, taking length of trailing variable length string into account, forcing
		word alignment.}

		PROCEDURE GetFocus(VAR theFocusRec: FocusRec);
		{ gets the current focus into a focusrec. The theFocusRec.itsPortInfo.clipRgn must be a valid
		rgn.  Useful for saving and restoring the focus behind MacApp's back.}

		PROCEDURE SetFocus(theFocusRec: FocusRec);
		{ sets the current focus from a focus rec }

		PROCEDURE GetViewPortInfo(VAR theViewPortInfo: ViewPortInfo);
		{ gets the current PortInfo into a ViewPortInfo. theViewPortInfo.clipRgn must be a valid rgn. }

		PROCEDURE SetViewPortInfo(theViewPortInfo: ViewPortInfo);
		{ sets the current PortInfo from a ViewPortInfo }

		{ V i e w  S y s t e m	U T I L I T I E S }

		FUNCTION IsRectVisible(r: Rect): BOOLEAN;
		{ Determine if a rect is visible in the current grafport. }

		FUNCTION IsRectDrawable(r: Rect): BOOLEAN;
		{ Determine if any portion of a rect intersects the ports visRgn AND clipRgn. }

		PROCEDURE VisibleRect(VAR r: Rect);
		{ trims the supplied rect down to its intersection with the port's visRgn }

		PROCEDURE DrawableRect(VAR r: Rect);
		{ trims the supplied rect down to its intersection with the port's visRgn AND clipRgn. }

		FUNCTION IsPtVisible(pt: Point): BOOLEAN;
		{ Determine if a pt intersects the ports visRgn }

		FUNCTION IsPtDrawable(pt: Point): BOOLEAN;
		{ Determine if a pt intersects the ports visRgn AND clipRgn. }

		PROCEDURE PtAndRgn(aPoint: Point; aRegion: RgnHandle);
		{ Adds the given point to the given region }

		PROCEDURE VRectAndRgn(theVRect: VRect; theRgn: RgnHandle);
		{ Call this procedure to add theVRect to theRgn. }

		PROCEDURE WriteFocus;
		{ Writes information about the current focus to the output }
END.
