{[a-,body+,h-,o=100,r+,rec+,t=4,u+,#+,j=20/57/1$,n+]}
{ UViewServer.p }
{ Copyright  1984-1991 by Apple Computer Inc.	All rights reserved. }

UNIT UViewServer;

	INTERFACE

		USES
			Types, MacAppTypes, Memory, UPascalObject, UObject;

		CONST
			{ argument of NewPaletteWindow }
			kLeftPalette		= h;
			kTopPalette 		= v;

		TYPE

			TView				= OBJECT; external;
			TWindow 			= OBJECT; external;
			TDocument			= OBJECT; external;

			TViewServer			= OBJECT (TObject)		{ TViewServer is responsible for creating views. }

				PROCEDURE TViewServer.Initialize; OVERRIDE;
				{ Puts a newly created TViewServer into a known state. Subclasses should
				override it and call INHERITED. Typically used to put the object into a state where
				it can be safely FREED. }

				PROCEDURE TViewServer.IViewServer;
				{ Initializes the TViewServer. }

				PROCEDURE TViewServer.Free; OVERRIDE;
				{ Sets the global gViewServer to NIL. }

				{ ------------------------------------------------------------------------------------	}
				{ Procedural View Creation																}
				{ ------------------------------------------------------------------------------------	}
			
				FUNCTION TViewServer.NewPaletteWindow(itsRsrcID: INTEGER; wantHScrollBar, wantVScrollBar: BOOLEAN;
										  itsDocument: TDocument; itsMainView: TView; itsPaletteView: TView;
										  sizePalette: INTEGER; whichWay: VHSelect): TWindow;
				{ Used for creating MacDraw-like windows with 1 non-scrolling palette along the left edge
				(whichWay = h), or a non-scrolling status area at the top of the window (whichWay = v), and
				a main view that may or may not scroll. Signals Failure if the window could not be created.
				}
		
				FUNCTION TViewServer.NewSimpleWindow(itsRsrcID: INTEGER; wantHScrollBar, wantVScrollBar: BOOLEAN;
										 itsDocument: TDocument; itsView: TView): TWindow;
				{ Used for creating simple windows that contain 1 view and may or may not scroll. Signals
				Failure if the window could not be created. }
		
				FUNCTION TViewServer.NewTWindow(itsRsrcID: INTEGER; itsDocument: TDocument): TWindow;
				{ Used for creating a window object. Used within NewPaletteWindow & NewSimpleWindow.
				SIgnals Failure if the window could not be created. }
		
				{ ------------------------------------------------------------------------------------	}
				{ Template View Creation																}
				{ ------------------------------------------------------------------------------------	}
			
				FUNCTION TViewServer.NewTemplateWindow(viewRsrcID: INTEGER; itsDocument: TDocument): TWindow;
				{ Used for creating a window using a 'WIND' resource and a 'view' template. }
		
				FUNCTION TViewServer.DoCreateViews(itsDocument: TDocument;
												   parentView: TView;
												   itsRsrcID: INTEGER;
												   subViewOffset: VPoint): TView;
				{ This is the method responsible for creating view hierarchies from resource
				templates. If fNextHandler is not NIL it calls its DoCreateViews method. Otherwise
				it creates the desired view heirarchy. }

				FUNCTION TViewServer.CreateAView(itsDocument: TDocument;
												 itsSuperView: TView;
												 VAR itsParams: Ptr): TView;
				{ For creating a single view from a resource template view. If fNextHandler is not
				NIL calls its CreateAView method, otherwise it proceeds to create the desired view.
				}

				END;

		VAR
			{$Push} 									{$J+}
			gViewServer:		TViewServer;			{ the globally accessible view server }
			{$Pop}

END.
