{[a-,body+,h-,o=100,r+,rec+,t=4,u+,#+,j=20/57/1$,n-]}
{ UWindow.p }
{ Copyright  1984-1991 by Apple Computer Inc.	All rights reserved. }

UNIT UWindow;

	INTERFACE

		USES
			{}
			Types, MacAppTypes, Memory, UPascalObject, UObject, AppleEvents, UEvtHandler, Balloons, UAdorners, UView, 
			UEvent, UCommand
			, Windows;

		CONST

			kFloatWindowKind	= 128;					{ the windowKind of floating windows }
			CurDeKind			= $A22; 				{ lo memory: the windowKind associated with
														 curDeactive }

		TYPE

			WindowTemplate		= PACKED RECORD
				procID: 			INTEGER;
				hasGoAway:			BOOLEAN;
				resizable:			BOOLEAN;
				isModal:			BOOLEAN;
				doFirstClick:		BOOLEAN;
				freeOnClosing:		BOOLEAN;
				disposeOnFree:		BOOLEAN;
				closesDocument: 	BOOLEAN;
				openInitially:		BOOLEAN;
				mustAdaptToScreen:	BOOLEAN;
				stagger:			BOOLEAN;
				mustForceOnScreen:	BOOLEAN;
				vertCenter: 		BOOLEAN;
				horzCenter: 		BOOLEAN;
				filler: 			0..7;
				targetID:			IDType;
				title:				Str255; 			{Actually variable length}
				END;
			WindowTemplatePtr	= ^WindowTemplate;

			TDocument			= OBJECT; external;
			TToolboxEvent		= OBJECT; EXTERNAL;

			TWindow 			= OBJECT (TView)		{ Corresponds to a desktop Window. The
														 outermost of a nested set of views. }
				fWMgrWindow:		WindowPtr;			{ the window manager window Ptr }
				fProcID:			INTEGER;			{ the proc ID of the window }
				fUpdating:			BOOLEAN;			{ True when TWindow.Update has control.
														 Changes behaviour of TWindow.Focus to clip
														 TO the update region instead of clipping
														 OUT the update region }
				fMoveBounds:		Rect;				{ bounds over which the window may be moved
														 }
				fResizeLimits:		Rect;				{ limits over which the window may be
														 resized }
				fTarget:			TEvtHandler;		{ when this window is activated; the
														 TEvtHandler to SetTarget on }
				fTargetID:			IDType; 			{ corresponding ID to fTarget }
				fPreDocname:		INTEGER;			{ amount of window title that precedes the
														 document name }
				fConstTitle:		INTEGER;			{ amount of the title name that is constant
														 }
				fIsActive:			BOOLEAN;			{ is this window active }
				fIsResizable:		BOOLEAN;			{ is this window resizeable }
				fIsClosable:		BOOLEAN;			{ is this window closable }
				fFreeOnClosing: 	BOOLEAN;			{ should this window free itself when it
														 closes }
				fDisposeOnFree: 	BOOLEAN;			{ should the window manager window be
														 disposed when this window frees }
				fClosesDocument:	BOOLEAN;			{ does this window close the document }
				fOpenInitially: 	BOOLEAN;			{ should this window be opened when its
														 document is asked to show its windows }
				fIsModal:			BOOLEAN;			{ should this window behave modally }
				fDoFirstClick:		BOOLEAN;			{ should a click in this window not only
														 select it if it is inactive but also be
														 sent to its contents }
				fMustAdapt: 		BOOLEAN;			{ does this window require adaption to the
														 screen }
				fMustHorzCenter:	BOOLEAN;			{ does this window require centering
														 horizontally }
				fMustVertCenter:	BOOLEAN;			{ does this window require centering
														 vertically }
				fMustStagger:		BOOLEAN;			{ does this window require staggering }
				fMustForceOnScreen: BOOLEAN;			{ does this window require forcing on the
														 screen }
				fAdapted:			BOOLEAN;			{ has this window's size ever been adapted
														 to the screen }
				fHorzCentered:		BOOLEAN;			{ has this window been centered horizontally
														 }
				fVertCentered:		BOOLEAN;			{ has this window been centered vertically }
				fStaggered: 		BOOLEAN;			{ has this window been staggered }
				fForcedOnScreen:	BOOLEAN;			{ has this window been forced on the screen
														 }
				fFloats:			BOOLEAN;			{ does this window float above non-floating
														 windows (NOT SUPPORTED IN 2.0) }
				fDismissed: 		BOOLEAN;			{ Has the dialog been dismissed? }
				fHideOnSuspend: 	BOOLEAN;			{ does this window get hid on suspend? }
				fWasHiddenOnSuspend: BOOLEAN;			{ TRUE if this window was hid on suspend }
				fContRgnInset:		Point;				{ topleft inset of cont rgn in struc rgn }
				fContDifference:	Point;				{ total amount the content is less than and
														 offset into the structure (Accounts for
														 title bar, etc.) }
				fGenerateActivates: BOOLEAN;			{ a flag indicating whether or not
														 TWindow.Show should generate the necessary
														 activate events }

								{ Creation/Destruction Methods }

				PROCEDURE TWindow.Initialize; OVERRIDE;
				{ Puts a newly created TWindow into a known state. Subclasses should
				override it and call INHERITED. Typically used to put the object into a state where
				it can be safely FREED. }

				PROCEDURE TWindow.IWindow(itsDocument: TDocument;
										  itsWMgrWindow: WindowPtr;
										  canResize, canClose, disposeOnFree: BOOLEAN);
				{ Initialize a window procedurally. }

				PROCEDURE TWindow.IRes(itsDocument: TDocument;
									   itsSuperView: TView;
									   VAR itsParams: Ptr); OVERRIDE;
				{ Initialize a window from a resource template. }

				FUNCTION TWindow.GetBehindWindowPtr: WindowPtr;
				{ return the windowPtr in the windowlist behind which this window gets created }

				PROCEDURE TWindow.WRes(theResource: ViewRsrcHandle;
									   VAR itsParams: Ptr); OVERRIDE;
				{ For writing a window out to a resource. }

				PROCEDURE TWindow.WriteRes(theResource: ViewRsrcHandle;
										   VAR itsParams: Ptr); OVERRIDE;
				{ For writing a TWindow to a resource, works in conjunction with the previous
				method. }

				PROCEDURE TWindow.Free; OVERRIDE;
				{ Frees the window if fDisposeOnFree is TRUE, also deletes the window from the
				document if there is one, otherwise deletes it from the applications list of free
				windows. }

				{ Open/Close Methods }

				PROCEDURE TWindow.Open; OVERRIDE;
				{ Makes the window visible by showing it before calling inherited Open. }

				PROCEDURE TWindow.Close; OVERRIDE;
				{ Calls inherited Close and the hides & deactivates the window. }

				PROCEDURE TWindow.CloseByUser;
				{ If the window is shown and it can be closed handles the closing. }

				PROCEDURE TWindow.GoAwayByUser(theMouse: VPoint); OVERRIDE;
				{ Responsible for calling CloseByUser. }

				{ Activation Methods }

				PROCEDURE TWindow.Activate(entering: BOOLEAN); OVERRIDE;
				{ Activate the window. }

				{ Size/Location Methods }

				PROCEDURE TWindow.Locate(newLoc: VPoint;
										 invalidate: BOOLEAN); OVERRIDE;
				{ If fWMgrWindow is not NIL then MoveWindow to relocate the window. }

				PROCEDURE TWindow.MoveByUser(theMouse: VPoint); OVERRIDE;
				{ Handles the user moving the window by dragging the title bar, calls DragWindow to
				handle movement. }

				PROCEDURE TWindow.ResizeByUser(theMouse: VPoint); OVERRIDE;
				{ Handles the user resizing the window by dragging the growbox, calls GrowWindow to
				handle movement and Resize to resize the window. }

				PROCEDURE TWindow.Resize(newSize: VPoint;
										 invalidate: BOOLEAN); OVERRIDE;
				{ Used to resize the window by setting its width & height to the parameters passed
				in. }

				PROCEDURE TWindow.Zoom(partCode: INTEGER);
				{ Responsible for actually handling the zooming of a window when the user clicks in
				the zoombox. }

				PROCEDURE TWindow.ZoomByUser(theMouse: VPoint;
											 partCode: INTEGER); OVERRIDE;
				{ handles the user clicking in the zoombox to zoom the window, calls Zoom. }

				{ Focusing Methods }

				FUNCTION TWindow.Focus: BOOLEAN; OVERRIDE;
				{ Attempts to set the Focus to the window, returns TRUE if it succeeds, FALSE if not.
				}

				FUNCTION TWindow.FocusOnSuperView: BOOLEAN; OVERRIDE;
				{ Attempts to set the Focus to our superview, returning TRUE if it succeeds or FALSE
				if not. }

				{ Drawing Methods }

				PROCEDURE TWindow.DrawResizeIcon;
				{ Draws the grow icon by calling the toolbox routine DrawGrowIcon. }

				{ Mouse Handling Methods }

				FUNCTION TWindow.HandleMouseDown(theMouse: VPoint;
												 event: TToolboxEvent;
												 hysteresis: Point): BOOLEAN; OVERRIDE;
				{ Handles mouse downs in the window. }

				FUNCTION TWindow.GetPartCode(theMouse: VPoint): INTEGER; OVERRIDE;
				{ determines what part of the window theMouse is in }

				FUNCTION TWindow.HasPendingUpdate: BOOLEAN; OVERRIDE;
				{ Returns TRUE if there is a pending update event for this window, FALSE if not. }

				PROCEDURE TWindow.Update; OVERRIDE;
				{ Handle the update region and drawcontents on SELF. }

				PROCEDURE TWindow.DoInvalidateRgn(badRgn: RgnHandle); OVERRIDE;
				{ If a WMgrWindow is available calls InvalRgn }

				PROCEDURE TWindow.DoValidateRgn(goodRgn: RgnHandle); OVERRIDE;
				{ If a WMgrWindow is available calls ValidRgn }

				{ Menu Handling Methods}

				FUNCTION TWindow.AllowsMenuAccess: BOOLEAN;
				{ By default returns TRUE. }

				PROCEDURE TWindow.DoSetupMenus; OVERRIDE;
				{ If the window is not modal calls inherited DoSetupMenus. }

				PROCEDURE TWindow.DoMenuCommand(aCmdNumber: CmdNumber); OVERRIDE;
				{ Handles closing the window. }

				{ Target and view management }

				PROCEDURE TWindow.SetTarget(newTarget: TEvtHandler); OVERRIDE;
				{ Set fTarget to 'newTarget' and if we are the active window then call
				gApplication.SetTarget. }
				
				FUNCTION TWindow.GetTarget: TEvtHandler;
				{ Return the current fTarget of this window }

				PROCEDURE TWindow.RemovedASubView(theSubView: TView); OVERRIDE;
				{ Overridden to fix up the target, if the subview removed was the target }

				{ Cursor Handling }

				FUNCTION TWindow.HandlesCursor: BOOLEAN; OVERRIDE;
				{ Returns TRUE if fDoesHandleCursor is TRUE, and the window is active,
				or fDoFirstClick is TRUE }

				FUNCTION TWindow.LetsSubViewsHandleCursor: BOOLEAN; OVERRIDE;

				FUNCTION TWindow.HandlesHelp: BOOLEAN; OVERRIDE;
				{ Returns the value of fDoesHandleHelp }

				FUNCTION TWindow.LetsSubViewsHandleHelp: BOOLEAN; OVERRIDE;

				{ Misc. Methods }

				PROCEDURE TWindow.AboutToLoseControl;
				{ called when the application is about to lose control }

				PROCEDURE TWindow.RegainControl;
				{ called when the application is regaining control }

				PROCEDURE TWindow.Show(state, redraw: BOOLEAN); OVERRIDE;
				{ Called to make the window visible or not based on 'state'. }

				FUNCTION TWindow.IsShown: BOOLEAN; OVERRIDE;
				{ Returns TRUE if the window is currently shown. }

				FUNCTION TWindow.IsActive: BOOLEAN; OVERRIDE;
				{ Returns TRUE if the window is currently active. }

				PROCEDURE TWindow.Select;
				{ Called to select the window. }

				PROCEDURE TWindow.SetTitle(newTitle: Str255);
				{ Sets the windows title to 'newTitle'. }

				PROCEDURE TWindow.GetTitle(VAR theTitle: Str255);
				{ Gets the current title of the window. }

				PROCEDURE TWindow.SetTitleForDoc(newDocTitle: Str255);
				{ Called when the documents name is known or changed }

				PROCEDURE TWindow.AdaptToScreen;
				{ Resizes the window to fit the screen on which it is being displayed. }

				PROCEDURE TWindow.ForceOnScreen;
				{ ForceOnScreen gaurantees that some minimal drag area is accessible to the user,
				  to be dragged to the desired location. }

				PROCEDURE TWindow.Center(horizontally, vertically, forDialog: BOOLEAN);
				{ Centers the window on the screen either horizontally, vertically or both. }

				PROCEDURE TWindow.SimpleStagger(dh, dv: INTEGER;
												VAR counter: INTEGER);
				{ Handles the staggering of windows as they are opened. }

				FUNCTION TWindow.GetGrafPort: GrafPtr; OVERRIDE;
				{ Returns the windows GrafPtr. }

				FUNCTION TWindow.GetMaxIntersectedDevice(VAR screenRect: Rect): GDHandle;
				{ Returns the screenRect of the most intersected device & its GDHandle, or
				  if CQD isn't available it returns GetGrayRgn intersected with screenbits.bounds
				  and NIL for the GDHandle. }

				FUNCTION TWindow.GetWindow: TWindow; OVERRIDE;
				{ Returns SELF. }

				PROCEDURE TWindow.SetResizeLimits(itsMinSize, itsMaxSize: Point);
				{ Used to set the limits to which the window can be resized. }

				PROCEDURE TWindow.InstallDocument(itsDocument: TDocument);
				{ Sets our fDocument field to 'itsDocument' and then installs the SELF in the
				document, if the document is not NIL. }

				FUNCTION TWindow.IsDraggable(whichRect: Rect): BOOLEAN;
				{ Returns TRUE if any of the corner points of whichRect are draggable. }

				FUNCTION TWindow.IsHiddenOnSuspend: BOOLEAN;
				{ Returns TRUE if this window gets hidden when the application is suspended. }

				FUNCTION TWindow.IsDismissed: BOOLEAN;
				{ Returns TRUE if this dialog has been dismissed. }

				PROCEDURE TWindow.Dismiss;
				{ dismissed the dialog that is currently being posed modally }

				FUNCTION TWindow.BuildWindowRgns(build: BOOLEAN): BOOLEAN;
				{ IF build=kBuild THEN ensures that the window regions are valid; sets
				fContRgnInset, fContDifference.
				Cleans up if necessary, when done with window regions.
				Returns the old state of the window regions. }

				PROCEDURE TWindow.PoseModally;
				{ poses this window in a modal state, called (usually) by TDialogView.PoseModally }

				{ Target management }

				PROCEDURE TWindow.DoKeyCommand(event: TToolboxEvent); OVERRIDE;
				{ Overridden to handle the tab key }

				{ Inspecting Methods }

				PROCEDURE TWindow.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields. }

				PROCEDURE TWindow.GetInspectorName(VAR inspectorName: Str255); OVERRIDE;
				{ Used by the Inspector to get the name of this class. }

				END;

{--------------------------------------------------------------------------------------------------}
			TCloseWindowCommand = OBJECT (TNoChangesCommand) { Tells the application close a window.
															  }
				PROCEDURE TCloseWindowCommand.ICloseWindowCommand(itsCmdNumber: CmdNumber;
																  itsWindow: TWindow);
				{ Initialize the CloseWindowCommand procedurally. }

				PROCEDURE TCloseWindowCommand.DoIt; OVERRIDE;
				{ tell the application to close a window. }

				PROCEDURE TCloseWindowCommand.Fields(obj: TObject); OVERRIDE;
				{ Used by the Inspector and the Debugger to display the contents of this class's
				fields. }

				END;

{--------------------------------------------------------------------------------------------------}
			{ utility routines }

		FUNCTION IsDocumentWindow(aWindow: WindowPtr): BOOLEAN;

		FUNCTION IsDialog(window: WindowPtr): BOOLEAN;

		FUNCTION IsFloatWindow(window: WindowPtr): BOOLEAN;

		FUNCTION GetLastFloatingWindowPtr: WindowPtr;

		FUNCTION MAGetActiveWindow: WindowPtr;

		FUNCTION MAFrontWindow: WindowPtr;

		VAR
			{$Push} 									{$J+}
			gStdWMoveBounds:	Rect;					{ standard boundsRect to pass to DragWindow
														 Toolbox routine }
			gStdWSizeRect:		Rect;					{ standard sizeRect to pass to GrowWindow
														 Toolbox routine }
			gStdWScreenRect:	Rect;					{ Used to force a window onto the screen. If
														 the window is not in the GrayRgn, then at
														 least one corner of the window must lie
														 within gStdWScreenRect. }
			gStdStaggerCount:	INTEGER;				{ Used to stagger windows created from
														 templates }
			{$Pop}
END.
