// UAdorners.cp 
// Copyright  1984-1991 by Apple Computer Inc. All rights reserved.

#ifndef __UADORNERS__
#include "UAdorners.h"
#endif

#ifndef __UGEOMETRY__
#include <UGeometry.h>
#endif

#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

#ifndef __ERRORS__
#include <Errors.h>
#endif

#ifndef __UMACAPPUTILITIES__
#include <UMacAppUtilities.h>
#endif

#ifndef __UMACAPPGLOBALS__
#include <UMacAppGlobals.h>
#endif

#ifndef __UWINDOW__
#include <UWindow.h>
#endif

#ifndef __ULIST__
#include <UList.h>
#endif


//--------------------------------------------------------------------------------------------------
// Definition for global variables declared in interface:

// Global, pre-allocated, shared adorners

TDimAdorner* gDimAdorner;
TDrawAdorner* gDrawAdorner;
TEraseAdorner* gEraseAdorner;
THiliteAdorner* gHiliteAdorner;
TPrintAdorner* gPrintAdorner;
TResizeIconAdorner* gResizeIconAdorner;
TSelectionAdorner* gSelectionAdorner;


RGBColor gRGBVeryLtGray;
RGBColor gRGBLtGray;
RGBColor gRGBGray;
RGBColor gRGBDkGray;

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerRes

CAdornmentIterator::CAdornmentIterator(TAdornment* theAdornment,
						   ArrayIndex itsLowBound,
						   ArrayIndex itsHighBound,
						   Boolean itsForward) :
	CArrayIterator(theAdornment ? theAdornment->fAdorners : NULL, itsLowBound, itsHighBound, itsForward)
{
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerRes

CAdornmentIterator::CAdornmentIterator(TAdornment* theAdornment,
						   Boolean itsForward) :
	CArrayIterator(theAdornment ? theAdornment->fAdorners : NULL, itsForward)
{
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerRes

CAdornmentIterator::CAdornmentIterator(TAdornment* theAdornment) :
	CArrayIterator(theAdornment ? theAdornment->fAdorners : NULL)
{
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerRes

CAdornmentIterator::CAdornmentIterator(TAdornerList* theAdornerList) :
	CArrayIterator(theAdornerList)
{
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerRes

TAdorner* CAdornmentIterator::CurrentAdorner(void)
{
	return ((TAdornerList*)fDynamicArray)->AdornerAt(fCurrentIndex);
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerRes

TAdorner* CAdornmentIterator::FirstAdorner(void)
{
	this->Reset();
	if (this->More())
		return ((TAdornerList*)fDynamicArray)->AdornerAt(fCurrentIndex);
	else
		return NULL;
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerRes

TAdorner* CAdornmentIterator::NextAdorner(void)
{
	this->Advance();
	if (this->More())
		return ((TAdornerList*)fDynamicArray)->AdornerAt(fCurrentIndex);
	else
		return NULL;
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAInit

pascal void InitUAdorners(void)
{
	// create and initialize the system-supplied adorners 

	DontDeadStrip(TDimAdorner);
	DontDeadStrip(TDrawAdorner);
	DontDeadStrip(TEraseAdorner);
	DontDeadStrip(TFrameAdorner);
	DontDeadStrip(THiliteAdorner);
	DontDeadStrip(TLineBottomAdorner);
	DontDeadStrip(TLineLeftAdorner);
	DontDeadStrip(TLineRightAdorner);
	DontDeadStrip(TLineTopAdorner);
	DontDeadStrip(TOvalAdorner);
	DontDeadStrip(TPrintAdorner);
	DontDeadStrip(TResizeIconAdorner);
	DontDeadStrip(TRRectAdorner);
	DontDeadStrip(TSelectionAdorner);

	RegisterStdType("TDimAdorner", kDimAdorner);
	RegisterStdType("TDrawAdorner", kDrawAdorner);
	RegisterStdType("TEraseAdorner", kEraseAdorner);
	RegisterStdType("TFrameAdorner", kFrameAdorner);
	RegisterStdType("THiliteAdorner", kHiliteAdorner);
	RegisterStdType("TLineBottomAdorner", kLineBottomAdorner);
	RegisterStdType("TLineLeftAdorner", kLineLeftAdorner);
	RegisterStdType("TLineRightAdorner", kLineRightAdorner);
	RegisterStdType("TLineTopAdorner", kLineTopAdorner);
	RegisterStdType("TOvalAdorner", kOvalAdorner);
	RegisterStdType("TPrintAdorner", kPrintAdorner);
	RegisterStdType("TResizeIconAdorner", kResizeIconAdorner);
	RegisterStdType("TRRectAdorner", kRRectAdorner);
	RegisterStdType("TSelectionAdorner", kSelectionAdorner);

	// Create the global, shared, pre-allocated adorners
	
	gDimAdorner = new TDimAdorner;
	gDimAdorner->IDimAdorner();

	gDrawAdorner = new TDrawAdorner;
	gDrawAdorner->IDrawAdorner();

	gEraseAdorner = new TEraseAdorner;
	gEraseAdorner->IEraseAdorner();

	gHiliteAdorner = new THiliteAdorner;
	gHiliteAdorner->IHiliteAdorner();

	gPrintAdorner = new TPrintAdorner;
	gPrintAdorner->IPrintAdorner();

	gResizeIconAdorner = new TResizeIconAdorner;
	gResizeIconAdorner->IResizeIconAdorner();

	gSelectionAdorner = new TSelectionAdorner;
	gSelectionAdorner->ISelectionAdorner();

	// Setup some basic RGB greyscale colors 
	SetRGBColor(gRGBVeryLtGray, 58333, 58333, 58333);// Setup the very light gray global 	
	SetRGBColor(gRGBLtGray, 49152, 49152, 49152);// Setup the light gray global		
	SetRGBColor(gRGBGray, 37000, 37000, 37000);	// Setup the medium gray global 		
	SetRGBColor(gRGBDkGray, 20000, 20000, 20000);// Setup the dark gray global 		
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerNonRes

pascal TAdorner* NewStdAdorner(IDType signature,
							  MAName className,
							  IDType itsID,
							  Boolean freeOnDeletion)
{
	TAdorner* anAdorner = (TAdorner*) NewObjectBySignature(signature,className);
	anAdorner->IAdorner(itsID,freeOnDeletion);
	return anAdorner;
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerNonRes

pascal void TAdorner::Initialize(void)			// override 
{
	inherited::Initialize();
	
	fID = kNoIdentifier;
	fFreeOnDeletion = TRUE;
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerNonRes

pascal void TAdorner::IAdorner(IDType itsID,
								Boolean freeOnDeletion)
{
	this->IObject();

	fID = itsID;
	fFreeOnDeletion = freeOnDeletion;
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerNonRes

pascal void TAdorner::AssociateWithAdornment(TAdornment* /* itsAdornment */,
									   Boolean associate)
{
	if (!associate && fFreeOnDeletion)
			this->Free();
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerNonRes

pascal void TAdorner::DrawContents(TView* itsView, 
							const VRect& area)
{
	this->Draw(itsView, area);
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerNonRes

pascal void TAdorner::Draw(TView* itsView, 
							 const VRect& area)
{
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerRes

pascal void TAdorner::DrawLine(VHSelect vhs,
							   short lh,
							   short lv,
							   short lto)
{
	MoveTo(lh, lv);
	if (vhs == hSel)
		Line(lto - lh, 0);
	else
		Line(0, lto - lv);
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerNonRes

pascal void TAdorner::DoHighlightSelection(TView* itsView, 
										   const VRect& area,
										   HLState fromHL,
										   HLState toHL)
{
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerRes

pascal void TAdorner::InvalidateAdorner(TView* itsView)
{
	if (itsView)
		itsView->ForceRedraw();
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerRes

pascal void TAdorner::ViewChangedSize ( 	TView* itsView,
											const VPoint& delta,
											Boolean	invalidate )
{
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerNonRes

pascal void TDimAdorner::Initialize()	// Override
{
	inherited::Initialize();
	fID = kDimAdorner;
	fFreeOnDeletion = FALSE;
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerNonRes

pascal void TDimAdorner::IDimAdorner()
{
	this->IAdorner(kDimAdorner,kDontFreeOnDeletion);
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerRes

pascal void TDimAdorner::Draw(TView* itsView, 
								const VRect&)
{
	itsView->Dim();
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerNonRes

pascal void TDrawAdorner::Initialize()	// Override
{
	inherited::Initialize();
	fID = kDrawAdorner;
	fFreeOnDeletion = FALSE;
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerNonRes

pascal void TDrawAdorner::IDrawAdorner()
{
	this->IAdorner(kDrawAdorner,kDontFreeOnDeletion);
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerRes

pascal void TDrawAdorner::Draw(TView* itsView, 
								 const VRect& area)
{
	PenNormal();
	itsView->Draw(area);
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerNonRes

pascal void TEraseAdorner::Initialize()	// Override
{
	inherited::Initialize();
	fID = kEraseAdorner;
	fFreeOnDeletion = FALSE;
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerNonRes

pascal void TEraseAdorner::IEraseAdorner()
{
	this->IAdorner(kEraseAdorner,kDontFreeOnDeletion);
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerRes

pascal void TEraseAdorner::Draw(TView* itsView, 
								  const VRect& area)
{
	Rect qdRect;

	itsView->ViewToQDRect(area, qdRect);
	EraseRect(qdRect);
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerNonRes

pascal void THiliteAdorner::Initialize()	// Override
{
	inherited::Initialize();
	fID = kHiliteAdorner;
	fFreeOnDeletion = FALSE;
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerNonRes

pascal void THiliteAdorner::IHiliteAdorner()
{
	this->IAdorner(kHiliteAdorner,kDontFreeOnDeletion);
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerRes

pascal void THiliteAdorner::Draw(TView* itsView, 
									const VRect&)
{
	itsView->Hilite();
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerNonRes

pascal void TResizeIconAdorner::Initialize()	// Override
{
	inherited::Initialize();
	fID = kResizeIconAdorner;
	fFreeOnDeletion = FALSE;
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerNonRes

pascal void TResizeIconAdorner::IResizeIconAdorner()
{
	this->IAdorner(kResizeIconAdorner,kDontFreeOnDeletion); 
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerRes

pascal void TResizeIconAdorner::DoHighlightSelection(TView* itsView, 
													 const VRect& ,
													 HLState,
													 HLState)
{
	((TWindow *)itsView)->DrawResizeIcon();
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerNonRes

pascal void TSelectionAdorner::Initialize()	// Override
{
	inherited::Initialize();
	fID = kSelectionAdorner;
	fFreeOnDeletion = FALSE;
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerNonRes

pascal void TSelectionAdorner::ISelectionAdorner()
{
	this->IAdorner(kSelectionAdorner,kDontFreeOnDeletion);
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerRes

pascal void TSelectionAdorner::DoHighlightSelection(TView* itsView, 
													const VRect& ,
													HLState fromHL,
													HLState toHL)
{
	PenNormal();
	itsView->DoHighlightSelection(fromHL, toHL);
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerNonRes

pascal void TPrintAdorner::Initialize()	// Override
{
	inherited::Initialize();
	fID = kPrintAdorner;
	fFreeOnDeletion = FALSE;
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerNonRes

pascal void TPrintAdorner::IPrintAdorner()
{
	this->IAdorner(kPrintAdorner, kDontFreeOnDeletion);
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerRes

pascal void TPrintAdorner::DoHighlightSelection(TView* itsView, 
												const VRect& area,
												HLState fromHL,
												HLState)
{
	// only do this in the "DrawContents" case; in which case "fromHL" is hlOff 
	if (fromHL == hlOff)
		itsView->DoDrawPrintFeedback(area);
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerNonRes

pascal void TFrameAdorner::IFrameAdorner(IDType itsID,
										Boolean freeOnDeletion)
{
	this->IAdorner(itsID,freeOnDeletion);
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerRes

pascal void TFrameAdorner::Draw(TView* itsView, 
									const VRect& area)
{
	VRect itsAdornExtent;
	Rect qdArea;

	itsView->GetAdornExtent(itsAdornExtent);

	if ((area & itsAdornExtent).Valid())	// is there something to draw?
	{
		itsView->ViewToQDRect(itsAdornExtent, qdArea);// convert to QD coord's 
		FrameRect(qdArea);
	}
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerNonRes

pascal void TLineBottomAdorner::ILineBottomAdorner(IDType itsID, 
													Boolean freeOnDeletion)
{
	this->IAdorner(itsID,freeOnDeletion);
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerRes

pascal void TLineBottomAdorner::Draw(TView* itsView, 
										const VRect& area)
{
	VRect itsAdornExtent;
	Rect qdArea;
	PenState pnState;

	itsView->GetAdornExtent(itsAdornExtent);

	if ((area & itsAdornExtent).Valid())	// is there something to draw?
	{
		GetPenState(pnState);
		itsView->ViewToQDRect(itsAdornExtent, qdArea);// convert to QD coord's 
		this->DrawLine(hSel, qdArea.left, qdArea.bottom - pnState.pnSize.v, qdArea.right);// account for pen height 
	}
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerNonRes

pascal void TLineLeftAdorner::ILineLeftAdorner(IDType itsID, 
												Boolean freeOnDeletion)
{
	this->IAdorner(itsID,freeOnDeletion);
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerRes

pascal void TLineLeftAdorner::Draw(TView* itsView, 
										const VRect& area)
{
	VRect itsAdornExtent;
	Rect qdArea;

	itsView->GetAdornExtent(itsAdornExtent);

	if ((area & itsAdornExtent).Valid())	// is there something to draw?
	{
		itsView->ViewToQDRect(itsAdornExtent, qdArea);// convert to QD coord's 
		this->DrawLine(vSel, qdArea.left, qdArea.top, qdArea.bottom);
	}
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerNonRes

pascal void TLineRightAdorner::ILineRightAdorner(IDType itsID, 
												Boolean freeOnDeletion)
{
	this->IAdorner(itsID,freeOnDeletion);
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerRes

pascal void TLineRightAdorner::Draw(TView* itsView, 
										const VRect& area)
{
	VRect itsAdornExtent;
	Rect qdArea;
	PenState pnState;

	itsView->GetAdornExtent(itsAdornExtent);

	if ((area & itsAdornExtent).Valid())	// is there something to draw?
	{
		GetPenState(pnState);
		itsView->ViewToQDRect(itsAdornExtent, qdArea);// convert to QD coord's 
		this->DrawLine(vSel, qdArea.right - pnState.pnSize.h, qdArea.top, qdArea.bottom);// account for pen width 
	}
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerNonRes

pascal void TLineTopAdorner::ILineTopAdorner(IDType itsID, 
											 Boolean freeOnDeletion)
{
	this->IAdorner(itsID,freeOnDeletion);
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerRes

pascal void TLineTopAdorner::Draw(TView* itsView, 
									const VRect& area)
{
	VRect itsAdornExtent;
	Rect qdArea;

	itsView->GetAdornExtent(itsAdornExtent);

	if ((area & itsAdornExtent).Valid())	// is there something to draw?
	{
		itsView->ViewToQDRect(itsAdornExtent, qdArea);// convert to QD coord's 
		this->DrawLine(hSel, qdArea.left, qdArea.top, qdArea.right);
	}
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerNonRes

pascal void TOvalAdorner::IOvalAdorner(IDType itsID, 
										Boolean freeOnDeletion)
{
	this->IAdorner(itsID,freeOnDeletion);
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerRes

pascal void TOvalAdorner::Draw(TView* itsView, 
									const VRect& area)
{
	VRect itsAdornExtent;
	Rect qdArea;

	itsView->GetAdornExtent(itsAdornExtent);

	if ((area & itsAdornExtent).Valid())	// is there something to draw?
	{
		itsView->ViewToQDRect(itsAdornExtent, qdArea);// convert to QD coord's 
		FrameOval(qdArea);
	}
}


//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerNonRes

pascal void TRRectAdorner::IRRectAdorner(IDType itsID, 
										Boolean freeOnDeletion)
{
	this->IAdorner(itsID,freeOnDeletion);
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerRes

pascal void TRRectAdorner::Draw(TView* itsView, 
									const VRect& area)
{
	VRect itsAdornExtent;
	Rect qdArea;

	itsView->GetAdornExtent(itsAdornExtent);

	if ((area & itsAdornExtent).Valid())	// is there something to draw?
	{
		itsView->ViewToQDRect(itsAdornExtent, qdArea);// convert to QD coord's 
		FrameRoundRect(qdArea, 16, 16);
	}
}


//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerRes

pascal void TAdornerList::IAdornerList(void)
{
	this->ISortedDynamicArray(0,sizeof(AdornerElement));
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAApplicationRes

pascal short TAdornerList::CompareElements(void* Element1,
								   void* Element2)	// Override
{
	AdornPriority priority1 = ((AdornerElementPtr) Element1)->priority;
	AdornPriority priority2 = ((AdornerElementPtr) Element2)->priority;
	if (priority1 > priority2)
		return kItem1GreaterThanItem2;
	else if (priority1 < priority2)
		return kItem1LessThanItem2;
	else
		return kItem1EqualItem2;
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAApplicationRes

pascal void TAdornerList::AddAdorner(TAdorner* itsAdorner,
									AdornPriority itsPriority)
{
//!!! Protect against multiple insertion?
	AdornerElement anAdornerElement(itsPriority,itsAdorner);
	
	this->InsertElementInOrder(&anAdornerElement);
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerRes

pascal TAdorner* TAdornerList::AdornerAt(ArrayIndex i)
{
	return ((AdornerElementPtr) this->ComputeAddress(i))->adorner;
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAFields

pascal void TAdornerList::Fields(TObject* obj)	// override 
{
	obj->DoToField("TAdornerList", NULL, bClass);
	inherited::Fields(obj);
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerNonRes

pascal void TAdornment::Initialize(void)		// override 
{
	inherited::Initialize();
	fView = NULL;
	fAdorners = NULL;
	fPenSize.v = 1;
	fPenSize.h = 1;
	fDrawShadow = FALSE;
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerNonRes

pascal void TAdornment::IAdornment(IDType itsID, 
									TView* itsView,
								   Point itsPenSize)
{
	FailInfo fi;
	TAdornerList * itsAdornerList;

	this->IAdorner(itsID,kFreeOnDeletion); // Adornments are not shared
	fView = itsView;
	
	if (fi.Try())
	{
		itsAdornerList = new TAdornerList;		// !!! needs a failure handler 
		fi.Success();
	}
	else
	{											//	Failure Handler.
		this->Free();
		fi.ReSignal();							// Optional. Omit to continue.
	}

	itsAdornerList->IAdornerList();
	fAdorners = itsAdornerList;
	fPenSize = itsPenSize;
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerNonRes

pascal void TAdornment::Free(void)				// override 
{
	CAdornmentIterator iter(this);
	
	for (TAdorner* anAdorner = iter.FirstAdorner(); iter.More(); anAdorner = iter.NextAdorner())
		this->DeleteAdorner(anAdorner, kDontRedraw);
	fAdorners = (TAdornerList *)FreeIfObject(fAdorners);
	
	inherited::Free();
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerRes

pascal void TAdornment::AddAdorner(TAdorner* anAdorner,
									AdornPriority itsPriority,
									Boolean invalidate)
{
	//!!! add protection against multiple insertion ?
	{
		anAdorner->AssociateWithAdornment(this, TRUE);
		fAdorners->AddAdorner(anAdorner,itsPriority);
		if (invalidate)
			anAdorner->InvalidateAdorner(fView);
	}

}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerRes

pascal void TAdornment::DeleteAdorner(TAdorner* anAdorner,
									   Boolean invalidate)
{
	CAdornmentIterator iter(this);
	
	for (TAdorner* theAdorner = iter.FirstAdorner(); iter.More(); theAdorner = iter.NextAdorner())
	{
		if (theAdorner == anAdorner)
		{
			if (invalidate)
				theAdorner->InvalidateAdorner(fView);
			anAdorner->AssociateWithAdornment(this, FALSE);	// anAdorner might get freed here 
			fAdorners->DeleteElementsAt(iter.fCurrentIndex, 1);	
		}
	}
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerRes
pascal void TAdornment::DeleteAdornerByID(IDType itsID,
									Boolean invalidate)
{
	CAdornmentIterator iter(this);
	
	for (TAdorner* theAdorner = iter.FirstAdorner(); iter.More(); theAdorner = iter.NextAdorner())
	{
		if (theAdorner->fID == itsID)
		{
			if (invalidate)
				theAdorner->InvalidateAdorner(fView);
			theAdorner->AssociateWithAdornment(this, FALSE);	// anAdorner might get freed here 
			fAdorners->DeleteElementsAt(iter.fCurrentIndex, 1);	
		}
	}
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerRes

pascal TAdorner* TAdornment::AdornerAt(ArrayIndex i)
{
	if (fAdorners)
		return fAdorners->AdornerAt(i);
	else
		return NULL;
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerRes

pascal TAdorner* TAdornment::AdornerWithID(IDType itsID)
{
	CAdornmentIterator iter(this);	
	
	for (TAdorner* anAdorner = iter.FirstAdorner(); iter.More(); anAdorner = iter.NextAdorner())
	{
		if (anAdorner->fID == itsID)
			return anAdorner;
	}
	return NULL;
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerRes

pascal void TAdornment::DrawContents(TView* itsView, 
								const VRect& area)	// Override
{
	this->PreDraw(itsView, area);
	this->Draw(itsView, area);
	this->PostDraw(itsView, area);
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerRes

pascal void TAdornment::PreDraw(TView* /* itsView */, 
								const VRect&)	
{
	if (fDrawShadow)
	{
		// !!! it would be nice if at this point we could "push" a failure handler which would
        // call CloseRgn to complete the recording of drawing to the rgn if a failure
        // occured during calls to the adorners
		OpenRgn();
		ShowPen();									// so drawing occurs 
	}
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerRes

pascal void TAdornment::PostDraw(TView* /* itsView */, 
								 const VRect&)	
{
	if (fDrawShadow)
	{
		PenState pnState;
	
		// ...all adorners have drawn, so close the rgn 
	
	#if qDebug
		UseTempRgn("TAdornment::PostDraw");
		UseTempRgn2("TAdornment::PostDraw");
	#endif
	
		CloseRgn(gTempRgn);
	
		CopyRgn(gTempRgn, gTempRgn2);
		GetPenState(pnState);
		OffsetRgn(gTempRgn2, pnState.pnSize.h, pnState.pnSize.v);
		DiffRgn(gTempRgn2, gTempRgn, gTempRgn2);
		PaintRgn(gTempRgn2);
	
	#if qDebug
		DoneWithTempRgn();
		DoneWithTempRgn2();
	#endif
	}
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerRes

pascal void TAdornment::Draw(TView* itsView, 
								const VRect& area)	// Override
{
	PenState savePenState;
	VRect localArea = area;
	CAdornmentIterator iter(this);
	
	
	PenNormal();
	PenSize(fPenSize.h, fPenSize.v);
	GetPenState(savePenState);
	for (TAdorner* anAdorner = iter.FirstAdorner(); iter.More(); anAdorner = iter.NextAdorner())
	{
		SetPenState(savePenState);
		anAdorner->Draw(itsView, localArea);
	}
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerRes

pascal void TAdornment::DoHighlightSelection(TView* itsView, 
											 const VRect& area,
											 HLState fromHL,
											 HLState toHL)
{
	PenState savePenState;
	HLState localFromHL = fromHL;
	HLState localToHL = toHL;

	/* The direction of iteration is controlled by the following state transition diagram:
	  toHL
	  || 	OFF 	 ||		   DIM		  || 	  ON
	  -----------||-------------||-------------------||-------------------
	  OFF	  || 	(NA)	 ||		 Forward 	  || 	Forward
	  -----------||-------------||-------------------||-------------------
	  fromHL		   DIM	  ||   Backward	 ||		  (NA)		  || 	Forward
	  -----------||-------------||-------------------||-------------------
	  ON	  ||   Backward	 ||		Backward	  || 	 (NA)
	  -----------||-------------||-------------------||-------------------
	*/

	PenNormal();
	PenSize(fPenSize.h, fPenSize.v);
	GetPenState(savePenState);
	
	if (((fromHL == hlOn) && ((toHL == hlDim) || (toHL == hlOff))) || ((fromHL == hlDim) && (toHL == hlOff)))
	{
		CAdornmentIterator iter(this, kIterateBackward);
		
		for (TAdorner* anAdorner = iter.FirstAdorner(); iter.More(); anAdorner = iter.NextAdorner())
		{
			SetPenState(savePenState);
			anAdorner->DoHighlightSelection(itsView, area, localFromHL, localToHL);
		}
	}
	else if ((((fromHL == hlOff) || (fromHL == hlDim)) && (toHL == hlOn)) || ((fromHL == hlOff) && (toHL == hlDim)))
	{
		CAdornmentIterator iter(this);
		
		for (TAdorner* anAdorner = iter.FirstAdorner(); iter.More(); anAdorner = iter.NextAdorner())
		{
			SetPenState(savePenState);
			anAdorner->DoHighlightSelection(itsView, area, localFromHL, localToHL);
		}
	}
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAdornerRes

pascal void TAdornment::ViewChangedSize ( 	TView* itsView,
											const VPoint& delta,
											Boolean	invalidate )	// OVERRIDE
{
	if ( fAdorners )
	{
		if ( itsView->Focus () )
		{
			CAdornmentIterator iter(this);
				
			for (TAdorner* anAdorner = iter.FirstAdorner(); iter.More(); anAdorner = iter.NextAdorner())
			{
				anAdorner->ViewChangedSize ( itsView, delta, invalidate );
			}
		}
	}

}
