// UAssociation.cp
// Copyright  1988-1991 by Apple Computer Inc.  All rights reserved.

#ifndef __UASSOCIATION__
#include <UAssociation.h>
#endif

#ifndef __TOOLUTILS__
#include <ToolUtils.h>
#endif

#ifndef __ERRORS__
#include <Errors.h>
#endif

#ifndef __UFAILURE__
#include <UFailure.h>
#endif

#ifndef __UMACAPPUTILITIES__
#include <UMacAppUtilities.h>
#endif

#ifndef __UITERATOR__
#include <UIterator.h>
#endif

//--------------------------------------------------------------------------------------------------
#pragma segment MAAssociationRes

pascal void TEntry::Initialize(void) // override 
{
	inherited::Initialize();
	fKey = NULL;
	fValue = NULL;
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAssociationRes

pascal void TEntry::IEntry(const Str255& itsKey,
						   const Str255& itsValue)
{
	FailInfo fi;

	this->IObject();

	if (fi.Try())
	{
		fKey = NewString(itsKey);
		FailNIL(fKey);

		fValue = NewString(itsValue);
		FailNIL(fValue);
		fi.Success();
	}
	else
	{
		this->Free();
		fi.ReSignal();
	}
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAssociationRes

pascal void TEntry::Free(void)					// override 
{
	fKey = (StringHandle)DisposeIfHandle((Handle)fKey);
	fValue = (StringHandle)DisposeIfHandle((Handle)fValue);

	inherited::Free();
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAssociationRes

pascal void TEntry::SetValue(const Str255& value)
{
	SetString(fValue, value);
	if ((**fValue) != value)
		FailOSErr(memFullErr);					// SetString may attempt to increase the
												//  handle size, yet it does not return
												//  errors!
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAFields

pascal void TEntry::Fields(TObject* obj)		// override 
{
	obj->DoToField("TEntry", (Ptr)NULL, bClass);
	obj->DoToField("fKey", (Ptr) & fKey, bStringHandle);
	obj->DoToField("fValue", (Ptr) & fValue, bStringHandle);

	inherited::Fields(obj);
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAssociationRes

pascal short TEntriesList::Compare(TObject* item1,
								   TObject* item2)
{
	if ((**((TEntry *)item1)->fKey) < (**((TEntry *)item2)->fKey))
		return kItem1LessThanItem2;
	else if ((**((TEntry *)item1)->fKey) > (**((TEntry *)item2)->fKey))
		return kItem1GreaterThanItem2;
	else
		return kItem1EqualItem2;
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAssociationRes

pascal void TEntriesList::IEntriesList(void)
{
	this->ISortedList();
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAFields

pascal void TEntriesList::Fields(TObject* obj)	// override 
{
	obj->DoToField("TEntriesList", (Ptr)NULL, bClass);

	inherited::Fields(obj);
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAssociationRes

pascal void TAssociation::Initialize(void)		// override 
{
	inherited::Initialize();
	fEntries = NULL;
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAssociationRes

pascal void TAssociation::IAssociation(void)
{
	TEntriesList * anEntriesList;
	FailInfo fi;

	this->IObject();

	if (fi.Try())
	{
		anEntriesList = new TEntriesList;
		anEntriesList->IEntriesList();
		fEntries = anEntriesList;
#if qDebug
		fEntries->SetEltType("TEntry");
#endif

		fi.Success();
	}
	else
	{
		this->Free();
		fi.ReSignal();
	}
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAssociationRes

pascal void TAssociation::Free(void)			// override 
{
	fEntries = (TEntriesList *)FreeListIfObject(fEntries);

	inherited::Free();
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAssociationRes

pascal void TAssociation::EachEntryDo(DoToEntryType DoToEntry, void* staticLink)
{
	fEntries->Each((DoToObjectType)DoToEntry, staticLink);
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAssociationRes

class CEntryWithKey
{
	// Fields
	const Str255& fKeyStr;

public:

	// Constructor
	CEntryWithKey(const Str255& thekeyStr) :
		fKeyStr(thekeyStr)
	{
	}

	// Method
	pascal CompareResult TestKey(TObject* anItem);
};

#pragma segment MAAssociationRes

pascal CompareResult CEntryWithKey::TestKey(TObject* anItem)
{
	if (fKeyStr < **(((TEntry *)anItem)->fKey))
		return kItemGreaterThanCriteria;
	else if (fKeyStr > **(((TEntry *)anItem)->fKey))
		return kItemLessThanCriteria;
	else
		return kItemEqualCriteria;
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAssociationRes

pascal TEntry* TAssociation::EntryWithKey(const Str255& keyStr)
{
	CEntryWithKey aCEntryWithKey(keyStr);

	return (TEntry *)(fEntries->Search((CompareObjectType) & CEntryWithKey::TestKey, &aCEntryWithKey));
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAssociationRes

pascal TEntry* TAssociation::EntryWithValue(const Str255& valueStr)
{
	TEntry* returnEntry = NULL;
	CArrayIterator iter(fEntries);
	
	for (ArrayIndex i = iter.FirstIndex(); iter.More(); i = iter.NextIndex())
	{
		TEntry* anEntry = (TEntry*)fEntries->At(i);
		
		if (valueStr == **(anEntry->fValue))
		{
			returnEntry = anEntry;
			break;
		}
	}
	return returnEntry;
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAssociationRes

pascal TEntry* TAssociation::FirstEntryThat(TestEntryType TestEntry, void* staticLink)
{
	return (TEntry *)(fEntries->FirstThat((TestObjectType)TestEntry, staticLink));
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAssociationRes

pascal void TAssociation::InsertEntry(const Str255& keyStr,
									  const Str255& valueStr)
{
	TEntry * anEntry;

	anEntry = this->EntryWithKey(keyStr);
	if (anEntry)
	{
		anEntry->SetValue(valueStr);
	}
	else
	{
		anEntry = new TEntry;
		anEntry->IEntry(keyStr, valueStr);
		fEntries->Insert(anEntry);
	}
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAssociationRes

pascal Boolean TAssociation::KeyAt(const Str255& valueStr,
								   Str255& keyStr)
{
	TEntry * theEntry = NULL;
	CArrayIterator iter(fEntries);
	
	for (ArrayIndex i = iter.FirstIndex(); iter.More(); i = iter.NextIndex())
	{
		TEntry* anEntry = (TEntry*)fEntries->At(i);
		
		if (valueStr == **(anEntry->fValue))
		{
			theEntry = anEntry;
			break;
		}
	}
	
	if (!theEntry)
	{
		keyStr = "";
		return FALSE;
	}
	else
	{
		keyStr = **((String255Handle)(theEntry->fKey));
		return TRUE;
	}
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAssociationRes

pascal void TAssociation::RemoveValueAt(const Str255& keyStr)
{
	TEntry * theEntry;
	CEntryWithKey aCEntryWithKey(keyStr);

	theEntry = (TEntry *)(fEntries->Search((CompareObjectType) & CEntryWithKey::TestKey, &aCEntryWithKey));
	if (theEntry)
		fEntries->Delete(theEntry);
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAssociationRes

pascal void TAssociation::RemoveKeyAt(const Str255& valueStr)
{
	TEntry * theEntry = NULL;
	CArrayIterator iter(fEntries);
	
	for (ArrayIndex i = iter.FirstIndex(); iter.More(); i = iter.NextIndex())
	{
		TEntry* anEntry = (TEntry*)fEntries->At(i);
		
		if (valueStr == **(anEntry->fValue))
		{
			theEntry = anEntry;
			break;
		}
	}
	if (theEntry)
		fEntries->Delete(theEntry);
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAAssociationRes

pascal Boolean TAssociation::ValueAt(const Str255& keyStr,
									 Str255& valueStr)
{
	TEntry * theEntry;
	CEntryWithKey aCEntryWithKey(keyStr);

	theEntry = (TEntry *)(fEntries->Search((CompareObjectType) & CEntryWithKey::TestKey, &aCEntryWithKey));
	if (!theEntry)
	{
		valueStr = "";
		return FALSE;
	}
	else
	{
		valueStr = **((String255Handle)(theEntry->fValue));
		return TRUE;
	}
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAFields

pascal void TAssociation::Fields(TObject* obj)	// override 
{
	obj->DoToField("TAssociation", (Ptr)NULL, bClass);
	obj->DoToField("fEntries", (Ptr) & fEntries, bObject);

	inherited::Fields(obj);
}


