//UFailure.cp
//Copyright  1985-1991 Apple Computer, Inc.  All rights reserved.

#ifndef __UFAILURE__
#include "UFailure.h"
#endif

#ifndef __STDIO__
#include <StdIo.h>
#endif

#ifndef __MEMORY__
#include <Memory.h>
#endif

#ifndef __RESOURCES__
#include <Resources.h>
#endif

#ifndef __ERRORS__
#include <Errors.h>
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif

#ifndef __UDEBUG__
#include "UDebug.h"
#endif

FailInfoPtr gTopHandler = NULL;
MAName pWho;									// used serially. Avoids putting it on stack

#if qDebug
Boolean gAskAboutAlloc = FALSE;
Boolean gAskFailure = FALSE;
#endif

extern pascal void DoFailure(FailInfoPtr pf);

//--------------------------------------------------------------------------------------------------
#pragma segment MAFailureRes

pascal void Assertion(const Boolean condition,
					  const Str255& 
#if qDebug
description
#endif
)
{
	if (!condition)
	{
#if qDebug
		GetCallersMethodName(pWho);
		if (CanReadLn())
		{
			fprintf(stderr, "Assertion failed in %s: %s\n", (char *) pWho, (char *) description);
			EnterMacAppDebugger();
			if (ReadYesNo("Do you want to signal failure?"))
				Failure(minErr, 0);				/* ??? silent failure, but someday 0 messages
												  need to be non-silent */
		}
		else
		{
			DebugStr("Assertion failed in " + pWho + ": " + description);
			Failure(minErr, 0);					/* ??? silent failure, but someday 0 messages
												  need to be non-silent */
		}
#else
		Failure(minErr, 0);						/* ??? silent failure, but someday 0 messages
												  need to be non-silent */
#endif

	}
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAFailureRes

pascal void EachFailureHandlerDo(DoToHandlerType DoToHandler,
							  void* staticLink)
{
	for (FailInfoPtr pf = gTopHandler; (pf); pf = pf->nextInfo)
		(*DoToHandler)(pf, staticLink);
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAFailureRes

pascal void FailMemError(void)
{
	OSErr e = MemError();

#if qDebug
	if (gAskFailure && (e == noErr) && CanReadLn())
	{
		GetCallersMethodName(pWho);
		e = ReadInteger("FailMemError called by " + pWho + ".  Enter return error: ");
	}
#endif

	if (e != noErr)
		Failure(e, 0);
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAFailureRes

pascal void FailNewMessage(short error,
						   long oldMessage,
						   long newMessage)
{
	if (!oldMessage)
		Failure(error, newMessage);
	else
		Failure(error, oldMessage);
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAFailureRes

pascal void FailNIL(void* p)
{
	// no check for gAskFailure here, since we do this when objects are created. 
	if (!p)
		Failure(memFullErr, 0);
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAFailureRes

pascal void FailNILResource(Handle r)
{
	OSErr e;

#if qDebug
	if (gAskFailure && (r) && CanReadLn())
	{
		GetCallersMethodName(pWho);
		if (ReadYesNo("FailNilResource called by " + pWho + ".  Return NULL?: "))
			r = NULL;
	}
#endif

	if (!r)
	{
		e = ResError();
		if (e == noErr)
			e = resNotFound;
		Failure(e, 0);
	}
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAFailureRes

pascal void FailOSErr(short error)
{
#if qDebug
	if (gAskFailure && (error == noErr) && CanReadLn())
	{
		short debugError;

		GetCallersMethodName(pWho);
		debugError = ReadInteger("FailOSErr called by " + pWho + ".  Enter return error: ");
		if (debugError != noErr)
			Failure(debugError, 0);
	}
#else
	if (error != noErr)
		Failure(error, 0);
#endif

}

//--------------------------------------------------------------------------------------------------
#pragma segment MAFailureRes

pascal void FailResError(void)

{
	OSErr e = ResError();

#if qDebug
	if (gAskFailure && (e == noErr) && CanReadLn())
	{
		GetCallersMethodName(pWho);
		e = ReadInteger("FailResError called by " + pWho + ".  Enter return error: ");
	}
#endif

	if (e != noErr)
		Failure(e, 0);
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAFailureRes

pascal void Failure(short error,
					long message)

{
	FailInfoPtr pf = gTopHandler;
#if qDebug
	long pc;
#endif

	if (pf)
	{
		/*pop the stack first, because calling the handler is likely to
		  result in a call to Failure*/
		gTopHandler = pf->nextInfo;

#if qDebug
		if (CanWriteLn() && ((error != 0) || gIntenseDebugging))// only show 0 errors if _really_ looking
		{
			GetCallersMethodName(pWho);
			fprintf(stderr, "Failure signaled by: %s\n", (char *) pWho);
			pc = pf->whoPC;
			GetMethodName(pc, pWho);

			fprintf(stderr, "Failure caught by: %s\n", (char *) pWho);
			fprintf(stderr, "        error: %1d message: %1d (%1d/%1d)\n", error, message, message >> 16, message & 0x0000FFFF);
		}
#endif

		pf->error = error;
		pf->message = message;
		DoFailure(pf);							// Go execute the failure handler
	}
	else
	{
		if (qDebug)
			ProgramBreak("Failure called, but no handler!");
		else
			DebugStr("Failure called, but no handler!");
	}
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAFailureRes

pascal void ProgramBreak(const Str255& grievance)
{
	Str255 report(grievance);

#if qDebug
	if (CanReadLn())
	{
		DebugForceOutput(forceOn, forceUnchanged);
		fprintf(stderr, "ProgramBreak: %s\n", (char *) grievance);
		DebugEndForce();
		EnterMacAppDebugger();
	}
	else
		DebugStr(report);
#else
	DebugStr(report);
#endif

}

//--------------------------------------------------------------------------------------------------
#pragma segment MAFailureRes

pascal void ProgramReport(const Str255& grievance,
						  const Boolean
#if qDebug
breakInDebugger
#endif
)

{
	Str255 report = grievance;

#if qDebug
	if (CanReadLn())
	{
		DebugForceOutput(forceOn, forceUnchanged);
		fprintf(stderr, "ProgramReport: %s\n", (char *) grievance);
		DebugEndForce();
		if (breakInDebugger)
			EnterMacAppDebugger();
	}
	else
		DebugStr(report);
#else
	DebugStr(report);
#endif

}

//--------------------------------------------------------------------------------------------------
#pragma segment MAFailureRes

pascal Boolean HandlerExists(FailInfoPtr testFailInfoPtr)

{
	FailInfoPtr pf = gTopHandler;

	while (pf)
	{
		if (pf == testFailInfoPtr)
			return TRUE;
		pf = pf->nextInfo;
	}
	return FALSE;
}

//--------------------------------------------------------------------------------------------------
#pragma segment MAFailureRes

pascal void Success(FailInfo& fi)

{
#if qDebugMsg
	if (gTopHandler != &fi)
	{
		fprintf(stderr, "gTopHandler: %p, parameter: %p\n", gTopHandler, &fi);
		fprintf(stderr, "Problem with Success: ");
		if (HandlerExists(&fi))
			fprintf(stderr, "too few ");
		else
			fprintf(stderr, "too many ");
		fprintf(stderr, "calls to Success\n");
		ProgramBreak("");
	}
#endif

	gTopHandler = fi.nextInfo;
}


