// UGrabberTracker.cp
// Copyright  1988-1991 by Apple Computer, Inc. All rights reserved.

#ifndef __UGRABBERTRACKER__
#include "UGrabberTracker.h"
#endif

#ifndef __UGEOMETRY__
#include <UGeometry.h>
#endif

#ifndef __UDOCUMENT__
#include "UDocument.h"
#endif

#ifndef __USCROLLER__
#include "UScroller.h"
#endif

#ifndef __UMACAPPUTILITIES__
#include "UMacAppUtilities.h"
#endif


//--------------------------------------------------------------------------------------------------
#pragma segment GrabberDoCommand

pascal void TGrabberTracker::IGrabberTracker(CmdNumber itsCmdNumber,
											 TDocument* itsDocument,
											 TView* itsView,
											 TScroller* itsScroller,
											 const VPoint& itsMouse)

{
	this->INoChangesTracker(itsCmdNumber, itsDocument, itsView, itsScroller, itsMouse);
}

//--------------------------------------------------------------------------------------------------
#pragma segment GrabberDoCommand

pascal TTracker* TGrabberTracker::TrackMouse(TrackPhase aTrackPhase,
											 VPoint& anchorPoint,
											 VPoint& previousPoint,
											 VPoint& nextPoint,
											 Boolean mouseDidMove)// override 

{
	switch (aTrackPhase)
	{
		case trackBegin:
			break;

		case trackContinue:
			if (mouseDidMove)
			{
				VPoint delta = - fScroller->fTranslation;

				delta.ConstrainTo(VRect(anchorPoint - nextPoint, fScroller->fMaxTranslation - fScroller->fTranslation));

				if (delta != gZeroVPt)
				{
					fScroller->ScrollBy(delta, True);
					fScroller->Update();
				}

				nextPoint += delta;

				previousPoint = nextPoint;
				anchorPoint = nextPoint;
			}
			break;

		case trackEnd:
			break;
	}											// switch 
	return this;								// keep tracking with me 
}

//--------------------------------------------------------------------------------------------------
#pragma segment GrabberDoCommand

pascal void TGrabberTracker::TrackFeedback(TrackPhase /* aTrackPhase */,
										const VPoint& /* anchorPoint */,
										const VPoint& /* previousPoint */,
										const VPoint& /* nextPoint */,
										Boolean /* mouseDidMove */,
										Boolean /* turnItOn */)// override

{
	// NO feedback please 
}

//--------------------------------------------------------------------------------------------------
#pragma segment GrabberDoCommand

pascal void TGrabberTracker::AutoScroll(const VPoint& delta)// override

{
	// NO AutoScroll please 
}


